/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){

    dojo.provide("com.ibm.epm.web.core.app.internal.EPMApplication");
    
    dojo.require("jazz.app._JazzApplication");
    dojo.require("com.ibm.epm.web.core.ui.internal.util.CSHelp");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants");
    dojo.require("com.ibm.epm.web.core.app.internal.RepositoryInitialize");
	dojo.require("com.ibm.epm.web.core.app.internal.ProcessInitializer");
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.TopNav");
	
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.ConfigureEnvPage");
	dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
	dojo.require("com.ibm.epm.web.core.client.internal.EPMWebUIServiceClient");

    dojo.requireLocalization("com.ibm.epm.web.core.ui", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
    
	var ConfigureEnvPage = new com.ibm.epm.web.core.ui.internal.EPMView.ConfigureEnvPage();
    var EPMAppConstants = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants;
	var RepositoryInitialize = com.ibm.epm.web.core.app.internal.RepositoryInitialize;
	var ProcessInitializer = com.ibm.epm.web.core.app.internal.ProcessInitializer;

	var TopNav = com.ibm.epm.web.core.ui.internal.EPMView.TopNav;

	var messages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var EPMWebUIServiceClient = com.ibm.epm.web.core.client.internal.EPMWebUIServiceClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
	var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;

   	/**
     * Displays the Project Management perspective and artifacts associated with a project area.
     */
    dojo.declare("com.ibm.epm.web.core.app.internal.EPMApplication", jazz.app._JazzApplication, {
		productName: messages.productName,
		
		productIconSmall: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/header/RPC_16.png"),
		
		productIconLarge: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/header/RPC_31.png"),
	
		constructor: function(args) {	
			this.messages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.setName(messages.productName); //Set default browser title
			this.config.productTitleImg = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/header/Rational-Project-Conductor.png");
			this.config.buttons.push({ //create the home button
								label: "Home",
								iconSrc: this.productIconLarge,
								align: "left"
								//pageId: ""//TODO Add the Home page Jscript module
							}); 
	        this.initializationData();
			
			//hardcode jazz.net footer links until branding solution is available.
			if (location.href.indexOf("jazz.net") > 0 && location.href.indexOf("localhost") > 0) {
				this.config.footer = {links: [ 
					{ label: messages.contact, href: "/legal/contact.jsp" },
					{ label: messages.termsOfUse, href: "/legal/terms.jsp" }
				]};
			}
	
	        CSHelpEngine.init(EPMAppConstants.CSHELP_URL, EPMAppConstants.CSHELP_INFO_CENTER_LABEL, EPMAppConstants.CSHELP_INFO_CENTER_HREF);
		},

	 	initializationData: function(){
			this.getUserRoles();
			RepositoryInitialize.init(this, "_onSuccess", "onFailure"); //initialize repo features
		},
		
		/**
		 * Method to check the user roles and set the global variable "_userIsAdmin" to true if user has JazzAdmins and JazzProjectAdmins
		 */
		getUserRoles: function () {
			var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
			AdminClient.getUserRoles(srh);
		},
		
		authRequestPassed: function(response) {
			if (response.userId) {
				dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
			}
			var roles = response.roles;
			for (var i=0; i < roles.length; i++) {
				if ((roles[i] === "JazzAdmins") || (roles[i] === "JazzProjectAdmins")) {
					this._userIsAdmin = true;
					break;
				}
			}
		},

		authRequestFailed: function(errorObject) {
			this.onFailure(errorObj);
		},
		
		_getAdminProjectArea: function(){
			var srh = new ServiceResponseHandler(this, "getAdminProjectAreaSuccess", "getAdminProjectAreaFailure");
			EPMWebUIServiceClient.getAdminProjectArea(srh);
		},
		
		getAdminProjectAreaSuccess: function(/* ResponseDTO */ responseDTO) {
			console.log(responseDTO);
			if (responseDTO) {
				if (responseDTO.itemId == "ADMIN_PROJECT_AREA_ABSENT") {
					this.initDB(ConfigureEnvPage);
					this.workbench.setTitle(this.messages.configureEnvTitle);
				} else {
					// Initialize project areas global object for accessing every where.
					ProcessInitializer.init(this, responseDTO);
					
					// Initialise top nav to be visible for multiple pages
					this._topNav = new TopNav();
					
					// Suppress display of EPMView Page in the nav bar
					this.config.suppressedPages = ["com.ibm.epm.EPMViewPage"];
					
					this.start();
				}
			}
		},
	
		initDB: function(widget) {
			jazz.util.theming.initialize();
			this.ui.init({		//send an abridged version of this.config
				productName: this.productName, 
				productIconLarge: this.productIconLarge,
				productTitleImg: this.config.productTitleImg,
				showLoginLogout: this.config.showLoginLogout,
				login: this.config.login,
				logout: this.config.logout,
				user: this.config.user,
				buttons: this.config.buttons
			});
			this.setName(messages.productName); //Set default browser title
			
			dojo.addClass(dojo.body(), "jazz-app-body");	
			this.workbench.rootNode().appendChild(this.ui.domNode);
	
			dojo.body().removeChild(dojo.byId("net-jazz-ajax-InitialLoadMessage"));
			
			this.ui.setContent(widget);
		},
			
		getAdminProjectAreaFailure: function(errorObj) {
			this.onFailure(errorObj);
		},
	
		_onSuccess: function() {
			this._getAdminProjectArea();
		}
    });
    
})();
