/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.resource.assignment.web.client.internal.ResourceAssignmentClient");
    
    dojo.require("com.ibm.epm.web.core.transport.EPMServerClient");
    dojo.require("com.ibm.epm.web.core.transport.ServiceRequest");
    dojo.require("com.ibm.epm.web.core.util.internal.Constants");
    
    var ServiceRequest = com.ibm.epm.web.core.transport.ServiceRequest;
    var EpmServerClient = com.ibm.epm.web.core.transport.EPMServerClient;
    
    var _SERVICE_URI = "com.ibm.team.dashboard.viewlets.service.internal.teams.ITeamsService";
    var EPMConstants = com.ibm.epm.web.core.util.internal.Constants;
    
    com.ibm.epm.resource.assignment.web.client.internal.ResourceAssignmentClient = {
    
        _PROJECT_RESOURCE_URI: "epm/resources/user",
        _TASK_ASSIGNMENT: "/taskAssignment",
        _PROJECT_ASSIGNMENT: "/projectAssignment",
        _PROFILE_URI: "/profile",
        _BASE_URI: "epm/resources",
        _QUERY_TYPE_STRING: "type",
        
        _QUERY_TYPE_VALUE: "taskAssignment&execute=true&jsonExpression=",
        
        _QUERY_URI: "epm/query/",
		
		 _ASSIGNEMNT_COMPONENT_SERVICE : "epm/services/assignment",
		 
		 _GET_USER_TASK_ASSIGNMENTS_FUNTION: "getUserAssignments",
		 
		 _GET_PROFILE_TASK_ASSIGNMENTS_FUNCTION : "getProfileAssignments",
		 
		 _HAS_ACTUALS_FUNCTION : "hasActuals",
		 
		 _VALIDATE_OVERALLOCATION_EXTENSION : "validateOverAllocationAndExtensionOfProjectAllocation",
		
        createURI: function(responseHandler, relativeURI, params){
            var serviceRequest = new ServiceRequest(this._BASE_URI + "/" + relativeURI, "post", params);
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        deleteURI: function(responseHandler, relativeURI){
            var serviceRequest = new ServiceRequest(this._BASE_URI + "/" + relativeURI, "delete");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getURI: function(responseHandler, relativeURI){
            var serviceRequest = new ServiceRequest(this._BASE_URI + "/" + relativeURI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getProjectProfile: function(responseHandler, relativeUserURI, projectId){
            var args = {};
            args.projectId = projectId;
            
            var serviceRequest = new ServiceRequest("epm/resources/" + relativeUserURI, "get", args);
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getSystemProfile: function(responseHandler){
        
        
            var serviceRequest = new ServiceRequest(this._BASE_URI + this._PROFILE_URI, "get");
            
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        getResourceData_Href: function(responseHandler, relativeUserURI){
            var serviceRequest = new ServiceRequest("epm/resources/" + relativeUserURI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getAssignedResourceData_Href: function(responseHandler, relativeUserURI){
        
            var serviceRequest = new ServiceRequest("epm/resources/" + relativeUserURI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        
        getAllResources: function(responseHandler){
            this._invokeWorkService(responseHandler, "get", null);
            
        },
        
        getAllResourcesWithPagination: function(responseHandler, args){
            this._invokeWorkService(responseHandler, "get", args);
            
        },
        
        
        /*
         * Get all the resources which are assigned to the task selected
         */
        getAllTaskAssignmentResourcesWithPagination: function(responseHandler, args){
        
            var serviceRequest = new ServiceRequest("epm/resources/" + args, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        getAllProjectAssignmentResourcesWithPagination: function(responseHandler, args){
            console.log("fetching data ARG");
            // The agrs contains the realted Href
            var serviceRequest = new ServiceRequest("epm/resources/" + args, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        unassignResource: function(responseHandler, uri){
            var serviceRequest = new ServiceRequest(this._BASE_URI + "/" + uri, "delete", null);
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /*
         * Invoke an operation to assign the resource to the task selected.
         *
         */
        postAssignResource: function(responseHandler, args, wbsNodeID, userID, wbsNodeType, userName, role, contourType, contourVal, assignmentID, costRate, sellingRate, labourChargecode, revenueChargecode){
        
            args.contourValue = contourVal;
            args.contourType = contourType;
            args.wbsNodeType = wbsNodeType;
            
            console.log(args.wbsNodeType);
            console.log("assignmentID");
            
            
            var serviceRequest;
            if (wbsNodeType == EPMConstants.EPM_PROJECT) {
                var actualArgs = dojo.clone(args);
                
                actualArgs.userURI = userID;
                actualArgs.assignmentType = userID.substring(0, userID.indexOf('/'));
                
                
                if (assignmentID == null || assignmentID == "") {
                    serviceRequest = new ServiceRequest("epm/resources/" + wbsNodeID + "/projectAssignment", "post", actualArgs);
                }
                else {
                    serviceRequest = new ServiceRequest("epm/resources/" + wbsNodeID + "/projectAssignment/" + assignmentID, "post", actualArgs);
                }
            }
            else {
                args.assignmentType = userID.substring(0, userID.indexOf('/'));
                console.debug("role==>" + role);
                args.role = role;
                
                
                args.costRate = costRate;
                args.sellingRate = sellingRate;
                args.labourChargecode = labourChargecode;
                args.revenueChargecode = revenueChargecode;
                
                if (assignmentID == null || assignmentID == "") {
                    serviceRequest = new ServiceRequest("epm/resources/" + userID + "/taskAssignment", "post", args);
                }
                else {
                    serviceRequest = new ServiceRequest("epm/resources/" + userID + "/taskAssignment/" + assignmentID, "post", args);
                }
            }
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /**
         * Invokes the requested method on the work REST service.
         */
        _invokeWorkService: function(responseHandler, method, args){
            if (arguments.length == 2) {
                var serviceRequest = new ServiceRequest(this._PROJECT_RESOURCE_URI, method);
            }
            else {
                serviceRequest = new ServiceRequest(this._PROJECT_RESOURCE_URI, method, args);
            }
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
        },
        
        /*
         * It returns all the task assignment of a particular user.
         */
        getAllTaskAssignments: function(responseHandler, user, project, getAllAssignments){
			
			user = user + this._TASK_ASSIGNMENT;
        
			if(getAllAssignments){
				var jsonExpression = new TermExpression([new AttributeExpression("selfURI", "contains", user), new AttributeExpression("link:com.ibm.epm.assignment.taskLink:source/category/itemId", "is", project)]);
			}else{
				var jsonExpression = new TermExpression([new AttributeExpression("selfURI", "contains", user),new AttributeExpression("active", "is", true), new AttributeExpression("link:com.ibm.epm.assignment.taskLink:source/category/itemId", "is", project)]);
			}
	            
	            var jsonExpressionString = jsonExpression.toString();
	            
	            var getURI = this._QUERY_URI;
	            
	            getURI += "?" + this._QUERY_TYPE_STRING + "=" + this._QUERY_TYPE_VALUE; 
	            getURI += jsonExpression;
	            
	            // TODO: CLEAN this up
	            getURI += '&';
	            
	            
	            var serviceRequest = new ServiceRequest(getURI, "get");
	            
	            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
			
			
        },
		
		getAllTaskAssignmentForUser:function(responseHandler,args){
			
			var SERVICE_URI = this._ASSIGNEMNT_COMPONENT_SERVICE;
			var method = this._GET_USER_TASK_ASSIGNMENTS_FUNTION;
			
			var serviceRequest = new ServiceRequest(SERVICE_URI + "/"+method, "get", args);
			EpmServerClient.operateOnResource(serviceRequest, responseHandler);
			
		},
		getAllTaskAssignmentsForProfile:function(responseHandler,args){
			
				var SERVICE_URI = this._ASSIGNEMNT_COMPONENT_SERVICE;
				var method = this._GET_PROFILE_TASK_ASSIGNMENTS_FUNCTION;
				
				var serviceRequest = new ServiceRequest(SERVICE_URI + "/"+method, "get", args);
				EpmServerClient.operateOnResource(serviceRequest, responseHandler);
		},
		
		getWorkingHours:function(responseHandler,args){
			
			var SERVICE_URI = this._ASSIGNEMNT_COMPONENT_SERVICE;
			var method = "getWorkingHours";
				            
	       var serviceRequest = new ServiceRequest(SERVICE_URI + "/"+method, "get", args);
			EpmServerClient.operateOnResource(serviceRequest, responseHandler,true);
		},
		
		hasActuals:function(responseHandler,args){
			
			var SERVICE_URI = this._ASSIGNEMNT_COMPONENT_SERVICE;
			var method = this._HAS_ACTUALS_FUNCTION;
				            
	       	var serviceRequest = new ServiceRequest(SERVICE_URI + "/"+method, "get", args);
			EpmServerClient.operateOnResource(serviceRequest, responseHandler,true);
			
		},
		
		validateOverAllocationAndExtensionOfProjectAllocation:function(responseHandler,args){
			
			var SERVICE_URI = this._ASSIGNEMNT_COMPONENT_SERVICE;
			var method = this._VALIDATE_OVERALLOCATION_EXTENSION;
				            
	       	var serviceRequest = new ServiceRequest(SERVICE_URI + "/"+method, "get", args);
			EpmServerClient.operateOnResource(serviceRequest, responseHandler,true);
			
		}
    };
	    /**
     * Contstruct attribute query expression JSON object representing some
     * mathematical or logical sub-expression against a ECore model
     * EAttribute.<p>
     *
     * Overridden toString() method will stringify the object into a
     * form that can be passed over the wire to the server.
     *
     * @param {Object} attributeId EAttribute XPath expression
     * @param {Object} operator logical or mathematical operator
     * @param {Object} values optional constant operands
     * @param {Object} variables optional EAttribute operands
     */
    function AttributeExpression(attributeId, operator, values, variables){
        /**
         * Return array representation of val
         *
         * @param {Object} val object to be turned into an Array object of
         * its contents
         */
        var arrayify = function(val){
            if (typeof(val) == "undefined") {
                return [];
            }
            else 
                if (!(val instanceof Array)) {
                    return [val];
                }
            
            return val;
        };
        
        this.attributeId = attributeId;
        this.operator = operator;
        this.values = arrayify(values);
        this.variables = arrayify(variables);
        
        /**
         * Override the built-in Object.toString() method to generate
         * stringified representation of JSON object for sending over
         * the wire to server.
         *
         * @returns string representation of JSON query object
         */
        this.toString = function(){
            /**
             * Stringify the contents of an array object
             *
             * @param {Object} array the array
             */
            function dumpArray(array){
                var str = "";
                
                for (var i = 0; i < array.length; i++) {
                
                    if (str.length > 0) {
                        str += ", ";
                    }
                    str += '"' + array[i].toString() + '"';
                }
                
                return str;
            }
            
            return '{"attributeId": "' + this.attributeId + '",' +
                   ' "operator": "' + this.operator + '",' +
                   ' "values": [' + dumpArray(this.values) + '],' +
                   ' "variables": [' + dumpArray(this.variables) + ']}';
        };
    }
    
    /**
     * Contstruct a logical query expression JSON object representing some
     * one or more sub-expressions that can be either themselves
     * TermExpression's or AttributeExpression's.<p>
     *
     * Overridden toString() method will stringify the object into a
     * form that can be passed over the wire to the server.
     *
     * @param {Object} attributeExpressions optional array of attribute
     * query sub-expressions
     * @param {Object} termExpressions optional array of logical query
     * sub-expressions
     * @param {Object} similarityExpressions optional array of
     * similarity (???) sub-expressions
     * @param {Object} operator operator that links all the sub-expressions;
     * defaults to AND
     */
    function TermExpression(attributeExpressions, termExpressions, similarityExpressions, operator){
        /**
         * Return array representation of val
         *
         * @param {Object} val object to be turned into an Array object of
         * its contents
         */
        var arrayify = function(val){
            if (typeof(val) == "undefined") {
                return [];
            }
            else 
                if (!(val instanceof Array)) {
                    return [val];
                }
            
            return val;
        };
        
        this.attributeExpressions = arrayify(attributeExpressions);
        this.termExpressions = arrayify(termExpressions);
        this.similarityExpressions = arrayify(similarityExpressions);
        this.operator = (typeof(operator) == "undefined") ? "AND" : operator;
        
        /**
         * Override the built-in Object.toString() method to generate
         * stringified representation of JSON object for sending over
         * the wire to server.
         *
         * @returns string representation of JSON query object
         */
        this.toString = function(){
            /**
             * Stringify the contents of an array object
             *
             * @param {Object} array the array
             */
            function dumpArray(array){
                var str = "";
                
                for (var i = 0; i < array.length; i++) {
                
                    if (str.length > 0) {
                        str += ", ";
                    }
                    
                    var item = array[i];
                    
                    if (typeof(item) == "object") {
                        str += item.toString();
                    }
                    else {
                        str += '"' + item.toString() + '"';
                    }
                }
                
                return str;
            }
            
            return '' +
  '{"attributeExpressions": [' + dumpArray(this.attributeExpressions) + '],' +
  ' "termExpressions": [' + dumpArray(this.termExpressions) + '],' +
  ' "similarityExpressions": [' + dumpArray(this.similarityExpressions) + '],' +
  ' "operator": "' + this.operator + '"}';
        };
    }
})();
