/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.EPMRateAssignmentGridController");
    dojo.require("com.ibm.epm.web.core.grid.controller.EPMGridController");
    dojo.require("com.ibm.epm.web.core.util.internal.Constants");
	dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
	dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
	dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
        
    var epmGridController = com.ibm.epm.web.core.grid.controller.EPMGridController;
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var DateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    
    dojo.declare("com.ibm.epm.web.core.grid.controller.EPMRateAssignmentGridController", epmGridController, {
    	        
        rates : null,
        
        buildGridModel: function(){
            this._updateRatesModel();
        },
        
        _updateRatesModel: function(){
            var data = null;
            
            if (this.rates == null)
            {
                this._createBlankRow();
                return;
            }
            
			if (this._allElementsURI == "RateAssignment") {
				var references = this.rates;
				
				if (this.rates && this.rates.constructor ==Array && this.rates.length == 0) {
					this._createBlankRow();
					return;
				}
				data = new Array(1);
				for (var k in references) {
					if (!references[k].id) {
						references[k].id = k + "_unsaved";
					}
					var itemHandle = {
						relativeHref: references[k].id
					};
					data.push(itemHandle);
				}
				
			}
            if (data)
            {
                this.updateModel(false, data);
            }
            else {
                this._createBlankRow();
            }
        },
		
		_getAndUpdateRow: function(rowURI, rowID, linkName)
		{
			if (this._allElementsURI != "RateAssignment")
			{
				this.inherited(arguments);
			}
			else 
			{
				if (this.assignment !== null)
				{
					var row = this._getEmptyRow(rowID);
						
					var references = this.rates;
					for (var j in references) {
						if ((references[j].id) == rowURI)
						{
							row.fromDate = references[j].fromDate;
							row.toDate = references[j].toDate;
							row.costRate = references[j].costRate;
							row.sellingRate = references[j].sellingRate;
							break;
						}
					}
					this._updateGridRow(row, rowID, linkName);
					this._getLinkedResources(row, rowID);
				}					
			}
        },
        
        persistGrid: function()
        {
			if (this._allElementsURI != "RateAssignment")
			{
				this.inherited(arguments);
			}
			else
			{
				var rateInstances = new Array();
				for (var i = 0; i < this._gridData.getRowCount(); i++)
				{
                	if (this._gridData.getRow)
                	{
                    	var row = this._gridData.getRow(i);
                    	if (row)
                    	{
							var rateInstance = {
								fromDate : DateUtil.formatDateForSave(row.fromDate),
								toDate : DateUtil.formatDateForSave(row.toDate),
								costRate : row.costRate,
								sellingRate : row.sellingRate
							}
							if(row.id.indexOf("_unsaved") < 0)
							{
								rateInstance.id = row.id;
							}
							rateInstances.push(rateInstance);
                    	}
                	}
            	}
            	return rateInstances;
			}                     
        },
       /*
        * Deletes the row from the rates grid
        * Incoming :row - row object
        * Outgoing :None
        */
        _deleteElementByRow: function(row){
            // Get the Delete call URI
            // Delete URI wud be same as relatedHref
            var deleteURI = dojo.isArray(row.id[0]) ? row.id[0] : row.id;
            var publishObj = {
                controllerID: null,
                uniqueIdentifier: null,
                rowid: deleteURI
            };
            if (this.epmUniqueID) {
                publishObj.uniqueIdentifier = this.epmUniqueID;
            }
            if (this.id) {
                publishObj.controllerID = this.id;
            }
            var self = this;
            if (row.id == null) {
                return;
            }
            self._removeGridRow(deleteURI);
            dojo.publish(GridEvents.ROW_DELETED, [publishObj]);
            
        }
    });
})();
