/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.controller.TreeBasedGridController");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.AbstractGridController");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
    dojo.require("com.ibm.epm.web.core.util.internal.Constants");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants");
    
    // Localization
    dojo.requireLocalization("com.ibm.epm.web.core.ui", "Messages");
    
    var CoreMessages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages");
    var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
    var abstractGridController = com.ibm.epm.web.core.grid.controller.AbstractGridController;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var EPMConstants = com.ibm.epm.web.core.util.internal.Constants;
    var GridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var saveForCollapseInProgress = false;
	    
    dojo.declare("com.ibm.epm.web.core.grid.controller.TreeBasedGridController", abstractGridController, {
        _baseTreeID: null,
        _subscriptions: null,
		//The _buidModel flag is used to avoid the call buildGridModel twice from the view. While creating the view
		//buildGridModel will be called. Also from the tree loaded event the build grid Model will be called.
		_buildModel: true,
        
        initializeController: function(treeID, viewer, model, linksToBeLoaded){
            // Calling super function
            this.inherited(arguments);
            // Initializing treeID
            if (treeID != null && treeID != "") {
                this._baseTreeID = treeID;
            }
        },
        
        isInitialized: function(){
            var initialized = false;
            if (this.inherited(arguments) &&
            this._baseTreeID != null) {
                initialized = true;
            }
            return initialized;
        },
        
        destroy: function(){
            this._tree = null;
            this._unsubscribeToAllEvents();
            this.inherited(arguments);
        },
        
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions !== null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
        },
        
		_subscribeToAllEvents: function(){
            this._subscribeToEvent(EPMTreePubSub.ON_COLLAPSE, this, "handleTreeCollapse");
            this._subscribeToEvent(EPMTreePubSub.ON_EXPAND, this, "handleTreeExpand");
            this._subscribeToEvent(EPMTreePubSub.ON_CREATE_NODE, this, "_wait_n_updateModel");
			this._subscribeToEvent(EPMTreePubSub.ROOT_NODES_LOADED, this, "_check_n_updateModel");
			this._subscribeToEvent(EPMTreePubSub.ON_DELETE_NODE, this, "_wait_n_updateModel");
			this._subscribeToEvent(EPMTreePubSub.ON_NODE_REFRESH, this, "_wait_n_updateModel_refreshRows");
			this._subscribeToEvent(EPMTreePubSub.ON_NODE_DELETE, this, "_wait_n_updateModel_removeRows");
		},
		
		/**
		 * The post create method will be called soon after the widget is created.
		 */
        postCreate: function(){
            this.inherited(arguments);
            this._subscriptions = [];
			this._subscribeToAllEvents();
			this._isHidden_n_Dirty = false;
			this.markDirtyOnHidden = false;
        },
		
		handleTreeExpand: function(){
			console.debug("Call to the Expand: " + this);
			console.debug("Is view visible" + this.isViewActive());
			if(this._baseTreeID == EPMConstants.WBS_TREE_ID){
				// for WBS tree check for active.
				if(this.isViewActive() == false){
					this._isHidden_n_Dirty = true;
				}else{
					this._wait_n_updateModel(arguments);
				}
			}else{
				this._wait_n_updateModel(arguments);
			}
			
		},
		
		handleTreeCollapse: function(publishObj){
			console.debug("Call to the collapse: " + this);
			console.debug("Is view visible" + this.isViewActive());
			if (this._baseTreeID == EPMConstants.WBS_TREE_ID) {
				// for WBS tree check for active.
				if (!this.isViewActive()) {
					this._isHidden_n_Dirty = true;
				} else {
					this._wait_n_updateModel_onCollapse(publishObj);
				}
			} else {
				this._wait_n_updateModel_onCollapse(publishObj);
			}
		},
        
        _subscribeToEvent: function(/*Constant String*/topic, /*Object*/ context, /*String*/ handler){
            var handle = dojo.subscribe(topic, context, handler);
            this._subscriptions.push(handle);
        },
		/**
		 * Checks if this grid is for the tree which has published the event and then calls build model.
		 * @param {Object} publishedObj the object published from tree contains the tree id which published this event.
		 */
		_check_n_updateModel: function(publishedObj){
			var treeDetails = publishedObj;
            if (dojo.isArray(treeDetails) && treeDetails[0]) {
                treeDetails = publishedObj[0];
            }
			if(treeDetails.treeID && treeDetails.treeID == this._baseTreeID){
				this._buildModel = true;
				this.buildGridModel();
			}
		},

		/**
		 * Checks if controller has any active views, returns true if the view is active, else false
		 */
		isViewActive: function() {
			for (var i in this._viewers) {
				var view = this._viewers[i];
                if (view) {
					var viewlet = view._viewlet;
					if (viewlet) {
						var site = viewlet.getSite();
						if (site) {
							if (site.isActive) {
								var active = site.isActive();
								if (active) {
									return true;
								}
							}
						}
					}
				}
			}
			return false;
		},

		/**
		 * Checks if controller has any views with dirty viewlets, returns true if the view has dirty viewlet, else false
		 */
		isViewletDirty: function() {
			for (var i in this._viewers) {
				var view = this._viewers[i];
                if (view) {
					var viewlet = view._viewlet;
					if (viewlet) {
						// Check if the viewlet is marked dirty and user has not selected to discard the changes
						var dirty = viewlet.isDirty() && (!viewlet.isDiscarded());
						return dirty;
					}
				}
			}
			return false;
		},
		
		/**
		 * Builds the list of modified rows from the given list including any modified children rows as well
		 */
		getModifiedChildRows: function(rows) {
			var EPM_Constants = com.ibm.epm.web.core.ui.internal.EPMConstants;
			
			// Array of rows modified
			var rowsToBeSaved = [];
            for (var i = 0; i < rows.length; i++) {
				var rowId = rows[i].identifier[0];
				// Ignore the temp child added
				if ((rowId) && (rowId.indexOf(EPM_Constants.TEMPCHILD) == -1)) {
					var rowIndex = this.getRowIndex(rowId);
					if (rowIndex > 0) {
						var row = this.getRowByIndex(rowIndex);
						if (row) {
							var saveObject = this._gridModel.getSaveObject(row);
							if (saveObject != null) {
								var rowToBeSaved = {
									saveObject: saveObject,
									row: row
								};
								// Set the isParentCollapsed flag to true
								rowToBeSaved.saveObject.isParentCollapsed = true;
								rowsToBeSaved.push(rowToBeSaved);
							}
							// The main resource might not be modified but the link might be
							// modified hence saving the links. The links save will not be sequential.
							this._saveLinks(null, row);
						}
					}
					
					// recursively call for all children
					var childRows = rows[i].children;
					if (childRows) {
						var childRowsToBeSaved = this.getModifiedChildRows(childRows);
						for (var j = 0; j < childRowsToBeSaved.length; j++) {
							rowsToBeSaved.push(childRowsToBeSaved[j]);
						}						
					}
				}
			}
			return rowsToBeSaved;
		},

		/**
		 * Check if the view has been marked dirty, in case of overview viewlet, check if schedule viewlet is initialized and is dirty, then 
		 * ignore the event from overview and consider the schedule viewlet
		 */
		_checkIfScheduleviewDirty: function() {
			var dirty = this.isViewletDirty();
			if ((dirty) && (this._baseGridID == EPMConstants.OVERVIEW_GRID_ID)) {
				var controllerID = com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants.ScheduleGridController;
				var scheduleController = controllerUtil.getTreeBasedGridController(controllerID);
				if (scheduleController.isInitialized()) {
					return (!scheduleController.isViewletDirty());
				}
			}			
			return dirty;
		},
		
		/**
		 * Event handler on collapse of the tree node, check if the collapsed node has any unsaved children,
		 * if yes save it and then update the grid model
		 */
		_wait_n_updateModel_onCollapse: function(publishObj) {
			// Check if the controller has any active view, any view marked dirty or non viewlets but based on EPM tree
			if (this.isViewActive() ||
				this._checkIfScheduleviewDirty() ||
				(this._baseTreeID && (this._baseTreeID != EPMConstants.WBS_TREE_ID))) {
				// Check for unsaved children items and save it on collapse
				if (publishObj && publishObj.children) {
					var children = publishObj.children;
					
					// Need to add an undefined element at the start of the array to fix the count issue
					// since persistGrid is doing it, post 1.0 need to fix in both places
					var collapsedRowsToBeSaved = new Array(1);
					var rowsToBeSaved = this.getModifiedChildRows(children);
					collapsedRowsToBeSaved = collapsedRowsToBeSaved.concat(rowsToBeSaved);
					var modifiedRowsCount = collapsedRowsToBeSaved.length;
					if ((modifiedRowsCount > 1) && (!saveForCollapseInProgress)) {
						
						saveForCollapseInProgress = true;					
						this.saveRequests = modifiedRowsCount - 1;
						Util._lockPage(CoreMessages.SAVING);

						this._persistSequentially(collapsedRowsToBeSaved, 1, modifiedRowsCount);
						
						// Wait for the save to complete after completion toggle the save button and refresh the parent collapsed row
						var self = this;
						var intervalID = setInterval(function(){
							if (self.saveRequests <= 1) {
								clearInterval(intervalID);								
								self._updateAfterSave(publishObj);
							}
						}, 50);
					}
				}
				if (publishObj.collapseAll) {
					this._checkSaveButton();
				}
			}

			this._wait_n_updateModel();
		},

		/**
		 * Check if the save button needs to be enabled or disabled
		 */
		_checkSaveButton: function() {
			// Check if save button needs to be enabled
			var checkForModifiedInGrid = this.checkForUnsavedRows(null);
			for (var i in this._viewers) {
				var view = this._viewers[i];
				if (view) {											
					if (view._viewlet) {
						// There are more changes in the WBS grid or tree, toggle the save button
						view._viewlet.toggleSaveButton(!checkForModifiedInGrid);
					} else if (view._containingPage){
						// Handle the save button in the admin view
						view._containingPage.disableSaveButton(!checkForModifiedInGrid);
					} else if (view.containerViewlet) {
						// Handle the save button for the pools viewlet
						view.containerViewlet.enableSaveButton(checkForModifiedInGrid);
					}
				}
			}
		},
		
		/**
		 * Update the collapsed parent, toggle save button
		 */		
		_updateAfterSave: function(publishObj) {
			// Clear the flag to accept more save requests
			saveForCollapseInProgress = false;

			this._checkSaveButton();
			
			// update the parent node
			if (this._baseTreeID == EPMConstants.WBS_TREE_ID) {
				parentRowId = publishObj.identifier[0];
				this._updateParent = true;
				this._getAndUpdateRow(parentRowId, parentRowId, null);
			}
		},
		
        _wait_n_updateModel: function(){
            var self = this;
            if (this._tree) {
                this._isBuzy = true;
                var allVisibleItems = self._tree.getAllVisibleItems();
                self.updateModel(true, allVisibleItems);
            }
        },
        
		/**
		 * To refresh a set of rows as a ripple effect of tree refresh.
		 * @param {Object} publishedObject
		 */
        _wait_n_updateModel_refreshRows: function(publishedObject){
			if (publishedObject && publishedObject.treeID) {
				if(publishedObject.treeID == this._baseTreeID){
					var collectionItemHandle = publishedObject.collectionItemHandle;
					if (this._tree) {
						this._isBuzy = true;
						this.refreshSetOfRows(collectionItemHandle);
					}
				}
			}
        },

		/**
		 * Remove rows from the grid and propagate removal to gantt.
		 * @param {Object} publishedObject
		 */		
		_wait_n_updateModel_removeRows: function(publishedObject){
            if (publishedObject && publishedObject.treeID) {
				if (publishedObject.treeID == this._baseTreeID) {
					var collectionItemHandle = publishedObject.collectionItemHandle;
					if (this._tree) {
						this._isBuzy = true;
						this.removeUnsavedRows(collectionItemHandle);
					}
				}
			}
        },
		/**
		 * Waits for some time to get the tree.
		 * Once tree is got it gets the visible items from the tree. 
		 * Calls the abstract grid method to update the model.
		 */
        buildGridModel: function(){
			if(!this._buildModel){
				return;
			}
            var self = this;
            var count = 1;
            var intervalID = setInterval(function(){
                if (self._tree != null) {
					var visibleitems = self._tree.getAllVisibleItems();
					clearInterval(intervalID);
					self.updateModel(false, visibleitems);
                }else if (count > 20) {
					//wait for only 20 times in case of empty tree and set time out
					clearInterval(intervalID);
                }else {
                    self._tree = dijit.byId(self._baseTreeID);
                }
                count = count + 1;
            }, 50);
        },
        
        checkForReplaceNode: function(nodeToBeReplaced){
            //Temp fix. Actual fix should be at published event level
            if (nodeToBeReplaced.uniqueIdentifier &&
            nodeToBeReplaced.uniqueIdentifier.treeID &&
            nodeToBeReplaced.uniqueIdentifier.treeID == this._baseTreeID &&
            (this._gridData)) {
                return true;
            }
            return false;
        },
        
        /**
         *  _removeWBSElement : Method to remove the selected WBS element
         *  Note: Adding this method here as its used by the WBS Grid and Gantt views
         *  Incoming : TreeNode - nodeWidget
         *  Outgoing : none
         */
        _removeWBSElement: function(/*TreeNode*/ nodeWidget, depSelected){
			this._tree.confirmAndRemoveNode(nodeWidget, depSelected);
			if (!this._tree.hasNewNode) {
				var deletedRowId = dojo.isArray(nodeWidget.item.identifier)?nodeWidget.item.identifier[0]:nodeWidget.item.identifier;
				var checkForModifiedInGrid = this.checkForUnsavedRows(deletedRowId);
				if (!checkForModifiedInGrid) {
					for (var i in this._viewers) {
						var view = this._viewers[i];
						if (view && view._viewlet) {
							// There are no changes in the grid or tree, disable the save button
							view._viewlet.toggleSaveButton(true);
						}
					}
				}
			}
        },
        
        /**
         * Clear the error message
         */
          _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },

        /**
         * Display and clear the error message 
         * incoming : Error Message Object
         * OutGoing : None
         */
        _displayAndClearErrorMessage: function(errorMessage) {
            var self = this;
            EPMErrorHandler.setError(errorMessage);
            
            // Set an interval and clear the error message
            var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 30000);  
        },
                
        addUniqueIdentifier: function(publishObj){
            publishObj.treeID = this._baseTreeID;
        },
        
        /**
         * Handler to update the parent-child links on the server
         * after a drag-n-drop operation in the tree.
         * 
         * publishObj = {
                sourceItem: source item,
                targetItem: target item (new parent),
                oldParentItem: old parent item,
                treeID: treeID (Tree where DND was initiated),
                postToServer: postToServer (Boolean to indicate if we need to POST to server)
         * }
         * 
         * postToServer: true (if POST to server is needed)
         *               false (if POST to server is *not* needed).
         *               
         * The latter will be the case when we did an earlier POST after DND and it failed.
         * Now, we just need to update the row's parentId reference back to the original
         * parent but not POST again to the server.
         */
        _onParentChildChange: function(publishObj) {
            if (publishObj && publishObj.treeID === this._baseTreeID) {
                if (publishObj.sourceItem && publishObj.targetItem && publishObj.oldParentItem) {
                    var sourceID = publishObj.sourceItem.identifier[0];
                    var sourceGridItem = this._gridData.store._getItemByIdentity(sourceID);
                    var targetID = publishObj.parentIdentifierID;
                    var targetGridItem = this._gridData.store._getItemByIdentity(targetID);
                    if (sourceGridItem && targetGridItem) {
                        
                        var sourceRowIndex = this._gridData.getRowByIdentity(sourceID);
                        var row = this._gridData.getRow(sourceRowIndex);
                        if (row) {
	                        // Store the new parent reference
                            row.parentId = targetID;
                            row.rpcSequenceValue = publishObj.rpcSequenceValue;
                            // Check if we need to really save this change to the server or not
                            // The latter will be the case when the parent change could not
                            // be persisted on the server. In that case, we just need to update
                            // the parentId reference of the row but not do a POST to the server.
                            if (publishObj.postToServer) {
                                var saveObject = this._gridModel.getSaveObject(row);
                                if (!saveObject) {
                                    // NOTE: There is no reason why the saveObject should ever be null/undefined
                                    // since we just changed row.parentId. Also, this code is hit when
                                    // the user does a Drag-n-drop in the tree. The fact that the saveObject 
                                    // is null means that the nodes were not aligned correctly in the 
                                    // tree somehow (based on the parent-child links from the server). Hence, we  
                                    // must preserve the user initiated action and will build our own
                                    // saveObject with just the right data to persist the change.
                                    
                                    console.log("WARNING: Nodes not aligned before DND");
                                    console.log("WARNING: DND Data -> Source:" + sourceID + 
                                        " Target:" + targetID +
                                        " Old Parent:" + publishObj.oldParentItem.identifier[0]
                                    );
                                    
                                    var params = {
                                        Parent: targetID,
										rpcSequenceValue: publishObj.rpcSequenceValue
                                    };
                                    
                                    saveObject = {
                                        isNew: false,
                                        params: params,
                                        rowURI: row.href  
                                    };     
                                }    
                                
								saveObject.params.updateSequenceValueWithParent = true;
                                // Make the server call to persist the change                                 
                                this._saveOnParentChildChange(saveObject, row, sourceRowIndex, publishObj);
                                
                            }    
                        }
                    }
                }
            }        
        },
        
        /**
         * Just makes the post call to the server on the url specified by the save object.
         * @param {Object} saveObject: The object which has the uri and post params.
         * @param {Object} row : The row that will be saved.
         * @param {int} rowIndex : The index of the Row.
         * @param {Object} publishObj: Object to be published on save {
                sourceItem: source item,
                targetItem: target item (new parent),
                oldParentItem: old parent item,
                treeID: treeID (Tree where DND was initiated),
                postToServer: postToServer (Boolean to indicate if we need to POST to server)
         * }
         */
        _saveOnParentChildChange: function(saveObject, row, rowIndex, publishObj){
            var rowID = row.id;
            var self = this;
            
            var persistRowFunctions = {
                _persistRowSuccess: function(savedElement){
                    // Publish an event for the tree to update the parent child links
                    // of the node that moved wrt its original parent and the new parent
                    
                    // Indicate that the POST was successful
                    publishObj.error = false;
                    dojo.publish(EPMTreePubSub.ON_PARENT_CHILD_SAVE, [publishObj]);      
                },
                
                _persistRowFailure: function(errorMsg){
                    // On failure, we will be moving the moved node back to its original parent
                    // with a message on why it failed to persist the change.
                    console.log("ERROR: DND Save Failure Due To: " + errorMsg.message);
                    console.log("ERROR: DND Data -> Source:" + publishObj.sourceItem.identifier + 
                        " Target:" + publishObj.targetItem.identifier +
                        " Old Parent:" + publishObj.oldParentItem.identifier
                    );
                    self._displayAndClearErrorMessage(errorMsg.message);
                    
                    // Indicate that the POST was *not* successful
                    publishObj.error = true;
                    dojo.publish(EPMTreePubSub.ON_PARENT_CHILD_SAVE, [publishObj]);
                }
            };
            
            var srh = new ServiceResponseHandler(persistRowFunctions, "_persistRowSuccess", "_persistRowFailure");
            if (saveObject && saveObject.rowURI && saveObject.params) {
                // Lock the page when the DND structure change is in progress
                Util._lockPage(CoreMessages.DND_IN_PROGRESS);    
                GridModelClient.postURI(srh, saveObject.rowURI, saveObject.params);
            }
        },
        
        /*
         * Synchronize the grid to match the structure in the tree
         * 
         * publishObj = {
                sourceItem: source item,
                targetItem: target item (new parent),
                oldParentItem: old parent item,
                treeID: treeID (Tree where DND was initiated)
         * }
         */
        _onParentChildChangeSyncGrid: function (publishObj) {
            
            if (publishObj && publishObj.treeID === this._baseTreeID) {
                // First remove the rows
                this._wait_n_updateModel_removeRows(publishObj)
                
                // Next add the rows
                this._wait_n_updateModel(publishObj);
            }    
        },

        /*
         * Indicates if the grid or the tree associated with the controller is being edited
         */		
		isEditInProgress: function() {
			// Check if the tree is being edited
			if ((this._tree) && (this._tree.editing)) {
				return true;
			}

			// Check if any of the viewers are being edited
			for (var i in this._viewers) {
				var view = this._viewers[i];
				if (view && (view.cellEditing)) {
					return true;
				}
			}

			return false;	
		}
    });
})();
