/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.grid.view.AbstractGridView");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGrid");
    
    // EPM Core Constants
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    
    // Events published by the EPMTree
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    
    dojo.declare("com.ibm.epm.web.core.grid.view.AbstractGridView", [dijit._Widget, dijit._Templated], {
        lastSelectedWBSElement: "",
        _treeID: null,
        _isBuzy: false,
        _subscriptions: null,
		overRideCellClasses: true,
        
        constructor: function(treeID){
            if (treeID != null && treeID != "") {
                this._treeID = treeID;
            }
            else {
                this._treeID = null;
            }
            
            this._subscriptions = new Array(1);
			
			this._pconnects = [];
            
            // Listen to the EPMTree mouse hover/select events
            this._subscribeToEvent(EPMTreePubSub.ON_SELECT, this, "_onSelect");
            
            // Listen to the EPMGrid mouse hover/select events
            this._subscribeToEvent(EPMGridPubSub.ON_ROW_CLICK, this, "_selectGridOnRowClick");
			
			// Listen to the EPMGrid mouse hover/select events
            this._subscribeToEvent(EPMGridPubSub.ON_HEADER_CELL_MOUSE_OVER, this, "_onHeaderCellMouseOver");
            
            //Listen to the new row selection
            this._subscribeToEvent(EPMGridPubSub.ON_ROW_SELECTED, this, "_afterRowSelection");
            
			/*
			 * Handling this event is necessary as when the side navigation is adjusted, the grid should be redrawn 
			 */		
			 
			this._subscribeToEvent("jazz.ui.SideNavigationLayout.slideLeftNav.onEnd", this, "redrawGrid");		
            
            // Handlers for keyboard events
            this._keyDownFromTreeEvent = null;
            this._keyDownFromGridEvent = null;
        },
		
		/**
		 * Redraw the grid.
		 * @param {Object} event
		 */
		redrawGrid: function(event){
			
            if (this._generalGrid) {
                if (this._generalGrid.update) {
                    this._generalGrid.update();
                }    
            }     
		},
		
		// Change the tree to instruct the controller to fetch the data from new URI
		setTreeId:function(treeId){
			
			this._treeID = treeId;
			
		},
        
        _subscribeToEvent: function(/*Constant String*/topic, /*Object*/ context, /*String*/ handler){
            var handle = dojo.subscribe(topic, context, handler);
            this._subscriptions.push(handle);
        },
        
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions != null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
        },
        
        destroy: function(){
			console.log("destroying the grid view: " + this.id);

			dojo.forEach(this._pconnects, dojo.disconnect);
			this._pconnects = [];
			
            if (this._controller) {
                this._controller.destroy();
                delete this._controller;
            }
			var gridAttachPoint = this.getAttachpoint();
            if (this._generalGrid && this._generalGrid.domNode) {
            	if(gridAttachPoint && gridAttachPoint.children && gridAttachPoint.children.length > 0){
                	gridAttachPoint.removeChild(this._generalGrid.domNode);
                }
                delete this._generalGrid;
            }
            this._unsubscribeToAllEvents();
            this.unsubscribeToKeyboardEvents();
        },
        
        buildGrid: function(viewer, model, controller, linksToBeLoaded){
            if (!this._controller) {
                if (controller == null) {
                    throw "controller cannot be null when controller is not created";
                }
                this._controller = controller;
            }
            if (this._controller.isInitialized() == false) {
                this._controller.initializeController(this._treeID, viewer, model, linksToBeLoaded);
            }
            if (!this._generalGrid) {
                var dataModel = this._controller.getDataModel();
                if (dataModel) {
                    this.createView(dataModel);
                }
            }
			this._controller.sortableFields = this.getSortableColums();
            this._controller.buildGridModel();
        },
		
		//default method if no sortable columns are defined in child classes
		getSortableColums:function()
		{
			var sortableFields ={};			
			return sortableFields;
		},
		
		getViewGrid: function(){
			if(this._generalGrid){
				return this._generalGrid;
			}
		},
        
        /**
         *  _selectGridOnMouseHover : Event handler for mouse hover event of EPMGrid, can be overridden in the views
         *  Incoming :
         *  	data - event object
         *		data.gridID - unique identifier of the grid
         *		data.row - rowIndex
         *  Outgoing : none
         *
         */
        _selectGridOnMouseHover: function(data){
        
        },
        
        /**
         *  _deselectGridOnMouseOut : Event handler for mouse out event of EPMGrid, can be overridden in the views
         *  Incoming :
         *  	data - event object
         *		data.gridID - unique identifier of the grid
         *		data.row - rowIndex
         *  Outgoing : none
         *
         */
        _deselectGridOnMouseOut: function(data){
        
        },
        
        /**
         *  _selectGridOnRowClick : Event handler for row click event of EPMGrid, can be overridden in the views
         *  Incoming :
         *  	data - event object
         *		data.gridID - unique identifier of the grid
         *		data.row - rowIndex
         *  Outgoing : none
         *
         */
        _selectGridOnRowClick: function(data){
        
        },
        
        /**
         *  _onMouseOver : Event handler for mouse over event of EPMTree
         *  Incoming :
         *  	data - nodeWidget.item; // Store Item for the node
         * 		data.treeID		: The unique identifier.
         *  Outgoing : none
         *
         */
        _onMouseOver: function(data){
            var rowIndex = this._validateAndGetRow(data);
            if (rowIndex >= 0) {
                if (this._generalGrid) {
                    // Add the new row to the selection list
                    var event = {
                        rowIndex: rowIndex
                    };
                    this._generalGrid.onMouseOverRow(event);
                }
            }
        },
        
        /**
         *  _onMouseOut : Event handler for mouse out event of EPMTree
         *  Incoming :
         *  	data - nodeWidget.item; // Store Item for the node
         * 		data.treeID		: The unique identifier.
         *  Outgoing : none
         *
         */
        _onMouseOut: function(data){
            var rowIndex = this._validateAndGetRow(data);
            if (rowIndex >= 0) {
                if (this._generalGrid) {
                    // Check if the highlighted row is a selected row
                    if ((this._generalGrid._selected == null) || (this._generalGrid._selected !== rowIndex)) {
                        // Deselection takes place only if no rows are selected <i.e. this._generalGrid._selected == null>
                        // or if the row isnt a selected row
                        var event = {
                            rowIndex: rowIndex
                        };
                        this._generalGrid.onMouseOutRow(event);
                    }
                }
            }
        },
        
        /**
         *  _onSelect : Event handler for node select event of EPMTree
         *  Incoming :
         *  	data - nodeWidget.item; // Store Item for the node
         * 		data.treeID		: The unique identifier.
         *  Outgoing : none
         *
         */
        _onSelect: function(data){
            var rowIndex = this._validateAndGetRow(data);
            if (rowIndex >= 0) {
                if (this._generalGrid) {
                    // Select this row as selected
                    this._generalGrid.selection.select(rowIndex);
                    // Cache the selected node for future processing
                    this._generalGrid._selected = rowIndex;
                }
            }
        },
        
        /**
         *  _onBlur : Event handler for on Blur event of EPMTree
         *  Incoming :
         * 		data.treeID		: The unique identifier.
         *  Outgoing : none
         *
         */
        _onBlur: function(data){
            var bFound = this._validateTreeId(data);
            if ((bFound == true) && (this._generalGrid)) {
                // Deselect the previously selected node
                this._generalGrid.selection.deselect(this._generalGrid._selected);
                // Reset the _selected variable
                this._generalGrid._selected = null;
            }
			this._generalGrid.edit.apply();
        },
        
        /**
         * _validateAndGetRow: Method to validate the published data and verify the tree id.
         * Incoming: publishedItem by tree.
         * Outgoing: rowIndex.
         *
         */
        _validateAndGetRow: function(publishedItem){
            var rowIndex = -1;
            if (this._validateTreeId(publishedItem) == true) {
                var dataModel = this._controller.getDataModel();
                if (dataModel) {
                    rowIndex = dataModel.getRowByIdentity(publishedItem.identifier);
                }
            }
            return rowIndex;
        },
        
        /**
         * _validateTreeId: Method to validate if the published data specified the controller's tree ID.
         * Incoming: publishedItem by tree.
         * Outgoing: true if tree id matches, false if not.
         *
         */
        _validateTreeId: function(publishedItem){
            var bFound = false;
            if ((publishedItem) && (this._controller)) {
                // Process the event only if the treeID matches with the controller's tree ID
                if (publishedItem.treeID == this._controller._baseTreeID) {
                    bFound = true;
                }
            }
            return bFound;
        },
        
        /**
         * createView: Creates the viewer [EPMGrid]
         * Incoming:
         *	gridModel
         * Outgoing:
         *  none
         */
        createView: function(gridModel){
            var gridLayout = this.getGridLayout();
            // a grid layout is an array of views.
            var layout = [gridLayout];
            var title = this.getGridTitle();
            
            this._generalGrid = new com.ibm.epm.web.core.ui.internal.EPMGrid({
                title: title,
                model: gridModel,
                structure: gridLayout,
				overRideCellClasses: this.overRideCellClasses
            });
            
            if (this._generalGrid) {
				this._generalGrid._gridID=this.getGridInternalID();
                this._controller._baseGridID = this._generalGrid._gridID;
                this.getAttachpoint().appendChild(this._generalGrid.domNode);
				this._pconnects.push( dojo.connect(this._generalGrid, "onStyleRow", this, this._onStyleRow) );
                this._generalGrid.isCellEditable = dojo.hitch(this, this.isCellEditable);
                this._generalGrid.update();
                this.update();
            }
        },
		
		isCellEditable: function() {
			return true;
		},
		
        /**
         * _onStyleRow: Implement onStyleRow to decorate the row object
         * Incoming: row domnode
         * Outgoing: none
         */
		_onStyleRow: function(row) {
			if (this._controller && this._controller._gridData && this._controller._gridData.getRow) {
				var modelRow = this._controller._gridData.getRow(row.index);
				if (modelRow && modelRow.errorObject) {
					row.customClasses += " epmGridErrorRow";
				}
			}
		},
		
		_markRowAsInvalid: function(error, rowID){
			var rowIndex = this._controller.getRowIndex(rowID);
			var view = this._generalGrid.views.views[0];
			
			var cellCnt = this._generalGrid.layout.cellCount;
			for(var i = 0; i < cellCnt; i++){
				var cellNode = view.getCellNode(rowIndex, i);
				if(cellNode){
					dojo.style(cellNode, "background", "#FFE0E0");
				}
			}
		},
        
        /**
         * getGridTitle: Used to set the title of the grid, needs to be overridden by the inheriting view
         * Incoming:
         *	gridModel
         * Outgoing:
         *  none
         */
        getGridTitle: function(){
            console.log("getGridTitle() is to be overwritten by the inheriting view to set the Grid's title");
            return EPMConstants.DEFAULT_GRID_TITLE;
        },
		  /**
         * getGridInternalID: Used to set the unique ID of the grid, needs to be overridden by the inheriting view
         * Incoming:
         *	gridModel
         * Outgoing:
         *  none
         */
        getGridInternalID: function(){            
            return EPMConstants.DEFAULT_GRID_TITLE;
        },
        
        getAttachpoint: function(){
            console.log("getAttachpoint() is to be overwritten by the inheriting grid for its attach point");
        },
        
        /**
         * Checks whether the grid and controller are present.
         * Then calls the persistGrid of the controller.
         * The persistGrid call is by default parallel. To make sequential,
         * pass in the optional arg with a value of true. lockPage only locks
         * for the duration of the save if synchronizedSave is true, otherwise
         * the lock lasts long enough just to initiate the asynchronous requests
         * but not to wait for them to complete.
         * @param {boolean} synchronizedSave
         * @param {boolean} lockPage  - if true, locks the page during save 
         */
        persistGrid: function(synchronizedSave, lockPage){
            if (!synchronizedSave) {
                synchronizedSave = false;
            }
			lockPage = (lockPage==false) ? lockPage : true;
            if (this._generalGrid) {
                this._generalGrid.edit.apply();
                if (this._controller && this._controller.persistGrid) {
                    this._controller.persistGrid(synchronizedSave, lockPage);
                }
            }
        },
        
        refreshGrid: function(){
            if (this._generalGrid) {
                this._generalGrid.edit.cancel();
                this._generalGrid.model.measure();
                
                if (this._controller && this._controller.refreshGrid) {
                    this._controller.refreshGrid();
                }
            }
        },
        
		discardUnsavedGridChanges: function(){
            if (this._generalGrid) {
                this._generalGrid.edit.cancel();
                this._generalGrid.model.measure();
                
                if (this._controller && this._controller.discardUnsavedGridChanges) {
                    this._controller.discardUnsavedGridChanges();
                }
            }
		},
		
        rowUpdated: function(row, rowID, rowIndex){
            if (this._generalGrid) {
                this._generalGrid.updateRow(rowID);
            }
        },
        
        /**
         * newRowInserted: Method to insert the new row in the grid view
         * Incoming:
         * 	row - data in the row,
         *  rowID - row id used in the model,
         *  rowIndex - row index used in the grid view
         * Outgoing: none
         *
         */
        newRowInserted: function(row, rowID, rowIndex){
        
            if (this._generalGrid) {
                this._generalGrid.updateRow(rowID);
            }
            
            // select the newly added row
            if (rowID.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1) {
                this._selectRow(rowID);
            }
        },
        
        removeRows: function(removedRows){
            //TODO if something needs to be done on collapse node it wil be done here
        },
        
		/**
		 * scrollToRow: Scroll to the specified row index. If row does not exist, this
		 * will just log it and do nothing. If pagination is in use, it'll also change
		 * the currently shown page to contain this row. This does nothing if the grid
		 * does not have enough rows to require a scrollbar or pagination.
		 * 
		 * Incoming:
		 *  rowIndex - row index used in the grid view
		 */
		scrollToRow: function(rowIndex) {
            this._generalGrid.scrollToRow(rowIndex);			
		},
		
		/**
		 * scrollToSelectedRow: If a row is selected, scrolls it into view by calling
		 * scrollToRow() with its row number. If more than one row is selected, the first
		 * selection is used. If no row is selected, this will just log it and do nothing.
		 */
		scrollToSelectedRow: function() {
            var rowIndex = this._generalGrid.selection.selectedIndex;
			if (rowIndex >= 0)
			    this.scrollToRow(rowIndex);
		    else
		        console.log("scrollToSelectedRow: no selected row to scroll to");		
		},
		
		/*
		 * Automatically starts editing the first cell of the selected row.
		 * This is useful to call just after adding a new row (which will most likely
		 * need a name change after adding the row to a grid.)
		 */
		startFirstCellEdit: function() {
		    if (this._generalGrid) {
                var rowIndex = this._generalGrid.selection.selectedIndex;
                var cell = this._generalGrid.getCell(0);
                        
		    	if (rowIndex >= 0){
			        this._generalGrid.focus.setFocusCell(cell, rowIndex);
					this._generalGrid.edit.setEditCell(cell, rowIndex);					
				}
		        else
		            console.log("startFirstCellEdit: no selected row to edit");
		    }		
		},
		
		/**
		 * scrollToBottomRow: Scrolls the last row into view by calling
		 * scrollToRow() with its row number. It will do nothing if the grid has no rows.
		 */
		scrollToBottomRow: function() {
			var rowIndex = this._generalGrid.scroller.rowCount;
			if (rowIndex > 0)
			    this.scrollToRow(rowIndex);
		},
		
        /**
         * _selectRow: Method to insert the new row in the grid view
         * Incoming:
         * 	rowID - id of the new element,
         * Outgoing: none
         *
         */
        _selectRow: function(rowID){
            var rowIndex = this._controller.getRowIndex(rowID);
            if ((rowIndex >= 0) && (this._generalGrid)) {
                // Select this row as selected
                this._generalGrid.selection.select(rowIndex);
                
                // Cache the selected row for future processing
                this._generalGrid._selected = rowIndex;
                
                var publishObj = {
                    gridID: this._generalGrid._gridID,
                    rowIndex: rowIndex
                };
                dojo.publish(EPMGridPubSub.ON_ROW_CLICK, [publishObj]);
            }
        },
        
        /* selectGridRow: function(newElementPrefix){
         var selectedRowIndexes =  this.selection.getSelected();
         if(selectedRowIndexes != null && selectedRowIndexes.length > 0){
         var selectedRow = this.model.data[selectedRowIndexes[0]];
         if(selectedRow){
         if(selectedRow.id != this.lastSelectedElement && selectedRow.id.indexOf(newElementPrefix) == -1){
         dojo.publish("SelectRow", [selectedRow]);
         this.lastSelectedElement = selectedRow.id;
         }
         }
         }
         },*/
		/**
		 * Some of the grids need a scrollbar when their height reaches more than the minimum height.
		 * For example, timesheet and admin grids.
		 */
        update: function(){
            var s = dojo.contentBox(this.domNode);
            if (this._generalGrid) {
                this._generalGrid.resizeGrid(s);
            }
        },
        
		/**
		 * Tree based grids need to grow automatically with the tree. Hence we'll resize the grid with autoHeight.
		 */
		updateWithAutoHeight: function(){
            var s = dojo.contentBox(this.domNode);
            if (this._generalGrid) {
                this._generalGrid.resizeGridWithAutoHeight(s);
            }
		},
		
        handleError: function(errorMessage, modelRow, rowID){
			//Default implementation: Mark the rows to indicate the errors occured for this row
			if (modelRow && rowID) {
				var rowIndex = this._controller.getRowIndex(rowID);
				modelRow.errorObject = {
					hasError: true,
					errorMessage: errorMessage
				}
				this._generalGrid.updateRowStyles(rowIndex);
			}
			//Now that we have a default implementation, commenting the console statement
            //console.log("handleError is to be overridden by the individual viewer");
        },
        
        _displayError: function(errorMessage){
            var self = this;
            EPMErrorHandler.setError(errorMessage.errorMessage, errorMessage.errorCode);
            // Set an interval and clear the error message
            var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);
        },
        
        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },
        
        /*
         * Method for handling keyboard actions initiated from the tree. 
         */
         _onKeyDownFromTree: function(data) {
             // Empty implementation for sub-classes to override
         },
         
        /*
         * Method for handling keyboard actions initiated from the grid.
         */
        _onKeyDownFromGrid: function(data) {
           // Empty implementation for sub-classes to override     
        },
                
        /*
         * Method for handling parent-child change (drag-n-drop) actions initiated from the tree.
         */
        _onParentChildChangeSyncGrid: function(data) {
           // Empty implementation for sub-classes to override     
        },
        
        subscribeToKeyboardEvents: function() {
            
            if (this._keyDownFromTreeEvent == null) {
                // Listen to EPMTree Key Down events
                this._keyDownFromTreeEvent = dojo.subscribe(EPMTreePubSub.ON_KEY_DOWN, this, this._onKeyDownFromTree);
            }    
             
            if (this._keyDownFromGridEvent == null) { 
                // Listen to the EPMGrid Key Down events
                this._keyDownFromGridEvent = dojo.subscribe(EPMGridPubSub.ON_KEY_DOWN, this, this._onKeyDownFromGrid);
            }    
        },
        
        unsubscribeToKeyboardEvents: function() {
            
            if (this._keyDownFromTreeEvent) {
                // Unsubscribe from EPMTree Key Down events
                dojo.unsubscribe(this._keyDownFromTreeEvent);
                this._keyDownFromTreeEvent = null;
            }    
            
            if (this._keyDownFromGridEvent) { 
                // Unsubscribe from EPMGrid Key Down events
                dojo.unsubscribe(this._keyDownFromGridEvent);
                this._keyDownFromGridEvent = null;
            }    
        },
        
        isUnsavedRow: function(rowID) {
            var unsaved = false;
            if (rowID.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1) {
                unsaved = true;
            }
            
            return unsaved;
        },
        
        saveChangesToModel: function() {
            if (this._generalGrid) {
                this._generalGrid.edit.apply();
            }    
        },
		
		/**
         *  _onHeaderCellMouseOver : Event handler for row click event of EPMGrid, can be overridden in the views
         *  Incoming :
         *  	data - event object
         *		data.gridID - unique identifier of the grid
         *  Outgoing : none
         *
         */
        _onHeaderCellMouseOver: function(data){
			if (data.cellNode) {
				if (data.cell.title) {
					data.cellNode.title = data.cell.title;
				}else {
					data.cellNode.title = data.cell.name;
				}
			}
        },
        
		/**
         *  _afterRowSelection : Event handler for row select event of EPMGrid, can be overridden in the views
         *  Incoming :
         *  	data - event object
         *		data.gridID - unique identifier of the grid
         *  Outgoing : none
         *
         */
        _afterRowSelection: function(data) {
        }
    });
})();
