/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.transport.EPMServerClient");

	dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
	dojo.require("jazz.app.auth");
	dojo.require("jazz.app.proxy");
	dojo.require("jazz.client.xhr");
	dojo.require("net.jazz.ajax.core.internal.util");

	dojo.require("dojo.i18n");
	dojo.requireLocalization("com.ibm.team.repository.web.transport", "TeamServerClientMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.transport", "TeamServerClientMessages");

    //
    // Provides a generic remoting mechanism to the team server's REST services.
    // @since 0.5
    //
    
    //
    // Aliases
    //
    
    var callContributedCode = net.jazz.ajax.core.internal.util.callContributedCode;
    
    /**
     * Invokes the service specified by <code>serviceRequest</code>
     * asynchronously and calls the appropriate function specified by
     * <code>serviceResponseHandler</code> in the context of
     * <code>serviceResponseHandler</code>.
     *
     * If the service returns successfully, invokeService will call back to
     * the function defined by handler.getCallbackName(); if the service call
     * results in an error, invokeService will call back to the function defined
     * by handler.getErrorCallbackName().
     *
     * @since 0.5
     */
    function invokeService(serviceRequest, serviceResponseHandler){
        var bindArgs = _getBindArgs(serviceRequest, serviceResponseHandler, false, true);
        _handleBindArgs(bindArgs, serviceRequest, serviceResponseHandler);
    }
    
    com.ibm.epm.web.core.transport.EPMServerClient.invokeService = invokeService;
    
    /**
     * Invokes the service specified by <code>serviceRequest</code>
     * asyncronously and calls the appropriate function specified by
     * <code>serviceResponseHandler</code> in the context of
     * <code>serviceResponseHandler</code>.
     *
     * If the service returns successfully, invokeService will call back to
     * the function defined by handler.getCallbackName(); if the service call
     * results in an error, invokeService will call back to the function defined
     * by handler.getErrorCallbackName().
     *
     * @since 0.5
     */
    function operateOnResource(serviceRequest, serviceResponseHandler, hasSoapEnvelope){
        var _hasSoapEnvelope = hasSoapEnvelope || false;
        var bindArgs = _getBindArgs(serviceRequest, serviceResponseHandler, true, _hasSoapEnvelope);
        _handleBindArgs(bindArgs, serviceRequest, serviceResponseHandler, _hasSoapEnvelope);
    }
    
    com.ibm.epm.web.core.transport.EPMServerClient.operateOnResource = operateOnResource;
        
    //
    // Internal consts and functions
    //
    
    var JAZZ_URL = dojo.baseUrl + "../../";

    var X_AUTH_MSG = "X-com-ibm-team-repository-web-auth-msg";
    var USERID_COOKIE = "JazzLoginCookie";
    
    function _handleBindArgs(xhrArgs, serviceRequest, serviceResponseHandler, hasSoapEnvelope){
		 var xhr = null;
		 if (serviceRequest.getMethodType() == "POST") {
            xhr = "rawXhrPost";
        }
        else 
            if (serviceRequest.getMethodType() == "PUT") {
                xhr = "rawXhrPut";
            }
            else {
                if (serviceRequest.getMethodType() == "DELETE") {
                    xhr = "xhrDelete";
                }
            	else {
                	xhr = "xhrGet";
            	}
			}
		// execute the remote method
		var deferred = jazz.client[xhr](xhrArgs);
		
		// add a standard callback for either success or error responses
		deferred.addBoth(function(response){
			handleCallback(response, deferred.ioArgs, serviceResponseHandler, hasSoapEnvelope);
		});
		
		// add a reference to the XHR object to the service request object to
		// support cancellation
		//serviceRequest._setDeferred(deferred);
    }

	//
	// Internal consts and functions
	//
	
	function handleCallback(response, ioArgs, serviceResponseHandler, hasSoapEnvelope) {
		if(!(response instanceof Error)) {
			// success callback
			
			var responseObj = _demarshallResponse(response, ioArgs, serviceResponseHandler, hasSoapEnvelope);
			if (responseObj != null) {			
				net.jazz.ajax.core.internal.util.callContributedCode(function(){
					serviceResponseHandler.doCallback(responseObj);
				});
				return;
			} 
			// Update call will only return the status. and response will be null.
			if (!response && ioArgs.xhr.status == 200 && ioArgs.xhr.statusText === "OK") {
				serviceResponseHandler.doCallback(null);
			}
		} else {
			// error callback
			
			if (dojo.isObject(response) && response.dojoType && response.dojoType === "cancel") {
				// cancellation case
				return; // early return - see Defect 71631
			}
			
			// remote error case
			var errorObj = {
				status: ioArgs.xhr.status
			};
			
			if(errorObj.status === 403) {
				// in the case of 403 Not Authorized, the JEE server will never call
				// our code, so we don't expect the response text to contain
				// anything useful
				dojo.mixin(errorObj, {
					message: messages.noAccessWeb,
					exception: []
				});
			}
			else {
				// in all other cases we expect that the Jazz server will have sent
				// a response body containing a JSON representation of the problem
				// that occurred
				
				// manually demarshall JSON since Dojo doesn't on error paths
				//Defect 14255: We are duplicating message and errorMessage. Donot remove this code
				//This is required since the default service expects it as "message" and Tara resource as "errorMessage"
				var responseObj = dojo.fromJson(response.responseText);
				dojo.mixin(errorObj, {
					errorClass: responseObj.errorClass,
					message: responseObj.errorMessage,
					errorMessage: responseObj.errorMessage,
					errorCode: responseObj.errorCode,
					exception: responseObj.errorTrace,
					errorData: responseObj.errorData
				});
			}
			
			// send error callback
			serviceResponseHandler.doErrorCallback(errorObj);
		}
	}    

    // Summary: return client time zone in format: GMT+5:45 or GMT-7:00
    // Date.getTimezoneOffset( ) returns the number of minutes difference 
    // between the GMT or UTC time and the local time.
    function _getTimeZoneOffset(){
        var tzOffset = (new Date()).getTimezoneOffset();
        if (tzOffset == 0) {
            return "";
        }
        var sign = "-";
        if (tzOffset < 0) {
            sign = "+";
            tzOffset = -tzOffset;
        }
        
        var minutes = (tzOffset % 60);
        if (minutes < 10) { // server does not accept: GMT8:0, only allow GMT8:00
            minutes = "0" + minutes;
        }
        var hour = Math.round(tzOffset / 60);
        if (minutes > 0) {
            hour--;
        }
        
        tzOffset = "GMT" + sign + hour + ":" + minutes;
        return tzOffset;
    }
    
    function _getBindArgs(serviceRequest, serviceResponseHandler, isResourceCall, hasSoapEnvelope){
		var isPOST = serviceRequest.getMethodType() === "POST";
		
		var xhrArgs = {
			url: _assembleURL(serviceRequest, isResourceCall),
			// ask the server to send JSON and expect form encoded data
			headers: {
				"accept": "text/json",
				"content-type": "application/x-www-form-urlencoded; charset=utf-8",
				"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
			},
			// ask Dojo to expect and automatically demarshall a JSON response body
			handleAs: "json",
			// if a post, add the query string to the document body
			postData: isPOST ? serviceRequest.getQueryString() : null
		};

        return xhrArgs;
    }

    /*
     * Assembles a URL, appending a query string to GET queries.  Handles both
     * non-secure and secure contexts.
     */
    function _assembleURL(serviceRequest, isResourceCall){
        var method = serviceRequest.getMethod();
        var methodType = serviceRequest.getMethodType();
        
        // assemble basic url
        var url = "";
        if (isResourceCall != null && isResourceCall === false) { // the service call.
            url = _getServicePrefix() + serviceRequest.getServiceInterface() + "/";
            var urlResource = method.substring(methodType.length, methodType.length + 1).toLowerCase() +
            method.substring(methodType.length + 1);
			if(dojo.isIE){
            	url = encodeURI(url + urlResource);
			}
			else{
				url = url + urlResource;
			}
        }
        else {
			if(dojo.isIE){
				url = JAZZ_URL + encodeURI(serviceRequest.getServiceInterface()) + "/";			
			}
			else{
				url = JAZZ_URL + serviceRequest.getServiceInterface() + "/";
			}
        }
        
        
        // if this is a GET request, append the query string
        if (methodType == "GET") {
            var tmpQueryString = serviceRequest.getQueryString();
            var queryString = tmpQueryString ? ("?" + tmpQueryString) : "";
            url += queryString;
        }
        return url;
    }
  
	/**
	 * The service prefix makes assumptions about the Jazz URL structure based on:
	 * 
	 * - Where Dojo lives
	 * - Where services live
	 * 
	 * These are really assumptions based on the Jazz 0.6 architecture and should
	 * not be used for anything other than traditional services.
	 */
	function _getServicePrefix() {
		var dataServerRoot = dojo.getObject("net.jazz.ajax._dataServerRoot");
		return dataServerRoot ? dataServerRoot + "/service/" : dojo.baseUrl + "../../service/";
	}
	
    // export for the sake of workitem.web, which depends on this (though it shouldn't)
    com.ibm.epm.web.core.transport.EPMServerClient._getServicePrefix = _getServicePrefix;
    
    //
    // Internal demarshalling functions
    //
    
    function _demarshallResponse(evaldObj, ioArgs, serviceResponseHandler, hasSoapEnvelope){
        if (hasSoapEnvelope == false) {
            return _demarshallResourceResponse(evaldObj, ioArgs, serviceResponseHandler);
        }
        else {
            return _demarshallServiceResponse(evaldObj);
        }
    }
    
    function _demarshallResourceResponse(resource, ioArgs, serviceResponseHandler){

		if (!resource){
			var responseStatus = ioArgs.xhr.status;
			if (responseStatus != 200) {
				var errorObj = {
					errorCode: responseStatus,
					errorMessage: responseStatus
				};
				serviceResponseHandler.doErrorCallback(errorObj);
			}
			return null;
		}else if (resource.errorCode || resource.errorMessage){
			serviceResponseHandler.doErrorCallback(resource);
			return null;
		}else if (resource.EPMCollection){
            return resource.EPMCollection;
        }else if (resource.EPMEntry){
			return resource.EPMEntry;
        }else{
			return getFirstChild(resource);                
		}
    }
    /**
     * In the new resource format the server will not have any header fields.
     * prviously we used to get the epmcollection or epm entry.
     * now if you make a call to /project then we will get the array or projects.
     * So the client that calls the server has to know what is the root of the response.
     * to avoid that we are passing the root only so that generic code can be written.
     * @param {Object} resource
     */
    function getFirstChild(resource){
        var firstChildName = null;
        for (var i in resource) {
            if (firstChildName == null) {
                firstChildName = i;
                break;
            }
        }
        return resource[firstChildName];
    }
    
    function _demarshallServiceResponse(evaldObj){
        var returnValue = evaldObj["soapenv:Body"].response.returnValue;
        
        var obj = [];
        
        if (returnValue.values) { // array of EMF objects 
            obj = returnValue.values;
        }
        else 
            if (returnValue.value) { // single EMF object
                obj = returnValue.value;
            }
            else 
                if (returnValue.dataArgs) { // data query
                    obj = _demarshallDataQuery(returnValue.dataArgs);
                }
        
        return obj;
    }
    
    function _demarshallDataQuery(srcRows){
        var targetRows = [];
        for (var i = 0; i < srcRows.length; i++) {
            var srcCols = srcRows[i].value;
            targetRows.push(srcCols);
            //TODO need to talk to sandy on the below lines.
            //		var targetCols= [];
            //		for (var j=0; j<srcCols.length; j++) {
            //			targetCols.push(srcCols[j].value);
            //		}
            //		targetRows.push(targetCols);
        }
        return targetRows;
    }
    
})();
