/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.financial.web.ui.internal.AppadminCodes");
    
    
    // Widgets
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesGridView");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesTreeWidget");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridView");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesTreeWidget");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.MouseTooltip");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.FinancialConstants");
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.financial.web.ui", "Messages");
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var FinancialConstants = com.ibm.epm.web.core.ui.internal.FinancialConstants;
    var HIDE_DELAY = 5000;
    var SHOW_DELAY = 500;
    
    
    dojo.declare("com.ibm.epm.financial.web.ui.internal.AppadminCodes", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.financial.web", "ui/internal/templates/AppadminCodes.html"),
        _codesTree: null,
        _codesGrid: null,
        _codeType: null,
        
		constructor: function(){
				this.addBtn=Messages.ADD;
		},
		
		/*
         * postCreate - This function is used for initialize also it calls the super class method
         */
        postCreate: function(){
            this.inherited(arguments);
            this.mouseTooltip = new com.ibm.epm.web.core.ui.internal.MouseTooltip();
            // List of subscriptions for this Widget
            this._subscriptions = [];
            
        },
        
        _refreshCodes: function(){
            this.destroy();
            this._createWidgets();
            this.eventSubscribe();
        },
        
        /*
         * load - This function is used to load all the widgets.
         */
        load: function(parent){
            this.parent = parent;
            this._codeType = parent._codeType;
            this._refreshCodes();
        },
        
        needToHandleEvent: function(publishedItem){
        
            var publishedData = null;
            if (dojo.isArray(publishedItem)) {
                publishedData = publishedItem[0];
            }
            else {
                publishedData = publishedItem;
            }
            
            var boolNeedToHandleEvent = false;
            
            if (this._codeType == FinancialConstants.CHARGE_CODE_TYPE) {
                if (this._codesTree &&
                this._codesTree._chargecodesTree &&
                publishedData.treeID &&
                publishedData.treeID == this._codesTree._chargecodesTree.id) {
                    boolNeedToHandleEvent = true;
                }
            }
            else {
                if (this._codesTree &&
                this._codesTree._timecodesTree &&
                publishedData.treeID &&
                publishedData.treeID == this._codesTree._timecodesTree.id) {
                    boolNeedToHandleEvent = true;
                }
            }
            return boolNeedToHandleEvent;
        },
        
        /*
         * destroy - This function is used for deinitialize or clean up the widgets
         */
        destroy: function(){
            if (this._codesTree) {
                this._treeContent.removeChild(this._codesTree.domNode);
                this._codesTree.destroy();
                delete this._codesTree;
            }
            if (this._codesGrid) {
                this._gridContent.removeChild(this._codesGrid.domNode);
                this._codesGrid.destroy();
                delete this._codesGrid;
            }
            this._unsubscribeToAllEvents();
        },
        
        /**
         * eventSubscribeToAllEvents: Method to subscribe to all events. <br>
         */
        eventSubscribe: function(){
            this._subscriptions = [dojo.subscribe(EPMTreePubSub.ON_MOUSE_OVER, this, "_onCodeTreeMouseOver"), dojo.subscribe(EPMTreePubSub.ON_MOUSE_OUT, this, "_onCodeTreeMouseOut")];
        },
        
        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions != null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._subscriptions = [];
        },
        
        /**
         * _getTreeDomNode: Method to get the tree node's domNode
         */
        _getTreeDomNode: function(){
            var domTreeNode = null;
            if (this._codeType == FinancialConstants.CHARGE_CODE_TYPE) {
                if ((this._codesTree) && (this._codesTree._chargecodesTree)) {
                    domTreeNode = this._codesTree._chargecodesTree.getNodeHover();
                }
            }
            else {
                if ((this._codesTree) && (this._codesTree._timecodesTree)) {
                    domTreeNode = this._codesTree._timecodesTree.getNodeHover();
                }
            }
            return domTreeNode ? domTreeNode.contentNode : null;
        },
        
        /**
         * _onGridCellMouseOver: To display the calendar attributes as tooltip
         * @param {Object} e
         */
        _onCodeTreeMouseOver: function(e){
			
            if (this.needToHandleEvent(e) == false) {
                return;
            }
			
            if (this._codesGrid) {
                if (dojo.isArray(e.href)) {
                    this._selectedrowID = e.href[0];
                }
                else {
                    this._selectedrowID = e.href;
                }
                var domTreeNode = this._getTreeDomNode();
				if ((this._codesTree) && this._codesTree.menuImage) {
					this._codesTree.menuImage.style.visibility = "hidden";
				}
                this._computeHint(domTreeNode);
            }
        },
        
        /**
         * 	_computeHint: 	Retrieve the attributes of the selected calendar in the grid,
         * 					copies to tooltip
         * 	@param {Object} e
         * 					event e is used to get the selected row to _calendarInfo,
         * 					copies the required attributes to toolTipInfo
         */
        _computeHint: function(e){
        
            if (this._codesGrid &&
            this._codesGrid._controller &&
            this._codesGrid._controller._gridData) {
            
                var rowIndex = this._codesGrid._controller.getRowIndex(this._selectedrowID);
                var selectedRow = this._codesGrid._controller.getRowByIndex(rowIndex);
                
                if (selectedRow) {
                    if (!this._showTimer) {
                        this._showTimer = setTimeout(dojo.hitch(this, function(){
                            var toolTipInfo = {
                                "Name": selectedRow.name,
                                "Description": selectedRow.description,
                                "Code": selectedRow.code
                            };
                            if (this._codeType == FinancialConstants.CHARGE_CODE_TYPE) {
                                toolTipInfo.Billable = selectedRow.billable;
                                toolTipInfo.State = selectedRow.state;
                            }
                            this.showTooltip(toolTipInfo, e);
                        }), SHOW_DELAY);
                    }
                    
                    if (!this._hideTimer) {
                        this._hideTimer = setTimeout(dojo.hitch(this, function(){
                            this.hideTooltip(e);
                        }), HIDE_DELAY);
                    }
                }
            }
        },
        
        /**
         * showTooltip: Creates and displays the tooltip
         * 				Uses mouseTooltip to display tooltip
         * @param {Object} toolTipInfo
         * @param {Object} e
         */
        showTooltip: function(toolTipInfo, e){
            //ToolTipInfo structure is :
            //"Name"
            //"Description"
            //"Code"	
            var msg = "<b>";
            if (toolTipInfo.Name) {
                msg += Messages.CHARGECODE_NAME + ": " + toolTipInfo.Name;
                msg += "</b><br>";
            }
            if (toolTipInfo.Description) {
                msg += Messages.CHARGECODE_DESCRIPTION + ": " + toolTipInfo.Description;
                msg += "</b><br>";
            }
            if (toolTipInfo.Code) {
                msg += Messages.CHARGECODE_CODE + ": " + toolTipInfo.Code;
                msg += "</b><br>";
            }
            if (toolTipInfo.Billable) {
                msg += Messages.CHARGECODE_BILLABLE + ": " + toolTipInfo.Billable;
                msg += "</b><br>";
            }
            if (toolTipInfo.State) {
                msg += Messages.CHARGECODE_STATE + ": " + toolTipInfo.State;
                msg += "</b><br>";
            }
            this.mouseTooltip._open({
                innerHTML: msg,
                target: e,
                zIndex: 10001
            });
        },
        
        /**
         * _onGridCellMouseOut: To remove the tooltip on mouse over
         * @param {Object} e
         */
        _onCodeTreeMouseOut: function(e){
            if (this.needToHandleEvent(e) == false) {
                return;
            }
            if (this._showTimer) {
                clearTimeout(this._showTimer);
                delete this._showTimer;
            }
            if (this._hideTimer) {
                clearTimeout(this._hideTimer);
                delete this._hideTimer;
            }
            var domTreeNode = this._getTreeDomNode();
            if (this.mouseTooltip) {
                this.hideTooltip(domTreeNode);
            }
        },
        
        /**
         * hideTooltip: To close the created mouseTooltip
         * 				Uses mouseTooltip close() method
         * @param {Object} e
         */
        hideTooltip: function(e){
            this.mouseTooltip._close(e);
            dijit._masterTT._onDeck = null;
        },
        
        /*
         * assignChargecode - This function called when the add button is clicked.
         * It will add the charge code to the project associated charge code grid.
         */
        _assignCode: function(){
            if ((this._codesGrid) && (this._codesGrid._generalGrid)) {
                var selectedRowIndexes = this._codesGrid._generalGrid.selection.getSelected();
                if (selectedRowIndexes !== null && selectedRowIndexes.length > 0) {
                    var selectedRow = this._codesGrid._generalGrid.model.getRow(selectedRowIndexes[0]);
                    if (this.parent) {
                        if (selectedRow) {
                            if (!this.parent._isLinkavailable(selectedRow.href)) {
                                if (!this._checkIfcodePresent(this.parent._associatedCodes, selectedRow.href)) {
                                    if (this.parent._associatedCodes === "") {
                                        this.parent._associatedCodes = selectedRow.href;
                                    }
                                    else {
                                        this.parent._associatedCodes = this.parent._associatedCodes +
                                        "," +
                                        selectedRow.href;
                                    }
                                }
                            }
                            else {
                                //remove from deassocaited list
                                var codes = this.parent._deassociatedCodes;
                                this.parent._deassociatedCodes = this.parent._removeCodefromlist(selectedRow.href, codes);
                            }
                            var itemHandle = {
                                title: selectedRow.name,
                                relativeHref: selectedRow.href
                            };
                            if (this.parent._codesGrid) {
                                this.parent._codesGrid._insertCode(itemHandle);
								this.parent._codesGrid.update();
								this.parent.parentViewlet._toggleSaveButton(false);
                            }
                        }
                    }
                }
                else {
                    if (this._codeType == FinancialConstants.CHARGE_CODE_TYPE) {
                        alert(Messages.CHARGECODE_NOT_CREATED);
                    }
                    else {
                        alert(Messages.TIMECODE_NOT_CREATED);
                    }
                }
            }
            else {
                if (this._codeType == FinancialConstants.CHARGE_CODE_TYPE) {
                    alert(Messages.CHARGECODE_NOT_CREATED);
                }
                else {
                    alert(Messages.TIMECODE_NOT_CREATED);
                }
            }
            this.destroy();
            this.parent._closedialog();
        },
        
        /**
         *
         * @param {Object} listOfTimecodeURI
         * @param {Object} selectedTimecodeURI
         *
         * It checks if the code which needs to be assigned/removed from the assignment exists or not.
         */
        _checkIfcodePresent: function(listOfcodeURI, selectedcodeURI){
        
            var tempcodeURIArray = new Array(1);
            var iscodePresent = false;
            tempcodeURIArray = listOfcodeURI.split(",");
            var numberOfTimecodes = tempcodeURIArray.length;
            for (var i = 0; i < numberOfTimecodes; i++) {
            
                if (tempcodeURIArray[i] === selectedcodeURI) {
                    iscodePresent = true;
                    break;
                }
            }
            return iscodePresent;
        },
        
        /*
         * _createWidgets - This function is used for creating all the widgets
         */
        _createWidgets: function(){
            if (this._codeType == FinancialConstants.CHARGE_CODE_TYPE) {
                this._createChargeCodeWidgets();
            }
            else {
                this._createTimeCodeWidgets();
            }
        },
        
        /*
         * This function will create the charge code tree and grid
         */
        _createChargeCodeWidgets: function(){
            if (this._codesTree === null) {
                this._codesTree = new com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesTreeWidget();
                this._treeContent.appendChild(this._codesTree.domNode);
            }
            this._codesTree._buildChargecodesTree(/*renameAllowed*/false);
            
            if (this._codesGrid === null) {
                this._codesGrid = new com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesGridView({
                    _treeID: "ChargecodesTree"
                });
                this._gridContent.appendChild(this._codesGrid.domNode);
            }
            this._codesGrid.buildChargecodesGrid();
        },
        
        /*
         * This function will create the time code tree and grid
         */
        _createTimeCodeWidgets: function(){
            if (this._codesTree === null) {
                this._codesTree = new com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesTreeWidget();
                this._treeContent.appendChild(this._codesTree.domNode);
            }
            this._codesTree._buildTimecodesTree(/*renameAllowed*/false);
            
            if (this._codesGrid === null) {
                this._codesGrid = new com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridView({
                    _treeID: "TimecodesTree"
                });
                this._gridContent.appendChild(this._codesGrid.domNode);
            }
            this._codesGrid.buildTimecodesGrid();
        }
    });
})();
