/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.financial.web.ui.internal.AssociateProjectCodes");
    dojo.require("dijit.Toolbar");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.financial.web.ui.internal.AssociateProjectCodesGridView");
    dojo.require("com.ibm.epm.financial.web.ui.internal.AppadminCodes");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.web.core.ui.internal.FinancialConstants");
    dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
	dojo.require("jazz.ui.Dialog");
	dojo.requireLocalization("com.ibm.epm.financial.web.ui", "Messages");
	
	var Messages = dojo.i18n.getLocalization("com.ibm.epm.financial.web.ui", "Messages");
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    var FinancialConstants = com.ibm.epm.web.core.ui.internal.FinancialConstants;
    var EPMTreeEvents = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	
    dojo.declare("com.ibm.epm.financial.web.ui.internal.AssociateProjectCodes", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.financial.web", "ui/internal/templates/AssociateProjectCodes.html"),
        _codesGrid: null,
        _codeType: "",
        _loadedelement: null,
        _associatedCodes: "",
        _deassociatedCodes: "",
        
		/**
		 * postCreate:
		 * 			
		 */
        postCreate: function(){
									
        },
		
		/**
		 * load: Creates & loads the toolbar & financial grids.
		 * @param {Object} codeUri
		 */
        load: function(parent, codeUri, loadedElement){
			this.parentViewlet = parent;
            this._codeType = codeUri;
			this._loadedelement = loadedElement;
            this._refreshWidget();
        },
        
		_refreshWidget: function(){
            this.destroy();
            this._createWidgets();
        },
        
		/**
		 * destroy: 
		 * 3. Not destroying the codesGrid as that is initiating as null.
		 * 		And destroying codesGrid is resulting error message in IE7. 
		 */
        destroy: function(){
        },
        
		/**
		 * _createWidgets: Builds the Chargecodes & Timecodes grid.
		 */
        _createWidgets: function(){
            if (this._codesGrid == null) {
                this._codesGrid = new com.ibm.epm.financial.web.ui.internal.AssociateProjectCodesGridView({
                    _treeID: this._codeType
                });
                this._gridContent.appendChild(this._codesGrid.domNode);
            }
            this._codesGrid.buildCodesGrid(this, this._codeType, this._loadedelement);
        },
        
        _addCodes: function(){
            var self = this;
            var div = document.createElement("div");
            // Nullify the dialog if already exists	
            if (self.newDlg) {
				self.newDlg.destroy();
                self.newDlg = null;
            }
			this.appadminCodes = new com.ibm.epm.financial.web.ui.internal.AppadminCodes();
 		    self.newDlg = new jazz.ui.Dialog({
                contentNode : this.appadminCodes.domNode,
				primaryTitle: (this._codeType == FinancialConstants.CHARGE_CODE_TYPE) ? Messages.CHARGECODE_TITLE: Messages.TIMECODE_TITLE,
				width: "350px",
				height: "auto",
				onOpen: dojo.hitch(this, this.refreshCodes),
				onClose: dojo.hitch(this, this.closedialog)
            });
        },
		
		/*
         * This method is called when we click on the close button.
         * Closes the dialog and destroys the widgets.
         */
        _closedialog: function(){
			if (this.appadminCodes) {
				this.appadminCodes.destroy();
			}
			if(this.newDlg){
            	this.newDlg.close();
				this.newDlg.destroy();
				this.newDlg = null;
			}
        },
		
		refreshCodes:function(){
			this.appadminCodes.load(this); 
		},
        
        /**
         * This will be called when press closeicon on the top of the dialog
         */
        closedialog: function(){
			if (this.appadminCodes) {
                this.appadminCodes.destroy();
            }
            return true;           
        },
        
		/**
		 * _saveProjectCodes: Saves the association & de-association of chargecodes / timecodes. 
		 * 						Post call without set interval.
		 */
        _getProjectCodes: function(inputparams){
			if (!this._loadedelement) { return; }
            var modifiedProjectCodes = false;
            if (this._codeType != FinancialConstants.CHARGE_CODE_TYPE) {
                if (this._associatedCodes != "") {
                    inputparams.associatedTimecodes = this._associatedCodes;
                    modifiedProjectCodes = true;
                }
                if (this._deassociatedCodes != "") {
                    inputparams.deassociatedTimecodes = this._deassociatedCodes;
                    modifiedProjectCodes = true;
                }
            }
            else {
                if (this._associatedCodes != "") {
                    inputparams.associatedChargecodes = this._associatedCodes;
                    modifiedProjectCodes = true;
                }
                if (this._deassociatedCodes != "") {
                    inputparams.deassociatedChargecodes = this._deassociatedCodes;
                    modifiedProjectCodes = true;
                }
			}
			this._associatedCodes = "";
            this._deassociatedCodes = "";
			return modifiedProjectCodes;
        },
        
        _isLinkavailable: function(href){
            var linkType = this._codeType == FinancialConstants.CHARGE_CODE_TYPE ? FinancialConstants.CHARGE_CODE_LINK_NAME : FinancialConstants.TIME_CODE_LINK_NAME;
            var codeLinks = getLink(this._loadedelement, linkType);
            if (codeLinks) {
                for (var j in codeLinks) {
                    var itemHandle = codeLinks[j];
                    if (itemHandle.relativeHref == href) {
                        return true;
                    }
                }
            }
            return false;
        },
        
        _removeCodefromlist: function(removehref, codes){
            var newlist = "";
            var href = null;
            var tempcodeURIArray = new Array(1);
            tempcodeURIArray = codes.split(",");
            var numberOfTimecodes = tempcodeURIArray.length;
            for (var i = 0; i < numberOfTimecodes; i++) {
                if (tempcodeURIArray[i] !== removehref) {
                    if (newlist !== "") {
                        newlist = newlist + "," + tempcodeURIArray[i];
                    }
                    else {
                        newlist = tempcodeURIArray[i];
                    }
                }
            }
            return newlist;
        },
       
		/**
		 * The method _deleteCodes will be called when the remove button is pressed 
		 * from the wbs detail viewlet
		 */
        _deleteCodes: function(){
            var selectedRowIndexes = this._codesGrid._generalGrid.selection.getSelected();
			var toDeleted = new Array();
            if (selectedRowIndexes !== null && selectedRowIndexes.length > 0) {
				for (var cnt = 0; cnt < selectedRowIndexes.length; cnt++) {
					var selectedRow = this._codesGrid._generalGrid.model.getRow(selectedRowIndexes[cnt]);
					if (selectedRow) {
						toDeleted.push(selectedRow.id);
						if (this._isLinkavailable(selectedRow.href)) {
						
							if (this._deassociatedCodes === "") {
								this._deassociatedCodes = selectedRow.href;
							}
							else {
								this._deassociatedCodes = this._deassociatedCodes +
								"," +
								selectedRow.href;
							}
						}
						else {
							//remove from assocaited list
							var codes = this._associatedCodes;
							this._associatedCodes = this._removeCodefromlist(selectedRow.href, codes);
						}
					}
				}
				
				this._codesGrid.removeCodes(toDeleted);
				this._codesGrid.update();
            }
			if (this.confirmDialog) {
				this.confirmDialog.hide();
				this.confirmDialog.destroy();
			}
			this.parentViewlet._toggleSaveButton(false);			 
        },
		
		/**
          * Clears the error message being displayed
          */
        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },
        
        /**
         * Display and clear the error message
         * incoming : Error Message Object
         * OutGoing : None
         */
        _displayAndClearErrorMessage: function(errorMessage){
            var self = this;
            EPMErrorHandler.setError(errorMessage);
            
            // Set an interval and clear the error message
            var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);
        }	
    });
})();
