/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.financial.web.ui.internal.AssociateProjectCodesGridView");
    
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.financial.web.ui.internal.AssociateProjectCodesGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.web.core.ui.internal.FinancialConstants");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
	 // Localization
    dojo.requireLocalization("com.ibm.epm.financial.web.ui", "Messages");
    
    var controllerFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var FinancialConstants = com.ibm.epm.web.core.ui.internal.FinancialConstants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
	var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
	var Messages = dojo.i18n.getLocalization("com.ibm.epm.financial.web.ui", "Messages");
    
    
    dojo.declare("com.ibm.epm.financial.web.ui.internal.AssociateProjectCodesGridView", abstractView, {
        codeType: "",
        gridControllerID: "",
        templatePath: dojo.moduleUrl("com.ibm.epm.financial.web", "ui/internal/templates/AssociateProjectCodesGrid.html"),
        
		/**
         * postCreate: Method called after a widget's dom has been setup, all initialization and event subscriptions are done here <br>
         * @param none
         */
        postCreate: function(){
			this._rowHoverArray = [];
			this._pconnects = [];
        },
		
        /* 
         * buildChargecodesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with TimecodesGrid.
         */
        buildCodesGrid: function(parentWid, Uri, loadedElement){
			this.parentWidget = parentWid;
            this.codeType = Uri;
            this.gridControllerID = (this.codeType == FinancialConstants.CHARGE_CODE_TYPE ? FinancialConstants.CHARGE_CODE_GRID_CONTROLLER : FinancialConstants.TIME_CODE_GRID_CONTROLLER);
            this.gridControllerID = this.gridControllerID + "_" + controllerUtil.getRandomNumber();
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.financial.web.ui.internal.AssociateProjectCodesGridModel();
                this._gridModel.load(this.codeType);
            }
            var associateCodesController = controllerFactory.getEPMFinancialGridController(this.gridControllerID);
            associateCodesController.destroy(); //To Do - Get rid of this recreating controller
            delete associateCodesController;
            associateCodesController = controllerFactory.getEPMFinancialGridController(this.gridControllerID);
            associateCodesController.loadedElement = loadedElement;
            this.buildGrid(this, this._gridModel, associateCodesController);
        },
        
        /*
         * Returns the Grid Title
         */
        getGridTitle: function(){
            return ((this.codeType == FinancialConstants.CHARGE_CODE_TYPE) ? Messages.CHARGECODE_GRID_TITLE : Messages.TIMECODE_GRID_TITLE);
        },
        
         /*
         * Returns the Grid Internal ID
         */
        getGridInternalID: function(){
            return ((this.codeType == FinancialConstants.CHARGE_CODE_TYPE) ? FinancialConstants.CHARGECODE_GRID_TITLE : FinancialConstants.TIMECODE_GRID_TITLE);
        },
        
		onCellClick: function(event){
            if (event) {
                if ((event.cell.field == "action") && (event.target.name == "deleteImage")) {
					this.parentWidget._deleteCodes();
                }            	
			}
        },
        
		/**
		 * The method will be called when Grid view is getting destroyed
		 */
		destroy: function(){
			dojo.forEach(this._pconnects, dojo.disconnect);
            this._pconnects = [];
        },
		
        /*
         * This is called when to remove the code when the add button is pressed
         * in the pop up
         *
         */
        _insertCode: function(itemHandle){
            var associateCodesController = controllerFactory.getEPMFinancialGridController(this.gridControllerID);
            associateCodesController._insertElement(itemHandle, null, false);
			this.scrollToSelectedRow();
        },
        
        /*
         * This is called when to remove the code when the remove button is pressed
         *
         */
        _removeCode: function(rowId){
            var associateCodesController = controllerUtil.getEPMFinancialGridController(this.gridControllerID);
            associateCodesController._removeGridRow(rowId);
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.codesGridAttachPoint;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
        
			_displayActions = function(inRowIndex){
				return '<img name="deleteImage" style="margin-right: 5px; margin-left: 5px; cursor: pointer;" title="' + Messages.REMOVE +
                    '" alt="' + Messages.REMOVE + '" src =' +
                    dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/actions/action_unassign.gif") +
                    " />";
			};
			var gridLayout = null;
            if (this.codeType == FinancialConstants.CHARGE_CODE_TYPE) {
                gridLayout = [{
                    noscroll: false,
                    rows: [[{
                        name: Messages.CHARGECODE_NAME,
                        width: "30%",
                        field: "name",
						cellClasses: "com-ibm-epm-financial-web-ui-internal-AssociateProjectCodesGrid_Name_Col"
                    }, {
                        name: Messages.CHARGECODE_DESCRIPTION,
                        width: "30%",
                        field: "description"
                    }, {
                        name: Messages.CHARGECODE_CODE,
                        width: "10%",
                        field: "code"
                    }, {
                        name: Messages.CHARGECODE_BILLABLE,
                        width: "10%",
                        field: "billable",
                        styles: 'text-align: center;'
                    }, {
                        name: Messages.CHARGECODE_STATE,
                        width: "10%",
                        field: "state"
                    }, {
		                name: "  ",
		                width: "auto",
		                field: "action",
						get: _displayActions
            		}]]
                }];
            }
            else {
                gridLayout = [{
                    noscroll: false,
                    rows: [[{
                        name: Messages.CHARGECODE_NAME,
                        width: "30%",
                        field: "name",
						cellClasses: "com-ibm-epm-financial-web-ui-internal-AssociateProjectCodesGrid_Name_Col"
                    }, {
                        name: Messages.CHARGECODE_DESCRIPTION,
                        width: "30%",
                        field: "description"
                    }, {
                        name:Messages.CHARGECODE_CODE,
                        width: "10%",
                        field: "code"
                    }, {
		                name: "  ",
		                width: "auto",
		                field: "action",
						get: _displayActions
	            	}]]
                }];
            }
            
            return gridLayout;
        },
		
		/**
		 * The method removeCodes will be called when user select a code and press the remove button from the
		 * wbs details viewlet
		 * @param {Object} toDeletedRowids
		 */
		removeCodes: function(toDeletedRowids)
		{
			var associateCodesController = controllerFactory.getEPMFinancialGridController(this.gridControllerID);
			if (dojo.isArray(toDeletedRowids)) {
				for (var j in toDeletedRowids) {
					associateCodesController._removeGridRow(toDeletedRowids[j]);
				}
			}
		},
		
		/**
		 * createView - method will call the createView of the abstractView
		 * And also set the grid to grow auto.
		 */
		createView: function(){
			this.inherited('createView', arguments);
			
			this._generalGrid.autoHeight = true;
			this._generalGrid.update();
			
			this._pconnects.push(dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_DBL_CLICK, this, this.onCellClick));
            this._pconnects.push(dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_CLICK, this, this.onCellClick));
		}
    });
})();

