/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009, 2010. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/
dojo.provide("com.ibm.epm.web.core.ui.internal.EPMSequenceValue"); //$NON-NLS-1$

//dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$

(function() {

//var Assert							= com.ibm.team.foundation.Assert;

var BASE= 64;

var SEQUENCE_VALUE_SEPARATOR		= " "; //$NON-NLS-1$
var SEQUENCE_VALUE_NEWITEM			= null;
var SEQUENCE_VALUE_ACCEPTED			= "000000"; //$NON-NLS-1$
var SEQUENCE_VALUE_INITIAL			= "O00000"; // OH-null-null-null-null-null :) //$NON-NLS-1$
var SEQUENCE_VALUE_ONE				= "000001"; //$NON-NLS-1$

dojo.declare("com.ibm.epm.web.core.ui.internal.EPMSequenceValue", null, { //$NON-NLS-1$

	__sequenceValue: null,
	
	constructor: function(sequenceValue) {
		this.__sequenceValue= sequenceValue;
	},

	// ---- API ------------------------------------------------------------------------------------------------------------

	isNew: function() {
		return this.__sequenceValue == SEQUENCE_VALUE_NEWITEM;
	},

	serializeSequenceValue: function(scope) {
		return this.isNew() ? null : scope + SEQUENCE_VALUE_SEPARATOR + this.__sequenceValue;
	},

	getRawValue: function() {
		return this.__sequenceValue;
	},

	// ---- Comparable implementation --------------------------------------------------------------------------------------

	compareTo: function(other) {
		if (this.__sequenceValue == null || other.__sequenceValue == null)
			return (this.__sequenceValue == null ? 1 : 0) - (other.__sequenceValue == null ? 1 : 0);

		return compare(this.__sequenceValue, other.__sequenceValue);
	},

	equals: function(other) {
		if (this === other)
			return true;

		if (this.declaredClass !== other.declaredClass)
			return false;

		return this.compareTo(other) == 0;
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	__sentinel: null // terminates this class definition
});

var NEW= new com.ibm.epm.web.core.ui.internal.EPMSequenceValue(SEQUENCE_VALUE_NEWITEM);
var INITIAL= new com.ibm.epm.web.core.ui.internal.EPMSequenceValue(SEQUENCE_VALUE_INITIAL);

var alphabetStart= "0".charCodeAt(0); //$NON-NLS-1$

function valueOf(characterCode) {
	return characterCode - alphabetStart;
}

function toChar(value) {
	return String.fromCharCode(value + alphabetStart);
}

function compare(left, right) {
	return left == right ? 0 : left < right ? -1 : 1;
}

function add(left, right) {
	var overflow= false;
	var result= ""; //$NON-NLS-1$
	var i= Math.max(left.length, right.length) - 1;
	while (i >= 0) {
		var v1= left.length > i ? valueOf(left.charCodeAt(i)) : 0;
		var v2= right.length > i ? valueOf(right.charCodeAt(i)) : 0;
		var value= v1 + v2 + (overflow ? 1 : 0);
		overflow= value >= BASE;
		result= toChar(value - (overflow ? BASE : 0)) + result;
		i--;
	}

	if (overflow) {
		// using this SequenceValue implementation, we cannot generate numbers big enough to produce an overflow before we did
		// scheduled 2^35-1 work items to the end of a queue; this takes at least 1104 years of continuously scheduling 
		// a work item a a second. Therefore, if this log statement is seen before the year 3113, you are most likely 
		// observing a bug that introducing huge numbers into the system we never intended to be there :)
		console.log("Overflow while adding sequence values %s and %s", left, right); //$NON-NLS-1$
	}

	return result;
}

function subtract(minuend, subtrahend) {
	//Assert.isLegal(compare(minuend, subtrahend) >= 0);
	var borrow= false;
	var result= ""; //$NON-NLS-1$
	var i= Math.max(minuend.length, subtrahend.length) - 1;
	while (i >= 0) {
		var v1= minuend.length > i ? valueOf(minuend.charCodeAt(i)) : 0;
		var v2= subtrahend.length > i ? valueOf(subtrahend.charCodeAt(i)) : 0;
		var value= v1 - v2 - (borrow ? 1 : 0);
		borrow= value < 0;
		result= toChar((borrow ? BASE : 0) + value) + result;
		i--;
	}
	//Assert.isTrue(!borrow);
	return result;
}

function divideBy2(dividend) {
	var push= 0;
	var result= ""; //$NON-NLS-1$
	for (var i= 0; i < dividend.length; i++) {
		var value= valueOf(dividend.charCodeAt(i));
		result+= toChar((push << 5) + (value >> 1));
		push= value & 1;
	}

	if (push > 0) {
		result+= toChar(BASE >> 1);
	}
	return result;
}

// ---- SequenceValue Factory ----------------------------------------------------------------------------------------------

com.ibm.epm.web.core.ui.internal.EPMSequenceValue.NEW= NEW;
com.ibm.epm.web.core.ui.internal.EPMSequenceValue.INITIAL= INITIAL;

com.ibm.epm.web.core.ui.internal.EPMSequenceValue.FACTORY= {};

com.ibm.epm.web.core.ui.internal.EPMSequenceValue.FACTORY.successor= function(sequenceValue) {
	//Assert.isTrue(!sequenceValue.isNew());
	return new com.ibm.epm.web.core.ui.internal.EPMSequenceValue(add(sequenceValue.__sequenceValue, SEQUENCE_VALUE_ONE));
};

com.ibm.epm.web.core.ui.internal.EPMSequenceValue.FACTORY.predecessor= function(sequenceValue) {
	//Assert.isTrue(!sequenceValue.isNew());
	return new com.ibm.epm.web.core.ui.internal.EPMSequenceValue(subtract(sequenceValue.__sequenceValue, SEQUENCE_VALUE_ONE));
};

com.ibm.epm.web.core.ui.internal.EPMSequenceValue.FACTORY.between= function(sequenceValue1, sequenceValue2) {
	//Assert.isTrue(!sequenceValue1.isNew());
	//Assert.isTrue(!sequenceValue2.isNew());
	//Assert.isTrue(!sequenceValue1.equals(sequenceValue2));
	
	var v1= sequenceValue1.__sequenceValue;
	var v2= sequenceValue2.__sequenceValue;
	
	if (compare(v1, v2) > 0) {
		v1= sequenceValue2.__sequenceValue;
		v2= sequenceValue1.__sequenceValue;
	}
	
	return new com.ibm.epm.web.core.ui.internal.EPMSequenceValue(add(v1, divideBy2(subtract(v2, v1))));
};

com.ibm.epm.web.core.ui.internal.EPMSequenceValue.FACTORY.valueOf= function(sequenceValue, scope) {
	if (sequenceValue != null) {
		var parts= sequenceValue.split(" "); //$NON-NLS-1$
		
		if (parts.length == 2 && parts[0] == scope) {
			return SEQUENCE_VALUE_ACCEPTED == parts[1] ? NEW : new com.ibm.epm.web.core.ui.internal.EPMSequenceValue(parts[1]);
		}
	}
	return NEW;
};

})();
