/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.web.core.ui.internal.EPMTimesheetTree_temp");

// Dijit Tree Base class
dojo.require("dijit.Tree");

// Context Menu widget
dojo.require("com.ibm.epm.web.core.ui.internal.EPMMenu");

// Icon class constants
dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");

// Context Menu widget
var EPMMenu = com.ibm.epm.web.core.ui.internal.EPMMenu;

var IconClass = com.ibm.epm.web.core.ui.internal.IconClassConstants;

(function() {

dojo.declare("com.ibm.epm.web.core.ui.internal.EPMTimesheetTree_temp",
	dijit.Tree, {

	templateString:"<div class=\"dijitTreeContainer\" style=\"\" waiRole=\"tree\"\r\n\t" +
			"dojoAttachEvent=\"onclick:_onClick,onmouseover:_onMouseOver,onmouseout:_onMouseOut," +
			"onkeypress:_onKeyPress\">\r\n",	
	
	// Table containing the structure of the context menu	
	_ctxtItems: null,
	
	// Context Menu		
	_ctxMenu: null,      		
	
	// _TreeNode, the active tree node in right-clicking  for context menu
	_ctxNode: null, 

	postCreate: function(){		
		this.inherited('postCreate', arguments);
		// this._ctxtItems should be initialized in the derived class
		this._ctxMenu = new EPMMenu ({menuItems: this._ctxtItems});
	},
	
	destroy: function() {
		this.inherited('destroy', arguments);
	},

	/*
	 * Get the tree node by its item's identity
	getNodeByItemIdentity: function(identity) {
		if(identity === null) {
			return null;
		}
		return this._itemNodeMap[identity];		
	},*/	

	/*
	 * Overrides dijit tree function, used to define the iconclass to be used with the TreeNode
	 */
	getIconClass: function(/* dojo.data.Item */ item, /*Boolean*/ opened) {		
		//console.log("Enter getIconClass");
		var iconclass = "";
		
		if ((item) && (item.resourcetype)) {
			// FIXME: Check how this can be done in the derived class
			switch (item.resourcetype[0]) {
				case "project":
					if (this.mayHaveChildren(item)) {
						if (opened) {
							return IconClass.PROJECT_OPENED_ICON_CLASS;
						} else {
							return IconClass.PROJECT_OPENED_ICON_CLASS;
						}
					}
					return IconClass.PROJECT_CLOSED_ICON_CLASS;

				 case "task":
				 	return 	IconClass.TASK_ICON_CLASS;
		
				case "program":
					return IconClass.PROGRAM_ICON_CLASS;
					
				default:
					if(item.iconClass){
						return item.iconClass;
					}else{
						return item.resourcetype + "Icon";
					}
					break;
			}	
		}
		
		if (item) {
			//console.log(item.iconClass);
			iconclass = item.iconClass;
			return iconclass;
		}
		//console.log("Exit getIconClass");
		
	},
	
	/*
	 * translates click events into commands for the controller to process
	 */
	_onClick: function(/*Event*/ e) {
		var domElement = e.target;

		// find node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		if (nodeWidget) {
			if(nodeWidget.isExpandable){
				// If the user has clicked on the expand icon then call _onExpandoClick
				if(domElement == nodeWidget.expandoNode ||
						domElement == nodeWidget.expandoNodeText){			
					this._onExpandoClick({node:nodeWidget});
				}
			}
			
			// If click on the label or the icon set focus on the node
			if(domElement == nodeWidget.labelNode ||
				domElement == nodeWidget.iconNode){ 
				this.onClick(nodeWidget.item, nodeWidget);
				this.focusNode(nodeWidget);
			}									
		}
				
		dojo.publish("OnSelect", [nodeWidget.item]);
			
		dojo.stopEvent(e);
	},
			
	/*
	 * Used to handle any processing on mouse over event
	 */
	_onMouseOver: function(e) {
		var domElement = e.target;

		// find node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		if(!nodeWidget || !nodeWidget.isTreeNode){
			return;
		}

		var element = nodeWidget.item;
		if (element) {
			// Publish mouseover event
			dojo.publish("OnMouseOver", [element]);
			if (nodeWidget.domNode) {
				nodeWidget.domNode.title = element.name;
			}		
		}

		var labelNode = nodeWidget.labelNode;
	
		// Add drop down img for popup menu only for project and summary task
//		if ((nodeWidget.item) && (nodeWidget.item.dropdown[0] == true)) {
//
//			if (!labelNode.menuImg)
//			{
//				this._addDropDownIcon(labelNode);
//				labelNode.menuImg.style.visibility = "";
//			}
//			else
//			{
//				// Icon is already attached make it visible
//				labelNode.menuImg.style.visibility = "";
//			}
//		}
		//console.log("Exit _onMouseOver");		
	},

	/*
	 * Used to handle any processing on mouse out event
	 */
	_onMouseOut: function(e) {
		var domElement = e.target;

		// find node
		var nodeWidget = dijit.getEnclosingWidget(domElement);	
		if(!nodeWidget || !nodeWidget.isTreeNode){
			return;
		}

		var element = nodeWidget.item;
		if (element) {		
			// Publish mouseout event
			dojo.publish("OnMouseOut", [element]);
		}	
	},

	/*
	 * override dijit.Tree implementation to change Handle focus function in FF3
	 */
	focusNode: function(/* _tree.Node */ node){
		// In FF3, somehow "onfocus" event is not fired when clicking on tree nodes, and _onTreeFocus won't get called
		// so we set dijitTreeLabelFocused class by ourselves
		if(dojo.isFF >= 3) {
			if(node != this.lastFocused){
				this.blurNode();
			}
			var labelNode = node.labelNode;
			// set tabIndex so that the tab key can find this node
			labelNode.setAttribute("tabIndex", "0");
			dojo.addClass(labelNode, "dijitTreeLabelFocused");
			this.lastFocused = node;
		}
		this.inherited("focusNode", arguments);
		// Publish mouseover event
		dojo.publish("OnMouseOver", [node.item]);
	},

	/*
	 * override dijit.Tree implementation to change node style after expand
	 */
	_expandNode: function(/*_TreeNode*/ node) {		
		// summary: called when the user has requested to expand the node
		var tree = node.tree;
		if (tree.lastFocused) {
			tree.focusNode(node);
		}

		// No children return
		if (!node.isExpandable){
			return;
		}

		var store = this.model.store;
		var getValue = store.getValue;

		// Processing done based on the state of the node
		switch (node.state){
			case "LOADING":
				// ignore clicks while we are in the process of loading data
				return;

			case "UNCHECKED":
				// need to load all the children, and then expand
				node.markProcessing();
				var _this = this;
				var onComplete = function(childItems){
					node.unmarkProcessing();
					_this._onLoadAllItems(node, childItems);
				};
				this.getItemChildren(node.item, onComplete);
				if (node.item) {
					var iconClass = this.getIconClass(node.item, true);
					node.iconNode.className = "dijitInline dijitTreeIcon " + iconClass;
					dojo.publish("OnExpand", [node.item]);
				}
				break;

			default:
				// data is already loaded; just proceed
				if (node.expand){	// top level Tree doesn't have expand() method
					node.expand();
					
					node.containerNode.style.overflow = "";
					
					if (this.persist && node.item){
						this._openedItemIds[this.model.store.getIdentity(node.item)] = true;
						this._saveState();
						if (node.item) {							
							node.item._openedItemIds = this._openedItemIds;
							var iconClass = this.getIconClass(node.item, true);
							node.iconNode.className = "dijitInline dijitTreeIcon " + iconClass;
							dojo.publish("OnExpand", [node.item]);
						}						
					}
				}
				break;
		}
	},

	/*
	 * override dijit.Tree implementation to change node style after collapse
	 */
	_collapseNode: function(/*_TreeNode*/ node) {
		// summary: called when the user has requested to collapse the node

		if (node.isExpandable) {
			if(node.state == "LOADING"){
				// ignore clicks while we are in the process of loading data
				return;
			}
			if (this.lastFocused) {
				// are we collapsing a descendant with focus?
				if (dojo.isDescendant(this.lastFocused.domNode, node.domNode)) {
					this.focusNode(node);
				} else {
					this.focusNode(node);					
				}
			}
			// collapse and publish the event
			node.collapse();
			var iconClass = this.getIconClass(node.item, false);
			node.iconNode.className = "dijitInline dijitTreeIcon " + iconClass;
			dojo.publish("OnCollapse", [node.item]);
			
			node.containerNode.style.overflow = "";
			
			if(this.persist && node.item){
				delete this._openedItemIds[this.model.store.getIdentity(node.item)];
				this._saveState();
			}
		}
	},
	getAllVisibleItems: function(){
		var allVisibleItems = new Array(1);
		var topLevelItems = this.model.store._getItemsArray();
		
		for(var item in topLevelItems){
			
			var topLevelNode = this._itemNodeMap[topLevelItems[item].identifier[0]];
			allVisibleItems.push(topLevelNode.item);
			this._addVisibleChildren(allVisibleItems, topLevelNode);
			
		}
		return allVisibleItems;
	},
	
	_addVisibleChildren: function(/*Array */allVisibleItems, /*TreeNode */currentNode){
		var self = this;
		if(currentNode.hasChildren()){
			var children = currentNode.getChildren();
			dojo.forEach(children, function(child){
				var childVisible = self._checkVisibility(child);
				if(childVisible == true){
					allVisibleItems.push(child.item);
					self._addVisibleChildren(allVisibleItems, child);
				}
			});
		}else{
			return
		}
	},
	
	_checkVisibility: function(currentNode){
		var nodeVisible = false;
		var nodeID = currentNode.item.identifier[0];
		
		if(nodeID.lastIndexOf("tempChild") == -1){
			var currentItemNode = this._itemNodeMap[nodeID];
			if(currentItemNode != null){
				var nodeVisiblityStyle = dojo.style(currentItemNode.domNode.parentNode, "display");
				if(nodeVisiblityStyle != "none"){
					nodeVisible = true;
				}
			}
		}
		return nodeVisible;
	},

	_expandAllTopLevelNodes: function() {
		// Ensure we have a root node
		if (!this.rootNode) {
			return;
		}
		// Get all of the top level nodes
		var treeNodeList = this.rootNode.getChildren();
		// Ensure we have an array
		if ((treeNodeList instanceof Array) === false) {
			return;
		}
		// Expand each top level node
		for (var index = 0; index < treeNodeList.length; index++) {
			var treeNode = treeNodeList[index];
			// Ensure the node is a folder
			if (!treeNode) {
				continue;
			}
			else 
				if (!treeNode.isExpandable) {
					continue;
				}
			// Expand the node
			this._expandNode(treeNode);
		}
	}
   });
})();