/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/**
 * Generic EPMToolBar.
 */

if (!dojo._hasResource["com.ibm.epm.web.core.ui.internal.EPMToolbar"]) {
    dojo._hasResource["com.ibm.epm.web.core.ui.internal.EPMToolbar"] = true;
    dojo.provide("com.ibm.epm.web.core.ui.internal.EPMToolbar");

    dojo.require("dijit.form.Button");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");

    dojo.declare(
            "com.ibm.epm.web.core.ui.internal.EPMToolbar",
            [dijit._Widget, dijit._Templated],
    {

		templatePath: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/templates/EPMToolbar.html"),

        postCreate: function() {
			
			this._addToolbarTitle();
			this.addContentLeftToolbar(this.leftContentList);
			this.addContentRightToolbar(this.rightContentList);
        },
		
		/**
		 * If any title needs to be put inside the toolbar, then the user can add title also.
		 * There is separate css for toolbar title.
		 */
		_addToolbarTitle: function(){
			
			if(this.title){
				
				this.titleLabel = document.createElement("label");
				this.textNode = document.createTextNode(this.title);
				this.titleLabel.appendChild(this.textNode);
				this.toolbarTitleDIV.appendChild(this.titleLabel);
			}
		},
		
		updateToolbarTitle: function(newTitle) {
			this.title = newTitle;
			if (!this.titleLabel) {
				this._addToolbarTitle();
			}
			else {
				if ( this.titleLabel.hasChildNodes() ) {
					for (var i=0; i<this.titleLabel.childNodes.length; i++) {
						this.titleLabel.removeChild(this.titleLabel.childNodes[i]);
					}
				}
				this.textNode = document.createTextNode(this.title);
				this.titleLabel.appendChild(this.textNode);
			}
		},
		
		/**
		 * Adding content to the left of the toolbar. 
		 */
		addContentLeftToolbar: function(){
			
			if(this.leftContentList){
				
				this._addContent(this.leftContentList, this.toolbarLeftDIV);
			}
		},
		
		/**
		 * Adding content to the right of the toolbar.
		 */
		addContentRightToolbar: function(){
			
			if(this.rightContentList){
				
				this._addContent(this.rightContentList, this.toolbarRightDIV);
			}
		},
		
		/**
		 * 
		 * @param {Object} content
		 * @param {Object} attachPoint
		 */
		_addContent: function(content, attachPoint){
			
			for(var i = 0; i < content.length; i++){
					
				if(content[i].domNode){
					if(!content[i].border){
						dojo.addClass(content[i].domNode, "buttonWithoutBorder");	
					}
					attachPoint.appendChild(content[i].domNode);	
				} else {
					if(content[i].anchorLabel) {
						dojo.addClass(content[i], "anchorLabelClass");
					}
					attachPoint.appendChild(content[i]);
				}
			}
		},

		/**
		 * To destroy the instance of this object.
		 */
		destroy: function(){
			
		}

    });
}