/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
dojo.provide("com.ibm.epm.web.core.ui.internal.EPMView.Button");

dojo.require("jazz.ui.Button");
dojo.require("com.ibm.epm.web.core.util.internal.Constants");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
dojo.require("com.ibm.epm.web.core.client.internal.WorkItemClient");
dojo.require("com.ibm.epm.web.core.util.internal.Constants");

dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");


var Constants = com.ibm.epm.web.core.util.internal.Constants;
var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
var IconClass = com.ibm.epm.web.core.ui.internal.IconClassConstants; 
var EPMWorkItemClient = com.ibm.epm.web.core.client.internal.WorkItemClient;
var ControllersFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;

var WBS_Constants = com.ibm.epm.web.core.util.internal.Constants;

    dojo.declare("com.ibm.epm.web.core.ui.internal.EPMView.Button", [jazz.ui.Button], {
		CREATE_BUTTON_EVENT: "createButtonEvent",
		DEFAULT_PUBLISHED_OBJECT: "defaultPublishedObject",

		/* Defaults */
		// In IE7 if not initialized to false, the menu is never loaded
		menuOpen: false,
			
        constructor: function(args){
            this.tree = args.tree;
        },
        
		/**
		 * This method is called when the user presses a single click on the create button.
		 */
        _handleClick: function(event){
           	dojo.publish(this.CREATE_BUTTON_EVENT, [this.DEFAULT_PUBLISHED_OBJECT]);
			event.preventDefault();
			event.stopPropagation();
			dojo.stopEvent(event);
        },
		
		_openMenu: function(event){
			dojo.stopEvent(event);
            this._button.blur();
			if (!this.tree._wbsTree._selectedNode) {
				return;
			}
            if (this.menuOpen || this.disabled) {
				this._closeMenu();
				event.preventDefault();
				return;
			}
			
			//Fetch the resource type and work item type.
			var parentNode = this.tree._wbsTree._selectedNode;
			var resourceType = parentNode.item.resourcetype;
			resourceType = dojo.isArray(resourceType) ? resourceType[0] : resourceType;
			
			if (resourceType.indexOf(WBS_Constants.EPM_PROGRAM) !== -1) {
				//loading the wi config for project with URI  + item.identifier + with name  + item.name
				this.createAndAddMenu(resourceType, null, null, parentNode);
			}
			this._fetchWIConfigAndOpenMenu(parentNode);
		},
		
		_fetchWIConfigAndOpenMenu: function(parentNode){
			var resourceType = parentNode.item.resourcetype;
			resourceType = dojo.isArray(resourceType) ? resourceType[0] : resourceType;
			var workItemType = parentNode.item.workItemType;
			
			var containingProjectID = this.tree._wbsTree._getContainingProjectID(parentNode.item);
			if(containingProjectID == null){
				return;
			}
			
			var allWIConfig = ControllersFactory.getWIConfigHandler().getAllWIConfig(containingProjectID);
			if (allWIConfig == null && resourceType.indexOf(WBS_Constants.EPM_PROJECT) !== -1) {
				//loading the wi config for project with URI  + item.identifier + with name  + item.name
				ControllersFactory.getWIConfigHandler().fetchConfig(node.identifier, node.name);
			}
			var tempThis = this;
			var loadConfigInterval = setInterval(function(){
				if(allWIConfig == EPMConstants.LOADING_CONFIG){
					allWIConfig = ControllersFactory.getWIConfigHandler().getAllWIConfig(containingProjectID);
				}else{
					clearInterval(loadConfigInterval);
					tempThis.createAndAddMenu(resourceType, allWIConfig, workItemType, parentNode);
				}
			},100);
		},
		
		createAndAddMenu: function(resourceType, allWIConfig, workItemType, parentNode){
			if((this.tree) 
				&& (this.tree.getCommandButtonMenuItems)){
				var commandButtonMenuItems = 	this.tree.getCommandButtonMenuItems(resourceType
							, allWIConfig, workItemType, parentNode);
				this._openButtonMenu(commandButtonMenuItems);
			}
		},
		_openButtonMenu: function(commandButtonMenuItems){
			var menu = new dijit.Menu({
				onExecute: dojo.hitch(this, this._closeMenu)
			});
			dojo.addClass(menu.domNode, "jazz-ui-Button-Menu");
			for(var i in commandButtonMenuItems){
				var item = commandButtonMenuItems[i];
				menu.addChild(item);
			}
			
			var buttonMenu = new com.ibm.epm.web.core.ui.internal.EPMView.ButtonMenu();			
			buttonMenu._menuArea.appendChild(menu.domNode);			
			document.body.appendChild(buttonMenu.domNode);
			
            var pos = dojo.coords(this._buttonWrapper, true);
            menu.domNode.style.width = pos.w - 4 + "px";
            buttonMenu.domNode.style.paddingTop = pos.h - 7 + "px";
            buttonMenu.domNode.style.top = pos.y + "px";
            buttonMenu.domNode.style.left = pos.x + "px";
			
            this._menuCloseOnBody = this.connect(document.body, "onclick", this._closeMenu);
            this._menuCloseOnMenu = this.connect(this, "_onblur", this._closeMenu);
			this._menuCloseOnKeyPress = this.connect(document.body, "onkeypress", this.listenToKeyEvents);
			menu.focus();
			
			this._buttonMenu = buttonMenu;
			this._menu = menu;
		},
	
		listenToKeyEvents: function(e){
			var keys = dojo.keys;
			switch (e.keyCode) {
				case keys.ESCAPE:
				this.disconnect(this._menuCloseOnKeyPress);
				this._menuCloseOnKeyPress = null;
				this._closeMenu();
				break;
			}
		},

		_closeMenu: function(){
			if(this._buttonMenu){
				this.inherited(arguments);
				delete this._buttonMenu;
			}if(this._menu){
				delete this._menu;
			}
		}
        
    });
dojo.declare("com.ibm.epm.web.core.ui.internal.EPMView.ButtonMenu",
	[dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/EPMView/templates/ButtonMenu.html")
	
});    
})();
