/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

 (function() {
dojo.provide("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");

dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dijit.Dialog");

var ViewConstants = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util = {
	/*
	 * Returns the user id of the currently authenticated user.
	 */
	/* string */ getAuthenticatedUserId: function() {
		return getAuthenticatedContributor().userId;
	},

	/*
	 * Returns an HREF that can be used as a link to the given dashboard info. Tab is
	 * optional, and if specified, will take you to that specific tab. The scopeItem
	 * param indicates the target dashboard depending on the indicated scope. For
	 * personal dashboards, scopeItem is the itemId of the dashboard, for team dashboards
	 * it's an array of segments in the team area path and for project dashboards it's
	 * the project area name.
	 * 
	 * If projectName is specified, it will be shown in the context of that project,
	 * otherwise it will be assumed to be in the current project context.
	 */
	/* string */ getDashboardHref: function(/* string */ scope, /* string */ scopeItem, /* string */ tabId, /* string */ projectName) {
		var href = [];
		if (projectName) {
			var href = [net.jazz.ajax._contextRoot];
			href.push("/web/projects/");
			href.push(encodeURIComponent(projectName));
			if (djConfig.isDebug) {
				href.push("?debug=true");
			}
		}
		
		href.push("#action=");
		href.push(ViewConstants.ACTION_VIEW_EPMVIEW);
		switch(scope) {
			case ViewConstants.SCOPE_CONTRIBUTOR:
				href.push("&");
				href.push(ViewConstants.URL_PARAM_ID);
				href.push("=");
				href.push(scopeItem);
				break;
			case ViewConstants.SCOPE_TEAM_AREA:
				var path = [];
				for (var i = 0; i < scopeItem.length; i++) {
					if (path.length > 0) {
						path.push("/");
					}
					path.push(encodeURIComponent(scopeItem[i]));
				}
				href.push("&");
				href.push(ViewConstants.URL_PARAM_TEAM);
				href.push("=");
				href.push(path.join(""));
				break;
			case ViewConstants.SCOPE_PROJECT_AREA:
				break;
		}
		if (tabId) {
			href.push("&");
			href.push(ViewConstants.URL_PARAM_TAB);
			href.push("=");
			href.push(tabId);
		}
		return href.join("");
	},
	
	/*
	 * Returns an HREF that can be used as a link to the given dashboard. Tab is optional,
	 * and if specified, will take you to that specific tab.
	 */
	/* string */ getDashboardItemHref: function(/* DashboardBundleDTO */ dashboardBundle, /* string */ tabId) {
		var dashboardItem = dashboardBundle.dashboard;
		var scope = "";
		var scopeItem = "";
		if (dashboardItem) {
			scope = dashboardItem.scope;
			switch (scope) {
				case ViewConstants.SCOPE_CONTRIBUTOR:
					scopeItem = dashboardItem.itemId;
					break;
				case ViewConstants.SCOPE_TEAM_AREA:
					scopeItem = dashboardBundle.defaults.team.path;
					break;
				case ViewConstants.SCOPE_PROJECT_AREA:
					scopeItem = dashboardBundle.defaults.project.name;
					break;
			}
		}
		return this.getDashboardHref(scope, scopeItem, tabId);
	},

	/*
	 * showMessage: Show the message in the dashboard, used to indicate progress etc
	 */	
	showMessage: function(message){
		jazz.app.currentApplication.workbench.showMessage(message);
	},

	/*
	 * hideMessageArea: Hide the message in the dashboard
	 */	
	hideMessageArea: function(){
		jazz.app.currentApplication.workbench.hideMessageArea();
	},

	/*
	 * userIsAdmin: Returns true if the logged on user is Admin
	 */		
    userIsAdmin: function(){
        return com.ibm.team.repository.web.ui.internal.Util.userIsAdmin();
    },

	/*
	 * _lockPage: 
	 * 		displays message text on dashboard header,
	 * 		disables the complete page.
	 */
	_lockPage: function(message) {
		if (!this.dialog) {
			this.showMessage(message);
            var div = document.createElement("div");
            document.body.appendChild(div);
            this.dialog = new dijit.Dialog({
                refocus: false
            }, div);
            this.dialog.show();
            this.dialog.domNode.style.zIndex = 999;
			this.dialog.domNode.style.display = "none";
		}
	},
	
	/*
	 * _releasePage:
	 * 		removes the message text from dashboard header,
	 * 		brings back the page to active state. 
	 */
	_releasePage: function() {
		if (this.dialog != null) {
			this.hideMessageArea();
			this.dialog.hide();
			this.dialog.destroy();
			delete this.dialog;
		}
	}
};

})();