/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/**
 * Script for building and updating the dashboard in the top navigation bar
 */
 (function() {
dojo.provide("com.ibm.epm.web.core.ui.internal.EPMView.TopNav");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");

// needed for com.ibm.team.dashboard.web.ui.internal.LeftNavDashboards
dojo.require("com.ibm.team.dashboard.web.ui.internal.LeftNav");

dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants");

var DashboardConstants = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.ViewConstants;
var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

dojo.requireLocalization("com.ibm.epm.web.core.ui", "Messages");

dojo.declare("com.ibm.epm.web.core.ui.internal.EPMView.TopNav",
	dijit._Widget, {
		
	/*
	 * Creates the initial (empty) top nav content, and asynchronously
	 * populates it.
	 */
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages");
		var leftNavSectionDiv = this.domNode;
		this.isLoaded = false;
		
		// Used to attach to the leftlinks of the jazz app navbar
		var navbarLeft = dojo.getObject("jazz.app.currentApplication.ui.navbarLeft");
		this._topNav = navbarLeft;
		this.getSharedDashboards();
		return leftNavSectionDiv;
	},

	/*
	 * Fetch all the shared dashboards for the user from the server to update the top nav bar with links
	 */	
	getSharedDashboards: function() {
		// Used to update the nav bar once all the predefined views are loaded
		this._noOfViews = 0;
		
		var serviceObj = {
			_topNav: this,
			success: function(results) {
				var _topNav = this._topNav;
				_topNav._navInfo = results;
				_topNav._populate();
			},
			failure: function(errorObj) {
				console.error(Constants.LeftNav.LEFT_NAV_INFO_ERROR + errorObj.message);
			}
		};
		/*
		* ( string projectAreaItemId, string filter, int startIndex, int maxResults, string token, string orderBy, boolean orderByAscending, ServiceResponseHandler srh, ServiceTracker tracker)
		*/
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getSharedDashboards(getCurrentProjectArea().itemId, null, 0, 50, null, null, false, srh, null);
	},
	
	/*
	 * Update the internal array to manage the user views appearing on the top nav bar
	 */	
	_populate: function() {
		this._usrViews = [];
		if (this._navInfo) {
			if (this._navInfo.dashboards) {
				for (var i = 0; i < this._navInfo.dashboards.length; i++) {
					this.getTemplateId(this._navInfo.dashboards[i]);
				}
			}
			else {
				// No views
				this.isLoaded = true;
				this.creatingViews = false;
			}
		}
	},

	/*
	 * Update the top nav bar, used to refresh the top nav bar views based on new creation or deletion of views
	 */		
	_updateNavbar: function(){
		this._cleanNavBar();
    	this._buildNavBar();			
	},

	/*
	 * Build the top nav bar with the predefined views
	 */	
	_buildNavBar: function() {
		if (!this._usrViews ||!this._usrViews.length )
		{
			return;
		}
		
		var navBarItem;
		for (var i = 0; i < this._usrViews.length; i++) {
			navBarItem = this._doAddNavbarItem(this._usrViews[i].id, this._usrViews[i].title, this._usrViews[i].selected);
			// update the usrViews with created button
			if (navBarItem) {
				this._usrViews[i].navBarItem = navBarItem;
			}
		}

		// Add the Admin link
		if (jazz.app.currentApplication._userIsAdmin) {			
			this._addAdminButton();
		}
	},

	/*
	 * Remove all the child nodes except for the create view button.
	 */		
	_cleanNavBar:function(){
		if ( this._topNav.hasChildNodes() )
		{
   		 while ( this._topNav.childNodes.length > 0 )
   			 {
   	   		  this._topNav.removeChild( this._topNav.firstChild );       
   			 } 
		}
	},
	
	/*
	 * Adds a link to Top Nav on the top right handside corner to go to "Admin Page"
	 */ 
	_addAdminButton: function(){
		var divElement = document.createElement("div");
				
		// Add Admin link
		if (!this.adminLink) {
			var adminArgs = {callback: dojo.hitch(this, this._openAdminView),
							 onclick: true,
							 label: this.messages.TARA_ADMIN_VIEW_BUTTON,
							 href: this.getAdminLinkHref(),
							 align: "left"};
			this.adminLink = this._createNavBarItem(adminArgs);
		}
	},

	/**
	 * Get the url for the RPC admin link  
	 */
	getAdminLinkHref : function() {
		var contextRoot = net.jazz.ajax._contextRoot;		
		var adminLink = "https://" + window.location.host + contextRoot + DashboardConstants.TARA_ADMIN_HREF;
		return adminLink;
	},
	
	/**
	 * Opens the admin view in a new Browser Tab/Window
	 * TODO To follow consistent mechanism of opening RTC related views.  
	 */
	_openAdminView: function(){
		//open admin window
		window.open(this.getAdminLinkHref());
		return false;	
	},

	/*
	 * Adds a new nav bar item in the user views array
	 */ 	
	_addNewNavBarItem:function(id, title){
		if (this._usrViews.length < 3) {
			var view = {
				"id": id,
				"title": title,
				"url": "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_EPMVIEW + "&" + DashboardConstants.URL_PARAM_ID + "=" + id,
				"selected": false
			};
			this._usrViews.push(view);
		}
        this._updateNavbar();
	},

	/*
	 * Adds a new nav bar item to the top nav bar
	 */		
	_doAddNavbarItem: function(/* string */ itemId, /* string */title, selected) {
		var href = "#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_EPMVIEW + "&" + DashboardConstants.URL_PARAM_ID + "=" + itemId;
		var args = {};
		args.href = href;
		args.label = title;
		args.pageId = itemId;
		args.callback = dojo.hitch(this, this._doButtonHighlight);
		args.onclick = false;
		args.selected = selected;

		button = this._createNavBarItem(args);
		return button;
	},

	/*
	 * This method is used to update the href stored in the button to be in sync with the current active tab for the selected view
	 */
	updateSelectedNavBarItemHref: function(url) {
		if (this.selectedNavBarItem) {
			this.selectedNavBarItem.href = url;
		}		
	},
	
	/*
	 * This method is called for every personaldashboard built from a template. If the template Id is the one for the views then add the dashboard details to the user views.
	 */	
	updateViews: function(personalDashboard, templateId) {
		var view;
		
		switch (templateId) {
			case DashboardConstants.WORK_VIEW_TEMPLATE_ID:
				view ={
					"id":personalDashboard.itemId,
					"title":personalDashboard.title,
					"url":"#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_EPMVIEW + "&" + DashboardConstants.URL_PARAM_ID + "=" + personalDashboard.itemId,
					"selected": false,
					"templateId": templateId
				};
				this._usrViews[0] = view;
				this._noOfViews++;
				break;

			case DashboardConstants.RESOURCE_VIEW_TEMPLATE_ID:
				view ={
					"id":personalDashboard.itemId,
					"title":personalDashboard.title,
					"url":"#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_EPMVIEW + "&" + DashboardConstants.URL_PARAM_ID + "=" + personalDashboard.itemId,
					"selected": false,
					"templateId": templateId
				};
				this._usrViews[1] = view;
				this._noOfViews++;
				break;

			case DashboardConstants.TIME_VIEW_TEMPLATE_ID:
				view ={
					"id": personalDashboard.itemId,
					"title":personalDashboard.title,
					"url":"#" + Constants.URL_PARAM_ACTION + "=" + DashboardConstants.ACTION_VIEW_EPMVIEW + "&" + DashboardConstants.URL_PARAM_ID + "=" + personalDashboard.itemId,
					"selected": false,
					"templateId": templateId
				};
				this._usrViews[2] = view;
				this._noOfViews++;
				break;
		}

		if (this._noOfViews == DashboardConstants.TOTAL_NO_OF_VIEWS) {
			this._updateNavbar();
			this.isLoaded = true;
		}
	},

	/*
	 * This method fetches the templateId used to create the given personalDashboard
	 */	
	getTemplateId: function(personalDashboard) {
		var self = this;
		var serviceObj = {
			success: function(/* ResponseDTO */ responseDTO) {
				var templateId;
				
				switch (responseDTO.status) {
					case "ERROR":
						// invalid request or server error
						console.error(responseDTO.msg);
						break;
						
					case "OK":
						// fall through
					default:
						if ((responseDTO.data) && (responseDTO.data.dashboard) && (responseDTO.data.dashboard.role)) {
							templateId = responseDTO.data.dashboard.role;
						}
						break;
				}
				if (templateId) {
					self.updateViews(personalDashboard, templateId);
				}
			},
			failure: function(errorObj) {
				console.error(errorObj.message);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.getDashboard(personalDashboard.itemId, null, getCurrentProjectArea().name, srh);
	},

	/*
	 * Returns the default view stored in the user views array
	 */	
	getView: function(itemId) {
		//Return a view id from the user views.
		if (!this._usrViews ||!this._usrViews.length )
		{
			return;
		} else {
			// Adding hooks to handle static urls for being referenced from Welcome page
			switch (itemId) {
				// Work View: https://<ip>/jazz/epmWeb#action=com.ibm.epm.web.core.viewEPMPage&id=workManagement
				case DashboardConstants.WORK_VIEW_TEMPLATE_ID:
					return this._usrViews[0].url;

				// Resource View: https://<ip>/jazz/epmWeb#action=com.ibm.epm.web.core.viewEPMPage&id=resourceManagement
				case DashboardConstants.RESOURCE_VIEW_TEMPLATE_ID:
					return this._usrViews[1].url;

				// Time View: https://<ip>/jazz/epmWeb#action=com.ibm.epm.web.core.viewEPMPage&id=timeManagement
				case DashboardConstants.TIME_VIEW_TEMPLATE_ID:
					return this._usrViews[2].url;

				default:
					break;
			}
			
			var length = this._usrViews.length;
			for (var i = 0; i < length; i++)
			{
				if (itemId) {
					if ((this._usrViews[i]) && (this._usrViews[i].id == itemId)) {
						return this._usrViews[i].url;
					}
				} else {
					if ((this._usrViews[i]) && (this._usrViews[i].selected == true)) {
						return this._usrViews[i].url;
					}
				}
			}
			return this._usrViews[0].url;	
		}
	},

	/*
	 * Returns the id of the default view
	 */	
	getViewId: function(itemId){
		//Return a view id from the user views.
		if (!this._usrViews ||!this._usrViews.length )
		{
			return;
		} else {
			// Adding hooks to handle static urls for being referenced from Welcome page
			switch (itemId) {
				// Work View: https://<ip>/jazz/epmWeb#action=com.ibm.epm.web.core.viewEPMPage&id=workManagement
				case DashboardConstants.WORK_VIEW_TEMPLATE_ID:
					return this._usrViews[0].id;

				// Resource View: https://<ip>/jazz/epmWeb#action=com.ibm.epm.web.core.viewEPMPage&id=resourceManagement
				case DashboardConstants.RESOURCE_VIEW_TEMPLATE_ID:
					return this._usrViews[1].id;

				// Time View: https://<ip>/jazz/epmWeb#action=com.ibm.epm.web.core.viewEPMPage&id=timeManagement
				case DashboardConstants.TIME_VIEW_TEMPLATE_ID:
					return this._usrViews[2].id;

				default:
					break;
			}
			
			if (itemId) {
				return itemId;
			} else {
				var length = this._usrViews.length;
				for (var i = 0; i < length; i++) {
					if ((this._usrViews[i]) && (this._usrViews[i].selected == true)) {
						return this._usrViews[i].id;
					}
				}
				return this._usrViews[0].id;
			}
		}
	},
	
	/* domNode */ findListNode: function(/* string */ id) {
		return dojo.byId(id + Constants.LeftNav.LI_ID_SUFFIX);
	},
	
	destroy: function() {
		this.inherited(arguments);
	},

	/*
	 * Creates a button, puts it in the navbar and returns a reference.
	 * args: iconSrc, 
	 * 		 title - displayed on the button,
	 * 		 href - href for the button,
	 * 		 callback - callback function on clicking the button,
	 * 		 align: "left" (default) or "right" - attaches the button to the left links or right links of the application's nav bar
	 */
	_createNavBarItem: function(args) {
		var label = args.label ? args.label : "";
		var button = document.createElement("a");
		if (args.pageId){
			button.pageId = args.pageId;	
		
			args.href = "#action=com.ibm.epm.web.core.viewEPMPage&id=" + args.pageId;
		}
		button.href = args.href ? args.href : "#";		

		if (args.callback) {
			if (args.onclick) {
				button.onclick = args.callback;
			}
			else {
				dojo.connect(button, 'onclick', args.callback);
			}
		}

		button.title = label;
		if (args.iconSrc) {
			var icon = document.createElement("img");
			icon.alt = label;
			icon.src = args.iconSrc;
			button.appendChild(icon);			
		}
		button.appendChild(document.createTextNode(label));
		var navbarLinks;
		if (args.align && args.align == "right") {
			navbarLinks = dojo.getObject("jazz.app.currentApplication.ui.navbarRight");
		} else {
			navbarLinks = dojo.getObject("jazz.app.currentApplication.ui.navbarLeft");
		}
		if (navbarLinks) {
			navbarLinks.appendChild(button);
		}

		if (args.selected) {
			this._buttonHighlight(button);
		}
		return button;
	},

	/*
	 * Method to highlight the nav bar item corresponding to the itemId of the view which is passed in the parameter
	 */	
	selectNavItem: function(itemId) {
		if (!this._usrViews || !this._usrViews.length)
		{
			// If the view is loaded before the this._usrViews array gets populated
			this.selectedNavBarItemId = itemId;
			return;
		}
		
		var length = this._usrViews.length;
		for (var i = 0; i < length; i++)
		{
			if ((this._usrViews[i]) && (this._usrViews[i].id == itemId)) {
				var button = this._usrViews[i].navBarItem;
				if (button) {
					this._buttonHighlight(button);
				}
			}
		}		
	},
	
	/*
	 * Event handler for navBarItem to highlight the name of the view in the navBar indicating its selection
	 */	
	_doButtonHighlight: function (event) {
		if ((event) && (event.target)) {
			var element = event.target;
			this._buttonHighlight(element);
		}
	},

	/*
	 * Method to update the css class for the selected or unselected button
	 */	
	_buttonHighlight: function (element) {
		var unselectedTitle;
		var selectedTitle = element.title;
		
		if (this.selectedNavBarItem) {
			// If its the current selected view, simply return
			if (selectedTitle == this.selectedNavBarItem.title) {
				return;
			}
			dojo.removeClass(this.selectedNavBarItem,"selected");
			unselectedTitle = this.selectedNavBarItem.title;
		}
		
		dojo.addClass(element,"selected");
		this.selectedNavBarItem = element;
		
		// update usrViews
		var length = this._usrViews.length;
		for (var i = 0; i < length; i++) {
			if (this._usrViews[i].title == unselectedTitle) {
				this._usrViews[i].selected = false;
			}
			if (this._usrViews[i].title == selectedTitle) {
				this._usrViews[i].selected = true;
			}
		}
	}
});

})();
