/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.web.core.ui.internal.EPMWBSTree");

dojo.require("com.ibm.epm.web.core.ui.internal.EPMTree");
dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
dojo.require("com.ibm.epm.web.core.util.internal.Constants");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");

var EPMTreeEvents = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
var CoreConstants = com.ibm.epm.web.core.util.internal.Constants;

var EPMTree = com.ibm.epm.web.core.ui.internal.EPMTree;
var EPMTreeClient = com.ibm.epm.web.core.client.internal.EPMTreeClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

(function(){
	
	var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;

    dojo.declare("com.ibm.epm.web.core.ui.internal.EPMWBSTree", EPMTree, {
    
        //TODO : IMP - Have to remove this function to get data for the tree to separate class so that
        // different data fetching mechanisms can be implemented
        
        // Get all the Root level tree nodes for the Tree
        getRootNodes: function(){
        
            var self = this;
            var serviceObj = {
                success: function(data){                    
					if ((!data)) {
						var item = self.getItemNodeByIdentity("Root");
                    	if (item) {
                        	self._deleteItemNode(item);
                    	}
                        console.log("No response");
                        return;
                    }
                    var item = null;
                    var isEmptyCollection = true;
                    for (var i in data) {
                        // add all the required information to the variable
                        
                        var currentElementLink = data[i];
						if(currentElementLink.program){
                        	item = self._getTreeNodeModel(currentElementLink.program, null);
						}
						else{
							//TODO check with sandy
							item = self._getTreeNodeModel(currentElementLink.project, null);
						}
                        item.dropdown = true;
                        if (self._treeStore && self.getItemNodeByIdentity(item.identifier)) {
                            self._updateNode(item);
                        }
                        else {
                            self._addNode(null, item, i);
                            self._addTempChildNode(item);
                        }
                        isEmptyCollection = false;
                        
                        if (self._openedItemIds) {
                            if (self._openedItemIds[item.identifier]) {
                                //Expand Node
                                var itemNode = self.getNodeByItemIdentity(item.identifier);
                                if (itemNode) {
                                    self._expandNode(itemNode);
                                }
                            }
                        }
                    }
					var publishObj = {
					 	treeID: self._treeID
					 };
                	dojo.publish(EPMTreeEvents.ROOT_NODES_LOADED, [publishObj]);
                    if (isEmptyCollection && self._containingWidget && self._containingWidget.handleEmptyCollection) {
                        self._containingWidget.handleEmptyCollection();
                    }
                },
                
                failure: function(errorObj){
					// If server returned error while fetching all root elements
					// Remove the root node in the tree and disable the create button
					var item = self.getItemNodeByIdentity("Root");
                	if (item) {
                    	self._deleteItemNode(item);
						self._containingWidget._createDisabled = true;				
                	}

                    //Show the error
                    var errMsg = errorObj.errorMessage || "Cannot Retrieve Information";
                    self._displayErrorMessage(errMsg);
                    if (self._containingWidget && self._containingWidget.handleRootElementFailure) {
                        self._containingWidget.handleRootElementFailure(errorObj);
                    }
                    
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            EPMTreeClient.getAllRootElementsForWBS(srh);
        },
		
		getChildNodes: function(uri){
			
            var self = this;
            var parentIdentifier = null;
            
			if ((typeof(uri) == "undefined")  || (uri == null)){
				return;
			}
			
            if (dojo.isArray(uri)) {
                parentIdentifier = uri[0];
            }
            else {
                parentIdentifier = uri;
            }
            
            var serviceObj = {
                success: function(data){
                    if (!data) {                        
                        return;
                    }
                    var node;
					console.debug("Fetching the children for : " + self.id + " for element " + parentIdentifier);
					if (self.id == CoreConstants.WBS_TREE_ID && parentIdentifier.indexOf(CoreConstants.EPM_PROJECT)) {
						self._fetchConfig_addChildrenByLinks(data, parentIdentifier);
					}else{
						self._addChiildrenByLinks(data, parentIdentifier);
					}
                },
                
                failure: function(errorObj){
                    //Show the error
                    var parentNode = self.getNodeByItemIdentity(parentIdentifier);
                    self._removeTempChild(parentNode.item);
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			uri+="?noSchedule=true&noPercents=true";
            EPMTreeClient.getURI(srh, uri);
        },
		
		_fetchConfig_addChildrenByLinks: function(data, projectIdentifier){
			var allWIConfig = ControllersFactory.getWIConfigHandler().getAllWIConfig(projectIdentifier);
			
			var tempThis = this;
			var loadConfigInterval = setInterval(function(){
				if(allWIConfig == EPMConstants.LOADING_CONFIG){
					allWIConfig = ControllersFactory.getWIConfigHandler().getAllWIConfig(projectIdentifier);
					console.debug("waiting for the config to be loaded");
				}else{
					clearInterval(loadConfigInterval);
					tempThis._addChiildrenByLinks(data, projectIdentifier);
				}
			},100);
		}
    });
})();
