/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.GanttBar");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojo.date");
    dojo.require("dojo.date.locale");
    //dojo.require("dojo.dnd.move");
    
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub");
    dojo.provide("com.ibm.epm.work.web.ui.internal.WBSConstants");
    
    dojo.require("dijit.Tooltip");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttBarResizeHandle");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttTimeLine");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttMover");
	dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
	
	// Localization
	dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    /*
     * This is a wrapper widget for a div inside a div with resize handler
     */
    //CONSTANTS FOR GANTT BAR
    var BAR_OFFSET = 28;
    var ROW_HEIGHT = 21.11;
    var ROW_BORDER_HEIGHT = 1;
	var ORIGINAL_OUTER_BAR_BORDER_WIDTH = 2;
	var CRITICAL_PATH_BORDER_WIDTH = 1;
    var INNER_BAR_MARGIN = 1;
    var MILESTONE_IMAGE_WIDTH = 16;
    var HIDE_DELAY = 5000;
    var SHOW_DELAY = 500;
    
    var WBSGridGanttPubSub = com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub;
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var dateUtil =  com.ibm.epm.web.core.util.internal.DateUtil;
	var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
	
    /* defineWidget */
    dojo.declare("com.ibm.epm.work.web.ui.internal.GanttBar", [dijit._Widget, dijit._Templated], {
        templateString: '<div class="com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv" tabindex="-1" dojoAttachPoint="_outerBar">' +
        '<div class="com-ibm-epm-work-web-ui-internal-GanttBar-innerBarDiv" dojoAttachPoint="_innerBar">' +
        '</div>' +
        '</div>',
        
        //
        // summary: Makes a pane's content dragable by/within it's surface
        //
        // description: This widget creates a div nested in a div with resizing capability
        // properties:
        visible: false, //To store the visibility state of the gantt bar
        index: 0, //This index is used to position the gantt bar from the top
        ganttBarInfo: null, //This stores all the wbs information
        ganttTimelineRef: null, //Stores the reference to the timeline
        bCreateResizers: true, //boolean value to allow creation of resizers
        OUTER_BAR_BORDER_WIDTH: 2,			
        /**
         *  postCreate : set display to none for outer and inner bars. Register for mouse events and subscribe to events
         *  Incoming : none
         *  Outgoing : none
         *
         */
        postCreate: function(){
            dojo.style(this._outerBar, "display", "none");
            dojo.style(this._innerBar, "display", "none");
            dojo.forEach(["onMouseOver", "onMouseOut", "onClick", "onMouseDown"], function(event){
                this.connect(this.domNode, event.toLowerCase(), "_" + event);
            }, this);
            
            if (this.bCreateResizers) {
                this.createResizers();
            }
            this._subscriptions = [dojo.subscribe(WBSGridGanttPubSub.ON_GANTTBAR_RESIZE_START, this, "_afterGanttBarResizeStart"), 
                                   dojo.subscribe(WBSGridGanttPubSub.ON_GANTTBAR_RESIZE_END, this, "_afterGanttBarResizeEnd"), 
                                   dojo.subscribe(WBSConstants.DND_MOVE_START, this, "_afterGanttBarMoveStart"), 
                                   dojo.subscribe(WBSConstants.DND_MOVE_STOP, this, "_afterGanttBarMoveStop"), //Abort any drag operations
                                   dojo.subscribe(WBSGridGanttPubSub.ON_GANTTBAR_RESIZE_ABORT, this, "_afterAbortResize"), 
                                   dojo.subscribe(WBSGridGanttPubSub.ON_GANTTBAR_MOVE_ABORT, this, "_afterAbortMove")];
 			this.resizable = true;
            this._constantWidth = false;
        },
        
        /**
         *  _onMouseOver : hover over the gantt bar
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        setIndex: function(newIndex){
            this.index = newIndex;
            //Since new index has been set, update the mover to change the constraint box for movement
       		this._createMover();
        },
        
        /**
         *  _onMouseOver : hover over the gantt bar
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _onMouseOver: function(/*Event*/e){
            this._onHover(e);
        },
        
        /**
         *  _onMouseOut : Hover out of the gantt bar
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _onMouseOut: function(/*Event*/e){
            if (dojo.isDescendant(e.relatedTarget, e.target)) {
                // false event; just moved from target to target child; ignore.
                return;
            }
            this._onUnHover(e);
        },
        
		/**
		 * _onMouseDown:Grey out the gantt bar if it is non-editable. <br>
		 */
		_onMouseDown: function(/*Event*/ e){
			if (!this.resizable) {
				this.disabled = true;
				dojo.style(this._innerBar, "width", "100%");
				dojo.addClass(this._innerBar, "innerBarDiv-non_editable");
				dojo.addClass(this._outerBar ,"outerDiv-disabled");
				
				this.mouseUpHandle = dojo.connect(document, "onmouseup", this, "changeStyle");

//				commented out for defect 9457				
//		        e.stopPropagation();
//		        e.preventDefault();
//		        dojo.stopEvent(e);
			}
		},


		/**
		 * changeStyle: undo styling of the non-editable gantt bar after mouse up event.
		 */		
		changeStyle: function(){
			dojo.disconnect(this.mouseUpHandle);

			var percentComplete = this.ganttBarInfo.percentC;
			if (this.ganttBarInfo.assignmenttype == WBSConstants.ASSIGNMENT_TYPE_EFFORT) {
				percentComplete = this.ganttBarInfo.percentEC;
			} else if (this.ganttBarInfo.assignmenttype == WBSConstants.ASSIGNMENT_TYPE_DURATION) {
				percentComplete = this.ganttBarInfo.percentDC;
			}
			
			if (percentComplete && percentComplete !== "-") {
				dojo.style(this._innerBar, "width", percentComplete + "%");
			}
			else {
				dojo.style(this._innerBar, "width", "0%");
			}

			dojo.removeClass(this._innerBar, "innerBarDiv-non_editable");
			dojo.removeClass(this._outerBar ,".outerDiv-disabled");
		},

	    /**
         *  makeIrresizable : Method to disable the resizer for the gantt bar
         */	
		makeIrresizable: function(){
			this.resizable = false;
			if (this.oresizer) {
				this._outerBar.removeChild(this.oresizer.domNode);
				this.oresizer = null;
			}
			if (this.oReverseresizer) {
				this._outerBar.removeChild(this.oReverseresizer.domNode);
				this.oReverseresizer = null;
			}
			if (this.moverHandle) {
				this.moverHandle.destroy();
				this.moverHandle = null;
			}
		},

	    /**
         *  _onHover : On hover show tooltip
         *  Incoming : Event e
         *  Outgoing : none
         *
         */		
        _onHover: function(/*Event*/e){
            if (!this._outerBar.parentNode.isDragging) {
                if (!this._showTimer) {
                    this._showTimer = setTimeout(dojo.hitch(this, function(){
                        var toolTipInfo = {
                            "itemName": this.ganttBarInfo.name,
                            "startDate": this.ganttBarInfo.startDate,
                            "finishDate": this.ganttBarInfo.endDate,										
                            "percentageComplete": this.ganttBarInfo.percentC
                        };

						if (this.ganttBarInfo.assignmenttype == WBSConstants.ASSIGNMENT_TYPE_EFFORT && (this.ganttBarInfo.percentEC > 0)) {
							this._updateToolTipForPercentEC(toolTipInfo);
						} else if (this.ganttBarInfo.assignmenttype == WBSConstants.ASSIGNMENT_TYPE_DURATION && (this.ganttBarInfo.percentDC > 0)) {
							this._updateToolTipForPercentDC(toolTipInfo);
						}
											
                        this.showTooltip(toolTipInfo);
                    }), SHOW_DELAY);
                }
                if (!this._hideTimer) {
                    this._hideTimer = setTimeout(dojo.hitch(this, function(){
                        this.hideTooltip(e);
                    }), HIDE_DELAY);
                }
                
                // Highlight the entire row [Tree + Grid + Gantt]
                var publishobj = [];
                publishobj.identifier = this.ganttBarInfo.id;
                publishobj.treeID = WBSConstants.WBS_TREE_ID;
                publishobj.gridID = WBSConstants.WBS_SCHEDULE_GRID_ID;
                publishobj.rowIndex = this.index;
                dojo.publish(WBSGridGanttPubSub.ON_GANTT_MOUSE_OVER, [publishobj]);
			}
        },

        /**
         *  _updateToolTipForPercentEC : Update tooltip info for percentEC value
         *  Incoming : toolTipInfo
         *  Outgoing : none
         *
         */ 
		_updateToolTipForPercentEC: function(toolTipInfo) {
			var percentEC = 0;
			if (this.ganttBarInfo.percentEC !== "-") {
				percentEC = this.ganttBarInfo.percentEC;
			}
			toolTipInfo.percentageEffortComplete = percentEC;							
		},

        /**
         *  _updateToolTipForPercentDC : Update tooltip info for percentDC value
         *  Incoming : toolTipInfo
         *  Outgoing : none
         *
         */        
		_updateToolTipForPercentDC: function(toolTipInfo) {
			var percentDC = 0;
			if (this.ganttBarInfo.percentDC !== "-") {
				percentDC = this.ganttBarInfo.percentDC;
			}
			toolTipInfo.percentageDurationComplete = percentDC;							
		},
		
        /**
         *  _onUnHover : On unhover hide the tooltip
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _onUnHover: function(/*Event*/e){
            if (!this._outerBar.parentNode.isDragging) {
                // keep a tooltip open if the associated element has focus
                
                if (this._showTimer) {
                    clearTimeout(this._showTimer);
                    delete this._showTimer;
                }
                if (this._hideTimer) {
                    clearTimeout(this._hideTimer);
                    delete this._hideTimer;
                }
                this.hideTooltip(e);
                
                // Deselect the entire row [Tree + Grid + Gantt]
                var publishobj = [];
                publishobj.identifier = this.ganttBarInfo.id;
                publishobj.treeID = WBSConstants.WBS_TREE_ID;
                publishobj.gridID = WBSConstants.WBS_SCHEDULE_GRID_ID;
                publishobj.rowIndex = this.index;
                dojo.publish(WBSGridGanttPubSub.ON_GANTT_MOUSE_OUT, [publishobj]);
            }
        },
        
        /**
         * _onClick: publish events to select the row in grid n gantt and deselects the previously selected row. <br>
         * Incoming: event e.
         * Outgoing: None.
         *
         */
        _onClick: function(e){
            // select the entire row [Tree + Grid + Gantt]
            var publishobj = [];
            publishobj.identifier = this.ganttBarInfo.id;
            publishobj.treeID = WBSConstants.WBS_TREE_ID;
            publishobj.gridID = WBSConstants.WBS_SCHEDULE_GRID_ID;
            publishobj.rowIndex = this.index;
            dojo.publish(WBSGridGanttPubSub.ON_GANTT_ROW_SELECT, [publishobj]);
        },
        
        /**
         *  updateTooltipWithCP : Update the tooltip with CP info
         *  Incoming : current tooltip data
         *  Outgoing : updated tooltip data
         *
         */
        updateTooltipWithCP: function(toolTipInfo){
            if (this.ganttBarInfo.criticalPath) {
                toolTipInfo.earlyFinishDate = this.ganttBarInfo.earlyFinishDate;
                toolTipInfo.earlyStartDate = this.ganttBarInfo.earlyStartDate;
                toolTipInfo.lateFinishDate = this.ganttBarInfo.lateFinishDate;
                toolTipInfo.lateStartDate = this.ganttBarInfo.lateStartDate;
            }
            
            return toolTipInfo;
        },
        
        /**
         *  showTooltip : Prepare the html for tooltip and show it
         *  Incoming : tooltip data
         *  Outgoing : none
         *
         */
        showTooltip: function(toolTipInfo){
            //ToolTipInfo structure is :
            //"itemName"
            //"startDate"
            //"finishDate"
			//"EffortComplete" or "Duration Complete"
            //"percentageComplete" if user overrides %C in timesheet
            // When CP is enabled the below additional info is shown
            //"earlyFinishDate"
            //"earlyStartDate"
            //"lateFinishDate"
            //"lateStartDate"
            // Enable this once the format of the tooltip is finalised
            //toolTipInfo = this.updateTooltipWithCP(toolTipInfo);
            var msg = "<b>";
            if (toolTipInfo.itemName) {
                msg += toolTipInfo.itemName;
                msg += "</b><br>";
            }
            if (toolTipInfo.startDate) {
                msg += dojo.date.locale.format(toolTipInfo.startDate, {
                    selector: 'date',
                    formatLength: 'short'
                });
                msg += " - ";
            }
            if (toolTipInfo.finishDate) {
                msg += dojo.date.locale.format(toolTipInfo.finishDate, {
                    selector: 'date',
                    formatLength: 'short'
                });
                msg += "<br>";
            }
            if (toolTipInfo.percentageEffortComplete >= 0) {
                msg += toolTipInfo.percentageEffortComplete;
                msg += WorkMessages.PERCENTAGE_EFFORT_COMPLETE;
                msg += "<br>";
            }
            if (toolTipInfo.percentageDurationComplete >= 0) {
                msg += toolTipInfo.percentageDurationComplete;
                msg += WorkMessages.PERCENTAGE_DURATION_COMPLETE;
                msg += "<br>";
            }
            if (toolTipInfo.percentageComplete >= 0) {
                msg += toolTipInfo.percentageComplete;
                msg += WorkMessages.PERCENTAGE_COMPLETE;
                msg += "<br>";
            }
            if (toolTipInfo.earlyStartDate) {
                msg += WorkMessages.ES;
                msg += dojo.date.locale.format(toolTipInfo.earlyStartDate, {
                    selector: 'date',
                    formatLength: 'short'
                });
                msg += "<br>";
            }
            if (toolTipInfo.earlyFinishDate) {
                msg += WorkMessages.EF;
                msg += dojo.date.locale.format(toolTipInfo.earlyFinishDate, {
                    selector: 'date',
                    formatLength: 'short'
                });
                msg += "<br>";
            }
            if (toolTipInfo.lateStartDate) {
                msg += WorkMessages.LS;
                msg += dojo.date.locale.format(toolTipInfo.lateStartDate, {
                    selector: 'date',
                    formatLength: 'short'
                });
                msg += "<br>";
            }
            if (toolTipInfo.lateFinishDate) {
                msg += WorkMessages.LF;
                msg += dojo.date.locale.format(toolTipInfo.lateFinishDate, {
                    selector: 'date',
                    formatLength: 'short'
                });
                msg += "<br>";
            }
			//Tooltip.show function first validates the domNode and doesn't show tooltip again if it already exists. 
			if (dijit._masterTT && dijit._masterTT.aroundNode == this.domNode) {
				dijit.hideTooltip(this.domNode);
			}
            dijit.showTooltip(msg, this.domNode);
			
        },
        
        //
        /**
         *  hideTooltip : Hide the tooltip
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        hideTooltip: function(e){
            dijit.hideTooltip(this.domNode);
            // FIXME: make sure that pesky tooltip doesn't reappear!
            // would be nice if there were a way to hide tooltip without regard to aroundNode.
            dijit._masterTT._onDeck = null;
        },
        
        /**
         *  showCritPathBar : This function to change the display properties of the bar if its in critical path
         *  Incoming : none
         *  Outgoing : none
         *
         */
        showCritPathBar: function(){
			dojo.addClass(this._outerBar, "com-ibm-epm-work-web-ui-internal-GanttBar-critPathBarDiv");
			this.setOuterBarBorderWidth();
        },
        
        /**
         *  hideCritPathBar : This function to reset the display properties of the bar as normal
         *  Incoming : none
         *  Outgoing : none
         *  Removing the condition of checking the workitemtype to task  , as there can be any other type as welll
         */
        hideCritPathBar: function(){
             dojo.removeClass(this._outerBar, "com-ibm-epm-work-web-ui-internal-GanttBar-critPathBarDiv");
			 this.resetOuterBarBorderWidth();
			 this.adjustWidth();
         },
		
		/**
		 * When Critical path is checked, the border width is increased to 3px, it impacts the calculation.
		 * Hence changing the outer bar width whenever the critical path is checked.  
		 */
		setOuterBarBorderWidth: function(){
			this.OUTER_BAR_BORDER_WIDTH = ORIGINAL_OUTER_BAR_BORDER_WIDTH + CRITICAL_PATH_BORDER_WIDTH;
		},
		
		/**
		 * Reset the width when the critical path is unchecked or the gantt bar is hidden.
		 */
		resetOuterBarBorderWidth: function(){
			this.OUTER_BAR_BORDER_WIDTH = ORIGINAL_OUTER_BAR_BORDER_WIDTH;
		},

        /**
         *  setVisualProperties : This function sets the different visual parameters for the gantt bar
         *  Incoming : Top, Left, width coordinate of the bar
         *  Outgoing : none
         *
         */
        setVisualProperties: function(oTop, oLeft, oWidth){
            if (this.ganttBarInfo.resourcetype != WBSConstants.EPM_MILESTONE_TYPE) {
                //Set the position and size of the box
                var adjustedWidth = (oWidth - this.OUTER_BAR_BORDER_WIDTH * 2) > 0 ? (oWidth - this.OUTER_BAR_BORDER_WIDTH * 2) : this.ganttTimelineRef.getDayWidth();

				if( oTop !== this.top ) {
                	dojo.style(this._outerBar, "top", oTop + 'px');
                }
                if( oLeft !== this.left ) {
                	dojo.style(this._outerBar, "left", oLeft + 'px');
                }
                if ( adjustedWidth !== this.width ) {
                	dojo.style(this._outerBar, "width", adjustedWidth + 'px');
                }

				this.top = oTop;
				this.left = oLeft;
				this.width = adjustedWidth;

                var innerBarMaxWidth = oWidth - this.OUTER_BAR_BORDER_WIDTH * 2 - INNER_BAR_MARGIN * 2; // = 6 ( 2 pixels for margin and 4 for border thickness)
                if (innerBarMaxWidth < 0) {
                    innerBarMaxWidth = 0;
                }
                //TODO: margin always returns me 0, dont understand why? hence setting offset to 2px for now
                //var offset = dojo.style(this._innerBar, "margin");
                var offset = 2;

				var percentComplete = this.ganttBarInfo.percentC;

				if (this.ganttBarInfo.assignmenttype == WBSConstants.ASSIGNMENT_TYPE_EFFORT) {
					percentComplete = this.ganttBarInfo.percentEC;
				} else if (this.ganttBarInfo.assignmenttype == WBSConstants.ASSIGNMENT_TYPE_DURATION) {
					percentComplete = this.ganttBarInfo.percentDC;
				}

				var iWidth = Math.round(percentComplete * innerBarMaxWidth / 100);
                //dojo.style(this._innerBar, "width", iWidth + "px");

				// percentComplete can have "-" for cases where its not applicable
				// added a check to ensure that the style of the inner bar is not set
				if (percentComplete && percentComplete !== "-") {
					dojo.style(this._innerBar, "width", percentComplete + "%");
				} else {
					dojo.style(this._innerBar, "width", "0%");
				}

				if (this.ganttBarInfo.resourcetype == WBSConstants.WORKITEM_TYPE) {
					this.setInnerBarVisualPropertiesForResource(this.ganttBarInfo.workItemType);
				}
				else {
					this.setInnerBarVisualPropertiesForResource(this.ganttBarInfo.resourcetype);
				}
            }
            else {
                //To center align the milestone in the given day width
                var milestoneLeft = oLeft - MILESTONE_IMAGE_WIDTH / 2 + this.ganttTimelineRef.getDayWidth() / 2;
                dojo.style(this._outerBar, "top", oTop + 'px');
                dojo.style(this._outerBar, "left", milestoneLeft + 'px');
                dojo.style(this._outerBar, "width", MILESTONE_IMAGE_WIDTH + 'px'); //This only has an image of width 16px
            }
        },
        /**
         *  setInnerBarVisualPropertiesForResource : This function is used to set the inner bar visual properties based on the
         *	resource type
         *  Incoming : resourcetype
         *  Outgoing : none
         *
         */        
        setInnerBarVisualPropertiesForResource: function(resourcetype) {
        	switch(resourcetype) {
        	case WBSConstants.EPM_PROGRAM_TYPE:
				dojo.addClass(this._outerBar, "outerBarDiv-epm_program_type");
        		dojo.addClass(this._innerBar, "innerBarDiv-epm_program_type");
        		break;
        		
			case WBSConstants.EPM_PROJECT_TYPE:
				dojo.addClass(this._outerBar, "outerBarDiv-epm_project_type");
				dojo.addClass(this._innerBar, "innerBarDiv-epm_project_type");
				break;    

			case WBSConstants.ITERATION_TYPE:
			case WBSConstants.EPM_SUMMARY_TASK_TYPE:				
				dojo.addClass(this._outerBar, "outerBarDiv-epm_iteration_or_summarytask_type");
				dojo.addClass(this._innerBar, "innerBarDiv-epm_iteration_or_summarytask_type");
				break;

			default:
				break;
			}
        },
        
        /**
         *  createResizers : This function attaches the resizers needed to stretch/move the gantt bars
         *  Incoming : none
         *  Outgoing : none
         *
         */
        createResizers: function(){
            //			var innerHandle = document.createElement('div');
            //			this._innerBar.appendChild(innerHandle);
            //			this.iresizer = new com.ibm.epm.work.web.ui.internal.GanttBarResizeHandle({
            //				targetContainer: this._innerBar, 
            //				activeResize: true, 
            //				minWidth: this.ganttTimelineRef.getDayWidth(),
            //				minHeight: 8,
            //				resizeAxis: "x",
            //				style: "bottom: 2px",
            //				right: 2,
            //				position: "absolute"
            //			}, innerHandle);
            //			this.iresizer.startup();
            
            var outerHandle = document.createElement('div');
            this._outerBar.appendChild(outerHandle);
            this.oresizer = new com.ibm.epm.work.web.ui.internal.GanttBarResizeHandle({
                targetContainer: this._outerBar,
                minWidth: this.ganttTimelineRef.getDayWidth(),
                minHeight: 8,
                resizeAxis: "x",
                style: "bottom: 2px",
                right: 2,
                position: "absolute"
            }, outerHandle);
            
            this.oresizer.startup();
            
            var outerReverseHandle = document.createElement('div');
            this._outerBar.appendChild(outerReverseHandle);
            this.oReverseresizer = new com.ibm.epm.work.web.ui.internal.GanttBarResizeHandle({
                targetContainer: this._outerBar,
                minWidth: this.ganttTimelineRef.getDayWidth(),
                minHeight: 8,
                resizeAxis: "-x",
                style: "bottom: 2px",
                right: 2,
                position: "absolute"
            }, outerReverseHandle);
            this.oReverseresizer.startup();
            
            this._createMover();
        },
        
        /**
         *  getWBSData : return the ganttbarinfo by reference
         *  Incoming : none
         *  Outgoing : gantt bar info
         *
         */
        getWBSData: function(){
            return this.ganttBarInfo;
        },
        
        /**
         *  updateMilestone : Special case of updating a milestone
         *  Incoming : none
         *  Outgoing : none
         *
         */
        updateMilestone: function(){
            //Remove the resizers here. The constructor of this class creates for all. we cannot filter based on resourceType at construction time
            if (this.oresizer) {
                this._outerBar.removeChild(this.oresizer.domNode);
                this.oresizer = null;
            }
            if (this.oReverseresizer) {
                this._outerBar.removeChild(this.oReverseresizer.domNode);
                this.oReverseresizer = null;
            }
            //THIS WILL GET CHANGED ONCE MILESTONE HAS START DATE
            // console.log('milestone is taking todays date, this will change when we get start date from server');
            /*var todayDate = new Date();
             todayDate = (new Date(todayDate.getFullYear(), todayDate.getMonth(), todayDate.getDate()));
             this.ganttBarInfo.startDate = todayDate;//todo: will get from server
             this.ganttBarInfo.endDate = todayDate; //todo: will get from server
             this.ganttBarInfo.duration = '1'; //todo: will get from server*/
            if (dojo.hasClass(this._outerBar, "com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv")) {
                dojo.removeClass(this._outerBar, "com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv");
                dojo.addClass(this._outerBar, "com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv-Milestone");
                this._outerBar.innerHTML = "<img src = " + dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/ganttmilestone.gif") + "/>";
            }
        },
        
        /**
         *  _internalUpdate : The gantt bar's position and styling is done here
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _internalUpdate: function(){
            //Take the start date, end date and duration information and update the coordinates
            //Update only if you have valid data
            if (this.ganttBarInfo.resourcetype == WBSConstants.EPM_MILESTONE_TYPE) {
                this.updateMilestone();
            }
            
           if (((this.ganttBarInfo.startDate) && (this.ganttBarInfo.startDate instanceof Date)) &&
            ((this.ganttBarInfo.endDate) && (this.ganttBarInfo.startDate instanceof Date))) {
                var stPixel = this.ganttTimelineRef.getDateToPixel(this.ganttBarInfo.startDate);
                var endPixel = this.ganttTimelineRef.getDateToPixel(this.ganttBarInfo.endDate);
                var dayWidth = this.ganttTimelineRef.getDayWidth();
                
                var outerTop = BAR_OFFSET + (this.index) * (ROW_HEIGHT + ROW_BORDER_HEIGHT); //Top is 28 for first row and position is shifted 25 +1 for border pixels down.
                this.setVisualProperties(outerTop, stPixel, endPixel - stPixel + 1 * dayWidth); //width = end date - start date + 1		
                dojo.style(this._outerBar, "display", "inline");
                
                //Inner bar not shown for milestones
                if (this.ganttBarInfo.resourcetype != WBSConstants.EPM_MILESTONE_TYPE) {
                    dojo.style(this._innerBar, "display", "inline");
                }
                
                this.visible = true;
            }
        },
        
        //Most important function to update the position and state of a gantt bar
        /**
         *  updateBar : Update the visual properties of the bar
         *  Incoming : none
         *  Outgoing : none
         *
         */
        updateBar: function(){
        	//console.log("updateBar:" + this.ganttBarInfo.id);
            this._internalUpdate();
        },
        
		/**
		 * Post hideCritical path, the width of the bar need to be adjusted.
		 */
		adjustWidth: function(){
           if (((this.ganttBarInfo.startDate) && (this.ganttBarInfo.startDate instanceof Date)) &&
            ((this.ganttBarInfo.endDate) && (this.ganttBarInfo.startDate instanceof Date))) {
                var stPixel = this.ganttTimelineRef.getDateToPixel(this.ganttBarInfo.startDate);
                var endPixel = this.ganttTimelineRef.getDateToPixel(this.ganttBarInfo.endDate);
                var dayWidth = this.ganttTimelineRef.getDayWidth();
                
                var outerTop = BAR_OFFSET + (this.index) * (ROW_HEIGHT + ROW_BORDER_HEIGHT); //Top is 28 for first row and position is shifted 25 +1 for border pixels down.
                this.setVisualProperties(outerTop, stPixel, endPixel - stPixel + 1 * dayWidth); //width = end date - start date + 1		
            }
		},
		
        /**
         *  _afterGanttBarResizeStart : This function is called just after the resize of the bar starts
         *  Incoming : div node
         *  Outgoing : none
         *
         */
        _afterGanttBarResizeStart: function(targetContainer){
            if ((this._outerBar == targetContainer) ||
            (this._outerBar == targetContainer.parentNode)) {
                this._coords = dojo.coords(this._outerBar, true);
                this._afterDragStarts();
            }
        },
        
        /**
         *  _afterGanttBarResizeEnd : This function is called just after the resize of the bar ends
         *                            Take the new coordinates of the gantt bar, compute the new dates and then update the server
         *  Incoming : div node
         *  Outgoing : none
         *
         */
        _afterGanttBarResizeEnd: function(targetContainer){
            if (targetContainer.id !== '') {
                if (this._outerBar == targetContainer) {
					this.snapToNearestDate();
					
					var currentStartDate = this.getCurrentStartDate();
					var currentFinishDate =  this.getCurrentFinishDate();
					
					//Check if the date is modified.
					if ((dateUtil.compareDates(this.ganttBarInfo.startDate, currentStartDate) !== 0) ||
						(dateUtil.compareDates(this.ganttBarInfo.endDate, currentFinishDate) !== 0)) {
	
						this.ganttBarInfo.startDate = currentStartDate;
						this.ganttBarInfo.endDate = currentFinishDate;
						//Publish that the gantt row's schedule have been changed
						dojo.publish(WBSGridGanttPubSub.ON_AFTER_UPDATE_GANTTBAR, [this.domNode.id]);
					}
					this._afterDragComplete();
				}
            }
            else {
                if (this._outerBar == targetContainer.parentNode) {
                    this._afterDragComplete();
                    //todo: post inner bar resize work
                }
            }
        },
        
        /**
         *  _afterGanttBarMoveStop : This is called the gantt bar has stopped moving
         *  Incoming : mover object
         *  Outgoing : none
         *
         */
        _afterGanttBarMoveStop: function(mover){
            //We get move calls for all the gantt bars. Select only the one that was currently moved.
            if ((mover.node == this._outerBar) && (this._outerBar.parentNode.isDragging)) {
                console.log("the bar has been moved, now need to update the dates");
                this._afterGanttBarResizeEnd(this._outerBar);
                this._constantWidth = false;
            }
        },
        
        /**
         *  _afterGanttBarMoveStart : This is called the gantt bar has started moving
         *  Incoming : mover object
         *  Outgoing : none
         *
         */
        _afterGanttBarMoveStart: function(mover){
            //We get move calls for all the gantt bars. Select only the one that was currently moved.
            if (mover.node == this._outerBar) {
                this._constantWidth = true;
                var curStartDate = this.getCurrentStartDate();
                var curFinishDate = this.getCurrentFinishDate();
                var stPixel = this.ganttTimelineRef.getDateToPixel(curStartDate);
                var endPixel = this.ganttTimelineRef.getDateToPixel(curFinishDate);
                var dayWidth = this.ganttTimelineRef.getDayWidth();
                this._currentWidth = endPixel - stPixel + 1 * dayWidth;
                this._coords = dojo.coords(this._outerBar, true);
                this._afterDragStarts();   
            }
        },
        
        /**
         *  _afterDragStarts : Drag operations: When bar starts moving, a clone is created to show the old position of the bar
         *		               The dragged bar is decorated to be shown with a color or opaque level
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _afterDragStarts: function(){
            this._outerBar.parentNode.isDragging = true;
            this._mover = dojo.connect(document, "onmousemove", this, "_onGanttBarMove");
            this.hideTooltip();
            this._createBarClone();
            dojo.addClass(this._outerBar, "movingOuterGanttBar");
            dojo.addClass(this._innerBar, "movingInnerGanttBar");
        },
        
        /**
         *  _afterDragComplete : Drag operations: When bar stops moving, the clone is removed
         *  					 The dragged bar is reset to old css class
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _afterDragComplete: function(){
            this._outerBar.parentNode.isDragging = false;
            dojo.disconnect(this._mover);
            this.hideTooltip();
            this._removeClonedBar();
            dojo.removeClass(this._outerBar, "movingOuterGanttBar");
            dojo.removeClass(this._innerBar, "movingInnerGanttBar");
        },
        
        /**
         *  _createBarClone : Create the clone of the current bar
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createBarClone: function(){
            this.ganttBarClone = dojo.clone(this.domNode);
            this.domNode.parentNode.appendChild(this.ganttBarClone);
        },
        
        /**
         *  _removeClonedBar : //Removes the clone of the current bar
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _removeClonedBar: function(){
            this.domNode.parentNode.removeChild(this.ganttBarClone);
        },
        
        /**
         *  getCurrentStartDate : Returns the lower date according to the left coordinate of the gantt bar
         *  Incoming : none
         *  Outgoing : Date object
         *
         */
        getCurrentStartDate: function(){
            var c = dojo.coords(this._outerBar, true);
            var newStartDateInPixel = c.l;
            return this.ganttTimelineRef.getPixelToDate(newStartDateInPixel);
        },
        
        /**
         *  getCurrentStartDateHigher : Returns the higher date according to the left coordinate of the gantt bar
         *  Incoming : none
         *  Outgoing : Date object
         *
         */
        getCurrentStartDateHigher: function(){
            var c = dojo.coords(this._outerBar, true);
            var newStartDateInPixel = c.l;
            return this.ganttTimelineRef.getPixelToDateHigher(newStartDateInPixel);
        },
        
        /**
         *  getCurrentFinishDate : Returns the lower date according to the right coordinate of the gantt bar
         *  Incoming : none
         *  Outgoing : Date object
         *
         */
        getCurrentFinishDate: function(){
            if (this.ganttBarInfo.resourcetype == WBSConstants.EPM_MILESTONE_TYPE) {
                return this.getCurrentStartDate();
            }
            else {
                var c = dojo.coords(this._outerBar, true);
                var newFinishDateInPixel = c.l + c.w - this.OUTER_BAR_BORDER_WIDTH * 2;
                return this.ganttTimelineRef.getPixelToDate(newFinishDateInPixel);
            }
        },
        
        /**
         *  getCurrentFinishDateHigher : Returns the higher date according to the right coordinate of the gantt bar
         *  Incoming : none
         *  Outgoing : Date object
         *
         */
        getCurrentFinishDateHigher: function(){
            if (this.ganttBarInfo.resourcetype == WBSConstants.EPM_MILESTONE_TYPE) {
                return this.getCurrentStartDate();
            }
            else {
                var c = dojo.coords(this._outerBar, true);
                var newFinishDateInPixel = c.l + c.w - this.OUTER_BAR_BORDER_WIDTH * 2;
                return this.ganttTimelineRef.getPixelToDateHigher(newFinishDateInPixel);
            }
        },
        
        /**
         *  _onGanttBarMove : Update the tooltip during the bar move
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _onGanttBarMove: function(){
            var curStartDate = this.getCurrentStartDate();
            var curFinishDate = this.getCurrentFinishDate();
            
            if ((this.lastUpdatedStartDate != curStartDate) || (this.lastUpdatedFinishDate != curFinishDate)) {
                var toolTipInfo = {
                    "itemName": this.ganttBarInfo.name,
                    "startDate": this.getCurrentStartDate(),
                    "finishDate": this.getCurrentFinishDate()
                };
                this.showTooltip(toolTipInfo);
                this.lastUpdatedStartDate = curStartDate;
                this.lastUpdatedFinishDate = curFinishDate;
            }
        },
        
        /**
         *  snapToNearestDate : After move of gantt bar is complete, adject the position to snap to the nearest date
         *  Incoming : none
         *  Outgoing : none
         *
         */
        snapToNearestDate: function(){
            var curStartDate = this.getCurrentStartDate();
            var curFinishDate = this.getCurrentFinishDate();
            
            var stPixel = this.ganttTimelineRef.getDateToPixel(curStartDate);
            var endPixel = this.ganttTimelineRef.getDateToPixel(curFinishDate);
            var dayWidth = this.ganttTimelineRef.getDayWidth();
            var c = dojo.coords(this._outerBar, true);
            
            if (this._constantWidth) {
                // This is the drag case; we don't need to calculate the width
                if (this._currentWidth) {
                    var width = this._currentWidth;
                    var moveDirection = ((c.l - this._coords.l) > 0) ? 1 : -1;
                    var distanceMoved = Math.abs(c.l - this._coords.l);
                    var startPixel = this._coords.l;
                    // Check if the bar was moved more than 50% of the width
                    // If not then we stay where we were before the move; the
                    // older value before move is stored in this._coords.l
                    if (distanceMoved > (dayWidth/2)) {
                        if (moveDirection > 0) {
                            // The bar was moved to the right
                            // Get the next higher date
                            var highStartDate = this.ganttTimelineRef.getPixelToDateHigher(c.l);
                            startPixel = this.ganttTimelineRef.getDateToPixel(highStartDate);
                        } else {
                            // The bar was moved to the left
                            // Keep the lower date
                            startPixel = stPixel;
                        }  
                    }
                    this.setVisualProperties(c.t, startPixel, width);
                }
            } else {
                
                // Check if the bar was moved to reduce the duration from left or right
                var reducedMoveFromLeft = ((this._coords.l - c.l) < 0) ? true: false;
                var reducedMoveFromRight = ((this._coords.w - c.w) > 0) ? true: false;
                if (reducedMoveFromLeft) {
                
                	// Get the start date on the higher end
                    var newCurStartDate = this.getCurrentStartDateHigher();
                    var newStPixel = this.ganttTimelineRef.getDateToPixel(newCurStartDate);
                    var difference = newStPixel - c.l;
                    
                    // Check if the difference between the new start date
                    // and the current start pointer is less than half a day's width.
                    // If yes, then reset the start date to the new start date
                    // We are doing this as the start pointer is closer to the
                    // higher date than it is to the lower date.
                    if (difference < (dayWidth/5)) {
                        stPixel = newStPixel;
                    }
                } else {
                    if (reducedMoveFromRight) {
                        
                        // Get the finish date on the higher end
                        var newCurFinishDate = this.getCurrentFinishDateHigher();
                        var newEndPixel = this.ganttTimelineRef.getDateToPixel(newCurFinishDate);
                        
                        var difference = newEndPixel - (c.l + c.w - this.OUTER_BAR_BORDER_WIDTH * 2);

	                    // Check if the difference between the new finish date
    	                // and the current end pointer is less than half a day's width.
        	            // If yes, then reset the finish date to the new finish date
            	        // We are doing this as the end pointer is closer to the
                	    // higher date than it is to the lower date.                        
                        if (difference < dayWidth/5) {
                            endPixel = newEndPixel;
                        }
                        
                        // If the difference between the start and end dates
                        // is an exact 1 day, then set the end date to be the same
                        // as the start date; later we will be adding a day's worth
                        // of a difference below. This is caused by of our
                        // approximation of decimals while calculating the finish date.
                        var difference = (endPixel - stPixel);
                        if (difference <= dayWidth) {
                            endPixel = stPixel;
                        }
                    }
                }
                
                var width = endPixel - stPixel + 1 * dayWidth;
                this._currentWidth = width;
                
                //This will reposition the bar to the nearest date
                this.setVisualProperties(c.t, stPixel, width);
            }            
        },
        
        /**
         *  removeResizers : Remove all resizers. This is required to remove all mouse listeners attached to these
         *  Incoming : none
         *  Outgoing : none
         *
         */
        removeResizers: function(){
            if (this.iresizer) {
                this._innerBar.removeChild(this.iresizer.domNode);
                this.iresizer.destroy();
                this.iresizer = null;
            }
            if (this.oresizer) {
                this._outerBar.removeChild(this.oresizer.domNode);
                this.oresizer.destroy();
                this.oresizer = null;
            }
            if (this.oReverseresizer) {
                this._outerBar.removeChild(this.oReverseresizer.domNode);
                this.oReverseresizer.destroy();
                this.oReverseresizer = null;
            }
            if (this.moverHandle) {
                this.moverHandle.destroy();
                this.moverHandle = null;
            }
        },
        
        /**
         *  _createMover : Create the Gantt mover widget. this allows the gantt bar to moved in a constrained manner
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createMover: function(){
            //Donot create a mover if the resizers are also not created
            //This may happen during dependency creation process where we disallow any move operations
            //console.log("_createMover");
            if (this.oresizer && this.oReverseresizer) {
                if (this.moverHandle) {
                    this.moverHandle.destroy();
                    this.moverHandle = null;
                }
                var outerTop = BAR_OFFSET + (this.index) * (ROW_HEIGHT + ROW_BORDER_HEIGHT); //Top is 30 for first row and position is shifted 25 +1 for border pixels down.
                var mb = {
                    l: 0,
                    t: outerTop,
                    w: this.ganttTimelineRef.getTotalTimeLineWidth(),
                    h: 0
                };
                this.moverHandle = new com.ibm.epm.work.web.ui.internal.GanttMover(this.domNode, {
                    box: mb
                });
            }
        },
        
        /**
         *  setResizeLowerBounds : Set the minimum size of resizer according to the current day width
         *  Incoming : none
         *  Outgoing : none
         *
         */
        setResizeLowerBounds: function(){
            var minWidth = this.ganttTimelineRef.getDayWidth();
            if (this.iresizer) {
                this.iresizer.minWidth = minWidth;
            }
            if (this.oresizer) {
                this.oresizer.minWidth = minWidth;
            }
            if (this.oReverseresizer) {
                this.oReverseresizer.minWidth = minWidth;
            }
        },
        
        /**
         *  _doAfterAbortDrag : Complete the post abort and reset the dates
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _doAfterAbortDrag: function(){
            //complete the drag
            this._afterDragComplete();
            //Do the update, since the dates havent been changed, it does the work of reset
            this._internalUpdate();
        },
        
        /**
         *  _afterAbortResize : Resize is aborted
         *  Incoming : div node
         *  Outgoing : none
         *
         */
        _afterAbortResize: function(targetContainer){
            //We get move calls for all the gantt bars. Select only the one that was currently moved.
            if ((this._outerBar == targetContainer) ||
            (this._outerBar == targetContainer.parentNode)) {
                this._doAfterAbortDrag();
            }
        },
        
        /**
         *  _afterAbortMove : Move is aborted
         *  Incoming : div node
         *  Outgoing : none
         *
         */
        _afterAbortMove: function(targetContainer){
            //We get move calls for all the gantt bars. Select only the one that was currently moved.
            //Extra check to see if the node is currently being dragged.
            if (((this._outerBar == targetContainer) || (this._outerBar == targetContainer.parentNode)) &&
            (this._outerBar.parentNode.isDragging)) {
                this._doAfterAbortDrag();
            }
        }
        
    });
    
})();
