/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.work.web.ui.internal.GanttBarResizeHandle");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.fx");

dojo.require("com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub");
dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");

var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
var WBSGridGanttPubSub = com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub;

dojo.declare("com.ibm.epm.work.web.ui.internal.GanttBarResizeHandle", [dijit._Widget, dijit._Templated], {
    // summary: A dragable handle used to resize an attached node.
    // description:
    //	The handle on the bottom-right corner of FloatingPane or other widgets that allows
    //	the widget to be resized.
    //	Typically not used directly.
    //
    // targetId: String
    //	id of the Widget OR DomNode that I will size
    targetId: '',
    
    // targetContainer: DomNode
    //	over-ride targetId and attch this handle directly to a reference of a DomNode
    targetContainer: null,
    
    // resizeAxis: String
    //	one of: x|y|-x limit resizing to a single axis, default to x 
    resizeAxis: "x",
    
    // minHeight: Integer
    //	smallest height in px resized node can be
    minHeight: 100,
    
    // minWidth: Integer
    //	smallest width in px resize node can be
    minWidth: 100,
    
    templateString: '<div dojoAttachPoint="resizeHandle" class="GanttBarResizeHandle"><div></div></div>',
    
    /**
     *  postCreate : Handle initialization
     *  Incoming : none
     *  Outgoing : none
     *
     */
    postCreate: function(){
        // summary: setup our one major listener upon creation
        this.connect(this.resizeHandle, "onmousedown", "_beginSizing");
        this.animateSizing = false;
        
        if (!this.minSize) {
            this.minSize = {
                w: this.minWidth,
                h: this.minHeight
            };
        }
        // should we modify the css for the cursor hover to n-resize nw-resize and w-resize?
        this._resizeX = this._resizeY = this._resizeX_opposite = false;
        switch (this.resizeAxis.toLowerCase()) {
            case "x":
                this._resizeX = true;
                dojo.addClass(this.resizeHandle, "GanttBarResizeW");
                break;
            case "y":
                this._resizeY = true;
                dojo.addClass(this.resizeHandle, "GanttBarResizeN");
                break;
            case "-x":
                this._resizeX_opposite = true;
                dojo.addClass(this.resizeHandle, "GanttBarResizeW");
                dojo.addClass(this.resizeHandle, "GanttBarResizeHandleReverseDirection");
                dojo.removeClass(this.resizeHandle, "GanttBarResizeHandle");
                break;
        }
    },
    
    /**
     *  _beginSizing : On mouse down this function is called to get the target node and hitch other mouse calls
     *  Incoming : Event e
     *  Outgoing : none
     *
     */
    _beginSizing: function(/*Event*/e){
        // summary: setup movement listeners and calculate initial size
        
        if (this._isSizing) {
            return false;
        }
        
        this.targetWidget = dijit.byId(this.targetId);
        
        this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : dojo.byId(this.targetId);
        if (this.targetContainer) {
            this.targetDomNode = this.targetContainer;
        }
        if (!this.targetDomNode) {
            return false;
        }
        
        this._isSizing = true;
        this.startPoint = {
            'x': e.clientX,
            'y': e.clientY
        };
        
        // FIXME: this is funky: marginBox adds height, contentBox ignores padding (expected, but foo!)
        var mb = (this.targetWidget) ? dojo.marginBox(this.targetDomNode) : dojo.contentBox(this.targetDomNode);
        this.startSize = {
            'w': mb.w,
            'h': mb.h
        };
        this.coords = dojo.coords(this.targetDomNode, true);
        
        this._pconnects = [];
        this._pconnects.push(dojo.connect(document, "onmousemove", this, "_updateSizing"));
        this._pconnects.push(dojo.connect(document, "onmouseup", this, "_endSizing"));
        this._pconnects.push(dojo.connect(document, "onkeypress", this, "_abortSizing"));
        
        e.stopPropagation();
        e.preventDefault();
        dojo.stopEvent(e);
        
        this.targetDomNode.uniqueIdentifier = {
            treeID: WBSConstants.WBS_TREE_ID
        };
        dojo.publish(WBSGridGanttPubSub.ON_GANTTBAR_RESIZE_START, [this.targetDomNode]);
    },
    
    /**
     *  _updateSizing : called when moving the ResizeHandle determines
     new size based on settings/position and sets styles
     *  Incoming : Event e
     *  Outgoing : none
     *
     */
    _updateSizing: function(/*Event*/e){
    
        this._changeSizing(e);
        e.preventDefault();
    },
    
    /**
     *  _getNewCoords : returns the new coordinates of the node being moved
     *  Incoming : Event e
     *  Outgoing : Object {l,t,w,h}
     *
     */
    _getNewCoords: function(/* Event */e){
    
        // On IE, if you move the mouse above/to the left of the object being resized,
        // sometimes clientX/Y aren't set, apparently.  Just ignore the event.
        try {
            if (!e.clientX || !e.clientY) {
                return false;
            }
        } 
        catch (e) {
            // sometimes you get an exception accessing above fields...
            return false;
        }
        this._activeResizeLastEvent = e;
        
        var dx = this.startPoint.x - e.clientX;
        var dy = this.startPoint.y - e.clientY;
        
        var newW = (this._resizeX) ? this.startSize.w - dx : this.startSize.w;
        var newH = (this._resizeY) ? this.startSize.h - dy : this.startSize.h;
        if (this._resizeX_opposite) {
            newW = this.startSize.w + dx;
        }
        var newL = (this._resizeX_opposite) ? this.coords.l - dx : this.coords.l;
        var newT = this.coords.t; //todo: to be modified for reverse Y direction
        // minimum size check
        if (this.minSize) {
            if (newW < this.minSize.w) {
                newW = this.minSize.w;
                if (this._resizeX_opposite) {
                    newL = this.coords.l + this.startSize.w - this.minSize.w;
                }
            }
            if (newH < this.minSize.h) {
                newH = this.minSize.h;
            }
        }
        return {
            l: newL,
            t: newT,
            w: newW,
            h: newH
        }; // Object
    },
    
    /**
     *  _changeSizing : changes the coordinates of the node
     *  Incoming : Event e
     *  Outgoing : none
     *
     */
    _changeSizing: function(/*Event*/e){
        // summary: apply sizing information based on information in (e) to attached node
        var tmp = this._getNewCoords(e);
        if (tmp === false) {
            return;
        }
        
        dojo.style(this.targetDomNode, "left", tmp.l + "px");
        dojo.style(this.targetDomNode, "top", tmp.t + "px");
        dojo.style(this.targetDomNode, "width", tmp.w + "px");
        dojo.style(this.targetDomNode, "height", tmp.h + "px");
    },
    
    /**
     *  _endSizing : After completing resizing disconnect all mouse listeners
     *  Incoming : Event e
     *  Outgoing : none
     *
     */
    _endSizing: function(/*Event*/e){
        // summary: disconnect listenrs and cleanup sizing
        dojo.forEach(this._pconnects, dojo.disconnect);
        this._isSizing = false;
        this.onResize(e);
        
        e.stopPropagation();
        e.preventDefault();
        dojo.stopEvent(e);
    },
    
    /**
     *  onResize : after resize
     *  Incoming : Event e
     *  Outgoing : none
     *
     */
    onResize: function(e){
        // summary: Stub fired when sizing is done, for things like Grid
        dojo.publish(WBSGridGanttPubSub.ON_GANTTBAR_RESIZE_END, [this.targetDomNode]);
    },
    
    /**
     *  _abortSizing : event to handle all keypress
     *  Incoming : Event e
     *  Outgoing : none
     *
     */
    _abortSizing: function(e){
        if (this._isSizing) {
            var key = e.keyCode || e.charCode;
            var k = dojo.keys;
            
            if (key == k.ESCAPE) {
                dojo.forEach(this._pconnects, dojo.disconnect);
                this._isSizing = false;
                dojo.publish(WBSGridGanttPubSub.ON_GANTTBAR_RESIZE_ABORT, [this.targetDomNode]);
            }
        }
    }
    
});
