/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.GanttDragPane");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    
    /*
     * This is a wrapper widget over DragPane
     */
    /* defineWidget */
    dojo.declare("com.ibm.epm.work.web.ui.internal.GanttDragPane", [dijit._Widget, dijit._Templated], {
        templateString: '<div tabindex="0" class="com-ibm-epm-work-web-ui-internal-GanttDragPane">' +
        '</div>',
        
        //
        // summary: Makes a pane's content dragable by/within it's surface
        //
        // description:
        //		A small widget which takes a node with overflow:auto and
        //		allows dragging to position the content. Useful with images,
        //		or for just adding "something" to a overflow-able div.
        //
        // invert: Boolean
        //		Naturally, the behavior is to invert the axis of the drag.
        //		Setting invert:false will make the pane drag in the same
        //		direction as the mouse.
        invert: true,
        controller: null, //the view which adds this drag pane
        /**
         *  postCreate : constructor
         *  Incoming : none
         *  Outgoing : none
         *
         */
        postCreate: function(){
            this.inherited(arguments);
            var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/openhand.cur") + "'),pointer";
            dojo.style(this.domNode, "cursor", cursorUrl);
            this.connect(this.domNode, "onmousedown", "_down");
			this.connect(this.domNode, "onmouseover", "_mouseover");
            this.connect(this.domNode, "onmouseup", "_up");
            this.connect(this.domNode, "onmouseout", "_up");
			this.connect(document, "onkeydown", "_keydown"); 
			this.connect(document, "onkeyup", "_resetCtrlKeyPressed");
			this.ctrlKeyPressed = false;
        },
        
        /**
         *  _down : upon mouse down register for mouse move and make the gantt view selectable
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _down: function(e){
            if ((e.target.className == "GanttView-ganttDragPaneContent") ||
            (e.target.className == "com-ibm-epm-work-web-ui-internal-GanttDragPane")) {
                // summary: mousedown handler, start the dragging
                dojo.setSelectable(this.domNode, false);
                dojo.disconnect(this._mover);
                
                var t = this.domNode;
				if (!this._isDependencyEnabled()) {
					var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/closedhand.cur") + "'),pointer";
					dojo.style(t, "cursor", cursorUrl);
				}
                
                this.beforeMove = this.domNode.scrollLeft;
                this._x = e.pageX;
                this._y = e.pageY;
                dojo.setSelectable(t, false);
                this._mover = dojo.connect(t, "onmousemove", this, "_move");
            } else if((e.target.className == "com-ibm-epm-work-web-ui-internal-GanttDependency-outerDiv")){
				//If no dependency is already selected, create the array and current selected dependency.
				if(!this.selectedDependency){
					this.selectedDependency = e.target;
					this.selectSingleDependency(e.target);
				}else{
					this.selectedDependency = e.target;
					//There is already some dependency selected, so check for ctrl key press event.
					if (this.ctrlKeyPressed) {
						//Go through the array, if the dependency was already selected, deselect it and remove from array.
						var index = dojo.indexOf(this.selectedDependencyArray, this.selectedDependency);
						if(index > -1){
							this.selectedDependencyArray.splice(index, 1);
							this._highlightDependency(this.selectedDependency, false);
						}else{
							//Select multiple dependencies
							this.selectedDependencyArray.push(e.target);
							this._highlightDependency(this.selectedDependency, true);
						}
					}else{
							//reset the array and select dependency.
							//deselect all the dependencies already created.
							for(var i=0; i<this.selectedDependencyArray.length; i++){
								this._highlightDependency(this.selectedDependencyArray[i], false);
							}
							this.selectSingleDependency(this.selectedDependency);
						}

				}
			}
        },
		
		/**
		 * selectSingleDependency: select the dependency and highlight the arrow.
		 * @param {Object} dependency
		 */
        selectSingleDependency: function(dependency){
			this.selectedDependencyArray = new Array(); 
			this.selectedDependencyArray.push(dependency);
			this._highlightDependency(dependency, true);
		},
		
		/**
		 *_highlightDependency: Method to select or deselect the dependency. 
		 * @param {Object} dependency: dependency to be selected/deselected.
		 * @param {Object} isSelected: if true highlight the dependency else remove the focus.
		 */
		_highlightDependency: function(dependency, isSelected){
			var children = dependency.childNodes;
			if (isSelected) {
				dojo.forEach(children, function(child){
					dojo.addClass(child, "com-ibm-epm-work-web-ui-internal-GanttDependency-outerDiv-selected");
				});
			}else{
				dojo.forEach(children, function(child){
					if(dojo.hasClass(child,"com-ibm-epm-work-web-ui-internal-GanttDependency-outerDiv-selected")){
						dojo.removeClass(child,"com-ibm-epm-work-web-ui-internal-GanttDependency-outerDiv-selected");
					}
				});
			}
		},

		
		_keydown: function(event){
            if (event.ctrlKey) {
                this.ctrlKeyPressed = true;
	        }else{
				this.ctrlKeyPressed = false;
			}
			
			var ClassName = event.target.className.split(" ",1);
			// On Alt-Up and Alt-down key press move the focus up and down.
			if (ClassName == "com-ibm-epm-work-web-ui-internal-GanttBar-outerBarDiv"){
                var nodeWidget = dijit.getEnclosingWidget(event.target);
				rowIndex = nodeWidget.index;
				var ganttId;
				if(event.altKey && event.keyCode === 38 && rowIndex > 0){
					ganttId = this.controller._visibleGanttBarArray[rowIndex - 1 ];                	
				}else if(event.altKey && event.keyCode === 40 && rowIndex < this.controller._visibleGanttBarArray.length - 1){
					ganttId = this.controller._visibleGanttBarArray[rowIndex + 1];					
				}  
				if(ganttId){
					nodeWidget.domNode.setAttribute('tabIndex', "-1");
					gantt = dijit.byId(ganttId);
					gantt.domNode.setAttribute('tabIndex', "0");
					gantt.domNode.focus(); 				
				}
            }
			
			// On Alt-Left and Alt-Right key press move the focus up and down.
			if ((event.target.className == "GanttView-ganttDragPaneContent") ||
            (event.target.className == "com-ibm-epm-work-web-ui-internal-GanttDragPane")) {
				this.beforeMove = this.domNode.scrollLeft;
				if(event.keyCode === 37){
                	this.domNode.scrollLeft += 5;
				}else if(event.keyCode === 39){
					this.domNode.scrollLeft += -5;
				} 
				var clientWidth = this.domNode.clientWidth;
                var currentScrollLeft = this.domNode.scrollLeft;
                var currentScrollWidth = this.domNode.scrollWidth;
                if ((currentScrollLeft - this.beforeMove) > 0) {
                    if ((clientWidth + currentScrollLeft) >= (currentScrollWidth - this.controller.getCurrentTimeLineBlockWidth())) {
                        this.controller.addTimeBlockToRight();
                        this.domNode.scrollLeft -= this.controller.getCurrentTimeLineBlockWidth();
                    }
               }
                if ((currentScrollLeft - this.beforeMove) <= 0) {
                    if (this.domNode.scrollLeft <= this.controller.getCurrentTimeLineBlockWidth()) {
                        this.controller.addTimeBlockToLeft();
                        this.domNode.scrollLeft += this.controller.getCurrentTimeLineBlockWidth();
                    }
                }  
            }
			
			if(event.keyCode === 32 || event.keyCode === 13 ){
				if((event.target.className == "com-ibm-epm-work-web-ui-internal-GanttDependency-outerDiv")){              		
				//If no dependency is already selected, create the array and current selected dependency.
				if(!this.selectedDependency){
					this.selectedDependency = event.target;
					this.selectSingleDependency(event.target);
				}else{
					this.selectedDependency = event.target;
					//There is already some dependency selected, so check for ctrl key press event.
					if (this.ctrlKeyPressed) {
						//Go through the array, if the dependency was already selected, deselect it and remove from array.
						var index = dojo.indexOf(this.selectedDependencyArray, this.selectedDependency);
						if(index > -1){
							this.selectedDependencyArray.splice(index, 1);
							this._highlightDependency(this.selectedDependency, false);
						}else{
							//Select multiple dependencies
							this.selectedDependencyArray.push(event.target);
							this._highlightDependency(this.selectedDependency, true);
						}
					}else{
							//reset the array and select dependency.
							//deselect all the dependencies already created.
							for(var i=0; i<this.selectedDependencyArray.length; i++){
								this._highlightDependency(this.selectedDependencyArray[i], false);
							}
							this.selectSingleDependency(this.selectedDependency);
					}
				}
			}
			}
		},
		_resetCtrlKeyPressed: function(event){
	        this.ctrlKeyPressed = false;
		},
		
        /**
         *  _up : after mouse up, 1) disconnect the mover 2)create time blocks in the left or right section
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _up: function(e){
            if ((e.target.className == "GanttView-ganttDragPaneContent") ||
            (e.target.className == "com-ibm-epm-work-web-ui-internal-GanttDragPane")) {
                // summary: mouseup handler, stop the dragging
                
                dojo.setSelectable(this.domNode, true);
				if (!this._isDependencyEnabled()) {
					var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/openhand.cur") + "'),pointer";
					dojo.style(this.domNode, "cursor", cursorUrl);
				}
                dojo.disconnect(this._mover);
                
                var clientWidth = this.domNode.clientWidth;
                var currentScrollLeft = this.domNode.scrollLeft;
                var currentScrollWidth = this.domNode.scrollWidth;
                if ((currentScrollLeft - this.beforeMove) > 0) {
                    if ((clientWidth + currentScrollLeft) >= (currentScrollWidth - this.controller.getCurrentTimeLineBlockWidth())) {
                        this.controller.addTimeBlockToRight();
                        this.domNode.scrollLeft -= this.controller.getCurrentTimeLineBlockWidth();
                    }
               }
                if ((currentScrollLeft - this.beforeMove) <= 0) {
                    if (this.domNode.scrollLeft <= this.controller.getCurrentTimeLineBlockWidth()) {
                        this.controller.addTimeBlockToLeft();
                        this.domNode.scrollLeft += this.controller.getCurrentTimeLineBlockWidth();
                    }
                }
            }
        },
        
		/**
		 * _mouseover: Change the cursor type on hover. For dependency, it should be a pointer and for dragPane area it should be an open hand.
		 * Change the cursor only if the add dependency is not checked. <br> 
		 * @param {Object} e: mousehover event.
		 */
		_mouseover: function(e){
			if (!this._isDependencyEnabled()) {
				if (e.target.className == "com-ibm-epm-work-web-ui-internal-GanttDependency-outerDiv") {
					dojo.style(this.domNode, "cursor", "default");
				}
				else 
					if ((e.target.className == "GanttView-ganttDragPaneContent") ||
					(e.target.className == "com-ibm-epm-work-web-ui-internal-GanttDragPane")) {
						var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/openhand.cur") + "'),pointer";
						dojo.style(this.domNode, "cursor", "cursorUrl");
					}
			}
		},
		
        /**
         *  _move : Change the scroll position according to the mouse move
         *  Incoming : Event e
         *  Outgoing : none
         *
         */
        _move: function(e){
            if ((e.target.className == "GanttView-ganttDragPaneContent") ||
            (e.target.className == "com-ibm-epm-work-web-ui-internal-GanttDragPane") ||
            (e.target.className == "com-ibm-epm-work-web-ui-internal-GanttDependency-outerDiv")) {
                // summary: mousemove listener, offset the scroll amount by the delta
                //		since our last call.	
                var mod = this.invert ? 1 : -1;
                //Defect 1393: Donot allow y-scroll
                //this.domNode.scrollTop += (this._y - e.pageY) * mod;
                this.domNode.scrollLeft += (this._x - e.pageX) * mod;
                this._x = e.pageX;
                //this._y = e.pageY;
            }
        },
		_isDependencyEnabled: function(){
			return (this.controller.addDependencyButton.attr("checked"));
		}
    });
    
})();
