/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.GanttTimeLine");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttTimeBlock");
    
    /*
     * This is a widget to create a GanttTimeLine
     */
    /* defineWidget */
    dojo.declare("com.ibm.epm.work.web.ui.internal.GanttTimeLine", [dijit._Widget, dijit._Templated], {
        templateString: '<div class="com-ibm-epm-work-web-ui-internal-GanttTimeLine">' +
        '<div class="timeline_Row" dojoAttachPoint="_timeline_Row">' +
        '</div>' +
        '</div>',
        
        //
        // summary: 
        // Timeline widget to show the major and minor timeline
        // description:
        // The timeline widget creates 2 rows. The first row shows the major time and the second row shows the minor		
        // time	
        //		
        //
        timelineType: "Year", //values are year, quarter, month, week
        currentYear: new Date().getFullYear(), //Get The current year
        timelineLowerLimit: null, /*Date*/ //This is the lowest date visible in the timeline. 
        timelineUpperLimit: null, /*Date*/ //This is the maximum date visible in the timeline.
        /**
         *  postCreate : set the lower and upper timelimit and switch to year view
         *  Incoming : none
         *  Outgoing : none
         *
         */
        postCreate: function(){
        
            //We are setting the lower and upper timelimts to these dates
            //These can be programmed from the limits of all the projects present the project area
            //Currently we set it 2 years from now on either side
            var today = new Date();
            this.timelineLowerLimit = new Date(today.getFullYear() - 2, 0, 1);
            this.timelineUpperLimit = new Date(today.getFullYear() + 1, 11, 31);
            
            //By default launch the year view
            this.switchToYearView();
        },
        
        /**
         *  getDateToPixel : convert a date to a pixel
         *  Incoming : Date object
         *  Outgoing : returns the pixel position of the date //Integer
         *
         */
        getDateToPixel: function(/*Date*/gDate){
            var y = gDate.getFullYear();
            var m = gDate.getMonth() + 1; //month is a zero based index 0-11
            var d = gDate.getDate(); //Returns todays date 1-31 depending on the month
            //This is interesting !
            //When the gDate falls between the lower and upper limit of the current timeline
            //the function returns the appropriate pixel value else it returns 0 or maxwidth of timeline
            //This is required to give the clipped appearance of the gantt bar
            var date_location = 0;
            //We have adjusted the lower and upper bound incase the timeline is a weekly view
            //This is required since we need to show a complete week starting from say Sunday
            var lowerTime = this.timelineLowerLimit;
            var upperTime = this.timelineUpperLimit;
            if (this.timelineType == "Week") {
                lowerTime = this.timelineLowerLimit_Weekly;
                upperTime = this.timelineUpperLimit_Weekly;
            }
            
            if (gDate > lowerTime) {
                if (gDate < upperTime) {
                    //Date2 - Date1 = difference of days. Pixel of this date = difference*dayWidth + 1
                    date_location = dojo.date.difference(lowerTime, gDate, "day") * this.getDayWidth() + 1;
                }
                else {
                    date_location = dojo.style(this._timeline_Row, "width");
                }
            }
            else {
                date_location = 0;
            }
            return date_location;
        },
        
        /**
         *  getPixelToDate : maps the pixel to lower date using distance from the start date
         *  Incoming : Pixel //Integer
         *  Outgoing : Date object
         *
         */
        getPixelToDate: function(/*Integer*/dateInPixel){
            var lowerTime = this.timelineLowerLimit;
            var upperTime = this.timelineUpperLimit;
            if (this.timelineType == "Week") {
                lowerTime = this.timelineLowerLimit_Weekly;
                upperTime = this.timelineUpperLimit_Weekly;
            }
            
            var num_ofDays = Math.floor((dateInPixel - 1) / this.getDayWidth());
            var gDate = dojo.date.add(lowerTime, "day", num_ofDays);
            return gDate;
        },
        
        /**
         *  getPixelToDateHigher : maps the pixel to higher date using distance from the start date
         *  Incoming : Pixel //Integer
         *  Outgoing : Date object
         *
         */
        getPixelToDateHigher: function(/*Integer*/dateInPixel){
            var lowerTime = this.timelineLowerLimit;
            var upperTime = this.timelineUpperLimit;
            if (this.timelineType == "Week") {
                lowerTime = this.timelineLowerLimit_Weekly;
                upperTime = this.timelineUpperLimit_Weekly;
            }
            
            var num_ofDays = Math.ceil((dateInPixel - 1) / this.getDayWidth());
            var gDate = dojo.date.add(lowerTime, "day", num_ofDays);
            return gDate;
        },
        
        /**
         *  getToday : maps the todays date to pixel using distance from the start date
         *  Incoming : none
         *  Outgoing : returns the pixel position of todays date
         *
         */
        getToday: function(){
            var todayDate = new Date(); //This also returns the current time. Hence reconstructing another date with only year, month and day info
            return this.getDateToPixel(new Date(todayDate.getFullYear(), todayDate.getMonth(), todayDate.getDate()));
        },
        
        /**
         *  addTimelineBlockRight : Add a New Timeline Block at the Right side of GanttDragPane
         *  Incoming : none
         *  Outgoing : Boolean
         *
         */
        addTimelineBlockRight: function(){
            //when we add one timeline block, we need to remove the first timeline block
            this.removeFirstTimeLineBlock();
            
            switch (this.timelineType) {
                case "Year":
                    this._addYearBlock(true);
                    break;
                case "Quarter":
                    this._addQuarterBlock(true);
                    break;
                case "Month":
                    this._addMonthBlock(true);
                    break;
                case "Week":
                    this._addWeekBlock(true);
                    break;
            }
            return true;
        },
        
        /**
         *  addTimelineBlockLeft : Add a New Timeline Block at the Left side of GanttDragPane
         *  Incoming : none
         *  Outgoing : Boolean
         *
         */
        addTimelineBlockLeft: function(){
            //when we add one timeline block, we need to remove the last timeline block
            this.removeLastTimeLineBlock();
            switch (this.timelineType) {
                case "Year":
                    this._addYearBlock(false);
                    break;
                case "Quarter":
                    this._addQuarterBlock(false);
                    break;
                case "Month":
                    this._addMonthBlock(false);
                    break;
                case "Week":
                    this._addWeekBlock(false);
                    break;
            }
            return true;
        },
        
        /**
         *  getDayWidth : This function returns the width of the day specified for each timeline view
         *  Incoming : none
         *  Outgoing : day width corresponding to the current time block (Integer)
         *
         */
        getDayWidth: function(){
            //Each day can have different pixel representation according to the timeline
            var dayWidth = 2; //Give a default value of 2 pixels
            switch (this.timelineType) {
                case "Year":
                    dayWidth = 2;
                    break;
                case "Quarter":
                    dayWidth = 4;
                    break;
                case "Month":
                    dayWidth = 16;
                    break;
                case "Week":
                    dayWidth = 24;
                    break;
            }
            return dayWidth;
        },
        
        /**
         *  switchTimeline : This function changes the timeline rows from year to quarter to month to week
         *  			     There are two rows and each gets changed to show a zoom up or zoom down user experience
         *  Incoming : none
         *  Outgoing : Boolean
         *
         */
        switchTimeline: function(/*String*/newType, /*String*/ centerViewDayIndex){
            if (this.timelineType == newType) {
                return false;
            }
            
            //this.resetLowerAndUpperDates(centerViewDayIndex, newType);
            
            var done = false;
            this.timelineType = newType;
            switch (this.timelineType) {
                case "Year":
                    done = this.switchToYearView();
                    break;
                case "Quarter":
                    done = this.switchToQuarterView();
                    break;
                case "Month":
                    done = this.switchToMonthView();
                    break;
                case "Week":
                    done = this.switchToWeekView();
                    break;
            }
            return done;
        },
        
        //this function switches the timeline to year view.
        /**
         *  switchToYearView : this function switches the timeline to year view
         *  Incoming : none
         *  Outgoing : Boolean
         *
         */
        switchToYearView: function(){
            var todayDate = new Date();
            this.timelineLowerLimit = new Date(todayDate.getFullYear() - 2, 0, 1);
            this.timelineUpperLimit = new Date(todayDate.getFullYear() + 1, 11, 31);
            //The major view is year and the minor view is month
            
            this.removeLastTimeLine();
            
            var num_Years = this.timelineUpperLimit.getFullYear() - this.timelineLowerLimit.getFullYear() + 1;
            var blockWidth = 0;
            for (var yr = 0; yr < num_Years; yr++) {
                var startDate = dojo.date.add(this.timelineLowerLimit, "year", yr);
                var yearBlock = new com.ibm.epm.work.web.ui.internal.GanttTimeBlock({
                    blockStartDate: startDate,
                    dayWidth: this.getDayWidth()
                });
                yearBlock.createYearBlock();
                blockWidth += yearBlock.getBlockWidth();
                this._timeline_Row.appendChild(yearBlock.domNode);
            }
            dojo.style(this._timeline_Row, 'width', blockWidth + 'px');
            
            return true;
        },
        
        /**
         *  switchToQuarterView : this function switches the timeline to quarter view
         *  Incoming : none
         *  Outgoing : Boolean
         *
         */
        switchToQuarterView: function(){
            var todayDate = new Date();
            this.timelineLowerLimit = new Date(todayDate.getFullYear() - 1, 0, 1);
            this.timelineUpperLimit = new Date(todayDate.getFullYear() + 1, 11, 31);
            
            this.removeLastTimeLine();
            //The major view is quarter and the minor view is month
            
            var num_Years = this.timelineUpperLimit.getFullYear() - this.timelineLowerLimit.getFullYear() + 1;
            var num_Quarters = dojo.date.difference(this.timelineLowerLimit, this.timelineUpperLimit, "quarter") + 1;
            
            var blockWidth = 0;
            for (var quarter = 0; quarter < num_Quarters; quarter++) {
                var startDate = dojo.date.add(this.timelineLowerLimit, "month", quarter * 3); //advance by 3 months to get the start date of quarters
                var quarterBlock = new com.ibm.epm.work.web.ui.internal.GanttTimeBlock({
                    blockStartDate: startDate,
                    dayWidth: this.getDayWidth()
                });
                quarterBlock.createQuarterBlock((quarter % 4));
                blockWidth += quarterBlock.getBlockWidth();
                this._timeline_Row.appendChild(quarterBlock.domNode);
            }
            dojo.style(this._timeline_Row, 'width', blockWidth + 'px');
            
            return true;
        },
        
        /**
         *  switchToMonthView : this function switches the timeline to month view
         *  Incoming : none
         *  Outgoing : Boolean
         *
         */
        switchToMonthView: function(){
            var todayDate = new Date();
            this.timelineLowerLimit = new Date(todayDate.getFullYear(), 0, 1);
            this.timelineUpperLimit = new Date(todayDate.getFullYear(), 11, 31);
            
            this.removeLastTimeLine();
            //The major view is month and the minor view is days
            var num_Months = dojo.date.difference(this.timelineLowerLimit, this.timelineUpperLimit, "month") + 1;
            
            var blockWidth = 0;
            for (var month = 0; month < num_Months; month++) {
                var startDate = dojo.date.add(this.timelineLowerLimit, "month", month); //advance by 1 month to get the start date of month
                var monthBlock = new com.ibm.epm.work.web.ui.internal.GanttTimeBlock({
                    blockStartDate: startDate,
                    dayWidth: this.getDayWidth()
                });
                monthBlock.createMonthBlock((month % 12));
                blockWidth += monthBlock.getBlockWidth();
                this._timeline_Row.appendChild(monthBlock.domNode);
            }
            dojo.style(this._timeline_Row, 'width', blockWidth + 'px');
            return true;
        },
        
        /**
         *  switchToWeekView : this function switches the timeline to week view
         *  Incoming : none
         *  Outgoing : Boolean
         *
         */
        switchToWeekView: function(){
            var todayDate = new Date();
            this.timelineLowerLimit = new Date(todayDate.getFullYear(), 0, 1);
            this.timelineUpperLimit = new Date(todayDate.getFullYear(), 11, 31);
            
            this.removeLastTimeLine();
            //The major view is week and the minor view is days
            //Date.getDay() - Day of the week(0-6). 0 = Sunday, ... , 6 = Saturday
            //the lower limit might not start from monday, so decrement the date so that the first week starts from a monday
            var dayIndex = this.timelineLowerLimit.getDay();
            this.timelineLowerLimit_Weekly = new Date(this.timelineLowerLimit.getTime() - (dayIndex-1) * 24 * 60 * 60 * 1000);
            //the upper limit might not be a sunday, so decrement the date so that the week ends on a sunday
            dayIndex = this.timelineUpperLimit.getDay();
            this.timelineUpperLimit_Weekly = new Date(this.timelineUpperLimit.getTime() - dayIndex * 24 * 60 * 60 * 1000);
            
            var num_Weeks = dojo.date.difference(this.timelineLowerLimit_Weekly, this.timelineUpperLimit_Weekly, "week") + 1;
            
            var blockWidth = 0;
            for (var week = 0; week < num_Weeks; week++) {
                var startDate = dojo.date.add(this.timelineLowerLimit_Weekly, "week", week); //advance by 1 week to get the start date of week
                var weekBlock = new com.ibm.epm.work.web.ui.internal.GanttTimeBlock({
                    blockStartDate: startDate,
                    dayWidth: this.getDayWidth()
                });
                weekBlock.createWeekBlock();
                blockWidth += weekBlock.getBlockWidth();
                this._timeline_Row.appendChild(weekBlock.domNode);
            }
            dojo.style(this._timeline_Row, 'width', blockWidth + 'px');
            return true;
        },
        
        /**
         *  removeLastTimeLine : this function removes the timeline rows
         *  					 this is called before switching to any other timeline
         *  Incoming : none
         *  Outgoing : none
         *
         */
        removeLastTimeLine: function(){
            while (this._timeline_Row.firstChild) {
                var timeBlock = this._timeline_Row.lastChild;
                this._timeline_Row.removeChild(timeBlock);
                var wid = dijit.byId(timeBlock.id);
                wid.destroy();
            }
        },
        
        /**
         *  removeFirstTimeLineBlock : this function removes the first timeblock. This is required when we are a block in the right. using this
         *  							we remove one block from the left
         *  Incoming : none
         *  Outgoing : none
         *
         */
        removeFirstTimeLineBlock: function(){
            var timeBlock = this._timeline_Row.firstChild;
            if (timeBlock) {
                this._timeline_Row.removeChild(timeBlock);
                var wid = dijit.byId(timeBlock.id);
                wid.destroy();
            }
            else {
                console.log("No time block found and hence not deleting the first block");
            }
        },
        
        /**
         *  removeLastTimeLineBlock : this function removes the last timeblock. This is required when we are a block in the left. using this
         *  							we remove one block from the right
         *  Incoming : none
         *  Outgoing : none
         *
         */
        removeLastTimeLineBlock: function(){
            var timeBlock = this._timeline_Row.lastChild;
            if (timeBlock) {
                this._timeline_Row.removeChild(timeBlock);
                var wid = dijit.byId(timeBlock.id);
                wid.destroy();
            }
            else {
                console.log("No time block found and hence not deleting the first block");
            }
        },
        
        /**
         *  getTotalTimeLineWidth : this function returns the total width of the timeline.
         *  Incoming : none
         *  Outgoing : total timeline width (Integer)
         *
         */
        getTotalTimeLineWidth: function(){
            var totalWidth = 0;
            for (var i = 0; i < this._timeline_Row.childNodes.length; i++) {
                var timeBlock = this._timeline_Row.childNodes[0];
                var wid = dijit.byId(timeBlock.id);
                totalWidth += wid.getBlockWidth();
            }
            return totalWidth;
        },
        
        /**
         *  getTimeLineBlockWidth : this function returns the width of the timeblock. Since the width of days is different in each timescale these block widths varies
         *  Incoming : none
         *  Outgoing : total timeline width (Integer)
         *
         */
        getTimeLineBlockWidth: function(){
            var blockWidth = 0;
            var timeBlock = this._timeline_Row.firstChild;
            if (timeBlock) {
                var wid = dijit.byId(timeBlock.id);
                blockWidth = wid.getBlockWidth();
            }
            return blockWidth;
        },
        
        /**
         *  transitionToToday : if todays date doesnot fall in the timeline range then first a switch is done taking todays date as fulcrum
         *  Incoming : none
         *  Outgoing : Boolean
         *
         */
        transitionToToday: function(){
            var todayDate = new Date();
            if (todayDate > this.timelineLowerLimit && todayDate < this.timelineUpperLimit) {
                //do nothing, we just need to scroll, hence returning
                return false;
            }
            else {
                var done = false;
                this.timelineLowerLimit = new Date(todayDate.getFullYear() - 2, 0, 1);
                this.timelineUpperLimit = new Date(todayDate.getFullYear() + 1, 11, 31);
                switch (this.timelineType) {
                    case "Year":
                        done = this.switchToYearView();
                        break;
                    case "Quarter":
                        done = this.switchToQuarterView();
                        break;
                    case "Month":
                        done = this.switchToMonthView();
                        break;
                    case "Week":
                        done = this.switchToWeekView();
                        break;
                }
                return done;
            }
        },
        
        /**
         *  _addYearBlock : add a year block to the timeline
         *  Incoming : Boolean ( True to add to right, False to add to left )
         *  Outgoing : none
         *
         */
        _addYearBlock: function(toRight/*Boolean*/){
            var totalTimeLineWidth = this.getTotalTimeLineWidth();
            var startDate;
            if (toRight) {
                startDate = dojo.date.add(this.timelineUpperLimit, "day", 1); //Set the start date of the new block to upper limit + 1
            }
            else {
                startDate = dojo.date.add(this.timelineLowerLimit, "year", -1); //Set the start date of the new block to lower limit - 1
            }
            var yearBlock = new com.ibm.epm.work.web.ui.internal.GanttTimeBlock({
                blockStartDate: startDate,
                dayWidth: this.getDayWidth()
            });
            yearBlock.createYearBlock();
            var blockWidth = yearBlock.getBlockWidth();
            dojo.style(this._timeline_Row, 'width', (totalTimeLineWidth + blockWidth) + 'px');
            
            if (toRight) {
                this._timeline_Row.appendChild(yearBlock.domNode); //Append the block to end of the timeline
            }
            else {
                this._timeline_Row.insertBefore(yearBlock.domNode, this._timeline_Row.firstChild); //Insert the timeblock before the first timeline
            }
            
            var incr = (toRight) ? 1 : -1;
            this.timelineUpperLimit = dojo.date.add(this.timelineUpperLimit, "year", incr); //Increase/decrease upper limit by one year
            this.timelineLowerLimit = dojo.date.add(this.timelineLowerLimit, "year", incr); //Increase/decrease lower limit by one year
        },
        
        /**
         *  _addQuarterBlock : add a quarter block to the timeline
         *  Incoming : Boolean ( True to add to right, False to add to left )
         *  Outgoing : none
         *
         */
        _addQuarterBlock: function(toRight/*Boolean*/){
            var totalTimeLineWidth = this.getTotalTimeLineWidth();
            var startDate;
            if (toRight) {
                startDate = dojo.date.add(this.timelineUpperLimit, "day", 1); //Set the start date of the new block to upper limit + 1
            }
            else {
                startDate = dojo.date.add(this.timelineLowerLimit, "quarter", -1); //Set the start date of the new block to lower limit - 1
            }
            var quarterBlock = new com.ibm.epm.work.web.ui.internal.GanttTimeBlock({
                blockStartDate: startDate,
                dayWidth: this.getDayWidth()
            });
            quarterBlock.createQuarterBlock(this._getQuarterIndex(startDate));
            var blockWidth = quarterBlock.getBlockWidth();
            dojo.style(this._timeline_Row, 'width', (totalTimeLineWidth + blockWidth) + 'px');
            
            if (toRight) {
                this._timeline_Row.appendChild(quarterBlock.domNode); //Append the block to end of the timeline
            }
            else {
                this._timeline_Row.insertBefore(quarterBlock.domNode, this._timeline_Row.firstChild); //Insert the timeblock before the first timeline
            }
            
            var incr = (toRight) ? 1 : -1;
            this.timelineUpperLimit = dojo.date.add(this.timelineUpperLimit, "quarter", incr); //Increase/decrease upper limit by one quarter
            this.timelineLowerLimit = dojo.date.add(this.timelineLowerLimit, "quarter", incr); //Increase/decrease lower limit by one quarter
            //A VERY PERCULIAR DEFECT IN DOJO.DATE.ADD
            //when upperLimit date is 30 Sept and 1 quarter is added, it results in 30th December and not 31st December //day light saving???
            //similarly when a quarter is reduced from Mar31, it results in 30th December
            //Currently setting it to 31st December if this is encountered
            //We need to raise a defect with dojo toolkit on this
            if ((this.timelineUpperLimit.getMonth() == 11) && (this.timelineUpperLimit.getDate() == 30)) {
                this.timelineUpperLimit = new Date(this.timelineUpperLimit.getFullYear(), 11, 31);
            }
        },
        
        /**
         *  _addMonthBlock : add a month block to the timeline
         *  Incoming : Boolean ( True to add to right, False to add to left )
         *  Outgoing : none
         *
         */
        _addMonthBlock: function(toRight/*Boolean*/){
            var totalTimeLineWidth = this.getTotalTimeLineWidth();
            var startDate;
            if (toRight) {
                startDate = dojo.date.add(this.timelineUpperLimit, "day", 1); //Set the start date of the new block to upper limit + 1
            }
            else {
                startDate = dojo.date.add(this.timelineLowerLimit, "month", -1); //Set the start date of the new block to lower limit - 1
            }
            var monthBlock = new com.ibm.epm.work.web.ui.internal.GanttTimeBlock({
                blockStartDate: startDate,
                dayWidth: this.getDayWidth()
            });
            monthBlock.createMonthBlock(startDate.getMonth());
            var blockWidth = monthBlock.getBlockWidth();
            dojo.style(this._timeline_Row, 'width', (totalTimeLineWidth + blockWidth) + 'px');
            
            if (toRight) {
                this._timeline_Row.appendChild(monthBlock.domNode); //Append the block to end of the timeline
            }
            else {
                this._timeline_Row.insertBefore(monthBlock.domNode, this._timeline_Row.firstChild); //Insert the timeblock before the first timeline
            }
            
            var incr = (toRight) ? 1 : -1;
            this.timelineUpperLimit = dojo.date.add(this.timelineUpperLimit, "month", incr); //Increase/decrease upper limit by one quarter
            this.timelineLowerLimit = dojo.date.add(this.timelineLowerLimit, "month", incr); //Increase/decrease lower limit by one quarter
            //A VERY PERCULIAR DEFECT IN DOJO.DATE.ADD
            //when upperLimit date is 30 Sept and 1 quarter is added, it results in 30th December and not 31st December //day light saving???
            //similarly when a quarter is reduced from Mar31, it results in 30th December
            //Currently setting it to 31st December if this is encountered
            //We need to raise a defect with dojo toolkit on this
            if ((this.timelineUpperLimit.getMonth() == 11) && (this.timelineUpperLimit.getDate() == 30)) {
                this.timelineUpperLimit = new Date(this.timelineUpperLimit.getFullYear(), 11, 31);
            }
        },
        
        /**
         *  _addWeekBlock : add a week block to the timeline
         *  Incoming : Boolean ( True to add to right, False to add to left )
         *  Outgoing : none
         *
         */
        _addWeekBlock: function(toRight/*Boolean*/){
            var totalTimeLineWidth = this.getTotalTimeLineWidth();
            var startDate;
            if (toRight) {
                startDate = dojo.date.add(this.timelineUpperLimit_Weekly, "day", 1); //Set the start date of the new block to upper limit + 1
            }
            else {
                startDate = dojo.date.add(this.timelineLowerLimit_Weekly, "week", -1); //Set the start date of the new block to lower limit - 1
            }
            var weekBlock = new com.ibm.epm.work.web.ui.internal.GanttTimeBlock({
                blockStartDate: startDate,
                dayWidth: this.getDayWidth()
            });
            weekBlock.createWeekBlock();
            var blockWidth = weekBlock.getBlockWidth();
            dojo.style(this._timeline_Row, 'width', (totalTimeLineWidth + blockWidth) + 'px');
            
            if (toRight) {
                this._timeline_Row.appendChild(weekBlock.domNode); //Append the block to end of the timeline
            }
            else {
                this._timeline_Row.insertBefore(weekBlock.domNode, this._timeline_Row.firstChild); //Insert the timeblock before the first timeline
            }
            
            var incr = (toRight) ? 1 : -1;
            this.timelineUpperLimit_Weekly = dojo.date.add(this.timelineUpperLimit_Weekly, "week", incr); //Increase/decrease upper limit by one quarter
            this.timelineLowerLimit_Weekly = dojo.date.add(this.timelineLowerLimit_Weekly, "week", incr); //Increase/decrease lower limit by one quarter
        },
        
        /**
         *  _getQuarterIndex : return the index of the quarter based on the date
         *  Incoming : Date object
         *  Outgoing : Quarter index ( 0..3 )
         *
         */
        _getQuarterIndex: function(/*Date*/aDate){
            var yearStartDate = new Date(aDate.getFullYear(), 0, 1); //Set it as the start date of the year
            var quarterIndex = dojo.date.difference(yearStartDate, aDate, "quarter");
            return quarterIndex;
        },
        
        /**
         *  resetLowerAndUpperDates : Reset the lower and upper dates from the pivotDate, input parameter is in pixels
         *  Incoming : view center pixel (Integer), timeline type
         *  Outgoing : none
         *
         */
        resetLowerAndUpperDates: function(centerViewDayIndex, newTimelineType){
            var num_daysFromStart = centerViewDayIndex;
            var lowerDate;
            var upperDate;
            if (this.timelineType == "Week") {
                lowerDate = this.timelineLowerLimit_Weekly;
                upperDate = this.timelineUpperLimit_Weekly;
            }
            else {
                lowerDate = this.timelineLowerLimit;
                upperDate = this.timelineUpperLimit;
            }
            
            var pivotDate = dojo.date.add(lowerDate, "days", num_daysFromStart);
            
            if (newTimelineType == "Year") {
                this.timelineLowerLimit = new Date(pivotDate.getFullYear() - 2, 0, 1);
                this.timelineUpperLimit = new Date(pivotDate.getFullYear() + 1, 11, 31);
            }
            if (newTimelineType == "Quarter") {
                //4 quarters on either side
                lowerDate = dojo.date.add(pivotDate, "quarter", -4);
                upperDate = dojo.date.add(pivotDate, "quarter", 4);
                var maxDaysInMonth = dojo.date.getDaysInMonth(upperDate);
                this.timelineLowerLimit = new Date(lowerDate.getFullYear(), lowerDate.getMonth(), 1);
                this.timelineUpperLimit = new Date(upperDate.getFullYear(), upperDate, maxDaysInMonth);
            }
            if (newTimelineType == "Month") {
                //12 months on either side
                lowerDate = dojo.date.add(pivotDate, "month", -12);
                upperDate = dojo.date.add(pivotDate, "month", 12);
                var maxDaysInMonth1 = dojo.date.getDaysInMonth(upperDate);
                this.timelineLowerLimit = new Date(lowerDate.getFullYear(), lowerDate.getMonth(), 1);
                this.timelineUpperLimit = new Date(upperDate.getFullYear(), upperDate, maxDaysInMonth1);
            }
            if (newTimelineType == "Week") {
                //24 weeks on either side
                lowerDate = dojo.date.add(pivotDate, "week", -24);
                upperDate = dojo.date.add(pivotDate, "week", 24);
                
                var dayIndex = lowerDate.getDay();
                lowerDate = new Date(lowerDate.getTime() - dayIndex * 24 * 60 * 60 * 1000);
                dayIndex = upperDate.getDay();
                upperDate = new Date(upperDate.getTime() - dayIndex * 24 * 60 * 60 * 1000);
                
                this.timelineLowerLimit = lowerDate;
                this.timelineUpperLimit = upperDate;
            }
        }
    });
    
})();
