/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.GanttView");
    
	dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
    
    // dojo library
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dijit.form.Button");
    dojo.require("dijit.Menu");
    dojo.require("dojo.date");
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    
	dojo.require("com.ibm.epm.work.web.ui.internal.GanttDragPane");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttBar");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttTimeLine");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttRowBackground");
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttDependency");
    dojo.require("com.ibm.epm.work.web.ui.internal.GanttDependencyHandle");
    
    dojo.require("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGanttGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants");
	
	dojo.require("com.ibm.epm.work.web.ui.internal.BaselineSave");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
	
	// Needed for dojo.keys
	dojo.require("dojo._base.event");
	
    // EPM Core Constants
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
	// Web core constant for the tree id 
	dojo.require("com.ibm.epm.web.core.util.internal.Constants");
    
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
	dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
	dojo.require("com.ibm.epm.work.web.ui.internal.util.WBSUtil");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
	
	//jazz ui dialog
	dojo.require("jazz.ui.Dialog");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
    // Localization
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
	
    //The Gantt view shows the toolbar with various actions
    //It creates a drag pane and all elements are created on top of it
    //Currently we have a table to represent the timeline
    //We also create a lot of floating divs to give the alternate row effect as well as column lines
    
    var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    var controllerFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
    var controllerID = com.ibm.epm.work.web.ui.internal.grid.WBSControllerConstants.ScheduleGridController;
    
    var WorkViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    var WBSGridGanttPubSub = com.ibm.epm.work.web.ui.internal.WBSGridGanttPubSub;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
	var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
	var Constants = com.ibm.epm.web.core.util.internal.Constants; // For Tree id 
	var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
    var TODAYLINE_WIDTH = 2;
	var EPMConfirmDialog = com.ibm.epm.web.core.ui.internal.EPMConfirmDialog;
	var WBSUtil = com.ibm.epm.work.web.ui.internal.util.WBSUtil;
    var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
	
    dojo.declare("com.ibm.epm.work.web.ui.internal.GanttView", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/templates/GanttView.html"),
        dependencyDiv: null,
        
        /**
         *  postCreate : create all the skeleton for gantt view
         *  Incoming : none
         *  Outgoing : none
         *
         */
        postCreate: function(){
            this._buildGanttMainContent();
            
            //this._destroyWidgets();				//This is to called to clean up widgets created
            //1. Create viewline drop down
            this._createViewLineDropDown(); //This is the dropdown in the toolbar to enabled switching different timelines
            //2. Create the toolbar
            this._createToolbar();
        },
        
        _subscribeToEvents: function(){
            this._subscriptions = [
			dojo.subscribe(EPMTreePubSub.ON_SELECT, this, "onRowSelect"), //This is called when row is selected
 			dojo.subscribe(WBSGridGanttPubSub.ON_CREATE_DEPENDS, this, "onAfterUpdateDependency"), //This is called when Grid Row is updated and data is saved successfully to the server.
 			dojo.subscribe(WBSGridGanttPubSub.ON_AFTER_UPDATE_GANTTBAR, this, "onAfterUpdateGanttBar"), //This is called after the gantt bar has been moved or resized. This will enable redraw of dependencies
            // EPMGrid row select event
			dojo.subscribe(EPMGridPubSub.ON_ROW_CLICK, this, "onRowSelect"),            
			// Gantt bar row select event
			dojo.subscribe(WBSGridGanttPubSub.ON_GANTT_ROW_SELECT, this, "onRowSelect"),
			dojo.subscribe(GridEvents.LOADING_COMPLETED, this, "validateAndResizeDragPane")];
			
			this.keyBoardEvent = dojo.connect(document, "onkeypress", this, "_onKeyDownFromGantt");
        },
        
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions != null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._subscriptions = [];
			
			if (this.keyBoardEvent) {
                dojo.disconnect(this.keyBoardEvent);
            }
		},
        
        /**
         *  _createController : create the controller for the gantt view
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createController: function(){
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGanttGridModel();
            }
            if (!this._controller) {
                this._controller = controllerFactory.getGanttController(controllerID);
            }
            if (this._controller.isInitialized() == false) {
                this._controller.initializeController(this._treeID, this, this._gridModel, [WBSConstants.PROPOSED_SCHEDULE, WBSConstants.CRITICAL_PATH]);
                this._controller.buildGridModel();
            }
            else {
                this._controller.addViewer(this);
                this.createView(this._controller.getDataModel());
            }
        },
        
        _buildGanttMainContent: function(){
            this.ganttBarIdMap = []; //Associated array of identifier with widget id
            this._visibleGanttBarArray = []; // Indexed Array of visible gantt widget ids
            this.allGanttBarArray = []; //All gantt widgets
            this.ganttDependencies = [];
            this.rowIdMap = []; // Associated array of element ids with the widget
            this._allCriticalPathArray = []; // Indexed Array of critical path gantt widget ids
            this._createBackground(); //This creates the alternate row appearance
            //3. Create the draggable container
            this._createDragPane(); //This creates the drag pane over which we have the timeline and gantt bars
            //Create the gantt controller
            this._createController();
            
            this._subscribeToEvents();
        },
        
        
        /**
         * Checks weather controller is present.
         * Then calls the persistGrid of the controller.
         * The persistGrid call is sequential.
         */
        persistGrid: function(){
            if (this._controller.persistGrid) {
                this._controller.persistGrid(true);
            }
        },
        
        destroyMainContent: function(){
            if (this._controller) {
                this._controller.destroy();
                delete this._controller;
            }
            if (this._ganttDragPane) {
                this._ganttDragPane.destroy();
                delete this._ganttDragPane;
            }
            if (this.ganttRowBackground) {
                var totalRows = this.ganttRowBackground.totalRows();
                this.ganttRowBackground.removeRows(totalRows);
                delete this.ganttRowBackground;
            }
            this._unsubscribeToAllEvents();
        },
        
        /**
         *  resetLowerAndUpperDates : Basic call after the grid is created
         *  Incoming : gridDate array consisting of row informatin
         *  Outgoing : none
         *
         */
        createView: function(gridData){
            //get the basic rows from the gridData
            //loop thru them to create the initial view            
            if (gridData != null) {
                var rowCount = gridData.getRowCount();
                for (var i = 0; i < rowCount; i++) {
                    if (gridData.getRow) {
                        var row = gridData.getRow(i);
                        if (row) {
                            this.buildGanttBar(row, i);
                        }
                    }
                }
                //The view construction is complete. Now we need to set to the center of the view.
                this._gotoToday(); //This arranges the view to be center aligned with todays date
            }
        },
        
        /**
         *  rowUpdated : called when the grid's row has got updated
         *  Incoming : row object, row identifier, row Index
         *  Outgoing : none
         *
         */
        rowUpdated: function(row, rowID, rowIndex){
            //gantt row to be updated here
            if (row && row.id) {
                var ganttBar = this.getGanttWidgetFromIdentifier(rowID);
                if (!ganttBar) {
                    console.log("GANTT ROW NOT FOUND WITH ID = " + rowID);
                    this.buildGanttBar(row, rowIndex);
                }
                this.updateGanttBar(row);
            }
        },
        
        /**
         *  newRowInserted : Called after expand of tree which results in addition of nodes
         *  Incoming : row object, row identifier, row Index
         *  Outgoing : none
         *
         */
        newRowInserted: function(row, rowID, rowIndex){
            // new gantt row is created here
            if (row) {
                this.buildGanttBar(row, rowIndex);
                this.updateGanttBar(row);
                
                //Select the newly created row.
                if (rowID.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1) {
                    this._selectRowById(rowID);
                }
            }
        },
        
        //
        /**
         *  removeRows : Called after collapse in the tree which results in removal of nodes
         *  Incoming : Array of rows
         *  Outgoing : none
         *
         */
        removeRows: function(removedRows){
            //Added check to skip removal of loading row
            if(removedRows.length == 1 && !removedRows[0]) return;       
            //TODO if something needs to be done on collapse node it wil be done here     
            this.hideOrRemoveRow(removedRows, true); //True to hide the row
            this._redrawDependencies();
        },
        
        _createMenuItem: function(itemLabel, itemIconClass, timescaleID){
            var menuItem = new dijit.MenuItem({
                label: itemLabel,
                iconClass: itemIconClass,
                onClick: dojo.hitch(this, this._onTimelineViewChange, timescaleID)
            });
            menuItem.domNode.setAttribute("name", timescaleID);
            return menuItem;
        },
        
        /**
         *  _calculateCriticalPath : Check if the selected project is in the gantt array and invoke the critical path calculation for the selected project
         *  
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _calculateCriticalPath: function() {
            for (var i = 0; i < this.allGanttBarArray.length; i++) {
                var rowIndex = this.rowIdMap[this.allGanttBarArray[i]];
				if (this.selectedProject[0] == rowIndex) {
					// Reset the critical path array
					this._allCriticalPathArray = [];
					
		            // Calculate the critical path for the selected project
		            // Makes a get all call project/ID/criticalPath
					this._controller.calculateCriticalPathForProject(this._selectedNode, rowIndex);
				}
            }
        },
        
        /**
         *  _handleCriticalPathButton : onClick handler for CriticalPath button, sets the
         *  												display properties of the task in critical path
         *  Incoming : event
         *  Outgoing : none
         *
         */
        _handleCriticalPathButton: function(event){
            var checked = false;
            if (this.criticalPathButton) {
                checked = this.criticalPathButton.checked;
            }
			this.criticalPathButton.attr('checked', checked);
			// As schedule will be refreshed when the critical path is checked, we need to check if there are unsaved changes.
			//If there are unsaved changes, show a confirmation message to user. 
            if (this.criticalPathButton.checked === true) {
				/*
				 * Validate if the selected node is project else throw an Error msg & return
				 */
			    var isProjectSelected =  this._validateForProject(WorkViewletMessages.SELECT_PROJECT_FOR_CRITICAL_PATH);				
				if (isProjectSelected == false) {
					// revert the checked button state
					this.criticalPathButton.attr('checked', false);
					this._clearErrorMsg = true;
					return;
				} else {
					// clear the previous error msg if any
					this._controller._clearErrorMessage();
				}

				if (this.saveButton.disabled == false) {
					this.confirmDiscard(true);
				}
				else {
					//Calculate critical path.
					this.calculateAndDisplayCriticalPath();
				}
				this.criticalPathButton.attr('title', WorkViewletMessages.HIDE_CRITICAL_PATH);
			}
			else {
				this._displayCriticalPathForVisibleGanttBarArray();
				this.criticalPathButton.attr('title', WorkViewletMessages.SHOW_CRITICAL_PATH);
			}            
        },
        
		/**
		 * Method to show a confirmation dialog to user before discarding the changes.
		 * true - for calculate critical path operation
		 * false - for resolve dependency operation
		 */
		confirmDiscard: function(isCalculateCP){

            var buttonlist = [{
                label: WorkViewletMessages.OK,
                action: dojo.hitch(this, this._discardChanges, isCalculateCP)
            }, {
                label: WorkViewletMessages.CANCEL,
                action: dojo.hitch(this, this._cancelOperation, isCalculateCP)
            }];
            
			var labelList;
			
			if (isCalculateCP) {
				labelList = [{
					label: WorkViewletMessages.CONFIRM_SCHEDULE_UNSAVED_CHANGES_MESSAGE_CP
				}];
			} else {
				// resolve dependency operation
				labelList = [{
					label: WorkViewletMessages.CONFIRM_SCHEDULE_UNSAVED_CHANGES_MESSAGE_RD
				}];			
			} 

			var length = labelList.length;
			// Add a empty line
			labelList[length] = {
				label: "\n"
			};
			labelList[length + 1] = {
				label:  WorkViewletMessages.CONFIRM_SCHEDULE_UNSAVED_CHANGES_MESSAGE
        	};

	        this.discardUnsavedDialog = new EPMConfirmDialog({
	            labelList: labelList,
	            title: WorkViewletMessages.DISCARD_CHANGES_TITLE,
	            buttonlist: buttonlist
	        });
	        this.closeButtonHandler = dojo.connect(this.discardUnsavedDialog.dialog._close, "onclick", dojo.hitch(this, this._closedButtonNodeHandler, isCalculateCP));
		},
		
		/**
		 * Method to discard the unsaved changes and proceed with the operation like calculate the critical path, resolve
		 * dependencies. This function is called from critical path calculation and resolve dependency, the value of the param 'isCalculateCP'
		 * differentiates the behaviour between the two.
		 * isCalculateCP - true/false
		 * true - for calculate critical path operation
		 * false - for resolve dependency operation
		 */
		_discardChanges: function(isCalculateCP){
			
			//Clean up the dialog related unused stuff.
			if(this.closeButtonHandler){
				dojo.disconnect(this.closeButtonHandler);
				this.closeButtonHandler = null;
			}
			this.discardUnsavedDialog.destroy();

			// discard unsaved nodes
			dojo.publish(WBSGridGanttPubSub.DISCARD_UNSAVED_NODES);
			
			if (isCalculateCP) {
				this.calculateAndDisplayCriticalPath();
			} else {
				this._okToResolveDependency();
			}
		},

		/**
		 * Method to calculate the critical path and display the critical tasks.
		 */		
		calculateAndDisplayCriticalPath: function(){
			//calculate the critical path.
			this._calculateCriticalPath();
		},
		
		/**
		 * Method called when user presses cancel on the dialog.
		 * Destroy the dialog and reset the critical path button.
		 */
		_cancelOperation: function(isCalculateCP){
			if(this.closeButtonHandler){
				//Disconnect the close button handler.
				dojo.disconnect(this.closeButtonHandler);
				this.closeButtonHandler = null;
			}
			this.discardUnsavedDialog.destroy();
			
			if (isCalculateCP) {
				this.criticalPathButton.attr('checked', false);
			}
		},
		
		/**
		 * By default the close button only closes the dialog, we need to reset the critical path icon state as well. 
		 */
		_closedButtonNodeHandler: function(isCalculateCP){
			this.discardUnsavedDialog.destroy();
			
			if (isCalculateCP) {
				this.criticalPathButton.attr('checked', false);
			}

			//Disconnect the close button handler.
			dojo.disconnect(this.closeButtonHandler);
			this.closeButtonHandler = null;
		},
		
        /**
         *  _displayCriticalPathForVisibleGanttBarArray : Iterates through the list of gantt bars in the critical path and sets the display property for the visible ones
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _displayCriticalPathForVisibleGanttBarArray: function(){
            for (var i = 0; i < this._allCriticalPathArray.length; i++) {
                var rowIndex = this.rowIdMap[this._allCriticalPathArray[i]];
                this._displayCriticalPathForBar(rowIndex);
            }
        },
        
        /**
         *  _displayCriticalPathForBar : Set the display properties of the task with ID rowIndex in critical path if Critical Path is checked
         *
         *  Incoming : rowID
         *  Outgoing : none
         *
         */
        _displayCriticalPathForBar: function(rowIndex){
            var ganttBar = this.getGanttWidgetFromIdentifier(rowIndex);
            if (ganttBar != null) {
                this.toggleCriticalPath(ganttBar.ganttBarInfo);
                if (this.criticalPathButton.checked == true) {
                    ganttBar.showCritPathBar();
                }
                else {
                    ganttBar.hideCritPathBar();
                }
            }
        },
		
        /**
         *  _createViewLineDropDown : timeline menu and menut items are created here
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createViewLineDropDown: function(){
            this.leftToolbar = new dijit.Toolbar({
                layoutAlign: "top",
                style: "background : transparent; border: none"
            });
            this.timeScaleDivLabel = new dijit.form.Button({
                label: WorkViewletMessages.GANTT_TIMESCALE,
                disabled: true
            });
            this.leftToolbar.addChild(this.timeScaleDivLabel);
            
            this.timescale_menu = new dijit.Menu({
                style: "display: none;"
            });
            
            var menuItem1 = this._createMenuItem(WorkViewletMessages.GANTT_TIMESCALE_YEAR, "com-ibm-epm-work-web-ui-internal-GanttView-checkedButtonIcon", WBSConstants.GANTT_YEAR);
            this.timescale_menu.addChild(menuItem1);
            
            var menuItem2 = this._createMenuItem(WorkViewletMessages.GANTT_TIMESCALE_QUARTER, "", WBSConstants.GANTT_QUARTER);
            this.timescale_menu.addChild(menuItem2);
            
            var menuItem3 = this._createMenuItem(WorkViewletMessages.GANTT_TIMESCALE_MONTH, "", WBSConstants.GANTT_MONTH);
            this.timescale_menu.addChild(menuItem3);
            
            var menuItem4 = this._createMenuItem(WorkViewletMessages.GANTT_TIMESCALE_WEEK, "", WBSConstants.GANTT_WEEK);
            this.timescale_menu.addChild(menuItem4);
            
            this.timescale_menubutton = new dijit.form.ComboButton({
                label: WorkViewletMessages.GANTT_TIMESCALE_WEEK,
                dropDown: this.timescale_menu
            });
            this.leftToolbar.addChild(this.timescale_menubutton);
            this.viewButtons.appendChild(this.leftToolbar.domNode);
			
			// Set the default timeline view to 'Week'
            this._onTimelineViewChange(WBSConstants.GANTT_WEEK);
        },
        
        //This function is invoked on click of menu item from timescale drop down
        /**
         *  _onTimelineViewChange : Called when the drop down of timeline changes
         *  Incoming : timeline item id ( String )
         *  Outgoing : none
         *
         */
        _onTimelineViewChange: function(itemName){
            var menuItems = this.timescale_menu.getDescendants();
            for (var i = 0; i < menuItems.length; i++) {
                var menuItem = menuItems[i];
                if (menuItem.domNode.getAttribute("name") != itemName) {
                    dojo.removeClass(menuItem.iconNode, 'com-ibm-epm-work-web-ui-internal-GanttView-checkedButtonIcon');
                }
                else {
                    if (!dojo.hasClass(menuItem.iconNode, 'com-ibm-epm-work-web-ui-internal-GanttView-checkedButtonIcon')) {
                        dojo.addClass(menuItem.iconNode, 'com-ibm-epm-work-web-ui-internal-GanttView-checkedButtonIcon');
                        //Set the label of the combo box according to the current selection
                        this.timescale_menubutton.attr('label', menuItem.label);
                        
                        //Load the Gantt view for this timeline
                        //Store the date corresponding to the center of the view
                        var viewDay = (this._ganttDragPane.domNode.scrollLeft + this._ganttDragPane.domNode.clientWidth / 2) - this._ganttTable.getDayWidth();
                        var viewDate = this._ganttTable.getPixelToDate(viewDay);
                        var done = this._ganttTable.switchTimeline(menuItem.domNode.getAttribute("name"), viewDay);
                        if (done) {
                            this.updateAllControls();
                            //Rearrange the view so that it is center aligned similar to before transformation
                            viewDay = this._ganttTable.getDateToPixel(viewDate);
                            this._ganttDragPane.domNode.scrollLeft = viewDay - this._ganttDragPane.domNode.clientWidth / 2 + this._ganttTable.getDayWidth();
                            //Create the resizer handlers if not already done.
                            this.setGanttBarResizeMinSizes();
                        }
                    }
                }
            }
        },
        
        /**
         *  _createToolbar : All toolbar button are created here
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createToolbar: function(){
			this.removeButton = new dijit.form.Button({
                title: WorkViewletMessages.REMOVE_WBS,
				label : WorkViewletMessages.REMOVE_BUTTON,
				showLabel : false,
                iconClass: WBSConstants.REMOVE_WBS_ICON_CLASS,
                onClick: dojo.hitch(this, this._removeWBSCommand)
            });
			
			this.separator = new dijit.ToolbarSeparator();

            this.addDependencyButton = new dijit.form.ToggleButton({
                title: WorkViewletMessages.ADD_DEPENDENCY,
				label : WorkViewletMessages.ADD_DEPENDENCY,
				showLabel : false,
                iconClass: WBSConstants.DEPENDENCY_ICON_CLASS,
                onClick: dojo.hitch(this, this._startStopDependencyCommand)
            });
			this.resolveDependencyButton = new dijit.form.Button({
                title: WorkViewletMessages.RESOLVE_DEPENDENCY_BUTTON_TITLE,
				label : WorkViewletMessages.RESOLVE_DEPENDENCY,
				showLabel : false,
                iconClass: WBSConstants.RESOLVEDEP_ICON_CLASS,
                onClick: dojo.hitch(this, this._resolveDependency)
            });

            this.deleteDependencyButton = new dijit.form.Button({
                title: WorkViewletMessages.DELETE_DEPENDENCY,
				label : WorkViewletMessages.DELETE_DEPENDENCY,
				showLabel : false,
                iconClass: WBSConstants.DEL_DEPENDENCY_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteDependencyCommand)
            });			
			
            this.separator2 = new dijit.ToolbarSeparator();
            
			this.baseline = new dijit.form.Button({
				title: WorkViewletMessages.BASELINE,
				label : WorkViewletMessages.BASELINE,
				showLabel : false,
				iconClass: "baselineButtonIcon",
				onClick: dojo.hitch(this,this._baseline)
			});
			
			this.criticalPathButton = new dijit.form.ToggleButton({
                title: WorkViewletMessages.SHOW_CRITICAL_PATH,
				label : WorkViewletMessages.SHOW_CRITICAL_PATH,
				showLabel : false,
                iconClass: WBSConstants.CRITICAL_PATH_ICON_CLASS,
                onClick: dojo.hitch(this, this._handleCriticalPathButton)
            });

            this.gotoTodayButton = new dijit.form.Button({
                title: WorkViewletMessages.GOTO_TODAY,
				label : WorkViewletMessages.GOTO_TODAY,
				showLabel : false,
                iconClass: WBSConstants.GOTOTODAY_ICON_CLASS,
                onClick: dojo.hitch(this, this._gotoToday)
            });
			
			this.separator3 = new dijit.ToolbarSeparator();
			
			this.topToolbar.addChild(this.removeButton);
			this.topToolbar.addChild(this.separator);
            this.topToolbar.addChild(this.addDependencyButton);
			this.topToolbar.addChild(this.resolveDependencyButton);
            this.topToolbar.addChild(this.deleteDependencyButton);
			this.topToolbar.addChild(this.separator2);
			this.topToolbar.addChild(this.baseline);
            this.topToolbar.addChild(this.criticalPathButton);
            this.topToolbar.addChild(this.gotoTodayButton);
            this.topToolbar.addChild(this.separator3);
            this.imageButtons.appendChild(this.topToolbar.domNode);
        },
        
		/**
		 * Validate whether the selected node is project 
		 */
        _validateForProject: function(message){
            var tree = dijit.byId(Constants.WBS_TREE_ID);
            if (tree && tree._selectedNode) {
				this._selectedNode = tree._selectedNode;
                var item = tree._selectedNode.item;
                if (item && item.resourcetype == WBSConstants.EPM_PROJECT_TYPE) {
					this.selectedProject = item.identifier;
				    this.selectedProjectName = item.name;
					return true;
                }
            }
            
            this._displayAndClearErrorMessage(message);
			
            return false;
			                  		   
		},
		
		/**
		 * _baseline : creation of baseline Dialog
		 * 
		 * Uses the jazz.ui.dialog
		 * BaselineSave handles the operations
		 */
		
		_baseline : function() {
			/*
			 * Validate if the selected node is project 
			 * Else throw an Error msg & return
			 */
		    var boolValidateForProject =  this._validateForProject(WorkViewletMessages.SELECT_PROJECT_TO_CREATE_BASELINE);
			if (boolValidateForProject == false) {
				return;
			}

			if (this.newDlg) {
				this.newDlg.destroy();
	            this.newDlg = null;
	        }

			this.baselinedlg = new com.ibm.epm.work.web.ui.internal.BaselineSave();

			this.newDlg =  new jazz.ui.Dialog({
	            contentNode: this.baselinedlg.domNode,
	            primaryTitle: WorkViewletMessages.CREATE_BASELINE_TITLE,
				width: "400px",
				onOpen: dojo.hitch(this, this._openBaselineDlg),
				onClose: dojo.hitch(this, this._closeBaselineDlg)	
	        });

	        this.baselinedlg.load(this.newDlg, this.selectedProject, this.selectedProjectName);
		},

		/**
         *  _openBaselineDlg : This function is used to set the flag that the baseline dialog is opened
         */
        _openBaselineDlg: function(){
            this.baselineDlgOpen = true;
        },
				
		/**
         *  _closeBaselineDlg : This function is used to set the flag that the baseline dialog is closed
         */
        _closeBaselineDlg: function(){
            this.baselineDlgOpen = false;
        },
						
		/**
		 * Resolve Project Dependency
		 * Check if there are any unsaved changes, if yes confirm with the user to save or discard the changes
		 * before proceeding with resolving dependency
		 * 
		 */
		_resolveDependency: function() {
			/*
			 * Validate if the selected node is project 
			 * Else throw an Error msg & return
			 */
		    var boolValidateForProject =  this._validateForProject(WorkViewletMessages.SELECT_PROJECT_TO_RESOLVE_DEPENDENCIES);			
			if (boolValidateForProject == false) {
				return;
			}

			if (this.saveButton.disabled == false) {
				// Confirm discard of changes and resolve dependency
				this.confirmDiscard(false);
			} else {
				this._okToResolveDependency();
			}
		},

		/**
		 * Resolve Project Dependency
		 * Invoke a warning message 
		 * On Yes , make the call to the server 
		 * 
		 */
		_okToResolveDependency: function(){
			var buttonlist = [{
	            label: WorkViewletMessages.YES,
                action: dojo.hitch(this, this._YesButtonHandler,this._selectedNode, this.selectedProject)
            }, {
                label: WorkViewletMessages.NO,
                action: dojo.hitch(this, this._NoButtonHandler)
            }];

			var labelList = [{
				label: WorkViewletMessages.WARNING_BEFORE_RESOLVING_DEPENDENCY
            }];
				
			this.resolveDependencyDialog = new EPMConfirmDialog({
                                            labelList: labelList,
                                            title: WorkViewletMessages.RESOLVE_DEPENDENCY,
                                            buttonlist: buttonlist,
											onClose: dojo.hitch(this, this._closeResolveDepDlg)
                                        });
            this._openResolveDepDlg();               
            this.resolveDependencyDialog.show();
		},


		/**
         *  _openResolveDepDlg : This function is used to set the flag that the resolve dependency dialog is opened
         */
		_openResolveDepDlg: function() {
			this.resolveDepDlgOpen = true;
		},

		/**
         *  _closeResolveDepDlg : This function is used to set the flag that the resolve dependency dialog is closed
         */
		_closeResolveDepDlg: function() {
			this.resolveDepDlgOpen = false;
		},
		
		/**
		 * Make the call if user clicks on yes 
		 */
		_YesButtonHandler: function(ctxtNode, parentID){
			if (this.resolveDependencyDialog != null) {
				this.resolveDependencyDialog.destroy();
				this._controller._makeCalltoResolveDependency(ctxtNode, parentID);
			}
		},
		/**
		 * close and destroy if user clicks of No 
		 */
		_NoButtonHandler: function(){
            if(this.resolveDependencyDialog != null)
			{
				this.resolveDependencyDialog.destroy();
			}
        },

        /**
         *  _destroyToolbar : All toolbar button are removed here
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _destroyToolbar: function(){
            if (this.topToolbar) {
                this._removeDestroyDeleteChild(this.gotoTodayButton, this.topToolbar);
                this._removeDestroyDeleteChild(this.addDependencyButton, this.topToolbar);
				this._removeDestroyDeleteChild(this.resolveDependencyButton,this.topToolbar);
				this._removeDestroyDeleteChild(this.criticalPathButton, this.topToolbar);
                this._removeDestroyDeleteChild(this.separator, this.topToolbar);
				this._removeDestroyDeleteChild(this.separator2, this.topToolbar);
				this._removeDestroyDeleteChild(this.separator3, this.topToolbar);
               	this._removeDestroyDeleteChild(this.baselinebutton,this.topToolbar);
				this._removeDestroyDeleteChild(this.removeButton, this.topToolbar);
                
                if (this.leftToolbar) {
                    this._removeDestroyDeleteChild(this.separator1, this.leftToolbar);
                    
                    this.viewButtons.removeChild(this.leftToolbar.domNode);
                    this.leftToolbar.destroy();
                }

                this.imageButtons.removeChild(this.topToolbar.domNode);
            }
        },
        
        /**
         *  _removeDestroyDeleteChild : Removes the child object from parent object; destroys the child object and deletes it
         *  Incoming : childObject, parentObject
         *  Outgoing : none
         *
         */
        _removeDestroyDeleteChild: function(childObject, parentObject){
            if (childObject) {
                parentObject.removeChild(childObject);
                childObject.destroy();
                delete childObject;
            }
        },
        
        /**
         *  _createBackground : Create the container to hold all the background rows
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createBackground: function(){
            //This is the container class for all the rows
            this.ganttRowBackground = new com.ibm.epm.work.web.ui.internal.GanttRowBackground({
                container: this._ganttBackgroundRows
            });
        },
        
        /**
         *  _createDragPane : Create the drag pane which makes the entire view pannable
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _createDragPane: function(){
            //the drag pane is the foundation for the dragging behavior
            this._ganttDragPane = new com.ibm.epm.work.web.ui.internal.GanttDragPane({
                controller: this
            });
            this._mainContent.appendChild(this._ganttDragPane.domNode);
            
            //Creating a container inside the drag pane to contain the timeline and the gantt bars
            this._ganttDragPaneContentDiv = document.createElement('DIV');
            dojo.addClass(this._ganttDragPaneContentDiv, 'GanttView-ganttDragPaneContent');
            this._ganttDragPane.domNode.appendChild(this._ganttDragPaneContentDiv);
            
            
            //Table is created here, pass the current timeline as argument
            this._ganttTable = new com.ibm.epm.work.web.ui.internal.GanttTimeLine({
                timelineType: "Year"
            });
            this._ganttDragPaneContentDiv.appendChild(this._ganttTable.domNode);
            
            this.dependencyDiv = document.createElement('DIV');
            dojo.addClass(this.dependencyDiv, "gantt-Dependency-Container");
            
            this._ganttDragPaneContentDiv.appendChild(this.dependencyDiv);
            
            //TODO : Revisit today line creation, move it other place
            this._todayDateDiv = document.createElement('DIV');
            dojo.addClass(this._todayDateDiv, "gantt-Today-Div");
            var todayPosition = this._ganttTable.getToday() + (this._ganttTable.getDayWidth() - TODAYLINE_WIDTH) / 2;
            dojo.style(this._todayDateDiv, "left", todayPosition + 'px');
            dojo.style(this._todayDateDiv, "top", 0 + 'px');
            this._ganttDragPaneContentDiv.appendChild(this._todayDateDiv);
            
        },
        
        /**
         *  toggleCriticalPath : Enables the flag in ganttBarInfo based whether critical path is checked or not
         *  Incoming : ganttBarInfo
         *  Outgoing : none
         *
         */
        toggleCriticalPath: function(ganttBarInfo){
            if (ganttBarInfo) {
                ganttBarInfo.criticalPath = this.criticalPathButton.checked;
            }
        },
        
        /**
         *  insertCriticalPathBar : Insert the gantt bar into the critical path array
         *  Incoming : gantt bar
         *  Outgoing : none
         *
         */
        insertCriticalPathBar: function(ganttBarId){
            var index = this.indexOfInArray(this._allCriticalPathArray, ganttBarId);
            if (index == -1) {
                this._allCriticalPathArray.push(ganttBarId);
            }
        },
        
        /**
         *  removeCriticalPathBar : Remove the gantt bar from the critical path array
         *  Incoming : gantt bar
         *  Outgoing : none
         *
         */
        removeCriticalPathBar: function(ganttBarId){
            //Remove the entry from the critical path array
            var index = this.indexOfInArray(this._allCriticalPathArray, ganttBarId);
            if (index > 0) {
                for (var i = index + 1; i < this._allCriticalPathArray.length; i++) {
                    this._allCriticalPathArray[i - 1] = this._allCriticalPathArray[i];
                }
                //remove the last item
                this._allCriticalPathArray.pop();
            }
            else {
                //console.log("ROW NOT FOUND IN CRITICAL PATH ARRAY AND ATTEMPT WAS MADE TO REMOVE FROM IT");
            }
        },
        
        /**
         *  updateGanttBarWithCPInfo : Update the critical path info into the Gantt bar info
         *  Incoming : row object
         *  Outgoing : none
         *
         */
        updateGanttBarWithCPInfo: function(data){
            if (data) {
                var ganttBar = this.getGanttWidgetFromIdentifier(data.id);
                if (ganttBar) {
                    this.toggleCriticalPath(ganttBar);
                    if (ganttBar.ganttBarInfo) {
                        ganttBar.ganttBarInfo.earlyFinishDate = data.earlyFinishDate;
                        ganttBar.ganttBarInfo.earlyStartDate = data.earlyStartDate;
                        ganttBar.ganttBarInfo.lateFinishDate = data.lateFinishDate;
                        ganttBar.ganttBarInfo.lateStartDate = data.lateStartDate;
                    }
                }
            }
        },
        
        /**
         *  wbsInfo : Pack the wbsinfo into the following format
         *  Incoming : row object
         *  Outgoing : wbs json object
         *
         */
        wbsInfo: function(rowAttributes){
            var row = {
                "id": rowAttributes.id,
                "name": rowAttributes.name,
                "href": rowAttributes.href,
                "startDate": rowAttributes.plannedstart,
                "endDate": rowAttributes.plannedfinish,
                "duration": rowAttributes.plannedduration,
                "effort": rowAttributes.plannedeffort,
                "parent": rowAttributes.parentId
            };
            return row;
        },
        
        //
        /**
         *  buildGanttBar : This function creates a gantt bar if it not present
         *  Incoming : row data, row Index
         *  Outgoing : none
         *
         */
        buildGanttBar: function(data, rowIndex){
            if (data) {
                var row = this.wbsInfo(data);
                
                var ganttBar = this.getGanttWidgetFromIdentifier(row.id);
                var found = (ganttBar != null);
                if (!found) {
                    var dependencyCommandRunning = (this.dependencyHandle != null);
                    //create a new gantt bar
                    ganttBar = new com.ibm.epm.work.web.ui.internal.GanttBar({
                        index: rowIndex,
                        ganttBarInfo: row,
                        ganttTimelineRef: this._ganttTable,
                        bCreateResizers: (!dependencyCommandRunning)
                    });
                    this.ganttBarIdMap[row.id] = ganttBar.id; //push this id into the map
                    this.rowIdMap[ganttBar.id] = row.id;
                    this._ganttDragPaneContentDiv.appendChild(ganttBar.domNode);
                }
                else {
                    ganttBar.setIndex(rowIndex);
                }
                this.insertGanttBarIntoVisibleArray(rowIndex, ganttBar);
                this._incrVisibleGanttBarIndex(rowIndex);
				if(ganttBar.index == 0)
					ganttBar.domNode.setAttribute('tabIndex', "0");
            }
        },
        
        /**
         *  hideOrRemoveRow : This function hides or removes the gantt bar
         *  Incoming : index of row to be removed/hidden, flag to hide or remove
         *  			hideOrRemove = true => Hide the gantt bar ( for caching and doing a quick expand )
         *              hideOrRemove = false => Delete the gantt bar
         *  Outgoing : none
         *
         */
        hideOrRemoveRow: function(removedRows, hideOrRemove){
        	var savedEndIndex = -1;
        	var savedStartIndex = -1;
        	for (var i in removedRows) {
        		var removedRowId = removedRows[i];
	            var ganttBar = this.getGanttWidgetFromIdentifier(removedRowId);
	            if (ganttBar) {
	                if (hideOrRemove) {
	                    ganttBar.visible = false;
	                    dojo.style(ganttBar.domNode, "display", "none");
	                    if(savedStartIndex == -1)
	                    	savedStartIndex = ganttBar.index;
 						var rowData = ganttBar.getWBSData();
						this._removedependecies(rowData);
						savedEndIndex = ganttBar.index;
	                }
	            }
	            else {
	                console.log("gantt bar not found,  some problem!!! with " + ganttBar);
	            }
        	}
        	
        	//first remove the entry from the visible array
            if (savedStartIndex >= 0 && savedEndIndex >=0) {
	             var j =0;                	            	
                 for (var i = savedEndIndex + 1; i < this._visibleGanttBarArray.length; i++) {	 
	               	var ganttBar = dijit.byId(this._visibleGanttBarArray[i]);
	                if (ganttBar) {
	                    ganttBar.setIndex(savedStartIndex + j);
						if(ganttBar.index == 0)
							ganttBar.domNode.setAttribute('tabIndex', "0");
						this._visibleGanttBarArray[savedStartIndex + j] = this._visibleGanttBarArray[i];
 						j = j + 1;	
	                   
	                }
	            }    
	                   	                                      
                //remove the last item
                for (var i = 0 ; i <=(savedEndIndex - savedStartIndex); i++) {
                	this._visibleGanttBarArray.pop();
					this.removeBackgroundRow();
				}
            }
            else {
                console.log("ROW NOT FOUND IN VISIBLE ARRAY AND ATTEMPT WAS MADE TO REMOVE FROM IT");
            }

 
             
       		for (var i = savedStartIndex; i < this._visibleGanttBarArray.length; i++) {
                var ganttBar = dijit.byId(this._visibleGanttBarArray[i]);
                if (ganttBar) {
                    ganttBar.updateBar();
                }
            }   
        },
		
		/* Remove all dependecies on hinding or removing as we recreate during expand. */
		
		_removedependecies: function(rowData){
			if(rowData.successors ){
						var newSuccessorValues = this.getCommaSeparatedValues("" + "");
						var oldSuccessorValues = this.getCommaSeparatedValues(rowData.successors + "");
						
						if(rowData.successors && (oldSuccessorValues.length >= newSuccessorValues.length)){
							for (var i = 0; i < oldSuccessorValues.length; i++) {
								var bFound = false;
								var oldSuccessorValue = oldSuccessorValues[i];
								
								for (var j = 0; j < newSuccessorValues.length; j++) {
									if (oldSuccessorValue == newSuccessorValues[j]) {
										bFound = true;
										break;
									}
								}
								if (bFound == false) {
									this.removeDependency(rowData.id, oldSuccessorValue);
								}
							}
						}
					}
					
					if((rowData.predecessors)){
						var newPredecessorValues = this.getCommaSeparatedValues("" + "");
						var oldPredecessorValues = this.getCommaSeparatedValues(rowData.predecessors + "");
						
						if(rowData.predecessors && (oldPredecessorValues.length >= newPredecessorValues.length)){
							for (var i = 0; i < oldPredecessorValues.length; i++) {
								var bFound = false;
								var oldPredecessorValue = oldPredecessorValues[i];
								
								for (var j = 0; j < newPredecessorValues.length; j++) {
									if (oldPredecessorValue == newPredecessorValues[j]) {
										bFound = true;
										break;
									}
								}
								if (bFound == false) {
									this.removeDependency(oldPredecessorValue, rowData.id);
								}
							}
						}						
					}
		},
        
        /**
         *  updateGanttBar : Update the gantt bar with the wbs information
         *  Incoming : wbs data
         *  Outgoing : none
         *
         */
        updateGanttBar: function(data){
			// Wait till the calculation is complete
			if ((this.criticalPathButton.checked) &&
				(this._controller._calculateInProgress)) {
				var self = this;
				var intervalID = setInterval(function(){
					if (!self._controller._calculateInProgress) {
						clearInterval(intervalID);
						Util.hideMessageArea();
					}
				}, 50);
			}
			else {
				Util.hideMessageArea();
			}

            if (data && data.id) {
                var ganttBar = this.getGanttWidgetFromIdentifier(data.id);
                if (ganttBar) {
                    //console.log("UPDATING GANTT BAR WITH ID = " + data.id);
                    var rowData = ganttBar.getWBSData();
                    if (data.name) {
                        rowData.name = data.name;
                    }

					rowData.percentEC = data.percentEC;
					rowData.percentDC = data.percentDC;
					rowData.percentC = data.percentC;
					if (data.calculatedPercentC) {
						rowData.calculatedPercentC = data.calculatedPercentC;
					}

					rowData.hasAssignments = data.hasAssignments;
					if (data.hasChildren) {
						rowData.hasChildren = data.hasChildren;
					}

					if (data.resourcetype) {
						rowData.resourcetype = data.resourcetype;
					}
					else {
						rowData.resourcetype = this.getResourceType(data.id);
					}

					if (data.assignmenttype) {
						rowData.assignmenttype = data.assignmenttype;
					} else if (rowData.resourcetype == WBSConstants.ITERATION_TYPE) {
						// Iteration doesnt have the assignment type set, so fetch it from the parent project
						var parentRowIndex = this._controller.getRowIndex(data.parentId);
						var row = this._controller.getRowByIndex(parentRowIndex);
						if (row) {
							rowData.assignmenttype = row.assignmenttype;
						}
					}
					
					if ((data.hasChildren) || (data.hasAssignments) || ((data.resourcetype) && (data.resourcetype === WBSConstants.EPM_PROGRAM_TYPE))) {
					
						// Make the ganttBar ir-resizable for parent nodes, Program element and if the WBS element has any resource assignments
						ganttBar.makeIrresizable();
					}
					
                    
                    rowData.workItemType = dojo.isArray(data.workItemType) ? data.workItemType[0] : data.workItemType;

					rowData.startDate = this.removeTimeZoneFromDate(dojo.isArray(data.plannedstart) ? data.plannedstart[0] : data.plannedstart);
					rowData.endDate = this.removeTimeZoneFromDate(dojo.isArray(data.plannedfinish) ? data.plannedfinish[0] : data.plannedfinish);
					rowData.duration = dojo.isArray(data.plannedduration) ? data.plannedduration[0] : data.plannedduration;

					if(rowData.successors || data.successors){
						var newSuccessorValues = this.getCommaSeparatedValues(data.successors + "");
						var oldSuccessorValues = this.getCommaSeparatedValues(rowData.successors + "");
						
						if(rowData.successors && (oldSuccessorValues.length >= newSuccessorValues.length)){
							for (var i = 0; i < oldSuccessorValues.length; i++) {
								var bFound = false;
								var oldSuccessorValue = oldSuccessorValues[i];
								
								for (var j = 0; j < newSuccessorValues.length; j++) {
									if (oldSuccessorValue == newSuccessorValues[j]) {
										bFound = true;
										break;
									}
								}
								if (bFound == false) {
									this.removeDependency(data.id, oldSuccessorValue);
								}
							}
						}
						rowData.successors = data.successors;
					}
					
					if((rowData.predecessors) || data.predecessors){
						var newPredecessorValues = this.getCommaSeparatedValues(data.predecessors + "");
						var oldPredecessorValues = this.getCommaSeparatedValues(rowData.predecessors + "");
						
						if(rowData.predecessors && (oldPredecessorValues.length >= newPredecessorValues.length)){
							for (var i = 0; i < oldPredecessorValues.length; i++) {
								var bFound = false;
								var oldPredecessorValue = oldPredecessorValues[i];
								
								for (var j = 0; j < newPredecessorValues.length; j++) {
									if (oldPredecessorValue == newPredecessorValues[j]) {
										bFound = true;
										break;
									}
								}
								if (bFound == false) {
									this.removeDependency(oldPredecessorValue, data.id);
								}
							}
						}
						rowData.predecessors = data.predecessors;
					}                    
                    
                    // Update the critical path related info if slack is not -1
                    if (data.slack >= 0) {
                        if (data.earlyFinishDate) {
                            rowData.earlyFinishDate = (dojo.isArray(data.earlyFinishDate)) ? data.earlyFinishDate[0] : data.earlyFinishDate;
                        }
                        if (data.earlyStartDate) {
                            rowData.earlyStartDate = (dojo.isArray(data.earlyStartDate)) ? data.earlyStartDate[0] : data.earlyStartDate;
                        }
                        if (data.lateFinishDate) {
                            rowData.lateFinishDate = (dojo.isArray(data.lateFinishDate)) ? data.earlyFinishDate[0] : data.lateFinishDate;
                        }
                        if (data.lateStartDate) {
                            rowData.lateStartDate = (dojo.isArray(data.lateStartDate)) ? data.earlyFinishDate[0] : data.lateStartDate;
                        }
                        this.updateGanttBarWithCPInfo(rowData);
                    }
                    
					// Update the critical path array only if the elements are part of the selected project
					var containingProject = dojo.isArray(data.containingProject) ? data.containingProject[0] : data.containingProject;
					var selectedProject = dojo.isArray(this.selectedProject) ? this.selectedProject[0] : this.selectedProject;
					if (this.projectPresentInHierarchy(containingProject, selectedProject)) {
						if (data.slack == 0) {
							// Insert into critical path array
							this.insertCriticalPathBar(ganttBar.id);
							this._displayCriticalPathForBar(data.id);
						}
						else if (data.slack > 0) {
							// Remove from critical path array
							this.removeCriticalPathBar(ganttBar.id);
							ganttBar.hideCritPathBar();
						}
					}
                    
                    //Finally update the gantt bar widget to reflect these changes
                    ganttBar.updateBar();
                    
                    if (rowData.startDate && rowData.endDate && rowData.duration && rowData.successors) {
                        //console.log("rowdata has succ:" + rowData.successors);
                        this._drawDependenciesSuccessors(rowData);
                    }
                    
                    if (rowData.startDate && rowData.endDate && rowData.duration && rowData.predecessors) {
                        //console.log("rowdata has pred:" + rowData.predecessors);
                        this._drawDependenciesPredecessors(rowData);
                    }
                    if (rowData.startDate && rowData.endDate && rowData.duration) {
                        this._redrawDependencies();
                    }
                }
            }
        },

        /**
         *  projectPresentInHierarchy : Check if the sub project is part of the hierarchy of the selected project
         *  Incoming : project
         *  Outgoing : true if exists in hierarchy, else false
         *
         */        
		projectPresentInHierarchy: function(subproject, selectedProject) {
			if (subproject) {
				if (subproject == selectedProject) {
					return true;
				}
				else {
					var containingProjectRowIndex = this._controller.getRowIndex(subproject);
					var row = this._controller.getRowByIndex(containingProjectRowIndex);
					if (row) {
						var parentContainingProject = row.parentId;
						if (parentContainingProject) {
							if (parentContainingProject == selectedProject) {
								return true;
							}
							else 
								if (parentContainingProject.indexOf(WBSConstants.EPM_PROGRAM_TYPE) != -1) {
									return false;
								}
								else {
									return this.projectPresentInHierarchy(parentContainingProject, selectedProject);
								}
						}
					}
				}
				
			}
			return false;
		},
		
        /**
         *  onAfterUpdateDependency : This is called after gantt dependency gets created.<br>
         *  Incoming : row data
         *  Outgoing : none
         *
         */
        onAfterUpdateDependency: function(data){
            //introducing detailLevel and Identifier as these are not present in the Data object.
            data.detailLevel = true;
            data.id = data.sourceId;
            
            var sourceBarId = this.ganttBarIdMap[data.id];
            var source = dijit.byId(sourceBarId);
            
            var targetBarId = this.ganttBarIdMap[data.targetId];
            var target = dijit.byId(targetBarId);
			if (this._controller) {
            	if (this._isDependencyValid(source, target, WBSConstants.FINISH_TO_START)) {
                	this._addDependency(source, target, WBSConstants.FINISH_TO_START);
					var selectedRow = this._controller.getRowByIndex(target.index);
					this._controller.setRowData(selectedRow, selectedRow.id, target.index);
				}
            }
        },
        
        /**
         *  _drawDependenciesPredecessors : Draw all the predecessors in the view
         *  Incoming : row data
         *  Outgoing : none
         *
         */
        _drawDependenciesPredecessors: function(data){
            //Predecessors
            var predecessors = this.getCommaSeparatedValues(data.predecessors + "");
            for (var i = 0; i < predecessors.length; i++) {
            
                var sourceBarId = this.ganttBarIdMap[predecessors[i]];
                var source = dijit.byId(sourceBarId);
                
                var targetId = this.ganttBarIdMap[data.id];
                var target = dijit.byId(targetId);
                
                this._addDependency(source, target, WBSConstants.FINISH_TO_START);
            }
        },
        
        //Draw all the successors
        /**
         *  _drawDependenciesSuccessors : Draw all the successors in the view
         *  Incoming : row data
         *  Outgoing : none
         *
         */
        _drawDependenciesSuccessors: function(data){
            //Successors
            var successors = this.getCommaSeparatedValues(data.successors + "");
            
            for (var i = 0; i < successors.length; i++) {
                var targetBarId = this.ganttBarIdMap[successors[i]];
                var target = dijit.byId(targetBarId);
                
                var ganttBarId = this.ganttBarIdMap[data.id];
                var source = dijit.byId(ganttBarId);
                this._addDependency(source, target, WBSConstants.FINISH_TO_START);
                
            }
        },
        
        /**
         *  getCommaSeparatedValues : Returns an array from a comma separated string
         *  Incoming : comma separated string
         *  Outgoing : array
         *
         */
        getCommaSeparatedValues: function(str){
            var strArray = new Array();
            while (str.indexOf(",") >= 0) {
                strArray.push(str.substring(0, str.indexOf(",")));
                str = str.substring(str.indexOf(",") + 1, str.length);
            }
            
            strArray.push(str);
            return strArray;
        },
        
        /**
         *  getVisibleRowIndex : Get the index of the gantt bar from is dijit id
         *  Incoming : gantt widget id
         *  Outgoing : index ( Integer )
         *
         */
        getVisibleRowIndex: function(gId){
            var visIndex = -1;
            var ganttBar = dijit.byId(gId);
            if ((ganttBar)) {
                visIndex = ganttBar.index;
            }
            return visIndex;
        },
        
        /**
         *  addTimeBlockToRight : call Timeline Function to add a Timeline Block at the Right side of GanttDragPane
         *  Incoming : none
         *  Outgoing : none
         *
         */
        addTimeBlockToRight: function(){
            console.log("Now creating a new block in the right");
            this._ganttTable.addTimelineBlockRight();
            this.updateAllControls();
        },
        
        /**
         *  addTimeBlockToLeft : call Timeline Function to add a Timeline Block at the Left side of GanttDragPane
         *  Incoming : none
         *  Outgoing : none
         *
         */
        addTimeBlockToLeft: function(){
            console.log("Now creating a new block in the left");
            this._ganttTable.addTimelineBlockLeft();
            this.updateAllControls();
        },
        
        /**
         *  getCurrentTimeLineBlockWidth : Return the current timeline's block width
         *  Incoming : none
         *  Outgoing : timeline width (Integer)
         *
         */
        getCurrentTimeLineBlockWidth: function(){
            return this._ganttTable.getTimeLineBlockWidth();
        },
        
        /**
         *  getGanttWidgetFromIdentifier : 	This function returns the gantt widget taking the workitem id
         *                                  We have cached the gantt id to help in easy and fast access
         *  Incoming : work item id
         *  Outgoing : gantt dojo widget
         *
         */
        getGanttWidgetFromIdentifier: function(id){
            var ganttBarId = this.ganttBarIdMap[id];
            if (ganttBarId) {
                return dijit.byId(ganttBarId);
            }
            else {
                return null;
            }
        },
        
        /**
         *  onRowSelect : Action of row select
         *  Incoming : row data
         *  Outgoing : none
         *
         */
        onRowSelect: function(data){
		   
            if (data && this._controller) {
                //Verify the publishing event.
                if ((data.treeID === this._controller._baseTreeID) ||
                    (data.gridID === this._controller._baseGridID)) {
                    var id = data.identifier;
                    if ((id === null) && (data.rowIndex > -1)) {
                        var row = this._controller.getRowByIndex(data.rowIndex);
                        if (row) {
                            id = row.id;
                        }
                    }
                    this._selectRowById(id);
					
					if (this._clearErrorMsg) {
						this._clearErrorMsg = false;
						// clear the previous error msg if any
						this._controller._clearErrorMessage();	
					}
                }
            }
        },
        
        /**
         * _selectRowById: Select the row for the given ID. Deselect the previously selected row, if any. <br>
         * @param {Object} id: id of the row to be selected. <br>
         * Outgoing: None.
         * References: this.onRowSelect, this.newRowInserted.<br>
         */
        _selectRowById: function(id){
            if (id) {
                var ganttBar = this.getGanttWidgetFromIdentifier(id);
                var found = (ganttBar != null);
                if (found) {
                    var ganttBarId = ganttBar.id;
                    if (this.selectedGanttBarId != ganttBarId) {
                        // Deselect the previous selection
                        var visibleRowIndex = this.getVisibleRowIndex(this.selectedGanttBarId);
                        this.ganttRowBackground.highlight(visibleRowIndex, false);
                        
                        // Set the new selection
                        visibleRowIndex = this.getVisibleRowIndex(ganttBarId);
                        this.ganttRowBackground.highlight(visibleRowIndex, true);
                        this.selectedGanttBarId = ganttBarId;
                    }
                    else {
						id = dojo.isArray(id) ? id[0] : id;
                        if (id.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1) {
                            visibleRowIndex = this.getVisibleRowIndex(ganttBarId);
                            this.ganttRowBackground.highlight(visibleRowIndex, true);
                        }
                    }
                }
            }
        },
        /**
         *  onRowOnBlur : Action of row unselect
         *  Incoming : none
         *  Outgoing : none
         *
         */
        onRowOnBlur: function(data){
            if (data && this._controller) {
                if (data.treeID == this._controller._baseTreeID) {
                    // Deselect the previous selection
                    var visibleRowIndex = this.getVisibleRowIndex(this.selectedGanttBarId);
                    this.ganttRowBackground.highlight(visibleRowIndex, false);
                    
                    // reset the selected row
                    this.selectedGanttBarId = null;
                }
            }
        },
        
        /**
         *  _destroyWidgets : Destroy the timescale menu item
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _destroyWidgets: function(){
            //Remove the menu and its menu items
            this.timescale_menubutton.destroyRecursive(false);
            //Gantt bars will also be removed from here
        },
        
        //This function will iterate over all controls inside dragpane and reposition them
        /**
         *  updateAllControls : iterate over all gantt bars and update, redraw dependencies
         *  Incoming : none
         *  Outgoing : none
         *
         */
        updateAllControls: function(){
            //Update the today line
            var todayPosition = this._ganttTable.getToday() + (this._ganttTable.getDayWidth() - TODAYLINE_WIDTH) / 2;
            dojo.style(this._todayDateDiv, "left", todayPosition + 'px');
            dojo.style(this._todayDateDiv, "top", 0 + 'px');
            
            //Update all the gantt bars
            for (var i = 0; i < this.allGanttBarArray.length; i++) {
                var ganttWid = dijit.byId(this.allGanttBarArray[i]);
                if ((ganttWid) && (ganttWid.visible)) {
                    ganttWid.updateBar();
                }
            }
            //Update all the dependency arrows here
            this._redrawDependencies();
        },
        
        //this scrolls the view to bring the todays date in the center of the view
        /**
         *  _gotoToday : Set the view center aligned with todays date
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _gotoToday: function(){
            if (this._ganttTable.transitionToToday()) {
                this.updateAllControls();
            }
            this._ganttDragPane.domNode.scrollLeft = this._ganttTable.getToday() - this._ganttDragPane.domNode.clientWidth / 2;
        },
        
        /**
         * _isDependencyValid: Function to validate the dependency. <br>
         * source: Source of the dependency to be created. <br>
         * target: Successor of the dependency. <br>
         * type: Type of the dependency to be created, <br>
         */
        _isDependencyValid: function(source, target, type){
            if (source && target) {
                
                // Check for existing dependency between source and target
                if (this._dependencyAlreadyExists(source, target, type)) {
                    this._displayAndClearErrorMessage(
                        WorkViewletMessages.DEPENDENCY_ALREADY_EXISTS);
                    return false;
                }
                
                // Check for dependency on unsaved items
                if (this._controller) {
                    var sourceRow = this._controller.getRowByIndex(source.index);
                    var targetRow = this._controller.getRowByIndex(target.index);
                    var sourceRowId = sourceRow.id;
                    var targetRowId = targetRow.id;
                
                    if ((sourceRowId.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1) || 
                        (targetRowId.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1)) {
                         
                        this._displayAndClearErrorMessage(
                            WorkViewletMessages.DEPENDENCY_ON_UNSAVED_ITEM);
                        return false;
                    }
                }
                return true;
            }
        },
        /**
         *  _addDependency : This function creates a dependency between source and target nodes
         *  Incoming : source : gantt Widget, target: ganttWidget, type: dependency type
         *  Outgoing : none
         *
         */
        _addDependency: function(source, target, type){
			if (!this._dependencyAlreadyExists(source, target, type)) {
				//    	if(source.visible && target.visible) {
				if (source && target) {
					var dependency = new com.ibm.epm.work.web.ui.internal.GanttDependency({
						source: source,
						target: target,
						type: type
					});
					this.dependencyDiv.appendChild(dependency.domNode);
					this.ganttDependencies.push(dependency);
					var selectedRow = this._controller.getRowByIndex(target.index);
					if (selectedRow) {
						if (dojo.isArray(selectedRow.predecessors) && dojo.indexOf(selectedRow.predecessors, source.ganttBarInfo.id) == -1) {
							selectedRow.predecessors.push(source.ganttBarInfo.id);
						}
						else {
							selectedRow.predecessors = [source.ganttBarInfo.id];
						}
					}
				}
			}
        },
        
        /**
         * _dependencyAlreadyExists:
         * Incoming: source: predecessor node.
         * 			 target: Successor node.
         *			 type: type of the dependency.
         * Outgoing: Boolean value indicating whether source is already depedendent on destination.
         */
        _dependencyAlreadyExists: function(source, target, type){
            for (var i = 0; i < this.ganttDependencies.length; i++) {
                var existingDependency = this.ganttDependencies[i];
                if ((existingDependency.source == source) && (existingDependency.target == target) &&
                (existingDependency.type == type)) {
                    return true;
                }
            }
            return false;
        },
        
        /**
         *  _redrawDependencies : Refresh all the dependencies
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _redrawDependencies: function(){
            //Redraw existing dependencies if the view is active.
			if (this._controller && this._controller.isViewActive()) {
				for (var i = this.ganttDependencies.length - 1; i >= 0; i--) {
					var ganttDependency = this.ganttDependencies[i];
					//			if (ganttDependency.visible) {
					ganttDependency.redraw();
				//			}
				}
			}
        },
        
        /**
         *  removeDependency between the source and target ids specified.<br>
         *  source, target: source and target nodes of the dependency to be deleted. <br>
         *  
         */
        removeDependency: function(source, target){
            for (var ganttDependencyIndex = this.ganttDependencies.length - 1; ganttDependencyIndex >= 0; ganttDependencyIndex--) {
	            var ganttDependency = this.ganttDependencies[ganttDependencyIndex];
				if (ganttDependency.source.ganttBarInfo && ganttDependency.target.ganttBarInfo) {
					if (ganttDependency.source.ganttBarInfo.id == source && ganttDependency.target.ganttBarInfo.id == target) {
						ganttDependency.removeDependency();
						this.dependencyDiv.removeChild(ganttDependency.domNode);
						this.ganttDependencies.splice(ganttDependencyIndex, 1);
					}
				}
            }
        },

        /**
         *  indexOfInArray : Array utility function to return the index ( indexOf doesnot work for all browsers )
         *  Incoming : array, item
         *  Outgoing : index
         *
         */
        indexOfInArray: function(inputArray, obj){
            for (var i = 0; i < inputArray.length; i++) {
                if (inputArray[i] == obj) {
                    return i;
                }
            }
            return -1;
        },
        
        /**
         *  _getResourceIDFromURI : return the id from the URI
         *  Incoming : href
         *  Outgoing : resource ID
         *
         */
        _getResourceIDFromURI: function(relatedHref){
            var index = relatedHref.lastIndexOf("/");
            if (index > 0) {
                return relatedHref.substring(index + 1, relatedHref.length);
            }
        },
        
        /**
         * update: Called when the viewlet becomes active again. <br>
         */
        update: function(){
            this._gotoToday();
			this.adjustDragPaneHeight();
        },
        /**
         *  onAfterUpdateGanttBar : After changes in schedule in gantt view update the depedencies and update the schedule in grid
         *  Incoming : gantt widget id
         *  Outgoing : none
         *
         */
        onAfterUpdateGanttBar: function(ganttBarId){
            var ganttBar = dijit.byId(ganttBarId);
            var wbsData = ganttBar.getWBSData();
                       
            //1. Update the grid
            if (this._controller) {
                var selectedRow = this._controller.getRowByIndex(ganttBar.index);
                if (selectedRow) {
					// Update in the client the expected and planned for the element so that grid and gantt data are in sync
                    selectedRow.proposedstart = selectedRow.plannedstart = wbsData.startDate;
	                selectedRow.proposedfinish = selectedRow.plannedfinish = wbsData.endDate;
    	            this._controller.setRowData(selectedRow, selectedRow.id, ganttBar.index);
					//Do the auto calculation of schedule by considering the project calendar
					this._doAutoCalculation(selectedRow,wbsData,ganttBar.index);
									
                }
               
                else {
                    console.log("Error getting the grid row, hence not updating the grid row with schedule information");
                }
            }
            
            //2. Update all the dependencies
            if (ganttBar) {
                var rowData = ganttBar.getWBSData();
                if (((rowData.successors) && (rowData.successors !== '')) ||
                ((rowData.predecessors) && (rowData.predecessors !== ''))) {
                    this._redrawDependencies(); //Repaint of all dependencies in the view.
                }
                
                //publish an event to update the corrosponding Grid Row.
                /*var data = [];
                 data.id = rowData.id;
                 data.startDate =rowData.startDate ;
                 data.endDate = rowData.endDate;
                 dojo.publish(WBSGridGanttPubSub.ON_UPDATE_GANTTBAR,[data]);*/
            }
        },
        
		/**
		 * After the gantt is resized [ either finishdate or startdate has been changed ] 
		 * we need to calcuate the duration from the project calendar.
		 * Pass the row index , row and row.id for further updating the listener's with 
		 * the new value .
		 * @param {Object} selectedRow
		 */
		_doAutoCalculation: function(selectedRow,newData,ganttBar_index){
			if (this._controller && this._controller._gridData) {
				 var item = this._controller._gridData.store._getItemByIdentity(selectedRow.id);
				 var params = this.prepareTheParams(selectedRow,newData);
				 this._controller._getProposedScheduleFromCalendar(params,item,newData,selectedRow,ganttBar_index);
			}
		},
		
		/**
		 * Populate the required Parameters the server expects.
		 * @param {Object} selectedRow
		 * @param {Object} newData
		 */
		prepareTheParams: function(selectedRow,newData){
			
			var parentRowIndex = this._controller.getRowIndex(selectedRow.parentId);
	        var parentRow = this._controller.getRowByIndex(parentRowIndex);
			var projectId = WBSUtil._getProjectIdFromRow(selectedRow,parentRow);
			
		    var params =  {
									"sProposedStartDate": dateUtil.formatDateForSave(newData.startDate),
									"sProposedFinishDate" : dateUtil.formatDateForSave(newData.endDate),
									"iProposedDuration": newData.duration,
									"iProjectId" : projectId
			
						   };
			
			  params.bProposedDuration = true;
		      params.bProposedFinishDate = false;	
			  params.resourceType = selectedRow.resourcetype;
			  params.workitemType = selectedRow.workItemType;

			return params;
		},
		
		
        /**
         *  addBackgroundRow : Add a new background row
         *  Incoming : none
         *  Outgoing : none
         *
         */
        addBackgroundRow: function(){
            this.ganttRowBackground.addRow();
        },
        
        
        /**
         *  removeBackgroundRow : Remove a background row
         *  Incoming : none
         *  Outgoing : none
         *
         */
        removeBackgroundRow: function(){
            this.ganttRowBackground.removeRows(1);
			this.adjustDragPaneHeight();
        },
        
		/**
		 * validate and resize dragPane.
		 * When the row/s are added to the grid, AGC publishes an event, We subscribed to that event to fetch the latest viewlet size.
		 * This way, the dragPane has always the same height as the viewlet. 
		 */
		validateAndResizeDragPane: function(publishedObject){
			if(publishedObject && publishedObject.controllerID && publishedObject.controllerID == controllerID){
				this.adjustDragPaneHeight();
			}
		},
		
		/**
		 *Resize dragPane to viewlet height.
		 */
		adjustDragPaneHeight: function(){
			if (this._viewlet.domNode.clientHeight == 0 && this._viewlet.domNode.clientWidth == 0) {
				//domNode creation is not complete -- exiting to resolve IE7 issues
				return;
			}
			if (this._viewlet && this._viewlet.getSite().isActive() ) {
				var viewletHeight = this._viewlet.getSite().getHeight();
				dojo.style(this._ganttDragPane.domNode, "height", viewletHeight + "px");
			}
		},

        
        /**
         *  _startStopDependencyCommand : Toggle command to create dependency (create/delete the resizers and movers )
         *  Incoming : none
         *  Outgoing : none
         *
         */
        _startStopDependencyCommand: function(){
            if (!this.dependencyHandle) {
                this.addDependencyButton.attr('checked', true);

                var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/images/selectPredecessor.cur") + "'),pointer";
                dojo.style(this._ganttDragPane.domNode, "cursor", cursorUrl);
                
                //iterate over all ganttbars and remove their sizers and movers
                for (var i = 0; i < this.allGanttBarArray.length; i++) {
                    var ganttWid = dijit.byId(this.allGanttBarArray[i]);
                    if (ganttWid) {
                        ganttWid.removeResizers();
                    }
                }
                
                this.dependencyHandle = new com.ibm.epm.work.web.ui.internal.GanttDependencyHandle({
                    listener: this._mainContent,
                    container: this.dependencyDiv,
					dragPane:this._ganttDragPane
                });
            }
            else {
                this.addDependencyButton.attr('checked', false);
                this.dependencyHandle.destroyWidget();
                this.dependencyHandle = null;
                //iterate over all ganttbars and add the sizers and movers
                for (var j = 0; j < this.allGanttBarArray.length; j++) {
                    var ganttWid1 = dijit.byId(this.allGanttBarArray[j]);
                    if (ganttWid1 && (ganttWid1.resizable == true)) {
                        ganttWid1.createResizers();
                        ganttWid1.setResizeLowerBounds();
                    }
                }
            }
        },
        
		/**
		 * Method called when the user wants to remove WBS command is fired.
		 */
		_removeWBSCommand: function(){
			// Check if dependency is selected
			var selectedDependencies = this._ganttDragPane.selectedDependencyArray;
			if (this._controller) {
				var tree = dijit.byId(Constants.WBS_TREE_ID);
            	if (tree) {
					this._controller._removeWBSElement(tree._selectedNode, selectedDependencies);
				}
			}
		},
		
		/**
		 * Method called when the delete dependency command is fired.
		 */
		_deleteDependencyCommand: function(){
			//Drag pane maintains array of the selected dependencies.
			var selectedDependencies = this._ganttDragPane.selectedDependencyArray; 
			if (selectedDependencies) {
				//Multiple dependencies need to be deleted.
				for(var i=0; i<selectedDependencies.length; i++){
					var dependencyToBeDelete = selectedDependencies[i];
					if(dependencyToBeDelete && dependencyToBeDelete.id)
					var dependency = dijit.byId(dependencyToBeDelete.id);
					this.deleteDependency(dependency);
				}
			}
		},
		
		/**
		 * Method called from the _deleteDependencyCommand. It fetches the source and target nodes of the dependency and deletes the dependency.
		 * @param {Object} dependency: dependency to be deleted.
		 */
		deleteDependency: function(dependency){
			var sourceNode = dependency.source;
			var targetNode = dependency.target;
			if(sourceNode && targetNode){
				if(sourceNode.ganttBarInfo.id && targetNode.ganttBarInfo.id){
					//remove the dependency preview.
					this.removeDependency(sourceNode.ganttBarInfo.id, targetNode.ganttBarInfo.id);
					
					//Now update the ganttRow with these values.
	                var predecessorRow = this._controller.getRowByIndex(sourceNode.index);
					var successorRow = this._controller.getRowByIndex(targetNode.index);
					
					//Update predecessor ganttBarRow with new successor values.
	                if (predecessorRow) {
						var previousSuccessors = predecessorRow.successors;
						if(previousSuccessors){
							var index = dojo.indexOf(previousSuccessors, successorRow.id);
							if(index !== -1){
								previousSuccessors.splice(index, 1);
								predecessorRow.successors = previousSuccessors;
			                    this._controller.setRowData(predecessorRow, predecessorRow.id, sourceNode.index);
							}
						}
	                }
					
					//Update successor row with new predecessor values.
	                if (successorRow) {
						var previousPredecessors = successorRow.predecessors;
						if(previousPredecessors){
							var index =dojo.indexOf(previousPredecessors,predecessorRow.id);
							if(index !== -1){
								previousPredecessors.splice(index,1);
								successorRow.predecessors = previousPredecessors;
								this._controller.setRowData(successorRow, successorRow.id, targetNode.index);
							}
						}
	                }
					
					//Publish object to enable save button.
					var publishedObj = {
						sourceNode: sourceNode,
						targetNode: targetNode
					};
	                publishedObj.uniqueIdentifier = {
	                    treeID: WBSConstants.WBS_TREE_ID
	                };
					publishedObj.controllerID = controllerID;
					
					dojo.publish(WBSGridGanttPubSub.ON_DELETE_DEPENDENCY, [publishedObj]);
				}
			}
		},
		
        //
        /**
         *  setGanttBarResizeMinSizes : Set minimum sizes to the gantt bar. this is required according to each timeline since day width is different for each timeline
         *  Incoming : none
         *  Outgoing : none
         *
         */
        setGanttBarResizeMinSizes: function(){
            for (var i = 0; i < this.allGanttBarArray.length; i++) {
                var ganttWid = dijit.byId(this.allGanttBarArray[i]);
                if (ganttWid) {
                    ganttWid.setResizeLowerBounds();
                }
            }
        },
        
        /**
         *  destroy : Destructor
         *  Incoming : none
         *  Outgoing : none
         *
         */
        destroy: function(){
            if (this._controller) {
                this._controller.destroy();
                delete this._controller;
            }
            this._destroyWidgets();
			this.destroyMainContent();
            this._destroyToolbar();
            this.inherited(arguments);
        },
        
        /**
         *  getResourceType : This is a temporary function, it will be removed in future
         *  Incoming : none
         *  Outgoing : none
         *
         */
        getResourceType: function(workItemId){
            if (workItemId.substr(0, 7) == 'project') {
                return WBSConstants.EPM_PROJECT_TYPE;
            }
            if (workItemId.substr(0, 7) == 'program') {
                return WBSConstants.EPM_PROGRAM_TYPE;
            }
            
            if (workItemId.substr(8, 9) == 'milestone') {
                return WBSConstants.EPM_MILESTONE_TYPE;
            }
            if (workItemId.substr(0, 9) == 'workitems') {
                return WBSConstants.EPM_TASK_TYPE;
            }
            return '';
        },
        
        /**
         *  insertGanttBarIntoVisibleArray : Insert the new gantt bar into the visible array to index it
         *  Incoming : none
         *  Outgoing : none
         *
         */
        insertGanttBarIntoVisibleArray: function(rowIndex, ganttBar){
			//this gantt bar is being inserted
			if (rowIndex >= 0) {
				// Avoid cases of inserting duplicate bars, Example: in a case of an empty DB,
				// the gantt bars were added twice hence the bar was seen in the next line
				var indexVisibleArr = this.indexOfInArray(this._visibleGanttBarArray, ganttBar.id);
				if (indexVisibleArr == -1) {
					if (rowIndex < this._visibleGanttBarArray.length) {
						var oldLength = this._visibleGanttBarArray.length;
						for (var i = oldLength; i > rowIndex; i--) {
							this._visibleGanttBarArray[i] = this._visibleGanttBarArray[i - 1];
						}
						this._visibleGanttBarArray[rowIndex] = ganttBar.id;
	                    this.addBackgroundRow();
					}
					else {
						//inserted at the end
						this._visibleGanttBarArray[rowIndex] = ganttBar.id;
	                    this.addBackgroundRow();
					}
				}
				
				// Check if bar is already inserted
				var index = this.indexOfInArray(this.allGanttBarArray, ganttBar.id);
				if (index == -1) {
					this.allGanttBarArray.push(ganttBar.id);
				}
			}
        },
        
        /**
         *  _removeFromVisibleArray : Remove the id from the visible array
         *  Incoming : gantt widget id
         *  Outgoing : none
         *
         */
        _removeFromVisibleArray: function(ganttBarId){
            //first remove the entry from the visible array
            var index = this.indexOfInArray(this._visibleGanttBarArray, ganttBarId);
            if (index >= 0) {
                for (var i = index + 1; i < this._visibleGanttBarArray.length; i++) {
                    this._visibleGanttBarArray[i - 1] = this._visibleGanttBarArray[i];
                }
                //remove the last item
                this._visibleGanttBarArray.pop();
				this.removeBackgroundRow();
            }
            else {
                console.log("ROW NOT FOUND IN VISIBLE ARRAY AND ATTEMPT WAS MADE TO REMOVE FROM IT");
            }
        },
        
        /**
         *  _incrVisibleGanttBarIndex : Reindex all the affected bars after any expand/insertion starting from rowIndex + 1
         *  Incoming : row Index
         *  Outgoing : none
         *
         */
        _incrVisibleGanttBarIndex: function(rowIndex){
            for (var i = rowIndex + 1; i < this._visibleGanttBarArray.length; i++) {
                var ganttBar = dijit.byId(this._visibleGanttBarArray[i]);
                if (ganttBar) {
                    ganttBar.setIndex(ganttBar.index + 1);
                    ganttBar.updateBar();
                }
            }
        },
        
        /**
         *  _decrVisibleGanttBarIndex : Reindex all the affected bars after any collapse/deletion
         *  Incoming : row index
         *  Outgoing : none
         *
         */
        _decrVisibleGanttBarIndex: function(rowIndex){
            for (var i = rowIndex + 1; i < this._visibleGanttBarArray.length; i++) {
                var ganttBar = dijit.byId(this._visibleGanttBarArray[i]);
                if (ganttBar) {
                    ganttBar.setIndex(ganttBar.index - 1);
					if(ganttBar.index == 0)
						ganttBar.domNode.setAttribute('tabIndex', "0");
                    //ganttBar.updateBar();
                }
            }
        },
        
        handleError: function(errorObject){
            //TODO: Assign codes to errors		
            this._displayError(errorObject);
        },
		
        _displayError: function(errorObject){
	      this._displayAndClearErrorMessage(errorObject.errorMessage, errorObject.errorCode);
        },
        
        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },

        /*
         * Remove the time zone data from the Date object 
         * incoming : Date Object
         * OutGoing : parsed Date without timezone
         */		
		removeTimeZoneFromDate: function(dateObject) {
			var formatString = dateUtil.formatDateForSave(dateObject);
			var parsedDate = dateUtil.parseDate(formatString);
			return parsedDate;
		},
		
		/*
         * Method used to handle keyboard events from the gantt.         
         */
        _onKeyDownFromGantt: function(event){
			if (this._controller) {
				// Check is the view is active
				if (!this._controller.isViewActive()) {
					return;
				}

				var tree = dijit.byId(Constants.WBS_TREE_ID);
            	if (tree &&
					tree._containingWidget &&
					tree._containingWidget.newDlgOpen) {
						return;					
				}

				// Process the short cut key events only if the tree/grid are not in edit state
				var isEditing =  this._controller.isEditInProgress();
				if (isEditing) {
					return;
				}
				
				// Ignore the event if the baseline or resolve dependency dialog is already open
				if ((this.baselineDlgOpen) || (this.resolveDepDlgOpen) || (this.closeButtonHandler)) {
					return;
				}
			}
			 
			if ((event.shiftKey) && (!event.altKey) && (!event.ctrlKey)) {

				if ((event.keyChar === "d") ||
				    (event.keyChar === "D")) {

					// Add Dependency: Shift + d
					this._startStopDependencyCommand();
				} else if ((event.keyChar === "x") ||
						   (event.keyChar === "X")) {
						   	
					// Remove Dependency: Shift + x
					this._deleteDependencyCommand();
				} else if ((event.keyChar === "r") ||
						   (event.keyChar === "R")) {
						   	
					// Resolve Dependency: Shift + r
					this._resolveDependency();
				} else if ((event.keyChar === "c") ||
						   (event.keyChar === "C")) {
						   	
					// Calculate critical path: Shift + c
					this.criticalPathButton.checked = !this.criticalPathButton.checked;
					this._handleCriticalPathButton();
				} else if ((event.keyChar === "b") ||
						   (event.keyChar === "B")) {
						   	
					// Show baseline: Shift + b					
					this._baseline();
				} else if ((event.keyChar === "t") ||
						   (event.keyChar === "T")) {
						   	
					// GotoToday: Shift + t
					this._gotoToday();
				}
				else if ((event.keyChar === "f") ||
						 (event.keyChar === "F") && (this._viewlet)){
					
					// Refresh viewlet: shift + F
					this._viewlet.confirmAndRefreshViewlet();		 	
				}
			} 	

			if ((!event.altKey) && (!event.ctrlKey) && (!event.shiftKey)) {
				// Ignore key down from tree
				if ((this._viewlet) && (this._viewlet._onKeyDownFromTree)) {
					this._viewlet._onKeyDownFromTree = false;
					return;
				}
			}
		},
	
		
        /*
         * Display and clear the error message 
         * incoming : Error Message Object
         * OutGoing : None
         */
        _displayAndClearErrorMessage: function(errorMessage, errorCode) {
            var self = this;
            EPMErrorHandler.setError(errorMessage, errorCode);
            
            // Set an interval and clear the error message
            var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);  
        }
    });
})();
