/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/***************************************************************************************************************
 
 Approver Timesheet Grid Widget - List all the resources working under the logged in project manager
 
 ****************************************************************************************************************/
(function(){

    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.ProjectManagerTimesheetGrid");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGrid");
    dojo.require("com.ibm.epm.web.core.ui.internal.DojoData");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.ProjectManagerTimesheetGridRowData");
    dojo.require("com.ibm.epm.timesheet.web.client.internal.TimesheetClient");
    dojo.requireLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
	dojo.require("com.ibm.epm.web.core.client.QueryUtil");
    
    dojo.require("dojox.grid.compat._data.editors");
    dojo.require("dojox.grid.compat._data.dijitEditors");
    dojo.require("dojo.data.ItemFileWriteStore");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    
    var EPMGrid = com.ibm.epm.web.core.ui.internal.EPMGrid;
    var ProjectManagerTimesheetGridRowData = com.ibm.epm.timesheet.web.ui.internal.ProjectManagerTimesheetGridRowData;
    var TimesheetClient = com.ibm.epm.timesheet.web.client.internal.TimesheetClient;
    var TimesheetViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var QueryUtil = com.ibm.epm.web.core.client.QueryUtil;
    
    dojo.declare("com.ibm.epm.timesheet.web.ui.internal.ProjectManagerTimesheetGrid", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/templates/ProjectManagerTimesheetGrid.html"),
        
        containerViewlet: null,
        
        /*
         * Delete all the objects created for better garbage collection
         */
        _cleanup: function(){
        
            if (this._resourceGrid) {
            
                if (this._projectManagerGridContent.firstChild) {
                    this._projectManagerGridContent.removeChild(this._resourceGrid.domNode);
                }
                this._resourceGrid.destroy();
                delete this._resourceGrid;
                if (this._resourceGridStore) {
                    delete this._resourceGridStore;
                }
            }
        },
        
        /*
         * When the widget is first instantiated, this function is called first.
         */
        postCreate: function(){
        
            this.resourceName = null;
            this.resourceProjectMap = {};
			this._totalServerCalls = 0;
			this._serverCallCounter = 0;
            this._loadResourceDetails();
        },
        
        /*
         * Change the state of the viewlet busy status.
         */
        _toggleBusyStatus: function(state){
            if (this.containerViewlet.toggleBusyStatusOfViewlet) {
                this.containerViewlet.toggleBusyStatusOfViewlet(state);
            }
        },
        
        /*
         * On clicking the row of the project manager grid, the appropriate timesheet has to be
         * shown of the corresponding user selected.
         * Project manager grid shows the list of resources working under him.
         */
        _handleRowClickEvent: function(event){
        
			if(this.containerViewlet.switchResourceTimesheet){
				
				this.containerViewlet.switchResourceTimesheet(event);
			}
        },
		
		/**
		 * 
		 * @param {Object} rowIndex
		 * 
		 * Clear all the selection and select the row specified
		 */
		clearAndSelectRow: function(rowIndex){
			
			this._resourceGrid.selection.clear();
			this._resourceGrid.selection.select(rowIndex);
		},
		
		/**
		 * 
		 * @param {Object} href
		 * 
		 * Returns the row index of the user href.
		 */
		getRowIndex: function(href){
			return this._resourceGrid.model.getRowByIdentity(href);
		},
		
		
		/**
		 * 
		 * @param {Object} href
		 * 
		 * Returns the list of projects of the selected resource
		 */
		getProjectList: function(href){
			return this.resourceProjectMap[href];
		},
        
		/**
		 * 
		 * @param {Object} rowIndex
		 * 
		 * Returns the row object of the row selected in the project manager grid.
		 */
        getRow: function(rowIndex){
        
            if (this._resourceGrid) {
                if (this._resourceGrid.model !== null) {
                    return this._resourceGrid.model.getRow(rowIndex);
                }
            }
        },
        
		/**
		 * Get the list of all the resources with details working under the logged in project manager
         * and display in the grid.
		 */
        _loadResourceDetails: function(){
            this._toggleBusyStatus(true);
            this._getAllProjects();
        },
        
        /*
         * Get the list of all projects of a currently logged in approver.
         * The server side will take care of filtering the list of projects
         * depending upon the logged in user.
         * The URI to get the list of projects is: "/epm/resources/projects/".
         * The business unit (Tara) is hard coded now..
         * TODO: to remove the hard coded business unit.
         */
        _getAllProjects: function(){
        
            var self = this;
            this._toggleBusyStatus(true);
            
            var serviceObj = {
            
                success: function(projectList){
                    if (!projectList || projectList.length === 0) {
                        self._toggleBusyStatus(false);
                        self.containerViewlet._displayErrorMessage(TimesheetViewletMessages.PROJECT_LIST_EMPTY);
                        return;
                    }
                    
                    else {
						self._totalServerCalls = 1;
						
						var projectAreaList = [];
						
						for (var i in projectList) {
							
							var titleObj = dojo.fromJson(projectList[i].title);
							projectAreaList.push(titleObj.projectArea);
							
						}
						
						self._getProjectAssignmentsUsingQuery(projectAreaList);
                        
                    }
                },
                failure: function(errorObj){
                    self._toggleBusyStatus(false);
                    self.containerViewlet._displayErrorMessage(errorObj.errorMessage);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            TimesheetClient.getAllProjects(srh, "get");
        },
        
        /*
         * Get all the project assignments.
         * The project assignments contains the list of resources
         * assigned to the project.
         */
        _getProjectAssignmentsUsingQuery: function(projectList){
        
            var self = this;
            this._toggleBusyStatus(true);
            
            var serviceObj = {
            
                success: function(projectAssignments){
                    if (!projectAssignments) {
                        self._toggleBusyStatus(false);
						self._serverCallCounter++;
                        return;
                    }
                    else {
						
						var queryResult = projectAssignments;
						
						if (queryResult !== null) {
						
							for (var i = 0; i < queryResult.length; i++) {
								
								var projectAssignmentDetail = queryResult[i].projectAssignment;
						
			                    if (!projectAssignmentDetail || !projectAssignmentDetail[TimesheetConstants.PROJECT_RESOURCE_ASSIGNED] || !projectAssignmentDetail[TimesheetConstants.PROJECT_ASSIGNED]) {
									break;
			                    }
			                    else {
			                        var resourceHref = projectAssignmentDetail[TimesheetConstants.PROJECT_RESOURCE_ASSIGNED].relativeHref;
									var resourceTitle = projectAssignmentDetail[TimesheetConstants.PROJECT_RESOURCE_ASSIGNED].title;
			                        var title = projectAssignmentDetail[TimesheetConstants.PROJECT_ASSIGNED].title;
									var titleObj = dojo.fromJson(title);
									var projectName = titleObj.summary;
									var projectAreaId = titleObj.projectArea;
			                        
			                        if (self.resourceProjectMap[resourceHref]) {
			                            self.resourceProjectMap[resourceHref].push(projectAreaId);
			                        }
			                        else {
			                            self.resourceProjectMap[resourceHref] = new Array();
			                            self.resourceProjectMap[resourceHref].push(projectAreaId);
			                            self._buildResourceGrid(resourceHref,resourceTitle);
			                        }
			                    }
							}
						}
						self._toggleBusyStatus(false);
						self._serverCallCounter++;
                    }
                },
                failure: function(errorObj){
					self._serverCallCounter++;
                    self._toggleBusyStatus(false);
                    self.containerViewlet._displayErrorMessage(errorObj.errorMessage);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            TimesheetClient.getProjectAssignmentUsingQuery(srh, projectList, "post");
            
        },
		
        /*
         * Once the resource is available, add the resource to the grid.
         * If the resource is already present in the grid, then
         * update the list of projects for the resource in the grid.
         */
        _buildResourceGrid: function(resourceHref,resourceTitle){
        
            var resourceItem = new ProjectManagerTimesheetGridRowData(resourceHref);
            resourceItem.name = resourceTitle;//resourceHref.substring(resourceHref.lastIndexOf("/") + 1, resourceHref.length);
            
            var self = this;
            
            if (!this._resourceGridStore) {
            
                var item = new Array(1);
                item[0] = resourceItem;
                var gridData = {
                    identifier: 'href',
                    items: item
                };
                
                this._resourceGridStore = new dojo.data.ItemFileWriteStore({
                    data: gridData
                });
                this._resourceGridStore.save();
                this._createResourceGrid(this._resourceGridStore);
            }
            else {
                if (this._resourceGrid.model._rowIdentities[resourceHref]) {
                    return;
                }
                this._resourceGridStore.newItem(resourceItem);
                this._resourceGridStore.save();
            }
        },
        
        /*
         * Build the resource grid.
         */
        _createResourceGrid: function(gridData){
        
            var model = new com.ibm.epm.web.core.ui.internal.DojoData(null, gridData);
            var row = [{
            
                name: TimesheetViewletMessages.USERS,
                width: 10,
                field: "name"
            }];
            
            var view = {
                cells: [row]
            };
            
            // a grid layout is an array of views.
            var layout = [view];
            this._resourceGrid = new EPMGrid({
                title: TimesheetViewletMessages.RESOURCE_GRID_TITLE,
                model: model,
                autoWidth: true,
                autoHeight: true,
                structure: layout,
				overRideCellClasses: false
            });
            
            dojo.connect(this._resourceGrid, 'onRowClick', this, this._handleRowClickEvent);
			dojo.connect(this._resourceGrid, EPMGridPubSub.ON_KEY_DOWN, this, this._KeyDown);
			dojo.connect(this._resourceGrid, 'onCellMouseOver',this, this._onCellMouseOver);
            
            this._projectManagerGridContent.appendChild(this._resourceGrid.domNode);
			
			this._resourceGrid.selection.multiSelect = false;
            this._resourceGrid.render();
            this._resourceGrid.refresh();
			
        },
		
		_onCellMouseOver: function(event)
		{
			var publishObj = event;
            if (publishObj) {
				//This works in IE.
				var rowData1 = this.getRow(publishObj.rowIndex);
				var userid = rowData1.href.substring(rowData1.href.lastIndexOf("/") + 1, rowData1.href.length);
				if(publishObj.toElement){
					if(dojo.trim(publishObj.toElement.innerText) != ""){
						publishObj.cellNode.title = publishObj.toElement.innerText+" ("+userid+")";
					}else {
						publishObj.cellNode.title = publishObj.cell.name+" ("+userid+")";
					}
				}
		         else if (publishObj.cellNode) {
				 	if (dojo.trim(publishObj.cellNode.textContent) != "") {
						publishObj.cellNode.title = publishObj.cellNode.textContent+" ("+userid+")";
					}else {
						publishObj.cellNode.title = publishObj.cell.name+" ("+userid+")";
					}
				}				
            }            
		},
		
        /*
         * Just a place holder to do big things which have not been encountered yet. :-)
         */
        update: function(){
        
        },
		
		/* Control comes here on grid key down event */
		_KeyDown: function(e){
			var keys = dojo.keys;
			if(e.keyCode == keys.ENTER)
        		this._handleRowClickEvent(e);
        }
        
    });
    
})();
