/**
 * @author paul
 */
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.risk.web.ui.internal.RiskActionsGridView");
	dojo.require("com.ibm.epm.web.core.grid.controller.EPMRiskActionsGridController");
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskActionsGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskConstants");
	dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
	dojo.require("dijit.Menu");
	dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
	// Localization
    dojo.requireLocalization("com.ibm.epm.risk.web.ui", "Messages");
    
    var RiskConstants = com.ibm.epm.risk.web.ui.internal.RiskConstants;
    var ControllersFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var RiskMessages = dojo.i18n.getLocalization("com.ibm.epm.risk.web.ui", "Messages");
	var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
	var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    var strategy = null;
	var rownode = null;
	var strategyType = RiskConstants.TYPE_MITIGATION;
	var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
	
	dojo.declare("com.ibm.epm.risk.web.ui.internal.RiskActionsGridView", abstractView, {
        templateString: '<div class="com-ibm-epm-risk-web-ui-internal-RiskActionsGridView" dojoAttachPoint="_RiskActionsGridViewAttachPoint"></div>',
        
        /* 
         * buildRatesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with RiskGrid.
         */
        buildRiskActionsGridView: function(){
			this._rowHoverArray = [];
        	this._gridID = RiskConstants.RISK_ACTIONS;
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.risk.web.ui.internal.RiskActionsGridModel({_containingGrid: this});
            }
			
			if (!this._controller) {
				this._controller = ControllersFactory.getEPMRiskActionsGridController(RiskConstants.RISK_ACTIONS_GRID_CONTROLLER);
			}
            this.buildGrid(this, this._gridModel, this._controller);
        },

        /*
         * This is name the Grid.
         */
        getGridTitle: function(){
            return RiskMessages.RISK_ACTIONS_GRID_TITLE;
        },
        /*
         * This is unique ID of the Grid.
         */
        getGridInternalID: function(){
            return RiskConstants.RISK_ACTIONS;
        },
        /*
         * This is called by controller. Returns the Attach point to the controller.
         */
        getAttachpoint: function(){
            return this._RiskActionsGridViewAttachPoint;
        },
        
        
		/**
         *  postCreate : postCreate for the viewlet.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        postCreate: function(){
			this.strategy = null;
			this.rownode = null;
			this.strategyType = RiskConstants.TYPE_MITIGATION;
        },
		
		/**
		 * This method is called by the framework, after the grid is created.
		 * Subscribes to the required events 
		 */
		createView: function(){
            this.inherited('createView', arguments);
			dojo.subscribe(GridEvents.FIELD_CHANGED, this, this.updateActionsInModel);
			this._pconnects.push(dojo.connect(this._generalGrid, "onRowMouseOver", this, this._onMouseOver));
            this._pconnects.push(dojo.connect(this._generalGrid, "onRowMouseOut", this, this._onMouseOut));
			this._pconnects.push(dojo.connect(this._generalGrid, EPMGridPubSub.ON_CELL_CLICK, this, this.onCellClick));
			this._pconnects.push(dojo.connect(this._generalGrid, "onStartEdit", this, this.onStartEdit));
			this._pconnects.push(dojo.connect(this._generalGrid, "onApplyCellEdit", this, this.onApplyCellEdit));
		},

		/**
		 * Save the current values for the edited cell, locally, to be able to revert to this 
		 * value if the validation fails in 'onApplyCellEdit'
		 */
		onStartEdit: function(inCell, inRowIndex){
			var row = this._generalGrid.model.getRow(inRowIndex);
			if (row) {
				this.curCellField = inCell.field;
			}
		},
		
		/*
         * Override the function defined in EPMGrid.
         * Make action summary and duration uneditable if the work item is already created
         */
        isCellEditable: function(event) {			
			var rowIndex = parseInt(event.rowIndex, 10);
			if (isNaN(rowIndex)) {
			    return true;
			}
            var row = this._generalGrid.model.getRow(rowIndex);
			if (row && row.workitem) {
				return false;
			}
	        return true;
        },
		
		/**
		 * Validate the fields for valid values, if the validation fails revert to the previous value,
		 * which was saved in 'onStartEdit'
		 */
		onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
			//IE Issue : For the first edit the 0th index is undefined
			if (this._generalGrid && (this._generalGrid._click[0] || this._generalGrid._click[1])) {
				var row = this._generalGrid.model.getRow(inRowIndex);
				if (row) {
					var cellFieldName = this.curCellField;
					var item = this._generalGrid.model.store._getItemByIdentity(row.id);
					if (item) {
						if (cellFieldName === RiskConstants.RISK_ACTION_FIELD_SUMMARY) {
							inValue = inValue ? inValue.replace(/^\s*/, "").replace(/\s*$/, "") : inValue;
							if (!inValue || inValue.length === 0) {
								this._generalGrid.edit.cancel();
							}
						}else if(cellFieldName === RiskConstants.RISK_ACTION_FIELD_COST){
							var min = RiskConstants.RISK_COST_MIN;
							var max = RiskConstants.RISK_COST_MAX;
							if(!inValue || inValue < min || inValue > max ){
								this._generalGrid.edit.cancel();
							}
							
						}else if(cellFieldName === RiskConstants.RISK_ACTION_FIELD_DURATION){
							var mind = RiskConstants.RISK_ACTION_FIELD_DURATION_MIN;
							var maxd = RiskConstants.RISK_ACTION_FIELD_DURATION_MAX;
							if(!inValue || inValue < mind || inValue > maxd ){
								this._generalGrid.edit.cancel();
							}
						}
					}
				}
			}
		}, 
        /**
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var RiskActionsGridLayout = [{
                noscroll: false,
                defaultCell: {
                    styles: 'text-align: left;'
                },
                rows: [[{
                    name: RiskMessages.RISK_ACTION_NAME,
                    width: '20em',
                    field: RiskConstants.RISK_ACTION_FIELD_SUMMARY,
                    editor: dojox.grid.editors.Input,
					title: RiskMessages.RISK_ACTION_NAME_TITLE
                },
				{
                    name: RiskMessages.RISK_ACTION_FIELD_COST,
                    width: '5em',
                    field: RiskConstants.RISK_ACTION_FIELD_COST,
                    styles: 'vertical-align: top;',
                    editor: dojox.grid.editors.Dijit,
					constraint: {currency: 'USD'},
					editorClass: "dijit.form.CurrencyTextBox",
					title: RiskMessages.RISK_ACTION_COST_TITLE
                },
				{
                    name: RiskMessages.RISK_ACTION_DURATION,
                    width: '5em',
                    field: RiskConstants.RISK_ACTION_FIELD_DURATION,
                    styles: 'vertical-align: top;',
                    editor: dojox.grid.editors.Dijit,
					constraint: {min:1,max:200,places:0},
                	editorClass: "dijit.form.CurrencyTextBox",
					formatter:this.durationFormatter,
					title: RiskMessages.RISK_ACTION_DURATION_TITLE
                },
				{
                    name: RiskMessages.RISK_ACTION_WORKITEM_ID,
                    width: '6em',
                    field: RiskConstants.RISK_ACTION_FIELD_WORK_ITEM,
                    styles: 'vertical-align: top;',
					get: this.customColumn,
					formatter:this.emptyFormatter,
					title: RiskMessages.RISK_ACTION_WORKITEM_ID_TITLE
                },
				{
	                name: RiskMessages.RISK_ACTION_OPERATIONS,
	                width: 'auto',
	                field: RiskConstants.RISK_ACTION_FIELD_ACTION,
					styles: 'vertical-align: top;',
					value: "",
					title:RiskMessages.RISK_ACTION_OPERATIONS_TITLE
                }
				]]
            }];
            return RiskActionsGridLayout;
        },
		
		
		/**
		 * emptyFormatter: This function is just a dummy formatter placed to avoid the fix for defect 1817
		 * Incoming Argument: inDatum
		 * Outgoing Argument: inDatum
		 */
		emptyFormatter: function(inDatum){
			return inDatum;
		},
		/**
		 * Formatter for the duration field in the grid.
		 * @param {Object} inDatum
		 */
		durationFormatter: function(inDatum){
			return (inDatum && inDatum !== "" && inDatum > 0) ? inDatum : 1;
		},
		/**
		 * This returns the editor for the workitem column in the grid. 
		 * If a work item exists it displays the icon and the id,
		 * if a work item type is selected for creation, displays only an icon
		 * else this field is blank
		 */
		customColumn: function(inRowIndex){
			var data1 = this.grid.model.getRow(inRowIndex);
			if (!data1 || (data1.id.indexOf(RiskConstants.NEW_ELEMENT_PREFIX) >= 0)) {
				return; // Don't act before the data exists
			}
			var uriId = "";
			if (data1.workItemType && data1.workItemType.length > 0){
				var title=null;
				if (data1.workitem && data1.workitem.length > 0 && data1.workitem.lastIndexOf) {
					var index = data1.workitem.lastIndexOf("/");
					if (index > 0) {
						uriId = data1.workitem.substring(index + 1, data1.workitem.length);
						title = data1.workItemType + " " + uriId;
					}
				}
				else 
					if (data1.createtask && data1.createtask.length > 0) {
						title = RiskMessages.RISK_WI_CREATE + data1.workItemType + RiskMessages.RISK_WI_ONSAVE;
					}
				if(title){
					return '<img class="'+RiskConstants.RISK_ACTION_WI_MENU_CLASS+'" title="' + title + '" alt="' + title	+ '" src ="' +data1.workItemTypeIcon +'" />' + uriId;
				}
			}
			return uriId;
		},
	
	
         /**
         *  UpdateActions : This function is to update the grid row.
         *  Incoming Arguments : The modofied row values is paased.
         *  Outgoing Argument : none
         *
         */
        UpdateActions: function(data, rownd){
			this.rownode = rownd;
			if (this._controller && data) {
				this.strategyType = data.strategytype;
				this.strategy = data;
				this.refresh();
			}
        },
		
		refresh: function(){
			this.removeAllRows();
			this._controller.actions = [];
			if(this.pMenu){
				this.disconnect(this._menuCloseOnBody);
				this.disconnect(this._menuCloseOnEsckey);
				this.pMenu.destroy();
				this.pMenu = null;
			}
			if (this.strategy && this.strategy.actions && this.strategy.actions.length) {
				for (var i = 0; i < this.strategy.actions.length; i++) {
					if (!this.strategy.actions[i].deleted) {
						if (this.strategy.actions[i].task && this.strategy.actions[i].task[0]) {
							this.strategy.actions[i].workitem = this.strategy.actions[i].task[0].relativeHref;
							var titleObj = dojo.fromJson(this.strategy.actions[i].task[0].title);
							var ind = dojo.indexOf(this.containerViewlet.wiTypeIds, titleObj.workItemType);
							this.strategy.actions[i].summary = titleObj.summary;
							if (ind >= 0) {
								this.strategy.actions[i].workItemType = this.containerViewlet.wiTypes[ind];
								this.strategy.actions[i].workItemTypeIcon = this.containerViewlet.wiTypeIcons[ind];
							}
						}
						this._controller.actions.push(this.strategy.actions[i]);
					}
				}
				
			}
			this.buildGrid(this, this._gridModel, this._controller);
			if (this._generalGrid) {
				this._generalGrid.selection.unselectAll();
			}
		},
		
		/**
         *  getStartegyIndex : This goes through the list of strategies and gets the index of the strategy 
         *  for whcih the grid has loaded actions.
         *
         */
		getStartegyIndex: function(){
			if (this.rownode) {
				var strategies = null;
				if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
					strategies = this.rownode.mitigationstrategy;
				}
				else {
					strategies = this.rownode.contingencyplan;
				}
				
				if (strategies && strategies.length >= 0 && this.strategy) {
					for (var i = 0; i < strategies.length; i++) {
						if (strategies[i].id == this.strategy.id) {
							return i;
						}
					}
				}
			}
			return -1;
        },
		
		/**
         *  getaActionIndex : This goes through the list of actions and gets the index of the action 
         *  for the given id
         *
         */
		getaActionIndex:function(actions, id)
		{
			if (actions && actions.length >= 0) {
				for (var j = 0; j < actions.length; j++) {
					if (actions[j].id == id) {
						return j;
					}
				}
			}
			return -1;
		},
		
		/**
         *  updateActionsInModel : This goes updates the action field in the model for the 
         *  top grid. Goes through the list of 
         *
         */
		updateActionsInModel: function(data){
			if (data.controllerID == RiskConstants.RISK_ACTIONS_GRID_CONTROLLER) {
				var indx = this.getStartegyIndex();
				if (indx >= 0) {
					if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
						var indy = this.getaActionIndex(this.rownode.mitigationstrategy[indx].actions, data.row.id);
						if ((data.fieldName == "summary") && (this.rownode.mitigationstrategy[indx].actions[indy].summary != data.newValue)) {
							var inValue = data.newValue;
							inValue = inValue ? inValue.replace(/^\s*/, "").replace(/\s*$/, "") : inValue;
							if(inValue.length > 0){
								this.rownode.mitigationstrategy[indx].actions[indy].modifiedsummary = inValue;
								this.rownode.mitigationstrategy[indx].actions[indy].summary = inValue;	
							}
						}
						else 
							if ((data.fieldName == "cost") && (this.rownode.mitigationstrategy[indx].actions[indy].cost != data.newValue)) {
								this.rownode.mitigationstrategy[indx].actions[indy].modifiedcost = data.newValue;
								this.rownode.mitigationstrategy[indx].actions[indy].cost = data.newValue;
							}
						else 
								if ((data.fieldName == "duration") && (this.rownode.mitigationstrategy[indx].actions[indy].duration != data.newValue)) {
									this.rownode.mitigationstrategy[indx].actions[indy].modifiedduration = data.newValue;
									this.rownode.mitigationstrategy[indx].actions[indy].duration = data.newValue;
								}
					}
					else {
						var indz = this.getaActionIndex(this.rownode.contingencyplan[indx].actions, data.row.id);
						if ((data.fieldName == "summary") && (this.rownode.contingencyplan[indx].actions[indz].summary != data.newValue)) {
							var inValue = data.newValue;
							inValue = inValue ? inValue.replace(/^\s*/, "").replace(/\s*$/, "") : inValue;
							if (inValue.length > 0) {
								this.rownode.contingencyplan[indx].actions[indz].modifiedsummary = inValue;
								this.rownode.contingencyplan[indx].actions[indz].summary = inValue;
							}
						}
						else 
							if ((data.fieldName == "cost") && (this.rownode.contingencyplan[indx].actions[indz].cost != data.newValue)) {
								this.rownode.contingencyplan[indx].actions[indz].modifiedcost = data.newValue;
								this.rownode.contingencyplan[indx].actions[indz].cost = data.newValue;
							}
							else 
								if ((data.fieldName == "duration") && (this.rownode.contingencyplan[indx].actions[indz].duration != data.newValue)) {
									this.rownode.contingencyplan[indx].actions[indz].modifiedduration = data.newValue;
									this.rownode.contingencyplan[indx].actions[indz].duration = data.newValue;
								}
					}
				}
			} 
        },
		
        /**
         *  UpdateRow : This function is to update the grid row.
         *  Incoming Arguments : The modofied row values is paased.
         *  Outgoing Argument : none
         *
         */
        UpdateRow: function(data){
			if (this._generalGrid) {
				var data1 = this._generalGrid.model.getRow(data.rowIndex);
				data1.summary = data.summary;
				data1.cost = data.cost;
				data1.duration = data.duration;
			}
        },

		/*
         * This is to add a new row in the grid.
         *
         */
		addRiskAction: function(){
			if (this.strategy) {
                var rowID = this._controller.getUniqueRowID("_unsaved");
				
				var rows = {
					"relativeHref": rowID,
					"summary": RiskMessages.NEW_RISK_ACTION,
					"duration": 1,
					"cost": 0,
					"workitem": ""
				};
				this.insertActionsInModel(rows);
				this._controller._insertElement(rows, null, false);
				this.scrollToSelectedRow();
				this.startFirstCellEdit();
			}
        },
		
		/**
		 * When new risk action is added. Select 1st cell text in editable mode.
		 */
		startFirstCellEdit: function() {
		    if (this._generalGrid) {
				var rowIndex = this._generalGrid.selection.selectedIndex;
				var cell = this._generalGrid.getCell(0);
				
				if (rowIndex >= 0 && cell) {
					this._generalGrid.focus.setFocusCell(cell, rowIndex);
					this._generalGrid.edit.setEditCell(cell, rowIndex);
				}else {
					console.log("startFirstCellEdit: no selected row to edit");
				}
			}	
		},
		
		/**
         *  insertActionsInModel : Adds a new entry for the risk action in the model for the top grid, when a
         *  new action is added in the actions grid 
         *
         */
		insertActionsInModel: function(row){
			var indx = this.getStartegyIndex();
			if (indx >= 0) {
				var newrow = {
					"cost": row.cost,
					"duration": row.duration,
					"id": row.relativeHref,
					"summary": row.summary,
					"modifiedsummary": row.summary,
					"modifiedcost": row.cost,
					"modifiedduration": row.duration
				};
				if (this.strategyType === RiskConstants.TYPE_MITIGATION) {
					if (!this.rownode.mitigationstrategy[indx].actions || this.rownode.mitigationstrategy[indx].actions.length === 0) {
						this.rownode.mitigationstrategy[indx].actions = [];
					}
					this.rownode.mitigationstrategy[indx].actions.push(newrow);	
					if (this._controller) {
						this._controller.actions = this.rownode.mitigationstrategy[indx].actions;
					}			
				}
				else {
					if (!this.rownode.contingencyplan[indx].actions || this.rownode.contingencyplan[indx].actions.length === 0) {
						this.rownode.contingencyplan[indx].actions = [];
					}
					this.rownode.contingencyplan[indx].actions.push(newrow);
					if (this._controller) {
						this._controller.actions = this.rownode.contingencyplan[indx].actions;
					}	
				}
			}
		},
		
		/**
         *  deleteRiskAction : Throws a Delete Confirmation and then calls delete
         *
         */
		deleteRiskAction: function(event){
			var dialogContent;
			var buttonlist = [{
				label: RiskMessages.YES,
				action: dojo.hitch(this, this.deleteActions)
			}, {
				label: RiskMessages.NO
			}];
			dialogContent = RiskMessages.CONFIRM_ACTION_DELETE_MESSAGE;
			
			var labelList = [{
				label: dialogContent
			}];
			
			this.confirmDialog = new com.ibm.epm.web.core.ui.internal.EPMConfirmDialog({
				labelList: labelList,
				title: RiskMessages.DIALOG_TITLE,
				buttonlist: buttonlist
			});
			this.confirmDialog.show();
		},
		
		/**
         *  deleteActions : Deletes the action from the Actions grid
         *  and updates the model.
         *
         */
		deleteActions: function() { 
		    var workitemRisk = [];  
			var error = null;
			var errorMesg = null ;
			var _gridData = null;
			var rowToBeDeleted = [];
            // Get selected Rows and then call delete on them in a loop
            if (!this._generalGrid || !this._controller){return;}
			var selectedRows = this._generalGrid.selection.getSelected();
			if(selectedRows && selectedRows.length > 0) {
				for (var i = 0; i < selectedRows.length; i++) {
					var row = this._controller.getRowByIndex(selectedRows[i]);
					if( row.workitem ){
						errorMesg = dojo.string.substitute(RiskMessages.DELETE_STRATEGY_WITH_ACTION_WORKITEM);
						workitemRisk.push(errorMesg);
						continue;
					}
					var indx = this.getStartegyIndex();
					if (row && indx >= 0) {
						if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
							var indy = this.getaActionIndex(this.rownode.mitigationstrategy[indx].actions, row.id);
							if(row.id.indexOf(RiskConstants.NEW_ELEMENT_PREFIX) >= 0 ){
								this.rownode.mitigationstrategy[indx].actions.splice(indy,1);
							}else{
							
							this.rownode.mitigationstrategy[indx].actions[indy].deleted = true;
							this.containerViewlet.toggleSaveButton(false);
							}
						}
						else {
							var indz = this.getaActionIndex(this.rownode.contingencyplan[indx].actions, row.id);
							if(row.id.indexOf(RiskConstants.NEW_ELEMENT_PREFIX) >= 0 ){
								this.rownode.contingencyplan[indx].actions.splice(indz,1);
							}else{
								this.rownode.contingencyplan[indx].actions[indz].deleted = true;
								this.containerViewlet.toggleSaveButton(false);
							}
						}
					}
					rowToBeDeleted.push(selectedRows[i]);
				}
				
				if(this._controller){
					_gridData = this._controller.getDataModel(); 
				}
				
				this._generalGrid.selection.unselectAll();
				this._generalGrid.model.remove(rowToBeDeleted);
				
				if (_gridData) {
					_gridData.store.save();
				}
				if( workitemRisk && workitemRisk.length > 0){
					
					error = {errorMessage: workitemRisk[0]};
					this._displayError(error);
				}
			}
            this.closeDialog();
        },
        
		
		/**
		 * destroy risk confirm dialog.
		 */
		closeDialog: function(){
			this.confirmDialog.hide();
			this.confirmDialog.destroy();
		},
        /*
         * This is get the first row of the grid.
         *
         */
        getfirstRow: function(){
            var data1;
            if (!this._controller) {
                this._controller = ControllersFactory.getEPMRiskActionsGridController(RiskConstants.RISK_ACTIONS_GRID_CONTROLLER);
            	data1 = this._controller.getRowByIndex(0);
            }
            else {
                data1 = this._controller.getRowByIndex(0);
            }
            return data1;
        },
		
		/**
         *  removeAllRows : Removes all the rows from the grid. called before updating the grid with new rows. 
         */
		removeAllRows: function(){
			try {
				if (this._controller) {
					var _gridData = this._controller.getDataModel();
					if (_gridData && this._generalGrid) {
						var count = _gridData.getRowCount();
						this._generalGrid.beginUpdate();
						for (var i = 0; i < count; i++) {
							var row = _gridData.data[0];
							if (row) {
								_gridData.remove([0]);
							}
						}
						//save the model
						_gridData.store.save();
					}
				}
			} 
			catch (Error) {
				console.log("Error during removing all rows :" + Error.message);
			}
			finally {
				if (this._generalGrid) {
					this._generalGrid.endUpdate();
				}
			}
		},
		
		getSortableColums: function(){
			return {	summary:	"summary", 
						duration:	"duration", 
						cost:		"cost"		};
		},
		
		refreshRiskAction: function(){
			var strategies = null;
			if (this.rownode && this.rownode.loadedelement && this.rownode.loadedelement[0] && this.rownode.loadedelement[0].riskdetails) {
				if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
					strategies = this.rownode.loadedelement[0].riskdetails.mitigationStrategies;
				}
				else {
					strategies = this.rownode.loadedelement[0].riskdetails.contingencyPlans;
				}
			}
			var id = this.strategy.id;
			this.strategy = [];
			if (strategies && strategies.length >= 0 && this.strategy) {
				for (var i = 0; i < strategies.length; i++) {
					if (strategies[i].id == id) {
						this.strategy = dojo.clone(strategies[i]);
						break;
					}
				}
			}
			this.refresh();
		},
		
		/**
		 * This is called when the hover actions are clicked in the actions grid
		 * @param {Object} event
		 */
		onCellClick: function(event){
			if (this._generalGrid && event && event.cell.field == "action") {
				this._generalGrid.selection.unselectAll();
				this._generalGrid.selection.select(event.rowIndex);
				if (event.target.name == "deleteImage") {
					this.deleteRiskAction(event);
				}
				else 
					if (event.target.name == "createImage") {
						this.createWorkItem(event);
					}
			}
			event.stopPropagation();
			event.preventDefault();
			dojo.stopEvent(event);
		},
		
		 /**
         * Private Method
         * _MouseOut: Handle the on mouse over event on the grid row and add the action images to action column
         * Input: event e
         * outPut: None
         */
        _onMouseOver: function(e){
            if (!this._generalGrid || (e.rowIndex < 0) || (this._generalGrid.edit && this._generalGrid.edit.isEditing()) 
			|| (e.gridID != RiskConstants.RISK_ACTIONS)) {
				return;		//Do not process the row if an editor is open
			}
			this._clearAllActionRows();
			var row = this._generalGrid.model.getRow(e.rowIndex);
			if (row && (this._generalGrid.hasLayout() > 0)) {
				if(row.workitem && row.workitem.length > 0){
					return;
				}
				if(row.id.indexOf(RiskConstants.NEW_ELEMENT_PREFIX) >= 0) {
					var cellob = this._generalGrid.getCell(this._getColumnIndex("action"));
					var cellDomNd = cellob.getNode(e.rowIndex);
					cellDomNd.innerHTML = '<img name="deleteImage" style="margin-right:7px;cursor: pointer;" title="' 
						+ RiskMessages.REMOVE_STRATEGY_TITLE + '" alt="' + RiskMessages.REMOVE_STRATEGY_TITLE + '" src =' +
						dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/actions/action_delete.gif") + ' />';
					this._rowHoverArray.push(row.id);
					return;
				}
				var cellobj = this._generalGrid.getCell(this._getColumnIndex("action"));
				var cellDomNode = cellobj.getNode(e.rowIndex);
				cellDomNode.innerHTML = '<img name="createImage" id="createImage" text-align: overflow: visible; center; style="margin-right: 4px; cursor: pointer;" title="' +
					RiskMessages.RISK_ACTION_CREATE_WORKITEM_TITLE + '" alt="' + RiskMessages.RISK_ACTION_CREATE_WORKITEM_TITLE 
					+ '" src =' + dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/create.gif") +
						' />&nbsp<img name="deleteImage" style="margin-right:7px;cursor: pointer;" title="' 
					+ RiskMessages.REMOVE_STRATEGY_TITLE + '" alt="' + RiskMessages.REMOVE_STRATEGY_TITLE + '" src =' +
					dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/actions/action_delete.gif") + ' />';
				this._rowHoverArray.push(row.id);
			}
        },
        
        /*
         * Private Method
         * _MouseOut: Handle the on mouse out event on the grid row and remove the action images from action column
         * Input: event e
         * outPut: None
         */
        _onMouseOut: function(e){
            if (!this._generalGrid || (e.rowIndex < 0) || (this._generalGrid.edit && this._generalGrid.edit.isEditing())
			|| (e.gridID != RiskConstants.RISK_ACTIONS)) {
                return;	//Do not process the row if an editor is open
            }
            this._clearAllActionRows();
        },
        
        /*
         * Private Method
         * _clearOneActionRow: Clear action from this row
         * Input: event e
         * outPut: None
         */
        _clearOneActionRow: function(rowId){
			if (this._generalGrid) {
				var row = this._generalGrid.model.store._getItemByIdentity(rowId);
				if (row && (this._generalGrid.hasLayout() > 0)) {
					var cellobj = this._generalGrid.getCell(this._getColumnIndex("action"));
					var rowIndex = this._controller.getRowIndex(rowId);
					var cellDomNode = cellobj.getNode(rowIndex);
					cellDomNode.innerHTML = "";
				}
			}
        },
        
        /*
         * Private Method
         * _getColumnIndex: Returns the  column index of Action Column
         * Input: Column name
         * outPut: Integer index
         */
		_getColumnIndex: function(columnName) {
			for (var i = 0; i < this._generalGrid.layout.cellCount; i++) {
				if (this._generalGrid.layout.cells[i].field == columnName) {
					return i;
				}
			}
			return this._generalGrid.layout.cellCount - 1; 
		},
        
        /*
         * Private Method
         * _clearAllActionRow: Clear all actions from this row
         * Input: event e
         * outPut: None
         */
        _clearAllActionRows: function(){
            for (var i=0 ; i < this._rowHoverArray.length; i++) {
                this._clearOneActionRow(this._rowHoverArray[i]);
            }
            this._rowHoverArray = [];
        },
		
		/**
		 * Set the parameter in the grid model to create a work item for the selected row
		 * This sets the work item type
		 * @param {Object} event
		 */
		setCreateWorkItem: function(event){
			var rowIndex = this._generalGrid.selection.selectedIndex;
			var item = this._controller.getRowByIndex(rowIndex);
			if (event.target && event.target.innerHTML) {
				if (event.target.innerHTML == RiskMessages.RISK_CREATE_NO_WI) {
					item.createtask = "";
					item.workItemType = "";
					item.workItemTypeIcon = "";
				}
				else {
					var ind = dojo.indexOf(this.containerViewlet.wiTypes, event.target.innerHTML);
					if (ind >= 0) {
						item.createtask = this.containerViewlet.wiTypeIds[ind];
						item.workItemType = this.containerViewlet.wiTypes[ind];
						item.workItemTypeIcon = this.containerViewlet.wiTypeIcons[ind];
					}
				}
				var item1 = this._generalGrid.model.store._getItemByIdentity(item.id);
				this._generalGrid.model.store._setValueOrValues(item1, RiskConstants.RISK_ACTION_FIELD_WORK_ITEM, "", true);
			}
			
			var indx = this.getStartegyIndex();
			if (indx >= 0) {
				if (this.strategyType == RiskConstants.TYPE_MITIGATION) {
					var indy = this.getaActionIndex(this.rownode.mitigationstrategy[indx].actions, item.id);
					this.rownode.mitigationstrategy[indx].actions[indy].createtask = item.createtask;
					this.rownode.mitigationstrategy[indx].actions[indy].workItemType = item.workItemType;
					this.rownode.mitigationstrategy[indx].actions[indy].workItemTypeIcon = item.workItemTypeIcon;
				}
				else {
					var indz = this.getaActionIndex(this.rownode.contingencyplan[indx].actions, item.id);
					this.rownode.contingencyplan[indx].actions[indz].createtask = item.createtask;
					this.rownode.mitigationstrategy[indx].actions[indz].workItemType = item.workItemType;
					this.rownode.mitigationstrategy[indx].actions[indz].workItemTypeIcon = item.workItemTypeIcon;
				}
			}
			this.containerViewlet.toggleSaveButton(false);
		},
		
		/**
		 * Creates the actions menu to pop up when the create wi image is clicked
		 * This loads a list of wi types as sub menus
		 */
		createPopUpMenu: function(){
			if(!this.containerViewlet || !this.containerViewlet.wiTypes || this.containerViewlet.wiTypes.length === 0){
				return null;
			}
			
			this.pMenu = new dijit.Menu({	leftClickToOpen: true	});
			var pSubMenu = new dijit.Menu();
			var self = this;
			var menu = new dijit.MenuItem({	label: RiskMessages.RISK_CREATE_NO_WI,
											title: RiskMessages.RISK_CREATE_NO_WI,
											onClick: dojo.hitch(this, self.setCreateWorkItem),
											iconClass:RiskConstants.RISK_ACTION_WI_MENU_CLASS	});
			var img = new Image();
			img.src = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/remove.gif");
			menu.iconNode.appendChild(img);
			pSubMenu.addChild(menu);
			
			pSubMenu.addChild(new dijit.MenuSeparator());
			for (var wi in this.containerViewlet.wiTypes) {
				menu = new dijit.MenuItem({	label: this.containerViewlet.wiTypes[wi],
											title: this.containerViewlet.wiTypes[wi],
											onClick: dojo.hitch(this, self.setCreateWorkItem),
											iconClass:RiskConstants.RISK_ACTION_WI_MENU_CLASS	});
				img = new Image();
				img.src = this.containerViewlet.wiTypeIcons[wi];
				menu.iconNode.appendChild(img);
				pSubMenu.addChild(menu);
			}
			
			menu = new dijit.PopupMenuItem({	label: RiskMessages.RISK_ACTION_CREATE_MENU_TITLE,
												title: RiskMessages.RISK_ACTION_CREATE_MENU_TITLE,
												iconClass:RiskConstants.RISK_ACTION_WI_MENU_CLASS,
												popup: pSubMenu	});
			img = new Image();
			img.src = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/create.gif");
			menu.iconNode.appendChild(img);
			this.pMenu.addChild(menu);
			
			this._menuCloseOnBody = this.connect(this.pMenu, "_onBlur", this._closeMenu);
			this._menuCloseOnEsckey = this.connect(this.pMenu.domNode, "onkeydown", this._keydown);
				
			return this.pMenu;
		},
		/**
		 * This is called when the user clicks the create work item icon
		 * This creates a pop up menu where the user can select work item type to create
		 * @param {Object} event
		 */
		createWorkItem: function(event){
			if (this._isMenuOpen) {
                this._closeMenu();
            }
			if (!this.pMenu) {
				this.pMenu = this.createPopUpMenu();
			}

			if(!this.pMenu){return;}

			var coords = dojo.coords(event.target, true);
			this._isMenuOpen = dijit.popup.open({	popup: this.pMenu,
								parent: event.target,
								x:coords.x,	y:coords.y,
								onExecute: dojo.hitch(this, this._closeMenu)	});
			this.pMenu.focus();
		},
		_closeMenu: function(e){
            if (e) {	dojo.stopEvent(e);	}
            dijit.popup.close(this.pMenu);
            this._isMenuOpen = false;
        },
		
		_keydown: function(e){
			var key = e.keyCode || e.charCode;
            if (key == dojo.keys.ESCAPE) {
				this._closeMenu(e);
			}
		}
		
    });
})();
