/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.risk.web.ui.internal.RiskDetails");
    
	dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGrid");
    dojo.require("com.ibm.epm.web.core.ui.internal.DojoData");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskUtil");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("dojo.data.ItemFileReadStore");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
	dojo.require("dijit.form.Textarea");
	
    
    
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskGridView");
    
    // Localization
    dojo.requireLocalization("com.ibm.epm.risk.web.ui", "Messages");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskConstants");
    
    var RiskConstants = com.ibm.epm.risk.web.ui.internal.RiskConstants;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var dateUtil = com.ibm.epm.risk.web.ui.internal.RiskUtil;
	var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var RiskMessages = dojo.i18n.getLocalization("com.ibm.epm.risk.web.ui", "Messages");
    
    dojo.declare("com.ibm.epm.risk.web.ui.internal.RiskDetails", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.risk.web", "ui/internal/templates/RiskDetails.html"),
        wbsNodeType: "",
        wbsNodeName: "",
		widgetsInTemplate: true,
        rownode: null,
        riskMatrixRanges: [20, 40, 60, 80, 100],
		riskPrecisions: [RiskConstants.LOW, RiskConstants.MEDIUM, RiskConstants.HIGH],
		updateEventHandler: null,
        
		
		 constructor: function(){
			this._categoryLabel = RiskMessages.RISKCATEGORY + RiskMessages.COLON;
			this._impactLabel = RiskMessages.IMPACT + RiskMessages.COLON;
			this._identDateLabel = RiskMessages.IDENTIFICATION_DATE + RiskMessages.COLON;
			this._probabilityLabel = RiskMessages.PROBABILITY + RiskMessages.COLON;
			this._occurredDateLabel = RiskMessages.OCCURRED_DATE + RiskMessages.COLON;
			this._precisionLabel = RiskMessages.PRECISION + RiskMessages.COLON;
			this._ownerLabel = RiskMessages.OWNER + RiskMessages.COLON;
			this._consCostLabel = RiskMessages.CONSEQUENCE_COST + RiskMessages.COLON;
			this._stateLabel = RiskMessages.STATE + RiskMessages.COLON;
			this._pbCostLabel = RiskMessages.PROBABILITY_COST + RiskMessages.COLON;
			this._priorityLabel = RiskMessages.PRIORITY + RiskMessages.COLON;
			this._exposureLabel = RiskMessages.EXPOSURE + RiskMessages.COLON;
			this.vAxisLabel = RiskMessages.PROBABILITY_AXIS_LABEL;
			this.hAxisLabel = RiskMessages.IMPACT_AXIS_LABEL;
			this.labelTwenty = RiskMessages.CHART_LABEL_TWENTY;
			this.labelFourty = RiskMessages.CHART_LABEL_FOURTY;
			this.labelSixty = RiskMessages.CHART_LABEL_SIXTY;
			this.labelEighty = RiskMessages.CHART_LABEL_EIGHTY;
			this.labelHundred = RiskMessages.CHART_LABEL_HUNDRED;
        },
		
        /**
         *  postCreate : postCreate for the widget.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        postCreate: function(){
			if (!this.updateEventHandler) {
	            this.updateEventHandler = dojo.subscribe(RiskMessages.RISKUPDATE, this, "UpdateRiskValues");
			}
			
            this._createAttributeElements();
			this._identBox.constraints.selector = 'date';
            this._identBox.constraints.formatLength = 'medium';
			this._identBox.constraints.max = new Date();
			
			this._occurredBox.constraints.selector = 'date';
            this._occurredBox.constraints.formatLength = 'medium';
			this._occurredBox.constraints.max = new Date();
			
			dojo.connect(this._consCostBox, "onBlur", dojo.hitch(this, this.onConsCostChange));
			dojo.connect(this._occurredBox, "onBlur", dojo.hitch(this, this.onOccDateChange));
			dojo.connect(this._identBox, "onBlur", dojo.hitch(this, this.onIdentDateChange));
			
			this._identCal.src = this._occurredCal.src = dojo.moduleUrl("com.ibm.epm.admin.web" ,"ui/internal/view/images/calendar.gif");
        },
        
		destroy: function(){
			this.inherited(arguments);
			if (this.updateEventHandler) {
				dojo.unsubscribe(this.updateEventHandler);
				this.updateEventHandler = null;
			}
		},
        /**
         *  _createAttributeElements : This function is to create Text, combo box, and button for various attributes.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        _createAttributeElements: function(){
            this._createImpactCombo();
            this._create_probabilityBox();
            this._createPrecisionCombo();
			this._createRiskStateCombo();
            this._createRiskPriorityCombo();
			this._createRiskCategoryBox();
			this._createRiskOwnerBox();
        },
        
		 /**
         *  getIndexForValue : Gets the index for the incoming value
         *  Incoming Arguments : value
         */
		getIndexForValue : function(value){
        	return dojo.indexOf(this.riskMatrixRanges, value);
		},
		
		/**
         *  getValueForIndex : Gets the value for the incoming index
         *  Incoming Arguments : index
         */
		getValueForIndex : function(index){
        	return this.riskMatrixRanges[index];
		},
		
		/**
         *  getIndexForPrecisionValue : Gets the index for the incoming value of precision
         *  Incoming Arguments : index
         */
		getIndexForPrecisionValue : function(value){
			return dojo.indexOf(this.riskPrecisions, value);
		},
		
		/**
         *  getValueForPrecisionIndex : Gets the value for the incoming index of precision
         *  Incoming Arguments : index
         */
		getValueForPrecisionIndex : function(index){
        	return this.riskPrecisions[index];
		},
        /**
         *  UpdateRiskValues : This function is to populate various attributes
         *  Incoming Arguments : Data from server to populate the fields.
         *  Outgoing Argument : none
         *
         */
        UpdateRiskValues: function(data){
            this.rownode = data;
			this._consCostBox.attr("value",this.rownode.consequencescost);
            this._precisionBox.selectedIndex = this.getIndexForPrecisionValue(this.rownode.precision);
			this._impactBox.selectedIndex = this.getIndexForValue(this.rownode.impact);
            this._probabilityBox.selectedIndex = this.getIndexForValue(this.rownode.probability);
			
			this._updateRiskCategoryBox();
		   
		   this._updateRiskOwnerBox();
		   
		   this._updateRiskStateBox(this.rownode.state);
		   this._updateRiskPriorityBox(this.rownode.priority);
            
            var precision = this._convertAbbrToPrecisionValue(this.rownode.precision);
            
            var impactCellIndexOnLoad = parseInt(this.rownode.impact / RiskConstants.MANIPULATION_FACTORY, RiskConstants.RADIX_PARAMETER);
            //Probability index starts from top but in chart it starts from bottom,
            // So cell index is retrived by substracting from 5.
            var probabilityCellIndexOnLoad = 6 - parseInt(this.rownode.probability / RiskConstants.MANIPULATION_FACTORY, RiskConstants.RADIX_PARAMETER);
            
			this._identBox.attr("value",data.riskidentifieddate);
			this._occurredBox.attr("value", data.riskoccurreddate);
           
            this._updateRiskMatrix(impactCellIndexOnLoad, probabilityCellIndexOnLoad, precision);
            
        },
        
        /**
         * 	This function converts RISK abbrevation name to Values
         * 	L : LOW
         *  H : HIGH
         *  M : MEDIUM
         *
         */
        _convertAbbrToPrecisionValue: function(value){
            var precision = null;
            if (value === RiskConstants.HIGH) {
                precision = RiskConstants.HIGH_ABBR;
            }
            else 
                if (value === RiskConstants.LOW) {
                    precision = RiskConstants.LOW_ABBR;
                }
                else 
                    if (value === RiskConstants.MEDIUM) {
                        precision = RiskConstants.MEDIUM_ABBR;
                    }
            return precision;
        },
        
        /**
         *  _riskMatrixClicked : Upadtes the values of Probability, Impact and Precision
         *  in the model as well as in the drop downs.
         */
		_riskMatrixClicked: function(args)
		{
			if(args.target && args.target.cellIndex && args.target.parentNode && args.target.parentNode.rowIndex)
			{
				var cellIndex = args.target.cellIndex;
				var rowIndex = args.target.parentNode.rowIndex;
				if (args.target && args.target.firstChild) {
					var precision = this.riskmatrix.rows[rowIndex].cells[cellIndex].firstChild.innerHTML ? 
										this.riskmatrix.rows[rowIndex].cells[cellIndex].firstChild.innerHTML : "";
					if (precision.indexOf(RiskConstants.LOW_ABBR) >= 0) {
						precision = RiskConstants.MEDIUM_ABBR;
					}
					else if (precision.indexOf(RiskConstants.MEDIUM_ABBR) >= 0) {
							precision = RiskConstants.HIGH_ABBR;
					} else {
							precision = RiskConstants.LOW_ABBR;
					}
		            this._updateRiskMatrix(cellIndex, rowIndex, precision);
				}
			}
		},

		/**
		 * Resets the matrix to clear all the labels
		 */		
		resetMatrix: function(){
			if (this.riskmatrix && this.riskmatrix.rows) {
				for (var i = 1; i <= 5; i++) {
					var row = this.riskmatrix.rows[i];
					if (row && row.cells) {
						for (var j = 1; j <= 5; j++) {
							if (row.cells[j]) {
								row.cells[j].firstChild.innerHTML = "";
							}
						}
					}
				}
			}
		},
        /**
         * 		_updateRiskMatrix is called from _handleDoubleClickEvent or while loading the risk data and populating in viewlet.
         * 		Whenever double click happens in risk matrix, value of Impact, Probability and Precision are modified
         * 		and saved in model. When saved is clicked modified data is saved into repository.
         * 		Incoming arguments : cellIndex  -> Impact Value
         * 							 rowIndex   -> Probability Value
         *
         *
         */
        _updateRiskMatrix: function(cellIndex, rowIndex, precision){
            if (!precision || precision.length === 0 || cellIndex <1 || cellIndex > 5 || rowIndex < 0 || rowIndex > 5) {
				return;
			}
			
			if(!this.containerViewlet._riskGrid || !this.rownode){
				return;
			}
			if (!this.riskmatrix.rows || !this.riskmatrix.rows[rowIndex] || !this.riskmatrix.rows[rowIndex].cells ||
			!this.riskmatrix.rows[rowIndex].cells[cellIndex] || !this.riskmatrix.rows[rowIndex].cells[cellIndex].firstChild) {
				return;
			}
			
			if (this.riskmatrix.rows[rowIndex].cells[cellIndex].firstChild.innerHTML && precision.indexOf(this.riskmatrix.rows[rowIndex].cells[cellIndex].firstChild.innerHTML) >= 0) {
				return;
			}
			this.resetMatrix();
			
			this.riskmatrix.rows[rowIndex].cells[cellIndex].firstChild.innerHTML = precision;
			
			var precisionValue = "";
			if (precision === RiskConstants.LOW_ABBR) {
	           precisionValue = RiskConstants.LOW;
	        } else  if (precision === RiskConstants.MEDIUM_ABBR) {
				precisionValue = RiskConstants.MEDIUM;
			} else {
				precisionValue = RiskConstants.HIGH;
			}
	        
			if (this.rownode.precision != precisionValue) {
				this.rownode.precision = precisionValue;
				this._precisionBox.selectedIndex = this.getIndexForPrecisionValue(this.rownode.precision);
			}
			
			var impact = cellIndex * RiskConstants.MANIPULATION_FACTORY;
			if (this.rownode.impact != impact) {
				this.rownode.impact = impact;
				this._impactBox.selectedIndex =  this.getIndexForValue(this.rownode.impact);
			}
			
			var probability = RiskConstants.MANIPULATION_FACTORY * (6 - rowIndex);
	        if (this.rownode.probability != probability) {
				this.rownode.probability = probability;
				this._probabilityBox.selectedIndex = this.getIndexForValue(this.rownode.probability);
			} 
	        
			this.containerViewlet._riskGrid.UpdateRow(this.rownode);
	        this._Calculation();	
        },
        
        /*
         * 		_updateRiskMatrixChart : Update the Risk chart.
         *
         */
        _updateRiskMatrixChart: function(){
            this._updateRiskMatrix(this.rownode.impact / RiskConstants.MANIPULATION_FACTORY, (6 - this.rownode.probability / RiskConstants.MANIPULATION_FACTORY), this._convertAbbrToPrecisionValue(this.rownode.precision));
            
        },

        
        /**
         *  _createRiskCategoryBox : This function is to create the Risk Category Box and label.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        _createRiskCategoryBox: function(){
			if (this.containerViewlet._riskGrid) {
				var options = this.containerViewlet._riskGrid.getSelectOptionsForCategory();
				if (options && options.length > 0) {
					this._categoryBox.disabled = false;
					this.removeAllOptions(this._categoryBox);
					for (index in options) {
						this._categoryBox.options[index] = new Option(options[index],options[index]);
						this._categoryBox.options[index].title = options[index];
					}
				}
				else {
					this._categoryBox.disabled = true;
				}
			}
		},
		_updateRiskCategoryBox: function(){
			if(this._categoryBox && this._categoryBox.options)
			{
				this._createRiskCategoryBox();
			}
			var category = this.rownode.riskcategory ? this.rownode.riskcategory : "";
			this._categoryBox.selectedIndex = dojo.indexOf(this.containerViewlet._riskGrid.getSelectOptionsForCategory(), category);
			this._categoryBox.title=category;
        },
		
		updateCategoryCombo: function(){
			this._createRiskCategoryBox();
			this._updateRiskCategoryBox();
		},
		
		_categoryValueChanged: function(event){
			var value = event.target.value;
			this._categoryBox.title=value;
			if (this.rownode && this.containerViewlet._riskGrid) {
				this.rownode.riskcategory = value;
				this.containerViewlet._riskGrid.UpdateRowFields(this.rownode.rowIndex, value, "riskcategory");
			}
		},
		
		_ownerValueChanged: function(event){
			var value =  event.target.value;
			this._ownerBox.title=value;
			if (this.rownode && this.containerViewlet._riskGrid) {
				this.rownode.owner = value;
				this.containerViewlet._riskGrid.UpdateRowFields(this.rownode.rowIndex, value, "owner");
			}
		},
		
		updateOwnerCombo: function(){
			this._createRiskOwnerBox();
			this._updateRiskOwnerBox();
		},
        
        /**
         *  _createRiskOwnerBox : This function is to create the Risk Owner Box and label.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        _createRiskOwnerBox: function(){
			if (this.containerViewlet._riskGrid) {
				var options = this.containerViewlet._riskGrid.getSelectOptionsForOwner();
				if (options && options.length > 0) {
					this._ownerBox.disabled = false;
					this.removeAllOptions(this._ownerBox);
					for (index in options) {
						this._ownerBox.options[index] = new Option(options[index],options[index]);
					}
				}
				else {
					this._ownerBox.disabled = true;
				}
			}
			
		},
		
		removeAllOptions: function(optionsList)
		{
			if (optionsList.length > 0) {
				while (optionsList.length > 0) {
					optionsList.remove(optionsList.length-1);
				}
			}
		},
		_updateRiskOwnerBox: function(){
			this._createRiskOwnerBox();
			if (this.rownode) {
				var index = dojo.indexOf(this.containerViewlet._riskGrid.getSelectOptionsForOwner(), (this.rownode.owner ? this.rownode.owner : ""));
				if (0 > index && this.rownode && this.rownode.owner) {
					index = this.containerViewlet._riskGrid.getSelectOptionsForOwner().length;
					this._ownerBox.options[index] = new Option(this.rownode.owner, this.rownode.owner);
				}
				this._ownerBox.selectedIndex = index;
			}
        },
		
		_updateRiskStateBox: function(state){
			if (state && state.length > 0) {
				this._stateBox.disabled = false;
				this._stateBox.selectedIndex = dojo.indexOf(this.containerViewlet.stateLabels, state);
			}else{
				this._stateBox.disabled = true;
				this._stateBox.selectedIndex = -1;
			}
        },
		
		_updateRiskPriorityBox: function(priority){
			if (priority) {
				this._priorityBox.disabled = false;
				this._priorityBox.selectedIndex = dojo.indexOf(this.containerViewlet.priorityIDs, priority);
			}
        },
        
		updateStateAndPriorityCombos: function(){
			this._createRiskStateCombo();
			this._createRiskPriorityCombo();
			if (this.rownode) {
				this._updateRiskStateBox(this.rownode.state);
				this._updateRiskPriorityBox(this.rownode.priority);
			}
		},
		
        /**
         *  _createRiskStateBox : This function is to create the Risk State ComboBox and label.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        _createRiskStateCombo: function(){
			if (this.containerViewlet._riskGrid) {
				var options = this.containerViewlet.stateLabels;
				if (options && options.length > 0 && this._stateBox.options.length === 0) {
					this._stateBox.disabled = false;
					for (index in options) {
						this._stateBox.options[index] = new Option(options[index],options[index]);
					}
				}
				else {
					this._stateBox.disabled = true;
				}
			}
        },
		
		_stateValueChanged : function(event){
			var value = event.target.value;
			if (this.rownode && this.containerViewlet._riskGrid) {
				this.rownode.state = value;
				this.containerViewlet._riskGrid.UpdateRowFields(this.rownode.rowIndex, value, "state");
			}
                    
		},

        /**
         *  _createPriorityCombo : This function is to create the Risk Priority ComboBox and label.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        _createRiskPriorityCombo: function(){
			if (this.containerViewlet._riskGrid) {
				var options = this.containerViewlet.priorityLabels;
				if (options && options.length > 0 && this._priorityBox.options.length === 0) {
					this._priorityBox.disabled = false;
					for (index in options) {
						this._priorityBox.options[index] = new Option(options[index],options[index]);
					}
				}
				else {
					this._priorityBox.disabled = true;
				}
			}
        },
		
		_priorityValueChanged : function(event){
			var value = event.target.value;
			if (this.rownode && this.containerViewlet._riskGrid) {
				var indx = dojo.indexOf(this.containerViewlet.priorityLabels, value);
				if(indx >= 0){
				 	this.rownode.internalPriority = this.containerViewlet.priorityIDs[indx];
					this.containerViewlet._riskGrid.UpdateRowFields(this.rownode.rowIndex, this.rownode.internalPriority, "internalPriority");
				}
           	}
                    
		},
        
        
        /**yes
         * 
         *  _create_probabilityBox : This function is to create the IMPACT Combobox and label.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *  OnChange event upadates the value in RISK MATRIX Chart Y-Axis.
         *
         */
        _create_probabilityBox: function(){
         	this._probabilityBox.options[0] = new Option(RiskMessages.VAL_20);
			this._probabilityBox.options[1] = new Option(RiskMessages.VAL_40);
			this._probabilityBox.options[2] = new Option(RiskMessages.VAL_60);
			this._probabilityBox.options[3] = new Option(RiskMessages.VAL_80);
			this._probabilityBox.options[4] = new Option(RiskMessages.VAL_100);
			dojo.connect(this._probabilityBox, "onchange", dojo.hitch(this, this._probabilityValueChanged));
        },
		
		/**
         *  _probabilityValueChanged : Handles the onChange for the Probability list
         *  Incoming Arguments : event
         */
		_probabilityValueChanged : function(event){
			var value = this.getValueForIndex(this._probabilityBox.selectedIndex);
			if(this.rownode && value && (!this.rownode.probability || this.rownode.probability !== value)){
				this.rownode.probability = value;
				this._updateRiskMatrixChart();
			}
		},
		
        
        /**
         *  _createImpactCombo : This function is to create the IMPACT Combobox and label.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *  OnChange event upadates the value in RISK MATRIX Chart X-Axis.
         *
         */
        _createImpactCombo: function(){
         	this._impactBox.options[0] = new Option(RiskMessages.VAL_20);
			this._impactBox.options[1] = new Option(RiskMessages.VAL_40);
			this._impactBox.options[2] = new Option(RiskMessages.VAL_60);
			this._impactBox.options[3] = new Option(RiskMessages.VAL_80);
			this._impactBox.options[4] = new Option(RiskMessages.VAL_100);
			dojo.connect(this._impactBox, "onchange", dojo.hitch(this, this._impactValueChanged));
        },
		
		/**
         *  _impactValueChanged : Handles the onChange for the Impact list
         *  Incoming Arguments : event
         */
		_impactValueChanged : function(event){
			var value = this.getValueForIndex(this._impactBox.selectedIndex);
			if(this.rownode && value && (!this.rownode.impact || this.rownode.impact !== value)) {
				this.rownode.impact = value;
				this._updateRiskMatrixChart();
			}    
		},

        
        /**
         *  _createPrecisionCombo : This function is to create the Precision Combobox and label.
         *  Incoming Arguments : none
         *  Outgoing Argument : none
         *
         */
        _createPrecisionCombo: function(){
            this._precisionBox.options[0] = new Option(RiskMessages.LOW);
			this._precisionBox.options[1] = new Option(RiskMessages.MEDIUM);
			this._precisionBox.options[2] = new Option(RiskMessages.HIGH);
			dojo.connect(this._precisionBox, "onchange", dojo.hitch(this, this._precisionValueChanged));
        },
        
		/**
         *  _precisionValueChanged : Handles the onChange for the Precision list
         *  Incoming Arguments : event
         */
		_precisionValueChanged : function(event){
			var value = this.getValueForPrecisionIndex(this._precisionBox.selectedIndex);
			if(this.rownode  && value && (!this.rownode.precision || this.rownode.precision.indexOf(value) < 0)){
				this.rownode.precision = value;
				this._updateRiskMatrixChart();
			}         
		},
               
        /**
         *  onIdentDateChange : This function is called when Identification date is changed.
         *  Incoming Arguments : Probability value
         *  Outgoing Argument : none
         *
         */
        onIdentDateChange: function(){
			if(!this._identBox.value){
				this._identBox.attr("value",this.rownode.riskidentifieddate);
			}
			var value = this._identBox.value;
			if(this.containerViewlet._riskGrid && this.rownode !== null && this.rownode.riskidentifieddate != value)   {
			    this.rownode.riskidentifieddate = value;
				this.containerViewlet._riskGrid.UpdateRowFields(this.rownode.rowIndex, value, "riskidentifieddate");
			}
        },
        
        /**
         *  onOccDateChange : This function is called when Occurence date is changed.
         *  Incoming Arguments : Probability value
         *  Outgoing Argument : none
         *
         */
        onOccDateChange: function(){
			var value = this._occurredBox.value;
			if(this.containerViewlet._riskGrid && this.rownode && this.rownode.riskoccurreddate != value)   {
			    this.rownode.riskoccurreddate = value;
				this.containerViewlet._riskGrid.UpdateRowFields(this.rownode.rowIndex, value, "riskoccurreddate");
			}
        },
		
		/**
         *  onConsCostChange : This function is called when Consequence Cost is changed.
         *  Incoming Arguments : Probability value
         *  Outgoing Argument : none
         *
         */
        onConsCostChange: function(){
			var value = this._consCostBox.value;
			var min = RiskConstants.RISK_COST_MIN;
			var max = RiskConstants.RISK_COST_MAX;
			if (this.rownode && !value || (value < min ) || (value > max)) {
				value = this.rownode.consequencescost ? this.rownode.consequencescost : 0;
				this._consCostBox.attr("value",value);
			}
			
			if(this.containerViewlet._riskGrid && this.rownode && (this.rownode.consequencescost != value))   {
			    this.rownode.consequencescost = value;
				this.containerViewlet._riskGrid.UpdateRowFields(this.rownode.rowIndex, value, "consequencescost");
			}
			this._Calculation();
        },
		
  		/**
         *  _Calculation : This function is called when Consequence Cost, probability or impact is changed.
         *  Risk Exposure = Probabilty X Impact
         *  Probability Cost = Consequence Cost X Probability
         *
         */
        _Calculation: function(){
            if (this.rownode !== null) {
				var impact = parseInt(this.rownode.impact, RiskConstants.RADIX_PARAMETER)/ RiskConstants.HUNDRED;
				var probability = parseInt(this.rownode.probability, RiskConstants.RADIX_PARAMETER) / RiskConstants.HUNDRED;
				var exposure = impact * probability;
				var ccost = parseInt(this.rownode.consequencescost, RiskConstants.RADIX_PARAMETER);
                var pbCost = ccost * probability;
                this._pbCostBox.innerHTML = pbCost.toFixed(2);
                this._exposureBox.innerHTML = exposure.toFixed(2);
            }
        },
		
		_clickedIdentDateImage: function(event) {
			this._occurredBox._close();
            this._identBox.focus();
		},
		_clickedOccurredDateImage: function(event) {
			this._identBox._close();
            this._occurredBox.focus();
		},
		/**
		 * This method is called from the risk management view let , when the row selection changes in the risk grid.
		 * When the state is true, no rows are selected, hence the values in all the fields in the risk details section are cleared
		 * And the disabled state is also toggled.
		 */
		toggleRiskDetailsState: function(state) {
			this._identBox.disabled = state;
            this._occurredBox.disabled = state;
			this._probabilityBox.disabled = state;
			this._precisionBox.disabled = state;
			this._impactBox.disabled = state;
			this._stateBox.disabled = state;
			this._priorityBox.disabled = state;
			this._ownerBox.disabled = state;
			this._consCostBox.disabled = state;
			this._consCostBox.disabled = state;
			this._identBox.disabled = state;
			this._occurredBox.disabled = state;
			
			if (state) {
				this._probabilityBox.selectedIndex = -1;
				this._precisionBox.selectedIndex = -1;
				this._impactBox.selectedIndex = -1;
				this._stateBox.selectedIndex = -1;
				this._ownerBox.selectedIndex = -1;
				this._priorityBox.selectedIndex = -1;
				this._consCostBox.attr("value", "");
				this._identBox.attr("value","");
				this._occurredBox.attr("value","");
				this._pbCostBox.innerHTML = "";
				this._exposureBox.innerHTML = "";
				this.resetMatrix();
			}
		}
    });
})();
