/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.risk.web.ui.internal.RiskGridModel");
    
    dojo.require("com.ibm.epm.web.core.grid.model.AbstractGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskConstants");
    dojo.requireLocalization("com.ibm.epm.risk.web.ui", "Messages");
	dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    
    var RiskConstants = com.ibm.epm.risk.web.ui.internal.RiskConstants;
    var abstractGridModel = com.ibm.epm.web.core.grid.model.AbstractGridModel;
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
	var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
	var RiskMessages = dojo.i18n.getLocalization("com.ibm.epm.risk.web.ui", "Messages");
    
	var selectOptionsForCategory = [];
	var selectOptionsForCategoryId = [];
	var selectOptionsForOwner = [];
	var selectOptionsForOwnerId = [];
	 
    dojo.declare("com.ibm.epm.risk.web.ui.internal.RiskGridModel", abstractGridModel, {
    	newElementPrefix: "_unsaved",
    
		constructor: function(){
			this._containingGrid = arguments[0]._containingGrid;
		},
        /*
         * This is called by controller. Returns the empty row structure to the controller.
         */
        getEmptyRow: function(rowID,collectionItemHandle){
            
			var row;
			
			if ((rowID.indexOf("unsaved")) == -1) {
				
				row = {
					"id": rowID,
					"relativeHref": rowID,
					"riskid" :this._getRiskID(rowID),
					"href": rowID,
					"name": "",
					"impact": "",
					"probability": "",
					"description": "",
					"consequencescost": 0,
					"mitigationstrategy": "",
					"precision": "",
					"contingencyplan": "",
					"riskidentifieddate": "",
					"riskoccurreddate": "",
					"riskcategory": "",
					"owner": "",
					"state": "",
					"priority": "",
					"parent": "",
					"internalComments": "",
					"comments": "",
					"internalPriority": ""
				};
			}else{
				
				row = {
					"id": rowID,
					"relativeHref": rowID,
					"riskid" :"",
					"href": rowID,
					"name": collectionItemHandle.name?collectionItemHandle.name:"",
					"impact": "20",
					"probability": "20",
					"description": "",
					"consequencescost": 0,
					"mitigationstrategy": "",
					"precision": "Low",
					"contingencyplan": "",
					"riskidentifieddate": collectionItemHandle.riskidentifieddate?collectionItemHandle.riskidentifieddate:"",
					"riskoccurreddate": "",
					"riskcategory": "",
					"owner": "",
					"state": collectionItemHandle.state?collectionItemHandle.state:"",
					"priority": "",
					"parent": collectionItemHandle.parent,
					"internalComments": "",
					"comments": collectionItemHandle.comments?collectionItemHandle.comments:"",
					"internalPriority": ""
				};
				
				
			}
            
            return row;
        },
        
		
		/**
		 * _getRiskID: return the RISKID.
		 * @param {Object} relativeHref: URI to be parsed to get the RISK ID.
		 */
        _getRiskID: function(relativeHref){
			var uriId = "";
            if (relativeHref) {
	            var index = relativeHref.lastIndexOf("/");
	            if (index > 0) {
	                uriId = relativeHref.substring(index + 1, relativeHref.length);
	            }
			}
			return uriId;
        },
        
		isNewRow: function(rowHref){
			return (rowHref && rowHref.lastIndexOf(this.newElementPrefix) != -1);
		}, 
        /*
         * gets the links from the json returned from the server
         */
        getAssignedLink: function(loadedElement, linkIDs){
            for (var i in linkIDs) {
                var link = loadedElement[linkIDs[i]];
                if (link) {
                    if (dojo.isArray(link)) {
                        link = link[0];
                    }
                    return link;
                }
            }
        },
        
        /*
         * This is called by controller.
         * After inserting a row in Grid, the getAndUpdateRow() calls updateRowData with RowURI, RowID params.
         */
        updateRowData: function(row, loadedElement, linkName){
			  if (linkName === null) {
 
	                row.name = loadedElement.summary;
	                row.impact = loadedElement.riskdetails.impact.substr(1);
	                row.probability = loadedElement.riskdetails.probability.substr(1);
	                row.description = loadedElement.description;
	                row.consequencescost = loadedElement.riskdetails.consequenceCost;
	                row.precision = loadedElement.riskdetails.precision;
					if (loadedElement.riskdetails.identifiedDate) {
						row.riskidentifieddate = dateUtil.parseDate(loadedElement.riskdetails.identifiedDate);
					}else{
						row.riskidentifieddate ="";
					}
					if (loadedElement.riskdetails.occurredDate) {
						row.riskoccurreddate = dateUtil.parseDate(loadedElement.riskdetails.occurredDate);
					} else {
						row.riskoccurreddate ="";
					}
					if (loadedElement.category) {
						row.riskcategory = loadedElement.category[0].title;
					}
	                //row.owner = loadedElement.owner;
					
					selectOptionsForOwner = this._containingGrid.getSelectOptionsForOwner();
					selectOptionsForOwnerId = this._containingGrid.getSelectOptionsForOwnerId();
					if (selectOptionsForOwnerId && selectOptionsForOwnerId.length > 0) {
						var indx = dojo.indexOf(selectOptionsForOwnerId, loadedElement.owner);
						row.owner = selectOptionsForOwner[indx] ? selectOptionsForOwner[indx] : loadedElement.owner;
					}
					
					
					row.state = loadedElement.state;
					row.mitigationstrategy = dojo.clone(loadedElement.riskdetails.mitigationStrategies);
					row.contingencyplan = dojo.clone(loadedElement.riskdetails.contingencyPlans);
					row.internalComments = loadedElement.internalComments;
					row.comments = loadedElement.comments;
					row.priority = loadedElement.internalPriority;
					row.internalPriority = loadedElement.internalPriority;
					
					row.loadedelement = [loadedElement];
					this._containingGrid.selectFirstRowByDefault(row);
				}
        },
        
        /**
         *  Incoming Arguments : The modofied row values is passed.
         *  Outgoing Argument : none
         *
         */
        prepareSaveParams_link: function(row, linkName){
           return null;
		   // this.prepareSaveParams(row);
        },
        
        /**
         *  Incoming Arguments : The modofied row values is passed.
         *  Outgoing Argument : none
         *
         */
        getRowPostURI_link: function(row, linkName, newElementID){
            return null;
        },			
		
		 /**
         *  getOriginalRow : Gets the row from the model, given the id.
         * This is required since the order of the item in the grid is not
         * the same as the one in grid
         */
		getOriginalRow: function(id, strategies){
			if (id.indexOf("unsaved") == -1 && strategies && strategies.length >= 0) {
				for (var i = 0; i < strategies.length; i++) {
					if (strategies[i].id == id) {
						return strategies[i];
					}
				}
			}
			return null;
        },	
		
		/**
         *  prepareSaveParams: : This function is to prepare the parameters  for saving.
         *  Incoming Arguments : The modofied row values is passed.
         *  Outgoing Argument : none
         *
         */
        prepareSaveParams: function(row){
			selectOptionsForCategory = this._containingGrid.getSelectOptionsForCategory();
			selectOptionsForCategoryId = this._containingGrid.getSelectOptionsForCategoryId();
			selectOptionsForOwner = this._containingGrid.getSelectOptionsForOwner();
			selectOptionsForOwnerId = this._containingGrid.getSelectOptionsForOwnerId();
			
            var riskidentifieddate;
            var riskoccurreddate;
			var category = "";
			
            if (row.riskidentifieddate === null || row.riskidentifieddate === "") {
                riskidentifieddate = "NULL";
            }
            else {
                riskidentifieddate = dateUtil.formatDateForSave(row.riskidentifieddate);
            }
            if (row.riskoccurreddate === null || row.riskoccurreddate === "") {
                riskoccurreddate = "NULL";
            }
            else {
                riskoccurreddate = dateUtil.formatDateForSave(row.riskoccurreddate);
                
            }
			if (row.riskcategory !== "") {
                for (var i = 0; i < selectOptionsForCategory.length; i++) {
                     if(selectOptionsForCategory[i] == row.riskcategory){
					 	category = selectOptionsForCategoryId[i];
						break;
					 }                                
                }
            }
			
			this.msStrategyParams = this.buildPlanJSONs(row.mitigationstrategy);
			
			this.cpStrategyParams = this.buildPlanJSONs(row.contingencyplan);
			
            var saveParams = {
				workItemType : RiskConstants.RISK_TYPE,
                summary: row.name,
                impact: "L" + row.impact,
                probability: "L" + row.probability,
                description: row.description,
                consequenceCost: row.consequencescost,
                precision: row.precision,
                identifiedDate: riskidentifieddate,
                occurredDate: riskoccurreddate,
				category : category,
                projectUri : row.parent,
				internalComments : row.comments	,
				mitigationStrategies: this.msStrategyParams,
				contingencyPlans: this.cpStrategyParams
            };
			
			if(row.internalPriority){
				saveParams.internalPriority = row.internalPriority;
			}
			if(row.state){
				saveParams.state = row.state;
			}
			
			this.prepareOwnerParam(saveParams, row, selectOptionsForOwner);			
            return saveParams;
        },
		
		/**
		 * sets the "owner" save parameter if some thing is selected in the UI.
		 * @param {Object} saveParams: the save params that will be posted to the server.
		 * @param {Object} row : the row for which the save paramters needs to be prepared.
		 * @param {Object} selectOptionsForOwner: the values selected from the owner drop down.
		 */
		prepareOwnerParam: function(saveParams, row, selectOptionsForOwner){
			if (row.owner) {
				var indx = dojo.indexOf(selectOptionsForOwner,row.owner); 
                if(indx >= 0){
					 	saveParams.owner = selectOptionsForOwnerId[indx];
                }
            }
		},
		
		 /*
         * Get Post call URI method
         * Incoming : Row Info
         * Outgoing : Post call URI
         */
        _getNewElementURI: function(row){
            return RiskConstants.RISK;
        },
		
		
		/**
         *  buildPlanJSONs : This builds the parameters forthe risk
         * strategies and actions in the required format.
         * 
         */
		buildPlanJSONs: function(plans){
			var planJSONs = [];
			if (plans) {
				for (var p = 0; p < plans.length; p++) {
					var planJSON = this.buildStrategyParameterJSON(plans[p]);
					if (planJSON !== "") {
						planJSONs.push(planJSON);
					}
				}
			}
			return planJSONs;
		},
		
		/**
         *  buildStrategyParameterJSON : This builds the parameters forthe risk
         * strategies and actions in the required format.
         * 
         */
        buildStrategyParameterJSON: function(strategy){
			var strvar = new dojox.string.Builder("{\"riskstrategy\":{");
			var strategyUpdated = false;
			
			if (strategy.deleted) {
				strvar.append("\"delete\":\"true\"");
				strategyUpdated = true;
			}
			else {
				if (strategy.modifiedsummary) {
					strvar.append("\"summary\":\"");
					strvar.append(strategy.modifiedsummary);
					strategyUpdated = true;
					strvar.append("\"");
				}
				if (strategy.modifieddescription) {
					if (strategyUpdated) {
						strvar.append(",");
					}
					else {
						strategyUpdated = true;
					}
					strvar.append("\"description\":\"");
					strvar.append(strategy.modifieddescription);
					strvar.append("\"");
				}
				var actions = this.buildActionJSONs(strategy.actions);
					
				if (actions.length > 0) {
					if (strategyUpdated) {
						strvar.append(",");
					}
					else {
						strategyUpdated = true;
					}
					strvar.append("\"actions\":[");
					strvar.appendArray(actions);
					strvar.append("]");
				}
			}
			if (strategyUpdated && strategy.id.indexOf("unsaved") == -1) {
				strvar.append(",\"id\":\"");
				strvar.append(strategy.id);
				strvar.append("\"");
			}
			strvar.append("}}");
			if (!strategyUpdated) {
				strvar.clear();
			}
			return strvar.toString();
		},
		/**
         *  buildActionJSONs : This builds the parameters forthe risk
         * strategies and actions in the required format.
         * 
         */
		buildActionJSONs: function(strategyActions){
			var actions = [];
			if (strategyActions) {
				for (var k = 0; k < strategyActions.length; k++) {
					var action = strategyActions[k];
					var stract = new dojox.string.Builder("{");
					var actionUpdated = false;
					if (action.deleted) {
						stract.append("\"delete\":\"true\"");
						actionUpdated = true;
					}
					else {
						if (action.modifiedsummary) {
							stract.append("\"summary\":\"");
							stract.append(action.modifiedsummary);
							stract.append("\"");
							actionUpdated = true;
						}
						if (action.modifiedcost >= 0) {
							if (actionUpdated) {
								stract.append(",");
							}
							else {
								actionUpdated = true;
							}
							stract.append("\"cost\":\"");
							stract.append(action.modifiedcost);
							stract.append("\"");
						}
						if (action.modifiedduration >= 0) {
							if (actionUpdated) {
								stract.append(",");
							}
							else {
								actionUpdated = true;
							}
							stract.append("\"duration\":\"");
							stract.append(action.modifiedduration);
							stract.append("\"");
						}
						if (action.createtask) {
							if (actionUpdated) {
								stract.append(",");
							}
							else {
								actionUpdated = true;
							}
							stract.append("\"createtask\":\"");
							stract.append(action.createtask);
							stract.append("\"");
						}
					}
					if (actionUpdated && action.id.indexOf("unsaved") == -1) {
						stract.append(",\"id\":\"");
						stract.append(action.id);
						stract.append("\"");
					}
					if (actionUpdated) {
						stract.append("}");
						if (actions.length > 0) {
							actions.push(",");
						}
						actions.push(stract.toString());
					}
				}
			}
			return actions;
		}
		
    });
})();
