/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.risk.web.ui.internal.RiskGridView");
    
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskGridModel");
    dojo.require("com.ibm.epm.web.core.client.QueryUtil");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllersFactory");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskUtil");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
	dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");

    // Localization
    dojo.requireLocalization("com.ibm.epm.risk.web.ui", "Messages");
    dojo.require("com.ibm.epm.risk.web.ui.internal.RiskConstants");
	
    var dateUtil = com.ibm.epm.risk.web.ui.internal.RiskUtil;
    var RiskConstants = com.ibm.epm.risk.web.ui.internal.RiskConstants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var ControllersFactory = com.ibm.epm.web.core.grid.controller.ControllersFactory;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var QueryUtil = com.ibm.epm.web.core.client.QueryUtil;
    var RiskMessages = dojo.i18n.getLocalization("com.ibm.epm.risk.web.ui", "Messages");
	var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var RiskViewController = null;
	
    var selectOptionsForOwner = [];
	var selectOptionsForOwnerId = [];
    var selectOptionsForCategory = [];
    var selectOptionsForCategoryId = [];
	
    
    
    dojo.declare("com.ibm.epm.risk.web.ui.internal.DynamicSelect", dojox.grid.editors.Select, {
    
        fieldName: null,
        constructor: function(inCell){
        
            this.fieldName = inCell.field;
			this.title = " ";
            
        },
        format: function(inDatum, inRowIndex){
            if (this.fieldName === RiskConstants.RISK_OWNER_FIELD) {
				if (selectOptionsForOwner && selectOptionsForOwner.length > 0) {
					this.options = this.values = selectOptionsForOwner.slice();
				}
				else {
					this.options = this.values = [];
				}
				if(0 > dojo.indexOf(selectOptionsForOwnerId, inDatum)){
					this.options.push(inDatum);
				}
            }
            if (this.fieldName === RiskConstants.RISK_CATEGORY_FIELD) {
				if (selectOptionsForCategory && selectOptionsForCategory.length > 0) {
					this.options = this.values = selectOptionsForCategory;
				}
				else {
					this.options = this.values = [""];
				}
            }
			//This code is to set the tool tip for individual options in the drop down
			this.needFormatNode(inDatum, inRowIndex);
			var h = [ '<select class="dojoxGrid-select", title="'+(inDatum ? inDatum : ' ')+'">' ];
			for (var i = 0, o, v; ((o = this.options[i]) !== undefined) && ((v = this.values[i]) !== undefined); i++) {
				h.push("<option", (inDatum == v ? ' selected' : ''), ' value="' + v + '"', ' title="' + v + '"', ">", o, "</option>");
			}
			h.push('</select>');
			return h.join('');
        }
    });
    
    var DynamicSelect = com.ibm.epm.risk.web.ui.internal.DynamicSelect;
	/**
         *  Extending the DateTextBox to take care of the date picker showing the 
         *  default value of 1970 on first click
         *
         */
	dojo.declare("com.ibm.epm.risk.web.DateTextBox", dojox.grid.editors.DateTextBox, {
		setValue: function(inRowIndex, inValue){ 
		if (this.editor) {
			var leastDate = com.ibm.epm.web.core.util.internal.DateUtil.parseDate(RiskConstants.LEAST_DATE);
			if (inValue && inValue !== "" && (!isNaN(inValue)) && (0 >= dojo.date.compare(leastDate, inValue, 'date'))) {
				this.editor.attr('value', inValue);
			}
			else {
				this.editor.attr('value', "");
			}
		}
	},
	getEditorProps: function(inDatum){
			if (inDatum && inDatum !== "" && (!isNaN(inDatum))) {
				var leastDate = com.ibm.epm.web.core.util.internal.DateUtil.parseDate(RiskConstants.LEAST_DATE);
				if (0 >= dojo.date.compare(leastDate, inDatum, 'date')) {
					return dojo.mixin({}, {value: inDatum});
				}
			}
			return dojo.mixin({},{value : ""});
		
	},
	 formatNode: function(inNode, inDatum, inRowIndex){
	 	this.inherited(arguments);
	 	if (this.editor) {
	 		dojo.mixin(this.editor.constraints, { max: new Date()});
			var value = this.editor.attr('value');
			if (value) {
				this.editor.attr('title', value);
			}
			else {
				this.editor.attr('title', " ");
			}
	 	}
	 }
});
	    dojo.declare("com.ibm.epm.risk.web.ui.internal.RiskGridView", abstractView, {
        templateString: '<div class="com-ibm-epm-risk-web-ui-internal-riskViewGrid" dojoAttachPoint="_riskViewContent" style="height:250px;"></div>',
        newElementPrefix: "_unsaved",
		rowChangeflag : true,
       	
		/**
         *  postCreate : postCreate for the viewlet.
         */
        postCreate: function(){
			dojo.subscribe(GridEvents.FIELD_CHANGED, this, this.fieldChanged);		
			dojo.connect(this._generalGrid, 'onHeaderClick', this, this._setGridSort);	
        },
		
		/**
		 * This method is called by the framework, after the grid is created.
		 * Subscribes to the required events 
		 */
		createView: function(){
			this.inherited('createView', arguments);
			this._pconnects.push(dojo.connect(this._generalGrid, "onStartEdit", this, this.onStartEdit));
			this._pconnects.push(dojo.connect(this._generalGrid, "onApplyCellEdit", this, this.onApplyCellEdit));
		},

		/**
		 * Save the current values for the edited cell, locally, to be able to revert to this 
		 * value if the validation fails in 'onApplyCellEdit'
		 */
		onStartEdit: function(inCell, inRowIndex){
			var row = this._generalGrid.model.getRow(inRowIndex);
			if (row) {
				this.curCellField = inCell.field;
			}
		},
		
		/**
		 * Validate the fields for valid values, if the validation fails revert to the previous value,
		 * which was saved in 'onStartEdit'
		 */
		onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
			//IE Issue : For the first edit the 0th index is undefined
			if (this._generalGrid && this._generalGrid._click && (this._generalGrid._click[0] || this._generalGrid._click[1])) {
				var row = this._generalGrid.model.getRow(inRowIndex);
				if (row) {
					var cellFieldName = this.curCellField;
					var item = this._generalGrid.model.store._getItemByIdentity(row.id);
					if (item) {
						if (cellFieldName === RiskConstants.RISK_OCCURED_DATE_FIELD || cellFieldName === RiskConstants.RISK_IDENT_DATE_FIELD) {
							if (dojo.date.compare(new Date(), inValue, "date") < 0) {
								this._generalGrid.edit.cancel();
							}
						}else if(cellFieldName === RiskConstants.RISK_NAME_FIELD){
							inValue = inValue ? inValue.replace(/^\s*/, "").replace(/\s*$/, "") : inValue;
							if(!inValue || inValue.length === 0){
								this._generalGrid.edit.cancel();
							}
						}else if(cellFieldName === RiskConstants.RISK_CONS_COST_FIELD){
							var min = RiskConstants.RISK_COST_MIN;
							var max = RiskConstants.RISK_COST_MAX;
							if(!inValue || (inValue < min ) || (inValue > max)){
								this._generalGrid.edit.cancel();
							}
							
						}
						
					}
				}
			}
		}, 
		/**
         *
         * @param {Object} event
         * On clicking the header, the columns in the grid should be sorted.
         * Presently after catching this event, it is not passed to the
         * parent grid class, as no sorting is implemented.
         */
        _setGridSort: function(event){
            //Grid sorting happens at abstract level, the risk grid doesn't refresh accordingly
			//doing them here.
			this.selectFirstRowByDefault();
			
            event.stopPropagation();
            event.preventDefault();
            dojo.stopEvent(event);
        },
		/**
         *  fieldChanged : Updates the details section if these fields are updated in the grid
         *
         */
		fieldChanged: function(data){
            if (this._generalGrid && this.containerViewlet && data.controllerID == RiskConstants.RISK_GRID_CONTROLLER 
			&& (data.fieldName == RiskConstants.RISK_OCCURED_DATE_FIELD || data.fieldName == RiskConstants.RISK_CONS_COST_FIELD 
			|| data.fieldName == RiskConstants.RISK_IDENT_DATE_FIELD || data.fieldName == RiskConstants.RISK_OWNER_FIELD || 
			data.fieldName == RiskConstants.RISK_CATEGORY_FIELD)) {
				
				var rowIndex = this._generalGrid.selection.selectedIndex; 
				if(rowIndex >= 0 && this._controller){
					var data1 = this._controller.getRowByIndex(rowIndex);
					data1.rowIndex = rowIndex;
	                this.containerViewlet.updateDetails(data1);
	            }	
			}
			if (this.containerViewlet && data.controllerID == RiskConstants.RISK_GRID_CONTROLLER || data.controllerID == RiskConstants.RISK_ACTIONS_GRID_CONTROLLER || data.controllerID == RiskConstants.RISK_STRATEGIES_GRID_CONTROLLER){
				this.containerViewlet.toggleSaveButton(false);
			}
			if(data.fieldName == RiskConstants.RISK_NAME_FIELD){
				this.containerViewlet.setSubSectionTitles(data.newValue);
			}
        },
		/**
         * buildRatesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with RiskGrid.
         */
        buildRiskViewGrid: function(){
            if (!this._gridModel) {
			    this._gridModel = new com.ibm.epm.risk.web.ui.internal.RiskGridModel({
					_containingGrid: this	
				});
            }            
			if(!this.RiskViewController){
				this.RiskViewController = ControllersFactory.getEPMRiskGridQueryController(RiskConstants.RISK_GRID_CONTROLLER);
				this._controller = this.RiskViewController; 
			}
			else{
				this.RiskViewController.setAllElementsURI(this._treeID);
			}
            
			this.buildGrid(this, this._gridModel, this.RiskViewController);
			this.selectFirstRowByDefault();
        },
        
		/**
		* Selects the first row by default
		*/
		selectFirstRowByDefault: function(row){
			if(this._generalGrid && this._generalGrid.selection.selectedIndex < 0 && this._generalGrid.rowCount > 0){
				this._generalGrid.selection.select(0);
			}
		},
        setSelectOptionsForOwner: function(options, values){
        	if (options) {
				selectOptionsForOwner = options;
				selectOptionsForOwnerId = values;
				if(this.containerViewlet && this.containerViewlet._RiskDetails){
					this.containerViewlet._RiskDetails.updateOwnerCombo();
				}
			}
        },
        
        setSelectOptionsForCategory: function(options, values){
			if(options)
			{
				selectOptionsForCategory = options;
	            selectOptionsForCategoryId = values;
				if(this.containerViewlet && this.containerViewlet._RiskDetails){
					this.containerViewlet._RiskDetails.updateCategoryCombo();
				}
			}
		},
		       
		getSelectOptionsForOwner: function()
		{
			return selectOptionsForOwner;
		},
		
		getSelectOptionsForOwnerId: function()
		{
			return selectOptionsForOwnerId;
		},
		getSelectOptionsForCategory: function()
		{
			return selectOptionsForCategory;
		},
		
		getSelectOptionsForCategoryId: function()
		{
			return selectOptionsForCategoryId;
		},
		
        addrisk: function(projId){
            var rowID = this._controller.getUniqueRowID("_unsaved");
			var today = new Date();
            var row = {
				"id": rowID,
                "relativeHref": rowID,
                "name": RiskMessages.NEWRISK,
                "parent": projId,
				"riskidentifieddate": today
            
            };
            this._controller._insertElement(row, null, false);
            this.scrollToSelectedRow();
			this.startFirstCellEdit();
        },
        
        /*
         * This is name the Grid.
         */
        getGridTitle: function(){
            return RiskMessages.RISKGRID_TITLE;
        },
         getGridInternalID: function(){
            return  RiskConstants.RISKGRIDNAME;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this._riskViewContent;
        },

        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var RiskViewGridLayout = [{
                noscroll: false,
                defaultCell: {
                    styles: 'text-align: left;'
                },
                rows: [[{
                    name: RiskMessages.NAME,
                    width: 20,
                    field: RiskConstants.RISK_NAME_FIELD,
					editor: dojox.grid.editors.Input,
					title: RiskMessages.RISK_NAME_TITLE
                }, 
				{
                    name: RiskMessages.RISK_ID, //1
                    width: 5,
                    field: RiskConstants.RISKID_FIELD,
					title: RiskMessages.RISK_ID_TITLE
                }, {
                    name: RiskMessages.IDENTIFICATION_DATE, //2
                    width: 10,
                    field: RiskConstants.RISK_IDENT_DATE_FIELD,
                    formatter: dateUtil.formatDate,
					styles: 'vertical-align: top;',
                    editor: com.ibm.epm.risk.web.DateTextBox,
                    editorClass: "dijit.form.DateTextBox",
					constraint: {formatLength: 'short',selector: "date"},
					title: RiskMessages.IDENTIFICATION_DATE_TITLE
                }, {
                    name: RiskMessages.OCCURRED_DATE, //2
                    width: 10,
                    field: RiskConstants.RISK_OCCURED_DATE_FIELD,
                    formatter: dateUtil.formatDate,
					styles: 'vertical-align: top;',
                    editor: com.ibm.epm.risk.web.DateTextBox,
                    editorClass: "dijit.form.DateTextBox",
                    constraint: {formatLength: 'short',selector: "date"},
					title: RiskMessages.OCCURRED_DATE_TITLE
                }, {
                    name: RiskMessages.RISKCATEGORY, //2
                    width: 10,
                    field: RiskConstants.RISK_CATEGORY_FIELD,
					styles: 'vertical-align: top;',
                    editor: DynamicSelect,
					title: RiskMessages.RISKCATEGORY_TITLE
                }, {
                    name: RiskMessages.OWNER, //2
                    width: 10,
                    field: RiskConstants.RISK_OWNER_FIELD,
					styles: 'vertical-align: top;',
                    editor: DynamicSelect,
					title: RiskMessages.OWNER_TITLE
                },{
                    name: RiskMessages.CONSEQUENCE_COST, //2
                    width: 'auto',
                    field:  RiskConstants.RISK_CONS_COST_FIELD,
					styles: 'vertical-align: top;text-align: right;',
					editor: dojox.grid.editors.Input,
					constraint: {currency: 'USD'},
					editorClass: "dijit.form.CurrencyTextBox",
					title: RiskMessages.CONSEQUENCE_COST_TITLE
                }
				]]
            }];
            
            return RiskViewGridLayout;
        },
		getSortableColums:function()
		{
			return {	riskid:				"riskid",
						name:				"name",
						riskidentifieddate:	"riskidentifieddate",
						riskoccurreddate:	"riskoccurreddate",
						riskcategory:		"riskcategory",
						owner:				"owner",
						consequencescost:	"consequencescost"	};
		},
		
		/**
		 * This method is called form the risk management view let , when the row selection changes
		 * in the risk management view let 
		 */
		riskGridSelectionChanged: function(state){
			var rowIndex = this._generalGrid.selection.selectedIndex;
			if (state || !this._controller || 0 > rowIndex) {
				return;
			}
			var data = this._controller.getRowByIndex(rowIndex);
			if (data) {
				data.rowIndex = rowIndex;
				dojo.publish(RiskMessages.RISKUPDATE, [data]);
				this.containerViewlet.setSubSectionTitles(data.name);
				this.rowChangeflag = true;
			}
		},
        /**
         *  UpdateRow : This function is to update the grid row.
         *  Incoming Arguments : The modofied row values is paased.
         *  Outgoing Argument : none
         *
         */
        UpdateRow: function(data){
			if (!this._generalGrid) {
				return;
			}
            var data1 = this._generalGrid.model.getRow(data.rowIndex);
			if( !this.rowChangeflag && data1.loadedelement && (data1.loadedelement[0].riskdetails.precision != data.precision ||
				 data1.loadedelement[0].riskdetails.probability != (RiskConstants.LOW_ABBR + data.probability) || 
				 data1.loadedelement[0].riskdetails.impact != (RiskConstants.LOW_ABBR + data.impact))){
				this.containerViewlet.toggleSaveButton(false);
			}
			this.rowChangeflag = false;
            data1.name = data.name;
            data1.impact = data.impact;
            data1.probability = data.probability;
            data1.consequencescost = data.consequencescost;
            data1.precision = data.precision;
            data1.description = data.description;
            data1.contingencyplan = data.contingencyplan;
            data1.mitigationstrategy = data.mitigationstrategy;
            data1.riskidentifieddate = data.riskidentifieddate;
            data1.riskoccurreddate = data.riskoccurreddate;
            data1.riskcategory = data.riskcategory;
            data1.state = data.state;
			data1.internalComments = data.internalComments;
			data1.comments = data.comments;
			data1.owner = data.owner;
			
			this._generalGrid.model.update();
        },
		
		/**
         *  UpdateRowFields : This function is to update the grid row.
         *  This is called from the risk details section
         *  Incoming Arguments : The modofied row values is paased.
         *  Outgoing Argument : none
         *
         */
        UpdateRowFields: function(rowIndex, value, fieldname){
			if(fieldname && this._generalGrid)
			{
	            var data1 = this._generalGrid.model.getRow(rowIndex);
	            var item = this._generalGrid.model.store._getItemByIdentity(data1.id);
				this._generalGrid.model.store._setValueOrValues(item, fieldname, value, true);
				this.containerViewlet.toggleSaveButton(false);
			}
		},
        /**
         * This is get the first row of the grid.
         */
		getfirstRow: function(){
            var self = this;
            var data1;
            if (!self._controller) {
                var RiskViewController = ControllersFactory.getEPMRiskGridController("RiskViewController");
                data1 = RiskViewController.getRowByIndex(0);
            }
            else {
                data1 = self._controller.getRowByIndex(0);
            }
            return data1;
        },
        
        
        /**
         *  SaveGrid : This function is Save the modified grid values to server.        
         */
        SaveGrid: function(){
            this.persistGrid(true, true);
        },
        
        Refresh: function(){
			this.refreshGrid();
        },
		
		 /**
         *  rowUpdated : This function is called from the controller after the save.
         *  This publishes a risk update event, to update all the sections
         */
		rowUpdated: function(row, rowID, rowIndex){
            if (this._generalGrid) {
				this._generalGrid.updateRow(rowID);
				if (rowIndex >= 0 && this._controller && (rowIndex === this._generalGrid.selection.selectedIndex)) {
					var data1 = this._controller.getRowByIndex(rowIndex);
					data1.rowIndex = rowIndex;
					dojo.publish(RiskMessages.RISKUPDATE, [data1]);
				}
				this.containerViewlet.toggleSaveButton(true);
			}
        },
		
		deleteRisks: function(){   
            // Get selected Rows and then call delete on them in a loop
            var selectedRows = this._generalGrid.selection.getSelected();
            
            for (var rowIndex = 0; rowIndex < selectedRows.length; rowIndex++) {            
                this._controller._deleteElement(selectedRows[rowIndex]);                
            }            
        },
		
		/*
         * Deletes the Risk
         */
        deleterisks: function(){
        
            // Get selected Rows and then call delete on them in a loop
            var selectedRows = this._generalGrid.selection.getSelected();
            if (selectedRows.length !== 0) {
                var self = this;
                var buttonlist;
                var dialogContent;
                if (selectedRows.length > 1) {
                    buttonlist = [{
                        label: RiskMessages.YES,
                        action: dojo.hitch(this, this.deleteOne)
                    }, {
                        label: RiskMessages.YESTOALL,
                        action: dojo.hitch(this, this.deleteAll)
                    }, {
                        label: RiskMessages.NO,
                        action: dojo.hitch(this, this.noDelete)
                    }, {
                        label: RiskMessages.NOTOALL
                    }];
                    dialogContent = RiskMessages.DIALOG_CONTENT_MULTIPLE_RISKS_SELECTED;
                }
                else {
                    buttonlist = [{
                        label: RiskMessages.YES,
                        action: dojo.hitch(this, this.deleteAll)
                    }, {
                        label: RiskMessages.NO
                    }];
                    dialogContent = RiskMessages.DIALOG_CONTENT_SINGLE_RISK_SELECTED;
                }
                
                var labelList = [{
                    label: dialogContent
                }];
                
                this.confirmDialog = new com.ibm.epm.web.core.ui.internal.EPMConfirmDialog({
                    labelList: labelList,
                    title: RiskMessages.DIALOG_TITLE,
                    buttonlist: buttonlist
                });
                this.confirmDialog.show();
            }
            
        },
        
        deleteAll: function(){
        
            var selectedRows = this._generalGrid.selection.getSelected();
            for (var rowIndex = 0; rowIndex < selectedRows.length; rowIndex++) {
                this._controller._deleteElement(selectedRows[rowIndex]);
            }
            this.confirmDialog.hide();
            this.confirmDialog.destroy();
        },
        
        deleteOne: function(){
            var selectedRows = this._generalGrid.selection.getSelected();
            if (selectedRows.length !== 0) {
                this._controller._deleteElement(selectedRows[0]);
                this._generalGrid.selection.unselectAll();
                for (var i = 1; i < selectedRows.length; i++) {
                    this._generalGrid.selection.addToSelection(selectedRows[i] - 1);
                }
                if (selectedRows.length === 1) {
                    this.confirmDialog.hide();
                    this.confirmDialog.destroy();
                }
            }
        },
        
        noDelete: function(){
            var selectedRows = this._generalGrid.selection.getSelected();
            if (selectedRows.length !== 0) {
                this._generalGrid.selection.deselect(this._generalGrid.selection.getFirstSelected());
                if (selectedRows.length === 1) {
                    this.confirmDialog.hide();
                    this.confirmDialog.destroy();
                }
            }
        },
		
		/**
         * This method handles the error returned from the save operation.
         * Cleans the returned message to remove the Exception and displays the message
         **/
		handleError: function(errorMessage){
            this.inherited("handleError", arguments);
			var indx = errorMessage.errorMessage ? errorMessage.errorMessage.indexOf(":") + 1  : 0;
			if(indx > 0)
			{
				errorMessage.errorMessage = errorMessage.errorMessage.substring(indx);
			}
            this._displayError(errorMessage);
			if (this.containerViewlet) {
				this.containerViewlet.toggleSaveButton(false);
			}
        }
     });
})();
