/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.resource.web.ui.internal.StaticPoolAssignedUsersGridView");
    
    dojo.require("com.ibm.epm.web.core.grid.view.AbstractGridView");
    dojo.require("com.ibm.epm.resource.web.ui.internal.StaticPoolAssignedUsersGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    
    //service
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    //client
    dojo.require("com.ibm.epm.resource.web.client.internal.StaticPoolClient");
    // Constants
    dojo.require("com.ibm.epm.resource.web.ui.internal.ResourceConstants");
    // Localization
    dojo.requireLocalization("com.ibm.epm.resource.web.ui", "Messages");
    
    var Constants = com.ibm.epm.resource.web.ui.internal.ResourceConstants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var abstractView = com.ibm.epm.web.core.grid.view.AbstractGridView;
    var StaticPoolMessages = dojo.i18n.getLocalization("com.ibm.epm.resource.web.ui", "Messages");
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var StaticPoolClient = com.ibm.epm.resource.web.client.internal.StaticPoolClient;
    
    dojo.declare("com.ibm.epm.resource.web.ui.internal.StaticPoolAssignedUsersGridView", abstractView, {
        templatePath: dojo.moduleUrl("com.ibm.epm.resource.web", "ui/internal/templates/StaticPoolAssignedUsersGrid.html"),
        
        /*
         * Passing in a constructor arg of modifyAllowed = false will prevent the editing of
         * this view. This is needed for when a user is not allowed to make changes due to
         * having lack of permissions.
         */
        _modifyAllowed: true,
        
        constructor: function(args) {
            /*
             * If modifyAllowed is passed in and set to false, it'll disable
             * the editing of this grid.
             */
            if (args && (args.modifyAllowed !== undefined))
                this._modifyAllowed = args.modifyAllowed;
		  
        },
		
		postCreate:function(){
			this._addToolTip();
		},
       
	   /*
	    * Adding tooltip for the add anchor which is used for adding 
	    * resource to selected pool.
	    * Incoming : None
	    */
	   _addToolTip:function(){
		  this.anchor=document.getElementById(Constants.ADDUSER_ANCHOR);
		  this.anchor.title=StaticPoolMessages.ADD_ANCHOR_TOOLTIP;
	   },
	   
        /* 
         * buildStaticPoolsGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with StaticPoolsGrid.
         */
        buildStaticPoolAssignedUsersGrid: function(staticpoolname){
             // Label to indicate associate deassociate user
            this._createPoolNameLabel(staticpoolname);
            
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.resource.web.ui.internal.StaticPoolAssignedUsersGridModel(staticpoolname);
            }
            this.StaticPoolAssignedUsersController = controllerUtil.getEPMGridController(Constants.STATICPOOLASSIGNEDUSERS_GRID_TITLE);
            this.buildGrid(this, this._gridModel, this.StaticPoolAssignedUsersController);
        },
        
        
        /*
         * creating label with selected static pool name
         */
        _createPoolNameLabel: function(staticpoolname){
            var usersLabel = document.createElement("label");
            usersLabel.innerHTML = StaticPoolMessages.USERS_ASSOCIATION + " : -  \" " + staticpoolname + " \"";
            //this.LabelForUsers.appendChild(usersLabel);
        
        },
        
        /*
         * getGridTitle funtion - Override the abstract class and returns the title of the grid.
         *
         */
        getGridTitle: function(){
            return StaticPoolMessages.STATICPOOLASSIGNEDUSERS_GRID_TITLE;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.staticpoolAssignedusersGridAttachPoint;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var gridLayout = [{
                noscroll: false,
                rows: [[{
                    name: StaticPoolMessages.ASSNDUSERS_GRID_COLUMNNAME,
                    width: "95%",
                    field: "name"
                }, {
                    name: StaticPoolMessages.DELETE,
                    width: "5%",
					value: '<img alt="' + StaticPoolMessages.UNASSIGN_BUTTON_LABEL + '" src ="' + dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/remove.gif") + '" />'
                }]]
            }];
            
            return gridLayout;
        },
        
        handleError: function(errorObject){
            this.inherited("handleError", arguments);
            this._displayError(errorObject);
        },
        
        /* Adds Assigned Users to the Grid
         * Incoming : staticPoolHref selected static pool's href
         * Outgoing : None
         */
        addUsers: function(staticPoolHref){
            var self = this;
            var serviceObj = {
                success: function(data){
                    if (data) {
                        if (data.User) {
                            for (var i = 0; i < data.User.length; i++) {
                                var row = {
                                    name: data.User[i].title,
                                    relativeHref: data.User[i].relativeHref + "_" + Constants.UNSAVED,
                                    href: data.User[i].relativeHref
                                
                                };
                                self.StaticPoolAssignedUsersController._insertElement(row, null, false);
                            }
                        }
                    }
                    
                },
                
                failure: function(errorObj){
            		self._displayError(errorObj.errorMessage);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            StaticPoolClient.getURI(srh, staticPoolHref);
        },
        
        _addUserToStaticPool: function(selectedUserRow, selectedStaticPoolHref){
        
            for (var i = 0; i < selectedUserRow.length; i++) {
            
                this._internalAddUsertoStaticPool(selectedUserRow[i], selectedStaticPoolHref);
                
            }
            
        },
        
        /*
         * Gets all the Grid Rows
         */
        getAllAssignedUsers: function(){
        
            // Get the grid data model
            var userIdList = [];
            var _gridData = this._controller.getDataModel();
            if (_gridData != null) {
                var count = _gridData.getRowCount();
                
                for (var i = 0; i < count; i++) {
                
                    if (_gridData.getRow) {
                    
                        var rowData = _gridData.getRow(i);
                        
                        if (rowData) {
                            userIdList.push({
                                userId: rowData.href.substring(rowData.href.indexOf(Constants.USER_URI + Constants.SLASH) + (Constants.USER_URI + Constants.SLASH).length)
                            });
                        }
                    }
                    
                }
            }
            return userIdList;
            
        },
        /*
         * Add User to the Assigned users Grid
         */
        _internalAddUsertoStaticPool: function(selectedUserRow, selectedStaticPoolHref){
        
            var self = this;
            var href = Constants.USER_URI + Constants.SLASH + selectedUserRow.userId;
            var serviceObj = {
                success: function(data){
                    // Add the user to the grid. Adding unsaved tag will stop the Flat grid implementation to 
                    // fetch the user details form the server
                    
                    var row = {
                        name: selectedUserRow.name,
                        relativeHref: Constants.USER_URI + Constants.SLASH + selectedUserRow.userId + "_" + Constants.UNSAVED,
                        href: Constants.USER_URI + Constants.SLASH + selectedUserRow.userId
                    };
                    self.StaticPoolAssignedUsersController._insertElement(row, null, false);
					self.scrollToSelectedRow();
                },
                
                failure: function(errorObj){
            		self._displayError(errorObj.errorMessage);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            StaticPoolClient.assignUser(srh, selectedStaticPoolHref, href);
        },
        
        /*
         * removing the user from the selected staticpool
         * Incoming : None
         * Outgoing: None
         */
        _removeUsersFromStaticPool: function(selectedStaticPoolHref, rowIndex){
            var row;
            if (this._generalGrid) {
                row = this._generalGrid.model.getRow(rowIndex);
                this._removeUserFromStaticPool(row, selectedStaticPoolHref);
                //				var selectedRows = this._generalGrid.selection.getSelected();
                //				if (selectedRows) {
                //					for (var rowIndex = 0; rowIndex < selectedRows.length; rowIndex++) {
                //						row = this._generalGrid.model.getRow(selectedRows[rowIndex]);
                //						this._removeUserFromStaticPool(row, selectedStaticPoolHref);
                //					}
                //				}
            }
        },
        
        _removeUserFromStaticPool: function(selectedRow, selectedStaticPoolHref){
            var self = this;
            var serviceObj = {
                success: function(){
                    // Remove the Row after unlinking the user to pool
                    if (self._generalGrid) {
                        self.StaticPoolAssignedUsersController._removeGridRow(selectedRow.id);
                    }
                    
                },
                
                failure: function(errorObj){
            		self._displayError(errorObj.errorMessage);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            StaticPoolClient.unassignUser(srh, selectedStaticPoolHref, selectedRow.href);
        },
        
        /*
         * Removes all rows from the grid
         */
        removeAllRows: function(){
        
            var _gridData = this._controller.getDataModel();
            
            if (_gridData) {
            
                var count = _gridData.getRowCount();
                
                for (var i = 0; i < count; i++) {
                
                    var row = _gridData.data[0];
                    
                    if (row) {
                        _gridData.remove([0]);
                        _gridData.store.save();
                    }
                    
                }
            }
        },
        
        
        _refreshGrid: function(){
            this.destroy();
            this.buildStaticPoolAssignedUsersGrid();
        },
		 /**
         * getGridInternalID: Used to set the unique ID of the grid, needs to be overridden by the inheriting view
         * Incoming:
         *	gridModel
         * Outgoing:
         *  none
         */
        getGridInternalID: function(){            
            return Constants.STATICPOOLASSIGNEDUSERS_GRID_TITLE;
        }
		
    });
})();

