/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){

    dojo.provide("com.ibm.epm.resource.web.ui.internal.StaticPoolsTreeWidget");
    dojo.requireLocalization("com.ibm.epm.resource.web.ui", "Messages");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
    
    //TreeUtil
    dojo.require("com.ibm.epm.web.core.tree.TreeUtil");
    // Base class definition of EPMTree
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTree");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    
    // Constants
    dojo.require("com.ibm.epm.resource.web.ui.internal.ResourceConstants");
    
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    //TreeUtil
    var TreeUtil = com.ibm.epm.web.core.tree.TreeUtil;
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.resource.web.ui", "Messages");
    // Base class definition of EPMTree
    var EPMTree = com.ibm.epm.web.core.ui.internal.EPMTree;
    var IconClass = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
    var Constants = com.ibm.epm.resource.web.ui.internal.ResourceConstants;
    
    dojo.declare("com.ibm.epm.resource.web.ui.internal.StaticPoolsTreeWidget", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.resource.web", "ui/internal/templates/StaticPoolsTree.html"),
        
        _projectID: null,
        _lastUniqueRowID: 0,
        
        /*
         * This widget can be used both to create/rename pools as well as to
         * let the user select from a bunch of existing pools in a dialog box.
         * The latter case doesn't need all those extra features, so passing in
         * a constructor arg of modifyAllowed = false will disable the unnecessary
         * features.
         *
         * _modifyAllowed can also be set false if the user simply does not have
         * permissions to modify pools.
         */
		_modifyAllowed: true,
		
		// stores the various attributes of the 'Rename' menu item
		renameItem: {},
		
        constructor: function(args){
            /*
             * If modifyAllowed is passed in and set to false, it'll disable
             * the extra features not needed just to select pools from a dialog
             * box.
             */
            if (args && (args.modifyAllowed !== undefined))
				this._modifyAllowed = args.modifyAllowed;
        },
		
        /*
         * Creates the tree label & initiates context menu
         *
         * - Context menu only used in pools viewlet, not in dialog box.
         */
        postCreate: function(){
        	
			// initialise the 'Rename' menu item
            if (this._modifyAllowed) {
                this.renameItem = {
                    "type": EPMConstants.MENU_TYPE,
                    "id": EPMConstants.CODE_RENAME_ID,
                    "parentID": null,
                    "addSeparator": true,
                    label: EPMConstants.CODE_RENAME_LABEL,
                    iconClass: IconClass.RENAME_ICON_CLASS,
                    _onClick: dojo.hitch(this, this._rename)
        	    };
			}
			
            /*
             * Context Menu
             */
            this._subscriptions = [];
            if (this._modifyAllowed) {
                this._ctxtItems = null; // _TreeNode, the active tree node in right-clicking  for context menu
                this._initMenu();
                this.eventSubscribe();
            }
        },
		
		/**
         *  setctxMenu : the method will be called when the _addDropDownIcon method is called
         *  in the EPMTree.js
         *  Incoming : event object
         *  Outgoing : none
         *
         */
		setctxMenu: function(nodeItem){
			// If element is new(unsaved) then resourceType passed is EPMConstants.NEWElEMENTSUFFIX
			if (this._StaticPoolsTree._isNodeNew(nodeItem)) {
				this._buildNewElementMenu();
			}
			else{
				this._StaticPoolsTree.createMenu(this._ctxtItems);
			}
		},
		
        /**
         * _rename: called when user clicks on 'Rename' menu item.
         * @param {Object} TreeNode
         * Outgoing: none.
         */
        _rename: function(e){
            this._StaticPoolsTree.closeMenu();
            this._ctxNode = this._StaticPoolsTree.getNodeContext();
            this._StaticPoolsTree._renameNode(this._ctxNode);
        },
		
		/**
		 * _buildNewElementMenu: Method to build menu for a new element. <br>
		 */
		_buildNewElementMenu: function(){
			this._StaticPoolsTree.createMenu([this.renameItem]);
		},
		
        eventSubscribe: function(){
            this._subscriptions = [
			dojo.subscribe(Constants.ON_NEW_STATICPOOL, this, "_createStaticPoolElement"), 
			dojo.subscribe(GridEvents.REPLACE_NODE, this, "_replaceStaticPoolNode"),
			dojo.subscribe(GridEvents.ROW_DELETED, this, "_rowDeleted")];
 	    },
        
        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEvents: function(){
            if (this._subscriptions !== null) {
                for (var topic in this._subscriptions) {
                    var handler = this._subscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._subscriptions = [];
        },
		
        /**
         * _getTreeBasedController : Returns the treebased grid controller
         * Incoming : None
         * Outgoing : None
         */
        _getTreeBasedController: function(){
            if (!this.TreeBasedGridController) {
                this.TreeBasedGridController = controllerUtil.getTreeBasedGridController(Constants.STATICPOOL_CONTROLLER);
            }
            return this.TreeBasedGridController;
        },
        
        /**
         *  _selectNodeOnMouseHover : Event handler when there is a mouseover event on the Grid
         *  Incoming : publishedObject
         *  Outgoing : none
         *
         */
        _selectNodeOnMouseHover: function(publishedItem){
            // Highlight the selected node on mousehover
            var row = this._validateAndGetRow(publishedItem);
            if (row) {
                this._StaticPoolsTree._highlightSpecifiedNode(row.id, true);
            }
        },
        
        /**
         *  _deselectNodeOnMouseOut : Event handler when there is a mouseout event on the Grid
         *  Incoming : publishedObject
         *  Outgoing : none
         *
         */
        _deselectNodeOnMouseOut: function(publishedItem){
            // Deselect the highlighted node on mouseout
            var row = this._validateAndGetRow(publishedItem);
            if (row) {
                this._StaticPoolsTree._highlightSpecifiedNode(row.id, false);
            }
        },
        
        
        /**
         *  _selectNodeOnRowClick : Event handler when there is a row click event on the Grid
         *  Incoming : publishedObject
         *  Outgoing : none
         *
         */
        _selectNodeOnRowClick: function(row){
            if (row && row.id) {
				this._StaticPoolsTree._selectNodeById(row.id);
            }
        },
        
        /**
         * _validateAndGetRow: Method to validate the published data and verify the grid id.
         * Incoming: publishedItem by grid.
         * Outgoing: row selected/hovered upon.
         *
         */
        _validateAndGetRow: function(publishedItem){
            if (this._getTreeBasedController() && publishedItem.gridID == this._getTreeBasedController()._baseGridID) {
                if ((publishedItem) && (publishedItem.rowIndex > -1)) {
                    var row = this._getTreeBasedController().getRowByIndex(publishedItem.rowIndex);
                    return row;
                }
            }
            return null;
        },
        
        _buildStaticPoolsTree: function(){
            this._createTree();
            this._staticpoolsTreeContent.appendChild(this._StaticPoolsTree.domNode);
        },
        
        _createTree: function(){
            // Call the tree util to create the store
            
            this._StaticPoolsTree = dijit.byId(Constants.STATICPOOL_TREE_TITLE);
            if (!this._StaticPoolsTree) {
                var props = TreeUtil.getTreeStore(Constants.STATICPOOL_TREE_TITLE,Messages.POOL_TREE_ROOT_LABEL);
				props._containingWidget = this;
                this._StaticPoolsTree = new EPMTree(props);
            }
            this._StaticPoolsTree.startup();
            
            // Initialization Arguments : topLevelURI, linkType, ID
            var topLevelURI = Constants.STATICPOOL_URI;
            var linkType = Constants.CHILD_LINKNAME;
            this._StaticPoolsTree.initialize(topLevelURI, linkType, Constants.STATICPOOL_TREE_TITLE, 
                this._modifyAllowed ? [] : this._StaticPoolsTree.READONLY, this.menuImage);
                
            // Only create context menu for all nodes if required. If no menu is needed, then
            // header is not needed either.
            if (this._modifyAllowed) {
                this._StaticPoolsTree.createMenu(this._ctxtItems);
            
                if (this._projectID === null) {
                    var treeLabel = document.createElement("label");
                    treeLabel.innerHTML = Messages.STATICPOOLS_LABEL;
                    this._staticpoolsTreeLabel.appendChild(treeLabel);
                }
            }
            
            // If just showing a list of pools in a dialog, suppress the header
            else {
                dojo.style(this._StaticPoolsTree, "display", "none");
            }
        },
        
        /*
         * Defining menu items for StaticPools tree<br>
         * item.type: indicates whether its a popup menu or a menuItem. <br>
         * item.id: uniqueID of the item to be added. <br>
         * item.parentID: ID of the menu under which the item has to be created. <br>
         * item.addSeparator:Boolean value to depict whether to add a separator before adding item or not. <br>
         * item.label: label of the item. <br>
         * item.iconClass: the icon to be placed next to the label. <br>
         * item._onClick: the function to be performed when the menuItem is clicked. <br>
         */
        _initMenu: function(){
            this._ctxtItems = [
			{
                "type": EPMConstants.POPUP_TYPE,
                "id": EPMConstants.CREATE_ID,
                "parentID": null,
                "addSeparator": false,
                label: Messages.CREATE_LABEL,
                iconClass: IconClass.CREATE_ICON_CLASS
            }, {
                "type": EPMConstants.MENU_TYPE,
                "id": EPMConstants.CREATE_STATICPOOL_ID,
                "parentID": EPMConstants.CREATE_ID,
                "addSeparator": false,
                label: Messages.STATICPOOL_LABEL,
                iconClass: IconClass.STATICPOOLS_ICON_CLASS,
                _onClick: dojo.hitch(this, this._createStaticPool)
            },
			this.renameItem];
        },
        
        // Generate incremented number
        _incrementNumber: function(){
            this._lastUniqueRowID++;
            return this._lastUniqueRowID;
        },
        
        _getItemNodeByIdentity: function(identifier){
        
            var node = this._StaticPoolsTree.getNodeByItemIdentity(identifier);
            if (node) {
                return node.item;
            }
            else {
                return null;
            }
        },
        
        /*
         * Create new root level StaticPool
         * Incoming : None
         * Outgoing : None
         */
        _createRootStaticPool: function(){
        
            var tempID = this._incrementNumber();
                        
			var existingPoolNames = this._StaticPoolsTree.getRootLevelNames();
            var tempRootPoolname = this._getUniqueName(Messages.STATICPOOL_LABEL, existingPoolNames);
			var tempSPname = tempRootPoolname;
			
            var item = {
                "href": Constants.SLASH + Constants.STATICPOOL_URI + Constants.SLASH + tempID +
                Constants.NEW_ELEMENT_PREFIX,
                "name": tempSPname,
                "identifier": Constants.SLASH + Constants.STATICPOOL_URI + Constants.SLASH + tempID +
                Constants.NEW_ELEMENT_PREFIX,
                "detailLevel": false,
                "iconClass": IconClass.STATICPOOLS_ICON_CLASS,
                "resourcetype": Constants.SP_RESOURCE_TYPE
            };
            var publishedObject = {
				treeID: Constants.STATICPOOL_TREE_TITLE,
				item: item
			};
            this._buildNewElementMenu();
            this._StaticPoolsTree._addNode(null, item, 0);
            dojo.publish(EPMTreePubSub.ON_CREATE_NODE, [publishedObject]);
            this._StaticPoolsTree._selectNodeById(item.identifier);
            
        },
        
        _createStaticPool: function(e){
            var domNode = e.target;
            var menuWidget = dijit.getEnclosingWidget(domNode);
            var menuLabel = menuWidget.label;
            
            var tempID = this._incrementNumber();
            
            this._ctxNode = this._StaticPoolsTree.getNodeContext();
            var staticpoolname =this._getChildName(Messages.STATICPOOL_LABEL, this._ctxNode);
            
            if ((this._ctxNode.isExpandable == true) && (this._ctxNode.isExpanded == false)) {
                this._StaticPoolsTree._expandNode(this._ctxNode);
            }
            
            var parentID;
            if (dojo.isArray(this._ctxNode.item)) {
                if (dojo.isArray(this._ctxNode.item[0].identifier)) {
                    parentID = this._ctxNode.item[0].identifier[0];
                }
                else {
                    parentID = this._ctxNode.item[0].identifier;
                }
            }
            else {
                if (dojo.isArray(this._ctxNode.item.identifier)) {
                    parentID = this._ctxNode.item.identifier[0];
                }
                else {
                    parentID = this._ctxNode.item.identifier;
                }
            }
            var newnode = {
                "name": staticpoolname,
                "description": staticpoolname + " " + Messages.DESCRIPTION,
                "parentIdentifier": parentID
            
            };
            // Close the popup menu
            this._StaticPoolsTree.closeMenu();
            //			if (this._isMenuOpen) {
            //				this._closeMenu();
            //			}
            dojo.publish(Constants.ON_NEW_STATICPOOL, [newnode]);
            
            //this._createStaticPoolElement([newnode]);
            
        },
        
        /*
         * Create Static pool element
         * Incoming : row
         * Outgoing : None
         */
        _createStaticPoolElement: function(row){
        
            if (this._StaticPoolsTree) {
                var tempID = this._incrementNumber();
                var tempPoolname = row.name;
				
                //TODO : Move all strings to constant
                var item = {
                    "href": Constants.SLASH + Constants.STATICPOOL_URI + tempID + Constants.NEW_ELEMENT_PREFIX,
                    "name": tempPoolname,
                    "identifier": Constants.SLASH + Constants.STATICPOOL_URI + tempID + Constants.NEW_ELEMENT_PREFIX,
                    "parentIdentifier": row.parentIdentifier,
                    "detailLevel": false,
                    "iconClass": IconClass.STATICPOOLS_ICON_CLASS,
                    "resourcetype": Constants.SP_RESOURCE_TYPE
                };
                
                
                var node = this._getItemNodeByIdentity(row.parentIdentifier);
                
                // Since new element is added, Expand the current Tree Node to show the
                // new added child
                //var nodeWidget = this._wbsTree.lastFocused;
                if ((this._ctxNode.isExpandable == true) && (this._ctxNode.isExpanded == false)) {
                    this._StaticPoolsTree._expandNode(this._ctxNode);
                }
                this._buildNewElementMenu();
                this._StaticPoolsTree._addNode(node, item, 0);
                
                this._StaticPoolsTree._selectNodeById(item.identifier);
				var publishedObject = {
					treeID : Constants.STATICPOOL_TREE_TITLE
				}; 
				dojo.publish(EPMTreePubSub.ON_CREATE_NODE, [publishedObject]);
            }
        },
        
		 /**
         * _getChildName: fetches the list of children name and returns the unique name.
         * @param {Object} poolName: pool Name<br>
         * @param {Object} parentNode: parentNode under which the node is getting created.<br>
         * Outgoing: returns the unique name for the wbs element.<br>
         *
         */
        _getChildName: function(poolName, parentNode){
            var childNameCounter = 0;
            var childernNames = new Array();
            
            if (parentNode.hasChildren()) {
                var children = parentNode.getChildren();
                dojo.forEach(children, function(child){
                    childernNames.push(child.label);
                });
            }
            
            var uniqueName = this._getUniqueName(poolName, childernNames);
            return uniqueName;
        },
        
		
		 /**
         * _getUniqueName: retruns a unique name for the wbs element by comparing the name from the siblings.
         * Incoming:
         * @param {Object} poolName: constant representing the pool name
         * @param {Object} siblingNames: The name of siblings which shouldn't get duplicated.
         *
         * Outgoing: unique name of the WBS element.
         */
        _getUniqueName: function(poolName, siblingNames){
            var nameCounter = 0;
            do {
                nameCounter++;
                var proposedName = poolName + "_" + nameCounter;
                var nameAlreadyPresent = false;
                
                if (siblingNames) {
                    for (var i = 0; i < siblingNames.length; i++) {
                        if (proposedName == siblingNames[i]) {
                            nameAlreadyPresent = true;
                            break;
                        }
                    }
                }
            }
            while (nameAlreadyPresent == true);
            
            return proposedName;
        },
        // Replace the temp node in the tree with actual data from the server
        //Gets acitvated when Save operation is done by the Staticpool grid
        
        _replaceStaticPoolNode: function(publishedItem){
        
            var publishedData = null;
            var parentID = null;
            
            if (dojo.isArray(publishedItem)) {
                publishedData = publishedItem[0];
            }
            else {
                publishedData = publishedItem;
            }
            
            var localtreeId = publishedData.uniqueIdentifier.treeID;
            
            
            if (localtreeId == this._StaticPoolsTree.id && (this._StaticPoolsTree)) {
            
                var oldItem = this._getItemNodeByIdentity(publishedData.oldRowID);
                
                var newNodeData = publishedData.newObject;
                
                //_deleteNode
                //delete the old node and add the new node to the same parent node
                parentID = oldItem.parentIdentifier;
                this._StaticPoolsTree._deleteItemNode(oldItem);
                
                //Create the new item
                var resourceIconClass = newNodeData.resourcetype.substring(3) + "Icon";
                
                var newItem = {
                
                    "href": newNodeData.identifier,
                    "name": newNodeData.title,
                    "identifier": newNodeData.identifier, // using relatedHref itself as Identifier
                    "parentIdentifier": parentID,
                    "detailLevel": false,
                    "iconClass": IconClass.STATICPOOLS_ICON_CLASS,
                    "resourcetype": newNodeData.resourcetype
                
                };
                if (parentID) {
                    //Get the parent Item
                    var parentNode = this._getItemNodeByIdentity(parentID);
                    
                    //Add the new node
                    this._StaticPoolsTree._addNode(parentNode, newItem, 0);
                    
					// If the new is added due to collapse donot expand the parent node
					if (!publishedData.isParentCollapsed) {
						// Expand the parent node for the saved node
						this._StaticPoolsTree._expandNodeUsingID(parentID);
					}
                }
                else {
                    //Add the new node
                    this._StaticPoolsTree._addNode(null, newItem, 0);
                }
            }
        },
        
        /**
         *  _rowDeleted : Event handler when there is a row deleted on the Grid
         *  Incoming : publishedObject
         *  Outgoing : none
         *
         */
		_rowDeleted:function(publishedItem){
			var publishedData = null;
            if (dojo.isArray(publishedItem)) {
                publishedData = publishedItem[0];
            }
            else {
                publishedData = publishedItem;
            }
            var localtreeId = publishedData.uniqueIdentifier.treeID;
            
            if (localtreeId == Constants.STATICPOOL_TREE_TITLE && (this._StaticPoolsTree)) {
                if (publishedData.rowid) {
                    var nodeID = publishedData.rowid;
                    if (nodeID) {
                        var treeItem = this._getItemNodeByIdentity(nodeID);
                        this._StaticPoolsTree._deleteItemNode(treeItem);
												
						var publishObj = {
						 	treeID: localtreeId
						 };
						//Publishing this event so that the grid will be updated with latest tree visible 
						//elements. Take care of the ROW_DELETED event of the grid it shouldn't be published
						//on removal of row from the grid
	                	dojo.publish(EPMTreePubSub.ON_DELETE_NODE, [publishObj]);
                    }
                }
            }
			
		},


        _refreshTree: function(){
            this._StaticPoolsTree.refresh();
        },//refresh
        destroy: function(){ //Destroy datastore, treedata and tree
            if (this._StaticPoolsTree) {
                this._StaticPoolsTree.destroy();
            }
            TreeUtil.destroyTreeStore();
            delete this._StaticPoolsTree;
            delete TreeUtil;
            //unsubscribe to all events
            this._unsubscribeToAllEvents();
            
        }//destroy
    });
})();
