/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/


/**
 * Timesheet Viewlet for Approver
 */

(function(){
    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetApprovalViewlet");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridView");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.ProjectManagerTimesheetGrid");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.WeekTextBox");
    dojo.require("com.ibm.epm.web.core.app.internal.Util");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
	dojo.require("com.ibm.epm.timesheet.web.client.internal.TimesheetClient");
	dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConfirmDialog");
	dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
    
    dojo.requireLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    var TimesheetViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
    var WeekTextBox = com.ibm.epm.timesheet.web.ui.internal.WeekTextBox;
    var TimesheetGridView = com.ibm.epm.timesheet.web.ui.internal.TimesheetGridView;
    var epmUtil = com.ibm.epm.web.core.app.internal.Util;
    var ProjectManagerTimesheetGrid = com.ibm.epm.timesheet.web.ui.internal.ProjectManagerTimesheetGrid;
    var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
	var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
	var TimesheetClient = com.ibm.epm.timesheet.web.client.internal.TimesheetClient;
	var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
	var EPMDomNodeConstants = com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants;
	var EPMConfirmDialog = com.ibm.epm.web.core.ui.internal.EPMConfirmDialog;
	var DateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    var TimesheetAttributes = com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes;
	var DashboardUtil = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
	
    dojo.declare("com.ibm.epm.timesheet.web.ui.internal.TimesheetApprovalViewlet", Viewlet, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/templates/TimesheetApprovalViewlet.html"),
        contextHelpId:TimesheetConstants.APPROVER_TIMESHEET_VIEWLET_CONTEXT_HELP_ID,
		
		/**
		 * Function called by the viewlet constructor
		 */
        init: function(){
			this.inherited(arguments);

            this.projectManager = this._getProjectManagerID();
			/*
			 * Since, only the approver can edit the approver notes and also summary notes, 
			 * this flag is used to set the disabled state of the 2 editors appropriately.
			 * So, setting this field as true.
			 * 
			 */
			
            this.isProjectManager = true;
			
			/*
			 * To check whether the operation to be performed for change in week or different user selection.
			 */
			this._isWeekChange = false;
			this._newWeekSelected = null;
            
			this._previousUserSelected = null;
			
			this.resourceID = null;
            this.projectList = null;
            this.weekSelected = null;
			this._userGridRowIndex = null;
            this._createBasicToolbar();
            this._createWeekOfTextBox();
			this._createSelectUserLabel();
            this._createResourceDetailGrid();
        },
		
		/**
		 * 
		 * @param {Object} event
		 * 
		 * When the user is selected in the user grid, check if the timesheet of previously selected
		 * user is dirty or not. If it is, then invoke the save confirm operation, else
		 * load the timesheet of the selected user.
		 */
		switchResourceTimesheet: function(event){
			
			if(this.projectManagerTimesheetGrid){
				
				var rowIndex = event.rowIndex;
				var row = this.projectManagerTimesheetGrid.getRow(rowIndex);
				var projectList = this.projectManagerTimesheetGrid.getProjectList(row.href);
				
				if(!this.timesheetGrid){
					
					this.setProjectList(projectList);
					this.setResourceID(row.href);
					this._createResourceTimesheetGrid();
				} else {
					
					if(this.getResourceID() !== row.href){
					
						if (this.getSaveButtonStatus()) {
						
							this.setProjectList(projectList);
							this.setResourceID(row.href);
							this._createResourceTimesheetGrid();
						
						} else {
							
							this._previousUserSelected = this.getResourceID();
							this.setProjectList(projectList);
							this.setResourceID(row.href);
							this._isWeekChange = false;
							this._createTimesheetSaveDialog();
						}
					}
				}
			}
		},
		
		/**
         * On week change, if the user has updated any values in the grid for
         * the current week, then the user will prompted with the dialog;
         * The dialog contains a message asking the user whether to continue
         * or not.
         * The user can save the timesheet before continuing or
         * can discard the changes made in the grid.
         * 
         * Same function is called when the different user is selected in the users grid and 
         * the timesheet is in dirty state.
         */
        _createTimesheetSaveDialog: function(){
        
            var buttonlist = [{
                label: TimesheetViewletMessages.YES,
                action: dojo.hitch(this, this._yesButtonPressed)
            }, {
                label: TimesheetViewletMessages.NO,
                action: dojo.hitch(this, this._noButtonPressed)
            }, {
                label: TimesheetViewletMessages.CANCEL,
                action: dojo.hitch(this, this._resetPreviousSelection)
            }];
            
            var labelList = [{
                label: TimesheetViewletMessages.TIMESHEET_NOT_SAVED_DIALOG
            }, {
                label: TimesheetViewletMessages.TIMESHEET_SAVE_CONFIRMATION
            }];
            
            this.timesheetSaveDialog = new EPMConfirmDialog({
                labelList: labelList,
                title: TimesheetViewletMessages.TIMESHEET_DIALOG_NOT_SAVED,
                buttonlist: buttonlist
            });
			
			dojo.connect(this.timesheetSaveDialog.dialog, "onCancel", this, this._resetPreviousSelection);
            
            this.timesheetSaveDialog.show();
        },
        
        /**
         * When the "Yes" button is pressed in the Timesheet save dialog,
         * the timesheet for the current week is saved first and then
         * fresh timesheet is loaded for the week selected.
         */
        _yesButtonPressed: function(){
        
			this.timesheetSaveDialog.destroy();
            this._saveTimesheet(false);
            this._createResourceTimesheetGrid();
			if(this._newWeekSelected !==null){
				this.weekSelected = this._newWeekSelected;	
			}
			
        },
        
        /**
         * When the "No" button is pressed in the Timesheet save dialog,
         * the changes in the Timesheet grid for the current week are discarded
         * and the fresh timesheet is loaded for the week selected.
         */
        _noButtonPressed: function(event){
        
			this.timesheetSaveDialog.destroy();
            this._createResourceTimesheetGrid();
			if(this._newWeekSelected !==null){
				this.weekSelected = this._newWeekSelected;	
			}
        },
		
		/**
		 * 
		 * @param {Object} event
		 * 
		 * If the week is changed, then reset the week, else reset the selection of the user's grid.
		 */
		_resetPreviousSelection: function(event){
			
			this.timesheetSaveDialog.destroy();
			
			if(this._isWeekChange){
				
				this._isWeekChange = false;
				this._resetWeek();
			} else {
				
				this._resetUserGridSelection();
			}
				
		},
		
		_confirmAndRefresh: function()
		{
			if(!this.getSaveButtonStatus()){
				this._createTimesheetSaveDialog();
				return;
			}else{
				this._refresh();
			}
		},
		
		/**
		 * Reset the selection of the user. 
		 * Reset the projectList and the user selected
		 */
		_resetUserGridSelection: function(){
			
			if(this._previousUserSelected !== null){
				
				var rowIndex = this.projectManagerTimesheetGrid.getRowIndex(this._previousUserSelected);
				var row = this.projectManagerTimesheetGrid.getRow(rowIndex);
				var projectList = this.projectManagerTimesheetGrid.getProjectList(row.href);
				
				this.setProjectList(projectList);
				this.setResourceID(row.href);
				
				this.projectManagerTimesheetGrid.clearAndSelectRow(rowIndex);
			}
		},
		
		/**
		 * Reset the value of the week to previous week selected.
		 * If the user don't want to perform any operation, then 
		 * it is required the set the week value to previously selected week.
		 */
		_resetWeek: function(){
			
			this.weekTextBox.weekPicker._setValueAttr(this.weekSelected, true);
			if(this.weekTextBox.weekPicker._opened){
				this.weekTextBox.weekPicker.toggleOpenClose();	
				this.weekTextBox.weekPicker.focusNode.blur();
			}

		},
		
		/**
         * Week text box will enable the user to select the week of the timesheet.
         * This will trigger an action that will help to reload the data in the grid.
         */
        _createWeekOfTextBox: function(){
        
            var self = this;
            
            var weekLabel = document.createElement(EPMDomNodeConstants.LABEL);
            weekLabel.innerHTML = TimesheetViewletMessages.WEEK;
            this._weekLabelDIV.appendChild(weekLabel);
            
            this.weekTextBox = new WeekTextBox({
            
                onChange: function(value){
					
					if(value){
						self._reloadTimesheet(value);
					}
					this.weekPicker.focusNode.blur();
					//Explicitly setting the focus else where as the above call is not working
					self.refreshButton.focus();
					
                }
            });
            
            this.weekTextBox.buildView();
            this._weekPickerDIV.appendChild(this.weekTextBox.domNode);
			this.weekSelected = this.weekTextBox.getValue();
            
        },
        
        /**
         *
         * @param {Object} date
         *
         * If the week is changed, fetch the new timesheet of the resource for the new week selected.
         */
        _reloadTimesheet: function(date){
        
            if (this.weekSelected !== null) {
            
				var date1 = DateUtil.formatDate_Short(new Date(date));
                var date2 = DateUtil.formatDate_Short(new Date(this.weekSelected));
                
                if (date1 === date2) {
                    return;
                }
            }
            
            if (this.timesheetGrid) {
                this._weekChange(date);
            }
        },
		
		/**
         *
         * @param {Object} value
         * This event is triggered when the week is changed.
         * The new set of values are rendered in the grid taking one row at a time.
         */
        _weekChange: function(date){
        
			if (!this.getSaveButtonStatus()) {
					
				this._isWeekChange = true;
				this._newWeekSelected = date;
                this._createTimesheetSaveDialog();
            
			} else {
				
				if (this.toggleBusyStatusOfViewlet) {
                    this.toggleBusyStatusOfViewlet(true);
                }
				this.weekSelected = date;
                this._createResourceTimesheetGrid();
				
				if (this.toggleSaveButton) {
	                this.toggleSaveButton(true);
	            }
	            if (this.toggleSubmitButton) {
	                this.toggleSubmitButton(true);
	            }
			}
        },
		
        /**
         * Fetch the id of the logged in user.
         */
        _getProjectManagerID: function(){
        
            var loggedInUser = epmUtil.getLoggedInUser();
            return loggedInUser;
        },
        
        /**
         * Returns the value of the week selected
         */
        getWeekOfValue: function(){
            
			if (this.weekTextBox.getValue()) {
			  return this.weekSelected;
            }
            
        },
        
        /**
         * Returns the list of project to which the user is assigned under the specified project manager.
         * Not required for team member timesheet, so returns empty string.
         */
        getProjectList: function(){
            return this.projectList;
        },
        
        /**
         *
         * @param {Object} projectList
         * Set the list of project to which the user is assigned under the specified project manager.
         * Not required for team member timesheet, so returns empty string.
         */
        setProjectList: function(projectList){
            this.projectList = projectList;
        },
        
        /**
         * Fetch the selected resourceID
         */
        getResourceID: function(){
            return this.resourceID;
        },
        
        /**
         *
         * @param {Object} resourceID
         *
         * Set the selected resourceID
         */
        setResourceID: function(resourceID){
            this.resourceID = resourceID;
        },
        
        /**
         *
         * @param {Object} status
         * It shows the message while the viewlet is loading
         */
        toggleBusyStatusOfViewlet: function(status){
           // this.getSite().setLoading(status);
			
			if(status){
				DashboardUtil.showMessage(TimesheetViewletMessages.LOADING);
			}
			else{
				DashboardUtil.hideMessageArea();
			}
        },
		
		_createSelectUserLabel: function(){
			
			if(!this.selectUserLabel){
				this.selectUserLabel = document.createElement(EPMDomNodeConstants.LABEL);
			}
			
            this.selectUserLabel.innerHTML = TimesheetViewletMessages.SELECT_USER_LABEL;
            this._userSelectInfoDIV.appendChild(this.selectUserLabel);
		},
		
		_updateUserSelectedLabel: function(){
			
			if (this.selectUserLabel!=null) {
				this._userSelectInfoDIV.removeChild(this.selectUserLabel);					
				this.selectUserLabel=null;	
			}
		},
        
        /**
         * The basic toolbar will contain all the image buttons which are specific to any viewlet
         */
        _createBasicToolbar: function(){
        
			this.refreshButton = new dijit.form.Button({
                title: TimesheetViewletMessages.REFRESH,
				label : TimesheetViewletMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._confirmAndRefresh)
            });
			
			this.approveTSButton = new dijit.form.Button({
                title: TimesheetViewletMessages.APPROVE_TIMESHEET_ACTION,
				label : TimesheetViewletMessages.APPROVE_TIMESHEET_ACTION,
				showLabel : false,
                iconClass: "com-ibm-epm-timesheet-web-ui-internal-TimesheetNotesImages com-ibm-epm-timesheet-web-ui-internal-TimesheetNotesImages-approveTimesheetButtonIcon",
                onClick: dojo.hitch(this, this._approveTimesheet)
            });
			
			this.rejectTSButton = new dijit.form.Button({
                title: TimesheetViewletMessages.REJECT_TIMESHEET_ACTION,
				label : TimesheetViewletMessages.REJECT_TIMESHEET_ACTION,
				showLabel : false,
                iconClass: "com-ibm-epm-timesheet-web-ui-internal-TimesheetNotesImages com-ibm-epm-timesheet-web-ui-internal-TimesheetNotesImages-rejectTimesheetButtonIcon",
                onClick: dojo.hitch(this, this._rejectTimesheet)
            });
			
			this.saveButton = document.createElement(EPMDomNodeConstants.BUTTON);
            this.saveButton.innerHTML = TimesheetViewletMessages.SAVE;
            this.saveButton.title = TimesheetViewletMessages.SAVE;
			this.toggleSaveButton(true);
            dojo.connect(this.saveButton, EPMDomNodeConstants.ON_CLICK, this, this._saveTimesheet);
            
			
            var rightContentList = [this.approveTSButton, this.rejectTSButton, this.refreshButton, this.saveButton];
            
            this._toolBar = new EPMToolbar({
                rightContentList: rightContentList
            });
			            
			this.setCSHelpIdInToolbar(this.contextHelpId, this._toolBar);
			
            this._toolbarDIV.appendChild(this._toolBar.domNode);
        },
        
		/**
         * Returns the status of the save button
         */
        getSaveButtonStatus: function(){
            return this.saveButton.disabled;
        },
		
		/**
         * Save the timesheet
         */
        _saveTimesheet: function(){
        	this.toggleSaveButton(true);
			
			if (this.timesheetGrid) {
                this.timesheetGrid.saveTimesheet(false);
            }
        },
		
		
        /**
         * Instantiate the projectManagerTimesheetGrid and append the grid to the left nav of the splitter.
         */
        _createResourceDetailGrid: function(){
        
            this.projectManagerTimesheetGrid = new ProjectManagerTimesheetGrid({
                containerViewlet: this
            });
			
			if (this._userGridDIV) {
				this._userGridDIV.appendChild(this.projectManagerTimesheetGrid.domNode);
			}
        },
		
        /**
         *
         * @param {Object} resourceHref
         * @param {Object} projectList
         *
         * When the reosurce is selected in the resource grid, the appropriate timesheet
         * is loaded for the resource selected for the week selected in the week picker.
         */
        _createResourceTimesheetGrid: function(){
			
			
			//clear the label which asks user to select the user from list
            this._updateUserSelectedLabel();
			
			if(this.weekTextBox.getValue()){
				
				this.weekSelected = this.weekTextBox.getValue();
				this._newWeekSelected = null;
				this.toggleBusyStatusOfViewlet(true);
	            
				if (this.timesheetGrid) {
	                
					this._userTimesheetGridDIV.removeChild(this.timesheetGrid.domNode);
	                
	                this.timesheetGrid.destroy();
					delete this.timesheetGrid;
	            }
	            
				var queryURI = TimesheetClient.getApproverQueryURIForPost(this.getResourceID(), this.weekSelected, this.getProjectList());
				
				this.timesheetGrid = new TimesheetGridView({
	                _treeID: queryURI, containerViewlet: this
	            });
				
				this.timesheetGrid.setPostParams(TimesheetClient.getJSONExpressionForProjMgr(this.getResourceID(), this.weekSelected, this.getProjectList()));
				
				this.timesheetGrid.buildTimesheetGrid();
	            this._userTimesheetGridDIV.appendChild(this.timesheetGrid.domNode);
				
				if (this.toggleSaveButton) {
	                this.toggleSaveButton(true);
	            }
				

			}
            
        },
		
		
		
		/**
		 * This dialog box is instantiated if the approver wants to approve the 
		 * timesheet/task after editing and not saving the timesheet.
		 */
		_showTimesheetDirtyDialog: function(){
			
			var buttonlist = [{
                label: TimesheetViewletMessages.OK,
                action: dojo.hitch(this, this._okButtonPressed)
            }];
            
            var labelList = [{
                label: TimesheetViewletMessages.TIMESHEET_NOT_SAVED_DIALOG
            }, {
                label: TimesheetViewletMessages.TIMESHEET_PLEASE_SAVE_MESSAGE
            }];
            
            this.timesheetDirtyDialog = new EPMConfirmDialog({
                labelList: labelList,
                title: TimesheetViewletMessages.TIMESHEET_DIALOG_NOT_SAVED,
                buttonlist: buttonlist
            });
			
            this.timesheetDirtyDialog.show();
		},
		
        /**
         * When the "Yes" button is pressed in the Timesheet save dialog,
         * the timesheet for the current week is saved first and then
         * fresh timesheet is loaded for the week selected.
         */
        _okButtonPressed: function(){
        
            this.timesheetDirtyDialog.destroy();
        },
		
        /**
         * Approves the entire timesheet of the user selected.
         */
        _approveTimesheet: function(){
        
			if (!this.getSaveButtonStatus()) {
				
				this._showTimesheetDirtyDialog();
				return;
			}	
            if (this.timesheetGrid) {
                this.timesheetGrid.approveTimesheet(false);
            }
        },
        
        /**
         * Rejects the entire timesheet of the user selected.
         */
        _rejectTimesheet: function(){
        
			if (!this.getSaveButtonStatus()) {
				
				this._showTimesheetDirtyDialog();
				return;
			}
            if (this.timesheetGrid) {
                this.timesheetGrid.rejectTimesheet(false);
            }
        },
        
		_createUserTimesheet: function(){
			
			this.weekSelected = this.weekTextBox.getValue();
			
			var numberOfChildNodes = this._userTimesheetGridDIV.childNodes.length;
                        
            for (var i = 0; i < numberOfChildNodes; i++) {
                this._userTimesheetGridDIV.removeChild(this._userTimesheetGridDIV.childNodes[i]);
            }
            if (this.timesheetGrid) {
				
				delete this.timesheetGrid;
            }
		},
        
        /**
         * Reloads the entire grid (Resource grid and the timesheet grid)
         */
        _refresh: function(){
            
			this.toggleSaveButton(true);
			this.setResourceID(null);
			if (this._userGridDIV) {
				this._userGridDIV.removeChild(this.projectManagerTimesheetGrid.domNode);
			}
			
	        if (this.projectManagerTimesheetGrid) {
            
                this.projectManagerTimesheetGrid._cleanup();
                this.projectManagerTimesheetGrid.destroy();
                delete this.projectManagerTimesheetGrid;
            }
			
			this._createUserTimesheet();
            this._createResourceDetailGrid();
			this._createSelectUserLabel();
			
        },
		
		/**
         *
         * @param {Object} state
         * Save button is enabled once there is any updation in the grid,
         * else it is set to disabled.
         */
        toggleSaveButton: function(state){
        	
			this.saveButton.disabled = state;
			if(state==false)
			{
				this.setDirtyFlag();
			}
			else
			{
				this.resetDirtyFlag();
			}
        },
		
        /**
         *
         * @param {Object} errorMsg
         */
        _displayErrorMessage: function(errorMsg){
            EPMErrorHandler.setError(errorMsg);
			var self = this;
			var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);
        },
        
        /**
         *
         */
        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        },
		
		 /**
		 * refresh 
		 * Called from framework, useful while switching views/tabs
		 * to clear unsaved or unwanted grid data
		 */
        refresh: function(){
			
			//Just call internal implementation of timesheet viewlet refresh 
			//as all the states of widgets are restored here. 
			this._refresh();
			
		},
		
		/**
		 * 
		 * @param {Object} active
		 * 
		 * The week text box if clicked and no date is changed and at the same time if the user clicks on some other tab, 
		 * the weekPicker dialog is still open.
		 * So, check if the viewlet is active or not. If not active, check if the week picker is open.
		 * If open close the week picker dialog and remove the focus.
		 */
		setActive: function(active){
			this.inherited(arguments);
			if(!active){
				if(this.weekTextBox.weekPicker._opened){
					this.weekTextBox.weekPicker.toggleOpenClose();	
					this.weekTextBox.weekPicker.focusNode.blur();
				}
			}
		},
        
        /**
         * Destroy all the objects created in the viewlet.
         */
        destroy: function(){
			if (this._userGridDIV) {
				this._userGridDIV.removeChild(this.projectManagerTimesheetGrid.domNode);
			}
			
            if (this.projectManagerTimesheetGrid) {
            
                this.projectManagerTimesheetGrid._cleanup();
                this.projectManagerTimesheetGrid.destroy();
                delete this.projectManagerTimesheetGrid;
            }
			
			if (this.timesheetGrid) {
				
				delete this.timesheetGrid;
            }
        }
    });
    
})();
