/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/


(function(){
    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridModel");
    
    dojo.require("com.ibm.epm.web.core.grid.model.AbstractGridModel");
    dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil");
	dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
	
	dojo.require("dojo.date");
    dojo.require("dojo.date.locale");
	
    var AbstractGridModel = com.ibm.epm.web.core.grid.model.AbstractGridModel;
    var TimesheetAttributes = com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes;
	var TimesheetUtil = com.ibm.epm.timesheet.web.ui.internal.TimesheetUtil;
	var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
	var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
	
    dojo.declare("com.ibm.epm.timesheet.web.ui.internal.TimesheetGridModel", AbstractGridModel, {
    
		submit: false,
		approve: false,
		reject: false,
		actionOnSelected: false,
		
		constructor: function(){
			
			this.containerViewlet = arguments[0].containerViewlet;
		},
		
        /*
         * This is called by controller. Returns the empty row structure to the controller.
         */
        getEmptyRow: function(assignmentURI, queryResultObject){
			
			var item = {};
			
			item.id = assignmentURI;
			item.name = "";
            item[TimesheetAttributes.assignmentURI] = assignmentURI;
            item[TimesheetAttributes.sundayRegular] = "00:00";
            item[TimesheetAttributes.mondayRegular] = "00:00";
            item[TimesheetAttributes.tuesdayRegular] = "00:00";
            item[TimesheetAttributes.wednesdayRegular] = "00:00";
            item[TimesheetAttributes.thursdayRegular] = "00:00";
            item[TimesheetAttributes.fridayRegular] = "00:00";
            item[TimesheetAttributes.saturdayRegular] = "00:00";
            item[TimesheetAttributes.remainingWork] = "00:00";
            item[TimesheetAttributes.percentageComplete] = "";
            item[TimesheetAttributes.percentageProgress] = "";
            item[TimesheetAttributes.totalActualHours] = "00:00";
            item[TimesheetAttributes.containingProject] = "";
            item[TimesheetAttributes.summaryNotes] = "";
            item[TimesheetAttributes.approvalNotes] = "";
            item[TimesheetAttributes.timesheetApprovalState] = TimesheetConstants.TASK_UNSUBMITTED;
            item[TimesheetAttributes.timesheeturi] = "";
            item[TimesheetAttributes.timesheeturilist] = "";
            item[TimesheetAttributes.weekOf] = "";
            item[TimesheetAttributes.action] = "";
            item[TimesheetAttributes.title] = "";
            item[TimesheetAttributes.timecode] = "";
            item[TimesheetAttributes.timecodeURI] = "";
            item[TimesheetAttributes.taskURI] = "";
            item[TimesheetAttributes.actualScheduleStartDate] = "";
            item[TimesheetAttributes.actualScheduleFinishDate] = "";
            item[TimesheetAttributes.plannedScheduleStartDate] = "";
            item[TimesheetAttributes.plannedScheduleFinishDate] = "";
            item[TimesheetAttributes.plannedScheduleEffort] = "";
			item[TimesheetAttributes.additionalEffort] = 0;
			
			//by default its Effort type, will modify it later
			item[TimesheetAttributes.taskAssignmentType] = TimesheetAttributes.taskAssignmentTypeEffort;
			
			item[TimesheetAttributes.hoursID] = "";
			item[TimesheetAttributes.actions] = " ";
			
			item.totalWorkPerWeek = "00:00";
			
			return item;
            
        },
        
		/**
		 * 
		 * @param {Object} taskItem
		 * @param {Object} loadedElement
		 * 
		 * Update the actual and planned schedule of the task
		 */
		_updateSchedule: function(taskItem, loadedElement){
			
			taskItem[TimesheetAttributes.plannedScheduleStartDate] = TimesheetUtil.utilGetDateObjFromServerDate(loadedElement[TimesheetAttributes.plannedSchedule][TimesheetAttributes.startDate]);
        	taskItem[TimesheetAttributes.plannedScheduleFinishDate] = TimesheetUtil.utilGetDateObjFromServerDate(loadedElement[TimesheetAttributes.plannedSchedule][TimesheetAttributes.finishDate]);
       	 	taskItem[TimesheetAttributes.plannedScheduleEffort] = loadedElement[TimesheetAttributes.plannedSchedule][TimesheetAttributes.effort];
			taskItem[TimesheetAttributes.plannedScheduleDuration] = loadedElement[TimesheetAttributes.plannedSchedule][TimesheetAttributes.duration];

        	taskItem[TimesheetAttributes.actualScheduleStartDate] = TimesheetUtil.utilGetDateObjFromServerDate(loadedElement[TimesheetAttributes.actualSchedule][TimesheetAttributes.startDate]);
        	taskItem[TimesheetAttributes.actualScheduleFinishDate] = TimesheetUtil.utilGetDateObjFromServerDate(loadedElement[TimesheetAttributes.actualSchedule][TimesheetAttributes.finishDate]);
			taskItem[TimesheetAttributes.totalActualHours] = loadedElement[TimesheetAttributes.actualSchedule][TimesheetAttributes.effort];
			taskItem[TimesheetAttributes.totalActualDuration] = loadedElement[TimesheetAttributes.actualSchedule][TimesheetAttributes.duration];
			
			taskItem[TimesheetAttributes.remainingWork] = loadedElement[TimesheetAttributes.remainingWork];
            taskItem[TimesheetAttributes.percentageProgress] = loadedElement[TimesheetAttributes.percentageProgress];

			taskItem[TimesheetAttributes.additionalEffort] = loadedElement[TimesheetAttributes.additionalEffort];
			taskItem[TimesheetAttributes.percentageComplete] = loadedElement[TimesheetAttributes.percentageComplete];

			taskItem.loadedelement = [loadedElement];			
		},		
		
		/**
		 * 
		 * @param {Object} taskItem
		 * @param {Object} loadedElement
		 * 
		 * Fetch the task details from the task assignment and update the same 
		 * in the grid.
		 */
		_updateTaskFields: function(taskItem, loadedElement){
			
			var taskLink = getLink(loadedElement, TimesheetAttributes.TaskAssigned);
            if (taskLink) {
				
				taskItem[TimesheetAttributes.taskURI] = taskLink[TimesheetAttributes.relativeHref];
				
				var title = taskLink[TimesheetAttributes.title];
				var titleObj = dojo.fromJson(title);
				taskItem.name = titleObj.summary;
				taskItem[TimesheetAttributes.title] = titleObj.summary;
			}
			taskItem.loadedelement = [loadedElement];
		
		},
        
		/**
		 * 
		 * @param {Object} taskItem
		 * @param {Object} loadedElement
		 * 
		 * Check if the timesheets is present for the task assignment.
		 * If yes, compare with the current week selected and update the
		 * timesheet uri. 
		 */
		_updateTimesheetsLink: function(taskItem, loadedElement){
			
			var timesheetLink = getLink(loadedElement, TimesheetAttributes.Timesheets);
            if (timesheetLink) {
				
				var currentWeek = this.containerViewlet.getWeekOfValue();
                var currentWeekIndex = -1;
                var timesheeturilist = ''; // 'weekof1:uri1;weekof2:uri2;...'
                for (var i = 0; i < timesheetLink.length; i++) {
                
                    var tempDate = TimesheetUtil.utilGetDateObjFromServerDate(timesheetLink[i][TimesheetAttributes.title]);
                    timesheeturilist += timesheetLink[i].title + ':' + timesheetLink[i][TimesheetAttributes.relativeHref] + ';';
                    if (dojo.date.compare(currentWeek, tempDate, 'date') === 0) {
                    
                        currentWeekIndex = i;
                    }
                }
                
                taskItem[TimesheetAttributes.timesheeturilist] = timesheeturilist;
                var timesheetURI = ''; // create empty row for new summary timesheet
                if (currentWeekIndex != -1) { // if present
					timesheetURI = timesheetLink[currentWeekIndex][TimesheetAttributes.relativeHref];
					taskItem[TimesheetAttributes.timesheeturi] = timesheetURI;
				}
				else {
					/*
					 * In this case, there is no timesheet present for the current week.
					 * But the timesheet link is there which contains the links to timesheets of
					 * previous and next week.
					 * So, the user should be allowed to submit the timesheet, which will first create the timesheet and then
					 * submit the empty timesheet even if the values are not entered.
					 */
					if (this.containerViewlet.toggleSubmitButton) {
						this.containerViewlet.toggleSubmitButton(false);
					}
				}
			} else {
				/*
				 * If there is no timesheet link is present, it implies no timesheet object 
				 * is created at the server. So, the user should be allowed to submit the timesheet which 
				 * will first create the timesheet and will submit the timesheet with values entered.
				 */
				if (this.containerViewlet.toggleSubmitButton) {
					this.containerViewlet.toggleSubmitButton(false);
				}
			}
			
		},
		/**
		 * 
		 * @param {Object} taskItem
		 * @param {Object} timesheet
		 */
		_updateTimesheetsField: function(taskItem, timesheet){
			
			if(timesheet){
				
				if(timesheet[TimesheetAttributes.timesheetApprovalState]) {
					
	                taskItem[TimesheetAttributes.summaryNotes] = timesheet[TimesheetAttributes.summaryNotes];
	                taskItem[TimesheetAttributes.approvalNotes] = timesheet[TimesheetAttributes.approvalNotes];
	                taskItem[TimesheetAttributes.timesheetApprovalState] = timesheet[TimesheetAttributes.timesheetApprovalState];
	                
	                if (taskItem[TimesheetAttributes.timesheetApprovalState] === "" || taskItem[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TASK_UNSUBMITTED || taskItem[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TASK_REJECTED) {
	                    if (this.containerViewlet.toggleSubmitButton) {
	                        this.containerViewlet.toggleSubmitButton(false);
	                    }
	                } 
	                
					if (timesheet[TimesheetAttributes.TimeCode]) {
	                	taskItem[TimesheetAttributes.timecodeURI] = "";
						taskItem[TimesheetAttributes.timecode] = "";
	                    var numberOfTimecodes = timesheet[TimesheetAttributes.TimeCode].length;
	                    for (var i = 0; i < numberOfTimecodes; i++) {
	                    
	                        if (taskItem[TimesheetAttributes.timecodeURI] !== "") {
	                            taskItem[TimesheetAttributes.timecodeURI] = taskItem[TimesheetAttributes.timecodeURI] + "," + timesheet[TimesheetAttributes.TimeCode][i][TimesheetAttributes.relativeHref];
	                            taskItem[TimesheetAttributes.timecode] = taskItem[TimesheetAttributes.timecode] + "," + timesheet[TimesheetAttributes.TimeCode][i][TimesheetAttributes.title];
	                        }
	                        else {
	                            taskItem[TimesheetAttributes.timecodeURI] = timesheet[TimesheetAttributes.TimeCode][i][TimesheetAttributes.relativeHref];
	                            taskItem[TimesheetAttributes.timecode] = timesheet[TimesheetAttributes.TimeCode][i][TimesheetAttributes.title];
	                        }
	                    }
						taskItem.postTimecode = "";
	                    
	                }
				}
                
                if(timesheet[TimesheetAttributes.hours]){
					
					var hoursID = timesheet[TimesheetAttributes.hours][TimesheetAttributes.id];
                
					var sundayRegular = timesheet[TimesheetAttributes.hours][TimesheetAttributes.sundayRegular];
					var mondayRegular = timesheet[TimesheetAttributes.hours][TimesheetAttributes.mondayRegular];
					var tuesdayRegular = timesheet[TimesheetAttributes.hours][TimesheetAttributes.tuesdayRegular];
					var wednesdayRegular = timesheet[TimesheetAttributes.hours][TimesheetAttributes.wednesdayRegular];
					var thursdayRegular = timesheet[TimesheetAttributes.hours][TimesheetAttributes.thursdayRegular];
					var fridayRegular = timesheet[TimesheetAttributes.hours][TimesheetAttributes.fridayRegular];
					var saturdayRegular = timesheet[TimesheetAttributes.hours][TimesheetAttributes.saturdayRegular];
					
					taskItem[TimesheetAttributes.hoursID] = hoursID;
	                taskItem[TimesheetAttributes.sundayRegular] = TimesheetUtil.convertMinutesToHours(sundayRegular);
	                taskItem[TimesheetAttributes.mondayRegular] = TimesheetUtil.convertMinutesToHours(mondayRegular);
	                taskItem[TimesheetAttributes.tuesdayRegular] = TimesheetUtil.convertMinutesToHours(tuesdayRegular);
	                taskItem[TimesheetAttributes.wednesdayRegular] = TimesheetUtil.convertMinutesToHours(wednesdayRegular);
	                taskItem[TimesheetAttributes.thursdayRegular] = TimesheetUtil.convertMinutesToHours(thursdayRegular);
	                taskItem[TimesheetAttributes.fridayRegular] = TimesheetUtil.convertMinutesToHours(fridayRegular);
	                taskItem[TimesheetAttributes.saturdayRegular] = TimesheetUtil.convertMinutesToHours(saturdayRegular);
					
					var totalWork = sundayRegular + mondayRegular + tuesdayRegular + wednesdayRegular + thursdayRegular + fridayRegular + saturdayRegular;
	               
				    taskItem.totalWorkPerWeek = TimesheetUtil.convertMinutesToHours(totalWork);
				}
                
				taskItem.loadedelement[0][TimesheetAttributes.Timesheets] = timesheet;
				
			}
			
		},
		
		/**
		 * 
		 * @param {Object} taskItem
		 * @param {Object} workItemDetail
		 */
		_updateContainingProject: function(taskItem, workItemDetail){
			
			if(workItemDetail){
				
				taskItem[TimesheetAttributes.containingProject] = workItemDetail[TimesheetAttributes.containingProject];
				
				var constraint = workItemDetail[TimesheetAttributes.constraint];
				var constraintObj = dojo.fromJson(constraint);				
				taskItem[TimesheetAttributes.taskAssignmentType] = constraintObj.assignmentType;
								
				if(this.containerViewlet.timesheetGrid){
					
					this.containerViewlet.timesheetGrid._selectFirstRowByDefault();
				}


			}
		},
        
		/**
		 * 
		 * @param {Object} taskItem
		 * @param {Object} loadedElement
		 * @param {Object} linkName
		 * 
		 * This is called by controller.
         * After inserting a row in Grid, the getAndUpdateRow() calls updateRowData with RowURI, RowID params.
		 */
        updateRowData: function(taskItem, loadedElement, linkName){
            
			if (linkName === null) {
				
				this._updateSchedule(taskItem, loadedElement);
				
				this._updateTaskFields(taskItem, loadedElement);
				
				this._updateTimesheetsLink(taskItem, loadedElement);
				
				this._updateTimesheetsField(taskItem, loadedElement[TimesheetAttributes.Timesheets]);
				
            } 
			
			if(linkName === TimesheetAttributes.Timesheets){
				
				this._updateTimesheetsField(taskItem, loadedElement);
			}
			
			if(linkName === TimesheetAttributes.TaskAssigned){
				
				this._updateContainingProject(taskItem, loadedElement);
			}
			
        },
		
		/**
		 * 
		 * @param {Object} loadedElement
		 * @param {Object} link
		 * @param {Object} linkName
		 */
		getRowGetUri_Link: function(loadedElement, link, linkName){
			//Write your validation code here.
			if(linkName === TimesheetAttributes.Timesheets){
				
	            if (link) {
					
					var currentWeek = this.containerViewlet.getWeekOfValue();
	                var currentWeekIndex = -1;
	                var timesheeturilist = ''; // 'weekof1:uri1;weekof2:uri2;...'
	                for (var i = 0; i < link.length; i++) {
	                
	                    var tempDate = TimesheetUtil.utilGetDateObjFromServerDate(link[i][TimesheetAttributes.title]);
	                    timesheeturilist += link[i].title + ':' + link[i][TimesheetAttributes.relativeHref] + ';';
	                    if (dojo.date.compare(currentWeek, tempDate, 'date') === 0) {
	                    
	                        currentWeekIndex = i;
	                    }
	                }
	                
	                var timesheetURI = ''; // create empty row for new summary timesheet
	                if (currentWeekIndex != -1) { // if present
						timesheetURI = link[currentWeekIndex][TimesheetAttributes.relativeHref];
					}
					else {
					
						if (this.containerViewlet.toggleSubmitButton) {
							this.containerViewlet.toggleSubmitButton(false);
						}
					}
					
					if(timesheetURI !== ""){
						
						return timesheetURI;
					}else {
						return null;
					}
				}
				
			}
			if(link){
				return link.relativeHref;
			} else {
				return null;
			}	
			
			
			
		},
		
		/**
		 * 
		 * @param {Object} item
		 */
		prepareSaveParams: function(item){
			
            var tempModifiedItem = {};
            
            if (!isNaN(item[TimesheetAttributes.percentageComplete])) {
                tempModifiedItem[TimesheetAttributes.percentageComplete] = item[TimesheetAttributes.percentageComplete];
            }
            else {
                tempModifiedItem[TimesheetAttributes.percentageComplete] = "";
            }
            
			if (!isNaN(item[TimesheetAttributes.additionalEffort])) {
                tempModifiedItem[TimesheetAttributes.additionalEffort] = item[TimesheetAttributes.additionalEffort];
            }
            else {
                tempModifiedItem[TimesheetAttributes.additionalEffort] = "";
            }
            
            tempModifiedItem[TimesheetAttributes.summaryNotes] = item[TimesheetAttributes.summaryNotes];
            tempModifiedItem[TimesheetAttributes.approvalNotes] = item[TimesheetAttributes.approvalNotes];
            
            tempModifiedItem[TimesheetAttributes.hours] = "";
            
            if (item[TimesheetAttributes.hoursID]) {
            	tempModifiedItem[TimesheetAttributes.hours] += TimesheetAttributes.id + "=" + item[TimesheetAttributes.hoursID] + ",";
            }
            
			var sundayRegular = TimesheetUtil.convertHoursToMinutes(item[TimesheetAttributes.sundayRegular]);
            
			if (sundayRegular!=null) {
                tempModifiedItem[TimesheetAttributes.hours] += TimesheetAttributes.sundayRegular + "=" + sundayRegular + ",";
            }
			
			var mondayRegular = TimesheetUtil.convertHoursToMinutes(item[TimesheetAttributes.mondayRegular]);
            
            if (mondayRegular!=null) {
                tempModifiedItem[TimesheetAttributes.hours] += TimesheetAttributes.mondayRegular + "=" + mondayRegular + ",";
            }
            
			var tuesdayRegular = TimesheetUtil.convertHoursToMinutes(item[TimesheetAttributes.tuesdayRegular]);
			
            if (tuesdayRegular!=null) {
                tempModifiedItem[TimesheetAttributes.hours] += TimesheetAttributes.tuesdayRegular + "=" + tuesdayRegular + ",";
            }
            
			var wednesdayRegular = TimesheetUtil.convertHoursToMinutes(item[TimesheetAttributes.wednesdayRegular]);
			
            if (wednesdayRegular!=null) {
                tempModifiedItem[TimesheetAttributes.hours] += TimesheetAttributes.wednesdayRegular + "=" + wednesdayRegular + ",";
            }

			var thursdayRegular = TimesheetUtil.convertHoursToMinutes(item[TimesheetAttributes.thursdayRegular]);
			
            if (thursdayRegular!=null) {
                tempModifiedItem[TimesheetAttributes.hours] += TimesheetAttributes.thursdayRegular + "=" + thursdayRegular + ",";
            }

			var fridayRegular = TimesheetUtil.convertHoursToMinutes(item[TimesheetAttributes.fridayRegular]);
			
            if (fridayRegular!=null) {
                tempModifiedItem[TimesheetAttributes.hours] += TimesheetAttributes.fridayRegular + "=" + fridayRegular + ",";
            }
			
			var saturdayRegular = TimesheetUtil.convertHoursToMinutes(item[TimesheetAttributes.saturdayRegular]);
            
            if (saturdayRegular!=null) {
                tempModifiedItem[TimesheetAttributes.hours] += TimesheetAttributes.saturdayRegular + "=" + saturdayRegular + ",";
            }
            if (item[TimesheetAttributes.timecodeURI] !== null) {
                tempModifiedItem[TimesheetAttributes.timecodeURI] = item[TimesheetAttributes.timecodeURI];
            }
			
			if (item[TimesheetAttributes.timesheetApprovalState] !== null) {
                tempModifiedItem[TimesheetAttributes.timesheetApprovalState] = item[TimesheetAttributes.timesheetApprovalState];
            }
                        
            return tempModifiedItem;
        },
		
		/**
		 * 
		 * @param {Object} row
		 * @param {Object} linkName
		 * @param {Object} newElementID
		 */
		getRowPostURI_link: function(row, linkName, newElementID){
            return null;
        },
        
        prepareSaveParams_link: function(row, linkName){
            return null;
        },
		
		/**
		 * 
		 * @param {Object} row
		 */
		getRowPostURI: function(row){
			
			var timesheeturi = "";
			var assignmentURI = row[TimesheetAttributes.assignmentURI];
			if (row[TimesheetAttributes.timesheeturi] && row[TimesheetAttributes.timesheeturi] !== "") { // existing summarytimesheet
                timesheeturi = row[TimesheetAttributes.timesheeturi];
                
            } else {
				
				var resourceID = this.containerViewlet.getResourceID();
				if(resourceID.indexOf("user/") === -1){
					timesheeturi = "user/" + this.containerViewlet.getResourceID() + TimesheetConstants.TIMESHEET_URI;
				}
				else {
					timesheeturi = this.containerViewlet.getResourceID() + TimesheetConstants.TIMESHEET_URI;
				}
			}
			
			var rowURIObj = {
                rowURI: timesheeturi,
				assignmentURI: assignmentURI,
                isNew: false
            };
			return rowURIObj;
        },
		
		_isFieldModified: function(fieldValue, fieldValue_orig){
            if (fieldValue === null) {
                return false;
            }
            if (fieldValue != fieldValue_orig) {
                return true;
            }
            else {
                return false;
            }
        },
		
		/**
		 * 
		 * @param {Object} modifiedParams
		 * @param {Object} saveParams
		 * @param {Object} tempSaveParams
		 */
		_prepareTimecodeParams: function(modifiedParams, saveParams, tempSaveParams, field) {
			
			var timecodePost = new Array();
			var newTimecodeString = saveParams[field];
			var oldTimecodeString = tempSaveParams[field];
			
			var newTimecodeArray = new Array(1);
			newTimecodeArray = newTimecodeString.split(",");
			
			var oldTimecodeArray = new Array(1);
			oldTimecodeArray = oldTimecodeString.split(",");
			
			for(var i = 0; i < oldTimecodeArray.length; i++){
				
				var timecodeIndex = dojo.indexOf(newTimecodeArray, oldTimecodeArray[i]);
				if(timecodeIndex === -1){
					
					if(oldTimecodeArray[i]){
						timecodePost.push(oldTimecodeArray[i] + ":del");
					}
					
					
				} else {
					
					oldTimecodeArray.splice(i, 1);
					newTimecodeArray.splice(timecodeIndex, 1);
				}
				
				
			}
			dojo.forEach(newTimecodeArray, 
			    function(timecode, index, array) {
					if(timecode){
						timecodePost.push(timecode);
					}
			    }
			);
			
			return timecodePost;
		},
		
		
		/**
		 * 
		 * @param {Object} loadedelement
		 * @param {Object} saveParams
		 * @param {Object} row
		 * @param {Object} linkName
		 * 
		 * It checks multiple things like if the action is submit, approve or reject.
		 * Abstract grid post a request to update the resource only if certain fields are changed.
		 * But, for action like submit, approve or reject, it is not mandatory to change any value in the grid.
		 * And, still we need to post for state change workflow to get executed.
		 * 
		 */
		_isRowModified: function(loadedelement, saveParams, row, linkName){
			
			var tempSaveParams = "";
            var tempRow = this.getEmptyRow(row.id, loadedelement);
            this.updateRowData(tempRow, loadedelement, linkName);
            if (linkName !== null) {
                tempSaveParams = this.prepareSaveParams_link(tempRow, linkName);
            }
            else {
                tempSaveParams = this.prepareSaveParams(tempRow);
            }
            
            var modifiedParams = {};
            var modified = false;
            
            for (var field in saveParams) {
                if (this._isFieldModified(saveParams[field], tempSaveParams[field]) === true) {
                    modified = true;
					
					if(field === TimesheetAttributes.timecodeURI){
						
						var timecodePost = this._prepareTimecodeParams(modifiedParams, saveParams, tempSaveParams, field);
						modifiedParams[TimesheetAttributes.timecodeURI] = timecodePost;
					} else {
						modifiedParams[field] = saveParams[field];
					}
                    
                }
            }
			
			if (!row[TimesheetAttributes.timesheeturi] || row[TimesheetAttributes.timesheeturi] === "") { 

				modifiedParams[TimesheetAttributes.weekOf] = dojo.date.locale.format(this.containerViewlet.getWeekOfValue(), {
                        datePattern: 'yyyy-MM-dd',
                        selector: 'date'
                    });
                    
                modifiedParams[TimesheetAttributes.weekOf] += " 00:00:00";     
				
				modifiedParams[TimesheetAttributes.assignmentURI] = row[TimesheetAttributes.assignmentURI];	           
            }
			
			
			if(this.submit){
				
				if (this.containerViewlet.toggleSubmitButton) {
                    this.containerViewlet.toggleSubmitButton(true);
                }
				modifiedParams = this._prepareParamsForSubmitAction(row, modifiedParams);
				modified = true;
			}
			
			/*
			 * For approve or reject, it can be done at row level i.e per task assignment.
			 * So, need to check if the approval/rejection is per task assignment or 
			 * for the entire timesheet.
			 */
			if(this.approve){
				
				if(!this.actionOnSelected){
					
					modifiedParams = this._prepareParamsForApproveAction(row, modifiedParams);
					modified = true;
				}
			}
			
			if(this.reject){
				
				if(!this.actionOnSelected){
					
					modifiedParams = this._prepareParamsForRejectAction(row, modifiedParams);
					modified = true;
				}
			}
			
            if (modified === true) {
                return modifiedParams;
            }
            else {
                return null;
            }
        },
		
		/**
		 * 
		 * @param {Object} row
		 * @param {Object} modifiedParams
		 * 
		 * Checks if the timesheet state is submitted or approved. If yes, returns null, else, update the modified params
		 */
		_prepareParamsForSubmitAction: function(row, modifiedParams){
			
			if(row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_SUBMITTED || row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_APPROVED){
					modifiedParams = null;
				} else {
					modifiedParams[TimesheetAttributes.timesheetApprovalState] = TimesheetConstants.TIMESHEET_STATUS_SUBMITTED;		
				}
			return modifiedParams;
		},
		
		/**
		 * 
		 * @param {Object} row
		 * @param {Object} modifiedParams
		 * 
		 * Checks if the timesheet state is submitted. If no, returns null, else, update the modified params
		 */
		_prepareParamsForApproveAction: function(row, modifiedParams){
			
			if(row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TIMESHEET_STATUS_APPROVED){
						
				modifiedParams = null;
			} else {
				
				modifiedParams[TimesheetAttributes.timesheetApprovalState] = TimesheetConstants.TIMESHEET_STATUS_APPROVED;
			}	
			return modifiedParams;	
		},
		
		/**
		 * 
		 * @param {Object} row
		 * @param {Object} modifiedParams
		 * 
		 *  Checks if the timesheet state is submitted. If no, returns null, else, update the modified params.
		 */
		_prepareParamsForRejectAction: function(row, modifiedParams){
			
			if(row[TimesheetAttributes.timesheetApprovalState] !== TimesheetConstants.TIMESHEET_STATUS_SUBMITTED){
						
				modifiedParams = null;
			} else {
				
				modifiedParams[TimesheetAttributes.timesheetApprovalState] = TimesheetConstants.TIMESHEET_STATUS_REJECTED;
			}	
			return modifiedParams;	
		}
        
    });
})();
