/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.timesheet.web.ui.internal.TimesheetNotesView");
	
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants");
	dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants");
	dojo.require("com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes");
		// Localization
    dojo.requireLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
	
	

	var TimesheetViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.timesheet.web.ui", "Messages");
	var EPMDomNodeConstants = com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants;
	var SelfSizingTextAreaWidget = com.ibm.team.workitem.web.ui.internal.view.layout.SelfSizingTextAreaWidget;
	var TimesheetConstants = com.ibm.epm.timesheet.web.ui.internal.TimesheetConstants;
	var TimesheetAttributes = com.ibm.epm.timesheet.web.ui.internal.TimesheetAttributes;
		
	dojo.declare("com.ibm.epm.timesheet.web.ui.internal.TimesheetNotesView", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.epm.timesheet.web", "ui/internal/templates/TimesheetNotesView.html"),
		
		isApprover:"",
		rowIndexSelected:-1,
		timesheetGridController:null,
		timesheetGrid: "",
		containerViewlet: "",
		
		constructor: function()
		{
			this.okButton = TimesheetViewletMessages.OK;
			this.cancelButton = TimesheetViewletMessages.CANCEL;	

		},

		
		postCreate: function() {
			
			this.isApprover = this.containerViewlet.isProjectManager;
			this._createNotesLayout();			
			this._loadNotesFromTask();
			
		},
		
		_createNotesLayout: function()
		{
			var submitterNotesLabel = document.createElement(EPMDomNodeConstants.LABEL);
            submitterNotesLabel.innerHTML = TimesheetViewletMessages.SUMMARY_NOTES_LABEL;
            
			this._submitterLabelPoint.appendChild(submitterNotesLabel);
			this.submitterNotesField = new SelfSizingTextAreaWidget();
			this.submitterNotesField.setHeight("120px");
			this.submitterNotesField.setWidth("300px");


            this._submitterNotesFieldPoint.appendChild(this.submitterNotesField.domNode);
			
			var approverNotesLabel = document.createElement(EPMDomNodeConstants.LABEL);
            approverNotesLabel.innerHTML = TimesheetViewletMessages.APPROVER_NOTES_LABEL;
            
			
			this._approverLabelPoint.appendChild(approverNotesLabel);
			
			this.approverNotesField = new SelfSizingTextAreaWidget();
			this.approverNotesField.setHeight("120px");
			this.approverNotesField.setWidth("300px");


            this._approverNotesFieldPoint.appendChild(this.approverNotesField.domNode);	
			
			
			if(this.isApprover==false)
			{
				this.approverNotesField.textarea.disabled=true;
			}
			
			
		},
		
		_loadNotesFromTask :function()
		{
			
			if (this.timesheetGridController != null && this.rowIndexSelected != -1 ) {
				var row = this.timesheetGridController.getRowByIndex(this.rowIndexSelected);
				if (this.isApprover) {
					if (row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TASK_APPROVED) {
						this.approverNotesField.textarea.disabled = true;
						this.submitterNotesField.textarea.disabled = true;
					}
					else {
						if (row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TASK_SUBMITTED) {
							this.submitterNotesField.textarea.disabled = true;
						}
						else {
							this.submitterNotesField.textarea.disabled = false;
						}
						this.approverNotesField.textarea.disabled = false;
					}
				}
				else {
				
					if (row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TASK_SUBMITTED ||
					row[TimesheetAttributes.timesheetApprovalState] === TimesheetConstants.TASK_APPROVED) {
					
						this.submitterNotesField.textarea.disabled = true;
					}
					else {
						this.submitterNotesField.textarea.disabled = false;
					}
					
					this.approverNotesField.textarea.disabled=true;
				}
				
			var submitterNotes = row[TimesheetAttributes.summaryNotes];
            var approverNotes = row[TimesheetAttributes.approvalNotes];
			this._showSubmitterNotes(submitterNotes);
			this._showApproverNotes(approverNotes);
			
			}
			

		},
		
		/**
         *
         * @param {Object} summaryNotes
         * Set the value of the rich text editior for summary notes with
         * the value fetched from the server.
         */
        _showSubmitterNotes: function(summaryNotes){

			if(summaryNotes){
				this.submitterNotesField.setValue(summaryNotes);
			} else {
				this.submitterNotesField.setValue("");
			}
			
        },
        
        /**
         *
         * @param {Object} approverNotes
         * Set the value of the rich text editior for summary notes with
         * the value fetched from the server.
         */
        _showApproverNotes: function(approverNotes){
        	
			if(approverNotes){
				this.approverNotesField.setValue(approverNotes);
			} else {
				this.approverNotesField.setValue("");
			}
        },
		
		load : function(dialog)
		{
			this.dialog = dialog;
		},
		
		_closeNotes: function() {
			this.destroy();
		},
		
		_addNotes: function()
		{
			this._addApproverNotes();
			this._addSummaryNotes();
			
			this.destroy();
		},
		
		_addApproverNotes : function(){
			
			var notes = this.approverNotesField.getValue();
			this._saveNotes(notes, TimesheetAttributes.approvalNotes);
		},
		
		
		_addSummaryNotes : function(event){
			
			var notes = this.submitterNotesField.getValue();
			this._saveNotes(notes, TimesheetAttributes.summaryNotes);
		},		
		
       _saveNotes: function(notes, field){

			if (this.rowIndexSelected !== null) {               
                    
					var currentNotes = "";
					var actualNotes = "";
					var row = this.timesheetGridController.getRowByIndex(this.rowIndexSelected);
	                if (row) {
	                    
						var assignmentURI = row[TimesheetAttributes.assignmentURI];
						if(notes){
							currentNotes = dojo.string.trim(notes);
						}
						if(row[field]){
							actualNotes = dojo.string.trim(row[field]);
						}
	                    
	                    if (currentNotes !== actualNotes) {
	                    
	                        this.timesheetGrid.edit.apply();
	                        if (this.containerViewlet.toggleSaveButton) {
	                            this.containerViewlet.toggleSaveButton(false);
	                        }
	                        // This is required in order to update the timesheet store and push the modified data 
	                        // to the pending items of the store.
	                        var item = this.timesheetGrid.model.store._getItemByIdentity(assignmentURI);
	                        this.timesheetGrid.model.store._setValueOrValues(item, field, currentNotes, true);
	                    }
	                }
                }
            
        },


		
		destroy: function() {
			if (this.submitterNotesField) {
				delete this.submitterNotesField;
			}
			if (this.endDateWidget) {
				delete this.approverNotesField;
			}
			
			if (this.dialog) {
				this.dialog.close();
				this.dialog.destroy();
			}
		}
		
		 });
})();
