/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.TreeDNDAvatar");
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.TreeDNDAvatar", null, {
        
        constructor: function(manager){
            this.manager = manager;
            this.construct();
        },
        
    /**
     * Creates the avatar for DND
     * The avatar is a simple table of 1 row and 1 column
     * containing the icon and the label text of the node being moved.
     * Its color changes from red to green depending on whether the node
     * is allowed to be dropped on the target or not.
     */
    construct: function(){
        
        // Create a table
        var a = dojo.doc.createElement("table");
        a.className = "dojoDndAvatar";
        a.style.position = "absolute";
        a.style.zIndex = 1999;
        a.style.margin = "0px";
        var b = dojo.doc.createElement("tbody");
        var tr = dojo.doc.createElement("tr");
        var td = dojo.doc.createElement("td");
        tr.appendChild(td);
        dojo.style(tr, "opacity", 0.9);
        b.appendChild(tr);
        var source = this.manager.source, node;
        if (source) {
            if (this.manager.nodes && this.manager.nodes.length > 0) {
                var sourceNode = this.manager.nodes[0];
                
                if (sourceNode) {
                    var sourceWidget = dijit.getEnclosingWidget(sourceNode);
                    if (sourceWidget) {
                        
                        // Get a handle to the icon of the node
                        var iconSrc = sourceWidget.iconNode.src;
                        
                        // Get a handle to the label text of the node
                        var labelText = sourceWidget.label;
                        
                        // Create a span element to host the icon and the label text
                        var node = document.createElement('span');
                        var newImg = document.createElement('img');
                        newImg.src = iconSrc;
                        node.appendChild(newImg);
                        var newLabelSpan = document.createElement('span');
                        var newLabelTextNode = document.createTextNode(labelText);
                        newLabelSpan.appendChild(newLabelTextNode);
                        dojo.style(newLabelSpan, "textIndent", "1ex");
                        dojo.style(newImg, "verticalAlign", "middle");
                        
                        node.appendChild(newLabelSpan);
                        
                        // Apply styles to table
                        dojo.style(a, "borderWidth", "1px");
                        dojo.style(a, "borderSpacing", "2px");
                        dojo.style(a, "borderStyle", "solid");
                        dojo.style(a, "borderColor", "gray");
                        dojo.style(a, "borderCollapse", "separate");
                        dojo.style(a, "backgroundColor", "white");
                        
                        // Apply styles to the single cell in the table
                        dojo.style(td, "borderWidth", "1px");
                        dojo.style(td, "padding", "1px");
                        dojo.style(td, "borderStyle", "solid");
                        dojo.style(td, "borderColor", "#FE7A7A");
                        dojo.style(td, "backgroundColor", "#FE7A7A");
                        
                        td.appendChild(node);
                        tr.appendChild(td);
                        b.appendChild(tr);
                        a.appendChild(b);
                        this.node = a;
                        this.cell = td;
                    }
                }
            }
        }
    },

    /**
     * Destructor function
     */
    destroy: function(){
        dojo._destroyElement(this.node);
        this.node = false;
    },

    /**
     * Updates the avatar when it hovers over a new target node
     */
    update: function(){
        
        // If the target will accept this node, change color to green
        // Otherwise change it back to red
        if (this.manager.canDropFlag) {
            // Set background color to green
            dojo.style(this.cell, "backgroundColor", "#07FD07");
            dojo.style(this.cell, "borderColor", "#07FD07");
        } else {
            // Set background color to red
            dojo.style(this.cell, "backgroundColor", "#FE7A7A");
            dojo.style(this.cell, "borderColor", "#FE7A7A");
        }
    }
    });    
})();