/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.TreeDNDSource");

    dojo.require("dijit._tree.dndSource");
    dojo.require("com.ibm.epm.work.web.ui.internal.TreeDNDManager");
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSTreePubSub");
    
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var WBSTreePubSub = com.ibm.epm.work.web.ui.internal.WBSTreePubSub;
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.TreeDNDSource", dijit._tree.dndSource, {
        
        constructor: function(){
            if (this.topics) {
                this.topics.push(dojo.subscribe("/dnd/drop/before", this, "onDndBeforeDrop"))
            } else {
                this.topics = [
                    dojo.subscribe("/dnd/drop/before", this, "onDndBeforeDrop")
                ];
            }
            
			// Remove the existing handler for onselectstart event. This event is published only in IE. 
			// In dndContainer.constructor(), this event was stopped to prevent text selection
			// during DND. However, it caused the InlineEditBox to not work correctly in IE (For
			// details refer WI 13144). A new handler has been added to let the event be stopped
			// when the tree is not being edited. However, when it is being edited, its OK for this
			// event to be bubbled up.
			//
			if (dojo.isIE && this.events) {
				for (var i in this.events) {
					var event = this.events[i];
					if (event && event[1] === "onselectstart") {
						dojo.disconnect(event);
						dojo.connect(this.tree.domNode, "onselectstart", this, "onSelectStart");
					}
				}
			}    
        },
        
        /**
         * Checks if its OK for the source node to become the child of the target node
         * This method is overriden from dndSource.js.
         * 
         * @param {Object} target Reference to the new parent node
         * @param {Object} source Reference to the node being moved
         */        
        checkItemAcceptance: function(target, source) {
            var allowed = false;
            
            if (source && target) {
                // Get a reference to the source and target item(s)
                var targetWidget = dijit.getEnclosingWidget(target);
                if (targetWidget) {
                    var targetItem = targetWidget.item;
                    var sourceItem = null;
                    var items = source.getSelectedItems();
                    if (items && items.length > 0) {
                        sourceItem = items[0];
                    }
                    
                    // Check if its OK for this move to be allowed
                    allowed = this._validateAcceptance(sourceItem, targetItem);
                    
                    if ((targetWidget.labelNode) && (!this.tree._containingWidget._isALeafType(targetItem))) {
						if(this.tree.dragNdropAsPeer){
							this.applyTargetStyleForLeafNode(targetWidget.labelNode, allowed);
						}
						else{
                        	this.applyTargetStyle(targetWidget.labelNode, allowed);
						}
                    }
					else{
						this.tree.dragNdropAsPeer = true;
						this.applyTargetStyleForLeafNode(targetWidget.labelNode, allowed);
					}
                }    
            } else {
                
                // Its not possible to have checkItemAcceptance() to receive a null argument
                // for either source or target. If that is the case then it must be the situation 
                // when the tree thinks its dragging something when in reality it isn't. If that
                // is the case, we must bring the tree back to a state where it does not thinks it
                // dragging a node. Indicate this by calling the methods to cancel the drag-n-drop.
                // For details refer WI 14178.
                if (this.isDragging) {
                    var m = dojo.dnd.manager();
                    m.canDrop(false);
                    this.onDndCancel();
                    console.debug("TreeDNDSource.checkItemAcceptance(): DND was salvaged.")
                }
            }
            
            return allowed;
         },
         
         /**
          * Checks if the source item can become a child of the target item.
          *  
          * @param {Object} sourceItem Item reference of node being moved 
          * @param {Object} targetItem Item reference of the new parent node
          */
         _validateAcceptance: function (sourceItem, targetItem) {
             
            var allowed = false;                    
            if (sourceItem && targetItem) {
                
                 // The source/target cannot be an unsaved item
                if ((!this.tree._isNodeNew(sourceItem) && (!this.tree._isNodeNew(targetItem)))) {
                
                    // The source and the target cannot be the same item
                    if (sourceItem.identifier[0] !== targetItem.identifier[0]) {
                    
                        // The target cannot be the direct parent of the source. This is to prevent redundant moves.
						// <rkanna>For Manual re ordering we need to support the move between the direct parent.
						// so comment out the condition
                        //if (sourceItem.parentIdentifier[0] !== targetItem.identifier[0]) {
                        //Workitem cannot be a sibling of main project
						var targetParent = this.tree.getNodeByItemIdentity(targetItem.parentIdentifier);
                        if (targetParent) {
                            if (targetParent.item.resourcetype[0] === WBSConstants.EPM_PROGRAM_TYPE) {                            
                                // The source is a project directly under the program
                                if (this.tree.dragNdropAsPeer) {
                                    allowed = false;
									return allowed;
                                }
                            }
						}
                            // Programs cannot be moved
                            if (sourceItem.resourcetype[0] !== WBSConstants.EPM_PROGRAM_TYPE) {
                            
                                // Check if the source node is a project directly under a program.
                                // If yes, then its allowed to be moved so long as the target is another program.
                                // Sub-projects (projects under other projects) cannot be moved either.
                                var sourceParent = this.tree.getNodeByItemIdentity(sourceItem.parentIdentifier);
                                if (sourceParent) {
                                    if (sourceParent.item.resourcetype[0] === WBSConstants.EPM_PROGRAM_TYPE) {
                                    
                                        // The source is a project directly under the program
                                        if ((sourceParent.item.identifier[0] != targetItem.identifier[0]) && (targetItem.resourcetype[0] === WBSConstants.EPM_PROGRAM_TYPE)) {
                                            allowed = true;
                                        }
                                    }
                                    else {
                                    
                                        // The source is either a sub-project or any other WBS item type
                                        
                                        // Sub-projects cannot be moved
                                        if (sourceItem.resourcetype[0] !== WBSConstants.EPM_PROJECT_TYPE) {                                            
                                            var type = null;
                                            if (targetItem.workItemType && targetItem.workItemType.length > 0) {
                                                type = targetItem.workItemType[0]
                                            } else {
                                                if (targetItem.resourcetype && targetItem.resourcetype.length > 0) {
                                                    type = targetItem.resourcetype[0];
                                                }
                                            }
                                            
                                            // The target of the move cannot be a leaf type or a program type
                                            if (//!this.tree._containingWidget._isALeafType(targetItem) && 
                                                type !== WBSConstants.EPM_PROGRAM_TYPE) {
                                            
                                                // Check if the move is going to cause a circular dependency
                                                // If yes, then the move is disallowed
                                                if (this.checkForCircularDependency(sourceItem, targetItem)) {
                                                    
                                                    // Iterations can be moved under other iterations or projects
                                                    // They are not allowed to be moved under other tasks
                                                    if (sourceItem.resourcetype && sourceItem.resourcetype.length > 0 && 
                                                        sourceItem.resourcetype[0] === WBSConstants.ITERATION_TYPE) {
                                                        if (targetItem.resourcetype && targetItem.resourcetype.length > 0 &&
                                                        ((targetItem.resourcetype[0] === WBSConstants.ITERATION_TYPE) || 
                                                        (targetItem.resourcetype[0] === WBSConstants.EPM_PROJECT_TYPE))) {
                                                            allowed = true;
                                                        }
                                                    }
                                                    else {
                                                        allowed = true;   
                                                    }    
                                                }
                                            }  
                                        }
                                    }
                                }
                            }
                        //} direct parent check commentted out
                    }
                }
            }
            
            return allowed;        
        },
        
        /**
         * Handles the business logic before the source node is moved under the target.
         * This method is overriden from dndSource.js and is the handler of the 
         * 'dnd/start' event.
         * 
         * @param {Object} source Source container of the DND.
         * @param {Object} nodes Nodes being moved. NOTE: In our case, there should
         * be only one node that is moved as we do not allow multiple selection in our
         * tree.
         * @param {Object} copy Boolean to indicate if nodes should be copied or moved
         * NOTE: In our case, this is always false (meaning nodes are always moved).
         *                      true: copy
         *                      false: move
         * @param {Object} target Target node (new parent node)
         */        
        onDndBeforeDrop: function(source, nodes, copy, target) {
            if (target) {               
                if (nodes) {
                    if (nodes.length > 0) {
                        var sourceNode = dijit.getEnclosingWidget(nodes[0]);
                        var targetNode = dijit.getEnclosingWidget(target.current);
                        
                        if (sourceNode && targetNode) {
                            
                            // Remove style from the label since we are starting
                            // a DND
                            if (targetNode.labelNode) {
                                this.removeTargetStyle(targetNode.labelNode);
                            }
                            
                            var sourceItem = sourceNode.item;
                            var targetItem = targetNode.item;
                            
                            if (sourceItem && targetItem) {
                                // Again check if its OK for he source node to become
                                // the child of the target node. If not, then cancel
                                // the DND.
                                if (!this._validateAcceptance(sourceItem, targetItem)) {
                                    this._changeState("Container", "");
                                    this.isDragging = false;
                                    var m = dojo.dnd.manager();
                                    m.canDrop(false);  
                                }
                            }
                        }
                    }
                }
            }
        },
        
        /**
         * Checks if the source item is not the same as the target item. This is to 
         * prevent the source from getting dropped on itself thereby causing the node
         * to vanish from the tree (Isn't that a total BS and shouldn't they have built checks
         * around that in lower level code already. Alas!, they did not).
         * 
         * @param {Object} sourceItem Item reference to the node being moved
         * @param {Object} targetItem Item reference to the new parent node
         */
        checkIfDragValid: function(sourceItem, targetItem) {
            var valid = false;
            
            if (sourceItem && targetItem) {
                if (sourceItem.identifier[0] !== targetItem.identifier[0]) {
                    valid = true;  
                }
            }
            
            return valid;
        },
        
        /**
         * Performs the actual drop. This method is overridden from dndSource.js and is
         * the handler for 'dnd/drop' event. This method fires an event for the tree
         * to persist the changes on the server (which is handled in WBSTreeWidget)
         * 
         * @param {Object} source Source container of DND
         * @param {Object} nodes Nodes being moved. NOTE: In our case this is always
         * of length=one since we do not allow multiple selection in our tree.
         * @param {Object} copy Boolean to indicate to copy/move nodes. In our case this
         * is always false to indicate its a move and not a copy.
         */
        onDndDrop: function(source, nodes, copy){
            var target = this.current;
            if (target) {                
                if (nodes) {
                    if (nodes.length > 0) {
                        var sourceNode = dijit.getEnclosingWidget(nodes[0]);
                        var targetNode = dijit.getEnclosingWidget(target);
                        
                        if (sourceNode && targetNode) {
            
                            var sourceItem = sourceNode.item;
                            var targetItem = targetNode.item;
                            
                            if (sourceItem && targetItem) {                                    
                                if (this._validateAcceptance(sourceItem, targetItem)) {

                                    if((!this.tree._containingWidget._isALeafType(targetItem)) && (!this.tree.dragNdropAsPeer)){									
	                                    // Do the default impl since the drag is considered valid
	                                    this.inherited(arguments);
									}
									else{
										//As a peer Node
										this.onDndDropAsPeer(source, nodes, copy);
									}
                                    
                                    // Publish an event for the tree to adjust the grid after the move
                                    var publishObj = {
                                        sourceNode: sourceNode,
                                        targetNode: targetNode
                                    }
                            
                                    publishObj.uniqueIdentifier = {
                                        treeID: WBSConstants.WBS_TREE_ID
                                    };
                            		
									var isExpanded = true;
									if((!this.tree._containingWidget._isALeafType(targetItem)) && (!this.tree.dragNdropAsPeer)){
										var tempNode = this.tree.getItemNodeByIdentity(targetItem.identifier[0] + "_tempChild");
							            if (tempNode) {
							                isExpanded = false;
							            }
										if(!isExpanded){
											var self = this;
											//Waiting for the tree to expand
											var intervalID = setInterval(function(){
						                        console.log("inside set interval of Drag n Drop");
												var loadingNode = self.tree.getItemNodeByIdentity(targetItem.identifier[0] + "_tempChild");
									            if (loadingNode == null) {
													console.log("Loading node is removed");
													clearInterval(intervalID);
									                dojo.publish(WBSTreePubSub.ON_DRAG_N_DROP, [publishObj]);													
												}
						                    }, 50);										
										}
										else{
											dojo.publish(WBSTreePubSub.ON_DRAG_N_DROP, [publishObj]);
										}
					                }								
									else{
										dojo.publish(WBSTreePubSub.ON_DRAG_N_DROP, [publishObj]);
									}
                                    
                                } else {
                                    
                                    // Cancel the DND from proceeding
                                    var m = dojo.dnd.manager();
                                    m.canDrop(false);
                                    this.onDndCancel();
                                }    
                            }                    
                        }
                    }
                }               
           }
        },
        
		//For Reference
		onDndDropAsPeer: function(source, nodes, copy){
			if(this.containerState == "Over"){
				var tree = this.tree,
				model = tree.model,
				target = this.current,
				requeryRoot = false; // set to true iff top level items change
			
				this.isDragging = false;
				// Compute the new parent item
				var targetWidget = dijit.getEnclosingWidget(target),
				newParentItem = (targetWidget && targetWidget.item) || tree.item;
			
				var peerNodeParent = tree.getItemNodeByIdentity(newParentItem.parentIdentifier[0]);
				// If we are dragging from another source (or at least, another source
				// that points to a different data store), then we need to make new data
				// store items for each element in nodes[]. This call get the parameters
				// to pass to store.newItem()
				var newItemsParams;
				if(source != this){
					newItemsParams = this.itemCreator(nodes, target);
				}
			
				dojo.forEach(nodes, function(node, idx){
					if(source == this){
						// This is a node from my own tree, and we are moving it, not copying.
						// Remove item from old parent's children attribute.
						// TODO: dijit._tree.dndSelector should implement deleteSelectedNodes()
						// and this code should go there.
						var childTreeNode = dijit.getEnclosingWidget(node),
						childItem = childTreeNode.item,
						oldParentItem = childTreeNode.getParent().item;
						
						//Move to the new parent
						if(childItem.parentIdentifier[0] != newParentItem.parentIdentifier[0]){					
							model.pasteItem(childItem, oldParentItem, peerNodeParent, copy);
						}
						
					}else{
						model.newItem(newItemsParams[idx], oldParentItem);
					}
				}, this);
			
				
				// Expand the target node (if it's currently collapsed) so the user can see
				// where their node was dropped. In particular since that node is still selected.
				//this.tree._expandNode(targetWidget);
			}
			this.onDndCancel();
		}, 
		
		onMouseOver:function(event){			
			// Remove the style element from the target node's label
            // if a node was being dragged
            if (this.isDragging) {
				// If the shift key is pressed so it is consider as a peer
	            this.tree.dragNdropAsPeer = event.shiftKey;
				// code below is for handling depending on which TreeNode we are over
				var n = this._getChildByEvent(event); // the TreeNode				
				if(this.current){ this._removeItemClass(this.current, "Over"); }
				var m = dojo.dnd.manager();
				if(n){
					this._addItemClass(n, "Over");
					if(this.isDragging){
						if(this.checkItemAcceptance(n,m.source)){
			 				m.canDrop(this.targetState != "Disabled" && (!this.current || m.source != this || !(n in this.selection)));
				 		}
				 	}
				 }
            }
			this.inherited(arguments);			
		},
		
		applyTargetStyleForLeafNode: function(targetLabelNode, allowed) {
            if (targetLabelNode) {
                dojo.style(targetLabelNode, "borderBottomStyle", "solid");
                dojo.style(targetLabelNode, "borderBottomWidth", "2px");
                dojo.style(targetLabelNode, "marginTop", "0px");
                dojo.style(targetLabelNode, "marginLeft", "3px");
                dojo.style(targetLabelNode, "borderColor", "#316AC5"); 
				
				// Change the cursor appropriately if not allowed
                if (!allowed) {
                    var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web","ui/internal/images/notok.cur") + "'),pointer";
                    dojo.style(targetLabelNode, "cursor", cursorUrl);
                }                               
            }    
        },
        /**
         * Checks if the move will cause a circular dependency between the nodes
         * 
         * @param {Object} sourceItem Item reference to the node being moved
         * @param {Object} targetItem Item reference to the new parent node
         */
        checkForCircularDependency: function(sourceItem, targetItem) {
            var allowed = true;
            if (sourceItem && targetItem) {
                if (sourceItem.identifier[0] === targetItem.parentIdentifier[0]) {
                    allowed = false;
                } else {
                    var parent = this.tree.getNodeByItemIdentity(targetItem.parentIdentifier);
                    if (parent) {
                        var parentItem = parent.item;
                        if (parentItem) {
                            allowed = this.checkForCircularDependency(sourceItem, parentItem);
                        }    
                    }
                }
            }
            return allowed;
        },
        
        /**
         * Mouse event handler when mouse button is pressed
         * 
         * @param {Object} event Reference to event object
         */
        onMouseDown: function(event) {
            
            // If the tree is editing then stop this event
            if (this.tree.editing) {
                // Deselect any selected nodes from the tree
                this.selectNone();
            } else {
                this.inherited(arguments);
            }
        },

        /**
         * Mouse event handler when mouse button is released
         * 
         * @param {Object} event Reference to event object
         */        
        onMouseUp: function(event) {
            
            // If the tree is editing then stop this event
            if (this.tree.editing) {
                // Deselect any selected nodes from the tree
                this.selectNone();
            } else {
                this.inherited(arguments);
            }
        },
        
        /**
         * Mouse event handler when mouse button is pressed and mouse is moved
         * 
         * @param {Object} event Reference to event object
         */
        onMouseMove: function(event) {
            // If the tree is editing then stop this event
            if (this.tree.editing) {
                // Deselect any selected nodes from the tree
                this.selectNone();
            } else {
                this.inherited(arguments);
            }
        },

        
        /**
         * Overridden mouse event handler when mouse is over the tree node. 
         * The purpose of overriding this method is to remove the style element
         * applied to the target node when the mouse moved out of it.
         *  
         * @param {Object} event Reference to event object
         */
        onMouseOut: function(event) {
            this.inherited(arguments);
            
            // Remove the style element from the target node's label
            // if a node was being dragged
            if (this.isDragging) {
                var target = this.current;
                if (target) {
                    var targetWidget = dijit.getEnclosingWidget(target);
                    if (targetWidget && targetWidget.labelNode) {
                        this.removeTargetStyle(targetWidget.labelNode);
                    }        
                }   
            }
        },
        
        /**
         * Overidden method from dndSource.js. This override removes the style elements applied
         * to the target node when DND is canceled.
         */
        onDndCancel: function(){
            var targetWidget = dijit.getEnclosingWidget(this.current);
            if (targetWidget && targetWidget.labelNode) {
                this.removeTargetStyle(targetWidget.labelNode);
            }
            this.inherited(arguments);
        },
        
        /**
         * Applies a style element to the target node's label. The style element
         * consists of applying a red/green border around the label to indicate
         * if the target can accept the node being moved. In addition, it changes
         * the cursor to indicate whether its OK/NOT OK to drop the node being moved
         * onto the target node.
         * 
         * @param {Object} targetLabelNode Reference to the label node of the target node
         * @param {Object} allowed Boolean to indicate if the target node can accept/not accept
         *                  the node being moved.
         *                  true: Target can accept the node (apply a green border)
         *                  false: Target cannot accept the node (apply a red border)
         */
        applyTargetStyle: function(targetLabelNode, allowed) {
            if (targetLabelNode) {
                dojo.style(targetLabelNode, "borderStyle", "solid");
                dojo.style(targetLabelNode, "borderWidth", "1px");
                dojo.style(targetLabelNode, "marginTop", "0px");
                dojo.style(targetLabelNode, "marginLeft", "3px");
                dojo.style(targetLabelNode, "borderColor", "#316AC5");
                                
                // Change the cursor appropriately if not allowed
                if (!allowed) {
                    var cursorUrl = "url('" + dojo.moduleUrl("com.ibm.epm.work.web","ui/internal/images/notok.cur") + "'),pointer";
                    dojo.style(targetLabelNode, "cursor", cursorUrl);
                }
            }    
        },
        
        /**
         * Removes the style element applied to the target node's label in applyTargetStyle()
         * @param {Object} targetLabelNode Reference to the label node of the target node
         */
        removeTargetStyle: function(targetLabelNode) {
            if (targetLabelNode) {
                // Remove the styles from the node
                dojo.style(targetLabelNode, "borderStyle", "");
                dojo.style(targetLabelNode, "borderWidth", "");
                dojo.style(targetLabelNode, "borderColor", "");
                dojo.style(targetLabelNode, "marginLeft", "");
                var cursorUrl = "pointer";
                dojo.style(targetLabelNode, "cursor", cursorUrl);
            }
        },
	
		/**
		 * Handler of the onselectstart event. This event is published only in IE. 
		 * In dndContainer.constructor(), this event was stopped to prevent text selection
		 * during DND. However, it caused the InlineEditBox to not work correctly in IE (For
		 * details refer WI 13144). A new handler has been added to let the event be stopped
		 * when the tree is not being edited. However, when it is being edited, its OK for this
		 * event to be bubbled up.

		 * @param {Object} event Event object
		 */	
		onSelectStart: function(event) {
			if (!this.tree.editing) {
				dojo.stopEvent(event);
			}
		}                           
    });    
})();