/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.web.core.ui.internal.Viewlet");
    
    dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
    dojo.require("com.ibm.epm.web.core.ui.internal.util.CSHelp");
    dojo.require("jazz.app.ua.SectionHoverHelp");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
	  
	//localization
	dojo.requireLocalization("com.ibm.epm.web.core.ui", "Messages");

	var viewletMessages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui", "Messages");
    var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
    var csHelp = jazz.app.ua.SectionHoverHelp;
	var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
	
    /*
     * An abstract viewlet class which all viewlets must subclass.
     */
    dojo.declare("com.ibm.epm.web.core.ui.internal.Viewlet", Viewlet, {
    
        ctxtHelpId: null,
        csHelpEngine: null,
        cshelp: null,
        csHelpProviderId: null,
        _setUpdate: false,
		_dirtyFlag: false,
		_discardFlag: false,
		
        postCreate: function(){
            this.inherited('postCreate', arguments);
        },
        
		init: function() {
			// remove the header links
			var headerLink = this._viewletContainer._headerLink;
			if ((headerLink) && (headerLink.hasChildNodes())) {
				for (var i = 0; i < headerLink.childNodes.length; i++) {
					headerLink.removeChild(headerLink.childNodes[i]);
				}
			}
		},

        /*
         * Called by the individual viewlets to add the help button to the tool bar
         */		
		setCSHelpIdInToolbar: function(helpId, toolBar){
            this.ctxtHelpId = "com.ibm.rational.epm.cshelp." + helpId;
            if (toolBar) {
                if (this.ctxtHelpId) {
					this.cshelp = new csHelp({
						uaKey: this.ctxtHelpId,
						_imageSrc: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/help.gif"),
						_imageHoverSrc: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/help.gif")
					});
						
					dojo.style(this.cshelp.domNode, "marginRight", "0");
					this.cshelp.domNode.alt = viewletMessages.HELP;
					this.cshelp.domNode.setAttribute('tabIndex', "0");
					if (!dojo.isIE) {
						// Help icon is vertically aligned in IE
						dojo.style(this.cshelp.domNode, "verticalAlign", "middle");
					}
					dojo.style(this.cshelp.domNode, "paddingLeft", "5px");
					dojo.style(this.cshelp.domNode, "paddingRight", "8px");
					
					if (toolBar instanceof EPMToolbar) {
						// Insert the help icon before the refresh icon if exists else before the Save button
						var length = toolBar.toolbarRightDIV.childNodes.length;
						var insertBeforeButton = null;
						for (var i = 0; i < length; i++) {
							var title = toolBar.toolbarRightDIV.childNodes[i].title;
							if (title == viewletMessages.REFRESH) {
								insertBeforeButton = toolBar.toolbarRightDIV.childNodes[i];
								break;
							} else if ((title == viewletMessages.SAVE) || (title == viewletMessages.SAVE_ASSIGNMENT)) {								
								insertBeforeButton = toolBar.toolbarRightDIV.childNodes[i];
							}
						}
						if (insertBeforeButton) {
							toolBar.toolbarRightDIV.insertBefore(this.cshelp.domNode, insertBeforeButton);
						}

					}
					else if (toolBar instanceof dijit.Toolbar) {
						toolBar.addChild(this.cshelp);
					} else {
						// in this case the assumption is that the toolBar is div used for the informationLabel
						toolBar.appendChild(this.cshelp.domNode);
						dojo.style(this.cshelp.domNode, "float", "right");
					}
                }
            }
		},
		
		/*
         * Called by the individual viewlets to add the help button in the viewlet header
         */	
        setCSHelpId: function(helpId){
            this.ctxtHelpId = "com.ibm.rational.epm.cshelp." + helpId;
            if (this._site._container._headerButtons) {
                if (this.ctxtHelpId) {
                    this.cshelp = new csHelp({
                        uaKey: this.ctxtHelpId,
						_imageSrc: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/help.gif"),
						_imageHoverSrc: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/help.gif")
                    });
                    dojo.style(this.cshelp.domNode, "marginRight", "0");
                    dojo.style(this.cshelp.domNode, "float", "left");
                    var beforeNode = this._site._container._headerButtons.firstChild;
                    if (beforeNode != null) {
                        this._site._container._headerButtons.insertBefore(this.cshelp.domNode, beforeNode);
                    }
                }
                else {
                    var beforeNode = this._site._container._headerButtons.firstChild;
                    if (beforeNode != null && beforeNode.nodeType != 3) {
                        this._site._container._headerButtons.removeChild(beforeNode);
                    }
                }
            }
        },
        
        /*
         * Called by the platform to inform the viewlet whether or not it is active.
         * A viewlet is active only when it is visible in the user's browser meaning
         * the dashboard is the current workbench page and the viewlet is on the currently
         * displayed dashboard tab.
         */
        setActive: function(/* boolean */active){
			if (active) {
				if ((this._setUpdate) || (this._dirtyFlag)) {
					//The viewlet was modified when it was inactive and hence needs to be refreshed
					//All the subclass viewlets need to implement refresh properly for the entire viewlet to reload
					this.refresh();
					this._setUpdate = false;
				}
			}
        },
		
        setEditing: function(isEditing){
			this.inherited(arguments);	
		},
		
        /*
         * Repopulates the viewlet. New content should not be retrieved from the server.
         * Called when presentation of the viewlet is changed in some way, but new information
         * is not required.
         *
         * Viewlets should override.
         */
        update: function(){
            //if the viewlet is currently not active then mark it to be updated later 
            var active = this.getSite().isActive();
            if ((!active) && (!this._setUpdate)) {
                this._setUpdate = true;
            }
        },
		
		/**
		 * Method to set the flag to indicate user has chosen to discard the changes
		 */			
		setDiscardFlag:function(){
			this._discardFlag = true;
		},
		
		setDirtyFlag:function(){
			this._dirtyFlag = true;
		},

		/**
		 * Method to check if user has chosen to discard the changes
		 */
		isDiscarded: function(){
			return this._discardFlag;
		},
		
		isDirty: function(){
			return this._dirtyFlag;
		},

		/**
		 * Method to reset the dirty and discard flags 
		 */		
		resetDirtyFlag: function(){
			this._dirtyFlag = false;
			this._discardFlag = false;
		}
		
    });

})();
