/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.web.core.ui.internal.ViewletSplitter");
    dojo.require("jazz.ui.SideNavigationLayout");
    
    var SHOW_HIDE_CTRL_WIDTH = 9;
    var LEFT_NAV_MIN_WIDTH = 163;
    var LEFT_NAV_SNAP_WIDTH = 80;
    var SHOW_HIDE_COMMON_WIDTH = 3;
    var CONTENTAREA_PADDINF_LEFT = 5;
    
    dojo.declare("com.ibm.epm.web.core.ui.internal.ViewletSplitter", jazz.ui.SideNavigationLayout, {
    
        postCreate: function(){
            this.inherited(arguments);
            this._firstNavSectionLoaded = false;
            
            // Unergister the higher priority event and re-register with a default priority
            // THANKS A TON to Rob Retchless for figuring this out.
            // Remove these calls when 
            // https://jazz.net/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/94811
            // is fixed.
            jazz.util.ResizeNotifier.unregister(this._resizeEvent);
            this._resizeEvent = jazz.util.ResizeNotifier.register({context: this, callback: this._onResize});
        },
        
        _initDimensions: function(){
            // summary:
            //		Set the initial state of the elements.
            if (dojo.isIE == 6) {
                this._showHideButton.style.visibility = "hidden";
                this._showHideControl.style.cursor = "normal";
            }
            this.LEFT_NAV_MAX_WIDTH = 0;
            this.LEFT_NAV_DEFAULT_WIDTH = 0;
           /* dojo.style(this._showHideControl, "width", SHOW_HIDE_COMMON_WIDTH + 'px');
			dojo.style(this._showHideButton, "width", SHOW_HIDE_COMMON_WIDTH + 'px');
            dojo.style(this._contentArea, "paddingLeft", CONTENTAREA_PADDINF_LEFT + 'px');
            dojo.style(this._showHideControl, "borderRight", '#BDBE84');*/

            dojo.style(this._showHideControl, "backgroundColor", '#F8F8C8');
        },

		/**
         *  _highlightShowHideControl : Overriding the baseclass mouse hover event handler to set the background style for the sash
         *  Incoming :
         *		e - event
         *  Outgoing : none
         *
         */        
		_highlightShowHideControl: function(e) {
			this.inherited(arguments);
			dojo.style(this._showHideControl, "backgroundColor", '#F0F1BF');
		},

		/**
         *  _unhighlightShowHideControl : Overriding the baseclass mouse out event handler to set the background style for the sash
         *  Incoming :
         *		e - event
         *  Outgoing : none
         *
         */    		
		_unhighlightShowHideControl: function(e) {
			this.inherited(arguments);
			dojo.style(this._showHideControl, "backgroundColor", '#F8F8C8');
		},

		/**
         *  setSplitterAttributes : Sets the attributes of the viewlet splitter
         *  Incoming :
         *		sideNavDom - side navigation domnode
         *		perCentage - the percentage of the left of the viewlet splitter to be visible, if nothing is specified it defaults to 2.
         *  Outgoing : none
         *
         */
        setSplitterAttributes: function(sideNavDom, visibilityQuotient){
            var width = sideNavDom.clientWidth;
            if (width <= 0) {
                width = 200;
            }
			
			var visQuotient  = 2;			
			if (visibilityQuotient) {
				visQuotient = visibilityQuotient ;
			}
			
            this.LEFT_NAV_MAX_WIDTH = width - SHOW_HIDE_CTRL_WIDTH;
            this.LEFT_NAV_DEFAULT_WIDTH = (width / visQuotient) - SHOW_HIDE_CTRL_WIDTH;
            this._currentLeftNavState = "open";
            this._leftNav.style.width = this.LEFT_NAV_DEFAULT_WIDTH + "px";
            this._leftNavWidth = this.LEFT_NAV_DEFAULT_WIDTH;
			this._contentWrapper.style.width = this.LEFT_NAV_MAX_WIDTH - this.LEFT_NAV_DEFAULT_WIDTH + "px";
            switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
            
        },
		
        _startDragging: function(e){
            dojo.stopEvent(e);
            var data = {
                type: "ViewletSplitter"
            };
            dojo.publish("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onStart", [data]);
            // disable left nav resizing on IE6
            if (!(dojo.isIE && dojo.isIE <= 6)) {
                dojo.addClass(document.body, "net-jazz-ajax-SideNavigationLayout-dragCursor");
                if (this._dragListeners) {
                    for (var i = 0; i < this._dragListeners.length; i++) {
                        this.disconnect(this._dragListeners[i]);
                    }
                }
                this._dragListeners = [this.connect(document.body, "onmousemove", "_drag"), this.connect(document.body, "onmouseout", "_drag"), this.connect(document.body, "onmouseup", "_stopDragging"), this.connect(document.body, "ondragstart", "_cancelEvent"), this.connect(document.body, "onselectstart", "_cancelEvent")];
                this._isDragging = true;
            }
        },
        _drag: function(e){
            var x = e.pageX - dojo.coords(this._top).x - (SHOW_HIDE_CTRL_WIDTH / 2);
            if (x < LEFT_NAV_SNAP_WIDTH) {
                // snap closed
                this._leftNav.style.width = "0px";
                this._leftNav.style.left = -this._leftNavWidth + "px";
                this._leftNav.style.marginRight = -this._leftNavWidth + "px";
                this._currentLeftNavState = "close";
                if (dojo.isIE == 7) {
                    // IE7 bug: touch class to trigger re-paint
                    this._showHideButton.className = this._showHideButton.className;
                }
                switchClass(this._showHideButton, "showHideButtonExpanded", "showHideButtonCollapsed");
            }
            else 
                if (x < LEFT_NAV_MIN_WIDTH) {
                    // prevent further shrinking
                    this._leftNav.style.width = LEFT_NAV_MIN_WIDTH + "px";
                    this._leftNav.style.left = "0px";
                    this._leftNav.style.marginRight = "0px";
                    this._currentLeftNavState = "open";
                    if (dojo.isIE == 7) {
                        // IE7 bug: touch class to trigger re-paint
                        this._showHideButton.className = this._showHideButton.className;
                    }
                    switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
                }
                else 
                    if (x > this.LEFT_NAV_MAX_WIDTH) {
                        // prevent further expansion
                        this._leftNav.style.width = this.LEFT_NAV_MAX_WIDTH + "px";
                        this._leftNav.style.left = "0px";
                        this._leftNav.style.marginRight = "0px";
                        this._currentLeftNavState = "open";
                        if (dojo.isIE == 7) {
                            // IE7 bug: touch class to trigger re-paint
                            this._showHideButton.className = this._showHideButton.className;
                        }
                        switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
                    }
                    else {
                        // slide with mouse
                        this._leftNav.style.width = x + "px";
                        if (this._leftNav.style.left != "0px") {
                            this._leftNav.style.left = "0px";
                        }
                        if (this._leftNav.style.marginRight != "0px") {
                            this._leftNav.style.marginRight = "0px";
                        }
                        this._currentLeftNavState = "open";
                        if (dojo.isIE == 7) {
                            // IE7 bug: touch class to trigger re-paint
                            this._showHideButton.className = this._showHideButton.className;
                        }
                        switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
                    }
        },
        
        _stopDragging: function(e){
            dojo.removeClass(document.body, "net-jazz-ajax-SideNavigationLayout-dragCursor");
            if (this._dragListeners) {
                for (var i = 0; i < this._dragListeners.length; i++) {
                    this.disconnect(this._dragListeners[i]);
                }
                delete this._dragListeners;
            }
            var widthpx = this._leftNav.style.width;
            widthpx = widthpx.substring(0, widthpx.length - 2);
            if (widthpx >= LEFT_NAV_MIN_WIDTH) {
                this._leftNavWidth = widthpx;
            }
            this._isDragging = false;
            var data = {
                type: "ViewletSplitter"
            };
            dojo.publish("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onEnd", [data]);
        },
        
        _toggleLeftNav: function(e){
            // disable left nav resizing on IE6
            if (!(dojo.isIE && dojo.isIE <= 6)) {
                // description:
                //		If the left-nav is open, close it. If the left-nav is closed,
                //		open it.
                if (dojo.isIE >= 7) {
                    this._showHideButton.style.visibility = "hidden";
                }
                dojo.publish("net.jazz.ajax.ui.SideNavigationLayout.slideLeftNav.onStart");
                
                // animate by simultaneously changing margin-right and left
                var args = {
                    node: this._leftNav,
                    duration: 200,
                    properties: {
                        left: {
                            unit: "px"
                        },
                        marginRight: {
                            unit: "px"
                        }
                    }
                };
                if (this._currentLeftNavState === "open") {
                    // hide
                    args.properties.left.end = -this._leftNavWidth;
                    args.properties.marginRight.end = -this._leftNavWidth;
                    var anim = dojo.animateProperty(args);
                    this.connect(anim, "onEnd", this._onHideAnimationEnd);
                }
                else {
                    // show
                    args.properties.left.end = 0;
                    args.properties.marginRight.end = 0;
                    var anim = dojo.animateProperty(args);
                    this.connect(anim, "onEnd", this._onShowAnimationEnd);
                }
                anim.play();
            }
            var data = {
                type: "ViewletSplitter"
            };
            dojo.publish("com.ibm.epm.web.core.ui.internal.ViewletSplitter.onEnd", [data]);
        }
    });
    
    function switchClass(/* Node */node, /* string */ oldClass, /* string */ newClass){
        // remove old class if present
        var orig = node.className;
        var cls = orig;
        var t = dojo.trim((" " + cls + " ").replace(" " + oldClass + " ", " "));
        if (cls != t) {
            cls = t;
        }
        
        // add new class if present
        if ((" " + cls + " ").indexOf(" " + newClass + " ") < 0) {
            cls += (cls ? ' ' : '') + newClass;
        }
        
        // apply change if needed
        if (cls != orig) {
            node.className = cls;
        }
    }
})();
