/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.impex.web.ui.internal.WorkItemAttachments");

// Client
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.epm.impex.web.client.internal.DocumentClient");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("dojo.io.iframe");

dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("jazz.ui.ImagePreview");

(function(){ // module container
    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    
    var ImageClient = com.ibm.team.workitem.web.client.internal.ImageClient;
    var WorkItemClient = com.ibm.team.workitem.web.client.internal.WorkItemClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var DocumentClient = com.ibm.epm.impex.web.client.internal.DocumentClient;
    
    var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    //var ImagePool 				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;
    var ImagePreview = jazz.ui.ImagePreview;
    
    var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
    var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
    var isGuest = com.ibm.team.repository.web.client.session.isGuest;
    
    var UPLOAD_SERVICE_URI = "com.ibm.team.workitem.service.internal.rest.IAttachmentRestService/";
    var DOWNLOAD_SERVICE_URI = "com.ibm.team.workitem.service.internal.rest.IAttachmentRestService/repo/csid/Attachment/";
    
    var ACTION_PROP = "__jazzActionProp";
    var FILE_UPLOAD_RESPONSE_PREFIX = "File Upload Response:"; // Must match response prefix in AttachmentRestService.java 
    var EditorUtil = com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
    var WorkItemUtil = com.ibm.team.workitem.web.client.util;
    
    //Images
    var IMG_PIXEL = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/pixel.gif");
    var IMG_REMOVE = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/remove.gif");
    var IMG_REMOVE_HOVER = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/remove-hov.gif");
    var IMG_RESTORE = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/restore.gif");
    var IMG_RESTORE_HOVER = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/restore-hov.gif");
    var IMG_SORTUP = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/th-down-arrow.gif");
    var IMG_SORTDOWN = dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/images/th-up-arrow.gif");
    
    var SIZE_EQUALS = "size=";
    var MODIFIED_EQUALS = "modified=";
    var NEW_ATTACHMENT = "Now";
    var UNKNOWN = "Unknown";
    var _BYTES = " bytes";
    
    var SORT_DOWN = -1;
    var SORT_DEFAULT = 0;
    var SORT_UP = 1;
    
    // These variables must match LinkCommand.java and WorkItemLinksTypes.java
    var ADD_LINK = "addLink";
    var REMOVE_LINK = "removeLink";
    var UPDATE_STATE = "updateState";
    var ATTACHMENT_TYPE = "DocumentAttachment";
    
    // The following must match WorkItemRestService.javaa
    var UPDATE_LINKS = "updateLinks";
    
    var _currentBindRequest;
    var _currentUploadFrameNumber = 100;
    
    var COLUMN_CONFIG = [{
        label: "Name",
        headerClass: "name-header-column",
        dataFunction: "_addFileNameCell",
        isRedWhenDeleted: true,
        sortUp: function(a, b){
            if (a.fileName < b.fileName) {
                return 1;
            }
            return -1;
        },
        sortDown: function(a, b){
            if (a.fileName < b.fileName) {
                return -1;
            }
            return 1;
        }
    }, {
        label: "ID",
        headerClass: "id-header-column",
        dataFunction: "_addIdCell",
        isRedWhenDeleted: true,
        align: "right",
        sortUp: function(a, b){
            return b.id - a.id;
        },
        sortDown: function(a, b){
            return a.id - b.id;
        }
    }, {
        label: "Size",
        headerClass: "size-header-column",
        align: "right",
        dataFunction: "_addSizeCell",
        sortUp: function(a, b){
            return b.size - a.size;
        },
        sortDown: function(a, b){
            return a.size - b.size;
        }
    }, {
        label: "Created&nbsp;by",
        headerClass: "creator-header-column",
        dataFunction: "_addCreatorCell",
        sortUp: function(a, b){
            if (a.object.target.creator.name < b.object.target.creator.name) {
                return 1;
            }
            return -1;
        },
        sortDown: function(a, b){
            if (a.object.target.creator.name < b.object.target.creator.name) {
                return -1;
            }
            return 1;
        }
    }, {
        label: "Date",
        headerClass: "date-header-column",
        dataFunction: "_addDateCell",
        sortUp: function(a, b){
            var aModified = 0;
            if (a.modified && a.modified.sortDate) {
                aModified = a.modified.sortDate.getTime();
            }
            
            var bModified = 0;
            if (b.modified && a.modified.sortDate) {
                bModified = b.modified.sortDate.getTime();
            }
            
            return bModified - aModified;
        },
        sortDown: function(a, b){
            var aModified = 0;
            if (a.modified && a.modified.sortDate) {
                aModified = a.modified.sortDate.getTime();
            }
            
            var bModified = 0;
            if (b.modified && a.modified.sortDate) {
                bModified = b.modified.sortDate.getTime();
            }
            
            return aModified - bModified;
        }
        
    }, {
        label: "State",
        headerClass: "state-header-column",
        dataFunction: "_addStateCell"
        /*sortUp: function(a, b) {
         if(a.object.target.creator.name < b.object.target.creator.name) return 1;
         return -1;
         },
         sortDown: function(a, b) {
         if(a.object.target.creator.name < b.object.target.creator.name) return -1;
         return 1;
         }*/
    }];
    
    function displayBlock(node, isDisplayed){
        node.style.display = isDisplayed ? "block" : "none";
    }
    
    
    /* defineWidget */
    dojo.declare("com.ibm.epm.impex.web.ui.internal.WorkItemAttachments", [dijit._Widget, dijit._Templated], {
        _isUploadInProgress: false,
        _wasUploadCancelled: false,
        _priorFullPath: null,
        //TODO : get workItem not workItemId
        //dave : hardcoding id for now
        _contextWorkItemId: 0,
        _contextWorkItemHref: "",
        _contextWorkItemCategory: "",
        //TODO : get all styles from WorkItemEditor.css
        templatePath: dojo.moduleUrl("com.ibm.epm.impex.web", "ui/internal/templates/WorkItemAttachments.html"),
        
        postCreate: function(){
        
            this._headerCells = [];
            this._initializeUploadInput();
            this._defaultSort = COLUMN_CONFIG[4].sortUp;
            this._sortFunction = this._defaultSort;
            this.connect(this._cancelUploadButton, "onclick", "_handleCancelClick");
            
            this.connect(this._uploadForm, "onsubmit", "_performUploadSubmit");
            
            if (isGuest()) {
                this._attachmentHeaderArea.className = "attachmentHeaderArea-Guest";
            }
            
            this._initializeHeader();
        },
        destroyWidget: function(){
            //this.parentController.pageController.removeAllMouseListeners(this);
            if (this._imagePreview1) {
                this._imagePreview1.destroy();
            }
            if (this._imagePreview2) {
                this._imagePreview2.destroy();
            }
            if (this._uploadFileInput_onchange_handle) {
                dojo.disconnect(this._uploadFileInput_onchange_handle);
            }
            this.destroy();
        },
        showWidget: function(){
            this._showUploadFormOrMessage();
        },
        _initializeHeader: function(){
            var line = this._attachmentTableHead.insertRow(0);
            var row = this._attachmentTableHead.insertRow(1);
            
            for (var i = 0; i < COLUMN_CONFIG.length; i++) {
                var lineNode = document.createElement("th");
                lineNode.className = "header-line";
                lineNode.innerHTML = "&nbsp;";
                line.appendChild(lineNode);
                this._addHeaderColumn(row, COLUMN_CONFIG[i]);
            }
            var lineNode = document.createElement("th");
            lineNode.className = "header-line";
            lineNode.innerHTML = "&nbsp;";
            line.appendChild(lineNode);
            
            // Add a column for the delete/restore button
            var cell = document.createElement("TH");
            cell.appendChild(document.createTextNode(" "));
            row.appendChild(cell);
        },
        _addHeaderColumn: function(row, config){
            var cell = document.createElement("TH");
            row.appendChild(cell);
            var nobr = document.createElement("nobr");
            cell.__jazzConfig = config;
            cell.__jazzConfig.sortState = SORT_DEFAULT;
            cell.className = config.headerClass;
            var img = document.createElement("IMG");
            img.src = IMG_PIXEL;
            img.className = "header-img";
            
            var span = document.createElement("SPAN");
            span.innerHTML = config.label;
            if (config.align == "right") {
                nobr.appendChild(img);
                nobr.appendChild(span);
            }
            else {
                nobr.appendChild(span);
                nobr.appendChild(img);
            }
            
            
            cell.appendChild(nobr);
            cell.__img = img;
            this._headerCells.push(cell);
            this.connect(cell, "onmouseover", dojo.hitch(this, this._handleHeaderRowMouseOver));
            this.connect(cell, "onmouseout", dojo.hitch(this, this._handleHeaderRowMouseOut));
            this.connect(cell, "onmousedown", dojo.hitch(this, this._handleHeaderRowClick));
            
        },
        _handleHeaderRowMouseOver: function(e){
            var cell = e.currentTarget;
            cell.style.backgroundColor = "white";
            cell.style.borderBottomColor = "#FEBE47";
            cell.style.borderBottomWidth = "3px";
        },
        _handleHeaderRowMouseOut: function(e){
            var cell = e.currentTarget;
            cell.style.backgroundColor = "";
        },
        _handleHeaderRowClick: function(e){
            var cell = e.currentTarget;
            var sortState = cell.__jazzConfig.sortState;
            for (var i = 0; i < this._headerCells.length; i++) {
                this._headerCells[i].__img.src = IMG_PIXEL;
                this._headerCells[i].__jazzConfig.sortState = SORT_DEFAULT;
            }
            switch (sortState) {
                case SORT_UP: // up to default
                    cell.__img.src = IMG_PIXEL;
                    cell.__img.style.visibility = "hidden";
                    cell.__jazzConfig.sortState = SORT_DEFAULT;
                    this._sortData(cell.__jazzConfig, SORT_DEFAULT);
                    break;
                case SORT_DOWN:
                    cell.__img.src = IMG_SORTUP;
                    cell.__img.style.visibility = "visible";
                    cell.__jazzConfig.sortState = SORT_UP;
                    this._sortData(cell.__jazzConfig, SORT_UP);
                    break;
                default: // default to down
                    cell.__img.src = IMG_SORTDOWN;
                    cell.__img.style.visibility = "visible";
                    cell.__jazzConfig.sortState = SORT_DOWN;
                    this._sortData(cell.__jazzConfig, SORT_DOWN);
                    
            }
        },
        _sortData: function(config, sortState){
            switch (sortState) {
                case SORT_UP:
                    this._sortFunction = config.sortUp;
                    break;
                case SORT_DOWN:
                    this._sortFunction = config.sortDown;
                    break;
                default:
                    this._sortFunction = this._defaultSort;
            }
            this._rebindAttachments();
        },
        reset: function(){
            // reset attachment cache
            this._attachmentArray = [];
            
            // remove all rows 
            dojox.data.dom.removeChildren(this._attachmentTableBody);
            
            // initialize basic user interface based on browser and logged-in state
            
            // show the upload form if the user is logged in
            this._showUploadFormOrMessage();
            
            this._initializeUploadInput(); // reset the form
            //TODO: Cancel the upload operation
            displayBlock(this._uploadInProgressMessageArea, /*isDisplayed*/ false);
            
            // show the log in required message if the user is a guest;
            // otherwise hide
            displayBlock(this._loginRequiredMessageArea, /* isGuest() */ false);
            
        },
        
        // IE does not support setting the value of an input type=file, so the only way to clear the form
        // is to destroy it and recreate it
        _initializeUploadInput: function(){
        
            //		<input type="file" name="uploadFileInput" dojoAttachPoint="_uploadFileInput" class="UploadControl"/>
            var div = this._uploadInputDiv;
            dojox.data.dom.removeChildren(div);
            var input1 = document.createElement("INPUT");
            input1.type = "file";
            input1.name = "uploadFileInput";
            input1.className = "UploadControl";
            input1.disabled = false;
            this._uploadFileInput = input1;
            this._uploadFileInput_onchange_handle = dojo.connect(this._uploadFileInput, "onchange", this, "_performUploadSubmit");
            div.appendChild(input1);
        },
        
        // bind work item data to controls in the links tab
        bindWorkItem: function(workItem){
            this._attachmentTable.style.display = "none";
            this._noAttachmentsMessageArea.style.display = "inline";
            
            if (workItem.id == 0) {
                return;
            }
            //Enable browse
            this._uploadFileInput.disabled = false;
            
            this._contextWorkItemId = workItem.id;
            this._contextWorkItemHref = workItem.href;
            
            //var attachments = [];
            
            //this._noAttachmentsMessageArea.style.display="none";
            //this._attachmentTable.style.display="inline";
            
            var uri = this._contextWorkItemHref;
            var self = this;
            var serviceObj = {
                success: function(data){
                    var len = data.length;
                    if (!data || len == 0) {
                        //TODO
                    }
                    else {
                        // Set work item category
                        self._contextWorkItemCategory = data.categoryid;
                        self._getAllAttachments(data);
                        // Check for attachment links
                    }
                },
                
                failure: function(errorObj){
                    self.getSite().doneRefresh();
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            DocumentClient.getURI(srh, uri);
            
        },
        _getAllAttachments: function(data){
            var self = this;
            if (data.linkType) {
                for (var i = 0; i < data.linkType.length; i++) {
                    if (data.linkType[i] != null) {
                        if (data.linkType[i].name == ATTACHMENT_TYPE) {
                            if (data.linkType[i].link) {
                                for (var j = 0; j < data.linkType[i].link.length; j++) {
                                    var linkHref = data.linkType[i].link[j].relatedHref;
                                    var serviceObj = {
                                        success: function(data){
                                            var len = data.length;
                                            if (!data || len == 0) {
                                                //TODO
                                            }
                                            else {
                                                // Add attachment
                                                var size = data.size;
                                                var name = data.name ? data.name : UNKNOWN;
                                                var id = data.id;
                                                var attachmentId = data.attachmentId;
                                                var contentURI = data.contentURI;
                                                var creator = data.creator;
                                                var state = data.state;
                                                var actions = [];
                                                if (data.actions != null) {
                                                    actions = data.actions;
                                                }
                                                var modified;
                                                if (data.creationDate) {
                                                    modified = WorkItemUtil.getAllTimestamps(data.creationDate);
                                                }
                                                
                                                var attachment = {
                                                    object: data,
                                                    size: size,
                                                    modified: modified,
                                                    id: id,
                                                    fileName: name,
                                                    path: contentURI,
                                                    uuid: attachmentId,
                                                    creator: creator,
                                                    state: state,
                                                    actions: actions,
                                                    isNew: false,
                                                    isRemoved: false
                                                };
                                                self._attachmentArray.push(attachment);
                                            }
                                            //Atleast one attachment found, so show the header section.
                                            if (self._attachmentArray.length == 1) {
                                                self._attachmentTable.style.display = "inline";
                                                self._noAttachmentsMessageArea.style.display = "none";
                                            }
                                            self._rebindAttachments();
                                        },
                                        failure: function(errorObj){
                                            alert("Failed to get attachment");
                                        }
                                    };
                                    var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
                                    DocumentClient.getURI(srh, linkHref);
                                }
                            }
                        }
                    }
                }
            }
        },
        
        _rebindAttachments: function(){
            // remove all current rows
            dojox.data.dom.removeChildren(this._attachmentTableBody);
            
            this._attachmentArray.sort(this._sortFunction);
            
            for (var i = 0; i < this._attachmentArray.length; i++) {
                this._addAttachmentRow(this._attachmentArray[i]);
            }
        },
        // Values are like this: { filename: "myfile.txt", path: "d:\\my document\myfile.txt", uuid: "_AJ908334LJJOJI ");
        _addAttachmentRow: function(attachment, isFirst){
            this._noAttachmentsMessageArea.style.display = "none";
            this._attachmentTable.style.display = "";
            
            var tbody = this._attachmentTableBody;
            var row, cell, img;
            if (isFirst) {
                row = tbody.insertRow(0);
            }
            else {
                row = tbody.insertRow(-1);
            }
            
            for (var i = 0; i < COLUMN_CONFIG.length; i++) {
                cell = row.insertCell(-1);
                cell.__jazzConfig = COLUMN_CONFIG[i];
                cell.__jazzAttachment = attachment;
                this[COLUMN_CONFIG[i].dataFunction](cell, attachment);
                if (attachment.isRemoved && COLUMN_CONFIG[i].isRedWhenDeleted) {
                    dojo.addClass(cell, "cellDeleted");
                }
            }
            cell = row.insertCell(-1);
            cell.className = "action-column";
            if (attachment.isNew) {
                row.style.backgroundColor = "#F5F8FB";
            }
            var self = this;
            if (attachment.isRemoved) {
                row.style.backgroundColor = "#FFF9EF";
                img = document.createElement("IMG");
                img.src = IMG_RESTORE;
                img.title = "Restore the deleted attachment.";
                img.onmouseover = function(e){
                    img.src = IMG_RESTORE_HOVER;
                };
                img.onmouseout = function(e){
                    img.src = IMG_RESTORE;
                };
                cell.onmousedown = function(e){
                    self._performRestore(attachment);
                };
            }
            else {
                img = document.createElement("IMG");
                img.src = IMG_REMOVE;
                img.title = "Remove the attachment";
                img.onmouseover = function(e){
                    img.src = IMG_REMOVE_HOVER;
                };
                img.onmouseout = function(e){
                    img.src = IMG_REMOVE;
                };
                cell.onmousedown = function(e){
                    self._performRemove(attachment);
                };
                if (!attachment.isNew) {
                    img.style.visibility = "hidden";
                }
            }
            
            row.onmouseover = function(e){
                if (attachment.isRemoved) {
                    row.style.backgroundColor = "#FFF4E1";
                }
                else {
                    if (attachment.isNew) {
                        row.style.backgroundColor = "#EEF4F8";
                    }
                    else {
                        row.style.backgroundColor = "#F6F6F6";
                        img.style.visibility = "visible";
                    }
                }
            };
            row.onmouseout = function(e){
                if (attachment.isRemoved) {
                    row.style.backgroundColor = "#FFF9EF";
                }
                else {
                    if (attachment.isNew) {
                        row.style.backgroundColor = "#F5F8FB";
                    }
                    else {
                        row.style.backgroundColor = "";
                        img.style.visibility = "hidden";
                    }
                }
            };
            
            cell.appendChild(img);
            
        },
        _addIdCell: function(cell, attachment){
            cell.className = "id-data-column";
            
            var id = attachment.isNew ? "New" : attachment.id;
            
            var anchor = this._getSaveAnchor(attachment, attachment.fileName, id);
            cell.appendChild(anchor);
            
            anchor.popupFileName = attachment.fileName;
            anchor.parentCell = cell;
            var index = attachment.fileName.lastIndexOf(".");
            var ext = index == -1 ? null : attachment.fileName.substring(index + 1);
            ext = ext ? ext.toLowerCase() : ext;
            if (ext == "gif" || ext == "png" || ext == "jpeg" || ext == "jpg") {
                //				if(this._imagePreview1) this._imagePreview1.destroy();
                this._imagePreview1 = new ImagePreview({
                    connectNode: [anchor],
                    src: anchor.href,
                    filename: attachment.fileName
                }, document.createElement("div"));
            }
            else {
                anchor.title = attachment.fileName;
            }
            
        },
        _addFileNameCell: function(cell, attachment){
        
            var anchor = this._getSaveAnchor(attachment, attachment.fileName);
            cell.appendChild(anchor);
            var self = this;
            
            anchor.popupFileName = attachment.fileName;
            anchor.parentCell = cell;
            var index = attachment.fileName.lastIndexOf(".");
            var ext = index == -1 ? null : attachment.fileName.substring(index + 1);
            ext = ext ? ext.toLowerCase() : ext;
            if (ext == "gif" || ext == "png" || ext == "jpeg" || ext == "jpg") {
                //				if(this._imagePreview2) this._imagePreview2.destroy();
                this._imagePreview2 = new ImagePreview({
                    connectNode: [anchor],
                    src: anchor.href,
                    filename: attachment.fileName
                }, document.createElement("div"));
            }
            else {
                anchor.title = attachment.fileName;
            }
            
        },
        _addCreatorCell: function(cell, attachment){
            cell.appendChild(document.createTextNode(attachment.creator));
        },
        _addSizeCell: function(cell, attachment){
        
            cell.className = "size-data-column";
            var sizeSpan = document.createElement("span");
            
            var sizeText = "" + Math.ceil(attachment.size / 1024);
            
            var rgx = /(\d+)(\d{3})/;
            while (rgx.test(sizeText)) {
                sizeText = sizeText.replace(rgx, '$1' + ',' + '$2');
            }
            
            var fullSizeText = "" + attachment.size;
            rgx = /(\d+)(\d{3})/;
            while (rgx.test(fullSizeText)) {
                fullSizeText = fullSizeText.replace(rgx, '$1' + ',' + '$2');
            }
            fullSizeText = fullSizeText + _BYTES;
            
            
            sizeText = sizeText + " KB";
            dojox.data.dom.replaceChildren(sizeSpan, document.createTextNode(sizeText));
            sizeSpan.title = fullSizeText;
            
            cell.appendChild(sizeSpan);
            
        },
        _addDateCell: function(cell, attachment){
            cell.className = "attachment-date";
            var modifiedSpan = document.createElement("span");
            var modifiedText = "";
            if (attachment.modified) {
                modifiedText = attachment.modified.printTimestamp;
                modifiedSpan.title = attachment.modified.formattedTimestamp;
            }
            
            dojox.data.dom.replaceChildren(modifiedSpan, document.createTextNode(modifiedText));
            cell.appendChild(modifiedSpan);
        },
        
        _addStatusCell: function(cell, attachment){
            var prefix = document.createElement("SPAN");
            if (attachment.isNew) {
                //				"<b>To be added when this work item is saved:</b><br/>";
                var b = document.createElement("B");
                b.appendChild(document.createTextNode("To be added when this work item is saved:"));
                prefix.appendChild(b);
                prefix.appendChild(document.createElement("BR"));
            }
            if (attachment.isRemoved) {
                //				"<b>To be removed when this work item is saved.</b><br/>";
                var b = document.createElement("B");
                b.appendChild(document.createTextNode("To be removed when this work item is saved."));
                prefix.appendChild(b);
                prefix.appendChild(document.createElement("BR"));
            }
            var valueSpan = document.createElement("span");
            
            valueSpan.appendChild(prefix);
            valueSpan.appendChild(document.createTextNode(attachment.path));
            
            cell.appendChild(valueSpan);
        },
        
        _addStateCell: function(cell, attachment){
            var sel = document.createElement("select");
            sel.name = 'state' + attachment.id;
            var actions = attachment.actions;
            sel.options[0] = new Option(attachment.state, "");
            if (actions != null) {
                for (var i = 0; i < actions.length; i++) {
                    sel.options[i + 1] = new Option(actions[i].name, actions[i].id);
                }
            }
            cell.appendChild(sel);
        },
        _getSaveAnchor: function(attachment, text, altText){
            var saveAnchor = document.createElement("a");
            //				saveAnchor.className= "actionLink";
            saveAnchor.href = TeamServerClient._getServicePrefix() + DOWNLOAD_SERVICE_URI + text + "?itemId=" + attachment.uuid;
            saveAnchor.target = "_new";
            if (altText) {
                saveAnchor.appendChild(document.createTextNode(altText));
            }
            else {
                saveAnchor.appendChild(document.createTextNode(text));
            }
            
            return saveAnchor;
        },
        
        _performRemove: function(attachment){
            //TODO : Check this!
            //this.parentController.markDirty();
            for (var i = 0; i < this._attachmentArray.length; i++) {
                if (this._attachmentArray[i].uuid == attachment.uuid) {
                    if (this._attachmentArray[i].isNew) {
                        this._attachmentArray.splice(i, 1);
                    }
                    else {
                        this._attachmentArray[i].isRemoved = true;
                    }
                    break;
                }
            }
            this._rebindAttachments();
        },
        
        _performRestore: function(attachment){
        
            for (var i = 0; i < this._attachmentArray.length; i++) {
                if (this._attachmentArray[i].uuid == attachment.uuid) {
                    this._attachmentArray[i].isRemoved = false;
                    break;
                }
            }
            this._rebindAttachments();
        },
        _showUploadFormOrMessage: function(){
            if (this._isUploadInProgress) {
                return;
            }
            displayBlock(this._uploadForm, !isGuest());
        },
        // hide progress message and show clearedupload form
        _showFormHideMessage: function(){
            this._isUploadInProgress = false;
            displayBlock(this._uploadInProgressMessageArea, /*isDisplayed*/ false);
            this._initializeUploadInput(); // reset the form
            this._showUploadFormOrMessage();
        },
        isUploadInProgress: function(){
            return this._isUploadInProgress;
        },
        _handleCancelClick: function(e){
            e.preventDefault();
            this.cancelUploadInProgress();
        },
        cancelUploadInProgress: function(){
            if (this._currentBindRequest) {
                this._currentBindRequest.cancel();
                if (dojo.isIE) {
                    dojo.io.iframe.setSrc(dojo.io.iframe._frame, "", true);
                    this._currentBindRequest.isCancelled = true;
                }
                if (dojo.isMozilla) {
                    window.stop();
                }
            }
            
            this._showFormHideMessage();
        },
        _performUploadSubmit: function(e, isRestart){
            if (e) {
                e.preventDefault();
            }
            this._isUploadInProgress = true;
            // we always rebuild the input, so remove the change event
            dojo.disconnect(this._uploadFileInput_onchange_handle);
            
            // hide upload form and show progress message
            displayBlock(this._uploadForm, /*isDisplayed*/ false);
            displayBlock(this._uploadInProgressMessageArea, /*isDisplayed*/ true);
            
            var fullPath;
            // Check to see if the file is already loaded
            if (isRestart) {
                fullPath = this._priorFullPath;
            }
            else {
                fullPath = this._uploadFileInput.value;
            }
            if (fullPath.length === 0) {
                this._showFormHideMessage();
                return;
            }
            
            this._priorFullPath = fullPath;
            var parts = fullPath.split("\\");
            var fileName = parts[parts.length - 1];
            
            for (var i = 0; i < this._attachmentArray.length; i++) {
                if (this._attachmentArray[i].path == fullPath) {
                    alert("An attachment with the selected file name is already to be added when this work item is saved.\n" + fullPath);
                    this._showFormHideMessage();
                    return;
                }
            }
            
            var self = this;
            if (this._currentBindRequest != null) {
                this._currentBindRequest.cancel();
                this._currentBindRequest = null;
            }
            /*
             * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the
             * next one breaks. Resetting the iframe fixes the problem
             * Unfortunately this code reaches into iframe private variables.
             */
            if (dojo.io.iframe._frame) {
                document.body.removeChild(dojo.io.iframe._frame);
            }
            var frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
            dojo.io.iframe._iframeName = frameName;
            dojo.io.iframe._frame = null;
            
            // uploading file below			
            this._currentBindRequest = dojo.io.iframe.send({
                form: this._uploadForm,
                url: TeamServerClient._getServicePrefix() + "com.ibm.epm.impex.component.service.IEPMImpexService/",
                content: {
                    mimetype: "text/xml",
                    fileFields: "uploadFileInput"
                },
                handleAs: "html",
                handle: function(response, ioArgs){
                    if (response instanceof Error) {
                        if (response.dojoType && response.dojoType == "cancel") {
                            return; // this is a cancelled request, ignore it
                        }
                        self._currentBindRequest = null; // no need to cancel it any more
                        alert("An error occured during file upload.\n" + "The server or file upload service is not responding,\nor an invalid fully qualified file name was provided.");
                        self._showFormHideMessage();
                        console.debug("Error uploading file: " + response.message);
                        return;
                    }
                    var data = response;
                    alert("Import Successful");
                    dojo.publish("OnRefresh");
                    self._showFormHideMessage();
                }
            });
        },
        /*
         * Add commands to the updateLinks commands
         *
         */
        getSaveCommands: function(){
            if (!this._attachmentArray) {
                return null;
            }
            var self = this;
            var commands = [];
            for (var i = 0; i < this._attachmentArray.length; i++) {
                var attachment = this._attachmentArray[i];
                if (attachment.isNew) {
                    var command1 = {
                        cmd: ADD_LINK,
                        type: ATTACHMENT_TYPE,
                        attachmentId: attachment.uuid,
                        contextWorkItemId: self._contextWorkItemId,
                        name: attachment.fileName, // not needed... used in history
                        size: attachment.size
                    };
                    commands.push(command1);
                }
                else { // Not new
                    if (attachment.isRemoved) {
                        var command2 = {
                            cmd: REMOVE_LINK,
                            type: ATTACHMENT_TYPE,
                            id: attachment.id,
                            name: attachment.fileName // not needed... used in history
                        };
                        commands.push(command2);
                    }
                    
                    var stateVal = document.getElementsByName("state" + attachment.id)[0].value;
                    if (stateVal != "") {
                        var command3 = {
                            cmd: UPDATE_STATE,
                            type: ATTACHMENT_TYPE,
                            id: attachment.id,
                            action: stateVal,
                            name: attachment.fileName // not needed... used in history
                        };
                        commands.push(command3);
                    }
                }
            }
            return commands;
        }
    });
    
})();
