/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008,2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.web.core.ui.internal.WorkItems.WorkItemConfigurationsHandler");
	
	dojo.require("dijit._Widget");
	
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.epm.web.core.client.internal.EPMWebUIServiceClient");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
	
	var EPMWebUIServiceClient = com.ibm.epm.web.core.client.internal.EPMWebUIServiceClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
	
    dojo.declare("com.ibm.epm.web.core.ui.internal.WorkItems.WorkItemConfigurationsHandler", dijit._Widget, {
		_WIConfigs: {
			
		},
		
		_WIConfigsStatus: {
			
		},
		
		_childrenCount: {
			
		},

		fetchConfig: function(/*String*/ projectHref, /*String*/ projectName){
			if(this._WIConfigsStatus[projectHref] &&
					(this._WIConfigsStatus[projectHref] == EPMConstants.LOADED_CONFIG
					|| this._WIConfigsStatus[projectHref] == EPMConstants.LOADING_CONFIG)){
				return;
			}else{
				this._fetchConfigFromServer(projectHref, projectName);
			}
		},
		
		_fetchConfigFromServer: function(/*String*/ projectHref, /*String*/ projectName){
			var self = this;
            var fetchWIConfigFunctions = {
                success: function(configs){
					this._updateLocalConfigCache(configs.workItemTypeConfigurations, projectHref);
					this._updateLocalChildCountCache(configs, projectHref);
                },
                failure: function(errorObj){
					console.log(errorObj);
//                    self.handleError(errorObj.errorMessage, self.errorOpeningWorkItem);
                },
		
				_updateLocalConfigCache: function(configs, projectHref){
					if(configs == null){
						return;
					}
					var massagedConfigs = {};
					for(var i in configs){
						var config = configs[i];
						massagedConfigs[config.id] = config;
					}
					self._WIConfigs[projectHref] = massagedConfigs;
					self._WIConfigsStatus[projectHref] = EPMConstants.LOADED_CONFIG;
				},
		
				_updateLocalChildCountCache: function(configs, projectHref){
					if(configs == null){
						return;
					}
					self._childrenCount[projectHref] = configs.numberOfChildren;
				}
            };
            
            var srh = new ServiceResponseHandler(fetchWIConfigFunctions, "success", "failure");
			this._WIConfigsStatus[projectHref] = EPMConstants.LOADING_CONFIG;
			EPMWebUIServiceClient.getWorkItemsConfigs(srh, projectName);
		},
		
		getWIConfig: function(/*String*/ projectHref, /*String*/ wiType){
			var projectConfig = this.getAllWIConfig(projectHref);
			if(projectConfig != null){
				return projectConfig[wiType]; 
			}
			return null;
		},
		
		getAllWIConfig: function(/*String*/ projectHref){
			if(this._WIConfigsStatus[projectHref]){
				if(this._WIConfigsStatus[projectHref] == EPMConstants.LOADED_CONFIG){
					return this._WIConfigs[projectHref];
				}else{
					return this._WIConfigsStatus[projectHref];
				}
			} 
			return null;
		},
		/**
		 * 
		 * @param {String} projectHref : the project for which the child count needs to be fetched.
		 */
		getChildCount: function(/*String*/ projectHref){
			var childCnt = 0;
			if(this._childrenCount[projectHref]){
				childCnt = this._childrenCount[projectHref];
			}
			return childCnt;
		},
		
		getWIIconURI: function(/*String*/ projectHref, /*String*/ wiType){
			wiConfig = this.getWIConfig(projectHref, wiType);
			if(wiConfig != null){
				return wiConfig.iconUrl;
			}
			return null;
		}
    });
})();
