/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.details.WBSDetailsViewlet");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.Viewlet");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.work.web.ui.internal.details.WBSItemPageView");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
    dojo.require("com.ibm.epm.work.web.ui.internal.details.EPMWorkItemUtils");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.TreeEventListener");
	dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    var Viewlet = com.ibm.epm.web.core.ui.internal.Viewlet;
    var EPMTreeEvents = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
	var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
    var EPMWorkItemUtils = com.ibm.epm.work.web.ui.internal.details.EPMWorkItemUtils;
	var EPMDomNodeConstants = com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants;
    var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    var TreeEventListener = com.ibm.epm.web.core.ui.internal.TreeEventListener;
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.details.WBSDetailsViewlet", [Viewlet, TreeEventListener], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/details/templates/WBSDetailsViewlet.html"),
        contextHelpId: WBSConstants.WBS_DETAILS_VIEWLET_CONTEXT_HELP_ID, //	The Context Sensitive Help ID for the WBS grid viewlet.
        pageViewOpened: false,
		labelNode: null,
		noWBSTree: false,
		flagRefresh: false,
		
		/**
		 * init(): 	Viewlet initialization function.
		 * 			Creates the pageview for wbs item. Catches the ON_SELECT tree node event.
		 * 			Defines the work item types for which the details view to be displayed.  
		 */
        init: function(){
			this.inherited(arguments);
			
			this.setCSHelpId(this.contextHelpId);
            this._updateTitle();
			
            // Disable viewlet if no WBS tree
            if (this._disableViewletIfNoWBSTree())
                return;
        },
        
        _createInformationLabel: function() {
            this.labelNode = document.createElement(EPMDomNodeConstants.LABEL);
            this.labelNode.innerHTML = "";			
			this.setCSHelpIdInToolbar(this.contextHelpId, this._informationLabel);
            this._informationLabel.appendChild(this.labelNode);
        },
                
        /*
         * Disable viewlet and display an appropriate message if there is no WBS Tree
         * for the viewlet to make use of. Returns true if viewlet was disabled, signifying
         * to caller that further setup is not necessary.
         */
        _disableViewletIfNoWBSTree: function() {
            var tree = dijit.byId(WBSConstants.WBS_TREE_ID);
            if (tree)
                return false;

            this.noWBSTree = true;
            this.showInformationLabel(WorkMessages.WBS_TREE_MISSING);
            return true;
        },
        
		/**
		 * refresh: Calls method to update details view for the selected treeNode.
		 */
        refresh: function(){
            // Only refresh if the viewlet is not disabled due to having no WBS tree
            if (!this.noWBSTree){
				this.flagRefresh = true;
				this._createInformationLabel();
				this.pageview = new com.ibm.epm.work.web.ui.internal.details.WBSItemPageView();
	            this.workItemTypes = [WBSConstants.EPM_PROJECT_TYPE, WBSConstants.WORKITEM_TYPE,WBSConstants.EPM_RISK_TYPE];
    			this.initViewBasedOnTree();
			}
        },
        
		/**
		 * 1. Removing the node create & select event
		 * 2. Clearing the view content node
		 * 3. Clearing the information label
		 */
        destroy: function(){
			if (this.pageViewOpened) {
				this.pageViewOpened = false;
			}
			for (var i=0; i < this._informationLabel.childNodes.length; i++) {
				this._informationLabel.removeChild(this._informationLabel.childNodes[i]);
			}
        },
		
        /**
         * showInformationLabel: Displays a label. This label is used to inform user about;
         * 					i) No WBS Tree node selected
         * 				   ii) WBS Details view is not available for Programs & Iterations.	
         *                iii) WBS Tree is not here (viewlet is shown in RTC Dashboards page)
         * 		Adds the info-label class and sets the label text.
         */
        showInformationLabel: function(msg){
			dojo.removeClass(this._informationLabel, "com-ibm-team-workitem-wbs-info-HiddenClass");
            this.labelNode.innerHTML = msg;

			if (this.pageViewOpened) {
                this._content.removeChild(this.pageview.domNode);
				this.pageViewOpened = false;
			}
        },

        /**
         * hideInformationLabel: Hides the label node.
         * 		Removes the info-label class.
         */
        hideInformationLabel: function(){
			if (this.labelNode != null)
				this.labelNode.innerHTML = "";
			dojo.addClass(this._informationLabel, "com-ibm-team-workitem-wbs-info-HiddenClass");
        },
		
		/**
		* Get the current tree node to load the view
		* Incoming : None
		* Outgoing : None
		*/
		initViewBasedOnTree: function(){
			var tree = dijit.byId(WBSConstants.WBS_TREE_ID);
			if (tree && tree._selectedNode) {
				var item = tree._selectedNode.item;
				if (item && item.identifier != this.identifier) {
					var obj = {
						treeID: item.treeID,
						identifier: item.identifier,
						name: item.name,
						resourcetype: item.resourcetype,
						href: item.href
					};
					this._onWBSTreeRowClick(obj);
				}
			}
			else {
                this.showInformationLabel(WorkMessages.NO_TREE_NODE_SELECTED_MSG);
			}
		}, 
        
		/**
		 * _onWBSTreeRowClick: Validates the treeID and calls the addDomNode for only WBSTree.
		 * @param {Object} publishedItem
		 */
        _onWBSTreeRowClick: function(publishedItem){
			EPMWorkItemUtils.updateWITeamURI();
			if (this.displayDomNode(publishedItem) == false) {
				return;
			}
            EPMWorkItemUtils.selectedWBSTreeNode = publishedItem;
            this._addDomNode();
        },
		
		/**
		 * displayDomNode: Validates whether to display the domNode.
		 * 					Returns false in case the treeID is not WBSTree.
		 * @param {Object} treeItem
		 */
		displayDomNode: function(treeItem) {
			if ( !this.getSite().isActive() || !treeItem) {
				return false;
			}
			var treeName = "";
			if (treeItem.uniqueIdentifier) {	//For newly created tree node
				treeName = treeItem.uniqueIdentifier.treeID;
			}
			else {
				treeName = treeItem.treeID;
			}
			
			if (treeName != WBSConstants.WBS_TREE_ID) {
				return false;
			}
			return true;
		},		
        
		/**
		 * _addDomNode: Adding / removing the domNode from the content based on the selected WBS Item type.
		 * 				Displays the details view only for the project & work item types.
		 * _internalViewWorkItem() function used to register the pageID & actions. 
		 * Editor's view gets opened by the registered function from WBSItemPageView. 
		 */
        _addDomNode: function(){
            var publishedItem = EPMWorkItemUtils.selectedWBSTreeNode;
            var wiResType = dojo.isArray(publishedItem.resourcetype) ? publishedItem.resourcetype[0] : publishedItem.resourcetype;
            var tempArgs = {
                action: "",
                nodeSelected: publishedItem
            };
			
			//Using identifier instead of href. Reason: href is not available for the newly created tree node.
			var wiHref = dojo.isArray(publishedItem.identifier) ? publishedItem.identifier[0] : publishedItem.identifier;
			var isNew = ( wiHref.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1 ) ? true : false;
            var pos = dojo.indexOf(this.workItemTypes, wiResType);

            if (isNew) {	//Details view not available for unsaved work items
				this.showInformationLabel(WorkMessages.NEW_TREE_NODE_SELECTED_MSG);
			}
			else if (pos != -1) {	//Displaying Details view
				this.hideInformationLabel();
				this.pageview.controller.viewWorkItem(tempArgs, this.flagRefresh);
				this.flagRefresh = false;
                this._content.appendChild(this.pageview.domNode);
				this.pageViewOpened = true;
            }
            else {	//Details view available for only project & work items. Not displaying for others. 
				this.showInformationLabel(WorkMessages.DETAILS_NOT_AVAILABLE_MSG);				
            }
        },
        
        _updateTitle: function(){
            this.getSite().setTitle(WorkMessages.WBS_DETAILS_TITLE);
        },
		
        /**
         *  afterTreeModified : Overridden function from TreeEventListener
         *  Incoming : none
         *  Outgoing : none
         *
         */
        afterTreeModified: function(){
            this.update();
        },

		subscribeToTreeEvents: function(){
            this._eventSubscriptionHandler = [
				dojo.subscribe(EPMTreeEvents.ON_CREATE_NODE, this, "_onWBSTreeRowClick"),
				dojo.subscribe(EPMTreeEvents.ON_SELECT, this, "_onWBSTreeRowClick")
			];
        },
        
        unSubscribeFromTreeEvents: function(){
            if (this._eventSubscriptionHandler != null) {
                for (var i in this._eventSubscriptionHandler) {
                    var handler = this._eventSubscriptionHandler[i];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._eventSubscriptionHandler = [];
        },
		
        setActive: function(active){
			console.log("DETAILS VIEWLET IS NOW ACTIVE = " + active );

            if (!this.noWBSTree) {        
                if (active)
                    this.subscribeToTreeEvents();
                else
                    this.unSubscribeFromTreeEvents();
            }
            
            this.inherited(arguments);
        }

		
    });
    
})();

