/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.epm.work.web.ui.internal.details.WBSItemEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.date.posix");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor");
dojo.require("com.ibm.epm.work.web.ui.internal.details.EPMWorkItemUtils");
dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
dojo.require("com.ibm.epm.web.core.util.internal.Constants");

(function() {

	var WorkItemEditor			= com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor;
	var EPMWorkItemUtils 		= com.ibm.epm.work.web.ui.internal.details.EPMWorkItemUtils;
	var GridEvents 				= com.ibm.epm.web.core.grid.controller.PubSub;
	var WBSConstants			= com.ibm.epm.work.web.ui.internal.WBSConstants;
	var Constants 				= com.ibm.epm.web.core.util.internal.Constants;
    var EPMTreePubSub           = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
	dojo.declare("com.ibm.epm.work.web.ui.internal.details.WBSItemEditor", [dijit._Widget, WorkItemEditor],
	{
		wiTypeChanged: false,
		/**
		 * This is overwritten function from parent's WorkItemEditor.js file.
		 * DestroyWidget should be called whenever the page is destroyed and should
		 * propagate to all widgets.
		 */
		destroyWidget: function(){
			this.inherited(arguments);
		},

		/**
		 * This is overwritten function from parent's WorkItemEditor.js file.
		 * _destroyChildWidgets should be called while destroying the controller's child widgets
		 * @param {Object} skipHeader
		 */		
		_destroyChildWidgets: function(skipHeader){
			this.inherited(arguments);
		},
		
	/**
	 * This is overwritten function from parent's WorkItemEditor.js file.
	 * Used to register the tab click events with WBSItemPageController methods.
	 * 
	 * _createTabPart: Create a subtab for the work item editor.

	 * @param {String} logicalName this name to identify the editor part via
	 * 	this._editorParts and also the name to specify which widget to show
	 *  in the viewWorkItem URL.
	 * @param {widget} widget is a widget to associate
	 *  with the reference from this._editorParts[logicalName].
	 * @param {Object} createArgs arguments with which to initialize the widget
	 * @param {String} tabLinkLabel the label of the hyperlink that allows users
	 *  to view the particular tab.
	 */
		_createTabPart: function(logicalName, widget, tabLinkLabel){
			this.inherited(arguments);

			try {
			if (this._tabLinkContainer) {
				for (var i = 0; i < this._tabLinkContainer.childNodes.length; i++) {
					if (this._tabLinkContainer.childNodes[i]) {
						dojo.connect(this._tabLinkContainer.childNodes[i], "onclick", this, "_handleTabClick");
					}
				}
			}
			} catch(e) {
				console.debug(dojo.string.substitute(this.messages.errorCreatingTab, [ logicalName ]));
			}
		},
		
		/**
		 * _handleTabClick: "onclick" event handler for each tab in details view. 
		 * @param {Object} e
		 */
		_handleTabClick: function(e){
			if (e.stopPropagation) {
				e.stopPropagation();
				e.preventDefault();
			}
			e.cancelBubble = true;
			e.returnValue = false;
			this.updateURL(e.currentTarget.__jazzTabName);			
			this.showTab(e.currentTarget.__jazzTabName);
			return false;
		},
		
		/**
		 * updateURL: Updates the URL with clicked tabname appended.
		 * 				This is to make tab clicks in sync with WI Editor. 
		 * @param {Object} tabName
		 */
		updateURL: function(tabName) {
			var url = this.workingCopy.getUrl();
			url = url.replace(/com.ibm.team.workitem./, "com.ibm.epm.web.core.");
			var ix = url.indexOf("&tab=");
			if (tabName == "overview") {
				if (ix != -1) {
					url = url.substring(0, url.indexOf("&tab"));
				}
			}
			else {
				if (ix == -1) { 
					this.url = this.url + "&tab=" + tabName;
					url = url + "&tab=" + tabName;
				} else {
					url = (url + "&").replace(/&&/g, "&");
					this.url = url.replace(/&tab=.+&/, "") + "&tab=" + tabName;
					url = url.replace(/&tab=.+&/, "") + "&tab=" + tabName;
				}
			}
			var split = window.location.href.split("#");  
			location.href = split[0] + url;
		},
		
		/**
		 * save: Derived method of save() in WorkItemEditor.js
		 * 			Publishes the required events to update tree & grids data post save of workitem.
		 */
		save: function(){
			this.inherited(arguments);
			if (EPMWorkItemUtils.selectedWBSTreeNode.resourcetype[0] == WBSConstants.EPM_PROJECT_TYPE) {
				return;
			}
			dojo.publish(GridEvents.REFRESH_ROW_DATA, [EPMWorkItemUtils.selectedWBSTreeNode]);
			if ( (this._editorParts.header) && (this._editorParts.header.summary) ) {
				//Copying the summary fields value to the publish object.
				EPMWorkItemUtils.selectedWBSTreeNode.newTitle = this._editorParts.header.summary.value;
				dojo.publish(GridEvents.RENAME_NODE, [EPMWorkItemUtils.selectedWBSTreeNode]);
			}
			
			if (this.wiTypeChanged == true) {
				var newWIType = this._workItem.attributes.workItemType.id;
				this._updateTreeNodeIcon(newWIType);
			}
			this.wiTypeChanged = false;
		},
		
		/**
		 * _workItemTypeChanged: Derived method of _workItemTypeChanged() in WorkItemEditor.js
		 * 			
		 **/
		_workItemTypeChanged: function() {
			this.inherited(arguments);
			this.wiTypeChanged = true;
		},
		
		/**
		 * _updateTreeNodeIcon:
		 * 			This is to update the tree node icon after type change. 
		 * @param {Object} newWIType
		 */
		_updateTreeNodeIcon: function(newWIType) {
			var iconClass = "dijitTreeIcon " + newWIType + "Icon";
			var tree = dijit.byId(Constants.WBS_TREE_ID);
			var iconUrl = this._getIconUrlForWorkItemType(tree, newWIType);
			var label = this._getLabelForWorkItemType(tree, newWIType);

			var selectedNode = EPMWorkItemUtils.selectedWBSTreeNode;
            if ( (iconUrl) && (selectedNode)) {
                var treeNode = tree.getNodeByItemIdentity(selectedNode.identifier);
                if (treeNode) {
					treeNode.item.workItemType = newWIType;
                    treeNode.iconNode.src = iconUrl;
                    treeNode.iconNode.className = iconClass;
					treeNode.iconNode.alt = label;
					tree._addDropDownIcon(treeNode.item);
					this.wiTypeChanged = true;
					dojo.publish(EPMTreePubSub.TRIGGGER_DASHBOARD_ICON_UPDATE, [treeNode.iconNode, true]);
                }
            }
		},

	    /**
	     * _getIconUrlForWorkItemType: 
	     * 			Returns the iconUrl for the new work item type
	     * @param {Object} tree
	     * @param {Object} workItemType
	     **/
		_getIconUrlForWorkItemType: function(tree, workItemType) {
	        var iconUrl = null;
	        if (tree._containingWidget._controller) {
	            if (tree._containingWidget._controller.workItemTypes) {
	                for (var i in tree._containingWidget._controller.workItemTypes) {
	                    var workItem = tree._containingWidget._controller.workItemTypes[i];
	                        
	                    if (workItem) {
	                        if (workItem.id === workItemType) {
	                            iconUrl = workItem.iconUrl;
								break;
	                        }
	                    }
	                }
	            }
	        }
	        return iconUrl;
	    },
		
		/**
	     * _getIconLabelForWorkItemType: 
	     * 			Returns the label for the new work item type
	     * @param {Object} tree
	     * @param {Object} workItemType
	     **/
		_getLabelForWorkItemType: function(tree, workItemType) {
	        var label = null;
	        if (tree._containingWidget._controller) {
	            if (tree._containingWidget._controller.workItemTypes) {
	                for (var i in tree._containingWidget._controller.workItemTypes) {
	                    var workItem = tree._containingWidget._controller.workItemTypes[i];
	                        
	                    if (workItem) {
	                        if (workItem.id === workItemType) {
	                            label = workItem.label;
								break;
	                        }
	                    }
	                }
	            }
	        }
	        return label;
	    },
		
		/**
		 * refresh: 
		 * 		Derived method of refresh() in WorkItemEditor.js
		 * @param {Object} suppressConfirmation
		 */
		refresh: function(suppressConfirmation) {
			this.inherited(arguments);
		},

		/**
		 * _checkProjectArea: 
		 * 			Overriding the parent method to do nothing. 
		 * 			This is to fix focus() error in IE7.
		 * 			
		 * The parent method checks that the current project area matches the one that is selected
		 * 		and displays message.
		 */
		 _checkProjectArea: function() {

		 }
	});

})();
