/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

dojo.provide("com.ibm.epm.work.web.ui.internal.details.WBSItemPageView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.SideNavigationLayout");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.epm.work.web.ui.internal.details.WBSItemPageController");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageView");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentView");
dojo.require("com.ibm.epm.work.web.ui.internal.details.WBSItemEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.RecentWorkItemView");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorBanner");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyDialog");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function(){

    var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;
    var checkNonNegative = net.jazz.ajax.toolbox.lang.checkNonNegative;
    var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
	var WorkItemPageView = com.ibm.team.workitem.web.ui.internal.page.WorkItemPageView;
	var SelectWorkItemDialog = com.ibm.team.workitem.web.ui.internal.view.editor.SelectWorkItemDialog;
	var MoveCopyDialog = com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyDialog;
    
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    var WorkItemUtil = com.ibm.team.workitem.web.client.util;
    var WORK_ITEM_EDITOR_ID = "__jazzWorkItemEditor";
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.details.WBSItemPageView", [dijit._Widget, dijit._Templated, WorkItemPageView], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/details/templates/WBSItemPageView.html"),
        
        constructor: function(){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");
        },
        
		/**
		 * postCreate: 
		 * 1. Creates the controller
		 * 2. Initiates the navigation layout & content widget supporting multipane
		 */
        postCreate: function(){
            this.childWidgets = [];
			
//-->		Creating a dummy template string for the SelectWorkItemDialog, SelectMoveCopyDialog widgets. This is to fix the issue of unhandled exception in IE.
//			Created defect 94489 on RTC in jazz.net for the actual issue of removing [dijit_Templated] declaration from those widgets
//					which does not have actual HTML template or template string defined. 
//			The below lines setting the templateString for these widgets can be removed once the defect 94489 on RTC is fixed. 
//<--
			SelectWorkItemDialog.prototype.templateString = "<div></div>";
			MoveCopyDialog.prototype.templateString = "<div></div>";
			
//			Overwriting the MoveCopy dialog OK button event. Not supporting Copy work item action from details view.
//				Preventing the Copy action in the dialog, routing to the WBSItemPageController's preventWIAction() function.
//				preventWIAction() is called for all the un-supported actions in details view. 
			com.ibm.team.workitem.web.ui.internal.view.editor.SelectMoveCopyContent.prototype._handleOk = function()
			{ 
					if (this.radioCopy.checked) { 
						this.parentController.pageController.preventWIAction();
					}
					else {
						this.onOk({
							isMove: this.radioMove.checked,
							projectAreaItemId: this._projectSelect.value
						});
						this.reset();			
					}
			}; 

            this.controller = new com.ibm.epm.work.web.ui.internal.details.WBSItemPageController(this);
            
            this._leftNavigationLayout = new jazz.ui.SideNavigationLayout({}, this._appContainer);
            this._addChild(this._leftNavigationLayout);
            
            // Initialize the multipane content widget
            var parms = {
                parentController: this.controller
            };
            this._multipaneContentWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentView(parms);
            this._addChild(this._multipaneContentWidget);

			ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			var actionViewWorkItem = ActionRegistry._getActionEntry("com.ibm.epm.web.core.viewWorkItem", this.controller, "viewWorkItem").action;
			if (actionViewWorkItem) {
				delete ActionRegistry._actionRegistry[actionViewWorkItem._id].action;
			}
			ActionRegistry.registerAction("com.ibm.epm.web.core.viewWorkItem", this.controller, "viewWorkItem");

			actionViewWorkItem = ActionRegistry._getActionEntry("com.ibm.epm.web.core.similarityQuery", this.controller, "preventWIAction").action;
			if (actionViewWorkItem) {
				delete ActionRegistry._actionRegistry[actionViewWorkItem._id].action;
			}
			ActionRegistry.registerAction("com.ibm.epm.web.core.similarityQuery", this.controller, "preventWIAction");

            this._initializeNavigation();
        },
		
		/**
		 * destroy: To call parent's destroy method in WorkItemPageView.js
		 */
        destroy: function(){
            this.inherited(arguments);
        },
        
		/**
		 * This is overwritten function from parent's WorkItemPageView.js file.
		 * _initializeNavigation: This function not only initialized the navigation bar, also creates the required widgets that are required for the multipaneContentWidget.
		 * 						Initializing only the required widgets. For reference, see this function in WorkItemPageView.js.  
		 * 						EPMRecentWorkItemView widget file has been created in the first version of details view. 
		 * 						Removed this file and using the work item editor RecentWorkItemView file.  
		 * @param {Object} navigationView
		 */
        _initializeNavigation: function(navigationView){
			this._recentWorkItemWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.common.RecentWorkItemView( 
						{parentController: this.controller, heading: this.messages.headingRecentlyViewed});

			this._recentWorkItemWidget.navSection = this._leftNavigationLayout.addNavigationSection(this._recentWorkItemWidget, this._recentWorkItemWidget.heading, true);
            this._multipaneContentWidget.setItemListWidget(this._recentWorkItemWidget);
            
            var queryArgs = {
                parentController: this.controller
            };
            this._registerNavigation();
            
            this._unloadHandle = dojo.connect(window, "onunload", dojo.hitch(this, this.unload)); // use dojo.connect here
            
			// Ping all the working copies to add them  to recently viewed
			var workingCopies = com.ibm.team.workitem.web.cache.internal.Cache.getAllItems();
			for(var i=0;i<workingCopies.length;i++) {
				if(dojo.isFunction(workingCopies[i].notifyOfItemChanged)) {
					workingCopies[i].notifyOfItemChanged();
				}	
			}
        },
        
		/**
		 * * This is overwritten function from parent's WorkItemPageView.js file.
		 * _registerNavigation: This method needs to be called from _initializeNavigation().
		 * 						Executes parents method in WorkItemPageView.js file.
		 */
        _registerNavigation: function(){
			this.inherited(arguments);
        },
        
        _addChild: function(childWidget){
            if (!childWidget) {
                console.debug("Error adding null child widget in Work Item Page View");
            }
            else {
                this.childWidgets.push(childWidget);
            }
        },
        
        isWorkItemEditorAvailable: function(id){
            return this._multipaneContentWidget.isWidgetAvailable(WORK_ITEM_EDITOR_ID, id);
        },
        
		/**
		 * * This is overwritten function from parent's WorkItemPageView.js file.
		 * showWorkItemEditor: This gets called by parent controller while displaying the work item.
		 * 						Took the function code from the latest work item files in beta1.
		 * 						This calls _showWorkItemEditor(). 
		 * 						The actual code in showWorkItemEditor is splitted to 2 functions in beta1. 
		 * @param {Object} id
		 * @param {Object} tab
		 * @param {Object} isRefresh
		 */
		showWorkItemEditor: function(id, tab, isRefresh) {
			var args = {
				id: id
			}
			if(tab) args.tab = tab;
			if(isRefresh) args.isRefresh = isRefresh;
			this._showWorkItemEditor(args);
		},
		
		/**
		 * * This is overwritten function from parent's WorkItemPageView.js file.
		 * _showWorkItemEditor: Took the function code from the latest work item files in beta1.
		 * 						This is called by showWorkItemEditor() function. 
		 * @param {Object} parms
		 */
		_showWorkItemEditor: function(parms) {
			var id = parms.id || parms.tempId;
			var workItemEditorWidget = this._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, id);
			if(!workItemEditorWidget) {
				workItemEditorWidget = new com.ibm.epm.work.web.ui.internal.details.WBSItemEditor({isNew:true, parentController: this.controller});
			}
			
			parms.onSuccess= dojo.hitch(this,this._showWorkItemEditorComplete);
			parms.onError= dojo.hitch(this,this._showWorkItemEditorError)
		    workItemEditorWidget.initializeWorkItemEditor(parms);
		},

		_getWorkItemEditorBanner: function(workItemId) {

			if(this._workItemEditorBanner == null) {
				this._workItemEditorBanner = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorBanner(
							{parentController: this.controller, parentView: this});
				this._addChild(this._workItemEditorBanner);
			}
			
			this._updateWorkItemEditorBanner(workItemId);
			return this._workItemEditorBanner;
		},
		
		_updateWorkItemEditorBanner: function (workItemId) {
			// If no query yet run, return null
			var results;
			if(this.queryResultsWidget) { 
				results = this.queryResultsWidget.getResults();
			}
			this._workItemEditorBanner.setQueryInfo(workItemId, this.controller._lastQueryName, this.controller._lastQueryURL, results);
			
		},

		_showWorkItemEditorError: function() {
	        dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
		},
		_showWorkItemEditorComplete: function(parms /* workItem, workItemEditor, [tempId] */ ) {
			var banner = this._getWorkItemEditorBanner(parms.workItem.id);
			this.controller._setWorkItemPageTitle(parms.workItem);
			
			if(parms.isRefresh) {
				if(parms.workItem.attributes && parms.workItem.attributes.comment) {
					parms.workItemEditor.scrollTop = window.scrollY;
					parms.workItemEditor.scrollLeft = 0;
				} else {
					parms.workItemEditor.scrollTop = 0;
					parms.workItemEditor.scrollLeft = 0;
				}
			}


			this._setMainContent(
					WORK_ITEM_EDITOR_ID,
					parms.workItemEditor,
					 banner ? banner.domNode : null,
					parms.tempId || parms.workItem.id);
			

			if(parms.workItem.id < 1) { 
				parms.workItemEditor._editorParts.header.summary.focus();
			}
			this.setIsDirty(parms.workItemEditor, parms.workItemEditor.workingCopy.isChanged());		
			
			// This is deferred until after the wi loads to speed up loading
			dojo.publish(EventTopics.INITIALIZE_QUERIES, []);
		},
        
		/**
		 * * This is overwritten function from parent's WorkItemPageView.js file.
		 * _getWorkItemEditorWidget: Used to get the Item editor required by this page view
		 * @param {Object} workItem
		 * @param {Object} initialTab
		 * @param {Object} url
		 */
        _getWorkItemEditorWidget: function(workItem, initialTab, url){
            checkNotNull(workItem, "workItem");
            var workItemEditorWidget = this._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, workItem.id);
            
            if (!workItemEditorWidget) {
                workItemEditorWidget = new com.ibm.epm.work.web.ui.internal.details.WBSItemEditor({
                    parentController: this.controller,
                    url: url,
                    initialTab: initialTab
                });
            }
            return workItemEditorWidget;
        },
        
        setIsDirty: function(workItemEditorWidget, isDirty){
            this._multipaneContentWidget.setIsDirty(WORK_ITEM_EDITOR_ID, workItemEditorWidget, workItemEditorWidget._workItem.id, isDirty);
        },
        
        getCurrentWidgetId: function(){
            return this._currentMainWidgetId;
        },
        
		/**
		 * getCurrentWorkItem: Used by WI editor framework to get the current work item.
		 * 						Taking the function from the latest WI page view changes from beta1
		 */
		getCurrentWorkItem: function() {
			if(this._multipaneContentWidget && this._multipaneContentWidget.getCurrentWidget) {
				var widget = this._multipaneContentWidget.getCurrentWidget();
				if(widget && widget.workingCopy && widget.workingCopy.object) {
					return widget.workingCopy.object;
				}
			}
			return null;
		},
		
		/**
		 * _updateSaveParameters: Used by the WI editor framework while updating the save params.
		 * 							This function should exist inthe page view file of the WI editor.
		 */
		_updateSaveParameters: function() {
			this._recentWorkItemWidget.updateSaveParameters();
		}
		
    });
})();
