/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.msp.ImportWizard");

    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
	dojo.require("com.ibm.epm.admin.web.client.internal.calendar.CalendarClient");
	dojo.require("com.ibm.epm.work.web.ui.internal.msp.MapResourcesGridview");
	dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants");
	dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
	dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
	dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
	dojo.require("com.ibm.epm.work.web.ui.internal.WBSTreePubSub");
	
	// MSProject Import
    var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    var IMPEX_SERVICE_URI = "com.ibm.epm.impex.component.service.IEPMImpexService";

    var _currentUploadFrameNumber = 100;
    
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var calendarClient = com.ibm.epm.admin.web.client.internal.calendar.CalendarClient;	
	var GridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
	var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;
	
	var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;	
	var EPMDomNodeConstants = com.ibm.epm.web.core.ui.internal.EPMDomNodeConstants;
	var WBSTreePubSub = com.ibm.epm.work.web.ui.internal.WBSTreePubSub;
    var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");

	var assignmentTypes = [WBSConstants.ASSIGNMENT_TYPE_EFFORT, WBSConstants.ASSIGNMENT_TYPE_DURATION];

    dojo.declare("com.ibm.epm.work.web.ui.internal.msp.ImportWizard", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/msp/templates/ImportWizard.html"),
		widgetsInTemplate: true,
		_wbsTree : null,
		filepath : null,
		file_exists : false,
		_currentBindRequest : null,
		
		/**
		 * Constructor for Import wizard.
		 */		
        constructor: function() {
			this._startup();
			this._populateLabels();
			this.calendarsLinks = [];
			this.projects = [];
			this.profiles = [];
			this.profilesName = [];
			this._uploadForm = null;
			
        },
		
		/**
		 * This method is to destroy the contents of wizard on close.
		 */
		destroy: function() {			

			//Destroy mapping Grid.
			if(this.MapGrid){
				this.MapGrid.destroy();
				delete this.MapGrid;				
			}			
			if (this.dialog) {
				this.dialog.close();
				this.dialog.destroy();
			}
			dojo.disconnect(this.Close_handle);
		},
				
		        
		postCreate: function() {
		},
		/**
		 * Drop Down for AssignType.
		 */
		_createAssignTypeComboBox:function(){
			this.assignTypeDropDown = document.createElement(EPMDomNodeConstants.SELECT);
			this.assignTypeDropDown.options[0] = new Option(assignmentTypes[0]);
			this.assignTypeDropDown.options[1] = new Option(assignmentTypes[1]);
			this.assignTypeDropDown.disabled = true;
			this.assignTypeAttachPoint.appendChild(this.assignTypeDropDown);
		},
		
		/**
		 * This method is used to laod the content widgets
		 * @param {Object} wbsTree
		 * @param {Object} dialog
		 */
		load: function(wbsTree, dialog, parentID, parentName) {
			var self = this;
			this.dialog = dialog;
			this.parentNodeID = parentID;
			this._wbsTree = wbsTree;
			var ParentName = dijit.byId("ParentNameId");
			if(ParentName){
				ParentName.domNode.value = parentName;
			}
			// Creating Calendars list
			this._populateCalendarsList();

			this._createAssignTypeComboBox();
			
			this._populateProjectNameList();
			
			this._populateProfileList();
			
			
			this.Close_handle = dojo.connect(this.dialog._close, "onclick", dojo.hitch(self, self.destroy));
			this._createfileInputbox();	
			
			this.updateErrorMessage(" ");
			this.updateMapErrorMessage(" ");
			dijit.byId("ProjectNameId").focus();
			
		},
		
		/**
		 * This method will create and attach the message label to the HTML
		 */
		_addErrorlabel: function(message){
			
			if(message){				
				this.titleLabel = document.createElement("label");
				this.textNode = document.createTextNode(message);
				this.titleLabel.appendChild(this.textNode);				
			}
		},
		
		/**
		 * This method will display the error message
		 */
		updateErrorMessage: function(message) {
			if (!this.errorLabel) {
				this._addErrorlabel(message);
			}
			else {
				if ( this.errorLabel.hasChildNodes() ) {
					for (var i=0; i<this.errorLabel.childNodes.length; i++) {
						this.errorLabel.removeChild(this.errorLabel.childNodes[i]);
					}
				}
				this.textNode = document.createTextNode(message);
				this.errorLabel.appendChild(this.textNode);
			}
		},
		
		
		/**
		 * This method will display the error message
		 */
		updateMapErrorMessage: function(message) {
			if (!this.errorLabelMap) {
				this._addErrorlabel(message);
			}
			else {
				if ( this.errorLabelMap.hasChildNodes() ) {
					for (var i=0; i<this.errorLabelMap.childNodes.length; i++) {
						this.errorLabelMap.removeChild(this.errorLabelMap.childNodes[i]);
					}
				}
				this.textNode = document.createTextNode(message);
				this.errorLabelMap.appendChild(this.textNode);
			}
		},
		
		/**
		 * This method will called during the start up and will clean all
		 * the existing contents if any
		 */
		_startup: function () {
			var projectNameId = dijit.byId("ProjectNameId");
			if (projectNameId) {
				projectNameId.destroy();
			}
			var parentNameId = dijit.byId("ParentNameId");
			if (parentNameId) {
				parentNameId.destroy();
			}
			var assignTypeId = dijit.byId("AssignTypeId");
			if (assignTypeId) {
				assignTypeId.destroy();
			}
			var projectCalendarId = dijit.byId("ProjectCalendarId");
			if (projectCalendarId) {
				projectCalendarId.destroy();
			} 
			var projectProfileId = dijit.byId("ProjectProfileId");
			if (projectProfileId) {
				projectProfile.destroy();
			}
			var nextButtonId = dijit.byId("nextButtonId");
			if (nextButtonId) {
				nextButtonId.destroy();
			}
			var cancelButtonId = dijit.byId("cancelButtonId");
			if (cancelButtonId) {
				cancelButtonId.destroy();
			}
		},
		
		/**
		 * Add captions from mesaage file.
		 */
		_populateLabels: function() {
			this.projectName = WorkMessages.PROJECT_CAPTION;
			this.parentName = WorkMessages.PARENT_CAPTION;
			this.calendar = WorkMessages.CALENDAR_CAPTION;
			this.assignType = WorkMessages.ASSIGNMENT_TYPE_CAPTION;
			this.nextButton = WorkMessages.NEXT_BUTTON;
			this.cancelButton = WorkMessages.PROJECT_WIZARD_CANCEL_CAPTION; 
			this.cancelMapButton = WorkMessages.PROJECT_WIZARD_CANCEL_CAPTION;
			this.finishButton = WorkMessages.FINISH;
			this.previousButton = WorkMessages.PREVIOUS_BUTTON;
			this.selectfile = WorkMessages.SELECTFILE;
			this.mapresource = WorkMessages.MAPRESOURCE;			
			this.summary = WorkMessages.SUMMARY;
			this.file = WorkMessages.FILE;
			this.OkButton = WorkMessages.PROJECT_WIZARD_SAVE_CAPTION;
			this.ImportButton = WorkMessages.IMPORT_ANOTHER_PROJECT;
			this.profile = WorkMessages.PROFILE_CAPTION;
			this.applyButton = WorkMessages.APPLY;
		},
		
		/**
		 * Populate the calenders from server.
		 */
		_populateCalendarsList: function() {
            this.appadminCalendars = document.createElement(EPMDomNodeConstants.SELECT); 

            var self = this;
            var calendarFunctions = {
                success: function(calendars){
                    var len = calendars.length;
                    
                    if (calendars) {
                        for (var j = 0; j < calendars.length; j++) {
                            var calendarHandle = calendars[j];
							self.appadminCalendars.options[j] = new Option(calendarHandle.title);
							self.calendarsLinks.push(calendarHandle.relativeHref);
                        }
                    }
                },
                failure: function(errorMsg){
                }
            };
            var srh = new ServiceResponseHandler(calendarFunctions, "success", "failure");
            calendarClient.getAllCalendars(srh);

            self.calendarAttachPoint.appendChild(self.appadminCalendars);
		},	
		
		/**
		 * Populate the profiles from server.
		 */
		_populateProfileList: function() {
            this.profilesNode = document.createElement(EPMDomNodeConstants.SELECT);
            var self = this;
            var profileFunctions = {
                success: function(profile){
                    var len = profile.length;
                    
                    if (profile) {
                        for (var k = 0; k < profile.length; k++) {
							var profileHandle = profile[k];
							self.profilesNode.options[k] = new Option(profileHandle.title);
							self.profiles.push(profileHandle.relativeHref);
							self.profilesName.push(profileHandle.title);
                        }
                    }
					self._createMapGrid();
                },
                failure: function(errorMsg){
                }
            };
            var srh = new ServiceResponseHandler(profileFunctions, "success", "failure");
            GridModelClient.getURI(srh,WBSConstants.PROFILE_URI);
			
			self.profileAttachPoint.appendChild(self.profilesNode);
		},
		
		/**
		 * Populate the names of all projects into the a array.
		 */
		_populateProjectNameList: function() {            

            var self = this;
            var projectFunctions = {
                success: function(project){
                    var len = project.length;
                    
                    if (project) {
                        for (var j = 0; j < project.length; j++) {
							var title = project[j].title;
							var titleObj = dojo.fromJson(title);
							var name = titleObj.summary || title;
							self.projects.push(name);						
                        }
                    }
                },
                failure: function(errorMsg){
                }
            };
            var srh = new ServiceResponseHandler(projectFunctions, "success", "failure");
            GridModelClient.getURI(srh,WBSConstants.PROJECTS_URI);
		},	
		 	      
		/**
		 * Validate the field values before moving to next page.
		 */
		_validateInputs: function() {
			
			this.updateErrorMessage(" ");
			if(this.filepath === null || this.filepath.length === 0){
				this.updateErrorMessage(WorkMessages.UPLOAD_FILE_PATH);
				return false;
			}
			var projectNameId = dijit.byId("ProjectNameId");
			if (projectNameId) {
				this.projname = projectNameId.domNode.value;
				
				if (this.projname === "") {
					this.updateErrorMessage(WorkMessages.VALID_PROJECT_NAME);
					return false;
				}
				for (var i = 0; i < this.projects.length; i++) {
					if(this.projects[i] == this.projname){
						this.updateErrorMessage(WorkMessages.PROJECT_EXISTS);
						return false;
					}                            
                }
			}						
			return true;
		},	
		
		/**
		 * Create file input for file to server.
		 */
		_createfileInputbox : function(){
			dijit.byId("ProjectNameId").focus();
			if (this._uploadForm) {
				this.mspfileAttachPoint.removeChild(this._uploadForm);
				delete this._uploadForm;
			}
			dijit.byId("ProjectNameId").focus();
			this._uploadForm = document.createElement("form");
            
            if (dojo.isIE) {
                encType = this._uploadForm.getAttributeNode("enctype");
                encType.value = "multipart/form-data";
                formMethod = this._uploadForm.getAttributeNode("method");
                formMethod.value = "POST";
            }
            else {
                this._uploadForm.setAttribute("enctype", "multipart/form-data");
                this._uploadForm.setAttribute("method", "POST");
            }

            var input = document.createElement("INPUT");
            input.type = "file";
            input.name = "uploadFileInput";
            input.className = "UploadControl";
            input.disabled = false;
            this._uploadFileInput = input;
            this._uploadFileInput_onchange_hhbandle = dojo.connect(this._uploadFileInput, "onchange", this, "_performUploadSubmit");
            this._uploadForm.appendChild(input);

			this.mspfileAttachPoint.appendChild(this._uploadForm); 
			this.filepath = null;
			this.file_exists = false;            			
		},

		/**
		 * This method is to upload on submit of form.
		 */
		_performUploadSubmit: function(e, isRestart){			
			if (e) {
					e.preventDefault();
            }
			var self = this;
			var projectNameId = dijit.byId("ProjectNameId");					            
            
            this._createMapGrid();
            // we always rebuild the input, so remove the change event
            dojo.disconnect(this._uploadFileInput_onchange_handle);

            //var fullPath;
			this.filepath = null;
            // Check to see if the file is already loaded
            if (isRestart) {
                this.filepath = this._priorFullPath;
            }
            else {
                this.filepath = this._uploadFileInput.value;
            }
            if (this.filepath.length === 0) {
                return;
            }
            
            this._priorFullPath = this.filepath;
            var parts = this.filepath.split("\\");
            var fileName = parts[parts.length - 1];
            
            //JSLint will give an error here. Please ignore, otherwise it breaks MSP Import.
            if (this._currentBindRequest !== null) {
                this._currentBindRequest.cancel();
                this._currentBindRequest = null;
            }
            /*
             * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the
             * next one breaks. Resetting the iframe fixes the problem
             * Unfortunately this code reaches into iframe private variables.
             */
            if (dojo.io.iframe._frame) {
                document.body.removeChild(dojo.io.iframe._frame);
            }
            var frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
            dojo.io.iframe._iframeName = frameName;
            dojo.io.iframe._frame = null;
            
            var parentID = this.parentNodeID;
			
			dojo.removeClass(this._header, "IconClass");			
			dojo.addClass(this._header, "IconWaitingClass");
			
			this.file_exists = true;

            // uploading file below			
            this._currentBindRequest = dojo.io.iframe.send({
                form: this._uploadForm,
                url: TeamServerClient._getServicePrefix() + IMPEX_SERVICE_URI + "?VALIDATE=TRUE",
                content: {
                    mimetype: "text/xml",
                    fileFields: "uploadFileInput"
                },
				method:"post",
                handleAs: "xml",
                handle: function(response, ioArgs){
                    if (response instanceof Error) {
                        return;
                    }
                    
                    var data = response;
                    
                    if(response != null){
                   		var errorMesg = data.getElementsByTagName(WBSConstants.MSP_ERRORCODE);
                   		if( errorMesg.length > 0 )
                   		{
							self.updateErrorMessage(errorMesg[0].childNodes[0].nodeValue);
                   			dojo.removeClass(self._header, "IconWaitingClass");	
                   			dojo.addClass(self._header, "IconClass");
							self.filepath = null;
                   			return;
                   		}
                    }					
                    var userflag = false;
					if(dojo.isIE){
						self.projname = response.childNodes[1].childNodes[0].text;												
						for (var i = 1 ; i < response.childNodes[1].childNodes.length; i++) {
							var UID = response.childNodes[1].childNodes[i].childNodes[0].text;
							var user = response.childNodes[1].childNodes[i].childNodes[1].text;
							self.MapGrid.addelement(UID,user);
							userflag = true;
						}
					}else{
						self.projname = response.firstChild.childNodes[0].textContent;						
						for (var i = 1 ; i < response.firstChild.childNodes.length; i++) {
							var UID = response.firstChild.childNodes[i].childNodes[0].textContent;
							var user = response.firstChild.childNodes[i].childNodes[1].textContent;
							self.MapGrid.addelement(UID,user);
							userflag = true;
						}
					}
					var projectNameId = dijit.byId("ProjectNameId");					
					if (projectNameId) {
						dojo.byId("ProjectNameId").disabled = false;
						projectNameId.attr('value', self.projname);
					}
					if(userflag){						
						self.profilesNode.disabled = false;
					}else{						
						self.profilesNode.disabled = true;
					}
					dojo.removeClass(self._header, "IconWaitingClass");			
					dojo.addClass(self._header, "IconClass");									
                }
            });
		},
		
		/**
		 * This method is to Reupload on submit of form.
		 */
		_performReUpload: function(){			            
            var self = this;
            //JSLint will give an error here. Please ignore, otherwise it breaks MSP Import.
            if (this._currentBindRequest !== null) {
                this._currentBindRequest.cancel();
                this._currentBindRequest = null;
            }
            /*
             * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the
             * next one breaks. Resetting the iframe fixes the problem
             * Unfortunately this code reaches into iframe private variables.
             */
            if (dojo.io.iframe._frame) {
                document.body.removeChild(dojo.io.iframe._frame);
            }
            var frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
            dojo.io.iframe._iframeName = frameName;
            dojo.io.iframe._frame = null;
            
            var userMap = this._getuserList();
			if (userMap === null) {
				if (dojo.byId("continueButtonId")) {
					dojo.byId("continueButtonId").disabled = false;
				}
				if (dojo.byId("backButtonId")) {
					dojo.byId("backButtonId").disabled = false;
				}
				return;				
			}
			var selectedCalendarURI = self.calendarsLinks[self.appadminCalendars.selectedIndex];
			dojo.removeClass(this._header, "IconClass");			
			dojo.addClass(this._header, "IconWaitingClass");
            // uploading file below			
            this._currentBindRequest = dojo.io.iframe.send({
                form: self._uploadForm,
                url: TeamServerClient._getServicePrefix() + IMPEX_SERVICE_URI + encodeURI("?PARENT_ID=" + self.parentNodeID + "&CALENDAR_ID=" + selectedCalendarURI + "&Name=" + self.projname + "&userMap=" + userMap) ,
                content: {
                    mimetype: "text/xml",
                    fileFields: "uploadFileInput"
                },
				method:"post",
                handleAs: "xml",
                handle: function(response, ioArgs){
                    if (response instanceof Error) {
                        return;
                    }
                var data = response;
					
   				
				var projectHref = null;
				var summary = null;

				var responsedoc = data.getElementsByTagName("project");
				if(responsedoc[0]){
					projectHref = responsedoc[0].getAttribute("relativeHref");
					if(projectHref){
						self._insertProjectToTree(projectHref);							
					}	
				}
				
				summary = data.getElementsByTagName("Messages");
				dojo.removeClass(self._header, "IconWaitingClass");			
				dojo.addClass(self._header, "IconClass");
				self._showSumary(summary);									
								

                }
            });
		},
		

		
		/**
		 * This method is for moving to next page and checking the field values.
		 */
		_next : function() {
			
			if(!(this._validateInputs())){
				return;
			}
		
			dojo.removeClass(this._wizardMappingContent, "wizardUserMappingClassClosed");
			dojo.removeClass(this._wizardContent, "wizardFieldsClassOpen");
			dojo.removeClass(this._mainlabelcolumn1, "mainlabelcolumn1active");
			dojo.removeClass(this._mainlabelcolumn2, "mainlabelcolumn2");
			
			dojo.addClass(this._wizardMappingContent, "wizardUserMappingClassOpen");		
			dojo.addClass(this._wizardContent, "wizardFieldsClassClosed");
			dojo.addClass(this._mainlabelcolumn1, "mainlabelcolumn1");
			dojo.addClass(this._mainlabelcolumn2, "mainlabelcolumn2active");
			//Setting focus to the first cell in grid, if there is no user focus will be on first button.
			var rowIndex = this.MapGrid._generalGrid.scroller.rowCount;
			if( rowIndex > 0){
				this.MapGrid._generalGrid.scrollToRow(0);
				this.MapGrid._generalGrid.selection.select(0);
            	this.MapGrid._generalGrid.focus.setFocusIndex(0, 0);
			}else{
				if (dojo.byId("continueButtonId")) {
				dojo.byId("continueButtonId").focus();
				}
			}

		},
		
		/**
		 * This method is for going back to first page of wizard.
		 */
		_previous : function(){
			dojo.removeClass(this._wizardMappingContent, "wizardUserMappingClassOpen");
			dojo.removeClass(this._wizardContent, "wizardFieldsClassClosed");
			dojo.removeClass(this._mainlabelcolumn1, "mainlabelcolumn1");
			dojo.removeClass(this._mainlabelcolumn2, "mainlabelcolumn2active");
			
			dojo.addClass(this._wizardMappingContent, "wizardUserMappingClassClosed");		
			dojo.addClass(this._wizardContent, "wizardFieldsClassOpen");
			dojo.addClass(this._mainlabelcolumn1, "mainlabelcolumn1active");
			dojo.addClass(this._mainlabelcolumn2, "mainlabelcolumn2");
		},

		/**
		 * On close of wizard this will get called
		 */
		_closeWizard: function() {
			this.destroy();
		},

		
		/**
		 * This method will do actual Import of MSP project.
		 */
        _finish: function(){
			
			if(this.MapGrid.Unmappeduser()){
				this.updateMapErrorMessage(WorkMessages.PLEASE_MAP_USER);
				return;
			}else{
				this.updateMapErrorMessage(" ");
			}
			if (dojo.byId("previousButtonId")) {
				dojo.byId("previousButtonId").disabled = true;
			}
			if (dojo.byId("finishButtonId")) {
				dojo.byId("finishButtonId").disabled = true;
			}
			if (dojo.byId("cancelMapButtonId")) {
				dojo.byId("cancelMapButtonId").disabled = true;
			}
			if (dojo.byId("applyButtonId")) {
					dojo.byId("applyButtonId").disabled = true;
			}
			this.profilesNode.disabled = true;
			this._performReUpload();
			if(this.MapGrid){
				this.MapGrid.Inprogress = true;
			}
 		},
		
		/**
		 * This method will do show summary page of wizard.
		 */
		_showSumary : function (summarylist){
			var summary = new dojox.string.Builder();
			if(summarylist && summarylist[0] && summarylist[0].childNodes)
			{
				var children = summarylist[0].childNodes;
				for (var i = 0; i < children.length; i++) {
					if (dojo.isIE) {
						summary.append(children[i].text);
					}
					else{
						summary.append(children[i].textContent);
					}
					summary.append("\n");
				}
			}
			if (dojo.byId("reportTextboxId")) {
				dojo.byId("reportTextboxId").value = summary.toString();
				//WorkMessages.IMPORT_SUCCESS;
			}
			dojo.removeClass(this._wizardMappingContent, "wizardUserMappingClassOpen");
			dojo.removeClass(this._wizardSummaryContent, "wizardSummaryClassClosed");
			dojo.removeClass(this._mainlabelcolumn2, "mainlabelcolumn2active");
			dojo.removeClass(this._mainlabelcolumn3, "mainlabelcolumn3");
			
			dojo.addClass(this._wizardMappingContent, "wizardUserMappingClassClosed");		
			dojo.addClass(this._wizardSummaryContent, "wizardSummaryClassOpen");
			dojo.addClass(this._mainlabelcolumn2, "mainlabelcolumn2");
			dojo.addClass(this._mainlabelcolumn3, "mainlabelcolumn3active");          
        },

		/**
		 * This method is to create user mapping grid.
		 */	
		_createMapGrid: function(){	
					
			if (!this.MapGrid) {
				this.MapGrid = new com.ibm.epm.work.web.ui.internal.msp.MapResourcesGridview({
					_treeID: "_unsaved"
				});
			}else{
				this.removeAllRows(this.MapGrid);				
				this.MapGrid.setTreeId("_unsaved");
			}

            this.MapGrid.buildMapResourceGrid(this.profiles, this.profilesName);
			
			this._mapGrid.appendChild(this.MapGrid.domNode);
        },
		
		/**
		 * This method is to create user mapping grid.
		 */	
		_getuserList: function(){			
			if (this.MapGrid) {
				var userList = this.MapGrid.getuserMap();
				var selectedProfileURI = this.profiles[this.profilesNode.selectedIndex];
				if (selectedProfileURI) {
					if (userList) {
						return "{" + userList + ",-1:" + selectedProfileURI + "}";
					}else{
						return "{-1:" + selectedProfileURI + "}";
					}
				}else{
					return "{" + userList + "}";
				}
			}
			return null;
        },

				
		/*
		 * Removes all the rows of the Mapping Grid
		 */
		removeAllRows: function(grid){        
            var _gridData = grid._controller.getDataModel();            
            if (_gridData) {            
                var count = _gridData.getRowCount();                
                for (var i = 0; i < count; i++) {                
                    var row = _gridData.data[0];                    
                    if (row) {
                        _gridData.remove([0]);                        
                    }                    
                }				
				//save the model
				_gridData.store.save();
            }
        },

		/**
		 * This method is to move to first page of Import wizard.
		 */
		_newImport : function (){
			var projectNameId = dijit.byId("ProjectNameId");
			if (projectNameId) {				
				projectNameId.attr('value', "");
				projectNameId.disabled = true;
			}
			if(this.MapGrid){
				this.MapGrid.Inprogress = false;
			}
			if (dojo.byId("continueButtonId")) {
					dojo.byId("continueButtonId").disabled = true;
			}
			if (dojo.byId("applyButtonId")) {
					dojo.byId("applyButtonId").disabled = false;
			}
			if (dojo.byId("backButtonId")) {
					dojo.byId("backButtonId").disabled = true;
			}
			if (dojo.byId("cancelMapButtonId")) {
				dojo.byId("cancelMapButtonId").disabled = false;
			}
			if (dojo.byId("previousButtonId")) {
				dojo.byId("previousButtonId").disabled = false;
			}
			if (dojo.byId("finishButtonId")) {
				dojo.byId("finishButtonId").disabled = false;
			}	
			this.profilesNode.disabled = false;
			this.updateErrorMessage(" ");
			this._populateLabels();
			this._createfileInputbox();
			this._createMapGrid();
			dojo.removeClass(this._wizardSummaryContent, "wizardSummaryClassOpen");
			dojo.removeClass(this._wizardContent, "wizardFieldsClassClosed");
			dojo.removeClass(this._mainlabelcolumn1, "mainlabelcolumn1");
			dojo.removeClass(this._mainlabelcolumn3, "mainlabelcolumn3active");
			
			dojo.addClass(this._wizardSummaryContent, "wizardSummaryClassClosed");		
			dojo.addClass(this._wizardContent, "wizardFieldsClassOpen");
			dojo.addClass(this._mainlabelcolumn1, "mainlabelcolumn1active");
			dojo.addClass(this._mainlabelcolumn3, "mainlabelcolumn3");
		},
		
		/**
		 * _insertProjectToTree : Inserting the newly created project to the WBSTree.
		*/
		_insertProjectToTree: function(projectHref) {
			var projectName = dijit.byId("ProjectNameId").domNode.value;
			var projectLink = {
				relativeHref:projectHref,
				title: projectName
			}
			var item = this._wbsTree._getTreeNodeModel(projectLink, this.parentNodeID);
			var parentNode = this._wbsTree.getNodeByItemIdentity(this.parentNodeID);
            this._wbsTree._addNode(parentNode.item, item, 0);
			this._wbsTree._addTempChildNode(item);

            if ((parentNode.isExpandable) && (!parentNode.isExpanded)) {
                this._wbsTree._expandNode(parentNode);
            }
			var createdNode = this._wbsTree._itemNodeMap[projectHref];
			this._wbsTree._selectTreeNode(createdNode);
			this._publishCreateNodeEvent(item);
		},
		
		
		
		/**
		 * _publishCreateNodeEvent: To publish create node event after the project is successfully created.
		 * 							Overview & Schedule grids listens to this event and inserts corresponding project row in the grid store.
		 * Publish ON_CREATE_NODE: Not sending the published object. This is to make sure the grid's save button does not get enabled after project creation. 
		 */
        _publishCreateNodeEvent: function(item){
			
			var publishedObj = {
				name:item.name,
				identifier: item.identifier,
				resourcetype: item.resourcetype,
				treeID: WBSConstants.WBS_TREE_ID
            };
            
            dojo.publish(WBSTreePubSub.ON_CREATE_NODE, [publishedObj]);
        },
		
		/* _validateName : Function will validate whether upload file exist and  name exists		 
		 */
		
		_validateName : function(event){
			if (this.file_exists){           
				this._validateInputs();
			}             
		},
		
		/* applyProfile : Apply the selected profile to all unmapped user		 
		 */
		applyProfile: function(){
			var selectedProfileURI = this.profilesName[this.profilesNode.selectedIndex];
			if (this.MapGrid) {
				this.MapGrid.setprofile(selectedProfileURI);
			}
			this.updateMapErrorMessage(" ");
		}
		
	});
})();	
    
