/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

 (function() {
dojo.provide("com.ibm.epm.web.core.ui.internal.page.QueryEditorPageView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.SideNavigationLayout");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.epm.web.core.client.QueryUtil");
dojo.require("com.ibm.epm.web.core.ui.internal.page.QueryEditorPageController");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemWelcome");
dojo.require("com.ibm.epm.web.core.ui.internal.view.queryeditor.QueryEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.CompositeContentArea");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentView");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.RecentWorkItemView");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorBanner");
dojo.require("com.ibm.epm.web.core.ui.internal.page.QueryResultPageControl");
dojo.require("com.ibm.epm.web.core.ui.internal.view.query.QueryResults");
dojo.require("com.ibm.epm.web.core.ui.internal.view.query.SavedQueries");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.query.Search");
dojo.require("com.ibm.team.workitem.web.ui.internal.page.WorkItemPageView");
dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
// Localization
dojo.requireLocalization("com.ibm.epm.web.core.ui.internal.page", "QueryEditorMessages");


var QueryEditorMessages = dojo.i18n.getLocalization("com.ibm.epm.web.core.ui.internal.page", "QueryEditorMessages");
var checkNotNull		= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNonNegative	= net.jazz.ajax.toolbox.lang.checkNonNegative;

var PlatformUI			= net.jazz.ajax.ui.PlatformUI;
var ActionRegistry		= null; // initialize later

var QueryUtil		= com.ibm.epm.web.core.client.QueryUtil;

var WELCOME_PAGE_ID = "__jazzWorkItemWelcome";
var WORK_ITEM_EDITOR_ID = "__jazzWorkItemEditor";
var QUERY_RESULTS_ID    = "__jazzQueryResults";
var QUERY_BUILDER_ID    = "__jazzQueryBuilder";
var QUERY_COMPOSITE_ID  = "__jazzQueryComposite";
var WorkItemPageView = com.ibm.team.workitem.web.ui.internal.page.WorkItemPageView;
var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
 

/*
 * The main view for the work item web UI. Responsible for displaying left-nav
 * content and for managing the main content area.
 */
/* defineWidget */ dojo.declare(
	"com.ibm.epm.web.core.ui.internal.page.QueryEditorPageView",
	[dijit._Widget, dijit._Templated, WorkItemPageView],
	{
		
		queryType : "",
		
		constructor: function(queryType) {
			this.queryType= queryType;
		},
		/*
		 * Called by Dojo during widget creation lifecycle.
		 */
		postCreate: function() {
			
			this.childWidgets = [];
		 	this.controller = new com.ibm.epm.web.core.ui.internal.page.QueryEditorPageController(this);
			
			// Initialize the multipane content widget
			var parms= {parentController: this.controller};
			this._multipaneContentWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.common.MultipaneContentView(parms);
            this._addChild(this._multipaneContentWidget);
			                           			
			this._leftNavigationLayout = new net.jazz.ajax.ui.SideNavigationLayout({}, this._appContainer);
            this._addChild(this._leftNavigationLayout);
						
			// lazily initialize the ActionRegistry alias
			ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			
			// register actions
			//ActionRegistry.registerAction("com.ibm.team.workitem._showDefaultView", this.controller, "_showDefaultView");

			var validActions = ["search", "simpleQuery", "similarityQuery", "runSavedQuery", "runNewQuery", "newQuery", "editQuery", "runDefaultQuery", "newWorkItem", "viewWorkItem", "viewWelcome"];
			
			var self = this;
			dojo.forEach(validActions,
				function(x) {					
					var actionEntry = ActionRegistry._getActionEntry("com.ibm.epm.web.core." + x, this.controller, x).action;
					if (actionEntry) {
						delete ActionRegistry._actionRegistry[actionEntry._id].action;
					}								
					ActionRegistry.registerAction("com.ibm.epm.web.core." + x, self.controller, x);					
				}
			);

			this._initializeNavigation();
		},
		destroy: function() {

			dojo.disconnect(this._unloadHandle);
			
			this.controller.destroyController();

			dojo.forEach(this.childWidgets, function(child) {
				try {
					if(child.destroyWidget) {
						child.destroyWidget();
					} else {
						if(child.destroy) child.destroy();
					}
				} catch(e) {
					console.debug('Error destorying: ' + child);// ignore
				}
			});
			
			if(this._queryResultPageControl) {
				this._queryResultPageControl.destroy();
			}

			if(this._queryResultsWidget) {
				this.queryResultsWidget.destroy();
			}
			if(this._queryBottomPageControl) {
				this._queryBottomPageControl.destroy();
			}			
						
			if(this._multipaneContentWidget) {
				this._multipaneContentWidget.destroy();
			}			
			if(this._leftNavigationLayout) {
				this._leftNavigationLayout.destroy();
			}	
			
			if(this.welcomePageWidget) {
				this.welcomePageWidget.destroy();
			}			
						
			if(this._recentWorkItemWidget) {
				this._recentWorkItemWidget.destroy();
			}			
			if(this._unfinishedWorkItemWidget) {
				this._unfinishedWorkItemWidget.destroy();
			}	
						
 			this.inherited(arguments);

		},

		_addChild: function(childWidget) {
			if(!childWidget) {
				console.debug("Error adding null child widget in Work Item Page View");
			} else {
				this.childWidgets.push(childWidget);
			}
			
		},
		_initializeNavigation: function(navigationView) {
			this.welcomePageWidget = new com.ibm.team.workitem.web.ui.internal.page.WorkItemWelcome({});
			this._addChild(this.welcomePageWidget);
			
//			var searchInputWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.query.Search( 
//						{ actionHandler: dojo.hitch(this.controller, this.controller._handleQuickSearch) });
//			this._addChild(searchInputWidget);					
//			this._leftNavigationLayout._leftNav.appendChild(searchInputWidget.domNode);

//		TEMP: commented to remove create workitem button
//			if(!com.ibm.team.repository.web.client.session.isGuest()) {
//				this._newWorkItemCommand = this._leftNavigationLayout.addCommand({href: QueryUtil.getNewWorkItemURI(), imageClass: "create-workitem-command", buttonText: "Create Work Item", boldFirstWord: true, size: "large"});
//				this._addChild(this._newWorkItemCommand);
				//TEMP : Commented to avoid redirection to WorkItems view
				//this.controller._populateCreateWorkItemButton(this._newWorkItemCommand);
//			}

			this._newQueryCommand = this._leftNavigationLayout.addCommand({href: QueryUtil.getNewQueryURI(), imageClass: "create-query-command", buttonText: QueryEditorMessages.CREATE_QUERY_BUTTON_TEXT, boldFirstWord: true, size: "small"});
			this._addChild(this._newQueryCommand);
			
			this._recentWorkItemWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.common.RecentWorkItemView( 
						{parentController: this.controller, heading: "Recently Viewed"});
			this._addChild(this._recentWorkItemWidget);
			this._recentWorkItemWidget.setFilter({ showQuery: true, showSaved: true, showUnsaved: false } );
			
			this._unfinishedWorkItemWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.common.RecentWorkItemView( 
						{parentController: this.controller, heading: "Unsaved Work", hasSectionCount: true});
			this._addChild(this._unfinishedWorkItemWidget);
			this._unfinishedWorkItemWidget.setFilter({ showQuery: false, showSaved: false, showUnsaved: true });
			
			this._recentWorkItemWidget.navSection = this._leftNavigationLayout.addNavigationSection(this._recentWorkItemWidget, this._recentWorkItemWidget.heading, true);
			this._addChild(this._recentWorkItemWidget.navSection);		
			dojo.addClass(this._recentWorkItemWidget.navSection.domNode,"net-jazz-ajax-hidden");
			
			this._unfinishedWorkItemWidget.navSection = this._leftNavigationLayout.addNavigationSection(this._unfinishedWorkItemWidget, this._unfinishedWorkItemWidget.heading, true);
			this._addChild(this._unfinishedWorkItemWidget.navSection);
			dojo.addClass(this._unfinishedWorkItemWidget.navSection.domNode,"net-jazz-ajax-hidden");
			
			this._recentWorkItemWidget.addChildSubsection(this._unfinishedWorkItemWidget);
			
			this._multipaneContentWidget.setItemListWidget(this._recentWorkItemWidget);
			
			var queryArgs= {parentController: this.controller};

			// add 'My Queries' section only for logged-in users
			if (!com.ibm.team.repository.web.client.session.isGuest()) {
				queryArgs.queryType = "myQueries";
				queryArgs.type = this.queryType;
				var myQueriesWidget = /* createWidget */ new com.ibm.epm.web.core.ui.internal.view.query.SavedQueries(queryArgs);
				this._addChild(myQueriesWidget);
				var myQuerySection = this._leftNavigationLayout.addNavigationSection(myQueriesWidget, QueryEditorMessages.MYQUERY_SECTION_LABEL, true);
				this._addChild(myQuerySection);
				myQueriesWidget.populate(myQuerySection);
			}

			//Commented to remove shared queries section from view
			//queryArgs.queryType = "shared";
			//queryArgs.type = this.queryType;
			//var predefinedQueriesWidget = /* createWidget */ new com.ibm.epm.web.core.ui.internal.view.query.SavedQueries(queryArgs);
			//this._addChild(predefinedQueriesWidget);
			
			//var sharedQueriesSection = this._leftNavigationLayout.addNavigationSection(predefinedQueriesWidget, QueryEditorMessages.SHAREDQUERY_SECTION_LABEL, true);
			//this._addChild(sharedQueriesSection);
			//predefinedQueriesWidget.populate(sharedQueriesSection);
			
			this._registerNavigation();

			this._unloadHandle = dojo.connect(window, "onunload", dojo.hitch(this, this.unload)); // use dojo.connect here
			
		},

		unload: function() {
	//		this.destroy();  // DO NOT DESTROY - causes browser crashes
		},
		_registerNavigation: function() {
			if(QueryUtil.isDebugMode()) return; // too many clicks
			if (!this.navigateAwayHandle) {
				this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
			}
			if (!this.onBeforeUnloadHandle) {
				this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
			}
		},
		changesPending: function() {
			if(!this._unfinishedWorkItemWidget) return false;
			return this._unfinishedWorkItemWidget.sectionCount > 0;
		},
		_handleNavigate: function(urlObj){
			var winLoc = new dojo._Url(window.location.href);
			var workbenchNavigation = false;
			if (urlObj.host && urlObj.scheme && !(urlObj.host === winLoc.host && urlObj.scheme === winLoc.scheme)) {
				// different domain - allow onBeforeUnload to prompt
				var navigateAway = true;
			}
			else 
				if (urlObj.fragment && ((!urlObj.host && !urlObj.path) || (urlObj.path && urlObj.path === winLoc.path))) {
					// link within web UI
					if (urlObj.fragment.indexOf("com.ibm.team.workitem") < 0) {
						// we're navigating to another area of the web UI
						workbenchNavigation = true;
					}
					if (this._stripTabParam(urlObj.fragment) == this._stripTabParam(winLoc.fragment)) {
						// the link is to the current url (not including the tab param)
						var navigateAway = true;
					}
					else 
						var navigateAway = this.navigateAwayEvent(false, workbenchNavigation);
				// same exact base URI - prompt for save/cancel
				}
				else {
					var navigateAway = true;
				}
			return !navigateAway;
		},
	_stripTabParam: function(string) {
		if (!string)
			return string;
		var index = string.indexOf("&tab=");
		if (index == -1)
			return string;
		var result = string.substring(0,index);
		var end = string.substring(index+1);
		index = end.indexOf("&");
		if (index == -1)
			return result;
		return result + end.substring(index);
	},
		navigateAwayEvent: function(flag, navigation) {
			if(!navigation) return true;
			if (this.changesPending()) {
				if (confirm("Are you sure you want to navigate away from this page?\n\nYou have unsaved changes.  You can change pages but must remember to return to the Work Items page to finish your unsaved work.\n\nPress OK to continue, or Cancel to stay on the current page.")) {
					PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
					this.navigateAwayHandle = null;
					
					// Remove the lines to keep the warning when navigating away from anywhere
//					PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);
//					this.onBeforeUnloadHandle = null;
					
					return true;
				}
				else {
					return false;
				}
			}
			return true;
		},
		_handleNavigateAway: function(e) {
			return this.changesPending() ? "You have unsaved work in the Work Items page.  If you continue, your changes will be discarded." : false;
		},
		//
		// Other work item code may call these properties and methods.
		//
		
		/*
		 * Returns the internal id of the widget currently displayed in the
		 * page's content area.
		 */
		getCurrentWidgetId: function() {
			return this._currentMainWidgetId;
		},
		
		/*
		 * Returns the work item object currently displayed in the page's
		 * content area.
		 */
		getCurrentWorkItem: function() {
			if(this._multipaneContentWidget &&this._multipaneContentWidget.getCurrentWidget()) {
				return this._multipaneContentWidget.getCurrentWidget()._workItem;
			}
			return null;
		},
		
		isWorkItemEditorAvailable: function(id) {
			return this._multipaneContentWidget.isWidgetAvailable(WORK_ITEM_EDITOR_ID, id)
		},
		
		/*
		 * Shows the query object specified by <code>query</code> in the query
		 * builder.
		 * 
		 * @param {Object} query the query object to display in the builder.
		 * @param {boolean} keepResults (optional) whether or not to keep the
		 *  current query results view intact.
		 */
		showQueryInEditor: function(query, keepResults) {
			var queryBuilder= this._getQueryBuilderWidget(query.queryType);
			this._setCompositeContent(QUERY_BUILDER_ID, queryBuilder.domNode, null);
			if (!keepResults) {
				this._getQueryCompositeWidget().clearContent(QUERY_RESULTS_ID);
			}
			queryBuilder.showQuery(query);
		},
		// a new query came in... clear the old name 
		resetQueryEditor: function() {
			this._getQueryBuilderWidget().clearName();
		},

		showWelcomePage: function(errorMsg) {
			this.controller._setPageTitle("Work Items");
			if(errorMsg) {
				this.welcomePageWidget.setErrorMessage(errorMsg);
			} else {
				this.welcomePageWidget.clearErrorMessage();
			}
			this._setMainContent(WELCOME_PAGE_ID, this.welcomePageWidget, null, 0, /* updateNav */ false);
		},
		
		/*
		 * Shows the work item editor for the specified work item.
		 * <p>
		 * If the caller specifies a value for <code>fromQueryDescriptor</code>,
		 * display the meta-area widget for iterating through the query results
		 * in the context of editing work items.
		 * </p><p>
		 * If the caller specifies a value for <code>initialTab</code>, inform
		 * the work item editor to display that tab initially.
		 * </p>
		 * 
		 * @param {Object} workItem the work item object to display in an
		 *  editor.
		 * @param {Object} fromQueryDescriptor (optional) the query from which
		 *  this work item was selected.
		 * @param {String} initialTab (optional) the name of the initial work
		 *  item editor tab to display.
		 */
		showWorkItemEditor: function(workItem, initialTab, isRefresh, url) {
			checkNotNull(workItem, "workItem");
			
			this.controller._setWorkItemPageTitle(workItem);
			var workItemEditor= this._getWorkItemEditorWidget(workItem, initialTab, url);
			
			if(isRefresh) {
				workItemEditor.scrollTop = 0;
				workItemEditor.scrollLeft = 0;
			}

			var banner = this._getWorkItemEditorBanner(workItem.id);

			this._setMainContent(
					WORK_ITEM_EDITOR_ID,
					workItemEditor,
					 banner ? banner.domNode : null,
					workItem.id);
			
					
			var setIsDirty = workItemEditor.setWorkItem(workItem, initialTab);
			if(workItem.id < 1) { 
				workItemEditor._editorParts.header.summary.focus();
			}
			this.setIsDirty(workItemEditor, setIsDirty);		
		},
		
		/*
		 * For the work item identified by 'id', show the tab specified by
		 * 'tabName'. If the user clicked the 'Save' control on the left-nav,
		 * prompts user to confirm save.
		 * 
		 * @param {String} id the identifier for the work item editor for which
		 *  we'll show the specified tab.
		 * @param {String} tabName the identifier for the editor tab to show.
		 * @param {boolean} isDoingSave (optional) if 'true', confirms to save
		 *  the work item and saves on confirmation; defaults to 'false' if not
		 *  specified.
		 */
		_lastDoSaveId: 0,
		showEditorTab: function(id, tabName, doSaveId) {
			var isDoingSave = doSaveId != null;
			var workItemEditorWidget = this._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, id);
			this.controller._setWorkItemPageTitle(workItemEditorWidget._workItem);
			checkNotNull(workItemEditorWidget);
			
			var banner = this._getWorkItemEditorBanner(id);			
			workItemEditorWidget.showTab(tabName);
			this._currentMainWidgetId = WORK_ITEM_EDITOR_ID;
			this._multipaneContentWidget.showContent(WORK_ITEM_EDITOR_ID, workItemEditorWidget, id);
			this._updateWorkItemEditorBanner(id);			
			this._setMainContent(
					WORK_ITEM_EDITOR_ID,
					workItemEditorWidget,
					banner ? banner.domNode : null,
					id,
					/* skipNavUpdate */ true // already done in showContent
					);

			// The doSaveId shenanigans is to prevent the "Save" dialog from appearing after it is cancelled
			if (isDoingSave && doSaveId != this._lastDoSaveId) {
				this._lastDoSaveId = doSaveId;
				workItemEditorWidget.askToSave();
			}
		},
		_updateSaveParameters: function() {
			this._recentWorkItemWidget.updateSaveParameters();
		},
		/*
		 * 
		 */
		removeEditorTab: function(id, /* show next tab */ showNext) {
			this._multipaneContentWidget.removeCachedWidget(WORK_ITEM_EDITOR_ID, id);
			if(showNext) {
				if(!this._multipaneContentWidget.showNextView()) {
					this.controller._openDefaultView();
				}
			}
			
		},
		
		
		/*
		 * For the specified work item editor widget, mark as dirty if 'isDirty'
		 * is 'true' or clean if 'isDirty' is false.
		 * 
		 * @param {Object} workItemEditorWidget the work item editor instance to
		 *  mark as clean or dirty.
		 * @param {boolean} isDirty if 'true', marks the specified editor as
		 *  dirty; if 'false', marks the specified editor as clean.
		 */
		setIsDirty: function(workItemEditorWidget, isDirty) {
			this._multipaneContentWidget.setIsDirty(WORK_ITEM_EDITOR_ID, workItemEditorWidget, workItemEditorWidget._workItem.id, isDirty);
		},
		
		//
		// No code outside this module should access these properties and
		// methods.
		//
		
		templatePath: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/page/templates/QueryEditorPageView.html"),
//		templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/page/templates/QueryEditorPageView.css"),
		
		/*
		 * Required by Jazz web framework to make hyperlinking and back/forward
		 * work.
		 */
		getController: function() {
			return this.controller;
		},
		
		_getWorkItemEditorWidget: function(workItem, initialTab, url) {
			checkNotNull(workItem, "workItem");
			var workItemEditorWidget = this._multipaneContentWidget.getCachedWidget(WORK_ITEM_EDITOR_ID, workItem.id);
			
			if (!workItemEditorWidget) {
				workItemEditorWidget=
					/* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditor({parentController: this.controller, url: url, initialTab: initialTab});
//				this._addChild(workItemEditorWidget);  /* Not needed, destroyed by multipanecontentwidget */
			}

			return workItemEditorWidget;
		},
		setFilter: function(filterValue) {
			if(this._queryResultPageControl) { this._queryResultPageControl.setFilter(filterValue); }
			if(this._getQueryResultsWidget()) { this._getQueryResultsWidget().filter(filterValue); }
		},		
		showLastQueryResults: function() {
			this._setMainContent(QUERY_RESULTS_ID, this._getQueryCompositeWidget());
			var widget = this._getQueryResultsWidget();
			if(widget) { 
				widget.clearSelection(); 
				};
		},

		// Update the "New Query" link to give it a unique id so it will reset the query builder
		makeNewQueryLinkUnique: function() {
			if (this._newQueryCommand.domNode)
				this._newQueryCommand.domNode.href = QueryUtil.getNewQueryURI();			
		},

		showQueryResults: function(query, queryResultsSummaryPage, orderByAttrib, orderDirection, itemsPerPage, keepEditor) {
			checkNotNull(query, "query");
			checkNotNull(queryResultsSummaryPage, "queryResultsSummaryPage");
			
			// Org Code :-
			//checkNonNegative(queryResultsSummaryPage.totalCount, "queryResultsSummaryPage.totalCount");
			//checkNonNegative(queryResultsSummaryPage.startIndex, "queryResultsSummaryPage.startIndex");


			// Hardcoding the page control system
			queryResultsSummaryPage.startIndex = 0;
			queryResultsSummaryPage.limit = 10;
			
			
			if (!keepEditor)
				this._getQueryCompositeWidget().clearContent(QUERY_BUILDER_ID);
				
			var pageControlParms= {
				parentView: this,
				query: query,
				queryResultsSummaryPage: queryResultsSummaryPage,
				queryResultsWidget: this._getQueryResultsWidget(),
				queryOrderBy: orderByAttrib ? orderByAttrib : null,
				queryOrderDirection: orderDirection ? orderDirection : null,
				queryItemsPerPage: itemsPerPage ? itemsPerPage : null};
			
			var oldQueryResultPageControl = this._queryResultPageControl;
			var oldQueryResults = this.queryResultsWidget;
			
			this._queryResultPageControl = /* createWidget */ new com.ibm.epm.web.core.ui.internal.page.QueryResultPageControl(pageControlParms);
			// destroyed individually
			
			var results = this._getQueryResultsWidget(true);
			results.setQueryResultsTable(query, queryResultsSummaryPage, orderByAttrib, orderDirection, itemsPerPage);	

			this._setCompositeContent(
					QUERY_RESULTS_ID,
					results.domNode,
					this._queryResultPageControl.domNode);
					
			if (oldQueryResults) {
				oldQueryResults.destroy();
			}
			if (oldQueryResultPageControl) {
				oldQueryResultPageControl.destroy();
			}
		},
		resetFilter: function() {
			if(!this._queryResultPageControl) { return };
			this._queryResultPageControl._resetFilterControl();
		}, 
		_getQueryResultsWidget: function(forceCreate) {
			if (forceCreate || !this.queryResultsWidget) {
				var parms= {parentController: this.controller, parentView: this};
				
				this.queryResultsWidget	= /* createWidget */ new com.ibm.epm.web.core.ui.internal.view.query.QueryResults(parms);

			}
			return this.queryResultsWidget;
		},
		showQueryBottomPageControl: function(query, queryResultsSummaryPage, orderByAttrib, orderDirection, itemsPerPage, domNode) {

			var pageControlParms= {
				parentView: this,
				query: query,
				queryResultsSummaryPage: queryResultsSummaryPage,
				queryResultsWidget: this._getQueryResultsWidget(),
				queryOrderBy: orderByAttrib,
				queryOrderDirection: orderDirection,
				queryItemsPerPage: itemsPerPage ? itemsPerPage : null,
				isBottomControl: true
				};
			
			if(this._queryBottomPageControl) {
				this._queryBottomPageControl.destroy();
			}
			this._queryBottomPageControl =/* createWidget */ new com.ibm.epm.web.core.ui.internal.page.QueryResultPageControl(pageControlParms); 
			
			domNode.appendChild(this._queryBottomPageControl.domNode);

		},
		
		_getQueryBuilderWidget: function(queryType) {
			if (!this.queryBuilderWidget) {
				var parms= {parentController: this.controller};
				this.queryBuilderWidget= new com.ibm.epm.web.core.ui.internal.view.queryeditor.QueryEditor(parms);
				this._addChild(this.queryBuilderWidget);
			}
			
			if(queryType) {
					this.queryBuilderWidget.changeQueryTypeInEditor(queryType);			
			}
			return this.queryBuilderWidget;
		},
		
		_getQueryCompositeWidget: function() {
			if (!this.queryCompositeWidget) {
				var parms= {parentController: this.controller};
				this.queryCompositeWidget= /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.common.CompositeContentArea(parms);
				this._addChild(this.queryCompositeWidget);
			}
			return this.queryCompositeWidget;
		},
		
		_setMainContent: function(widgetId, mainAreaWidget, metaAreaWidgetDomNode, instanceId, /* updateNav */ skipNavUpdate) {
			this._currentMainWidgetId= widgetId;
			
			// Display the mainAreaWidget in the multi-pane content area
			this._multipaneContentWidget.showContent(widgetId, mainAreaWidget, instanceId, skipNavUpdate);
			
			
			// Set the content to display the multi-pane content area
			this._leftNavigationLayout.setMainContent(this._multipaneContentWidget.domNode, metaAreaWidgetDomNode);
		},
		/*
 		* Return the work item editor banner
 		* This banner is used on all work items and updated whenever a work item is loaded
 		* If the work item is found in the current query results, which always exist
 		* in the "Recently Viewed" cache, then show the banner, regardless of how
 		* the navigation to the work item occured.
 		*/
		_getWorkItemEditorBanner: function(workItemId) {

			if(this._workItemEditorBanner == null) {
				this._workItemEditorBanner = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorBanner(
							{parentController: this.controller, parentView: this});
				this._addChild(this._workItemEditorBanner);
			}
			
			this._updateWorkItemEditorBanner(workItemId);
			return this._workItemEditorBanner;
		},
		_updateWorkItemEditorBanner: function (workItemId) {
			// If no query yet run, return null
			var results;
			if(this.queryResultsWidget) { 
				results = this.queryResultsWidget.getResults();
			}
			this._workItemEditorBanner.setQueryInfo(workItemId, this.controller._lastQueryName, this.controller._lastQueryURL, results);
			
		},
		
		_setCompositeContent: function(widgetId, mainAreaWidget, metaAreaWidget) {
			var composite= this._getQueryCompositeWidget();
			composite.scrollLeft = 0;
			composite.scrollTop = 0;
			composite.setContent(widgetId, mainAreaWidget, metaAreaWidget);
			if (this._currentMainWidgetId != QUERY_COMPOSITE_ID)
				this._setMainContent(QUERY_COMPOSITE_ID, composite);
			else 
				// Display the mainAreaWidget in the multi-pane content area
				this._multipaneContentWidget.showContent(QUERY_COMPOSITE_ID, composite);

		},
		
		_displayError: function(errorMessage){
		   var self = this;
           EPMErrorHandler.setError(errorMessage);
            // Set an interval and clear the error message
            var intervalID = setInterval(function(){
                self._clearErrorMessage();
                clearInterval(intervalID);
            }, 10000);
        },
        
        _clearErrorMessage: function(){
            EPMErrorHandler.clearError();
        }
	}
);

})();
