/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.projectwizard.ProjectWizard");

    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.epm.web.core.grid.controller.GridModelClient");
	dojo.require("com.ibm.epm.admin.web.client.internal.calendar.CalendarClient");
    dojo.require("com.ibm.team.process.web.client.projectarea");
	dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
	dojo.require("com.ibm.epm.work.web.ui.internal.WBSTreePubSub");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");

	dojo.require("dijit.form.DateTextBox");
	dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
	dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
	
	dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");

    dojo.require("com.ibm.epm.web.core.transport.EPMServerClient");
    dojo.require("com.ibm.epm.web.core.transport.ServiceRequest");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
    
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var calendarClient = com.ibm.epm.admin.web.client.internal.calendar.CalendarClient;	
	var GridModelClient = com.ibm.epm.web.core.grid.controller.GridModelClient;
	var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var WBSTreePubSub = com.ibm.epm.work.web.ui.internal.WBSTreePubSub;
	var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
	var projectsURI = "projects";
	var assignmentTypes = [WBSConstants.ASSIGNMENT_TYPE_EFFORT, WBSConstants.ASSIGNMENT_TYPE_DURATION];
	var labourForecastTypes = [WBSConstants.LABOUR_FORECAST_TYPE_CALCULATE, WBSConstants.LABOUR_FORECAST_TYPE_MANUAL];
	var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
	var Util = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util;
		
	var IconClass = com.ibm.epm.web.core.ui.internal.IconClassConstants;
	
    var ServiceRequest = com.ibm.epm.web.core.transport.ServiceRequest;
    var EpmServerClient = com.ibm.epm.web.core.transport.EPMServerClient;

	var okButtonID = "wizardSaveButtonId";
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.projectwizard.ProjectWizard", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.epm.work.web", "ui/internal/projectwizard/templates/ProjectWizard.html"),
		widgetsInTemplate: true,
				
        constructor: function() {
			this._startup();
			this._populateLabels();
        },
		
		/**
		 * This method will called during the start up and will clean all
		 * the existing contents if any
		 */
		_startup: function () {
			var projectNameId = dojo.byId("ProjectNameId");
			if (projectNameId) {
				projectNameId.destroy();
			}
			var parentNameId = dojo.byId("ParentNameId");
			if (parentNameId) {
				parentNameId.destroy();
			}
			var assignTypeId = dijit.byId("AssignTypeId");
			if (assignTypeId) {
				assignTypeId.destroy();
			}
			var labourForecastTypeId = dijit.byId("labourForecastTypeId");
			if (labourForecastTypeId) {
				labourForecastTypeId.destroy();
			}
			var projectCalendarId = dijit.byId("ProjectCalendarId");
			if (projectCalendarId) {
				projectCalendarId.destroy();
			} 
			var startDateId = dijit.byId("StartDateId");
			if (startDateId) {
				startDateId.destroy();
			}
			var endDateId = dijit.byId("EndDateId");
			if (endDateId) {
				endDateId.destroy();
			}
			var saveButtonId = dijit.byId(okButtonID);
			if (saveButtonId) {
				saveButtonId.destroy();
			}
			var cancelButtonId = dijit.byId("cancelButtonId");
			if (cancelButtonId) {
				cancelButtonId.destroy();
			}
		},
		
		/**
		 * This method is to set labels for the content widgets
		 */
		_populateLabels: function() {
			this.projectName = WorkMessages.PROJECT_CAPTION;
			this.parentName = WorkMessages.PARENT_CAPTION;
			this.calendar = WorkMessages.CALENDAR_CAPTION;
			this.assignType = WorkMessages.ASSIGNMENT_TYPE_CAPTION;
			this.startDate = WorkMessages.START_DATE_CAPTION;
			this.endDate = WorkMessages.FINISH_DATE_CAPTION;
			this.labourForecastType = WorkMessages.LABOUR_FORECAST_TYPE_CAPTION;
			this.saveButton = WorkMessages.PROJECT_WIZARD_SAVE_CAPTION;
			this.cancelButton = WorkMessages.PROJECT_WIZARD_CANCEL_CAPTION; 
		},

		/**
		 * This method is to destroy the contents
		 */
		destroy: function() {
			if (this.startDateWidget) {
				delete this.startDateWidget;
			}
			if (this.endDateWidget) {
				delete this.endDateWidget;
			}
			if (this.appadminCalendars) {
				delete this.appadminCalendars;
			}
			if(this.assignTypeDropDown){
				delete this.assignTypeDropDown;
			}
			if (this.startDateEvent) {
				this.disconnect(this.startDateEvent);
				this.startDateEvent = null;
			}
			if (this.endDateEvent) {
				this.disconnect(this.endDateEvent);
				this.endDateEvent = null;
			}
			if (this.keyEvent) {
				this.disconnect(this.keyEvent);
				this.keyEvent = null;
			}
			if(this.labourForecastTypeDropDown){
				delete this.labourForecastTypeDropDown;
			}
			if (this.dialog) {
				this.dialog.close();
				this.dialog.destroy();
			}
		},
        
		/**
		 * This method is used to laod the content widgets
		 * After loading the fields, setting the focus to the project name field. This is to close the pop-up menu in WBS Tree.
		 * @param {Object} wbsTree
		 * @param {Object} dialog
		 * @param {Object} parentItem
		 */
		load: function(wbsTree, dialog, parentItem) {
			this.dialog = dialog;
			this.dialog._close.style.display = "none";	//Removing the 'X' close button from dialog toolbar. User can use only Cancel button to close project wizard.
			
			this.parentItem = parentItem;
			this.parentNodeID = dojo.isArray(parentItem.relativeHref) ? parentItem.relativeHref[0] : parentItem.relativeHref;
			this._wbsTree = wbsTree;
			if (parentItem.name) {
				dojo.byId("ParentNameId").value = dojo.isArray(parentItem.name) ? parentItem.name[0] : parentItem.name; 
			}
			else if (parentItem.summary) {
				dojo.byId("ParentNameId").value = dojo.isArray(parentItem.summary) ? parentItem.summary[0] : parentItem.summary; 
			}
			
			// Creating Calendars list
			this._populateCalendarsList();
			// Populating the process templates
//			this._populateProcessTemplates();
			// Creating Start Date, End Date fields
			this._createDateTextBox(this.startDateAttachPoint, this.startDate);
			this._createDateTextBox(this.endDateAttachPoint, this.endDate);
			this._setTheConstraintOfDates();// newly added to keep the constraint of the dates
			this._createAssignTypeComboBox();
			this._createLabourForecastTypeComboBox();
			
			//Connecting the keydown event & setting default focus
			this.keyEvent = this.connect(this.dialog, "onKeyDown", this._handleKeyEvent); 
			dojo.byId("ProjectNameId").focus();
			
			//Set the image soruce for calendar icon
			this._calStartDateImage.src = this._calEndDateImage.src = dojo.moduleUrl("com.ibm.epm.admin.web" ,"ui/internal/view/images/calendar.gif"); 
		},
		
		/**
		 * _populateCalendarsList:
		 * 		1. Create project under project. 
		 * 			It should pick the parent project's calendar & disable the calendar dropdown.
		 * 		2. Create project under program.
		 * 			It should pick the list of the system calendars and populate the calendar dropdown. 
		 */
		_populateCalendarsList: function() {
            this.appadminCalendars = document.createElement("select");

            var self = this;
			if ( (this.parentItem.workItemType) && (this.parentItem.workItemType == WBSConstants.EPM_PROJECT_TYPE) ) {
				var parentCalendar = this.parentItem.Calendar.title;
				this.appadminCalendars.options[0] = new Option(parentCalendar);
				this.appadminCalendars.disabled = true;
			}
			else {
	            var calendarFunctions = {
	                success: function(calendars){
	                    var len = calendars.length;
	                    
	                    if (!calendars || len == 0) {
	                        self.getSite().setSecondaryTitle("");
	                    }
	                    else {
	                        for (var j = 0; j < calendars.length; j++) {
	                            var calendarHandle = calendars[j];
								self.appadminCalendars.options[j] = new Option(calendarHandle.title);
								self.appadminCalendars.options[j].value = calendarHandle.relativeHref;
	                        }
	                    }
	                },
	                failure: function(errorMsg){
	                    console.log(errorMsg);
	                }
	            };
	            var srh = new ServiceResponseHandler(calendarFunctions, "success", "failure");
	            calendarClient.getAllCalendars(srh);
			}
			
            this.calendarAttachPoint.appendChild(self.appadminCalendars);
		},
		
		/**
		 * Populating the existing Process Templates in a dropdown for selection.
		 *
		_populateProcessTemplates: function() {
			this.prcTemplatesDropDown = document.createElement("select");
			
			var self = this;
            var processTemplates = {
                success: function(prcTemplates){
                	if ( (!prcTemplates) || (prcTemplates == null) ) {
                		alert(WorkMessages.NO_PROCESS_TEMPLATES_MSG);
                		self.destroy();
                	}
                	else {
	                	var prcTemplatesList = prcTemplates.response.returnValue.values;
	                    var len = prcTemplatesList.length;
	                    
	                    if (len == 0) {
	                        self.getSite().setSecondaryTitle("");
	                    }
	                    else {
	                        for (var j = 0; j < len; j++) {
	                            var processHandle = prcTemplatesList[j];
								self.prcTemplatesDropDown.options[j] = new Option(processHandle.name);
								self.prcTemplatesDropDown.options[j].value = processHandle.id; 
	                        }
	                    }
                	}
                },
                failure: function(errorMsg){
                    console.log(errorMsg);
                }
            };
            var srh = new ServiceResponseHandler(processTemplates, "success", "failure");
            var serviceRequest = new ServiceRequest(WorkMessages.PROCESS_TEMPLATES_URI, "get");
            EpmServerClient.operateOnResource(serviceRequest, srh);
			
			this.prcTemplateAttachPoint.appendChild(this.prcTemplatesDropDown);
		},*/

		/**
		 * Function to disable the range at the time of creation of the Widget 
		 * Start date picker will have only dates before Finish date enabled and similarly with the finish date
		 */
		
		_setTheConstraintOfDates: function(){
			if(this.startDateWidget && this.endDateWidget)
			{
                // NOTE: There is no default minimum or maximum for start date
                // A maximum for the start date is set when the user explicitly
                // selects the finish date. Unless that happens, the user can
                // select any date as start date.	
				
                // The default minimum end date is the current date.
                // There is no default maximum for the end date.
                dojo.mixin(this.endDateWidget.constraints, {
                    min: this.endDateWidget.getValue()
                });
				
			}
		},
		/**
		 * For each start date and end date pickers this function is called 
		 * Both are created and initialized with todays date and required constraints
		 * @param {Object} attachPoint
		 * @param {Object} valueDate
		 */
		_createDateTextBox: function(attachPoint, valueDate) {
		    dojo.addClass(this.domNode, "DateTextContainer"); 

			var constraints = {};
			constraints = { formatLength: 'medium' 	};

			var dateWidget = null;
			var self = this;
			if (valueDate == WorkMessages.START_DATE_CAPTION) {
				self.startDateWidget= /* createWidget */ new dijit.form.DateTextBox (
					{ isDate: true, containerToggle: "fade", constraints: constraints },
				attachPoint);
				this.startDateEvent = this.connect(this.startDateWidget, "onChange", function(e) {
					self._handleStartDateChange();
				});
				dateWidget = this.startDateWidget; 
			} 
			else if (valueDate == WorkMessages.FINISH_DATE_CAPTION) {
				this.endDateWidget= /* createWidget */ new dijit.form.DateTextBox (
					{ isDate: true, containerToggle: "fade", constraints: constraints },
				attachPoint);
                this.initialEndDateSetup = true;
				this.endDateEvent = this.connect(this.endDateWidget, "onChange", function(e) {
					self._handleEndDateChange();
				});
				dateWidget = this.endDateWidget;
			} 
			
			var date = null;
			date = new Date();

			if (dateWidget) {
				// Date widget dynamically replaced its class so need to set element style.
				dateWidget.setValue(date);
				dateWidget.domNode.style.width = "80%";
				dateWidget.domNode.style.margin = "0";
				dateWidget.domNode.style.fontFamily = "inherit";
			}
		},
		
		/**
		 * _createAssignTypeComboBox: Creates & populates Project Assignment Type selection list.
		 * 			In case parent work item is of type project, parent's assignment type is forced to the child work item. 
		 */
		_createAssignTypeComboBox:function(){
			this.assignTypeDropDown = document.createElement("select");
			if ( (this.parentItem.workItemType) && (this.parentItem.workItemType == WBSConstants.EPM_PROJECT_TYPE) ) {
				var parentAssignmentType = this.parentItem.constraint.assignmentType;
				this.assignTypeDropDown.options[0] = new Option(parentAssignmentType);
				this.assignTypeDropDown.disabled = true;
			}
			else {
				this.assignTypeDropDown.options[0] = new Option(assignmentTypes[0]);
				this.assignTypeDropDown.options[1] = new Option(assignmentTypes[1]);
			}
			this.assignTypeAttachPoint.appendChild(this.assignTypeDropDown);
		},
		
		/**
		 * The method _createLabourForecastTypeComboBox will create a combo box
		 * with the valid options. On saving project, this value will be used 
		 * to set 'calculateForecast' of the budget resource.
		 */
		_createLabourForecastTypeComboBox:function(){
			this.labourForecastTypeDropDown = document.createElement("select");

			this.labourForecastTypeDropDown.options[0] = new Option(labourForecastTypes[0]);
			//The manual option is only for main projects and not for sub projects
			if ( (this.parentItem.workItemType) && (this.parentItem.workItemType == WBSConstants.EPM_PROJECT_TYPE) ) {
				this.labourForecastTypeDropDown.disabled = true;
			} 
			else {
				this.labourForecastTypeDropDown.options[1] = new Option(labourForecastTypes[1]);
			} 

			this.labourForecastTypeAttachPoint.appendChild(this.labourForecastTypeDropDown);
		},
		
		_handleStartDateClick: function(e) {
			
			this.startDateWidget.focus();
		},
		
		/**
		 * On Start date change , set the constraint min of end date to the new start date
		 * @param {Object} e
		 */
		_handleStartDateChange: function(e) {
            if (this.startDateWidget && this.endDateWidget) {

                // Reset the minimum value of the end date to new start date                
                dojo.mixin(this.endDateWidget.constraints, {
                    min: this.startDateWidget.value
                });

                if (this.endDateChangedByUser == null) {
                    
                    // Set a flag that end date is being changed by start date
                    this.endDateChangedByStartDate = true;
                
                    // Reset the end date to the new start date 
                    this.endDateWidget.setValue(this.startDateWidget.value);
                }   
            }
		},
		
		_handleEndDateClick: function(e) {
			this.endDateWidget.focus();
		},
		
		/**
		 * On end date change , set the constraint max of start date to the new end date
		 * @param {Object} e
		 */
		_handleEndDateChange: function(e) {
            if (this.initialEndDateSetup != null) {
                // The end date change is being called by _createDateTextBox()
                // when the event handler is being setup.
                // Reset as the case being handled
                this.initialEndDateSetup = null;
            } else {    
                 if (this.startDateWidget && this.endDateWidget) {
                
                     if (this.endDateChangedByStartDate != null) {
                         // End date is being changed by start date
                         // Nothing to do here other than resetting the case as being handled
                         this.endDateChangedByStartDate = null;
                     } else {                         
                        // The user explicitly selected a new end date      
                        // Reset the maximum start date to the new end date
                        dojo.mixin(this.startDateWidget.constraints, {
                            max: this.endDateWidget.value
                        });
                        
                        this.endDateChangedByUser = true;
                     }                        
                 }
            }			
        },
		
		/**
		 * _handleKeyEvent: 
		 * 			saves project on enter key,
		 * 			closes wizard on escape key.
		 * @param {Object} e
		 */
		_handleKeyEvent: function(e) {
			if (e.keyCode == dojo.keys.ENTER) {	//Enter key
				if (dojo.byId(okButtonID).disabled == false) {
					this._saveProject();
				}
			}
			if (e.keyCode == dojo.keys.ESCAPE) {	//Esc key
				this._closeWizard();
			}
		},
		
		_saveProject: function() {
			if (!this._validateInputs()) {
				dojo.byId("InfoLabelId").innerHTML = WorkMessages.INVALID_WIZARD_INPUTS;
				dojo.byId("ProjectNameId").focus();
				return;				
			}
			dojo.byId("InfoLabelId").innerHTML = "";
			this._disableWizardFields(true);
			
			//Dispaying the save project status information
			var projName = dojo.byId("ProjectNameId").value;
			Util.showMessage(WorkMessages.SAVING+" "+projName);
			
			var projectHref;
			var self = this;
            var serviceObj = {
                success: function(data){
					projectHref = data.relativeHref;
					self._insertProjectToTree(data);
					Util.hideMessageArea();
					self.destroy();
                },
                failure: function(errorObj){
					self._disableWizardFields(false);
					Util.hideMessageArea();
					dojo.byId("InfoLabelId").innerHTML = errorObj.errorMessage;
					self.appadminCalendars.focus();
					dojo.byId("ProjectNameId").focus();
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			
			var params = self._prepareInputParams();
			GridModelClient.postURI(srh, "/" + projectsURI, params);
		},
		
		/**
		 * 
		 * @param {Object} state
		 */
		_disableWizardFields: function(state) {
			dojo.byId("ProjectNameId").disabled = state;
			if ( (!this.parentItem.workItemType) ) {
				this.appadminCalendars.disabled = state;
				this.assignTypeDropDown.disabled = state;
			}
//			this.prcTemplatesDropDown.disabled = state;
			dijit.byId("StartDateId").setDisabled(state);
			dijit.byId("EndDateId").setDisabled(state);
			this.labourForecastTypeDropDown.disabled = state;
			dojo.byId(okButtonID).disabled = state;
			dojo.byId("cancelButtonId").disabled = state;
		},
		
		/**
		 * This method is to prepare the save params which it will
		 * sent to server
		 */
		_prepareInputParams: function() {
			var inputparams = {};
			inputparams.summary = dojo.byId("ProjectNameId").value;
			
            var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
            var projectAreaName = getCurrentProjectArea().name;
            inputparams.ProjectAreaName = projectAreaName;
			
			var parentID = this.parentNodeID;
			if (dojo.indexOf(parentID, projectsURI) != -1) {
            	inputparams[WBSConstants.PARENT] = parentID;
			}else{
				inputparams[WBSConstants.PROGRAM_LINK] = parentID;
			}

			if ( (this.parentItem.workItemType) && (this.parentItem.workItemType == WBSConstants.EPM_PROJECT_TYPE) ) { //Copying the parent project's assignment type
				var selectedAssignType = this.parentItem.constraint.assignmentType;
			} 
			else {
				var selectedAssignType = assignmentTypes[this.assignTypeDropDown.selectedIndex];

				var selectedCalendarURI = this.appadminCalendars.options[this.appadminCalendars.selectedIndex].value; 
				inputparams.Calendar = selectedCalendarURI;
			}
			
			inputparams.constraint =  	WBSConstants.CONSTRAINTTYPE + WBSConstants.EQUALTO + WBSConstants.AS_SOON_AS_POSSIBLE 
										+ WBSConstants.COMMA +
										WBSConstants.ASSIGNMENT_TYPE_PARAM + WBSConstants.EQUALTO  + selectedAssignType;
			
			var proposedStartDate = this.startDateWidget.valueNode.value;
			var proposedFinishDate = this.endDateWidget.valueNode.value;

        	inputparams.proposedSchedule =  WBSConstants.STARTDATE + WBSConstants.EQUALTO +  proposedStartDate +WBSConstants.TIMESTAMP + 
									WBSConstants.COMMA + WBSConstants.FINISHDATE + WBSConstants.EQUALTO +  proposedFinishDate  + WBSConstants.TIMESTAMP ;
			
			inputparams.labourForecast = labourForecastTypes[this.labourForecastTypeDropDown.selectedIndex]; 
			
//			var selectedPrcTemplateID = this.prcTemplatesDropDown.options[this.prcTemplatesDropDown.selectedIndex].value;
//			inputparams.processDefinition = selectedPrcTemplateID;
			
			var parentNode = this._wbsTree.getNodeByItemIdentity(this.parentNodeID);
			inputparams.rpcSequenceValue = this._wbsTree._getSequencevalueForNewNode(parentNode);
			return inputparams;  		
		},
		
        _getResourceIDFromURI: function(relativeHref){
			var index = dojo.lastIndexOf(relativeHref, "/");
            if (index > 0) {
                return relativeHref.substring(index + 1, relativeHref.length);
            }
        },
		
		/**
		 * Validates the blank project name
		 * 		in case start date is > end date, copies the start date to end date. 
		 */
		_validateInputs: function() {
			var projName = dojo.byId("ProjectNameId").value;
			if (projName == "") {
				return false;
			}
			if (this.startDateWidget.valueNode.value > this.endDateWidget.valueNode.value) {
				var strStartDate = this.startDateWidget.valueNode.value;
				strStartDate = dateUtil.parseDate(strStartDate);
				this.endDateWidget.setValue(strStartDate);
				if (this.startDateWidget._opened) {
					this.startDateWidget._close();
				} 
				else if (this.endDateWidget._opened) {
					this.endDateWidget._close();
				}
			}
			return true;
		},
		
		_insertProjectToTree: function(projectLink) {
			var projectHref = projectLink.relativeHref;
			/*var iconUrl = IconClass.PROJECT_CLOSED_ICON_CLASS;
            var item = {
                "href": projectHref,
                "name": dojo.byId("ProjectNameId").value,
                "identifier": projectHref,
                "resourcetype": WBSConstants.EPM_PROJECT_TYPE,
                "workItemType": "",
                "parentIdentifier": this.parentNodeID,
                "detailLevel": false,
                "iconClass": null,
				"iconUrl" : iconUrl
            };*/
			var parentNode = this._wbsTree.getNodeByItemIdentity(this.parentNodeID);
			var item = this._wbsTree._getTreeNodeModel(projectLink, this.parentNodeID);
            this._wbsTree._addNode(parentNode.item, item, 0);

            if ((parentNode.isExpandable) && (!parentNode.isExpanded)) {
                this._wbsTree._expandNode(parentNode);
            }
			var createdNode = this._wbsTree._itemNodeMap[projectHref];
			this._wbsTree._selectTreeNode(createdNode);
			this._publishCreateNodeEvent(item);
		},
		
		/**
		 * _publishCreateNodeEvent: To publish create node event after the project is successfully created.
		 * 							Overview & Schedule grids listens to this event and inserts corresponding project row in the grid store.
		 * Publish ON_CREATE_NODE: Not sending the published object. This is to make sure the grid's save button does not get enabled after project creation. 
		 */
        _publishCreateNodeEvent: function(item){
			
			var publishedObj = {
				name:item.name,
				href:item.href,
				identifier: item.identifier,
				resourcetype: item.resourcetype,
				treeID: WBSConstants.WBS_TREE_ID
            };
            
            dojo.publish(WBSTreePubSub.ON_CREATE_NODE, [publishedObj]);
			
			//The publishedObject in the select node has the identifier
			//as an array
			publishedObj.identifier = [item.identifier];
			// Select the newly added row in the grid
			dojo.publish(EPMTreePubSub.ON_SELECT, [publishedObj]);			
        },
		
		_closeWizard: function() {
			this.destroy();
		}
		
	});
})();	
    
