/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarExceptionDialog");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    
    dojo.require("dijit.form.Textarea");
    // Client
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.admin.web.client.internal.calendar.CalendarClient");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    
    // Localization
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
    var Messages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var CalendarClient = com.ibm.epm.admin.web.client.internal.calendar.CalendarClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var DateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarExceptionDialog", [dijit._Widget, dijit._Templated], {
        widgetsInTemplate: true,
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/calendar/templates/CalendarExceptionDialog.html"),
        
        weekday: [Constants.SUNDAY, Constants.MONDAY, Constants.TUESDAY, Constants.WEDNESDAY, Constants.THURSDAY, Constants.FRIDAY, Constants.SATURDAY],
        weekdayNames: [Messages.SUNDAY, Messages.MONDAY, Messages.TUESDAY, Messages.WEDNESDAY, Messages.THURSDAY, Messages.FRIDAY, Messages.SATURDAY],
        monthNames: [Messages.JANUARY, Messages.FEBRUARY, Messages.MARCH, Messages.APRIL, Messages.MAY, Messages.JUNE, Messages.JULY, Messages.AUGUST, Messages.SEPTEMBER, Messages.OCTOBER, Messages.NOVEMBER, Messages.DECEMBER],
        
        /**
         * Constructor - Destroys widgets to be initialized if they exist already
         */
        constructor: function(){
            if (dijit.byId("calExFromDate")) {
                dijit.byId("calExFromDate").destroy();
            }
            if (dijit.byId("calExToDate")) {
                dijit.byId("calExToDate").destroy();
            }
            if (dijit.byId("calExComment")) {
                dijit.byId("calExComment").destroy();
            }
            this._populateLabelMsg();
            
            
        },
		
		postCreate: function() {
			this.inherited(arguments);
			this._pconnects = [];
			this._pconnects.push(dojo.connect(dijit.byId("calExComment"), "onChange", this, this._onCommentChange));

			this._fromDateCal.src = this._toDateCal.src = dojo.moduleUrl("com.ibm.epm.admin.web" ,"ui/internal/view/images/calendar.gif");
		},
        
        /**
         * Loads the exception dialog
         * @param date - date for which exception is being loaded
         * @param calendar - calendar widget which calls this method
         */
        loadData: function(date, calendar){
            this.date = date;
            this.calendar = calendar;
            this.calendarURI = calendar.calendarURI;
            this.availableHrs = calendar.hrsPerDay;
            if (calendar.hrsBlocked[date.getDate()]) {
                this.availableHrs = this.availableHrs - calendar.hrsBlocked[date.getDate()];
            }
            this.repeat = false;
            dijit.byId("calExFromDate").attr("value", date);
            dijit.byId("calExToDate").attr("value", date);
            this.calExHrsBlocked.value = this.availableHrs;
            if (this.calendar.exceptions[date.getDate()]) {
                this.exception = this.calendar.exceptions[date.getDate()];
                this._loadException();
            }
        },
        
        /**
         * Loads an exception by setting values to be displayed
         */
        _loadException: function(){
            // Dates
            dijit.byId("calExFromDate").attr("value", DateUtil.parseDate(this.exception.fromDate));
            dijit.byId("calExToDate").attr("value", DateUtil.parseDate(this.exception.toDate));
            dijit.byId("calExComment").attr("value", this.exception.comment);
            this.calExHrsBlocked.value = this.exception.hoursBlocked;
            if (!this.exception.isInherited) {
                this.calExDelButton.disabled = false;
            }
            else {
                this.calExSaveButton.disabled = true;
            }
            
            // Repeat
            if (this.exception.exceptionType == Constants.WEEKLY || this.exception.exceptionType == Constants.YEARLY) {
                this.calExRepeatCheckBox.checked = true;
                this._clickedRepeated();
                this.calExRepeatType.value = this.exception.exceptionType;
                this._changedRepeatType();
            }
        },
        
        /**
         * Populate the data into calendar association dialog Labels
         */
        _populateLabelMsg: function(){
            this.dialogSummary = Messages.CALENDAR_EXCEPTION_DIALOG_SUMMARY;
            this.removeExceptionBtn = Messages.REMOVE_EXCEPTION_BTN;
            this.okButtonText = Messages.OK;
            this.cancelButtonText = Messages.CANCEL;
            this.from = Messages.FROM_LABEL + ": ";
            this.to = Messages.TO_LABEL + ": ";
            this.hoursBlocked = Messages.HOURS_BLOCKED_LABEL + ": ";
            this.repeatException = Messages.REPEAT_EXCEPTION_LABEL;
            this.comment = Messages.COMMENT_LABEL;
            this.repeatType = Messages.REPEAR_TYPE_LABEL;
            this.WEEKLY = Messages.WEEKLY;
            this.YEARLY = Messages.YEARLY;
        },
        /**
         * Called onclick of save button to create/update exception
         */
        _saveException: function(){
            // Check for mandatory comment - do we really want this??
            var comment = dijit.byId("calExComment").attr("value");
            if (comment == "") {
                alert(Messages.ENTER_A_COMMENT);
                return;
            }
			if (comment.length > Constants.MAX_CHARACTERS_IN_COMMENT) {
				alert(Messages.MAX_COMMENT_LENGTH_ERROR);
				return;
			}
            
            // Check if hrsBlocked <= (hrsPerDay - hrsAlreadyBlocked)
            var hrsBlocked = this.calExHrsBlocked.value;
            if (hrsBlocked > this.availableHrs) {
                alert(Messages.YOU_CAN_BLOCK_UPTO + " " + this.availableHrs + " " + Messages.HOURS);
            }
            
            var currentValues = {
                "fromDate": DateUtil.formatDateForSave(dijit.byId("calExFromDate").attr("value")) + " 00:00:00",
                "toDate": DateUtil.formatDateForSave(dijit.byId("calExToDate").attr("value")) + " 00:00:00",
                "hoursBlocked": hrsBlocked,
                "comment": comment,
                "calendar": this.calendarURI
            };
            if (!this.repeat) {
                currentValues.exceptionType = Constants.SIMPLE;
            }
            else {
                currentValues.exceptionType = this.calExRepeatType.value;
                if (currentValues.exceptionType == Constants.WEEKLY) {
                    currentValues.dayOfWeek = this.weekday[this.date.getDay()];
                }
                else {
                    currentValues.dayOfMonth = this.date.getDate();
                    currentValues.monthOfYear = this.date.getMonth() + 1;
                }
            }
            
            if (!this.exception) {
                this._createException(currentValues);
                return;
            }
            
            var params = this._isExceptionModified(currentValues);
            if (params == null) {
                this._closeException();
                return;
            }
            this._updateException(params, currentValues);
        },
        
        /**
         * Creates an exception
         */
        _createException: function(params){
            var self = this;
            var serviceObj = {
                success: function(data){
                    self.calendar.addException(this.date, data, params);
                    self._closeException();
                },
                failure: function(errorObj){
                    var errorMessage = "";
                    if (errorObj && errorObj.errorMessage) {
                        errorMessage = " : " + errorObj.errorMessage;
                    }
                    alert(Messages.FAILED_TO_SAVE_EXCEPTION + errorMessage);
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            CalendarClient.createURI(Constants.EXCEPTION_URI, srh, params);
            
        },
        
        /**
         * Updates an exception
         */
        _updateException: function(params, currentValues){
            var self = this;
            var serviceObj = {
                success: function(data){
                    self.calendar.updateException(this.date, self.exception.self.relativeHref, currentValues);
                    self._closeException();
                },
                failure: function(errorObj){
                    alert(Messages.FAILED_TO_SAVE_EXCEPTION);
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            CalendarClient.updateURI(this.exception.self.relativeHref, srh, params);
        },
        
        /**
         * Called onclick of remove button to delete an exception
         */
        _deleteException: function(){
            var self = this;
            var serviceObj = {
                success: function(){
                    self.calendar.removeException(self.date, self.exception.self.relativeHref);
                    self._closeException();
                },
                failure: function(errorObj){
                    alert(Messages.FAILED_TO_REMOVE_EXCEPTION);
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            CalendarClient.deleteURI(srh, this.exception.self.relativeHref);
        },
        
        /**
         * Checks if the loaded exception has been updated
         * @return : Parameters updated
         */
        _isExceptionModified: function(currentValues){
            var modifiedParams = {};
            if (currentValues.fromDate != this.exception.fromDate) {
                modifiedParams.fromDate = currentValues.fromDate;
            }
            if (currentValues.toDate != this.exception.toDate) {
                modifiedParams.toDate = currentValues.toDate;
            }
            if (currentValues.comment != this.exception.comment) {
                modifiedParams.comment = currentValues.comment;
            }
            if (currentValues.exceptionType != this.exception.exceptionType) {
                modifiedParams.exceptionType = currentValues.exceptionType;
            }
            if (currentValues.dayOfWeek && (currentValues.dayOfWeek != this.exception.dayOfWeek)) {
                modifiedParams.dayOfWeek = currentValues.dayOfWeek;
            }
            if (currentValues.dayOfMonth && (currentValues.dayOfMonth != this.exception.dayOfMonth)) {
                modifiedParams.dayOfMonth = currentValues.dayOfMonth;
            }
            if (currentValues.monthOfYear && (currentValues.monthOfYear != this.exception.monthOfYear)) {
                modifiedParams.monthOfYear = currentValues.monthOfYear;
            }
            return modifiedParams;
        },
        
        /**
         * Called onclick of repeat checkbox to display dropdown
         */
        _clickedRepeated: function(){
            if (!this.repeat) {
                this._changedRepeatType();
                this._repeat.style.display = "";
                this.repeat = true;
            }
            else {
                this._repeat.style.display = "none";
                this.repeat = false;
            }
        },
        
        /**
         * Called onclick of from calendar image
         */
        _clickedFromDateImage: function(){
            if (dijit.byId("calExToDate")) {
                dijit.byId("calExToDate")._close();
            }
            if (dijit.byId("calExFromDate")) {
                dijit.byId("calExFromDate").focus();
            }
            
        },
        
        /**
         * Called onclick of to calendar image
         */
        _clickedToDateImage: function(){
            if (dijit.byId("calExFromDate")) {
                dijit.byId("calExFromDate")._close();
            }
            if (dijit.byId("calExFromDate")) {
                dijit.byId("calExToDate").focus();
            }
        },
        
        /**
         * Called onchange of repeatType to update 'every ....' field
         */
        _changedRepeatType: function(){
            var repType = this.calExRepeatType.value;
            var val = "(" + Messages.EVERY + " ";
            if (repType == Constants.WEEKLY) {
                val += this.weekdayNames[this.date.getDay()];
            }
            if (repType == Constants.YEARLY) {
                val += this.monthNames[this.date.getMonth()] + " " + this.date.getDate();
            }
            val += ")";
            
            this.calExRepeatOn.innerHTML = "";
            var textNode = document.createTextNode(val);
            this.calExRepeatOn.appendChild(textNode);
        },
        
        /**
         * Destroys the widget
         */
        destroy: function(){
            dojo.forEach(this._pconnects, dojo.disconnect);
            this._pconnects = [];

            dijit.byId("calExFromDate")._close();
            dijit.byId("calExToDate")._close();
            dijit.byId("calExFromDate").destroy();
            dijit.byId("calExToDate").destroy();
            dijit.byId("calExComment").destroy();
            this.inherited(arguments);
        },
        
        _closeException: function(){
            if (this.calendar) {
                this.calendar.closeExceptionDialog();
            }
        },
		
		_onCommentChange: function(newValue) {
			if(newValue.length > Constants.MAX_CHARACTERS_IN_COMMENT){
				newValue = newValue.substring(0, Constants.MAX_CHARACTERS_IN_COMMENT);
            	dijit.byId("calExComment").attr("value", newValue);
			}
		}
		
    });
})();
