/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGrid");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGridView");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
	
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGrid = com.ibm.epm.admin.web.ui.internal.view.AdminGrid;
    var GridEvents = com.ibm.epm.web.core.grid.controller.PubSub;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGrid", AdminGrid, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/calendar/templates/CalendarGrid.html"),
        
        _grid: null,
        
        postCreate: function(){
			this._createToolbar();
            if (this._grid == null) {
                this._createCalendarGrid();
            }
            this._registerNavigation();
        },
		
		   
        /**
         * Creates calendar grid
         */
        _createCalendarGrid: function(){
            this.disableSaveButton(true);
            this.disableDeleteButton(true);
            
            this._grid = new com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGridView({
                _treeID: "calendars"
            });
			this._grid.setContainingPage(this);
            this._content.appendChild(this._grid.domNode);
            this._grid.buildCalendarGrid();
        },
        
        /**
         * Called onclick of add calendar button
         */
        _addCalendar: function(){
            if (this._grid) {
                this._grid._addCalendar();
            }
        },
        
        /**
         * Called onclick of delete calendar button
         */
        _deleteCalendar: function(){
            if (this._grid) {
                this._grid._deleteCalendar();
            }
        },
        
        /**
         * Called onclick of save calendar button
         */
        _saveCalendar: function(){
            if (this._grid) {
                this.disableSaveButton(true);
                this._grid._saveCalendar();
            }
        },
        
        /**
         * Called onclick of refresh calendars button 
         */
        _refreshCalendar: function(){
            if (!this._confirmRefresh())
                return;

            if (this._grid) {
                this._content.removeChild(this._grid.domNode);
                this._grid.destroy();
            }
            this._createCalendarGrid();
			this.loadHandle = dojo.subscribe(GridEvents.LOADING_COMPLETED, this, "_onCalendarGridLoaded");
        },
        
        /**
         * Creating the toolbar. In appadmin tree-grid views, tree & grid will not have separate toolbar.
         * It's a common toolbar will have Add, Delete, Save, Refresh icons.
         */
        _createToolbar: function(){
            this.addButton = new dijit.form.Button({
                title: AdminMessages.ADD_CALENDAR,
				label : AdminMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addCalendar)
            });
            
            this.deleteButton = new dijit.form.Button({
                title: AdminMessages.DELETE_CALENDAR,
				label : AdminMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteCalendar)
            });
            
            this.refreshButton = new dijit.form.Button({
                title: AdminMessages.REFRESH,
				label : AdminMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshCalendar)
            });
            
            this.saveButton = this.createSaveButton(this._saveCalendar);
            
            var rightContentList = [this.addButton, this.deleteButton, this.refreshButton, this.saveButton];
            
            this.calendarToolbar = new EPMToolbar({
                rightContentList: rightContentList,
				title:AdminMessages.CALENDARS_ADMIN_PAGE_TITLE
            });
           	
            this._toolbar.appendChild(this.calendarToolbar.domNode);
        },
        
        destroy: function(){
            if (this._grid && this._grid.destroy) {
                this._content.removeChild(this._grid.domNode);
                this._grid.destroy();
                delete this._grid;
            }
            this._unregisterNavigation();            
        },
		
		_onCalendarGridLoaded: function(publishedData){
			if (publishedData.controllerID == Constants.CALENDAR_CONTROLLER) {
				if (this._grid && this._grid._generalGrid) {
					//After the loading has completed
					var index = this._grid._generalGrid.selection.getFirstSelected();
					index = (index >=0) ? index : 0;
					this._grid.selectRowAndBuildCalendar(index);
				}
			}
			if(this.loadHandle) {
				dojo.unsubscribe(this.loadHandle);
			}
		}
    });
})();
