/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGridView");
    
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGrid");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGridModel");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.AdminUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridView");
    dojo.require("com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog");    

  	var Messages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");    
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
	var adminUtil = com.ibm.epm.admin.web.ui.internal.Util.AdminUtil;
    var AdminGridView = com.ibm.epm.admin.web.ui.internal.view.AdminGridView;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGridView", AdminGridView, {
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/calendar/templates/CalendarGridView.html"),
        
        newElementPrefix: "_unsaved",
        
        /* 
         * buildCalendarGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with CalendarGrid.
         */
        buildCalendarGrid: function(isCompressedView){
            this._isCompressedView = isCompressedView;
            
            // Set flag to be used during the first row creation, to trigger grid
            // height change.
            if (!this._isCompressedView)
                this.initializeResizeHandling = true;
        
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGridModel();
            }
            var calendarController = controllerUtil.getEPMGridController("CalendarController");
            
            
            this.buildGrid(this, this._gridModel, calendarController);
            
            // Selecting default grid row
            // Note : Due to the asynchronous nature, we need to wait for _generalGrid to be defined
            var self = this;
            var intervalID = setInterval(function(){
                if (self._generalGrid) {
                    self._generalGrid.setGridRowSelection(0);
                    clearInterval(intervalID);
                }
            }, 50);
            
            this._subscribeToEvents();
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.calendarGridAttachPoint;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
        	if(this._isCompressedView)
        	{
        		var gridLayout = [{
	                noscroll: false,
	                defaultCell: {
	                    width: 25,
	                    styles: 'text-align: left;'
	                },
	                rows: [[{
	                    name: Messages.NAME,
	                    width: "auto",
	                    field: Constants.COLUMN_NAME
                	}]]
        		}];
                return gridLayout;
        	}
            var gridLayout = [{
                noscroll: false,
                defaultCell: {
                    width: 8,
                    editor: dojox.grid.editors.Input,
                    styles: 'text-align: left;'
                },
                rows: [[{
                    name: Messages.NAME,
                    width: 14,
                    field: Constants.COLUMN_NAME
                }, {
                    name: Messages.HOURS_PER_DAY,
                    width: "auto",
                    field: Constants.COLUMN_HOURSPERDAY,
					styles: Constants.RIGHT_ALIGN_TEXT_STYLE,
                    editor: dojox.grid.editors.Input
                }]]
            }];

            return gridLayout;
        },
        
        /*
         * Catch the end of the cell edit and perform validation.  Will also
         * activate the save button if a valid change actually happened.
         */
        _onApplyCellEdit: function(e) {
            if (e && e.gridID == Constants.ADMIN_CALENDAR_GRID) {
                // By default, edit is good unless it fails validation
                var changed = true;
                
                /*
                 * Validate only the columns that require it (number values).
                 */
                var row = this._generalGrid.model.getRow(e.rowIndex);
                if (row) {
                    var item = this._generalGrid.model.store._getItemByIdentity(row.id);
                    if (item && this._curCellField == Constants.COLUMN_HOURSPERDAY) {
                        changed = this._validateChangeInNumberField(this._cellOldValue, e.value, item, row, e.rowIndex, this._curCellField, 1, 24);
                        if (changed)
                            changed = this._validateChangeInDecimalField(this._cellOldValue, e.value, item, row, e.rowIndex, this._curCellField, 0);
                    }
                }
            
                // Notify save button to enable only if the cell changed
                if (changed)
                    this.inherited('_onApplyCellEdit', arguments);
            }
        },

        /**
         *  _afterRowSelection : Overiding  _afterRowSelection function in abstract view a publishing event.
         *  Incoming Arguments : Grid row data.
         *  Outgoing Argument : none
         *
         */
        _afterRowSelection: function(data){
            if (data && data.gridID == Constants.ADMIN_CALENDAR_GRID && this._generalGrid) {
            	this.selectRowAndBuildCalendar(data.rowIndex);
            }
        },
		
		selectRowAndBuildCalendar: function(rowIndex) {
            var data1 = this._generalGrid.model.getRow(rowIndex);
            if (data1) {
				//Publish Grid row select event - saved and unsaved row
                dojo.publish(Constants.SELECT_CALENDAR_EVENT, [data1]);
            }
		},
        
        _saveCalendar: function(){
            this.persistGrid(true);
        },
        
        /*
         * Returns the Grid Title
         */
        getGridTitle: function(){
            return Messages.ADMIN_CALENDAR_GRID;
        },
        
		/*
         * Returns the Grid Internal ID
         */
        getGridInternalID: function(){
            return Constants.ADMIN_CALENDAR_GRID;
        },
        
        /*
		 * Handle the error 
		 */
        handleError: function(errorMessage){
            this.inherited("handleError", arguments);
            adminUtil.displayErrorMessage("errorMsgCalendar",errorMessage);
        },
		
		/**
		 * Adds a new calendar
		 */
        _addCalendar: function(){
            var rowID = this._controller.getUniqueRowID(Constants.UNSAVED);
            var row = {
                "relativeHref": rowID
            };
            this._controller._insertElement(row, null, false);
            this.scrollToSelectedRow();
            this.startFirstCellEdit();
        },
        
        /**
         * Deletes selected calendars
         */
        _deleteCalendar: function(){
            this.confirmDialog = new com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog({
               gridController: this._controller,
			   grid: this._generalGrid
            });
        },
        
		/**
		 * @return Retrieves the sortable columns for this grid view
		 */
		
		getSortableColums:function(){
			var sortableFields ={};
			sortableFields[Constants.COLUMN_NAME] = Constants.COLUMN_NAME;
			sortableFields[Constants.COLUMN_HOURSPERDAY] = Constants.COLUMN_HOURSPERDAY;
			return sortableFields;
		}
		
    });
})();
