/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesGridModel");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    // Constants
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    
    var abstractGridModel = com.ibm.epm.admin.web.ui.internal.view.AdminGridModel;
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesGridModel", abstractGridModel, {
        newElementPrefix: "_unsaved",
        constructor: function(){
        },
        
        /*
         * This is called by controller. Returns the empty row structure to the controller.
         */
        getEmptyRow: function(rowID, treeElementHandle){
            var row = {
                "id": rowID,
                "href": rowID,
                "parentId": null,
                "name": treeElementHandle.title,
                "resourcetype": treeElementHandle.resourcetype,
                "description": "",
                "code": "",
                "billable": true,
                "state": Constants.ACTIVE,
				"accountHead": false,
                "loadedelement": null
            };
            if (treeElementHandle.parentIdentifier) {
                if (dojo.isArray(treeElementHandle.parentIdentifier)) {
                    row.parentId = treeElementHandle.parentIdentifier[0];
                }
                else {
                    row.parentId = treeElementHandle.parentIdentifier;
                }
            }
            return row;
        },
        
        /*
         * This is called by controller.
         * After inserting a row in Grid, the getAndUpdateRow() calls updateRowData with RowURI, RowID params.
         */
        updateRowData: function(row, loadedElement, linkName){
            if (linkName == null) {
                row.name = loadedElement.name;
                row.description = loadedElement.description;
                row.code = loadedElement.code;
				row.billable = loadedElement.billable;
				if(loadedElement.state == Constants.ACTIVE_STATE){
					row.state = Constants.ACTIVE; 
				}
				else{
					row.state = Constants.INACTIVE;
				}				
				row.accountHead = loadedElement.accountHead;
                row.loadedelement = [loadedElement];
                
                this._updateParent(row, loadedElement);
            }
        },
        
        _updateParent: function(row, loadedElement){
            var parentLink = getLink(loadedElement, Constants.ParentChargeCode);
            if (parentLink && parentLink[0]) {
                row.parentId = parentLink[0].relativeHref;
            }
        },
        
		/**
		 * The prepareSaveParams method will be called from the gridcontroller
		 * passing the orginal row as well as modified row. Then it will
		 * send only modified values will be send to server. 
		 * @param {Object} row
		 */
        prepareSaveParams: function(row){
            var saveParams = {
                name: row.name,
                description: row.description,
                code: row.code				
            };
			
			if(row.state == Constants.ACTIVE){
				saveParams.state = Constants.ACTIVE_STATE; 
			}
			else{
				saveParams.state = Constants.INACTIVE_STATE;
			}
			if (row.billable) {
				saveParams.billable = "true";
			}
			else {
				saveParams.billable = "false";
			}
			
			if (row.accountHead) {
				saveParams.accountHead = "true";
			}
			else {
				saveParams.accountHead = "false";
			}
			
            if (row.parentId != null && row.parentId != "") {
				var parent = getLink(row.loadedelement[0], Constants.ParentChargeCode);
				if (parent != null) {
					if (parent.relativeHref != row.parentId) {
						saveParams.parent = row.parentId;
						saveParams.ParentChargeCode = row.parentId + ":" + Constants.ADD;
					}
				}
				else {
					saveParams.parent = row.parentId;
					saveParams.ParentChargeCode = row.parentId + ":" + Constants.ADD;
				}               
            }
            return saveParams;
        },
        
        _getNewElementURI: function(row){
            return "chargecodes";
        }
    });
})();

