/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesGridView");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesGridModel");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.AdminUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
    dojo.require("com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridView");    
	 
	var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
	var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;	
	var adminUtil = com.ibm.epm.admin.web.ui.internal.Util.AdminUtil;
	var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var AdminGridView = com.ibm.epm.admin.web.ui.internal.view.AdminGridView;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesGridView", AdminGridView, {
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/chargecodes/templates/ChargecodesGrid.html"),
        
        postCreate: function(){
        	this._codesubscriptions = [];
			this.eventSubscribeCC();
        },
        
        /* 
         * buildChargecodesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with ChargecodesGrid.
         */
        buildChargecodesGrid: function(){
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.admin.web.ui.internal.view.chargecodes.ChargecodesGridModel();
            }
            var chargecodesController = controllerUtil.getTreeBasedGridController("ChargecodesController");
			chargecodesController._buildModel = false;
            this.buildGrid(this, this._gridModel, chargecodesController);
            
            this._subscribeToEvents();
        },
		
		/**
		 * _renameNode: Event handler when the tree publishes the EPMTreePubSub.ON_RENAME_NODE event.<br>
		 * 
		 * @param {Object} publishObj - The published object contains the tree id, the node.item, old and new values of the name field.
		 */
		_renameNode: function(publishObj) {
			 if ((publishObj) && (publishObj.treeID == Constants.CHARGE_CODE_TREE_ID)) {
			 	var id = publishObj.item.identifier[0];
			 	var item = this._generalGrid.model.store._getItemByIdentity(id);
                if (item) {
					this._generalGrid.model.store._setValueOrValues(item, Constants.NAME_FIELD_NAME, publishObj.newValue, true);
				}
			 }
		},
        
        /*
         * getGridTitle funtion - Override the abstract class and returns the title of the grid.
         *
         */
        getGridTitle: function(){
            return  AdminMessages.CHARGECODE_GRID_TITLE;
        },
         /*
         * getGridInternalID funtion - Override the abstract class and returns the ID of the grid.
         *
         */
        getGridInternalID: function(){
            return  Constants.CHARGECODE_GRID_TITLE;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.chargecodesGridAttachPoint;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var gridLayout = [{
                noscroll: false,
                rows: [[{
                    name: AdminMessages.CODE_DESCRIPTION_LABEL,
                    width: "35%",
                    field: "description",
                    editor: dojox.grid.editors.Input
                }, {
                    name: AdminMessages.CODE_CODE_LABEL,
                    width: "25%",
                    field: "code",
                    editor: dojox.grid.editors.Input
                }, {
                    name: AdminMessages.CODE_BILLABLE_LABEL,
                    width: "10%",
                    field: Constants.CHARGECODE_FIELD_BILLABLE,
                    styles: 'text-align: center;',
                    editor: dojox.grid.editors.bool
                }, {
                    name: AdminMessages.CODE_STATE_LABEL,
                    width: "15%",
                    field: "state",
                    editor: dojox.grid.editors.select,
                    options: ["Active", "Inactive"]
                }, {
                    name: AdminMessages.CODE_CATEGORY_LABEL,
                    width: "15%",
                    field: Constants.CHARGECODE_FIELD_CATEGORY,
                    styles: 'text-align: center;',
                    editor: dojox.grid.editors.bool					
                }]]
            }];
            
            return gridLayout;
        },
        
        handleError: function(errorObject){
            this.inherited("handleError", arguments);
            EPMErrorHandler.setError(errorObject.errorMessage, errorObject.errorCode);
        },
        
        _validateController: function(data){
        
            if (!this._controller) {
				return false;
			}
            
            if (data) {
                var resourcetype = null;
                if (dojo.isArray(data.resourcetype)) {
                    resourcetype = data.resourcetype[0];
                }
                else {
                    resourcetype = data.resourcetype;
                }
                if (resourcetype == Constants.CHARGE_CODE_TYPE) {
                    return true;
                }
            }
            return false;
        },
        _onMouseOver: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onMouseOut: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onSelect: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onBlur: function(){
        
        },
        
        _saveChargecodes: function(){
            this.persistGrid(true);
        },
		
		/**
		 * The method _deleteChargecodes will be called from chargecodes.js when the delete button is clicked 
		 * The method will prompt for the confirmation for the delete and do accordingly
		 */
		_deleteChargecodes: function(){
			this.confirmDialog = new com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog({
               gridController: this._controller,
			   grid: this._generalGrid
            });
        },
		
		destroy: function(){
            this._unsubscribeToAllEventsCC();
			this.inherited(arguments);
        },
		
		_onCodeCellMouseOver: function(publishData)
		{
			if (publishData.gridID ==  Constants.CHARGECODE_GRID_TITLE) {
				if((publishData.cell) && 
				    (publishData.cell.editor) &&
					(publishData.cell.editor.declaredClass) &&
					(publishData.cell.editor.declaredClass == Constants.BOOL_EDITOR_CLASS)){
						publishData.grid.edit.apply();
					}
			}			
		},
		
		/*
		 * This method is invoked whenever there is mouseover the header Nodes 
		 	  of the charge  code grid, and setting two different tooltips for the 
		 	  category and billable column.
		 * Incoming : publishData contains grid id and the event object
		 */
		_onCodeHeaderCellMouseOver:function(publishData){
			
			if (publishData.gridID == Constants.CHARGECODE_GRID_TITLE) {
				 if(publishData.cell) {
				    if(publishData.cell.field==Constants.CHARGECODE_FIELD_CATEGORY) {
						publishData.cell.name=AdminMessages.CATEGORY_TOOLTIP;
						
					}
					else if(publishData.cell.field==Constants.CHARGECODE_FIELD_BILLABLE){
						publishData.cell.name=AdminMessages.BILLABLE_TOOLTIP;
						
					}
				}
					
			}
		},
		
		
		/**
         * eventSubscribeToAllEvents: Method to subscribe to all events. <br>
         */
        eventSubscribeCC: function(){
            this._codesubscriptions = [dojo.subscribe(EPMGridPubSub.ON_CELL_MOUSE_OVER, this, "_onCodeCellMouseOver"),
										// Tree event on rename of node
									   dojo.subscribe(EPMTreePubSub.ON_RENAME_NODE, this, "_renameNode"),
									   dojo.subscribe(EPMTreePubSub.ON_KEY_DOWN, this, "_onKeyDownFromTree"),
									   dojo.subscribe(EPMGridPubSub.ON_HEADER_CELL_MOUSE_OVER,this,"_onCodeHeaderCellMouseOver")];
        },
        
        /*
		 * Handle the error 
		 */
        handleError: function(errorMessage){
            adminUtil.displayErrorMessage("errorMsgChargecodes",errorMessage);
        },
		
        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEventsCC: function(){
            if (this._codesubscriptions != null) {
                for (var topic in this._codesubscriptions) {
                    var handler = this._codesubscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._codesubscriptions = [];
        },
		
		/**
		 * createView - method will call the createView of the abstractView
		 * And also set the grid to grow auto.
		 */
		createView: function(){
			this.inherited('createView', arguments);
			
			this._generalGrid.autoHeight = true;
			this._generalGrid.update();
		},
		
		/* _onKeyDownFromTree : On key down in the tree control comes here, focus is set to the corresponging row's first cell in grid */
		
		_onKeyDownFromTree: function(data) {
            if(data.nodeWidgetItem) {
                var rowIndex = this._validateAndGetRow(data.nodeWidgetItem);
                // Set the focus on the correct row
                if (this._generalGrid) {
                    if (rowIndex >= 0) {
                        var keys = dojo.keys;
                        // Check if the tab key was pressed
                        if (data.keySet.keyCode == keys.TAB) {
                            // Set the focus to the first cell on the selected row
                            this._generalGrid.selection.select(rowIndex);
                            this._generalGrid.focus.setFocusIndex(rowIndex, 0);
                            return;
                        }     
                    }        
                }
            }
        }
    });
})();

