/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGrid");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGridView");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");
    
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var AdminGrid = com.ibm.epm.admin.web.ui.internal.view.AdminGrid;
    
    var AdminViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGrid", AdminGrid, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/competencies/templates/CompetenciesGrid.html"),
        
        _grid: null,
        
        postCreate: function(){
        	this._createToolbar();
            
            if (this._grid == null) {
                this._createCompetencyGrid();
            }
            
            this._registerNavigation();
        },
        
		
        /**
         * Creates the Competency Grid
         */
        _createCompetencyGrid: function(){
            this.disableSaveButton(true);
            this.disableDeleteButton(true);
        
            this._grid = new com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGridView({
                _treeID: Constants.COMPETENCY_URI
            });
			this._grid.setContainingPage(this);
            this.competenciesGridAttachPoint.appendChild(this._grid.domNode);
            
            this._grid.buildCompetencyGrid();
        },
        
        /**
         * Adds new Competency
         */
        _addCompetency: function(){
            if (this._grid) {
                this._grid.addCompetency();
            }
        },
        
        /**
         * Saves the changes in the Competency Grid
         */
        _saveCompetency: function(){
            if (this._grid) {
                this.disableSaveButton(true);
                this._grid.saveCompetency();
            }
        },
        
        /**
         * Refreshes the Competency Grid
         */
        _refreshCompetency: function(){
            if (!this._confirmRefresh())
                return;

            //Delete the grid and prepare again
            if (this._grid) {
                this.competenciesGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
            }
            
            this._createCompetencyGrid();
        },
        
        
        /**
         * Deletes the selected competencies
         */
        _deleteCompetency: function(){
        
            if (this._grid) {
                this._grid.deleteCompetencies();
                
            }
        },
        
        /**
         * Creates the Toolbar
         */
        _createToolbar: function(){
            this.addButton = new dijit.form.Button({
                title: AdminViewletMessages.ADD_COMPETENCY,
				label : AdminViewletMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addCompetency)
            });
            
            this.deleteButton = new dijit.form.Button({
                title: AdminViewletMessages.DELETE_COMPETENCY,
				label : AdminViewletMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteCompetency)
            });

            this.refreshButton = new dijit.form.Button({
                title: AdminViewletMessages.REFRESH,
				label : AdminViewletMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshCompetency)
            });
            
            this.saveButton = this.createSaveButton(this._saveCompetency);

            var rightContentList = [this.addButton, this.deleteButton, this.refreshButton, this.saveButton];
            
            this._competenciesGridToolBar = new EPMToolbar({
                rightContentList: rightContentList,
				title:AdminViewletMessages.COMPETENCIES_ADMIN_PAGE_TITLE
            });
            	
            this._toolbar.appendChild(this._competenciesGridToolBar.domNode);
        },
        
        destroy: function(){
            if (this._grid && this._grid.destroy) {
                this.competenciesGridAttachPoint.removeChild(this._grid.domNode);
                this._grid.destroy();
                delete this._grid;
            }
            this._unregisterNavigation();            
        }
    });
})();
