/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/**
 * @author suryakant
 */
(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGridView");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.AdminUtil");
    dojo.require("com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridView");    
    
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
	var adminUtil = com.ibm.epm.admin.web.ui.internal.Util.AdminUtil;
    var AdminGridView = com.ibm.epm.admin.web.ui.internal.view.AdminGridView;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGridView", AdminGridView, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/competencies/templates/CompetenciesGridView.html"),
        
        CompetencyGridController: null,
        
        /* 
         * Builds the Competency Grid
         */
        buildCompetencyGrid: function(){
            // Set flag to be used during the first row creation, to trigger grid
            // height change.
            this.initializeResizeHandling = true;
                
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.admin.web.ui.internal.view.competencies.CompetenciesGridModel();
            }
            this.CompetencyGridController = controllerUtil.getEPMGridController(Constants.COMPETENCY_CONTROLLER);
            this.buildGrid(this, this._gridModel, this.CompetencyGridController);
            
            this._subscribeToEvents();
        },
        
        /*
         * OnBlur funtion - Overriding this funtion to avoid deselection of Rows
         *
         */
        _onBlur: function(data){
        
            //DO nothing
            
            return;
            
        },
        
        /*
         * getGridTitle funtion - Override the abstract class and returns the title of the grid.
         *
         */
        getGridTitle: function(){
            return AdminMessages.COMPETENCY_GRID_TITLE;
        },
        
         /*
         * getGridInternalID funtion - Override the abstract class and returns the ID of the grid.
         *
         */
        getGridInternalID: function(){
            return Constants.COMPETENCY_GRID_TITLE;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.CompetenciesGridViewAttachPoint;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
        
        
            var CompetencyGridLayout = [{
                noscroll: false,
                defaultCell: {
                    width: 8,
                    editor: dojox.grid.editors.Input,
                    styles: 'text-align: left;'
                },
                rows: [[{
                    name: AdminMessages.NAME,
                    width: 20,
                    field: Constants.COLUMN_NAME
                }, {
                    name: AdminMessages.DESCRIPTION,
                    width: "auto",
                    field: Constants.COLUMN_DESCRIPTION                    
                }]]
            
            }];
            
            return CompetencyGridLayout;
        },
        
        
        
        /*
         * Adds a new Competency to the Grid
         * Incoming : None
         * Outgoing : None
         */
        addCompetency: function(){
            var rowID = this._controller.getUniqueRowID(Constants.UNSAVED);
            var row = {
                "relativeHref": rowID
            };

            this._controller._insertElement(row, null, false);
            this.scrollToSelectedRow();
            this.startFirstCellEdit();
        },
        
        /*
         * Saves the Competencies Grid Infomation
         * Incoming : None
         * Outgoing : None
         */
        saveCompetency: function(){
        
            this.persistGrid(true);
        },
        
        /*
         * Deletes the Competency
         */
        deleteCompetencies: function(){
		    this.confirmDialog = new com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog({
		           gridController: this._controller,
				   grid: this._generalGrid
		        });
        },
		
        /*
		 * Handle the error 
		 */
        handleError: function(errorMessage){
            this.inherited("handleError", arguments);
            adminUtil.displayErrorMessage("errorMsgCompetencies",errorMessage);
        },
		
		/*
         * Destroys the  Grid Widget
         * Incoming : None
         * Outgoing : None
         */
        _destroyResourceWidget: function(){
        
            this.destroy();
            
        },

		/**
		 * @return Retrieves the sortable columns for this grid view
		 */
		
		getSortableColums:function(){
			var sortableFields ={};
			sortableFields[Constants.COLUMN_NAME] = Constants.COLUMN_NAME;
			sortableFields[Constants.COLUMN_DESCRIPTION] = Constants.COLUMN_DESCRIPTION;
			return sortableFields;
		}
		
    });
})();
