/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

 (function() {
dojo.provide("com.ibm.epm.web.core.ui.internal.view.queryeditor.widgets.ListWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.epm.web.core.client.internal.QueryAttributeClient");
dojo.require("com.ibm.epm.web.core.transport.EPMServerClient");
dojo.require("com.ibm.epm.web.core.transport.ServiceRequest");

var ServiceRequest = com.ibm.epm.web.core.transport.ServiceRequest;
var EpmServerClient = com.ibm.epm.web.core.transport.EPMServerClient;
 


var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ImageClient= com.ibm.team.workitem.web.client.internal.ImageClient;
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var QueryAttributeClient= com.ibm.epm.web.core.client.internal.QueryAttributeClient;

/*
 * List Widget 
 */
dojo.declare("com.ibm.epm.web.core.ui.internal.view.queryeditor.widgets.ListWidget",  [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.epm.web.core", "ui/internal/view/queryeditor/widgets/templates/ListWidget.html"),
	
    _SKILL_URI: "epm/resources/skills",   
	_COMPETENCY_URI: "epm/resources/competencies",   
	_CALENDAR_URI: "epm/resources/calendars",   
	_PROFICIENCY_URI: "epm/resources/proficiencies",   
	_RATE_URI: "epm/resources/rates",   
	
   	_SKILL_NAME_ID_FOR_USER: "link:com.ibm.epm.resource.skillassignment:target/skill/name",
   	_SKILL_NAME_ID_FOR_PROFILE: "skillassignment/skill/name",   
	_COMPETENCY_NAME_ID_FOR_USER: "link:com.ibm.epm.resource.competencyassignment:target/competency/name",
	_COMPETENCY_NAME_ID_FOR_PROFILE: "competencyassignment/competency/name",   
	_CALENDAR_NAME_ID: "link:com.ibm.epm.resource.epmuser.calendarLink:target/name",   
	_PROFICIENCY_SKILL_ID_FOR_USER: "link:com.ibm.epm.resource.skillassignment:target/proficiency/name",
	_PROFICIENCY_SKILL_ID_FOR_PROFILE: "skillassignment/proficiency/name",   
	_PROFICIENCY_COMPETENCY_ID_FOR_USER: "link:com.ibm.epm.resource.competencyassignment:target/proficiency/name",
	_PROFICIENCY_COMPETENCY_ID_FOR_PROFILE: "competencyassignment/proficiency/name",   
	_RATE_ID_FOR_USER : "link:com.ibm.epm.resource.epmuser.rateLink:target/name",   
	_RATE_ID_FOR_PROFILE : "link:com.ibm.epm.resource.profile.rateLink:target/name",   
	
 
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		//alert("constructor from WorkItem");
		this.presentation= args.presentation;
		this._selection= args.selection || [];
		
		this._idLookup= new Object();
		this._selectMap= new Object();
		
		this._hasSelection= false;
	},
		
	postCreate: function() {
		this._select.size= 6;
		this._select.multiple= "multiple";

		var attributeId= this.presentation.attribute.id;

		var srh= new ServiceResponseHandler(this, "_initValues", "_handleError");			
		
		if(attributeId == this._SKILL_NAME_ID_FOR_USER || attributeId == this._SKILL_NAME_ID_FOR_PROFILE){
			this.getAllSkills(srh);
		}
		
		if(attributeId == this._CALENDAR_NAME_ID){
			this.getAllCalendars(srh);
		}

		if(attributeId == this._COMPETENCY_NAME_ID_FOR_USER || attributeId == this._COMPETENCY_NAME_ID_FOR_PROFILE){
			this.getAllCompetencies(srh);
		}
		
		if(attributeId == this._PROFICIENCY_SKILL_ID_FOR_USER ||
			attributeId == this._PROFICIENCY_COMPETENCY_ID_FOR_USER ||
			attributeId == this._PROFICIENCY_SKILL_ID_FOR_PROFILE ||
			attributeId == this._PROFICIENCY_COMPETENCY_ID_FOR_PROFILE){
			this.getAllProficiencies(srh);
		}
		
		if(attributeId == this._RATE_ID_FOR_USER || 
			attributeId == this._RATE_ID_FOR_PROFILE){
			this.getAllRates(srh);
		}
	},

	getAllSkills: function(responseHandler){
            var serviceRequest = new ServiceRequest(this._SKILL_URI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
    },
    
    getAllCompetencies: function(responseHandler){
            var serviceRequest = new ServiceRequest(this._COMPETENCY_URI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
    },
    
	getAllCalendars: function(responseHandler){
            var serviceRequest = new ServiceRequest(this._CALENDAR_URI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
    },	
    
    getAllProficiencies: function(responseHandler){
            var serviceRequest = new ServiceRequest(this._PROFICIENCY_URI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
    },	
    
    getAllRates: function(responseHandler){
            var serviceRequest = new ServiceRequest(this._RATE_URI, "get");
            EpmServerClient.operateOnResource(serviceRequest, responseHandler);
    },
        
	/* String[] */ getValues: function() {
		
		if (!this._hasSelection)
			return this._getInitialSelectionValues();
		
		var selection= new Array();
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].selected) {
				selection.push(select[i].text);
			}
		}
		return selection;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_getInitialSelectionValues: function() {
		var values= this._selection;
		var selection= new Array();
		for (var i= 0; i < values.length; i++) {
			selection.push(values[i].title);
		}
		return selection;
	},
	
	_onSelectionChange: function() {
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_initValues: function(/* UIItemSetDTO */ valueSet) {
		if (!valueSet || valueSet.length == 0)
			return;
		this._select.length= 0;
		for (var i= 0; i < valueSet.length; i++) {
			if(valueSet[i].title && !valueSet[i].label) {
				valueSet[i].label = valueSet[i].title;
			}	
		}
		
		this._addValues(valueSet);
		this._selectValues(this._selection);
	},
	
	_handleError: function(e) {
		if (this.errorCallback)
			this.errorCallback("Loading values failed", e);
	},
	
	_addValues: function(/* UIItemDTO[] */ values, /* boolean */ select) {
		if (!values || values.length == 0)
			return;
		var index= this._select.length;
		for (var i= 0; i < values.length; i++) {
			var option= this._createOption(values[i]);
			this._select[index++]= option;
			this._selectMap[values[i].label]= option;
			option.selected= select ? true : false;
		}
	},
	
	_selectValues: function(/* UIItemDTO[] */ selection) {
		this._hasSelection= true;
		var missing= new Array();
		for (var i= 0; i < selection.length; i++) {
			var item= selection[i];
			var option= this._selectMap[item.label];
			if (option) {
				option.selected= true;
			} else {
				if (item.arguments) // hack for category
					item.label= item.arguments;
				missing.push(item);
			}
		}
		this._addValues(missing, true);
	},
	
	_createOption: function(/* UIItemDTO */ item) {
		var option= new Option(item.title, item.idref, false, false);
		var indent= this._getIndent(item);
		if (indent > 0) {
			option.setAttribute("style", "padding-left: " + indent * 5 + "px");
		}
		if (item.iconUrl) {
			var url= ImageClient.getImageUrl(item.iconUrl);
			option.setAttribute("style", "background-image: url('" + url + "'); padding-left: 20px");
		}
		return option;
	},
	
	_getIndent: function(/* UIItemDTO */ item) {
		var label= item.label;
		var indent= 0;
		while (indent < label.length && label[indent] == ' ') 
			indent++;
		return indent;
	}
	
});

})();

