/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGridModel");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridModel");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    
    // Constants
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    
    //Localization
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.SystemProfileUtil");
    
    var SystemProfileUtil = com.ibm.epm.admin.web.ui.internal.Util.SystemProfileUtil;
    
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    
    var AdminViewletMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
    var abstractGridModel = com.ibm.epm.admin.web.ui.internal.view.AdminGridModel;
    
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGridModel", abstractGridModel, {
    
        newElementPrefix: Constants.UNSAVED,
        
        /*
         * This is called by controller. Returns the empty row structure to the controller.
         */
        getEmptyRow: function(rowID, collectionItemHandle){
            //If the Flag is undefined, the data is pulled from the server and controlled by the 
            //Abstract controller
            //else the data is inserted directly by the View
            
            var row;
            
            if (rowID.indexOf(this.newElementPrefix) == -1) {
            
                row = {
                    "id": rowID,
                    "href": rowID,
                    "name": " ",
                    "experience": "0",
                    "skillassignment": collectionItemHandle ? collectionItemHandle.skillassignment : [],
                    "competencyassignment": collectionItemHandle ? collectionItemHandle.competencyassignment : [],
                    "description": "",
                    "SkillValue": "", //This Field is used to display skill & proficiency values in the format SKILL(PROFICIENCY)
                    "CompetencyValue": "", //This Field is used to display competency & proficiency values in the format COMPETENCY(PROFICIENCY)
                    "profileType": Constants.SYSTEM_PROFILE,
                    "rate": "",
                    "rateURI": "",
					"skillURI":"",
					"competencyURI":""
                };
            }
            else {
            
                row = {
                    "id": rowID,
                    "href": rowID,
					"relativeHref":collectionItemHandle ? collectionItemHandle.relativeHref : rowID, 
                    "name": AdminViewletMessages.NEW_PROFILE,
                    "experience": "0",
                    "skillassignment": collectionItemHandle ? collectionItemHandle.skillassignment : [],
                    "competencyassignment": collectionItemHandle ? collectionItemHandle.competencyassignment : [],
                    "description": AdminViewletMessages.PROFILE_DESCRIPTION,
                    "SkillValue": collectionItemHandle ? collectionItemHandle.SkillValue : "", 
					"CompetencyValue": collectionItemHandle ? collectionItemHandle.CompetencyValue : "",
                    "profileType": Constants.SYSTEM_PROFILE,
                    "rate": "",
                    "rateURI": "",
					"skillURI":"",
					"competencyURI":""
                };
            }
            
            return row;
        },
        
        /*
         * This is called by controller.
         * After inserting a row in Grid, the getAndUpdateRow() calls updateRowData with RowURI, RowID params.
         */
        updateRowData: function(row, loadedElement, linkName){
            if (linkName == null) {
                row.name = loadedElement.name;
                row.experience = loadedElement.experienceInYears;
                row.description = loadedElement.description;
                row.loadedelement = [loadedElement];
                row.CompetencyValue = "";
                row.SkillValue = "";
				row.skillassignment = loadedElement.skillassignment ? loadedElement.skillassignment : [];
				row.competencyassignment = loadedElement.competencyassignment ? loadedElement.competencyassignment : [];
				row.skillparams=[];
				row.competencyparams=[];
                if (loadedElement.skillassignment && loadedElement.skillassignment.length > 0) {
					for (var i = 0; i < loadedElement.skillassignment.length; i++) {
						if (row.SkillValue.length > 0) {
								row.SkillValue = row.SkillValue + " , ";
						}
						var skill = loadedElement.skillassignment[i].skill ? loadedElement.skillassignment[i].skill.title : loadedElement.skillassignment[i].name;
						var proficiency = loadedElement.skillassignment[i].proficiency ? loadedElement.skillassignment[i].proficiency.title : loadedElement.skillassignment[i].proficiencyLevel;
						row.SkillValue = row.SkillValue + skill + "(" + proficiency + ")";
					}
				}
				
				if (loadedElement.competencyassignment && loadedElement.competencyassignment.length > 0) {
					for (var i = 0; i < loadedElement.competencyassignment.length; i++) {
						if (row.CompetencyValue.length > 0) {
							row.CompetencyValue = row.CompetencyValue + " , ";
						}
						var competency = loadedElement.competencyassignment[i].competency ? loadedElement.competencyassignment[i].competency.title : loadedElement.competencyassignment[i].name;
						var proficiency = loadedElement.competencyassignment[i].proficiency ? loadedElement.competencyassignment[i].proficiency.title : loadedElement.competencyassignment[i].proficiencyLevel
						row.CompetencyValue = row.CompetencyValue + competency + "(" + proficiency + ")";
					}
				}
				
                var rate = loadedElement[Constants.RATE];
				if (rate) {
					row.rate = rate.title;
					row.rateURI = rate.relativeHref;
				}
                
            }
        },
		
        /*
         * Prepares the parameter for the Post call
         * Incoming : Row infomation
         * Outgoing : Save Parameters
         */
        prepareSaveParams: function(row){
            
            if (row.experience == null || isNaN(row.experience)) {
                row.experience = "0";
            }
            
            var saveParams = {
                name: row.name,
                description: row.description,
                experienceInYears: row.experience
            };
            if(row.competencyparams && row.competencyparams.length > 0)
			{
				saveParams.competencyassignment = row.competencyparams; 
			}
			if(row.skillparams && row.skillparams.length > 0)
			{
				saveParams.skillassignment = row.skillparams; 
			}
			if (row.loadedelement) 
			{
				if (row.loadedelement[0].Rate) 
				{
					if (row.rateURI == "") 
					{
						saveParams.Rate = row.loadedelement[0].Rate.relativeHref + ":del";
					}
					else 
						if (row.loadedelement[0].Rate.relativeHref != row.rateURI) 
						{
							var rateArray = [];
							rateArray.push(row.loadedelement[0].Rate.relativeHref + ":del");
							rateArray.push(row.rateURI + ":add");
							saveParams.Rate = rateArray;
						}
				}
				else 
				{
					if (row.rateURI != "") 
					{
						saveParams.Rate = row.rateURI + ":add";
					}
				}
			}
			else
			{
				if (row.rateURI != "") 
					{
						saveParams.Rate = row.rateURI + ":add";
					}
			}
			
            return saveParams;
        },
        
        /*
         * Get Post call URI method
         * Incoming : Row Info
         * Outgoing : Post call URI
         */
        _getNewElementURI: function(row){
        
            return Constants.PROFILE_URI;
        },
        
        /*
         * URI for Row level Info
         * Incoming : RowHandle, Row Id
         * Outgoing : URI to get row information
         */
        getRowGetURI: function(rowElementHandle, rowID){
            return rowElementHandle.relativeHref;
        }
        
    });
})();
