/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGridView");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGridModel");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.AdminUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.systemprofile.RateEditor");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SkillProficiencyEditor");
	dojo.require("com.ibm.epm.admin.web.ui.internal.view.systemprofile.CompetencyProficiencyEditor");
    dojo.require("com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridView");    
    dojo.require("com.ibm.epm.web.core.util.internal.GridColumnUtil");
	
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var adminUtil = com.ibm.epm.admin.web.ui.internal.Util.AdminUtil;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGridView = com.ibm.epm.admin.web.ui.internal.view.AdminGridView;
    var GridColumnUtil = com.ibm.epm.web.core.util.internal.GridColumnUtil;
	    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGridView", AdminGridView, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/systemprofile/templates/SystemProfileGridView.html"),
        
        //Controller
        SystemProfileGridController: null,
        
        
        /* 
         * Builds the Profile Grid
         */
        buildSystemProfileGrid: function(){
            // Set flag to be used during the first row creation, to trigger grid
            // height change.
            this.initializeResizeHandling = true;
                
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.admin.web.ui.internal.view.systemprofile.SystemProfileGridModel();
            }
            this.systemProfileGridController = controllerUtil.getEPMGridController(Constants.SYSTEMPROFILE_CONTROLLER);
            this.buildGrid(this, this._gridModel, this.systemProfileGridController);
            
            this._subscribeToEvents();
        },
        
        
        /*
         * OnBlur funtion - Overriding this funtion to avoid deselection of Rows
         *
         */
        _onBlur: function(data){
        
            //DO nothing
            
            return;
            
        },
        
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.SystemProfileGridViewAttachPoint;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
        
        
            var systemProfileGridLayout = [{
                noscroll: false,
                defaultCell: {
                    width: 8,
                    editor: dojox.grid.editors.Input,
                    styles: 'text-align: left;'
                },
                rows: [[{
                    name: AdminMessages.NAME,
                    width: 20,
                    field: Constants.COLUMN_NAME
                }, {
                    name: AdminMessages.DESCRIPTION,
                    width: 15,
                    field: Constants.COLUMN_DESCRIPTION
                }, {
                    name: AdminMessages.EXPERIENCE,
                    width: 6,
                    field: Constants.COLUMN_EXPERIENCE,
					styles: Constants.RIGHT_ALIGN_TEXT_STYLE,
                    editor: dojox.grid.editors.Input,
                    formatter: GridColumnUtil.formatYears
                }, {
                    name: AdminMessages.SKILLS,
                    width: 10,
                    field: Constants.COLUMN_SKILL,
                    editor: com.ibm.epm.admin.web.ui.internal.view.systemprofile.SkillProficiencyEditor
                }, {
                    name: AdminMessages.COMPETENCIES,
                    width: 10,
                    field: Constants.COLUMN_COMPETENCY,
                    editor: com.ibm.epm.admin.web.ui.internal.view.systemprofile.CompetencyProficiencyEditor
                
                }, {
                    name: AdminMessages.RATE,
                    width: 10,
                    field: Constants.COLUMN_RATE,
                    editor: com.ibm.epm.admin.web.ui.internal.view.systemprofile.RateEditor
                }]]
            }];
            
            return systemProfileGridLayout;
        },
        
        /*
         * Catch the end of the cell edit and perform validation.  Will also
         * activate the save button if a valid change actually happened.
         */
        _onApplyCellEdit: function(e) {
            if (e && e.gridID == Constants.SYSTEM_PROFILE_GRID_TITLE) {
                // By default, edit is good unless it fails validation
                var changed = true;
                
                /*
                 * Validate only the columns that require it (number values).
                 */
                var row = this._generalGrid.model.getRow(e.rowIndex);
                if (row) {
                    var item = this._generalGrid.model.store._getItemByIdentity(row.id);
                    if (item && this._curCellField == Constants.COLUMN_EXPERIENCE) {
                        changed = this._validateChangeInNumberField(this._cellOldValue, e.value, item, row, e.rowIndex, this._curCellField);
                        if (changed)
                            changed = this._validateChangeInDecimalField(this._cellOldValue, e.value, item, row, e.rowIndex, this._curCellField, 0);
                    }
                }
            
                // Notify save button to enable only if the cell changed
                if (changed)
                    this.inherited('_onApplyCellEdit', arguments);
            }
        },

        getGridTitle: function(){
            return AdminMessages.SYSTEM_PROFILE_GRID_TITLE;
        },
        
        /*
         * getGridInternalID function - Override the abstract class and returns the ID of the grid.
         *
         */
        getGridInternalID: function(){
            return Constants.SYSTEM_PROFILE_GRID_TITLE;
        },
        
        /*
         * Profiles may have a horizontal scroll bar for the grid, so return true
         * in order to make the automatic grid height resizing work properly.
         */
        hasGridHScroll: function() {
            return true;
        },
        
        /*
         * Adds a new Profile to the Grid
         * Incoming : None
         * Outgoing : None
         */
        addProfile: function(){
            var rowID = this._controller.getUniqueRowID(Constants.UNSAVED);
            var row = {
                "relativeHref": rowID
            };
            
            this._controller._insertElement(row, null, false);
			this.scrollToSelectedRow();
            this.startFirstCellEdit();
        },
        
       /**
		* saveProfile is called while saving profile.
		* Incoming : None
		* Outgoing : None
		*/ 
		saveProfile: function(){
            this.persistGrid(true);
        },
        
        /*
         * Deletes the Profile
         */
        deleteProfiles: function(){
            this.confirmDialog = new com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog({
               gridController: this._controller,
			   grid: this._generalGrid
            });
        },
        
        /*
         * Handle the error
         */
        handleError: function(errorMessage){
            this.inherited("handleError", arguments);
            adminUtil.displayErrorMessage("errorMsgSystemProfile", errorMessage);
        },
        
        /*
         * Destroys the  Grid Widget
         * Incoming : None
         * Outgoing : None
         */
        _destroyResourceWidget: function(){
        
            this.destroy();
            
        },

		/**
		 * @return Retrieves the sortable columns for this grid view
		 */
		
		getSortableColums:function(){
			var sortableFields ={};
			sortableFields[Constants.COLUMN_NAME] = Constants.COLUMN_NAME;
			sortableFields[Constants.COLUMN_DESCRIPTION] = Constants.COLUMN_DESCRIPTION;
			sortableFields[Constants.COLUMN_EXPERIENCE] = Constants.COLUMN_EXPERIENCE;
			sortableFields[Constants.COLUMN_SKILL] = Constants.COLUMN_SKILL;
			sortableFields[Constants.COLUMN_COMPETENCY] = Constants.COLUMN_COMPETENCY;
			sortableFields[Constants.COLUMN_RATE] = Constants.COLUMN_RATE;
			return sortableFields;
		},

		/**
		 * createView - method will call the createView of the abstractView
		 * And also set the grid to grow auto.
		 */
		createView: function(){
            this.inherited('createView', arguments);
            this._generalGrid.isCellEditable = dojo.hitch(this, this.isCellEditable);
		},

        /**
         * Overriding method isCellEditable to stop editing of various fields
         */
        isCellEditable: function(event){
            var rowIndex = parseInt(event.rowIndex, 10);
            if (isNaN(rowIndex)) {
                return true;
            }
			//No inline editing of labor skill, competency and rate
			if (event.cell.field == Constants.COLUMN_SKILL || 
				event.cell.field == Constants.COLUMN_COMPETENCY || 
				event.cell.field == Constants.COLUMN_RATE) {
                return false;
			}
			return true;
		}		
        
    });
})();
