/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.timecodes.Timecodes");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridView");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesTreeWidget");
    dojo.require("dijit.form.Button");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGrid");
    
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var AdminGrid = com.ibm.epm.admin.web.ui.internal.view.AdminGrid;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.timecodes.Timecodes", AdminGrid, {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/timecodes/templates/Timecodes.html"),
        _TimecodesTree: null,
        _TimecodesGrid: null,
        
        postCreate: function(){
			this._createToolbar();
            this._refreshTimecodes();
        },
        
			
        /**
         * Creating the toolbar. In appadmin tree-grid views, tree & grid will not have separate toolbar.
         * It's a common toolbar will have Add, Delete, Save, Refresh icons.
         */
        _createToolbar: function(){
            this.addButton = new dijit.form.Button({
                title: AdminMessages.ADD_TIMECODE,
				label : AdminMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addTimecode)
            });
            
			this.deleteButton = new dijit.form.Button({
                title: AdminMessages.DELETE_TIMECODE,
				label : AdminMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._deleteTimecodes)
            });
			
            this.refreshButton = new dijit.form.Button({
                title: AdminMessages.REFRESH,
				label : AdminMessages.REFRESH,
				showLabel : false,
                iconClass: IconClassConstants.EPM_REFRESH_ICON_CLASS,
                onClick: dojo.hitch(this, this._refreshTimecodes)
            });
            
            this.saveButton = this.createSaveButton(this._saveTimecodes);
            
            var rightContentList = [this.addButton, this.deleteButton, this.refreshButton, this.saveButton];
            
            this._timecodesGridToolBar = new EPMToolbar({
                rightContentList: rightContentList,
				title:AdminMessages.TIMECODES_ADMIN_PAGE_TITLE
            });
           	
            this._toolbar.appendChild(this._timecodesGridToolBar.domNode);
        },
        
		_refreshTimecodes: function(){
            if (!this._confirmRefresh())
                return;

            this.destroy();
            this._createWidgets();
        },
        
		/**
		 * The method _deleteTimecodes will be called whenever the delete button is clicked
		 */        
		_deleteTimecodes: function(){
			if (this._TimecodesGrid) {
                this._TimecodesGrid._deleteTimecodes();
            }
		},
		
		/**
		 * _addTimecode - method will be called when
		 * the add button is clicked.
		 */		
        _addTimecode: function(){
        	if ((this._TimecodesGrid) &&
				(this._TimecodesGrid._generalGrid)){
					this._TimecodesGrid._generalGrid.edit.apply();
			}
            if (this._TimecodesTree) {
                this._TimecodesTree._createRootTimecode();
            }
        },
        
        _saveTimecodes: function(){
            if (this._TimecodesGrid) {
                this.disableSaveButton(true);
                this._TimecodesGrid._saveTimecodes();
            }
        },
        
        destroy: function(){
        
            if (this._TimecodesTree) {
                this._TimecodesTree.destroy();
                this._treeContent.removeChild(this._TimecodesTree.domNode);
                delete this._TimecodesTree;
            }
            if (this._TimecodesGrid) {
                this._TimecodesGrid.destroy();
                this._gridContent.removeChild(this._TimecodesGrid.domNode);
                delete this._TimecodesGrid;
            }
            this._unregisterNavigation();
        },
        
        _createWidgets: function(){
            this.disableSaveButton(true);
            this.disableDeleteButton(true);
            
            if (this._TimecodesTree == null) {
                this._TimecodesTree = new com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesTreeWidget();
			    this._TimecodesTree.setContainingPage(this);
                this._treeContent.appendChild(this._TimecodesTree.domNode);
            }
            this._TimecodesTree._buildTimecodesTree();
            
            if (this._TimecodesGrid == null) {
                this._TimecodesGrid = this._grid = new com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridView({
                    _treeID: "TimecodesTree"
                });
			    this._TimecodesGrid.setContainingPage(this);
                this._gridContent.appendChild(this._TimecodesGrid.domNode);
            }
            this._TimecodesGrid.buildTimecodesGrid();
            this._registerNavigation();
        }
    });
})();
