/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridView");
    
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridModel");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMErrorHandler");
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.AdminUtil");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
	dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
    dojo.require("com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.AdminGridView");    
	 
	var AdminMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");    
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var controllerUtil = com.ibm.epm.web.core.grid.controller.ControllerUtil;
    var EPMErrorHandler = com.ibm.epm.web.core.ui.internal.EPMErrorHandler;
	var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;	
	var adminUtil = com.ibm.epm.admin.web.ui.internal.Util.AdminUtil;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var AdminGridView = com.ibm.epm.admin.web.ui.internal.view.AdminGridView;
	
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridView", AdminGridView, {
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/timecodes/templates/TimecodesGrid.html"),
        
       /**
        * postCreate function will called after the
        * constructor get called
        */
        postCreate: function(){
            this._codesubscriptions = [];
			this.eventSubscribeTC();
        },
        
        /* 
         * buildTimecodesGrid calls the buildGrid() in abstractView.
         * buildGrid() creates the controller, initializes and the dataModel mapping will be done with TimecodesGrid.
         */
        buildTimecodesGrid: function(){
            if (!this._gridModel) {
                this._gridModel = new com.ibm.epm.admin.web.ui.internal.view.timecodes.TimecodesGridModel();
            }
            var timecodesController = controllerUtil.getTreeBasedGridController("TimecodesController");
			timecodesController._buildModel = false;
            this.buildGrid(this, this._gridModel, timecodesController);
            
            this._subscribeToEvents();
        },
        
		
		/**
		 * _renameNode: Event handler when the tree publishes the EPMTreePubSub.ON_RENAME_NODE event.<br>
		 * 
		 * @param {Object} publishObj - The published object contains the tree id, the node.item, old and new values of the name field.
		 */
		_renameNode: function(publishObj) {
			 if ((publishObj) && (publishObj.treeID == Constants.TIME_CODE_TREE_ID)) {
			 	var id = publishObj.item.identifier[0];
			 	var item = this._generalGrid.model.store._getItemByIdentity(id);
                if (item) {
					this._generalGrid.model.store._setValueOrValues(item, Constants.NAME_FIELD_NAME, publishObj.newValue, true);
				}
			 }
		},
        /*
         * getGridTitle funtion - Override the abstract class and returns the title of the grid.
         *
         */
        getGridTitle: function(){
            return AdminMessages.TIMECODE_GRID_TITLE;
        },
        
        /*
         * getGridInternalID function - Override the abstract class and returns the ID of the grid.
         *
         */
        getGridInternalID: function(){
            return Constants.TIMECODE_GRID_TITLE;
        },
        
        /*
         * This is called by controller. Returns the Attachpoint to the controller.
         */
        getAttachpoint: function(){
            return this.timecodesGridAttachPoint;
        },
        
        /*
         * This is called by abstractView while creating the view.
         * getGridLayout() is to return grid structure to abstractView.
         */
        getGridLayout: function(){
            var gridLayout = [{
                noscroll: false,
                rows: [[{
                    name: AdminMessages.CODE_DESCRIPTION_LABEL,
                    width: 20,
                    field: "description",
                    editor: dojox.grid.editors.Input
                }, {
                    name: AdminMessages.CODE_CODE_LABEL,
                    width: 10,
                    field: "code",
                    editor: dojox.grid.editors.Input
                }]]
            }];
            
            return gridLayout;
        },
        
        handleError: function(errorObject){
            EPMErrorHandler.setError(errorObject.errorMessage, errorObject.errorCode);
        },
        
        _validateController: function(data){
        
            if (!this._controller) 
                return false;
            
            if (data) {
                var resourcetype = null;
                if (dojo.isArray(data.resourcetype)) {
                    resourcetype = data.resourcetype[0];
                }
                else {
                    resourcetype = data.resourcetype;
                }
                if (resourcetype == Constants.TIME_CODE_TYPE) {
                    return true;
                }
            }
            return false;
        },
        
        _onMouseOver: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onMouseOut: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onSelect: function(data){
            if (this._validateController(data)) {
                this.inherited(arguments);
            }
        },
        
        _onBlur: function(){
        
        },
        
		/**
		 * The method _deleteTimecodes will be called from timecodes.js when the delete button is clicked 
		 * The method will prompt for the confirmation for the delete and do accordingly
		 */		
		_deleteTimecodes: function(){
			this.confirmDialog = new com.ibm.epm.web.core.util.internal.ConfirmDeleteDialog({
               gridController: this._controller,
			   grid: this._generalGrid
            });
        },
		
		/*
		 * Handle the error 
		 */
        handleError: function(errorMessage){
            this.inherited("handleError", arguments);
            adminUtil.displayErrorMessage("errorMsgTimecodes",errorMessage);
        },
        
        _saveTimecodes: function(){
            this.persistGrid(true);
        },
		
		/**
		 * destroy method will call the unsubscribeAllEvents and call base call destroy
		 */
		destroy: function(){
            this._unsubscribeToAllEventsTC();
			this.inherited(arguments);
        },
		
		/**
         * eventSubscribeToAllEvents: Method to subscribe to all events. <br>
         */
        eventSubscribeTC: function(){
            this._codesubscriptions = [// Tree event on rename of node
									   dojo.subscribe(EPMTreePubSub.ON_RENAME_NODE, this, "_renameNode"),
									   dojo.subscribe(EPMTreePubSub.ON_KEY_DOWN, this, "_onKeyDownFromTree")];
        },
        
        /**
         * _unsubscribeToAllEvents: Method to unsubscribe to all events. <br>
         */
        _unsubscribeToAllEventsTC: function(){
            if (this._codesubscriptions != null) {
                for (var topic in this._codesubscriptions) {
                    var handler = this._codesubscriptions[topic];
                    if (handler) {
                        dojo.unsubscribe(handler);
                    }
                }
            }
            this._codesubscriptions = [];
        },
		
		/**
		 * createView - method will call the createView of the abstractView
		 * And also set the grid to grow auto.
		 */
		createView: function(){
			this.inherited('createView', arguments);
			
			this._generalGrid.autoHeight = true;
			this._generalGrid.update();
		},
		
		/* _onKeyDownFromTree : On key down in the tree control comes here, focus is set to the corresponging row's first cell in grid */
		
		_onKeyDownFromTree: function(data) {
            if(data.nodeWidgetItem) {
                var rowIndex = this._validateAndGetRow(data.nodeWidgetItem);
                // Set the focus on the correct row
                if (this._generalGrid) {
                    if (rowIndex >= 0) {
                        var keys = dojo.keys;
                        // Check if the tab key was pressed
                        if (data.keySet.keyCode == keys.TAB) {
                            // Set the focus to the first cell on the selected row
                            this._generalGrid.selection.select(rowIndex);
                            this._generalGrid.focus.setFocusIndex(rowIndex, 0);
                            return;
                        }     
                    }        
                }
            }
        }
    });
})();

