/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
/**
 * EPM User Detail widget
 */
(function(){

    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserDetail");
    
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.epm.admin.web.client.internal.user.EPMUserClient");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserPoolsGridView");
    dojo.require("com.ibm.epm.resource.web.ui.internal.StaticPoolsTreeWidget");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreeDialog");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserSkillsGridView");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserCompetenciesGridView");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridDialog");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserConstants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGridView");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.calendar.Calendar");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMCalendarDialog");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMToolbar");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMTreePubSub");
    dojo.require("com.ibm.epm.web.core.ui.internal.IconClassConstants");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMGridPubSub");
    dojo.require("com.ibm.epm.admin.web.ui.internal.Util.Constants");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.rates.ResourceRateAssociation");
    dojo.require("jazz.ui.Dialog");
    dojo.require("dojo.number");
    dojo.require("com.ibm.epm.web.core.grid.controller.PubSub");
        
    dojo.requireLocalization("com.ibm.epm.admin.web.ui", "Messages");
    
    var EPMUserClient = com.ibm.epm.admin.web.client.internal.user.EPMUserClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var EPMGridDialog = com.ibm.epm.web.core.ui.internal.EPMGridDialog;
    var EPMUserSkillsGridView = com.ibm.epm.admin.web.ui.internal.view.user.EPMUserSkillsGridView;
    var EPMUserCompetenciesGridView = com.ibm.epm.admin.web.ui.internal.view.user.EPMUserCompetenciesGridView;
    var EPMUserPoolsGridView = com.ibm.epm.admin.web.ui.internal.view.user.EPMUserPoolsGridView;
    var EPMTreeDialog = com.ibm.epm.web.core.ui.internal.EPMTreeDialog;
    var UserDetailMessages = dojo.i18n.getLocalization("com.ibm.epm.admin.web.ui", "Messages");
    var EPMUserConstants = com.ibm.epm.admin.web.ui.internal.view.user.EPMUserConstants;
    var CalendarGridView = com.ibm.epm.admin.web.ui.internal.view.calendar.CalendarGridView;
    var Calendar = com.ibm.epm.admin.web.ui.internal.view.calendar.Calendar;
    var EPMCalendarDialog = com.ibm.epm.web.core.ui.internal.EPMCalendarDialog;
    var EPMToolbar = com.ibm.epm.web.core.ui.internal.EPMToolbar;
    var EPMTreePubSub = com.ibm.epm.web.core.ui.internal.EPMTreePubSub;
    var EPMGridPubSub = com.ibm.epm.web.core.ui.internal.EPMGridPubSub;
    var Constants = com.ibm.epm.admin.web.ui.internal.Util.Constants;
    var IconClassConstants = com.ibm.epm.web.core.ui.internal.IconClassConstants;
    var PubSub = com.ibm.epm.web.core.grid.controller.PubSub;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserDetail", [dijit._Widget, dijit._Templated], {
        _selectedEPMUser: "",
        _selectedPoolRow: null,
        postParameterMap: {},
        poolsTree: null,
        _rowSelectionSubscription: null,
        _loadingCompletedSubscription: null,
        
        // Currently selected rate for user. "" means none selected.
        _currentlySelectedRate: "",
        
        // Currently selected calendar for user. "" means none selected.
        _currentlySelectedCalendar: "",
        
        // Stores initial relative URI of rate assigned to user, or "" for no assigned rate
        _initialRateURI: "",
        
        // Stores initial relative URI of calendar assigned to user, or "" for no assigned calendar
        _initialCalendarURI: "",
        
        /*
         * Set to true if user has JazzAdmins rights, allowing user to edit any user details.
         * Otherwise, user can only edit SOME of their own details.
         */
        _userIsAdmin: false,
        
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/user/templates/EPMUserDetail.html"),
        
        constructor: function() {
            // Grab the I18N'ed summary text for each section, to be substituted into the HTML template
            this.skillsSummaryLabel = UserDetailMessages.USERMGMT_SKILL_SECTION_SUMMARY;
            this.competenciesSummaryLabel = UserDetailMessages.USERMGMT_COMPETENCY_SECTION_SUMMARY;
            this.poolsSummaryLabel = UserDetailMessages.USERMGMT_POOL_SECTION_SUMMARY;
            this.ratesSummaryLabel = UserDetailMessages.USERMGMT_RATE_SECTION_SUMMARY;
            this.calendarSummaryLabel = UserDetailMessages.USERMGMT_CALENDAR_SECTION_SUMMARY;
            
            // Labels and button text for rates section
            this.ratesNameLabel = UserDetailMessages.USERMGMT_RATE_NAME_LABEL;
            this.ratesCostLabel = UserDetailMessages.USERMGMT_RATE_COST_LABEL;
            this.ratesSellingLabel = UserDetailMessages.USERMGMT_RATE_SELLING_LABEL;
            this.ratesButtonLabel = UserDetailMessages.USERMGMT_RATE_RATESBUTTON_LABEL;
            
            // Labels and button text for calendar section
            this.calendarNameLabel = UserDetailMessages.USERMGMT_CALENDAR_NAME_LABEL;
            this.calendarHoursLabel = UserDetailMessages.USERMGMT_CALENDAR_HOURS_LABEL;
            this.calendarButtonLabel = UserDetailMessages.USERMGMT_CALENDAR_CALENDARBUTTON_LABEL;
            this.calendarEditExceptionsLabel = UserDetailMessages.USERMGMT_CALENDAR_EXCEPTIONSBUTTON_TOOLTIP;
            
            // Pass class name to the template so it doesn't have to be hardcoded in the template
            this.disabledIconClassName = IconClassConstants.EPM_DISABLED_ICON_CLASS;
        },
        
        postCreate: function() {
            /*
             * Determine if user is an admin and thus has rights to modify any user info.
             * Note that if user is not admin, the user will only have reached this code
             * for editing his own profile, not another user's info.
             */
            this._userIsAdmin = com.ibm.epm.web.core.ui.internal.EPMView.Dashboard.Util.userIsAdmin();
            
            /*
             * Create all the "title" toolbars for each section of the page.
             */
            this._createCalendarTitleToolbar();
            this._createSkillsTitleToolbar();
            this._createCompetenciesTitleToolbar();
            this._createRatesTitleToolbar();
            this._createPoolsTitleToolbar();
            
            /*
             * Create all the button toolbars for each section of the page.
             * (Not all sections have a toolbar.)
             */
            this._createSkillsToolbar();
            this._createCompetenciesToolbar();
            this._createPoolsToolbar();

            /*
             * Need to listen to selection of any grid row to enable/disable the Delete
             * buttons appropriately.
             */
            this._rowSelectionSubscription = 
                dojo.subscribe(EPMGridPubSub.ON_ROW_SELECTION_CHANGED, this, "_gridRowSelectionChanged");

            /*
             * Need to enable buttons after grids are done loading (so that buttons can't be pressed too
             * early).
             */
            this._loadingCompletedSubscription = dojo.subscribe(PubSub.LOADING_COMPLETED, this, "_onGridLoaded");
            
            /*
             * Reset vars that store pending save info.
             */
            this.skillsDeleted = new Array();
            this.competenciesDeleted = new Array();
            this.ratesMap = new Array();
            this.calendarAdded = null;
            this.poolsMap = new Array();
            this._initialCalendarID = null;
            
            /*
             * Retrieve data about this user from the server and create the grid DOM objects.
             */
            this._getUserDetail();
			
			// updated the image.
			this._exceptionButton.src = dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/images/edit.gif");
        },
        
        /**
         * Fetch the detail of the user selected in the user grid.
         * The detail contains the Skills, Competencies and Rates link of the user.
         * It also contains the calendar link.
         * Use these detail to populate the Skills, Competencies and Rates grid.
         */
        _getUserDetail: function(){
            var self = this;
            var serviceObj = {
            
                success: function(userDetail){
                    if (!userDetail) {
                        return;
                    }
                    if (userDetail.skillassignment) {
                        self._createSkillsGrid(userDetail.skillassignment);
                    }
                    else {
                        self._createSkillsGrid("");
                    }
                    if (userDetail.competencyassignment) {
                        self._createCompetencyGrid(userDetail.competencyassignment);
                    }
                    else {
                        self._createCompetencyGrid("");
                    }
                    if (userDetail.Rate) {
                        self._createRatesSection(userDetail.Rate);
                    }
                    else {
                        self._createRatesSection(null);
                    }
                    if (userDetail.Calendar) {
                        self._createCalendarSection(userDetail.Calendar);
                    }
                    else {
                        self._createCalendarSection(null);
                    }
                    if (userDetail.StaticPool) {
                        self._createPoolsGrid(userDetail.StaticPool);
                    }
                    else {
                        var poolsLink = "";
                        self._createPoolsGrid(poolsLink);
                    }
                },
                
                failure: function(errorObj){
                    console.log(errorObj.errorMessage);
                }
                
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            EPMUserClient.getUserDetail(srh, this._selectedEPMUser, "get");
        },
        
        /**
         * Creates the calendar section of the user selected.
         * It allows the user to add (but not change) the calendar, and also allows the user to
         * modify exceptions.
         */
        _createCalendarSection: function(calendarLink) {
            /*
             * Save initial calendar URI from the user's GET.  Will save "" if user had no calendar.
             * Used later to figure out what has changed and needs saving.
             */
            this._initialCalendarURI = (calendarLink != null) ? calendarLink.relativeHref : "";
            
            /*
             * Populate calendar section with details about the selected calendar (if any).
             */
            this._showCalendarDetail(this._initialCalendarURI);
        },
        
        /*
         * Given a calendar's relative URI, do a GET of the calendar and populate the calendar section with info
         * on that selected calendar.
         */
        _showCalendarDetail: function(calendarURI) {
            /*
             * User has calendar assigned already; fetch the details and display them.
             */
            if (calendarURI != "") {
                var self = this;
                var serviceObj = {
                    success: function(calendar) {
                        self._currentlySelectedCalendar = calendar;
                        self._calendarNameBox.value = calendar.name;
                        self._calendarNameBox.title = calendar.name;
                        self._calendarHoursBox.value = calendar.hrsPerDay;

                        // Enable appropriate button depending on whether calendar was specified already
                        if (self._initialCalendarURI != "") {
                            self._calendarButton.disabled = true;
                            self.disableExceptionButton(false);
                        }
                        else {
                            self._calendarButton.disabled = false;
                            self.disableExceptionButton(true);
                        }
                    },
                    failure: function(errorObj) {
                        self._currentlySelectedCalendar = null;
                        self._notifyFailedSave(errorObj);
                    }
                };
                var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
                EPMUserClient.getURI(srh, calendarURI);
            }

            /*
             * User has no calendar assigned; clear the fields.
             */            
            else {
                this._currentlySelectedCalendar = null;
                this._calendarNameBox.value = UserDetailMessages.USERMGMT_CALENDAR_NOT_SELECTED;
                this._calendarNameBox.title = UserDetailMessages.USERMGMT_CALENDAR_NOT_SELECTED;
                this._calendarHoursBox.value = "";
               
                // Enable appropriate button depending on whether calendar was specified already
                if (this._initialCalendarURI != "") {
                    this._calendarButton.disabled = true;
                    this.disableExceptionButton(false);
                }
                else {
                    this._calendarButton.disabled = false;
                    this.disableExceptionButton(true);
                }
            }
        },

        /*
         * Callback for when the exception button in the calendar section is clicked. Allows user to
         * edit the user's calendar exceptions. (Ignore any click when button is disabled.)
         */
        _exceptionButtonClicked: function(e) {
            if (!dojo.hasClass(this._exceptionButton, IconClassConstants.EPM_DISABLED_ICON_CLASS)) {
                this._calendarButtonClicked(e);
            }
        },
        
        /*
         * Callback for when the "Calendar" button in the calendar section is clicked. Allows user to add 
         * a calendar to the user.
         */
        _calendarButtonClicked: function(e) {
            /*
             * Calendar was already assigned to user and saved. Proceed to modify exceptions.
             */
            if (this._initialCalendarURI != "") {
                if (this.calendarDialog) {
                    this.calendarDialog.destroy();
                    this.calendarDialog = null;
                }
                        
                this.userCalendar = new Calendar();
                this.calendarDialog = new EPMCalendarDialog({
                    calendar: this.userCalendar,
                    title: UserDetailMessages.CALENDAR,
                    buttonlist: []
                });
                        
                this.userCalendar.load(this._initialCalendarURI, false, true, this.calendarDialog);
                        
                this.calendarDialog.show();
            }
            
            /*
             * No calendar assigned yet.  Present dialog to add calendar.
             */
            else {
                /*
                 * Create calendar selection dialog's grid to be placed in that dialog.
                 * Grid population will begin to happen asynchronously.
                 */
                this.calendarGrid = new CalendarGridView({
                    _treeID: EPMUserConstants.CALENDAR_URI
                });                    
                this.calendarGrid.buildCalendarGrid(true);

                /*
                 * Launch calendar selection dialog once its grid has been created.
                 */
                var self = this;
                var intervalID = setInterval(function() {
                    if (self.calendarGrid._generalGrid) {
                        clearInterval(intervalID);
                        var buttonlist = [
                            {
                                label: UserDetailMessages.OK,
                                action: dojo.hitch(self, self._assignCalendar)
                            },
                            {
                                label: UserDetailMessages.CANCEL,
                                action: dojo.hitch(self, self._cancelCalendar)
                            }
                        ];
                            
                        self.calendarGridDialog = new EPMGridDialog({
                            grid: self.calendarGrid,
                            title: UserDetailMessages.CALENDAR_ASSIGNMENT_DIALOG_TITLE,
                            buttonlist: buttonlist
                        });
                        self.calendarGridDialog.show();
                        self.calendarGrid._generalGrid.selection.multiSelect = false;
                        self.calendarGrid._generalGrid.render();
                        self.calendarGrid._generalGrid.refresh();
                    }
                }, 50);
            }
        },
                
        /*
         * Called when user clicks the "Assign" button in the calendar selection dialog.
         * Will retrieve calendar info from dialog and close the dialog.
         */
        _assignCalendar: function() {
            if ((this.calendarGrid) && (this.calendarGrid._generalGrid)) {
            
                // If calendar selected, put it in pending save
                var selectedRowIndexes = this.calendarGrid._generalGrid.selection.getSelected();
                if (!(!selectedRowIndexes || selectedRowIndexes === null)) {
                    var selectedRow = this.calendarGrid._generalGrid.model.getRow(selectedRowIndexes[0]);
                    this._replaceCalendar(selectedRow);
                }
            }
            
            this.calendarGrid.destroy();
            this.calendarGridDialog.destroy();
        },
        
        /*
         * User pressed Cancel button in the calendar selection dialog.
         */
        _cancelCalendar: function() {
            this.calendarGrid.destroy();
            this.calendarGridDialog.destroy();
        },
        
        /*
         * Update the UI with the newly selected calendar and put this calendar info into the pending
         * save info.
         */
        _replaceCalendar: function(selectedRow) {
            if (selectedRow != null) {
                // Only bother to update UI if the selected calendar actually changed
                if (selectedRow.id != this._initialCalendarURI) {
                    this.calendarAdded = selectedRow.id;
                    this._setDirty();
                    this._showCalendarDetail(this.calendarAdded);
                }
            }
        },
        
        /**
         * Creates the pools grid of the user selected.
         * It shows all the pools that the user possess. Pools can be added/removed.
         */
        _createPoolsGrid: function(poolsLink){
            this._userPoolsGrid = new EPMUserPoolsGridView({
                _treeID: EPMUserConstants.NEW_POOL_UNSAVED
            });
            this._userPoolsGrid.buildPoolGrid(poolsLink);
            this._poolsGridAttachPoint.appendChild(this._userPoolsGrid.domNode);
            
            /*
             * setInterval is required as the grid might not be processed completely and
             * an event needs to be captured once the grid is ready to use.
             */
            var self = this;
            var intervalID = setInterval(function(){
                if (self._userPoolsGrid._generalGrid) {
                    clearInterval(intervalID);
                    /*
                     * This is required as sometimes, even if the grid is ready, it is not rendered in the
                     * html and hence not visible. By rendering and refreshing the grid, it becomes visible
                     * and if by chance also the grid is not ready is taken care of.
                     * Also it is useful when the network connection is slow.
                     */
                    self._userPoolsGrid._generalGrid.render();
                    self._userPoolsGrid._generalGrid.refresh();
                }
            }, 50);
        },

        /**
         * When a new pool needs to be added to the user's profile, a dialog box pops up and shows
         * all the existing pools created by the Resource Manager. New pools can be selected from this list
         * and can be assigned to the user.
         */
        _addPools: function(){
            this._selectedPoolRow = null;
            
            this._poolsTree = new com.ibm.epm.resource.web.ui.internal.StaticPoolsTreeWidget({
                modifyAllowed: false
            });
            this._poolsTree._buildStaticPoolsTree();
            var buttonlist = [
                {
                    label: UserDetailMessages.OK,
                    action: dojo.hitch(this, this._assignPool)
                },
                {
                    label: UserDetailMessages.CANCEL,
                    action: dojo.hitch(this, this._cancelPool)
                }
            ];
            this.poolTreeDialog = new EPMTreeDialog({
                tree: this._poolsTree,
                title: UserDetailMessages.POOL,
                buttonlist: buttonlist
            });
            this.poolTreeDialog.show();
        },
        
        /*
         * Assign the selected pools to the user
         */
        _assignPool: function(){
            if (this._selectedPoolRow) {
                // check whether its in grid already before adding it to the grid
                if (this._userPoolsGrid._generalGrid.model.getRowByIdentity(this._selectedPoolRow.href) === -1) {
                    this._userPoolsGrid.addPool(this._selectedPoolRow);
                    this._updateMap(this._selectedPoolRow.href, this.poolsMap);
                    this._setDirty();
                    this._gridRowSelectionChanged({gridID: EPMUserConstants.EPM_USER_POOLS_GRID_TITLE});
                }
                
            }
            this._poolsTree.destroy();
            this.poolTreeDialog.destroy();
        },
        
        /*
         * Cancel the pool selection dialog.
         */
        _cancelPool: function() {
            this._poolsTree.destroy();
            this.poolTreeDialog.destroy();
        },
        
        /**
         * Remove the selected pools from the user
         */
        _removePool: function() {
            if ((this._userPoolsGrid) && (this._userPoolsGrid._generalGrid)) {
                var selectedRowIndexes = this._userPoolsGrid._generalGrid.selection.getSelected();
                if (!selectedRowIndexes || selectedRowIndexes === null || selectedRowIndexes.length == 0) {
                    return;
                }
                
                // Note that a length of 1 does not always mean that a row is really selected
                var changed = false;
                for (var i = 0; i < selectedRowIndexes.length; i++) {
                    var row = this._userPoolsGrid._generalGrid.model.getRow(selectedRowIndexes[i]);
                    if (!row)
                        break;
                    changed = true;
                    
                    var tempID = row.id;
                    // Remove the Unsaved Tag
                    var indexOfUnsaved = tempID.indexOf(EPMUserConstants.NEW_ELEMENT_UNSAVED_PREFIX);
                    
                    if (indexOfUnsaved != -1) {
                        tempID = tempID.substring(0, indexOfUnsaved);
                    }
                    
                    if (!this._isIDPresentInMap(tempID, this.poolsMap)) {
                        tempID = tempID + ":" + EPMUserConstants.DELETE;
                        this.poolsMap.push(tempID);
                    }
                }
                if (changed) {
                    this._userPoolsGrid._generalGrid.removeSelectedRows();
                    this._userPoolsGrid._generalGrid.model.store.save();
                    this._setDirty();
                }
            }
        },
        
        /*
         * Callback for when a grid finishes loading.  The page contains 3 grids, so any
         * of those 3 loading will call this, identifable by the controller a grid is using.
         */
        _onGridLoaded: function(e) {
            if (e) {
                if (e.controllerID == EPMUserConstants.EPM_USER_SKILLS_CONTROLLER_ID) {
                    this.disableAddButton(this.addSkillButton, false);
                }
                
                else if (e.controllerID == EPMUserConstants.EPM_USER_COMPETENCY_CONTROLLER_ID) {
                    this.disableAddButton(this.addCompetencyButton, false);
                }
                
                else if (e.controllerID == EPMUserConstants.EPM_USER_POOLS_CONTROLLER_ID) {
                    // Only admin users can edit pools
                    if (this._userIsAdmin)
                        this.disableAddButton(this.addPoolButton, false);
                }
            }
        },
        
        /**
         * Creates the skills grid of the user selected.
         * It shows all the skills that the user possess. Skills can be added/removed.
         */
        _createSkillsGrid: function(skillsLink) {
            this._userSkillsGrid = new EPMUserSkillsGridView({
                _treeID: EPMUserConstants.NEW_SKILL_UNSAVED
            });
            this._userSkillsGrid.buildSkillGrid(skillsLink);
            this._skillsGridAttachPoint.appendChild(this._userSkillsGrid.domNode);
            
            /*
             * setInterval is required as the grid might not be processed completely and
             * an event needs to be captured once the grid is ready to use.
             */
            var self = this;
            var intervalID = setInterval(function(){
                if (self._userSkillsGrid._generalGrid) {
					self._userSkillsGrid.setContainingPage(self._userManagementEditorPage);
                    clearInterval(intervalID);
                    /*
                     * This is required as sometimes, even if the grid is ready, it is not rendered in the
                     * html and hence not visible. By rendering and refreshing the grid, it becomes visible
                     * and if by chance also the grid is not ready is taken care of.
                     * Also it is useful when the network connection is slow.
                     */
                    self._userSkillsGrid._generalGrid.render();
                    self._userSkillsGrid._generalGrid.refresh();                    
                }
            }, 50);
        },
        
        /**
         * Toolbar containing the title of the calendar section.
         */
        _createCalendarTitleToolbar: function() {
            this.calendarTitleToolbar = new EPMToolbar({
                title: UserDetailMessages.USERMGMT_CALENDAR_SECTION_TITLE
            });
            
            this._calendarTitleToolbarAttachPoint.appendChild(this.calendarTitleToolbar.domNode);
        },
        
        /**
         * Toolbar containing the title of the skills section.
         */
        _createSkillsTitleToolbar: function() {
            this.skillTitleToolbar = new EPMToolbar({
                title: UserDetailMessages.USERMGMT_SKILL_SECTION_TITLE
            });
            
            this._SkillsTitleToolbarAttachPoint.appendChild(this.skillTitleToolbar.domNode);
        },
        
        /**
         * Toolbar containing the button to add/remove the skills from the grid.
         */
        _createSkillsToolbar: function() {
            this.addSkillButton = new dijit.form.Button({
                title: UserDetailMessages.ADD_SKILL,
				label : UserDetailMessages.ADD,
				showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addSkills)
            });
            this.disableAddButton(this.addSkillButton, true);
            
            this.deleteSkillButton = new dijit.form.Button({
                title: UserDetailMessages.DELETE_SKILL,
				label : UserDetailMessages.DELETE,
				showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._removeSkill)
            });
            this.disableDeleteButton(this.deleteSkillButton, true);
            
            var rightContentList = [this.addSkillButton, this.deleteSkillButton];
            
            this.skillToolbar = new EPMToolbar({
                rightContentList: rightContentList
            });
            
            this._SkillsToolbarAttachPoint.appendChild(this.skillToolbar.domNode);
        },
        
        /**
         * When a new skill needs to be added to the user's profile, a dropdown shows
         * all the existing skills created by the Admin. New skills can be selected from this list
         * and can be assigned to the user.
         */
        _addSkills: function(){
			if(this._userSkillsGrid){
				if(!this._userSkillsGrid.checkSkillsExist())
				{
					this.displayError(UserDetailMessages.NO_SKILL_FOUND);
					return;
				}
				if(!this._userSkillsGrid.checkProficienciesExist())
				{
					this.displayError(UserDetailMessages.NO_PROFICIENCY_FOUND);
					return;
				}
				this._userSkillsGrid.addAssignment();
			}
            this._gridRowSelectionChanged({gridID:Constants.SKILL_GRID_TITLE});
			this._setDirty(true);
        },
        
        /**
         * Assign the selected skills to the user
         */
        _assignSkill: function(){
			var skmap = [];
			if (this._userSkillsGrid._controller) {
				var modifiedRows = this._userSkillsGrid._controller.getModifiedRows();
				for(var i in modifiedRows){
					var selectedRow = modifiedRows[i].row;
					
					if (selectedRow) {
						SkillProficiency = this._userSkillsGrid.getSelectOptionsForProficiency();
						SkillProficiencyId = this._userSkillsGrid.getSelectOptionsForProficiencyId();
						var proficiencyId = null;
						for (var j = 0; j < SkillProficiency.length; j++) {
                     	if(SkillProficiency[j] == selectedRow.proficiencyLevel){
					 			proficiencyId = SkillProficiencyId[j];
								break;
					 		}                                
                		}
						
						Skills = this._userSkillsGrid.getSelectOptionsForSkill();
						SkillIds = this._userSkillsGrid.getSelectOptionsForSkillId();
						var skillId = null;
							for (var j = 0; j < Skills.length; j++) {
		                     	if(Skills[j] == selectedRow.name){
							 			skillId = SkillIds[j];
										break;
							 		}                                
	                		}
						}
						var skillassignmnetjson = new dojox.string.Builder("");
						var id="";
						if (selectedRow.id && selectedRow.id.length > 0 && selectedRow.id.indexOf(EPMUserConstants.UNSAVED) < 0) {
							id=selectedRow.id;
						}
						var value="";
						if (skillId != null) {
							value = "\"skill\":\""+skillId+"\"";
						}
						if (proficiencyId != null) {
							if(value.length>0)
								value += ",";
							value = value + "\"proficiency\":\""+proficiencyId+"\"";;
						}
						if (value.length > 0) {
							skillassignmnetjson.append("{\"skillassignments\" : {");
							skillassignmnetjson.append(value);
							skillassignmnetjson.append(",\"id\":\"" + id + "\"}}");
						}
						this._updateMapJson(skillassignmnetjson.toString(), skmap);
				}
				
				for(var i in this.skillsDeleted){
					var skillassignmnetjson = new dojox.string.Builder("{\"skillassignments\" : {");
                    	skillassignmnetjson.append("\"skill\":\" \" ");
						skillassignmnetjson.append(" ,\"proficiency\":\"");
						skillassignmnetjson.append("\",\"id\":\"");
						skillassignmnetjson.append(this.skillsDeleted[i]);
						skillassignmnetjson.append("\", \"delete\" : \"true\"}}");	
						this._updateMapJson(skillassignmnetjson.toString(), skmap);
					}	
					
			}
			return skmap;
        },
        
        _updateMap: function(id, map){
        
            var tempID = id + ":" + EPMUserConstants.DELETE;
            
            var flag = this._isIDPresentInMap(tempID, map);
            if (flag === false) {
            
                map.push(id);
            }
        },
        
        _isIDPresentInMap: function(id, map){
        
            var numberOfSkillsAdded = map.length;
            if (numberOfSkillsAdded === 0) {
            
                return false;
            }
            for (var i = 0; i < numberOfSkillsAdded; i++) {
				
                if (map[i] === id) {
                
                    map.splice(i, 1);
                    return true;
                }
            }
            return false;
        },
		
		_updateMapJson: function(id, map){
            var flag = this._isIDPresentInMapJson(id, map);
            if (flag === false) {            
                map.push(id);
            }
        },
		
		_isIDPresentInMapJson: function(id, map){
        
            var numberOfSkillsAdded = map.length;
            if (numberOfSkillsAdded === 0) {
            
                return false;
            }
            
			var Obj = null;
			if(dojo.fromJson(id).skillassignments){
				if(dojo.fromJson(id).skillassignments.skill){
					return false;					
				}
				if(!dojo.fromJson(id).skillassignments.skill){
					for (var i = 0; i < numberOfSkillsAdded; i++) {				
						if (dojo.fromJson(map[i]).skillassignments.skill == dojo.fromJson(id).skillassignments.id) {                
	                		map.splice(i, 1);
	                    	return true;
	                	}
					}	
						
				}
			}else if(dojo.fromJson(id).competencyassignments){
				if(dojo.fromJson(id).competencyassignments.competency){
					return false;					
				}
				if(!dojo.fromJson(id).competencyassignments.competency){
					for (var i = 0; i < numberOfSkillsAdded; i++) {				
						if (dojo.fromJson(map[i]).competencyassignments.competency == dojo.fromJson(id).competencyassignments.id) {                
	                		map.splice(i, 1);
	                    	return true;
	                	}
					}	
						
				}
			}	  
            return false;
        },
        
        /**
         * Remove the selected skills from the user
         */
        _removeSkill: function(){
            if ((this._userSkillsGrid) && (this._userSkillsGrid._generalGrid)) {
                var selectedRowIndexes = this._userSkillsGrid._generalGrid.selection.getSelected();
                if (selectedRowIndexes && selectedRowIndexes.length > 0) {
                    // Note that a length of 1 does not always mean that a row is really selected
                    var changed = false;
					for (var i = 0; i < selectedRowIndexes.length; i++) {
						var row = this._userSkillsGrid._generalGrid.model.getRow(selectedRowIndexes[i]);
						if (row) {
                            changed = true;
                            if (row.id.indexOf("unsaved") < 0)
                                this.skillsDeleted.push(row.id);
                        }
                    }
					if (changed) {
                        this._userSkillsGrid._generalGrid.removeSelectedRows();
                        this._userSkillsGrid._generalGrid.model.store.save();
                        this._userSkillsGrid._selectFirstRowByDefault();
                        this._setDirty();
                    }
				}
            }
        },
        
        /**
         * Creates the competencies grid of the user selected.
         * It shows all the competencies that the user possess. Competencies can be added/removed.
         */
        _createCompetencyGrid: function(competenciesLink){
            this._userCompetenciesGrid = new EPMUserCompetenciesGridView({
                _treeID: EPMUserConstants.NEW_COMPETENCY_UNSAVED
            });
            this._userCompetenciesGrid.buildCompetencyGrid(competenciesLink);
            this._competencyGridAttachPoint.appendChild(this._userCompetenciesGrid.domNode);
            /*
             * setInterval is required as the grid might not be processed completely and
             * an event needs to be captured once the grid is ready to use.
             */
            var self = this;
            var intervalID = setInterval(function(){
                if (self._userCompetenciesGrid._generalGrid) {
					self._userCompetenciesGrid.setContainingPage(self._userManagementEditorPage);
                    clearInterval(intervalID);
                    /*
                     * This is required as sometimes, even if the grid is ready, it is not rendered in the
                     * html and hence not visible. By rendering and refreshing the grid, it becomes visible
                     * and if by chance also the grid is not ready is taken care of.
                     * Also it is useful when the network connection is slow.
                     */
                    self._userCompetenciesGrid._generalGrid.render();
                    self._userCompetenciesGrid._generalGrid.refresh();
                }
            }, 50);
        },
        
        /**
         * Toolbar containing the title of the competencies section.
         */
        _createCompetenciesTitleToolbar: function(){
            this.competenciesTitleToolbar = new EPMToolbar({
                title: UserDetailMessages.USERMGMT_COMPETENCY_SECTION_TITLE
            });
            this._CompetenciesTitleToolbarAttachPoint.appendChild(this.competenciesTitleToolbar.domNode);
        },
        
        /**
         * Toolbar containing the button to add/remove the competencies from the grid.
         */
        _createCompetenciesToolbar: function(){
            this.addCompetencyButton = new dijit.form.Button({
                title: UserDetailMessages.ADD_COMPETENCY,
                label : UserDetailMessages.ADD,
                showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addCompetencies)
            });
            this.disableAddButton(this.addCompetencyButton, true);
            
            this.deleteCompetencyButton = new dijit.form.Button({
                title: UserDetailMessages.DELETE_COMPETENCY,
                label : UserDetailMessages.DELETE,
                showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._removeCompetency)
            });
            this.disableDeleteButton(this.deleteCompetencyButton, true);
                
            var rightContentList = [this.addCompetencyButton, this.deleteCompetencyButton];            
            
            this.competenciesToolbar = new EPMToolbar({
                rightContentList: rightContentList
            });
            this._CompetenciesToolbarAttachPoint.appendChild(this.competenciesToolbar.domNode);
        },
        
        /**
         * Toolbar containing the title of the rates section.
         */
        _createRatesTitleToolbar: function(){
            this.ratesTitleToolbar = new EPMToolbar({
                title: UserDetailMessages.USERMGMT_RATE_SECTION_TITLE
            });
            this._RatesTitleToolbarAttachPoint.appendChild(this.ratesTitleToolbar.domNode);
        },
        
        /**
         * When a new competency needs to be added to the user's profile, a dropdown shows
         * all the existing competencies created by the Admin. New competencies can be selected from this list
         * and can be assigned to the user.
         */
        _addCompetencies: function(){
			if(this._userCompetenciesGrid){
				if(!this._userCompetenciesGrid.checkCompetenciesExist())
				{
					this.displayError(UserDetailMessages.NO_COMPETENCY_FOUND);
					return;
				}
				if(!this._userCompetenciesGrid.checkProficienciesExist())
				{
					this.displayError(UserDetailMessages.NO_PROFICIENCY_FOUND);
					return;
				}
				this._userCompetenciesGrid.addAssignment();
			}
            this._gridRowSelectionChanged({gridID:Constants.COMPETENCY_GRID_TITLE});
			this._setDirty(true);
		},
        
		/**
		 * Display an error message at the top of User Editor
		 * Calls the displayError method in UserManagementEditorPage.
		 */
		displayError: function(message)
		{
			if(this._userManagementEditorPage){
				this._userManagementEditorPage.displayError(message);
			}
		},
        
        /**
         * Assign the selected competencies to the user
         */
         _assignCompetency: function(){
			var ckmap = [];
			if (this._userCompetenciesGrid._controller) {
				var modifiedRows = this._userCompetenciesGrid._controller.getModifiedRows();
				for(var i in modifiedRows){
					var selectedRow = modifiedRows[i].row;
                	if (selectedRow) {
							CompetencyProficiency = this._userCompetenciesGrid.getSelectOptionsForProficiency();
							CompetencyProficiencyId = this._userCompetenciesGrid.getSelectOptionsForProficiencyId();
							var proficiencyId = null;
							for (var j = 0; j < CompetencyProficiency.length; j++) {
	                     	if(CompetencyProficiency[j] == selectedRow.proficiencyLevel){
						 			proficiencyId = CompetencyProficiencyId[j];
									break;
						 		}                                
	                		}
							
							Competencies = this._userCompetenciesGrid.getSelectOptionsForCompetency();
							CompetencyIds = this._userCompetenciesGrid.getSelectOptionsForCompetencyId();
							var competencyId = null;
							for (var j = 0; j < Competencies.length; j++) {
	                     	if(Competencies[j] == selectedRow.name){
						 			competencyId = CompetencyIds[j];
									break;
						 		}                                
	                		}
						}
						
						var competencyassignmnetjson = new dojox.string.Builder("");
						var id="";
						if (selectedRow.id && selectedRow.id.length > 0 && selectedRow.id.indexOf(EPMUserConstants.UNSAVED) < 0) {
							id=selectedRow.id;
						}
						var value="";
						if (competencyId != null) {
							value = "\"competency\":\""+competencyId+"\"";
						}
						if (proficiencyId != null) {
							if(value.length>0)
								value += ",";
							value = value + "\"proficiency\":\""+proficiencyId+"\"";;
						}
						if (value.length > 0) {
							competencyassignmnetjson.append("{\"competencyassignments\" : {");
							competencyassignmnetjson.append(value);
							competencyassignmnetjson.append(",\"id\":\"" + id + "\"}}");
						}
                        this._updateMapJson(competencyassignmnetjson.toString(), ckmap);
				}
				for(var i in this.competenciesDeleted){
					var competencyassignmnetjson = new dojox.string.Builder("{\"competencyassignments\" : {");
                    	competencyassignmnetjson.append("\"competency\":\"\" ");
						competencyassignmnetjson.append(" ,\"proficiency\":\"");
						competencyassignmnetjson.append("\",\"id\":\"");
						competencyassignmnetjson.append(this.competenciesDeleted[i]);
						competencyassignmnetjson.append("\", \"delete\" : \"true\"}}");	
						this._updateMapJson(competencyassignmnetjson.toString(), ckmap);
					}	
			}
			return ckmap;
        },


        /**
         * Remove the selected competencies from the user
         */
        _removeCompetency: function(){
            if ((this._userCompetenciesGrid) && (this._userCompetenciesGrid._generalGrid)) {
                var selectedRowIndexes = this._userCompetenciesGrid._generalGrid.selection.getSelected();
                if (selectedRowIndexes && selectedRowIndexes.length > 0) {
                    // Note that a length of 1 does not always mean that a row is really selected
                    var changed = false;
					for (var i = 0; i < selectedRowIndexes.length; i++) {
						var row = this._userCompetenciesGrid._generalGrid.model.getRow(selectedRowIndexes[i]);
                        if (row) {
                            changed = true;
                            if (row.id.indexOf("unsaved") < 0)
                                this.competenciesDeleted.push(row.id);
						}
					}
                    if (changed) {
                        this._userCompetenciesGrid._generalGrid.removeSelectedRows();
                        this._userCompetenciesGrid._generalGrid.model.store.save();
                        this._userCompetenciesGrid._selectFirstRowByDefault();
                        this._setDirty();
                    }
				}
            }
            
        },
        
        /*
         * Callback for when the "Rates" button in the rates section is clicked. Allows user to add or 
         * change rates associated with the user.
         */
        _ratesButtonClicked: function(e) {
            // Nullify the dialog if already exists 
            if (this.rateDialog != null) {
                this.rateDialog.destroy();
                this.rateDialog = null;
            }
            
            this.rateInfo = new com.ibm.epm.admin.web.ui.internal.view.rates.ResourceRateAssociation(
                {summaryText: UserDetailMessages.USER_RATE_ASSOCIATION_DIALOG_SUMMARY});
            this.rateDialog = new jazz.ui.Dialog({
                contentNode : this.rateInfo.domNode,
                primaryTitle: UserDetailMessages.RATE_ASSIGNMENT_DIALOG_TITLE,
                width: "400px",
                height: "400px",
                onOpen: dojo.hitch(this, this._onRatesOpen),
                onClose: dojo.hitch(this, this._onRatesClose)
            });
        },
        
        /*
         * This method is called when the rate dialog opens.
         * Loads the rates into the grid.
         */
        _onRatesOpen: function(){
            // Set up the rate data to be passed to the dialog
            // (dialog expects data that looks like that of a grid row)
            if (this._currentlySelectedRate && this._currentlySelectedRate != null) {
                this._rateData = {
                    rate: this._currentlySelectedRate.name, 
                    rateURI: this._currentlySelectedRate.relativeHref};
            }
            else {
                this._rateData = {
                    rate: "", 
                    rateURI: ""};
            }
            
            this.rateInfo.loadResourceRate(this._rateData, this);
        },
        
        /*
         * This method is called no matter how the rate dialog manages to close.
         * Destroys the assignment info object.
         */
        _onRatesClose: function() {
            if (this.rateInfo) {
                this.rateInfo.destroy();
            }
            return true;    
        },

        /*
         * Called when user presses OK in the rates dialog to make a rate assignment change.
         * Sets new rate to be saved next time user presses Save.
         */        
        setRowData: function() {
            if (this._rateData && this._rateData.rateURI && this._rateData.rateURI != "") {
                this._currentlySelectedRate = {
                    name: this._rateData.rate,
                    relativeHref: this._rateData.rateURI};
                this._showRatesDetail(this._currentlySelectedRate.relativeHref);
            }
            else {
                this._currentlySelectedRate = null;
                this._showRatesDetail("");
            }

            this._updateRatesPendingSaveInfo();
        },
        
        /*
         * Compares the initial rate (if any) from the original GET of the user's rate
         * with the new data from the rates dialog, then creates the correct param map
         * needed to do this update during the next save.
         */
        _updateRatesPendingSaveInfo: function() {
            this.ratesMap = new Array();

            /*
             * Current selection is different than original, so param map needs updating.
             */
            if (this._initialRateURI != this._rateData.rateURI) {
                /*
                 * Change is to replace or remove existing rate, so need a param to delete original rate.
                 */
                if (this._initialRateURI != "") {
                    var delParam = this._initialRateURI + ":" + EPMUserConstants.DELETE;
                    this.ratesMap.push(delParam);
                }

                /*
                 * If rate is being replaced with another rate, add a param for that rate (otherwise
                 * we assume the rate was simply being unassigned without a replacement).
                 */
                if (this._rateData.rateURI != "")
                    this.ratesMap.push(this._rateData.rateURI);
                
                this._setDirty();
            }       
        },
        
        /*
         * This method is called when we click on the OK or Cancel button.
         * Closes the dialog and destroys the widgets.
         * (It is not called if "X" icon or Escape key is pressed.)
         */
        closeDialog: function() {
            if (this.rateDialog) {
                this.rateDialog.close();
                this.rateDialog.destroy();
                this.rateDialog = null;
            }
        },
        
        /**
         * Creates the rates section of the user selected.
         * It shows the pair of rates that the user may have assigned, and allows this to be changed.
         */
        _createRatesSection: function(ratesLink) {
            /*
             * Save initial rate URI from the user's GET.  Will save "" if user had no rate.
             * Used later to figure out what has changed and needs saving.
             */
            this._initialRateURI = (ratesLink != null) ? ratesLink.relativeHref : "";
            
            /*
             * Populate rates section with details about the selected rate (if any).
             */
            this._showRatesDetail(this._initialRateURI);
        },
        
        /*
         * Given a rate's relative URI, do a GET of the rate and populate the rate section with info
         * on that selected rate.
         */
        _showRatesDetail: function(ratesURI) {
            /*
             * User has rates assigned already; fetch the details and display them.
             */
            if (ratesURI != "") {
                var self = this;
                var serviceObj = {
                    success: function(rate) {
                       self._currentlySelectedRate = rate;
                       self._rateNameBox.value = rate.name;
                       self._rateNameBox.title = rate.name;
                       self._rateCostBox.value = dojo.number.format(
                           rate.costRate, {places: 2});
                       self._rateSellingBox.value = dojo.number.format(
                           rate.sellingRate, {places: 2});
                       
                       // Allow rate to be changed once current rate info has loaded (only if user is admin)
                       if (self._userIsAdmin) {
                           self._ratesButton.disabled = false;
                       }
                    },
                    failure: function(errorObj) {
                        self._currentlySelectedRate = null;
                        self._notifyFailedSave(errorObj);
                    }
                };
                var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
                EPMUserClient.getURI(srh, ratesURI);
            }

            /*
             * User has no rates assigned; clear the fields.
             */            
            else {
               this._currentlySelectedRate = null;
               this._rateNameBox.value = UserDetailMessages.USERMGMT_RATE_NOT_SELECTED;
               this._rateNameBox.title = UserDetailMessages.USERMGMT_RATE_NOT_SELECTED;
               this._rateCostBox.value = "";
               this._rateSellingBox.value = "";

               // Allow rate to be changed once current rate info has loaded (only if user is admin)
               if (this._userIsAdmin) {
                   this._ratesButton.disabled = false;
               }

            }
        },

        /**
         * Toolbar containing the title of the pools section.
         */
        _createPoolsTitleToolbar: function() {
            this.poolTitleToolbar = new EPMToolbar({
                title: UserDetailMessages.USERMGMT_POOL_SECTION_TITLE
            });
            
            this._poolTitleToolbarAttachPoint.appendChild(this.poolTitleToolbar.domNode);
        },
        
        /**
         * Toolbar containing the button to add/remove the pools from the grid.
         */
        _createPoolsToolbar: function() {
            this.addPoolButton = new dijit.form.Button({
                title: UserDetailMessages.ADD_POOL,
                label : UserDetailMessages.ADD,
                showLabel : false,
                iconClass: IconClassConstants.EPM_ADD_ICON_CLASS,
                onClick: dojo.hitch(this, this._addPools)
            });
            this.disableAddButton(this.addPoolButton, true);
            
            this.deletePoolButton = new dijit.form.Button({
                title: UserDetailMessages.DELETE_POOL,
                label : UserDetailMessages.DELETE,
                showLabel : false,
                iconClass: IconClassConstants.EPM_DELETE_ICON_CLASS,
                onClick: dojo.hitch(this, this._removePool)
            });
            this.disableDeleteButton(this.deletePoolButton, true);
            
            var rightContentList = [this.addPoolButton, this.deletePoolButton];
            
            this.poolToolbar = new EPMToolbar({
                rightContentList: rightContentList
            });
            
            this._poolToolbarAttachPoint.appendChild(this.poolToolbar.domNode);
            
            //subscribe click event
            if (!this.treeSelectHandler) {
                this.treeSelectHandler = dojo.subscribe(EPMTreePubSub.ON_SELECT, this, "_setSelectedPool");
            }
            if (!this.treeHoverHandler) {
                this.treeHoverHandler = dojo.subscribe(EPMTreePubSub.ON_MOUSE_OVER, this, "_hideMenuImage");
            }

        },
        
        _setSelectedPool: function(publishedItem){
            if (publishedItem) {
            
                if (publishedItem.identifier && publishedItem.href && publishedItem.name) {
                
                    var href = this._getValueFromArray(publishedItem.href);
                    var name = this._getValueFromArray(publishedItem.name);
                    
                    this._selectedPoolRow = {
                        "href": href,
                        "name": name,
                        "relativeHref": href + "_" + "unsaved"
                    
                    };
                    
                }
            }
        },
        
		/**
		 * MenuImage is not needed as this pool tree is a read only tree.
		 */
		_hideMenuImage: function(){
            if(this._poolsTree && this._poolsTree.menuImage){
				this._poolsTree.menuImage.style.visibility = "hidden";
			}
			
		},
		
        _getValueFromArray: function(array){
        
            if (dojo.isArray(array)) {
                return array[0];
            }
            return array;
        },
        
        /*
         * Disable the specified delete button if passed true, else enable it.
         */
        disableDeleteButton: function(button, state){
            var oldState = button.attr('disabled');
            if (oldState != state) {
                button.attr('disabled',state);
                if (state)
                    button.attr('iconClass',IconClassConstants.EPM_DELETE_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
                else
                    button.attr('iconClass',IconClassConstants.EPM_DELETE_ICON_CLASS);
            }
        },
        
        /*
         * Disable the specified add button if passed true, else enable it.
         */
        disableAddButton: function(button, state){
            var oldState = button.attr('disabled');
            if (oldState != state) {
                button.attr('disabled',state);
                if (state)
                    button.attr('iconClass',IconClassConstants.EPM_ADD_ICON_CLASS + " " + IconClassConstants.EPM_DISABLED_ICON_CLASS);
                else
                    button.attr('iconClass',IconClassConstants.EPM_ADD_ICON_CLASS);
            }
        },
        
        /*
         * Disable the calendar exception button if passed true, else enable it.
         */
        disableExceptionButton: function(state){
            var button = this._exceptionButton;
            if (state) {
                dojo.addClass(button, IconClassConstants.EPM_DISABLED_ICON_CLASS);
                button.style.cursor = "not-allowed";
                button.title = "";
            }
            else {
                dojo.removeClass(button, IconClassConstants.EPM_DISABLED_ICON_CLASS);
                button.style.cursor = "pointer";
                button.title = UserDetailMessages.USERMGMT_CALENDAR_EXCEPTIONSBUTTON_TOOLTIP;
            }
        },
        
        /*
         * Any row selection change (selected, unselected) causes this to be called.
         * Determine if state of delete button needs to be changed based on whether any
         * rows are selected or not. This page has several grids (including grids that
         * are not relevant to the delete buttons), so must check which grid.
         */
        _gridRowSelectionChanged: function(e){
            // Note that an empty grid will appear to have a single selected row, so must
            // check for that and treat it as having no selected rows

            if (e) {
                var id = e.gridID;
                
                var whichGrid = null;
                var whichButton = null;
                var disable = true;
                var disableForNonAdmin = false;
                
                if (id == Constants.SKILL_GRID_TITLE) {
                    whichGrid = this._userSkillsGrid;
                    whichButton = this.deleteSkillButton;
                }
            
                else if (id == Constants.COMPETENCY_GRID_TITLE) {
                    whichGrid = this._userCompetenciesGrid;
                    whichButton = this.deleteCompetencyButton;
                }
            
                else if (id == EPMUserConstants.EPM_USER_POOLS_GRID_TITLE) {
                    whichGrid = this._userPoolsGrid;
                    whichButton = this.deletePoolButton;

                    // Only admin users can change pools, so only enable on row select for admins
                    if (!this._userIsAdmin)
                        disableForNonAdmin = true;
                }

                // Ignore any other grid events since they won't have delete buttons to enable
            
                if (whichGrid != null) {

                    // If user is restricted due to not being admin, then this button is always disabled
                    if (disableForNonAdmin) {
                        disable = true;
                    }

                    // Otherwise, do further checks to determine if button needs to be enabled or disabled                   
                    else {
                        var selectedCount = whichGrid._generalGrid.selection.getSelectedCount();
                        
                        // Always disable if nothing is selected
                        if (selectedCount == 0) {
                            disable = true;
                        }
                        
                        // It appears something is selected, so enable if the selected row actually exists
                        else {
                            disable = false;
                            
                            var selectedRowIndexes = whichGrid._generalGrid.selection.getSelected();

                            if (selectedCount == 1) {
                                var row = whichGrid._generalGrid.model.getRow(selectedRowIndexes[0]);
                                if (!row) {
                                    disable = true; // There really isn't a selected row, so disable
                                }
                                                                
                                // Sometimes the grid just has a single "blank row" that it claims is selected; ignore this
                                else {
                                    if (row.id == "Blank_Row_unsaved")
                                        disable = true;
                                }
                            }
                        }
                    }

                    this.disableDeleteButton(whichButton, disable);
                }
        
            }
        },

        /*
         * Private function
         * saveUserEPMUserDetail: Saves the changes being done in User view
         * Incoming : None
         * Outgoing : None
         */
        saveUserEPMUserDetail: function(callback) {
            this._saveCallback = callback;
            
            if (this.calendarAdded != null) {
                var self = this;
                var serviceObj = {
                    success: function(calendar){
                        var calendarParam = {
                            name: calendar.name,
                            hrsPerDay: calendar.hrsPerDay,
                            calendarType: EPMUserConstants.CALENDAR_TYPE_ASSIGN,
                            calendarAdoptedFrom: self.calendarAdded,
                            User: self._selectedEPMUser
                        };
                        if (calendar.workingTimes) {
                            var workingTimesMap = [];
                            for (var i = 0; i < calendar.workingTimes.length; i++) {
                                var workingTime = EPMUserConstants.FROM_TIME + "=" + calendar.workingTimes[i].fromTime + "," +
                                EPMUserConstants.TO_TIME +
                                "=" +
                                calendar.workingTimes[i].toTime;
                                
                                workingTimesMap.push(workingTime);
                            }
                            calendarParam.workingTimes = workingTimesMap;
                        }
                        self._saveUserCalendar(calendarParam);
                    },
                    failure: function(errorObj){
                        self._notifyFailedSave(errorObj);
                        //                        alert(Messages.FAILED_TO_FETCH_CALENDAR);
                    }
                };
                var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
                EPMUserClient.getURI(srh, this.calendarAdded);
            }
            else {
                this._prepareSaveParam();
            }
            
        },
        
        /*
         * Private function
         * _saveUserCalendar: Create the user calendar
         * Incoming : CalendarParam
         * Outgoing : None
         */
        _saveUserCalendar: function(calendarParam){
            var self = this;
            var serviceObj = {
                success: function(calendar){
                    self._prepareSaveParam();
                },
                failure: function(errorObj){
                    self._notifyFailedSave(errorObj);
                    //                    alert(Messages.FAILED_TO_FETCH_CALENDAR);
                }
            };
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            EPMUserClient.saveUserCalendar(srh, calendarParam);
        },
        
        /*
         * Private function
         * _prepareSaveParam: Prepare the save param for User
         * Incoming : postParametersMap and flag
         * Outgoing : None
         */
        _prepareSaveParam: function(){
        
            var flag = false;
            var postParametersMap = {};
            
			var map = this._assignSkill();
            if (map.length > 0) {
                postParametersMap.skillassignments = map;
                flag = true;
            }

			map = this._assignCompetency();
            if (map.length > 0) {
                postParametersMap.competencyassignments = map;
                flag = true;
            }
            
            if (this.ratesMap.length > 0) {            
                postParametersMap.Rate = this.ratesMap;
                flag = true;
            }
            
            if (this.poolsMap.length > 0) {
                postParametersMap.StaticPool = this.poolsMap;
                flag = true;
            }
            
            if (flag) {
                this._saveUserDetails(postParametersMap);
            }
            else {
                this._notifySuccessfulSave();
                // this._cleanUpUserView();
            }
        },
        
        /*
         *
         */
        _cleanUpUserView: function(){
            this.skillsDeleted = [];
            this.calendarAdded = null;
            this.ratesMap = new Array();
            this.poolsMap = new Array();
            this._cleanUpGrids();
            this._getUserDetail();
        },
        
        /*
         * Call when the entire page was successfully saved,
         * to notify the RTC User Management container that this has
         * been accomplished.
         */       
        _notifySuccessfulSave: function() {
            this._saveCallback(this._userManagementEditorPage.pageID);
        },
        
        /*
         * Call when the page's save was unsuccessful,
         * to notify the RTC User Management container that this has
         * been accomplished and to report an error.
         */  
        _notifyFailedSave: function(errorObject) {
            this._saveCallback(this._userManagementEditorPage.pageID, 
                new Error(errorObject.errorMessage));
        },
        
        /*
         * Private function
         * _saveUserDetails: Saves the changes
         * Incoming : postParametersMap
         * Outgoing : None
         */
        _saveUserDetails: function(postParameterMap){
            var self = this;
            var serviceObj = {
                success: function(responseVal){
                    // self._cleanUpUserView(); // TODO this screws up the GUI somehow
					this.skillsDeleted = [];
                    self._notifySuccessfulSave();
                },
                
                failure: function(errorObj){
                    if (postParameterMap.Calendar) {
                        self._deleteUserCalendar(postParameterMap.Calendar, errorObj);
                    }
                    else {
                        //self.calendarAdded = new Array();
                        //self.destroy();
                        //self._getUserDetail();
                        console.log(errorObj.errorMessage);
                    }
                    
                    self._notifyFailedSave(errorObj);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            EPMUserClient.saveUserDetail(srh, this._selectedEPMUser, postParameterMap, "post");
        },

        /*
         * Private function
         * _deleteUserCalendar: Delete the User calendar if user changes are not saved properly
         * Incoming : Calendar and error object
         * Outgoing : None
         */
        _deleteUserCalendar: function(calendar, error){
            var self = this;
            var serviceObj = {
                success: function(responseVal){
                    //self.calendarAdded = new Array();
                    //self.destroy();
                    //self._getUserDetail();
                    console.log(error.errorMessage);
                },
                
                failure: function(errorObj){
                    self._notifyFailedSave(errorObj);
                    console.log(errorObj.errorMessage);
                }
            };
            
            var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
            EPMUserClient.deleteUserCalendar(srh, calendar);
        },
        
        /**
         * Cleans all the grids defined in this class.
         * Destroys the object, delete the object and remove the child nodes from all
         * the grid attachpoints.
         */
        _cleanUpGrids: function(){
	        if(this._userSkillsGrid){
				this._userSkillsGrid._containingPage = null;
			} 
			if(this._userCompetenciesGrid){
				this._userCompetenciesGrid._containingPage = null;
			} 
            var numberOfChildNodes = this._skillsGridAttachPoint.childNodes.length;
            
            for (var i = 0; i < numberOfChildNodes; i++) {
                this._skillsGridAttachPoint.removeChild(this._skillsGridAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._calendarGridAttachPoint.childNodes.length;
            
            for (i = 0; i < numberOfChildNodes; i++) {
                this._calendarGridAttachPoint.removeChild(this._calendarGridAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._competencyGridAttachPoint.childNodes.length;
            
            for (i = 0; i < numberOfChildNodes; i++) {
                this._competencyGridAttachPoint.removeChild(this._competencyGridAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._poolsGridAttachPoint.childNodes.length;
            
            for (i = 0; i < numberOfChildNodes; i++) {
                this._poolsGridAttachPoint.removeChild(this._poolsGridAttachPoint.childNodes[i]);
            }
            
            
            
            if (this._userSkillsGrid && this._userSkillsGrid.destroy) {
                this._userSkillsGrid.destroy();
                delete this._userSkillsGrid;
            }
            if (this._userCompetenciesGrid && this._userCompetenciesGrid.destroy) {
                this._userCompetenciesGrid.destroy();
                delete this._userCompetenciesGrid;
            }

            if (this.calendarDialog) {
                this.calendarDialog.destroy();
                this.calendarDialog = null;
            }
            
            if (this._userPoolsGrid && this._userPoolsGrid.destroy) {
                this._userPoolsGrid.destroy();
                delete this._userPoolsGrid;
            }
            if (this._poolsGrid && this._poolsGrid.destroy) {
                this._poolsGrid.destroy();
                delete this._poolsGrid;
            }
        },
        
        /**
         *  Cleans all the toolbar which were created in this view.
         *  Deletes, destroys and remove all the toolbars.
         */
        _cleanUpToolbars: function(){
        
            var numberOfChildNodes = this._calendarTitleToolbarAttachPoint.childNodes.length;
            for (var i = 0; i < numberOfChildNodes; i++) {
            
                this._calendarTitleToolbarAttachPoint.removeChild(this._calendarTitleToolbarAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._SkillsTitleToolbarAttachPoint.childNodes.length;
            for (i = 0; i < numberOfChildNodes; i++) {
                this._SkillsTitleToolbarAttachPoint.removeChild(this._SkillsTitleToolbarAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._SkillsToolbarAttachPoint.childNodes.length;
            for (i = 0; i < numberOfChildNodes; i++) {
                this._SkillsToolbarAttachPoint.removeChild(this._SkillsToolbarAttachPoint.childNodes[i]);
            }

            numberOfChildNodes = this._CompetenciesTitleToolbarAttachPoint.childNodes.length;
            for (i = 0; i < numberOfChildNodes; i++) {            
                this._CompetenciesTitleToolbarAttachPoint.removeChild(this._CompetenciesTitleToolbarAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._CompetenciesToolbarAttachPoint.childNodes.length;
            for (i = 0; i < numberOfChildNodes; i++) {
                this._CompetenciesToolbarAttachPoint.removeChild(this._CompetenciesToolbarAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._RatesTitleToolbarAttachPoint.childNodes.length;
            for (i = 0; i < numberOfChildNodes; i++) {
                this._RatesTitleToolbarAttachPoint.removeChild(this._RatesTitleToolbarAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._poolTitleToolbarAttachPoint.childNodes.length;
            for (i = 0; i < numberOfChildNodes; i++) {
                this._poolTitleToolbarAttachPoint.removeChild(this._poolTitleToolbarAttachPoint.childNodes[i]);
            }
            
            numberOfChildNodes = this._poolToolbarAttachPoint.childNodes.length;
            for (i = 0; i < numberOfChildNodes; i++) {
                this._poolToolbarAttachPoint.removeChild(this._poolToolbarAttachPoint.childNodes[i]);
            }
            
            if (this.calendarTitleToolbar && this.calendarTitleToolbar.destroy) {
                this.calendarTitleToolbar.destroy();
                delete this.calendarTitleToolbar;
            }
            
            if (this.skillToolbar && this.skillToolbar.destroy) {
                this.skillToolbar.destroy();
                delete this.skillToolbar;
            }
            
            if (this.skillTitleToolbar && this.skillTitleToolbar.destroy) {
                this.skillTitleToolbar.destroy();
                delete this.skillTitleToolbar;
            }
            
            if (this.competenciesToolbar && this.competenciesToolbar.destroy) {
                this.competenciesToolbar.destroy();
                delete this.competenciesToolbar;
            }
            
            if (this.competenciesTitleToolbar && this.competenciesTitleToolbar.destroy) {
                this.competenciesTitleToolbar.destroy();
                delete this.competenciesTitleToolbar;
            }
            
            if (this.ratesTitleToolbar && this.ratesTitleToolbar.destroy) {
                this.ratesTitleToolbar.destroy();
                delete this.ratesTitleToolbar;
            }

            if (this.poolToolbar && this.poolToolbar.destroy) {
                this.poolToolbar.destroy();
                delete this.poolToolbar;
            }
            
            if (this.poolTitleToolbar && this.poolTitleToolbar.destroy) {
                this.poolTitleToolbar.destroy();
                delete this.poolTitleToolbar;
            }
            
            
        },
        
        _cleanUpTree: function(){
            if (this._poolsTree && this._poolsTree.destroy) {
                this._poolsTree.destroy();
                delete this._poolsTree;
            }
            
            if (this.treeSelectHandler) {
                dojo.unsubscribe(this.treeSelectHandler);
                this.treeSelectHandler = null;
            }
            if (this.treeHoverHandler) {
                dojo.unsubscribe(this.treeHoverHandler);
                this.treeHoverHandler = null;
            }
        },
        
        /**
         * Notify the RTC User Management infrastructure that this page is now
         * dirty and will need to be saved.
         */
        _setDirty: function() {
            this._userManagementEditorPage.setDirty(true);
        },
        
        /**
         * This function is called when any widget is deleted.
         */
        destroy: function(){
            if (this._rowSelectionSubscription) {
                dojo.unsubscribe(this._rowSelectionSubscription);
                this._rowSelectionSubscription = null;
            }
            
            if (this._loadingCompletedSubscription) {
                dojo.unsubscribe(this._loadingCompletedSubscription);
                this._loadingCompletedSubscription = null;
            }
            
            this._cleanUpGrids();
            this._cleanUpTree();
            this._cleanUpToolbars();
            
        }
        
    });
})();
