/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

/** 
 * This tab inserts into the RTC User Management page, to provide RFP-PjM
 * user administration functionality within that RTC page.
 */
(function(){
    dojo.provide("com.ibm.epm.admin.web.ui.internal.view.user.UserManagementEditorPage");
    
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.ProfileEditorPage");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserDetail");
    dojo.require("com.ibm.epm.admin.web.ui.internal.view.user.EPMUserConstants");
    dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

    var ProfileEditorPage = com.ibm.team.repository.web.ui.internal.admin.users.ProfileEditorPage;
    var EPMUserDetail = com.ibm.epm.admin.web.ui.internal.view.user.EPMUserDetail;
    var EPMUserConstants = com.ibm.epm.admin.web.ui.internal.view.user.EPMUserConstants;
    var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    
    dojo.declare("com.ibm.epm.admin.web.ui.internal.view.user.UserManagementEditorPage", [ProfileEditorPage], {
    
        templatePath: dojo.moduleUrl("com.ibm.epm.admin.web", "ui/internal/view/user/templates/UserManagementEditorPage.html"),

        // Saved user ID from looking up the user by the RTC-provided UUID
        userId: null,

        /*
         * After looking up user info by its UUID in order to determine the user
         * name, look up and display the rest of the user details.
         */
        _populateUserData: function (data) {
            this.userId = data.userId;
            this._selectedEPMUser = EPMUserConstants.USER_URI + "/" + this.userId;
            this._showUserDetail();
        },

        /*
         * Called if error happens while looking up user by UUID.
         */
        _handleUserDataError: function (err) {
            console.log("ERROR: Could not look up user: " + this.userId);
        },

        /*
         * The RTC User Management infrastructure provides an itemUUID to each tab that
         * indicates what the currently selected user is. Editing of user info is then done on that
         * itemUUID. However, we need to look up our user info based on name, not on UUID, so that
         * itemUUID field isn't useful here without an additional service call.
         *
         * This service call will look up the rest of the user info by that UUID, storing only its
         * name, then populating the rest of the user details from further service calls to get
         * the needed info.
         */        
        postCreate: function() {
			var srh = new ServiceResponseHandler(this, "_populateUserData", "_handleUserDataError");
			AdminClient.getContributorByUUID({uuid: this.itemUUID}, srh);
        },
        
        /*
         * Show current information on the selected user.
         * The code first destroys any DOM objects that may already be showing
         * user info, then recreates them.
         */
        _showUserDetail: function() {
            this._destroyUserDetail();
            
            this._epmUserDetail = new EPMUserDetail({
                _selectedEPMUser: this._selectedEPMUser,
                _userManagementEditorPage: this
            });
            this._epmUserDetailSection.appendChild(this._epmUserDetail.domNode);
        },

        /*
         * Called by RTC User Management infrastructure when the user clicks the Save button.
         * The callback is how the code indicates back to RTC whether the save was a success
         * or failure.
         */
        doSave: function(callback) {
			this._epmUserDetail.saveUserEPMUserDetail(callback);
        },
		
		/*
		 * Display an error message.
		 * This is used to put a error message at the top of User Editor
		 */
		displayError: function(message)
		{
			if (this.pageContainer) {
				this.pageContainer.displayLongErrorMessage(message);
			}
		},		

        /*
         * Called when user clicks the Refresh button.
         */
        refresh: function() {
            /*
             * Look up user by its UUID to learn its name, then populate user details
             * using its name.
             */
            var srh = new ServiceResponseHandler(this, "_populateUserData", "_handleUserDataError");
            AdminClient.getContributorByUUID({uuid: this.itemUUID}, srh);
        },
        
        /*
         * Destroy DOM objects upon widget destruction.
         */
        destroy: function() {
            this._destroyUserDetail();
            this.inherited(arguments);
        },
        
        /*
         * Destroy DOM objects.
         */
        _destroyUserDetail: function() {
            if (this._epmUserDetail && this._epmUserDetail.destroy) {
                this._epmUserDetailSection.removeChild(this._epmUserDetail.domNode);
                this._epmUserDetail.destroy();
                delete this._epmUserDetail;
            }
        }
        
        
    });
})();
