/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.wbsGrids.AbstractWBSGridModel");
    
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    dojo.require("com.ibm.epm.web.core.grid.model.AbstractGridModel");
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    dojo.require("com.ibm.epm.web.core.ui.internal.EPMConstants");
	dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    //To get the Project Area
    dojo.require("com.ibm.team.process.web.client.projectarea");
    
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    var abstractGridModel = com.ibm.epm.web.core.grid.model.AbstractGridModel;
    
    var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    var WBSConstants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    var EPMConstants = com.ibm.epm.web.core.ui.internal.EPMConstants;
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");

	/**
	 * array of suffixes to be added to the post URI for new WBS element.  
	 */
	var newWBSElementURI = {
        epmrisk: "risk",
        program: "program",
        project: "projects",
        workitem: "workitems",
        iteration: "iteration"
    };
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.wbsGrids.AbstractWBSGridModel", abstractGridModel, {
    
        newElementPrefix: "_unsaved",
        
        constructor: function(){
        },
        
		/**
		 * _getWBSID: return the WBSID.
		 * @param {Object} relativeHref: URI to be parsed to get the WBS ID.
		 */
        _getWBSID: function(relativeHref){
            if (!relativeHref) {
                return "";
            }
            var index = relativeHref.lastIndexOf("/");
            if (index > 0) {
                return relativeHref.substring(index + 1, relativeHref.length);
            }
        },
        
        
		/**
		 * getRowGetURI: returns the rowID.
		 * @param {Object} treeElementHandle
		 * @param {Object} rowID
		 */
        getRowGetURI: function(treeElementHandle, rowID){
            console.log(treeElementHandle);
            return rowID;
        },
        
		/**
		 * getEmptyRow: returns the initialized row.
		 * @param {String} rowID: ID fow which the empty row is requested. <br>
		 * @param {Object} treeElementHandle: Contains some values passed by tree with which we can initialize the grid row eg, name, resourceType. <br>
		 */
        getEmptyRow: function(rowID, treeElementHandle){
            var now = new Date();
            var row = {
                "id": rowID,
                "wbsid": this._getWBSID(rowID),
                "name": treeElementHandle.title,
                "parentId": "",
                "resourcetype": treeElementHandle.resourcetype,
                "href": rowID,
                "plannedFor": null,
                "workItemType": null,
                "proposedstart": now,
                "proposedfinish": now,
                // Needs to be initialized to "0" instead of "" it fails while comparing with 0 during save of schedule
                "proposedduration": "0",
                "proposedeffort": "0",
				"plannedstart":  now,
	            "plannedfinish": now,
    	        "plannedduration": "0",
        	    "plannedeffort": "0",
                "loadedscheduleelement": "",
                "constraintType": WBSConstants.DEFAULT_CONSTRAINT_VALUE,
                "constraintDate":WBSConstants.DEFAULT_CONSTRAINT_VALUE,
                "constraintURI": "",
                "loadedconstraint": "",
                "loadedelement": null
            };
            
			//if new element then set the default attributes.
			if(this.isNewRow(rowID) && treeElementHandle.workItemType == WBSConstants.EPM_TASK_TYPE){
				row.constraintType = WBSConstants.AS_SOON_AS_POSSIBLE;
			}

			if (row.resourcetype == WBSConstants.ITERATION_TYPE) {
				row.wbsid = "";
			}
				
            if (treeElementHandle.parentIdentifier) {
                if (dojo.isArray(treeElementHandle.parentIdentifier)) {
                    row.parentId = treeElementHandle.parentIdentifier[0];
                }else {
                    row.parentId = treeElementHandle.parentIdentifier;
                }
            }
            if (treeElementHandle.workItemType) {
                row.workItemType = treeElementHandle.workItemType;
            }
			row.rpcSequenceValue = dojo.isArray(treeElementHandle.rpcSequenceValue)? treeElementHandle.rpcSequenceValue[0] : treeElementHandle.rpcSequenceValue;			
            return row;
        },
        
		/**
		 * Called after a get call to the server, used to update the row with the latest server data. <br>
		 * @param {Object} row: row to be updated with the latest information. <br>
		 * @param {Object} loadedElement: Element values fetched from server. <br>
		 * @param {Object} linkName: nullable field, if not null, used to update the link values of the row <br>
		 */
        updateRowData: function(row, loadedElement, linkName){
            if (linkName == null) {
                if (loadedElement.workItemType) {
                    row.workItemType = loadedElement.workItemType;
                }
                if (loadedElement.resourceType) {
                    row.resourcetype = loadedElement.resourceType;
                }
                else 
                    if (row.resourcetype) {
                        loadedElement.resourceType = row.resourcetype;
                    }
                this._updateName(row, loadedElement);
				this._updatePlannedForField(row, loadedElement);

				if (row.resourcetype == WBSConstants.ITERATION_TYPE) {
					row.wbsid = loadedElement.id;
				}

				if (row.plannedFor) {
					// if plannedFor is set then the parentId is set to that iteration
					row.parentId = row.plannedFor;
				}else{
					this._updateParent(row, loadedElement);					
				}
				row.loadedelement = [loadedElement];
				
				if (loadedElement.rpcSequenceValue) {
	                row.rpcSequenceValue = loadedElement.rpcSequenceValue;
	            }
            }

			if (loadedElement.constraint) {
				row.assignmenttype = loadedElement.constraint.assignmentType;
				if (row.resourcetype == WBSConstants.WORKITEM_TYPE && row.workItemType == WBSConstants.EPM_TASK_TYPE) {
					row.constraintType = loadedElement.constraint.constraintType;
					if (loadedElement.constraint.constraintType == WBSConstants.AS_SOON_AS_POSSIBLE) {
						row.constraintDate = WBSConstants.DEFAULT_CONSTRAINT_VALUE;
					}
					else {
						row.constraintDate = dateUtil.parseDate(loadedElement.constraint.constraintDate);
					}
				}
				else{
					row.constraintType = WBSConstants.DEFAULT_CONSTRAINT_VALUE;
					row.constraintDate = WBSConstants.DEFAULT_CONSTRAINT_VALUE;					
				}
			}
            this._updateProposedSchedule(row, loadedElement);
        },
        
		/**
		 * Method to update the name field. <br>
		 * @param {Object} row: Row for which the name has to be updated. <br>
		 * @param {Object} loadedElement: Element with the name value. <br>
		 */
        _updateName: function(row, loadedElement){
            if (loadedElement.Name) {// for proxy elements.
                row.name = loadedElement.Name;
            }
            else 
                if (loadedElement.name) {//for Programs.
                    row.name = loadedElement.name;
                }
                else 
                    if (loadedElement.summary) {// for work items.
                        row.name = loadedElement.summary;
                    }
        },
        
		/**
		 * Method to update the parent of the element. <br>
		 * @param {Object} row: row to be updated with the parent value. <br>
		 * @param {Object} loadedElement: loadedElement with the link information. <br>
		 */
        _updateParent: function(row, loadedElement){
            var parentLink = getLink(loadedElement, WBSConstants.PARENT);
            if (!parentLink) {
                parentLink = getLink(loadedElement, WBSConstants.PROGRAM_LINK);
            }
            if (parentLink) {
                if (dojo.isArray(parentLink)) {
                    row.parentId = parentLink[0].relativeHref;
                }
                else {
                    row.parentId = parentLink.relativeHref;
                }
            }
        },
        

		/**
		 * Method to update the planned for field, which is required for iteration. <br>
		 * @param {Object} row: Row for which the planned for value has to be updated. <br>
		 * @param {Object} loadedElement: Element from which we can fetch the link. <br>  
		 */
		_updatePlannedForField: function(row, loadedElement){
			var plannedForLink = getLink(loadedElement, WBSConstants.PLANNED_FOR);
			if(plannedForLink){
				row.plannedFor = dojo.isArray(plannedForLink)? plannedForLink[0].relativeHref: plannedFor.relativeHref;
			}
		},
		
		/**
		 * if the proposed schedule is part of the loaded element then updates the row with schedule values
		 * @param {Object} row : model row to which the schedule walues needs to be updated.
		 * @param {Object} loadedElement: the wbs element that is fetched from server.
		 * @return {boolean}: true if the scheule is updated else false.
		 */
		_updateProposedSchedule: function(row, loadedElement){
			var scheduleUpdated = false;
            if (loadedElement.proposedSchedule) {
                row.proposedstart = dateUtil.parseDate(loadedElement.proposedSchedule.startDate);
                row.proposedfinish = dateUtil.parseDate(loadedElement.proposedSchedule.finishDate);
                row.proposedduration = loadedElement.proposedSchedule.duration;
                row.proposedeffort = loadedElement.proposedSchedule.effort;
				scheduleUpdated = true;
            }
			return scheduleUpdated;
		},
        
		/**
		 * returns the postURI suffix for the new element. <br> 
		 * @param {Object} row: newElement to be posted. <br>
		 */
        _getNewElementURI: function(row){
            if (row.resourcetype) {
                return newWBSElementURI[row.resourcetype];
            }
            return null;
        },

		/**
		 * Method called prepare the data to be updated for a program resource <br>
		 * @param {Object} row: row from which the parameters to be saved are extracted <br>
		 * returns: All the data which needs to be saved for the (program) row(modified as well as unmodified). <br>
		 */
	    prepareSaveParamsForProgram: function(row) {
            var programParams = {
                name: row.name,
				summary: row.name
            };
            var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
            var projectAreaName = getCurrentProjectArea().name;
            programParams.projectArea = projectAreaName;
            this.prepareSaveParamsSchedule(row, programParams);
            return programParams;
	   },

		/**
		 * Method called to update the schedule in the save params <br>
		 * @param {Object} row: row from which the parameters to be saved are extracted <br>
		 * @param {Object} saveParams <br>
		 */	    
		prepareSaveParamsSchedule: function(row, saveParams) {
        	var startDate = dateUtil.formatDateForSave(row.proposedstart);
            var finishDate = dateUtil.formatDateForSave(row.proposedfinish);
            if (startDate != null && finishDate != null) {
				saveParams.proposedSchedule = WBSConstants.STARTDATE + WBSConstants.EQUALTO + startDate + WBSConstants.TIMESTAMP +
				WBSConstants.COMMA +
				WBSConstants.FINISHDATE +
				WBSConstants.EQUALTO +
				finishDate +
				WBSConstants.TIMESTAMP +
				WBSConstants.COMMA +
				WBSConstants.DURATION +
				WBSConstants.EQUALTO +
				row.proposedduration;
			}
		},
		    
		/**
		 * Method called to save client side data. <br>
		 * @param {Object} row: row from which the parameters to be saved are extracted <br>
		 * returns: All the data which needs to be saved for that row(modified as well as unmodified). <br>
		 */
        prepareSaveParams: function(row){
            if (row.resourcetype && row.resourcetype == WBSConstants.EPM_PROGRAM_TYPE) {
	            var programParams = this.prepareSaveParamsForProgram(row);
				return programParams;
            }

            var saveParams = {
                summary: row.name
            };
            
            if (row.parentId != null && row.parentId != "") {
                var parentID = row.parentId;
                if (parentID.indexOf(newWBSElementURI[WBSConstants.EPM_PROGRAM_TYPE]) != -1) {
                    saveParams[WBSConstants.PROGRAM_LINK] = row.parentId;
                }
                else {
                    saveParams[WBSConstants.PARENT] = row.parentId;
                }
            }
            
            var rowID = row.id;
            if (rowID.indexOf(EPMConstants.NEWElEMENTSUFFIX) != -1) {
                if (row.workItemType != null && row.workItemType != "") {
                    saveParams.workItemType = row.workItemType;
                }
            }
            var constraintDate = dateUtil.formatDateForSave(row.constraintDate);

            if ((row.resourcetype == WBSConstants.WORKITEM_TYPE && row.workItemType == WBSConstants.EPM_TASK_TYPE) && (row.constraintType !== WBSConstants.DEFAULT_CONSTRAINT_VALUE)) {
                saveParams.constraint = WBSConstants.CONSTRAINTTYPE + WBSConstants.EQUALTO + row.constraintType;
                if (constraintDate !== null && row.constraintType != WBSConstants.AS_SOON_AS_POSSIBLE) {
                    saveParams.constraint = saveParams.constraint + WBSConstants.COMMA + WBSConstants.CONSTRAINTDATE + WBSConstants.EQUALTO + constraintDate + WBSConstants.TIMESTAMP;
                }
            }

        	this.prepareSaveParamsSchedule(row, saveParams);
            
			if ((row.loadedelement != null) && (dojo.isArray(row.loadedelement))&& (row.loadedelement[0].rpcSequenceValue != row.rpcSequenceValue)) {
                saveParams.rpcSequenceValue = row.rpcSequenceValue;
            }else if ((row.loadedelement == null) || (row.loadedelement == "")){
				saveParams.rpcSequenceValue = row.rpcSequenceValue;
			}
			return saveParams;
        },
		/**
		 * Over written function from the abstract model to compare the schedule.
		 * @param {Object} saveParams : the save params prepared using the model row.
		 * @param {Object} tempSaveParams : The save params prepared using the loaded element.
		 * @param {String} field : The Current field which is checked if it is modified.
		 * @param {Object} row : The Model Row that is mapped to grid and is updated from ui.
		 * @param {Object} loadedelement : The backup data returned by the server.
		 * @return {boolean} if field is proposedSchedule then 
		 * 						if any of the schedule fields are modiefied then true.
		 * 						else false.
		 * 					else returns the result from the inherited method.
		 */
		_isFieldModified: function(saveParams, tempSaveParams, field, row, loadedelement){
			if(field == WBSConstants.PROPOSED_SCHEDULE_PARAM){
				var scheduleModified = false
				var row_orig = {	};
				if(this._updateProposedSchedule(row_orig, loadedelement) == true){
					var proposedScheduleParam = this._getModifiedScheduleParam(row, row_orig);
					if(proposedScheduleParam && proposedScheduleParam != ""){
						saveParams[WBSConstants.PROPOSED_SCHEDULE_PARAM] = proposedScheduleParam;
						scheduleModified = true;
					}
				}
				return scheduleModified;
			}else{
				return this.inherited(arguments);
			}
		},
		/**
		 * compares the schedule fields from row and row_orig and adds only the modified fields
		 * @param {Object} row : The Model Row that is mapped to grid and is updated from ui.
		 * @param {Object} row_orig : The temporary row that contains only the schedule fields updated.
		 * @return {String} : the comma seperated name value pairs of modified schedule parameters.
		 * 						Empty string in none of the schedule fields are modified.
		 */
		_getModifiedScheduleParam: function(row, row_orig){
			var modifiedScheduleParam = "";
			var startDate = dateUtil.formatDateForSave(row.proposedstart);
			var startDate_org = dateUtil.formatDateForSave(row_orig.proposedstart);
			if(this.compareFields(startDate, startDate_org, "proposedstart")){
				modifiedScheduleParam =  modifiedScheduleParam + WBSConstants.STARTDATE + WBSConstants.EQUALTO + startDate + WBSConstants.TIMESTAMP;
			}
			var finishDate = dateUtil.formatDateForSave(row.proposedfinish);
			var finishDate_org = dateUtil.formatDateForSave(row_orig.proposedfinish);			
			if(this.compareFields(finishDate, finishDate_org, "proposedfinish")){
				if(modifiedScheduleParam != ""){
					modifiedScheduleParam = modifiedScheduleParam + WBSConstants.COMMA;
				}
				modifiedScheduleParam =  modifiedScheduleParam +
                WBSConstants.FINISHDATE + WBSConstants.EQUALTO + finishDate + WBSConstants.TIMESTAMP;
			}
			var duration = row.proposedduration;
			var duration_org = row_orig.proposedduration;			
			if(this.compareFields(duration, duration_org, "proposedduration")){
				if(modifiedScheduleParam != ""){
					modifiedScheduleParam = modifiedScheduleParam + WBSConstants.COMMA;
				}
				modifiedScheduleParam =  modifiedScheduleParam +
                WBSConstants.DURATION + WBSConstants.EQUALTO + duration;
			}
			return modifiedScheduleParam;
		},
        
		/**
		 * returns the parameters to be saved for given link.<br>
		 * @param {Object} row: row for which the saveParams need to be fetched. <br>
		 * @param {Object} linkName: Link for which the params need to be fetched. <br>
		 */
		prepareSaveParams_link: function(row, linkName){
		},
		
		/**
		 * returns the postURI for the given link.<br>
		 * @param {Object} row: row to which the post call is requested. <br>
		 * @param {Object} linkName: corresponding link. <br>
		 * @param {Object} newElementID: nullable, not null for the new unsaved row(AGC.replaceNode) <br>
		 */
		 getRowPostURI_link: function(row, linkName, newElementID){
		 }
    });
})();
