/*******************************************************************************
* Licensed Materials - Property of IBM
* (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
* 
* Note to U.S. Government Users Restricted Rights:
* Use, duplication or disclosure restricted by GSA ADP Schedule
* Contract with IBM Corp. 
*******************************************************************************/

(function(){
    dojo.provide("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridModel");
    
    dojo.require("com.ibm.epm.web.core.grid.controller.ControllerUtil");
    dojo.require("com.ibm.epm.work.web.ui.internal.wbsGrids.AbstractWBSGridModel");
    
    dojo.require("com.ibm.epm.web.core.util.internal.DateUtil");
    
    dojo.require("com.ibm.epm.work.web.ui.internal.WBSConstants");
    
    //To get the Project Area
    dojo.require("com.ibm.team.process.web.client.projectarea");
    
    dojo.requireLocalization("com.ibm.epm.work.web.ui", "Messages");
    var WorkMessages = dojo.i18n.getLocalization("com.ibm.epm.work.web.ui", "Messages");
    
    var dateUtil = com.ibm.epm.web.core.util.internal.DateUtil;
    
    var getLink = com.ibm.epm.web.core.grid.controller.ControllerUtil.getLink;
    
    var abstractWBSModel = com.ibm.epm.work.web.ui.internal.wbsGrids.AbstractWBSGridModel;
    
    var Constants = com.ibm.epm.work.web.ui.internal.WBSConstants;
    
    
    dojo.declare("com.ibm.epm.work.web.ui.internal.wbsGrids.WBSGeneralGridModel", abstractWBSModel, {

        constructor: function(){
        },
        
		/**
		 * Method to initialize the overview grid. <br>
		 * @param {Object} rowID: id of the row to be initialized. <br>
		 * @param {Object} treeElementHandle: Contains some values passed by tree with which we can initialize the grid row eg, name, resourceType. <br>
		 */
        getEmptyRow: function(rowID, treeElementHandle){
			var row = this.inherited(arguments);        
			row.state = "";
            row.calendarName = "";
            row.calendarURI = "";
			row.assignmenttype = " - ";

            return row;
        },
        
		/**
		 * Called after a get call to the server, used to update the row with the latest server data. <br>
		 * @param {Object} row: row to be updated with the latest information. <br>
		 * @param {Object} loadedElement: Element values fetched from server. <br>
		 * @param {Object} linkName: nullable field, if not null, used to update the link values of the row <br>
		 */
        updateRowData: function(row, loadedElement, linkName){
			this.inherited(arguments);
            if (linkName == null) {
				if (loadedElement.state) {
					row.state = loadedElement.state;
				}
                if (loadedElement.wiTypeName) {
                    row.wiTypeName = loadedElement.wiTypeName;
                }else{
					row.wiTypeName = loadedElement.workItemType;
				}
                row.loadedelement = [loadedElement];
			    this._updateCalendarFields(row, loadedElement);
            }
        },
        
		/**
		 * Method to update the calendar fields, called from updateRowData. <br>
		 * @param {Object} row: row to which the calendar fields have to be updated. <br>
		 * @param {Object} loadedElement: element with the latest info values <br>
		 */
        _updateCalendarFields: function(row, loadedElement){
            var calendarLink = getLink(loadedElement, Constants.CALENDAR);
            if (calendarLink) {
                row.calendarName = calendarLink.title;
                row.calendarURI = calendarLink.relativeHref;
            }
        },
        
		/**
		 * Method called on overview grid save call. <br>
		 * @param {Object} row: row from which the parameters to be saved are extracted <br>
		 * returns: All the data which needs to be saved for that row(modified as well as unmodified). <br>
		 */
        prepareSaveParams: function(row){
			var saveParams = this.inherited(arguments);
            if (row.workItemType != null && row.workItemType != "") {
                saveParams.workItemType = row.workItemType;
            }
            if (row.calendarURI != null && row.calendarURI != "") {
                var assignedCalendarId = this._getResourceIDFromURI(row.calendarURI);
				if (assignedCalendarId) {
					saveParams.adoptCalendarID = assignedCalendarId;
				}
            }
            
            return saveParams;
        },
        
		/**
		 * Method to get the resourceID from URI. <br>
		 * @param {Object} relativeHref: HREF from which resourceID will be parsed. <br>
		 */
        _getResourceIDFromURI: function(relativeHref){
            var index = relativeHref.lastIndexOf("/");
            if (index > 0) {
                return relativeHref.substring(index + 1, relativeHref.length);
            }
        }

    });
})();
